/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/

dojo.provide("ibm.tivoli.tip.charts.dojo.Global");

dojo.require ("ibm.tivoli.tip.dijit.TIPMessageDialog");
// NLS
dojo.requireLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");

/****************************************************************************
* Based on TCR ibm.tivoli.tcr.Global
* Global 
*
* (Q) What is this?
* (A) Since the TIP application will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.charts.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.charts.dojo.Global.stdDateFormat = "%c %z";
ibm.tivoli.tip.charts.dojo.Global.instances     = [];
ibm.tivoli.tip.charts.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
****************************************************************************/
ibm.tivoli.tip.charts.dojo.Global.setSingleton = function (namespace, context) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.charts.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.charts.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
   var singleton = new ibm.tivoli.tip.charts.dojo._Global( context );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};

ibm.tivoli.tip.charts.dojo.Global.getSingleton = function( namespace, context )
{
  var singleton = ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.charts.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.charts.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
    singleton = new ibm.tivoli.tip.charts.dojo._Global( context );
   
    //save the new singleton instance on the Global memory space
    ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ] = singleton; 
  }  
  
  return( singleton ); 
};

/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.charts.dojo._Global",null,
{  
 requests: 0,
 queuedRequests: 0,
 
 constructor: function( context )
 {
      //add all functions, fields, objects inside context to this object
   dojo.mixin( this, context );
   
   this.controller = this.contextualURL( "/TIPChartController" );
   this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
 },
 
 /************************************************************************
 * Generate Conextual URL
 ************************************************************************/
 contextualURL : function( baseURL ) 
 {
    return( this.contextPath + baseURL );
 },

 /************************************************************************
 * Generate Namespaced ID from a BASE ID.
 ************************************************************************/
 namespaceID: function( baseID ) 
 {
    return( this.namespace + baseID );
 }, 
 
 /************************************************************************
 * Funnel all AJAX requests through global object
 ************************************************************************/
 submitRequest: function( requestKW )
 {
   //total requests made
   this.requests++;
   
   //current running requests
   this.queuedRequests++;
   
   //show wait dialog
   if ( this.queuedRequests == 1 )
   {
    //this.wait_timer = setInterval( dojo.lang.hitch( this, "showWaitDialog" ), 100 );
   }
   
   if ( requestKW.appendURL )
   {
     requestKW.url          = this.controller + requestKW.appendURL;
   }
   else
   {
  requestKW.url          = this.controller;
   }
   
   requestKW.handleAs    = "json";
   
   requestKW.mimetype       = "application/json";
 
   requestKW.timeoutSeconds = 0;  // TODO:  0 = no timeout
   requestKW.timeout        = dojo.hitch( this, this.requestTimeoutHandler );
 
   requestKW.error          = dojo.hitch( this, this.requestErrorHandler ); 
 
   requestKW.loadCallback   = requestKW.load; 
   requestKW.load           = dojo.hitch( this, this.requestLoadHandler ); 
   
   requestKW.preventCache   = true; //fix IE cache bug
 
   if ( requestKW.transport && ( requestKW.transport == ibm.tivoli.tip.charts.dojo.Global.IFrame ) ) 
   {
     this.IFrameRequestKW = requestKW;
   }
 
   dojo.xhrGet( requestKW ); 
 },
 
 /************************************************************************
 * Standard JSON request "load" handler - check for server-side errors
 * returned in the 'exc' field.  Forward to caller's "load" handler if none.
 ************************************************************************/
    requestLoadHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
   
   var kw = obj.args;

   if ( this.queuedRequests === 0 )
   {
  //clearInterval( this.wait_timer );
    //hide_wait_dialog( this.context.nameSpace );
   }   
   
   // check for availability of request KW; IFrame-based transport handled specially
   // see above method
   if ( ! kw )
   {
     if ( this.IFrameRequestKW )
     {
         kw = this.IFrameRequestKW;
       delete this.IFrameRequestKW;
     }
   }
 
   // was an exception detected and reported?
   if ( obj.exc )
   { 
     var exc = obj.exc;
 
     // ISC session timeout?
     if ( exc == "ISCSessionTimeoutException" ) 
     {
       this.refreshPage();
 
       return;
     }
     
     // some other server-side processing exception :-(  Exception handler/list defined?
     if ( kw && kw.excHandler && kw.excHandled )
     {
       if ( ( kw.excHandled == ibm.tivoli.tip.charts.dojo.Global.anyException ) || // handle ANY exc  
           ( kw.excHandled.indexOf( exc ) != -1 ) )     // handle this exc?
       {
         kw.excHandler( exc, obj, kw );  
 
         return;
       }
     }
 
     // exc message available?
     var excMsg = obj.excMsg ? obj.excMsg : "",
 
         msgMax = 500;
 
     if ( excMsg.length > msgMax ) 
     {
       excMsg = excMsg.substring( 0, msgMax ) + "...";
     }
 
     var mKW = { 
                messageId :                        this.nlsResources.BIND_ERROR_ID,
                message   : dojo.string.substitute(this.nlsResources.BIND_ERROR_MSG, { 0 : exc, 1 : excMsg } )
               } ;
 
     this.showErrorMessage( mKW );                                         
   }
 
   // No exception --> forward to originally specified "load" callback
   else
   {
     kw.loadCallback( data, obj );
   }
 },
 
 /************************************************************************
 * show error message dialog
 ************************************************************************/
 showErrorMessage: function( kwMessage )
 {
   //var mb = new TcrMsgBox( kwMessage );
 
   //mb.showError();
   
   console.debug( "Error occured" );
   console.debug( kwMessage );
 },

 /************************************************************************
 * Standard JSON request error handler
 ************************************************************************/
 requestErrorHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
 
   if ( this.queuedRequests === 0 )
   {
    //clearInterval( this.wait_timer );
    //hide_wait_dialog( this.context.nameSpace );
   }   
 
   this.showErrorMessage( 
                          { 
                            messageId :                        this.nlsResources.BIND_ERROR_ID,
                            message   : dojo.string.substitute(this.nlsResources.BIND_ERROR_MSG, { 0 : obj.message } )
                          } 
                        );                                         
 },

 /************************************************************************
 * Standard JSON request timeout handler
 ************************************************************************/
 requestTimeoutHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
 
   if ( this.queuedRequests === 0 )
   {
     //clearInterval( this.wait_timer );   
    //hide_wait_dialog( this.context.nameSpace );
   }   
 
   this.showErrorMessage( 
                          { 
                           messageId : this.nlsResources.BIND_TIMEOUT_ID,
                           message   : this.nlsResources.BIND_TIMEOUT_MSG
                          } 
                        );                                         
 },
 
 /************************************************************************
 * Format a DATE with NLS rules
 ************************************************************************/ 
 formatDate: function( date ) 
 {
    var formattedDate = dojo.date.strftime( date, ibm.tivoli.tip.charts.dojo.Global.stdDateFormat );
  
    return( formattedDate );
 },
 
 /************************************************************************
 * Refresh the page
 ************************************************************************/ 
 refreshPage: function()
 {
    window.location.reload( false );
 },
 
 handleServerException: function (exception) 
 {
 	console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
	console.error ("with server stack trace: ");
	console.error (exception.stackTrace);
	this.showMessageDialog (exception.type, exception.messageID, exception.messageString, false);
 },
 
 showMessageDialog: function (type, msgID, msg, showCheckbox) {
	var closeText = this.resources_.CLOSE;
	var callback = null;
	if (!showCheckbox) {
		showCheckbox = false;
		callback = null;
	} else {
		callback = dojo.hitch (this, this._onShowMessageClose);
	}
	
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
	        	                                      showCheckbox: showCheckbox,
	        	                                      callback: callback,
	            	                                  buttons: [ closeText ]
	                    	                        }); 
	modalWindow.show();
},

_onShowMessageClose: function (button, messageID, checked) {
	if (checked) {
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=setMessagePreference&arg_messageID=" + messageID + "&arg_neverShow=" + checked + "&arg_userName=" + this.username;
	    dojo.xhrGet({
                       url: url,
					    handleAs: "json-comment-filtered",
	          			error: dojo.hitch (this, this.handleError),
			  			timeout: dojo.hitch (this, this.handleTimeout)
                    });
    }
},

getAncestorByName: function (element, elementName) {
	var result=null;
	try {
		while(result===null && element!==null) {
			element=element.parentNode;
			if(element.nodeName.toLowerCase()==elementName.toLowerCase()) {
				result=element;
			}
		}
	} catch(e) {
		//no node
	}
	//console.log ("Nodename = " + result);
	return(result);
},
getAncestorByClass: function (element, className) {
	var result=null;
	try {
		while(result===null && element!==null) {
			element=element.parentNode;
			if(dojo.hasClass (element, className)) {
				result=element;
			}
		}
	} catch(e) {
		//no node
	}
	//console.log ("Nodename = " + result);
	return(result);
}

 
  
});

//Declare a new Error type to throw when a Global context error is detected
dojo.declare("ibm.tivoli.tip.charts.dojo.Global.InvalidContextError", Error, {});