/*
	Copyright (c) 2004-2007, The Dojo Foundation
	All Rights Reserved.

	Licensed under the Academic Free License version 2.1 or above OR the
	modified BSD license. For more information on Dojo licensing, see:

		http://dojotoolkit.org/book/dojo-book-0-9/introduction/licensing
*/

/*
	This is a compiled version of Dojo, built for deployment and not for
	development. To get an editable version, please visit:

		http://dojotoolkit.org

	for documentation and information on getting the source.
*/

if(!dojo._hasResource["ibm.tivoli.tip.charts.Version"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.Version"] = true;

dojo.provide("ibm.tivoli.tip.charts.Version");

ibm.tivoli.tip.charts.Version.getVersion = function() {
  return( "tip_1.1.0.1-200901151741" );
};
		

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");
dojo.provide("dojo.fx.Toggler");

dojo.fx.chain = function(/*dojo._Animation[]*/ animations){
	// summary: Chain a list of dojo._Animation s to run in sequence
	// example:
	//	|	dojo.fx.chain([
	//	|		dojo.fadeIn({ node:node }),
	//	|		dojo.fadeOut({ node:otherNode })
	//	|	]).play();
	//
	var first = animations.shift();
	var previous = first;
	dojo.forEach(animations, function(current){
		dojo.connect(previous, "onEnd", current, "play");
		previous = current;
	});
	return first; // dojo._Animation
};

dojo.fx.combine = function(/*dojo._Animation[]*/ animations){
	// summary: Combine a list of dojo._Animation s to run in parallel
	// example:
	//	|	dojo.fx.combine([
	//	|		dojo.fadeIn({ node:node }),
	//	|		dojo.fadeOut({ node:otherNode })
	//	|	]).play();
	var ctr = new dojo._Animation({ curve: [0, 1] });
	if(!animations.length){ return ctr; }
	// animations.sort(function(a, b){ return a.duration-b.duration; });
	ctr.duration = animations[0].duration;
	dojo.forEach(animations, function(current){
		dojo.forEach([ "play", "pause", "stop" ],
			function(e){
				if(current[e]){
					dojo.connect(ctr, e, current, e);
				}
			}
		);
	});
	return ctr; // dojo._Animation
};

dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc.
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.wipeIn, 
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	// node: DomNode
	//	the node to toggle
	node: null,

	// showFunc: Function
	//	The function that returns the dojo._Animation to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function	
	//	The function that returns the dojo._Animation to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//	Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//	Time in milliseconds to run the hide Animation
	hideDuration: 200,

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	show: function(delay){
		// summary: Toggle the node to showing
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		return this.hideAnim.play(delay || 0);
	}
});

dojo.fx.wipeIn = function(/*Object*/ args){
	// summary
	//		Returns an animation that will expand the
	//		node defined in 'args' object from it's current height to
	//		it's natural height (with no scrollbar).
	//		Node must have no margin/border/padding.
	args.node = dojo.byId(args.node);
	var node = args.node, s = node.style, o;

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			height: {
				// wrapped in functions so we wait till the last second to query (in case value has changed)
				start: function(){
					// start at current [computed] height, but use 1px rather than 0
					// because 0 causes IE to display the whole panel
					o = s.overflow;
					s.overflow="hidden";
					if(s.visibility=="hidden"||s.display=="none"){
						s.height="1px";
						s.display="";
						s.visibility="";
						return 1;
					}else{
						var height = dojo.style(node, "height");
						return Math.max(height, 1);
					}
				},
				end: function(){
					return node.scrollHeight;
				}
			}
		}
	}, args));

	dojo.connect(anim, "onEnd", function(){ 
		s.height = "auto";
		s.overflow = o;
	});

	return anim; // dojo._Animation
}

dojo.fx.wipeOut = function(/*Object*/ args){
	// summary
	//		Returns an animation that will shrink node defined in "args"
	//		from it's current height to 1px, and then hide it.
	var node = args.node = dojo.byId(args.node);
	var s = node.style;
	var o;

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			height: {
				end: 1 // 0 causes IE to display the whole panel
			}
		}
	}, args));

	dojo.connect(anim, "beforeBegin", function(){
		o = s.overflow;
		s.overflow = "hidden";
		s.display = "";
	});
	dojo.connect(anim, "onEnd", function(){
		s.overflow = o;
		s.height = "auto";
		s.display = "none";
	});

	return anim; // dojo._Animation
}

dojo.fx.slideTo = function(/*Object?*/ args){
	// summary
	//		Returns an animation that will slide "node" 
	//		defined in args Object from its current position to
	//		the position defined by (args.left, args.top).
	// example:
	//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", unit:"px" }).play()

	var node = (args.node = dojo.byId(args.node));
	
	var top = null;
	var left = null;
	
	var init = (function(n){
		return function(){
			var cs = dojo.getComputedStyle(n);
			var pos = cs.position;
			top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
			left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
			if(pos != 'absolute' && pos != 'relative'){
				var ret = dojo.coords(n, true);
				top = ret.y;
				left = ret.x;
				n.style.position="absolute";
				n.style.top=top+"px";
				n.style.left=left+"px";
			}
		};
	})(node);
	init();

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			top: { end: args.top||0 },
			left: { end: args.left||0 }
		}
	}, args));
	dojo.connect(anim, "beforeBegin", anim, init);

	return anim; // dojo._Animation
}

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become actived/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example:  Combobutton --> Menu -->
//		MenuItem.   The onBlur event for Combobutton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		Combobutton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit,
{
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary: tests whether the current selection is empty
		var _window = dojo.global;
		var _document = dojo.doc;
		if(_document.selection){ // IE
			return !_document.selection.createRange().text; // Boolean
		}else if(_window.getSelection){
			var selection = _window.getSelection();
			if(dojo.isString(selection)){ // Safari
				return !selection; // Boolean
			}else{ // Mozilla/W3
				return selection.isCollapsed || !selection.toString(); // Boolean
			}
		}
	},

	getBookmark: function(){
		// summary: Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bookmark, selection = dojo.doc.selection;
		if(selection){ // IE
			var range = selection.createRange();
			if(selection.type.toUpperCase()=='CONTROL'){
				bookmark = range.length ? dojo._toArray(range) : null;
			}else{
				bookmark = range.getBookmark();
			}
		}else{
			if(dojo.global.getSelection){
				selection = dojo.global.getSelection();
				if(selection){
					var range = selection.getRangeAt(0);
					bookmark = range.cloneRange();
				}
			}else{
				console.debug("No idea how to store the current selection for this browser!");
			}
		}
		return bookmark; // Array
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary: Moves current selection to a bookmark
		// bookmark: this should be a returned object from dojo.html.selection.getBookmark()
		var _document = dojo.doc;
		if(_document.selection){ // IE
			var range;
			if(dojo.isArray(bookmark)){
				range = _document.body.createControlRange();
				dojo.forEach(bookmark, range.addElement);
			}else{
				range = _document.selection.createRange();
				range.moveToBookmark(bookmark);
			}
			range.select();
		}else{ //Moz/W3C
			var selection = dojo.global.getSelection && dojo.global.getSelection();
			if(selection && selection.removeAllRanges){
				selection.removeAllRanges();
				selection.addRange(bookmark);
			}else{
				console.debug("No idea how to restore selection for this browser!");
			}
		}
	},

	getFocus: function(/*Widget*/menu, /*Window*/ openedForWindow){
		// summary:
		//	Returns the current focus and selection.
		//	Called when a popup appears (either a top level menu or a dialog),
		//	or when a toolbar/menubar receives focus
		//
		// menu:
		//	the menu that's being opened
		//
		// openedForWindow:
		//	iframe in which menu was opened
		//
		// returns:
		//	a handle to restore focus/selection

		return {
			// Node to return focus to
			node: menu && dojo.isDescendant(dijit._curFocus, menu.domNode) ? dijit._prevFocus : dijit._curFocus,

			// Previously selected text
			bookmark:
				!dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed) ?
				dojo.withGlobal(openedForWindow||dojo.global, dijit.getBookmark) :
				null,

			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase()=="iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}			
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item)
		if(bookmark && dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed)){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow||dojo.global, moveToBookmark, null, [bookmark]);
			}catch(e){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerWin: function(/*Window?*/targetWindow){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe) to detect when the user has clicked somewhere.
		//		Anyone that creates an iframe should call this function.

		if(!targetWindow){
			targetWindow = window;
		}

		dojo.connect(targetWindow.document, "onmousedown", null, function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			dijit._onTouchNode(evt.target||evt.srcElement);
		});
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's body
		var body = targetWindow.document.body || targetWindow.document.getElementsByTagName("body")[0];
		if(body){
			if(dojo.isIE){
				body.attachEvent('onactivate', function(evt){
					if(evt.srcElement.tagName.toLowerCase() != "body"){
						dijit._onFocusNode(evt.srcElement);
					}
				});
				body.attachEvent('ondeactivate', function(evt){ dijit._onBlurNode(evt.srcElement); });
			}else{
				body.addEventListener('focus', function(evt){ dijit._onFocusNode(evt.target); }, true);
				body.addEventListener('blur', function(evt){ dijit._onBlurNode(evt.target); }, true);
			}
		}
		body = null;	// prevent memory leak (apparent circular reference via closure)
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = false;
			w._setStateClass();
		}
		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer; dijit._setStack([]); }, 100);
	},

	_onTouchNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused or mouse-downed

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				if(node.dijitPopupParent){
					node=dijit.byId(node.dijitPopupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase()=="body"){
					// is this the root of the document or just the root of an iframe?
					if(node===dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead) and continue tracing up the document
					node=dojo.query("iframe").filter(function(iframe){ return iframe.contentDocument.body===node; })[0];
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId");
					if(id){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused
		if(node && node.tagName && node.tagName.toLowerCase() == "body"){
			return;
		}
		dijit._onTouchNode(node);
		if(node==dijit._curFocus){ return; }
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);

		// handle focus/blur styling
		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = true;
			w._setStateClass();
		}
	},

	_setStack: function(newStack){
		// summary
		//	The stack of active widgets has changed.  Send out appropriate events and record new stack

		var oldStack = dijit._activeStack;		
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			var widget = dijit.byId(oldStack[i]);
			if(widget){
				dojo.publish("widgetBlur", [widget]);
				if(widget._onBlur){
					widget._onBlur();
				}
			}
		}

		// for all element that have come into focus, send focus event
		for(var i=nCommon; i<newStack.length; i++){
			var widget = dijit.byId(newStack[i]);
			if(widget){
				dojo.publish("widgetFocus", [widget]);
				if(widget._onFocus){
					widget._onFocus();
				}
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(dijit.registerWin);

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	constructor: function(){
		// summary:
		//	A set of widgets indexed by id
		this._hash={};
	},

	add: function(/*Widget*/ widget){
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id]=widget;
	},

	remove: function(/*String*/ id){
		delete this._hash[id];
	},

	forEach: function(/*Function*/ func){
		for(var id in this._hash){
			func(this._hash[id]);
		}
	},

	filter: function(/*Function*/ filter){
		var res = new dijit.WidgetSet();
		this.forEach(function(widget){
			if(filter(widget)){ res.add(widget); }
		});
		return res;		// dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		return this._hash[id];
	},

	byClass: function(/*String*/ cls){
		return this.filter(function(widget){ return widget.declaredClass==cls; });	// dijit.WidgetSet
	}
	});

// registry: list of all widgets on page
dijit.registry = new dijit.WidgetSet();

dijit._widgetTypeCtr = {};

dijit.getUniqueId = function(/*String*/widgetType){
	// summary
	//	Generates a unique id for a given widgetType

	var id;
	do{
		id = widgetType + "_" +
			(dijit._widgetTypeCtr[widgetType] !== undefined ?
				++dijit._widgetTypeCtr[widgetType] : dijit._widgetTypeCtr[widgetType] = 0);
	}while(dijit.byId(id));
	return id; // String
};


if(dojo.isIE){
	// Only run this for IE because we think it's only necessary in that case,
	// and because it causes problems on FF.  See bug #3531 for details.
	dojo.addOnUnload(function(){
		dijit.registry.forEach(function(widget){ widget.destroy(); });
	});
}

dijit.byId = function(/*String|Widget*/id){
	// summary:
	//		Returns a widget by its id, or if passed a widget, no-op (like dojo.byId())
	return (dojo.isString(id)) ? dijit.registry.byId(id) : id; // Widget
};

dijit.byNode = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget as referenced by node
	return dijit.registry.byId(node.getAttribute("widgetId")); // Widget
};

dijit.getEnclosingWidget = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget whose dom tree contains node or null if
	//		the node is not contained within the dom tree of any widget
	while(node){
		if(node.getAttribute && node.getAttribute("widgetId")){
			return dijit.registry.byId(node.getAttribute("widgetId"));
		}
		node = node.parentNode;
	}
	return null;
};

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");

// ported from dojo.html.util

dijit.getViewport = function(){
	//	summary
	//	Returns the dimensions and scroll position of the viewable area of a browser window

	var _window = dojo.global;
	var _document = dojo.doc;

	// get viewport size
	var w = 0, h = 0;
	if(dojo.isMozilla){
		// mozilla
		// _window.innerHeight includes the height taken by the scroll bar
		// clientHeight is ideal but has DTD issues:
		// #4539: FF reverses the roles of body.clientHeight/Width and documentElement.clientHeight/Width based on the DTD!
		// check DTD to see whether body or documentElement returns the viewport dimensions using this algorithm:
		var minw, minh, maxw, maxh;
		if(_document.body.clientWidth>_document.documentElement.clientWidth){
			minw = _document.documentElement.clientWidth;
			maxw = _document.body.clientWidth;
		}else{
			maxw = _document.documentElement.clientWidth;
			minw = _document.body.clientWidth;
		}
		if(_document.body.clientHeight>_document.documentElement.clientHeight){
			minh = _document.documentElement.clientHeight;
			maxh = _document.body.clientHeight;
		}else{
			maxh = _document.documentElement.clientHeight;
			minh = _document.body.clientHeight;
		}
		w = (maxw > _window.innerWidth) ? minw : maxw;
		h = (maxh > _window.innerHeight) ? minh : maxh;
	}else if(!dojo.isOpera && _window.innerWidth){
		//in opera9, dojo.body().clientWidth should be used, instead
		//of window.innerWidth/document.documentElement.clientWidth
		//so we have to check whether it is opera
		w = _window.innerWidth;
		h = _window.innerHeight;
	}else if(dojo.isIE && _document.documentElement && _document.documentElement.clientHeight){
		w = _document.documentElement.clientWidth;
		h = _document.documentElement.clientHeight;
	}else if(dojo.body().clientWidth){
		// IE5, Opera
		w = dojo.body().clientWidth;
		h = dojo.body().clientHeight;
	}

	// get scroll position
	var scroll = dojo._docScroll();

	return { w: w, h: h, l: scroll.x, t: scroll.y };	//	object
};

dijit.placeOnScreen = function(
	/* DomNode */	node,
	/* Object */		pos,
	/* Object */		corners,
	/* boolean? */		tryOnly){
	//	summary:
	//		Keeps 'node' in the visible area of the screen while trying to
	//		place closest to pos.x, pos.y. The input coordinates are
	//		expected to be the desired document position.
	//
	//		Set which corner(s) you want to bind to, such as
	//		
	//			placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])
	//		
	//		The desired x/y will be treated as the topleft(TL)/topright(TR) or
	//		BottomLeft(BL)/BottomRight(BR) corner of the node. Each corner is tested
	//		and if a perfect match is found, it will be used. Otherwise, it goes through
	//		all of the specified corners, and choose the most appropriate one.
	//		
	//		NOTE: node is assumed to be absolutely or relatively positioned.

	var choices = dojo.map(corners, function(corner){ return { corner: corner, pos: pos }; });

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dijit.getViewport();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to document.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best=null;
	for(var i=0; i<choices.length; i++){
		var corner = choices[i].corner;
		var pos = choices[i].pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(corner);
		}

		// get node's size
		var oldDisplay = node.style.display;
		var oldVis = node.style.visibility;
		node.style.visibility = "hidden";
		node.style.display = "";
		var mb = dojo.marginBox(node);
		node.style.display = oldDisplay;
		node.style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = (corner.charAt(1)=='L' ? pos.x : Math.max(view.l, pos.x - mb.w)),
			startY = (corner.charAt(0)=='T' ? pos.y : Math.max(view.t, pos.y -  mb.h)),
			endX = (corner.charAt(1)=='L' ? Math.min(view.l+view.w, startX+mb.w) : pos.x),
			endY = (corner.charAt(0)=='T' ? Math.min(view.t+view.h, startY+mb.h) : pos.y),
			width = endX-startX,
			height = endY-startY,
			overflow = (mb.w-width) + (mb.h-height);

		if(best==null || overflow<best.overflow){
			best = {
				corner: corner,
				aroundCorner: choices[i].aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		if(overflow==0){
			break;
		}
	}

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	return best;
}

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	//	summary
	//	Like placeOnScreen, except it accepts aroundNode instead of x,y
	//	and attempts to place node around it.  Uses margin box dimensions.
	//
	//	aroundCorners
	//		specify Which corner of aroundNode should be
	//		used to place the node => which corner(s) of node to use (see the
	//		corners parameter in dijit.placeOnScreen)
	//		e.g. {'TL': 'BL', 'BL': 'TL'}
	//
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.


	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodeW = aroundNode.offsetWidth; //mb.w;
	var aroundNodeH = aroundNode.offsetHeight; //mb.h;
	var aroundNodePos = dojo.coords(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: aroundNodePos.x + (nodeCorner.charAt(1)=='L' ? 0 : aroundNodeW),
				y: aroundNodePos.y + (nodeCorner.charAt(0)=='T' ? 0 : aroundNodeH)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
}

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");

dijit.getDocumentWindow = function(doc){
	//	summary
	// 	Get window object associated with document doc

	// With Safari, there is not way to retrieve the window from the document, so we must fix it.
	if(dojo.isSafari && !doc._parentWindow){
		/*
			This is a Safari specific function that fix the reference to the parent
			window from the document object.
		*/
		var fix=function(win){
			win.document._parentWindow=win;
			for(var i=0; i<win.frames.length; i++){
				fix(win.frames[i]);
			}
		}
		fix(window.top);
	}

	//In some IE versions (at least 6.0), document.parentWindow does not return a
	//reference to the real window object (maybe a copy), so we must fix it as well
	//We use IE specific execScript to attach the real window reference to
	//document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow && !doc._parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc._parentWindow || doc.parentWindow || doc.defaultView;	//	Window
}

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





dijit.popup = new function(){
	// summary:
	//		This class is used to show/hide widgets as popups.
	//

	var stack = [],
		beginZIndex=1000,
		idGen = 1;

	this.open = function(/*Object*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// args: Object
		//		popup: Widget
		//			widget to display,
		//		parent: Widget
		//			the button etc. that is displaying this popup
		//		around: DomNode
		//			DOM node (typically a button); place popup relative to this node
		//		orient: Object
		//			structure specifying possible positions of popup relative to "around" node
		//		onCancel: Function
		//			callback when user has canceled the popup by
		//				1. hitting ESC or
		//				2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//				   ie: whenever popupWidget.onCancel() is called, args.onCancel is called
		//		onClose: Function
		//			callback whenever this popup is closed
		//		onExecute: Function
		//			callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		//
		// examples:
		//		1. opening at the mouse position
		//			dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//		2. opening the widget as a dropdown
		//			dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}  });
		//
		//	Note that whatever widget called dijit.popup.open() should also listen to it's own _onBlur callback
		//	(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var widget = args.popup,
			orient = args.orient || {'BL':'TL', 'TL':'BL'},
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+idGen++);

		// make wrapper div to hold widget and possibly hold iframe behind it.
		// we can't attach the iframe as a child of the widget.domNode because
		// widget.domNode might be a <table>, <ul>, etc.
		var wrapper = dojo.doc.createElement("div");
		wrapper.id = id;
		wrapper.className="dijitPopup";
		wrapper.style.zIndex = beginZIndex + stack.length;
		wrapper.style.visibility = "hidden";
		if(args.parent){
			wrapper.dijitPopupParent=args.parent.id;
		}
		dojo.body().appendChild(wrapper);

		widget.domNode.style.display="";
		wrapper.appendChild(widget.domNode);

		var iframe = new dijit.BackgroundIframe(wrapper);

		// position the wrapper node
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR']);

		wrapper.style.visibility = "visible";
		// TODO: use effects to fade in wrapper

		var handlers = [];

		// Compute the closest ancestor popup that's *not* a child of another popup.
		// Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		function getTopPopup(){
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--);
			return stack[pi];
		}

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.keyCode == dojo.keys.ESCAPE && args.onCancel){
				args.onCancel();
			}else if(evt.keyCode == dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", null, args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", null, function(){
			var topPopup = getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			wrapper: wrapper,
			iframe: iframe,
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			widget.onOpen(best);
		}

		return best;
	};

	this.close = function(/*Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				wrapper = top.wrapper,
				iframe = top.iframe,
				widget = top.widget,
				onClose = top.onClose;
	
			if(widget.onClose){
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);
	
			// #2685: check if the widget still has a domNode so ContentPane can change its URL without getting an error
			if(!widget||!widget.domNode){ return; }
			dojo.style(widget.domNode, "display", "none");
			dojo.body().appendChild(widget.domNode);
			iframe.destroy();
			dojo._destroyElement(wrapper);
	
			if(onClose){
				onClose();
			}
		}
	};
}();

dijit._frames = new function(){
	// summary: cache of iframes
	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var html="<iframe src='javascript:\"\"'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	var iframe = dojo.doc.createElement("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didnt work.
			dojo.body().appendChild(iframe);
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="";
		if(dojo.isIE){
			iframe.style.removeExpression("width");
			iframe.style.removeExpression("height");
		}
		queue.push(iframe);
	}
}();

// fill the queue
if(dojo.isIE && dojo.isIE < 7){
	dojo.addOnLoad(function(){
		var f = dijit._frames;
		dojo.forEach([f.pop()], f.push);
	});
}


dijit.BackgroundIframe = function(/* DomNode */node){
	//	summary:
	//		For IE z-index schenanigans. id attribute is required.
	//
	//	description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if((dojo.isIE && dojo.isIE < 7) || (dojo.isFF && dojo.isFF < 3 && dojo.hasClass(dojo.body(), "dijit_a11y"))){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE){
			iframe.style.setExpression("width", "document.getElementById('" + node.id + "').offsetWidth");
			iframe.style.setExpression("height", "document.getElementById('" + node.id + "').offsetHeight");
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	destroy: function(){
		//	summary: destroy the iframe
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");

dijit.scrollIntoView = function(/* DomNode */node){
	//	summary
	//	Scroll the passed node into view, if it is not.

	// don't rely on that node.scrollIntoView works just because the function is there
	// it doesnt work in Konqueror or Opera even though the function is there and probably
	// not safari either
	// dont like browser sniffs implementations but sometimes you have to use it
	if(dojo.isIE){
		//only call scrollIntoView if there is a scrollbar for this menu,
		//otherwise, scrollIntoView will scroll the window scrollbar
		if(dojo.marginBox(node.parentNode).h <= node.parentNode.scrollHeight){ //PORT was getBorderBox
			node.scrollIntoView(false);
		}
	}else if(dojo.isMozilla){
		node.scrollIntoView(false);
	}else{
		var parent = node.parentNode;
		var parentBottom = parent.scrollTop + dojo.marginBox(parent).h; //PORT was getBorderBox
		var nodeBottom = node.offsetTop + dojo.marginBox(node).h;
		if(parentBottom < nodeBottom){
			parent.scrollTop += (nodeBottom - parentBottom);
		}else if(parent.scrollTop > node.offsetTop){
			parent.scrollTop -= (parent.scrollTop - node.offsetTop);
		}
	}
};

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");

// ported from dojo.html.applyBrowserClass (style.js)

//	summary:
//		Applies pre-set class names based on browser & version to the
//		top-level HTML node.  Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.
(function(){
	var d = dojo;
	var ie = d.isIE;
	var opera = d.isOpera;
	var maj = Math.floor;
	var classes = {
		dj_ie: ie,
//		dj_ie55: ie == 5.5,
		dj_ie6: maj(ie) == 6,
		dj_ie7: maj(ie) == 7,
		dj_iequirks: ie && d.isQuirks,
// NOTE: Opera not supported by dijit
		dj_opera: opera,
		dj_opera8: maj(opera) == 8,
		dj_opera9: maj(opera) == 9,
		dj_khtml: d.isKhtml,
		dj_safari: d.isSafari,
		dj_gecko: d.isMozilla
	}; // no dojo unsupported browsers

	for(var p in classes){
		if(classes[p]){
			var html = dojo.doc.documentElement; //TODO browser-specific DOM magic needed?
			if(html.className){
				html.className += " " + p;
			}else{
				html.className = p;
			}
		}
	}
})();

}

if(!dojo._hasResource["dijit._base.bidi"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.bidi"] = true;
dojo.provide("dijit._base.bidi");

// summary: applies a class to the top of the document for right-to-left stylesheet rules

dojo.addOnLoad(function(){
	if(!dojo._isBodyLtr()){
		dojo.addClass(dojo.body(), "dijitRtl");
	}
});

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//	These functions are used to repetitively call a user specified callback
	//	method when a specific key or mouse click over a specific DOM node is
	//	held down for a specific amount of time.
	//	Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		this._currentTimeout = (this._currentTimeout < 0) ? this._initialDelay : ((this._subsequentDelay > 1) ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay));
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /* Object */ _this, /*DOMNode*/ node, /* Function */ callback, /* Object */ obj, /* Number */ subsequentDelay, /* Number */ initialDelay){
		// summary:
		//      Start a timed, repeating callback sequence.
		//      If already started, the function call is ignored.
		//      This method is not normally called by the user but can be
		//      when the normal listener code is insufficient.
		//	Parameters:
		//	evt: key or mouse event object to pass to the user callback
		//	_this: pointer to the user's widget space.
		//	node: the DOM node object to pass the the callback function
		//	callback: function to call until the sequence is stopped called with 3 parameters:
		//		count: integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		//		node: the DOM node object passed in
		//		evt: key or mouse event object
		//	obj: user space object used to uniquely identify each typematic sequence
		//	subsequentDelay: if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		//	initialDelay: the number of milliseconds until the 2nd event occurs, default=500ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//	  Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key.
		//	keyObject: an object defining the key to listen for.
		//		key: (mandatory) the keyCode (number) or character (string) to listen for.
		//		ctrlKey: desired ctrl key state to initiate the calback sequence:
		//			pressed (true)
		//			released (false)
		//			either (unspecified)
		//		altKey: same as ctrlKey but for the alt key
		//		shiftKey: same as ctrlKey but for the shift key
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.keyCode == keyObject.keyCode && (!keyObject.charCode || keyObject.charCode == evt.charCode) &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.ctrlKey) &&
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.ctrlKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(keyObject, _this, node, callback, keyObject, subsequentDelay, initialDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a typematic mouse click.
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				dojo.stopEvent(evt);
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
					setTimeout(dijit.typematic.stop, 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key and mouseclick.
		//	This is a thin wrapper to addKeyListener and addMouseListener.
		//	mouseNode: the DOM node object to listen on for mouse events.
		//	keyNode: the DOM node object to listen on for key events.
		//	See the addMouseListener and addKeyListener methods for other parameters.
		//	Returns an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Function that detects if we are in high-contrast mode or not,
		//		and sets up a timer to periodically confirm the value.
		//		figure out the background-image style property
		//		and apply that to the image.src property.
		// description:
		//		This must be a named function and not an anonymous
		//		function, so that the widget parsing code can make sure it
		//		registers its onload function after this function.
		//		DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = document.createElement("div");
		div.id = "a11yTestNode";
		div.style.cssText = 'border: 1px solid;'
			+ 'border-color:red green;'
			+ 'position: absolute;'
			+ 'height: 5px;'
			+ 'top: -999px;'
			+ 'background-image: url("' + dojo.moduleUrl("dijit", "form/templates/blank.gif") + '");';
		dojo.body().appendChild(div);

		// test it
		function check(){
			var cs = dojo.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor==cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			}
		}
		check();
		if(dojo.isIE){
			setInterval(check, 4000);
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit,
{
	hasWaiRole: function(/*Element*/ elem){
		// Summary: Return true if elem has a role attribute and false if not.
		if(elem.hasAttribute){
			return elem.hasAttribute("role");
		}else{
			return elem.getAttribute("role") ? true : false;
		}
	},

	getWaiRole: function(/*Element*/ elem){
		// Summary: Return the role of elem or an empty string if
		//		elem does not have a role.
		var value = elem.getAttribute("role");
		if(value){
			var prefixEnd = value.indexOf(":");
			return prefixEnd == -1 ? value : value.substring(prefixEnd+1);
		}else{
			return "";
		}
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// Summary: Set the role on elem. On Firefox 2 and below, "wairole:" is
		//		prepended to the provided role value.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttribute("role", "wairole:"+role);
		}else{
			elem.setAttribute("role", role);
		}
	},

	removeWaiRole: function(/*Element*/ elem){
		// Summary: Removes the role attribute from elem.
		elem.removeAttribute("role");
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return true if elem has a value for the given state and
		//		false if it does not.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.hasAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			if(elem.hasAttribute){
				return elem.hasAttribute("aria-"+state);
			}else{
				return elem.getAttribute("aria-"+state) ? true : false;
			}
		}
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return the value of the requested state on elem
		//		or an empty string if elem has no value for state.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.getAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			var value =  elem.getAttribute("aria-"+state);
			return value ? value : "";
		}
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// Summary: Set state on elem to value.
		//		On Firefox 2 and below, we set an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we set an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttributeNS("http://www.w3.org/2005/07/aaa",
				"aaa:"+state, value);
		}else{
			elem.setAttribute("aria-"+state, value);
		}
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Removes the given state from elem.
		//		On Firefox 2 and below, we remove the attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we remove the attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.removeAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			elem.removeAttribute("aria-"+state);
		}
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");



dojo.declare("dijit._Widget", null, {
	// summary:
	//		The foundation of dijit widgets. 	
	//
	// id: String
	//		a unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: String
	//	Language to display this widget in (like en-us).
	//	Defaults to brower's specified preferred language (typically the language of the OS)
	lang: "",

	// dir: String
	//  Bi-directional support, as defined by the HTML DIR attribute. Either left-to-right "ltr" or right-to-left "rtl".
	dir: "",

	// class: String
	// HTML class attribute
	"class": "",

	// style: String
	// HTML style attribute
	style: "",

	// title: String
	// HTML title attribute
	title: "",

	// srcNodeRef: DomNode
	//		pointer to original dom node
	srcNodeRef: null,

	// domNode: DomNode
	//		this is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPonit syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// attributeMap: Object
	//		A map of attributes and attachpoints -- typically standard HTML attributes -- to set
	//		on the widget's dom, at the "domNode" attach point, by default.
	//		Other node references can be specified as properties of 'this'
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},  // TODO: add on* handlers?

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(params, srcNodeRef){
		this.create(params, srcNodeRef);
	},

	create: function(params, srcNodeRef){
		// summary:
		//		To understand the process by which widgets are instantiated, it
		//		is critical to understand what other methods create calls and
		//		which of them you'll want to override. Of course, adventurous
		//		developers could override create entirely, but this should
		//		only be done as a last resort.
		//
		//		Below is a list of the methods that are called, in the order
		//		they are fired, along with notes about what they do and if/when
		//		you should over-ride them in your widget:
		//			
		//			postMixInProperties:
		//				a stub function that you can over-ride to modify
		//				variables that may have been naively assigned by
		//				mixInProperties
		//			# widget is added to manager object here
		//			buildRendering
		//				Subclasses use this method to handle all UI initialization
		//				Sets this.domNode.  Templated widgets do this automatically
		//				and otherwise it just uses the source dom node.
		//			postCreate
		//				a stub function that you can over-ride to modify take
		//				actions once the widget has been placed in the UI

		// store pointer to original dom tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects=[];

		// _attaches: String[]
		// 		names of all our dojoAttachPoint variables
		this._attaches=[];

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.
		if(!this.id){
			this.id=dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
		// The placement of these attributes is according to the property mapping in attributeMap.
		// Note special handling for 'style' and 'class' attributes which are lists and can
		// have elements from both old and new structures, and some attributes like "type"
		// cannot be processed this way as they are not mutable.
		if(this.domNode){
			for(var attr in this.attributeMap){
				var mapNode = this[this.attributeMap[attr] || "domNode"];
				var value = this[attr];
				if(typeof value != "object" && (value !== "" || (params && params[attr]))){
					switch(attr){
					case "class":
						dojo.addClass(mapNode, value);
						break;
					case "style":
						if(mapNode.style.cssText){
							mapNode.style.cssText += "; " + value;// FIXME: Opera
						}else{
							mapNode.style.cssText = value;
						}
						break;
					default:
						mapNode.setAttribute(attr, value);
					}
				}
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}	
	},

	postMixInProperties: function(){
		// summary
		//	Called after the parameters to the widget have been read-in,
		//	but before the widget template is instantiated.
		//	Especially useful to set properties that are referenced in the widget template.
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin TemplatedWidget, which overrides this method.
		this.domNode = this.srcNodeRef || dojo.doc.createElement('div');
	},

	postCreate: function(){
		// summary:
		//		Called after a widget's dom has been setup
	},

	startup: function(){
		// summary:
		//		Called after a widget's children, and other widgets on the page, have been created.
		//		Provides an opportunity to manipulate any children before they are displayed
		//		This is useful for composite widgets that need to control or layout sub-widgets
		//		Many layout widgets can use this as a wiring phase
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget and it's descendants. This is the generic
		// 		"destructor" function that all widget users should call to
		// 		cleanly discard with a widget. Once a widget is destroyed, it's
		// 		removed from the manager object.
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		this.destroyDescendants();
		this.destroy();
	},

	destroy: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget, but not its descendants
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?
		this.uninitialize();
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});
		this.destroyRendering(finalize);
		dijit.registry.remove(this.id);
	},

	destroyRendering: function(/*Boolean*/ finalize){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		if(this.bgIframe){
			this.bgIframe.destroy();
			delete this.bgIframe;
		}

		if(this.domNode){
			dojo._destroyElement(this.domNode);
			delete this.domNode;
		}

		if(this.srcNodeRef){
			dojo._destroyElement(this.srcNodeRef);
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.

		// TODO: should I destroy in the reverse order, to go bottom up?
		dojo.forEach(this.getDescendants(), function(widget){ widget.destroy(); });
	},

	uninitialize: function(){
		// summary:
		//		stub function. Over-ride to implement custom widget tear-down
		//		behavior.
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	toString: function(){
		// summary:
		//		returns a string that represents the widget. When a widget is
		//		cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversable
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//	return all the descendant widgets
		var list = dojo.query('[widgetId]', this.domNode);
		return list.map(dijit.byNode);		// Array
	},

	nodesWithKeyClick : ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String*/ event,
			/*String|Function*/ method){

		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		//		Special event: "ondijitclick" triggers on a click or enter-down or space-up
		//		Similar to dojo.connect() but takes three arguments rather than four.
		var handles =[];
		if(event == "ondijitclick"){
			var w = this;
			// add key based click activation for unsupported nodes.
			if(!this.nodesWithKeyClick[obj.nodeName]){
				handles.push(dojo.connect(obj, "onkeydown", this,
					function(e){
						if(e.keyCode == dojo.keys.ENTER){
							return (dojo.isString(method))?
								w[method](e) : method.call(w, e);
						}else if(e.keyCode == dojo.keys.SPACE){
							// stop space down as it causes IE to scroll
							// the browser window
							dojo.stopEvent(e);
						}
			 		}));
				handles.push(dojo.connect(obj, "onkeyup", this,
					function(e){
						if(e.keyCode == dojo.keys.SPACE){
							return dojo.isString(method) ?
								w[method](e) : method.call(w, e);
						}
			 		}));
			}
			event = "onclick";
		}
		handles.push(dojo.connect(obj, event, this, method));

		// return handles for FormElement and ComboBox
		this._connects.push(handles);
		return handles;
	},

	disconnect: function(/*Object*/ handles){
		// summary:
		//		Disconnects handle created by this.connect.
		//		Also removes handle from this widget's list of connects
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i]==handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Checks the DOM to for the text direction for bi-directional support
		// description:
		//		This method cannot be used during widget construction because the widget
		//		must first be connected to the DOM tree.  Parent nodes are searched for the
		//		'dir' attribute until one is found, otherwise left to right mode is assumed.
		//		See HTML spec, DIR attribute for more information.

		if(typeof this._ltr == "undefined"){
			this._ltr = dojo.getComputedStyle(this.domNode).direction != "rtl";
		}
		return this._ltr; //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	}
});

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

dojo.string.pad = function(/*String*/text, /*int*/size, /*String?*/ch, /*boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least 'size' length by
	//		filling with the character 'c' at either the start or end of the
	//		string. Pads at the start, by default.
	// text: the string to pad
	// size: length to provide padding
	// ch: character to pad, defaults to '0'
	// end: adds padding at the end if true, otherwise pads at start

	var out = String(text);
	if(!ch){
		ch = '0';
	}
	while(out.length < size){
		if(end){
			out += ch;
		}else{
			out = ch + out;
		}
	}
	return out;	// String
};

dojo.string.substitute = function(	/*String*/template, 
									/*Object or Array*/map, 
									/*Function?*/transform, 
									/*Object?*/thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// description:
	//		For example,
	//		|	dojo.string.substitute("File '${0}' is not found in directory '${1}'.",["foo.html","/temp"]);
	//		|	dojo.string.substitute("File '${name}' is not found in directory '${info.dir}'.",{name: "foo.html", info: {dir: "/temp"}});
	//		both return
	//			"File 'foo.html' is not found in directory '/temp'."
	// template: 
	//		a string with expressions in the form ${key} to be replaced or
	//		${key:format} which specifies a format function.  NOTE syntax has
	//		changed from %{key}
	// map: where to look for substitutions
	// transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. dojo.string.encodeXML
	// thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g, function(match, key, format){
		var value = dojo.getObject(key,false,map);
		if(format){ value = dojo.getObject(format,false,thisObject)(value);}
		if(transform){ value = transform(value, key); }
		return value.toString();
	}); // string
};

dojo.string.trim = function(/*String*/ str){
	// summary: trims whitespaces from both sides of the string
	// description:
	//	This version of trim() was taken from Steven Levithan's blog: 
	//	http://blog.stevenlevithan.com/archives/faster-trim-javascript.
	//	The short yet good-performing version of this function is 
	//	dojo.trim(), which is part of the base.
	str = str.replace(/^\s+/, '');
	for(var i = str.length - 1; i > 0; i--){
		if(/\S/.test(str.charAt(i))){
			str = str.substring(0, i + 1);
			break;
		}
	}
	return str;	// String
};

}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		RFC3339 (http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified by http://www.w3.org/TR/NOTE-datetime
	//		The following combinations are valid:
	// 			* dates only
	//				yyyy
	//				yyyy-MM
	//				yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//				THH:mm
	//				THH:mm:ss
	//				THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString);
	var result = null;

	if(match){
		match.shift();
		match[1] && match[1]--; // Javascript Date months are 0-based
		match[6] && (match[6] *= 1000); // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}).forEach(function(value, index){
				if(match[index] === undefined){
					match[index] = value;
				}
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0);

		var offset = 0;
		var zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
}

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*Object?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows RFC3339 (http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.
	//
	//	dateObject:
	//		A Date object
	//
	//	object {selector: string, zulu: boolean, milliseconds: boolean}
	//		selector- "date" or "time" for partial formatting of the Date object.
	//			Both date and time will be formatted by default.
	//		zulu- if true, UTC/GMT is used for a timezone
	//		milliseconds- if true, output milliseconds

	var _ = function(n){ return (n < 10) ? "0" + n : n; }
	options = options || {};
	var formattedDate = [];
	var getter = options.zulu ? "getUTC" : "get";
	var date = "";
	if(options.selector != "time"){
		date = [dateObject[getter+"FullYear"](), _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") + 
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
}

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");


dojo.parser = new function(){

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".  interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }"). 
					//  Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value.search(/[^\w\.]+/i) != -1){
						// TODO: "this" here won't work
						value = d.parser._nameAnonFunc(new Function(value), this);
					}
					return d.getObject(value, false);
				}catch(e){ return new Function(); }
			case "array":
				return value.split(/\s*,\s*/);
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};
	
	function getClassInfo(/*String*/ className){
		// className:
		//		fully qualified name (like "dijit.Button")
		// returns:
		//		structure like
		//			{ 
		//				cls: dijit.Button, 
		//				params: { label: "string", disabled: "boolean"}
		//			}

		if(!instanceClasses[className]){
			// get pointer to widget class
			var cls = d.getObject(className);
			if(!d.isFunction(cls)){
				throw new Error("Could not load class '" + className +
					"'. Did you spell the name correctly and use a full path, like 'dijit.form.Button'?");
			}
			var proto = cls.prototype;
	
			// get table of parameter names & types
			var params={};
			for(var name in proto){
				if(name.charAt(0)=="_"){ continue; } 	// skip internal properties
				var defVal = proto[name];
				params[name]=val2type(defVal);
			}

			instanceClasses[className] = { cls: cls, params: params };
		}
		return instanceClasses[className];
	}

	this._functionFromScript = function(script){
		var preamble = "";
		var suffix = "";
		var argsStr = script.getAttribute("args");
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	}

	this.instantiate = function(/* Array */nodes){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a layout method to allow them to connect with
		//		any children		
		var thelist = [];
		d.forEach(nodes, function(node){
			if(!node){ return; }
			var type = node.getAttribute("dojoType");
			if((!type)||(!type.length)){ return; }
			var clsInfo = getClassInfo(type);
			var clazz = clsInfo.cls;
			var ps = clazz._noScript||clazz.prototype._noScript;

			// read parameters (ie, attributes).
			// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
			var params = {};
			var attributes = node.attributes;
			for(var name in clsInfo.params){
				var item = attributes.getNamedItem(name);
				if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
				var value = item.value;
				// Deal with IE quirks for 'class' and 'style'
				switch(name){
				case "class":
					value = node.className;
					break;
				case "style":
					value = node.style && node.style.cssText; // FIXME: Opera?
				}
				var _type = clsInfo.params[name];
				params[name] = str2obj(value, _type);
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			if(!ps){
				var connects = [],	// functions to connect after instantiation
					calls = [];		// functions to call after instantiation

				d.query("> script[type^='dojo/']", node).orphan().forEach(function(script){
					var event = script.getAttribute("event"),
						type = script.getAttribute("type"),
						nf = d.parser._functionFromScript(script);
					if(event){
						if(type == "dojo/connect"){
							connects.push({event: event, func: nf});
						}else{
							params[event] = nf;
						}
					}else{
						calls.push(nf);
					}
				});
			}

			var markupFactory = clazz["markupFactory"];
			if(!markupFactory && clazz["prototype"]){
				markupFactory = clazz.prototype["markupFactory"];
			}
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			var jsname = node.getAttribute("jsId");
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			if(!ps){
				dojo.forEach(connects, function(connect){
					dojo.connect(instance, connect.event, null, connect.func);
				});
				dojo.forEach(calls, function(func){
					func.call(instance);
				});
			}
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		d.forEach(thelist, function(instance){
			if(	instance  && 
				(instance.startup) && 
				((!instance.getParent) || (!instance.getParent()))
			){
				instance.startup();
			}
		});
		return thelist;
	};

	this.parse = function(/*DomNode?*/ rootNode){
		// summary:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them Searches for
		//		dojoType="qualified.class.name"
		var list = d.query('[dojoType]', rootNode);
		// go build the object instances
		var instances = this.instantiate(list);
		return instances;
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){ 
		if(djConfig["parseOnLoad"] == true){
			dojo.parser.parse(); 
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.exists("dijit.wai.onload") && (dijit.wai.onload === dojo._loaders[0])){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

//TODO: ported from 0.4.x Dojo.  Can we reduce this?
dojo.parser._anonCtr = 0;
dojo.parser._anon = {}; // why is this property required?
dojo.parser._nameAnonFunc = function(/*Function*/anonFuncPtr, /*Object*/thisObj){
	// summary:
	//		Creates a reference to anonFuncPtr in thisObj with a completely
	//		unique name. The new name is returned as a String. 
	var jpn = "$joinpoint";
	var nso = (thisObj|| dojo.parser._anon);
	if(dojo.isIE){
		var cn = anonFuncPtr["__dojoNameCache"];
		if(cn && nso[cn] === anonFuncPtr){
			return anonFuncPtr["__dojoNameCache"];
		}
	}
	var ret = "__"+dojo.parser._anonCtr++;
	while(typeof nso[ret] != "undefined"){
		ret = "__"+dojo.parser._anonCtr++;
	}
	nso[ret] = anonFuncPtr;
	return ret; // String
}

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		mixin for widgets that are instantiated from a template

		// templateNode: DomNode
		//		a node that represents the widget template. Pre-empts both templateString and templatePath.
		templateNode: null,

		// templateString String:
		//		a string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		templateString: null,

		// templatePath: String
		//	Path to template (HTML file) for this widget
		templatePath: null,

		// widgetsInTemplate Boolean:
		//		should we parse the template to find widgets that might be
		//		declared in markup inside it? false by default.
		widgetsInTemplate: false,

		// containerNode DomNode:
		//		holds child elements. "containerNode" is generally set via a
		//		dojoAttachPoint assignment and it designates where children of
		//		the src dom node will be placed
		containerNode: null,

		// skipNodeCache Boolean:
		//		if using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				var className = this.declaredClass, _this = this;
				// Cache contains a string because we need to do property replacement
				// do the property replacement
				var tstr = dojo.string.substitute(cached, this, function(value, key){
					if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
					if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
					if(!value){ return ""; }

					// Substitution keys beginning with ! will skip the transform step,
					// in case a user wishes to insert unescaped markup, e.g. ${!foo}
					return key.charAt(0) == "!" ? value :
						// Safer substitution, see heading "Attribute values" in
						// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
						value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
				}, this);

				node = dijit._Templated._createNodesFromText(tstr)[0];
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points which should be defined on the template node.
			this._attachTemplateNodes(node);

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(node, source);
			}

			this.domNode = node;
			if(this.widgetsInTemplate){
				var childWidgets = dojo.parser.parse(node);
				this._attachTemplateNodes(childWidgets, function(n,p){
					return n[p];
				});
			}

			this._fillContent(source);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		relocate source contents to templated container node
			//		this.containerNode must be able to receive children, or exceptions will be thrown
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent	
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x=dojo.isArray(rootNode)?0:-1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode,'dojoType')){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while(point=points.shift()){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while(event=events.shift()){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}

			}
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String
	//		the URL to get the template from. dojo.uri.Uri is often passed as well.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL
	// Returns:
	//	Either string (if there are ${} variables that need to be replaced) or just
	//	a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		return cached;
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dijit._Templated._sanitizeTemplateString(dojo._getText(templatePath));
	}

	templateString = dojo.string.trim(templateString);

	if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		return (tmplts[key] = dijit._Templated._createNodesFromText(templateString)[0]); //Node
	}
};

dijit._Templated._sanitizeTemplateString = function(/*String*/tString){
	// summary: 
	//		Strips <?xml ...?> declarations so that external SVG and XML
	// 		documents can be added to a document without worry. Also, if the string
	//		is an HTML document, only the part inside the body tag is returned.
	if(tString){
		tString = tString.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
		var matches = tString.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
		if(matches){
			tString = matches[1];
		}
	}else{
		tString = "";
	}
	return tString; //String
};


if(dojo.isIE){
	dojo.addOnUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(!isNaN(value.nodeType)){ // isNode equivalent
				dojo._destroyElement(value);
			}
			delete cache[key];
		}
	});
}

(function(){
	var tagMap = {
		cell: {re: /^<t[dh][\s\r\n>]/i, pre: "<table><tbody><tr>", post: "</tr></tbody></table>"},
		row: {re: /^<tr[\s\r\n>]/i, pre: "<table><tbody>", post: "</tbody></table>"},
		section: {re: /^<(thead|tbody|tfoot)[\s\r\n>]/i, pre: "<table>", post: "</table>"}
	};

	// dummy container node used temporarily to hold nodes being created
	var tn;

	dijit._Templated._createNodesFromText = function(/*String*/text){
		// summary:
		//	Attempts to create a set of nodes based on the structure of the passed text.

		if(!tn){
			tn = dojo.doc.createElement("div");
			tn.style.display="none";
			dojo.body().appendChild(tn);
		}
		var tableType = "none";
		var rtext = text.replace(/^\s+/, "");
		for(var type in tagMap){
			var map = tagMap[type];
			if(map.re.test(rtext)){
				tableType = type;
				text = map.pre + text + map.post;
				break;
			}
		}

		tn.innerHTML = text;
		if(tn.normalize){
			tn.normalize();
		}

		var tag = { cell: "tr", row: "tbody", section: "table" }[tableType];
		var _parent = (typeof tag != "undefined") ?
						tn.getElementsByTagName(tag)[0] :
						tn;

		var nodes = [];
		while(_parent.firstChild){
			nodes.push(_parent.removeChild(_parent.firstChild));
		}
		tn.innerHTML="";
		return nodes;	//	Array
	}
})();

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
})

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: 200,

		templateString:"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\r\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\r\n\t<div class=\"dijitTooltipConnector\"></div>\r\n</div>\r\n",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
			// summary:
			//	Display tooltip w/specified contents to right specified node
			//	(To left if there's no space on the right, or if LTR==right)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			// Firefox bug. when innerHTML changes to be shorter than previous
			// one, the node size will not be updated until it moves.
			this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";

			// position the element and change CSS according to position	
			var align = this.isLeftToRight() ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
			this.domNode.className="dijitTooltip dijitTooltip" + (pos.corner=='BL' ? "Right" : "Left");//FIXME: might overwrite class

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		_onShow: function(){
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary: hide the tooltip
			if(!this.aroundNode || this.aroundNode !== aroundNode){
				return;
			}
			if(this._onDeck){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
				return;
			}
			this.fadeIn.stop();
			this.isShowingNow = false;
			this.aroundNode = null;
			this.fadeOut.play();
		},

		_onHide: function(){
			this.domNode.style.cssText="";	// to position offscreen again
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
	// summary:
	//	Display tooltip w/specified contents to right specified node
	//	(To left if there's no space on the right, or if LTR==right)
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode);
};

dijit.hideTooltip = function(aroundNode){
	// summary: hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String[]
		//		Id(s) of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		connectId: [],

		postCreate: function(){
			if(this.srcNodeRef){
				this.srcNodeRef.style.display = "none";
			}

			this._connectNodes = [];
			
			dojo.forEach(this.connectId, function(id) {
				var node = dojo.byId(id);
				if (node) {
					this._connectNodes.push(node);
					dojo.forEach(["onMouseOver", "onMouseOut", "onFocus", "onBlur", "onHover", "onUnHover"], function(event){
						this.connect(node, event.toLowerCase(), "_"+event);
					}, this);
					if(dojo.isIE){
						// BiDi workaround
						node.style.zoom = 1;
					}
				}
			}, this);
		},

		_onMouseOver: function(/*Event*/ e){
			this._onHover(e);
		},

		_onMouseOut: function(/*Event*/ e){
			if(dojo.isDescendant(e.relatedTarget, e.target)){
				// false event; just moved from target to target child; ignore.
				return;
			}
			this._onUnHover(e);
		},

		_onFocus: function(/*Event*/ e){
			this._focus = true;
			this._onHover(e);
		},
		
		_onBlur: function(/*Event*/ e){
			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// keep a tooltip open if the associated element has focus
			if(this._focus){ return; }
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary: display the tooltip; usually not called directly.
			target = target || this._connectNodes[0];
			if(!target){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target);
			
			this._connectNode = target;
		},

		close: function(){
			// summary: hide the tooltip; usually not called directly.
			dijit.hideTooltip(this._connectNode);
			delete this._connectNode;
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		uninitialize: function(){
			this.close();
		}
	}
);

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Contained",
	null,
	{
		// summary
		//		Mixin for widgets that are children of a container widget

		getParent: function(){
			// summary:
			//		returns the parent widget of this widget, assuming the parent
			//		implements dijit._Container
			for(var p=this.domNode.parentNode; p; p=p.parentNode){
				var id = p.getAttribute && p.getAttribute("widgetId");
				if(id){
					var parent = dijit.byId(id);
					return parent.isContainer ? parent : null;
				}
			}
			return null;
		},

		_getSibling: function(which){
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			if(!node){ return null; } // null
			var id = node.getAttribute("widgetId");
			return dijit.byId(id);
		},

		getPreviousSibling: function(){
			// summary:
			//		returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous");
		},

		getNextSibling: function(){
			// summary:
			//		returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next");
		}
	}
);

dojo.declare("dijit._Container",
	null,
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer

		isContainer: true,

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Process the given child widget, inserting it's dom node as
			//		a child of our dom node

			if(insertIndex === undefined){
				insertIndex = "last";
			}
			var refNode = this.containerNode || this.domNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = dojo.query("> [widgetid]", refNode);
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1]; insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget*/ widget){
			// summary:
			//		removes the passed widget instance from this widget but does
			//		not destroy it
			var node = widget.domNode;
			node.parentNode.removeChild(node);	// detach but don't destroy
		},

		_nextElement: function(node){
			do{
				node = node.nextSibling;
			}while(node && node.nodeType != 1);
			return node;
		},

		_firstElement: function(node){
			node = node.firstChild;
			if(node && node.nodeType != 1){
				node = this._nextElement(node);
			}
			return node;
		},

		getChildren: function(){
			// summary:
			//		Returns array of children widgets
			return dojo.query("> [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children
			var cn = this.containerNode || this.domNode;
			return !!this._firstElement(cn); // Boolean
		},

		_getSiblingOfChild: function(/*Widget*/ child, /*int*/ dir){
			// summary:
			//		get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			var node = child.domNode;
			var which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node ? dijit.byNode(node) : null;
		}
	}
);

dojo.declare("dijit._KeyNavContainer",
	[dijit._Container],
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().

/*=====
		// focusedChild: Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*Array*/ prevKeyCodes, /*Array*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: Array
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: Array
			//		Key codes for navigating to the next child.

			var keyCodes = this._keyNavCodes = {};
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			if(dojo.isIE){
				this.connect(this.domNode, "onactivate", "_onContainerFocus");
				this.connect(this.domNode, "ondeactivate", "_onContainerBlur");
			}else{
				this.connect(this.domNode, "onfocus", "_onContainerFocus");
				this.connect(this.domNode, "onblur", "_onContainerBlur");
			}
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_setTabIndexMinusOne"));
		},

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary: Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._setTabIndexMinusOne(widget);
		},

		focus: function(){
			// summary: Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary: Focus the first focusable child in the container.
			this.focusChild(this._getFirstFocusableChild());
		},

		focusNext: function(){
			// summary: Focus the next widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasNextFocalNode
					&& this.focusedChild.hasNextFocalNode()){
				this.focusedChild.focusNext();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			if(child.getFocalNodes){
				this.focusChild(child, child.getFocalNodes()[0]);
			}else{
				this.focusChild(child);
			}
		},

		focusPrev: function(){
			// summary: Focus the previous widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasPrevFocalNode
					&& this.focusedChild.hasPrevFocalNode()){
				this.focusedChild.focusPrev();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			if(child.getFocalNodes){
				var nodes = child.getFocalNodes();
				this.focusChild(child, nodes[nodes.length-1]);
			}else{
				this.focusChild(child);
			}
		},

		focusChild: function(/*Widget*/ widget, /*Node?*/ node){
			// summary: Focus widget. Optionally focus 'node' within widget.
			if(widget){
				if(this.focusedChild && widget !== this.focusedChild){
					this._onChildBlur(this.focusedChild);
				}
				this.focusedChild = widget;
				if(node && widget.focusFocalNode){
					widget.focusFocalNode(node);
				}else{
					widget.focus();
				}
			}
		},

		_setTabIndexMinusOne: function(/*Widget*/ widget){
			if(widget.getFocalNodes){
				dojo.forEach(widget.getFocalNodes(), function(node){
					node.setAttribute("tabIndex", -1);
				});
			}else{
				(widget.focusNode || widget.domNode).setAttribute("tabIndex", -1);
			}
		},

		_onContainerFocus: function(evt){
			this.domNode.setAttribute("tabIndex", -1);
			if(evt.target === this.domNode){
				this.focusFirstChild();
			}else{
				var widget = dijit.getEnclosingWidget(evt.target);
				if(widget && widget.isFocusable()){
					this.focusedChild = widget;
				}
			}
		},

		_onContainerBlur: function(evt){
			if(this.tabIndex){
				this.domNode.setAttribute("tabIndex", this.tabIndex);
			}
		},

		_onContainerKeypress: function(evt){
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
		},

		_getFirstFocusableChild: function(){
			return this._getNextFocusableChild(null, 1);
		},

		_getNextFocusableChild: function(child, dir){
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;
				}
				child = this._getSiblingOfChild(child, dir);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.Toolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Toolbar"] = true;
dojo.provide("dijit.Toolbar");





dojo.declare(
	"dijit.Toolbar",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
//			'<table style="table-layout: fixed" class="dijitReset dijitToolbarTable">' + // factor out style
//				'<tr class="dijitReset" dojoAttachPoint="containerNode"></tr>'+
//			'</table>' +
		'</div>',

	tabIndex: "0",

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	}
}
);

// Combine with dijit.MenuSeparator??
dojo.declare(
	"dijit.ToolbarSeparator",
	[ dijit._Widget, dijit._Templated ],
{
	// summary
	//	A line between two menu items
	templateString: '<div class="dijitToolbarSeparator dijitInline"></div>',
	postCreate: function(){ dojo.setSelectable(this.domNode, false); },
	isFocusable: function(){ return false; }
});

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");




dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated],
{
	/*
	Summary:
		FormElement widgets correspond to native HTML elements such as <input> or <button> or <select>.
		Each FormElement represents a single input value, and has a (possibly hidden) <input> element,
		to which it serializes its input value, so that form submission (either normal submission or via FormBind?)
		works as expected.

		All these widgets should have these attributes just like native HTML input elements.
		You can set them during widget construction, but after that they are read only.

		They also share some common methods.
	*/

	// baseClass: String
	//		Root CSS class of the widget (ex: dijitTextBox), used to add CSS classes of widget
	//		(ex: "dijitTextBox dijitTextBoxInvalid dijitTextBoxFocused dijitTextBoxInvalidFocused")
	//		See _setStateClass().
	baseClass: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// id: String
	//		Corresponds to the native HTML <input> element's attribute.
	//		Also becomes the id for the widget.
	id: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically 
	// require the 'name' attribute at element creation time.
	attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
		{id:"focusNode", tabIndex:"focusNode", alt:"focusNode"}),

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Set disabled state of widget.

		this.domNode.disabled = this.disabled = disabled;
		if(this.focusNode){
			this.focusNode.disabled = disabled;
		}
		if(disabled){
			//reset those, because after the domNode is disabled, we can no longer receive
			//mouse related events, see #4200
			this._hovering = false;
			this._active = false;
		}
		dijit.setWaiState(this.focusNode || this.domNode, "disabled", disabled);
		this._setStateClass();
	},


	_onMouse : function(/*Event*/ event){
		// summary:
		//	Sets _hovering, _active, and stateModifier properties depending on mouse state,
		//	then calls setStateClass() to set appropriate CSS classes for this.domNode.
		//
		//	To get a different CSS class for hover, send onmouseover and onmouseout events to this method.
		//	To get a different CSS class while mouse button is depressed, send onmousedown to this method.

		var mouseNode = event.target;
		if(mouseNode && mouseNode.getAttribute){
			this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
		}

		if(!this.disabled){
			switch(event.type){
				case "mouseenter" :	
				case "mouseover" :
					this._hovering = true;
					break;

				case "mouseout" :	
				case "mouseleave" :	
					this._hovering = false;
					break;

				case "mousedown" :
					this._active = true;
					// set a global event to handle mouseup, so it fires properly
					//	even if the cursor leaves the button
					var self = this;
					// #2685: use this.connect and disconnect so destroy works properly
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						self._active = false;
						self._setStateClass();
						self.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	isFocusable: function(){
		return !this.disabled && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		dijit.focus(this.focusNode);
	},

	_setStateClass: function(){
		// summary
		//	Update the visual state of the widget by setting the css classes on this.domNode
		//  (or this.stateNode if defined) by combining this.baseClass with
		//	various suffixes that represent the current widget state(s).
		//
		//	In the case where a widget has multiple
		//	states, it sets the class based on all possible
		//  combinations.  For example, an invalid form widget that is being hovered
		//	will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//	For complex widgets with multiple regions, there can be various hover/active states,
		//	such as "Hover" or "CloseButtonHover" (for tab buttons).
		//	This is controlled by a stateModifier="CloseButton" attribute on the close button node.
		//
		//	The widget may have one or more of the following states, determined
		//	by this.state, this.checked, this.valid, and this.selected:
		//		Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//		Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//		Selected - ex: currently selected tab will have this.selected==true
		//
		//	In addition, it may have at most one of the following states,
		//	based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//		Disabled	- if the widget is disabled
		//		Active		- if the mouse (or space/enter key?) is being pressed down
		//		Focused		- if the widget has focus
		//		Hover		- if the mouse is over the widget
		//
		//	(even if multiple af the above conditions are true we only pick the first matching one)


		// Get original (non state related, non baseClass related) class specified in template
		if(!("staticClass" in this)){
			this.staticClass = (this.stateNode||this.domNode).className;
		}

		// Compute new set of classes
		var classes = [ this.baseClass ];

		function multiply(modifier){
			classes=classes.concat(dojo.map(classes, function(c){ return c+modifier; }));
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		// Only one of these four can be applied.
		// Active trumps Focused, Focused trumps Hover, and Disabled trumps all.
		if(this.disabled){
			multiply("Disabled");
		}else if(this._active){
			multiply(this.stateModifier+"Active");
		}else{
			if(this._focused){
				multiply("Focused");
			}
			if((this.stateModifier || !this._focused) && this._hovering){
				multiply(this.stateModifier+"Hover");
			}
		}
		(this.stateNode || this.domNode).className = this.staticClass + " " + classes.join(" ");
	},

	onChange: function(newValue){
		// summary: callback when value is changed
	},

	postCreate: function(){
		this.setValue(this.value, null); // null reserved for initial value
		this.setDisabled(this.disabled);
		this._setStateClass();
	},

	setValue: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary: set the value of the widget.
		this._lastValue = newValue;
		dijit.setWaiState(this.focusNode || this.domNode, "valuenow", this.forWaiValuenow());
		if(priorityChange === undefined){ priorityChange = true; } // setValue with value only should fire onChange
		if(this._lastValueReported == undefined && priorityChange === null){ // don't report the initial value
			this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange) && 
			((newValue && newValue.toString)?newValue.toString():newValue) !== ((this._lastValueReported && this._lastValueReported.toString)?this._lastValueReported.toString():this._lastValueReported)){
			this._lastValueReported = newValue;
			this.onChange(newValue);
		}
	},

	getValue: function(){
		// summary: get the value of the widget.
		return this._lastValue;
	},

	undo: function(){
		// summary: restore the value to the last value passed to onChange
		this.setValue(this._lastValueReported, false);
	},

	_onKeyPress: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !e.shiftKey && !e.ctrlKey && !e.altKey){
			var v = this.getValue();
			var lv = this._lastValueReported;
			// Equality comparison of objects such as dates are done by reference so
			// two distinct objects are != even if they have the same data. So use
			// toStrings in case the values are objects.
			if((typeof lv != "undefined") && ((v!==null && v.toString)?v.toString():null) !== lv.toString()){	
				this.undo();
				dojo.stopEvent(e);
				return false;
			}
		}
		return true;
	},

	forWaiValuenow: function(){
		// summary: returns a value, reflecting the current state of the widget,
		//		to be used for the ARIA valuenow.
		// 		This method may be overridden by subclasses that want
		// 		to use something other than this.getValue() for valuenow
		return this.getValue();
	}
});

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");




dojo.declare("dijit.form.Button", dijit.form._FormWidget, {
/*
 * usage
 *	<button dojoType="button" onClick="...">Hello world</button>
 *
 *  var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
 *	dojo.body().appendChild(button1.domNode);
 */
	// summary
	//	Basically the same thing as a normal HTML button, but with special styling.

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \r\n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\r\n\t\t></button\r\n\t></div\r\n></div>\r\n",

	// TODO: set button's title to this.containerNode.innerText

	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel == false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
		}
		this.inherited(arguments);
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel == false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	}		
});

/*
 * usage
 *	<button dojoType="DropDownButton" label="Hello world"><div dojotype=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
 *	dojo.body().appendChild(button1);
 */
dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container], {
	// summary
	//		push the button and a menu shows up

	baseClass : "dijitDropDownButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey\"\r\n\t><div class='dijitRight'>\r\n\t<button class=\"dijitStretch dijitButtonNode dijitButtonContents\" type=\"${type}\"\r\n\t\tdojoAttachPoint=\"focusNode,titleNode\" waiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\r\n\t\t><div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div\r\n\t\t><span class=\"dijitButtonText\" \tdojoAttachPoint=\"containerNode,popupStateNode\"\r\n\t\tid=\"${id}_label\">${label}</span\r\n\t\t><span class='dijitA11yDownArrow'>&#9660;</span>\r\n\t</button>\r\n</div></div>\r\n",

	_fillContent: function(){
		// my inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.
		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dojo.body().appendChild(this.dropDown.domNode);
		this.dropDown.domNode.style.display="none";
	},

	_onArrowClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks on menu popup node
		if(this.disabled){ return; }
		this._toggleDropDown();
	},

	_onDropDownClick: function(/*Event*/ e){
		// on Firefox 2 on the Mac it is possible to fire onclick
		// by pressing enter down on a second element and transferring
		// focus to the DropDownButton;
		// we want to prevent opening our menu in this situation
		// and only do so if we have seen a keydown on this button;
		// e.detail != 0 means that we were fired by mouse
		var isMacFFlessThan3 = dojo.isFF && dojo.isFF < 3
			&& navigator.appVersion.indexOf("Macintosh") != -1;
		if(!isMacFFlessThan3 || e.detail != 0 || this._seenKeydown){
			this._onArrowClick(e);
		}
		this._seenKeydown = false;
	},

	_onDropDownKeydown: function(/*Event*/ e){
		this._seenKeydown = true;
	},

	_onDropDownBlur: function(/*Event*/ e){
		this._seenKeydown = false;
	},

	_onKey: function(/*Event*/ e){
		// summary: callback when the user presses a key on menu popup node
		if(this.disabled){ return; }
		if(e.keyCode == dojo.keys.DOWN_ARROW){
			if(!this.dropDown || this.dropDown.domNode.style.display=="none"){
				dojo.stopEvent(e);
				return this._toggleDropDown();
			}
		}
	},

	_onBlur: function(){
		// summary: called magically when focus has shifted away from this widget and it's dropdown
		this._closeDropDown();
		// don't focus on button.  the user has explicitly focused on something else.
	},

	_toggleDropDown: function(){
		// summary: toggle the drop-down widget; if it is up, close it, if not, open it
		if(this.disabled){ return; }
		dijit.focus(this.popupStateNode);
		var dropDown = this.dropDown;
		if(!dropDown){ return false; }
		if(!dropDown.isShowingNow){
			// If there's an href, then load that first, so we don't get a flicker
			if(dropDown.href && !dropDown.isLoaded){
				var self = this;
				var handler = dojo.connect(dropDown, "onLoad", function(){
					dojo.disconnect(handler);
					self._openDropDown();
				});
				dropDown._loadCheck(true);
				return;
			}else{
				this._openDropDown();
			}
		}else{
			this._closeDropDown();
		}
	},

	_openDropDown: function(){
		var dropDown = this.dropDown;
		var oldWidth=dropDown.domNode.style.width;
		var self = this;

		dijit.popup.open({
			parent: this,
			popup: dropDown,
			around: this.domNode,
			orient: this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'}
				: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
			onExecute: function(){
				self._closeDropDown(true);
			},
			onCancel: function(){
				self._closeDropDown(true);
			},
			onClose: function(){
				dropDown.domNode.style.width = oldWidth;
				self.popupStateNode.removeAttribute("popupActive");
				this._opened = false;
			}
		});
		if(this.domNode.offsetWidth > dropDown.domNode.offsetWidth){
			var adjustNode = null;
			if(!this.isLeftToRight()){
				adjustNode = dropDown.domNode.parentNode;
				var oldRight = adjustNode.offsetLeft + adjustNode.offsetWidth;
			}
			// make menu at least as wide as the button
			dojo.marginBox(dropDown.domNode, {w: this.domNode.offsetWidth});
			if(adjustNode){
				adjustNode.style.left = oldRight - this.domNode.offsetWidth + "px";
			}
		}
		this.popupStateNode.setAttribute("popupActive", "true");
		this._opened=true;
		if(dropDown.focus){
			dropDown.focus();
		}
		// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
	},
	
	_closeDropDown: function(/*Boolean*/ focus){
		if(this._opened){
			dijit.popup.close(this.dropDown);
			if(focus){ this.focus(); }
			this._opened = false;			
		}
	}
});

/*
 * usage
 *	<button dojoType="ComboButton" onClick="..."><span>Hello world</span><div dojoType=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
 *	dojo.body().appendChild(button1.domNode);
 */
dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary
	//		left side is normal button, right side displays menu
	templateString:"<table class='dijit dijitReset dijitInline dijitLeft'\r\n\tcellspacing='0' cellpadding='0'\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\">\r\n\t<tr>\r\n\t\t<td\tclass=\"dijitStretch dijitButtonContents dijitButtonNode\"\r\n\t\t\ttabIndex=\"${tabIndex}\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"  dojoAttachPoint=\"titleNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\">\r\n\t\t\t<div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div>\r\n\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t</td>\r\n\t\t<td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton'\r\n\t\t\tdojoAttachPoint=\"popupStateNode,focusNode\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onArrowClick, onkeypress:_onKey\"\r\n\t\t\tstateModifier=\"DownArrow\"\r\n\t\t\ttitle=\"${optionsTitle}\" name=\"${name}\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\r\n\t\t><div waiRole=\"presentation\">&#9660;</div>\r\n\t</td></tr>\r\n</table>\r\n",

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{id:"", name:""}),

	// optionsTitle: String
	//  text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	_focusedNode: null,

	postCreate: function(){
		this.inherited(arguments);
		this._focalNodes = [this.titleNode, this.popupStateNode];
		dojo.forEach(this._focalNodes, dojo.hitch(this, function(node){
			if(dojo.isIE){
				this.connect(node, "onactivate", this._onNodeFocus);
			}else{
				this.connect(node, "onfocus", this._onNodeFocus);
			}
		}));
	},

	focusFocalNode: function(node){
		// summary: Focus the focal node node.
		this._focusedNode = node;
		dijit.focus(node);
	},

	hasNextFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node following the focused one.
		//		False is returned if the last node has focus.
		return this._focusedNode !== this.getFocalNodes()[1];
	},

	focusNext: function(){
		// summary: Focus the focal node following the current node with focus
		//		or the first one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 1 : 0];
		dijit.focus(this._focusedNode);
	},

	hasPrevFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node before the focused one.
		//		False is returned if the first node has focus.
		return this._focusedNode !== this.getFocalNodes()[0];
	},

	focusPrev: function(){
		// summary: Focus the focal node before the current node with focus
		//		or the last one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 0 : 1];
		dijit.focus(this._focusedNode);
	},

	getFocalNodes: function(){
		// summary: Returns an array of focal nodes for this widget.
		return this._focalNodes;
	},

	_onNodeFocus: function(evt){
		this._focusedNode = evt.currentTarget;
	},

	_onBlur: function(evt){
		this.inherited(arguments);
		this._focusedNode = null;
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary
	//	A button that can be in two states (checked or not).
	//	Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	_clicked: function(/*Event*/ evt){
		this.setChecked(!this.checked);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary
		//	Programatically deselect the button
		this.checked = checked;
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", this.checked);
		this._setStateClass();		
		this.onChange(checked);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.TIPButton");




dojo.declare("ibm.tivoli.tip.dijit.TIPButton",
              dijit.form._FormWidget,
{

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t>\r\n\t\t\t<div dojoAttachPoint=\"buttonDiv\">\r\n              <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\r\n               <tr>\r\n                <td style=\"white-space: nowrap;\">\r\n\t\t\t\t  <div align=\"center\">\r\n\t\t\t\t\t<span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\">\r\n\t \t\t\t\t\t<span class=\"dijitToggleButtonIconChar\">&#10003</span>\r\n \t\t\t\t\t</span>\r\n\t\t\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t\t\t  </div>\r\n\t\t\t\t</td>\r\n               </tr>\r\n              </table>\r\n             </div>\t\t\t\r\n\t\t</button>\r\n\t</div>\r\n</div>\r\n",	

	constructor: function () {
	},
	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel === false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
    } else {
        // need to set button div width to 100% because
        // firefox has issues fitting the label into the button
        this.buttonDiv.style.width="100%";
    }
		this.inherited(arguments);
		//this.connectEvents();
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel === false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	},
	connectEvents: function () {
		dojo.connect (this, "onmousedown", this, this._onMouseDown);
		dojo.connect (this, "onmouseup", this, this._onMouseUp);
		dojo.connect (this, "onkeydown", this, this._onMouseDown);
		dojo.connect (this, "onkeydown", this, this._onMouseUp);
		
		dojo.connect (this, "onmouseover", this, this._onFocus);
		dojo.connect (this, "onmouseout", this, this._onBlur);
		dojo.connect (this, "onfocus", this, this._onFocus);
		dojo.connect (this, "onblur", this, this._onBlur);
		
	},
	_onMouseDown: function () {
		this.buttonDiv.className = "button-down";
	},
	_onMouseUp: function () {
		this.buttonDiv.className = "button-up";
	},
 	_onFocus: function () {
		this.buttonDiv.className = "button-over";
 	},
 	_onBlur: function () {
		this.buttonDiv.className = "button-out";
 	},
 	destroy: function () {
 		console.log ("enter TIPButton.destroy");
 		this.inherited (arguments);
 		console.log ("exit TIPButton.destroy");
 	}

});

}

if(!dojo._hasResource["dijit.form.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Form"] = true;
dojo.provide("dijit.form.Form");




dojo.declare("dijit.form._FormMixin", null,
	{
		/*
		summary:
			Widget corresponding to <form> tag, for validation and serialization

		usage:
			<form dojoType="dijit.form.Form" id="myForm">
				Name: <input type="text" name="name" />
			</form>
			myObj={name: "John Doe"};
			dijit.byId('myForm').setValues(myObj);

			myObj=dijit.byId('myForm').getValues();
		TODO:
		* Repeater
		* better handling for arrays.  Often form elements have names with [] like
		* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])

		*/

		// HTML <FORM> attributes

		action: "",
		method: "",
		enctype: "",
		name: "",
		"accept-charset": "",
		accept: "",
		target: "",

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{action: "", method: "", enctype: "", "accept-charset": "", accept: "", target: ""}),

		// execute: Function
		//	User defined function to do stuff when the user hits the submit button
		execute: function(/*Object*/ formContents){},

		// onCancel: Function
		//	Callback when user has canceled dialog, to notify container
		//	(user shouldn't override)
		onCancel: function(){},

		// onExecute: Function
		//	Callback when user is about to execute dialog, to notify container
		//	(user shouldn't override)
		onExecute: function(){},

		templateString: "<form dojoAttachPoint='containerNode' dojoAttachEvent='onsubmit:_onSubmit' name='${name}' enctype='multipart/form-data'></form>",

		_onSubmit: function(/*event*/e) {
			// summary: callback when user hits submit button
			dojo.stopEvent(e);
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.getValues());
		},

		submit: function() {
			// summary: programatically submit form
			this.containerNode.submit();
		},

		setValues: function(/*object*/obj) {
			// summary: fill in form values from a JSON structure

			// generate map from name --> [list of widgets with that name]
			var map = {};
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			// call setValue() or setChecked() for each widget, according to obj
			for(var name in map){
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(widgets[0].setChecked){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.setChecked(dojo.indexOf(values, w.value) != -1);
					});
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.setValue(values[i]);
					});					
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets

			dojo.forEach(this.containerNode.elements, function(element){
				if (element.name == ''){return};	// like "continue"	
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined") {
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if (typeof(myObj) == "undefined") {
					return;		// like "continue"
				}
				if (typeof(myObj[name]) == "undefined" && this.ignoreNullValues) {
					return;		// like "continue"
				}

				// TODO: widget values (just call setValue() on the widget)

				switch(element.type) {
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val==element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name]==element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function() {
			// summary: generate JSON structure from form values

			// get widget values
			var obj = {};
			dojo.forEach(this.getDescendants(), function(widget){
				var value = widget.getValue ? widget.getValue() : widget.value;
				var name = widget.name;
				if(!name){ return; }

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(widget.setChecked){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(widget.checked){
							dojo.setObject(name, value, obj);
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(widget.checked){
							ary.push(value);
						}
					}
				}else{
					// plain input
					dojo.setObject(name, value, obj);
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if (!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined") {
						myObj[nameA[0]]={}
					} // if

					if (nameA.length == 1) {
						myObj=myObj[nameA[0]];
					} else {
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if ((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type=="radio" && elm.checked)) {
					if(name == name.split("[")[0]) {
						myObj[name]=elm.value;
					} else {
						// can not set value when there is no name
					}
				} else if (elm.type == "checkbox" && elm.checked) {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if (elm.type == "select-multiple") {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					for (var jdx=0,len3=elm.options.length; jdx<len3; ++jdx) {
						if (elm.options[jdx].selected) {
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function() {
	 		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 		// make sure that every widget that has a validator function returns true
	 		return dojo.every(this.getDescendants(), function(widget){
	 			return !widget.isValid || widget.isValid();
	 		});
		}
	});

dojo.declare(
	"dijit.form.Form",
	[dijit._Widget, dijit._Templated, dijit.form._FormMixin],
	null
);

}

if(!dojo._hasResource["dojo.io.iframe"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.io.iframe"] = true;
dojo.provide("dojo.io.iframe");

dojo.io.iframe = {
	create: function(/*String*/fname, /*String*/onloadstr, /*String?*/uri){
		//	summary:
		//		Creates a hidden iframe in the page. Used mostly for IO
		//		transports.  You do not need to call this to start a
		//		dojo.io.iframe request. Just call send().
		//	fname: String
		//		The name of the iframe. Used for the name attribute on the
		//		iframe.
		//	onloadstr: String
		//		A string of JavaScript that will be executed when the content
		//		in the iframe loads.
		//	uri: String
		//		The value of the src attribute on the iframe element. If a
		//		value is not given, then dojo/resources/blank.html will be
		//		used.
		if(window[fname]){ return window[fname]; }
		if(window.frames[fname]){ return window.frames[fname]; }
		var cframe = null;
		var turi = uri;
		if(!turi){
			if(djConfig["useXDomain"] && !djConfig["dojoBlankHtmlUrl"]){
				console.debug("dojo.io.iframe.create: When using cross-domain Dojo builds,"
					+ " please save dojo/resources/blank.html to your domain and set djConfig.dojoBlankHtmlUrl"
					+ " to the path on your domain to blank.html");
			}
			turi = (djConfig["dojoBlankHtmlUrl"]||dojo.moduleUrl("dojo", "resources/blank.html"));
		}
		var ifrstr = dojo.isIE ? '<iframe name="'+fname+'" src="'+turi+'" onload="'+onloadstr+'">' : 'iframe';
		cframe = dojo.doc.createElement(ifrstr);
		with(cframe){
			name = fname;
			setAttribute("name", fname);
			id = fname;
		}
		dojo.body().appendChild(cframe);
		window[fname] = cframe;
	
		with(cframe.style){
			if(dojo.isSafari < 3){
				//We can't change the src in Safari 2.0.3 if absolute position. Bizarro.
				position = "absolute";
			}
			left = top = "1px";
			height = width = "1px";
			visibility = "hidden";
		}

		if(!dojo.isIE){
			this.setSrc(cframe, turi, true);
			cframe.onload = new Function(onloadstr);
		}

		return cframe;
	},

	setSrc: function(/*DOMNode*/iframe, /*String*/src, /*Boolean*/replace){
		//summary:
		//		Sets the URL that is loaded in an IFrame. The replace parameter
		//		indicates whether location.replace() should be used when
		//		changing the location of the iframe.
		try{
			if(!replace){
				if(dojo.isSafari){
					iframe.location = src;
				}else{
					frames[iframe.name].location = src;
				}
			}else{
				// Fun with DOM 0 incompatibilities!
				var idoc;
				if(dojo.isIE || dojo.isSafari > 2){
					idoc = iframe.contentWindow.document;
				}else if(dojo.isSafari){
					idoc = iframe.document;
				}else{ //  if(d.isMozilla){
					idoc = iframe.contentWindow;
				}
	
				//For Safari (at least 2.0.3) and Opera, if the iframe
				//has just been created but it doesn't have content
				//yet, then iframe.document may be null. In that case,
				//use iframe.location and return.
				if(!idoc){
					iframe.location = src;
					return;
				}else{
					idoc.location.replace(src);
				}
			}
		}catch(e){ 
			console.debug("dojo.io.iframe.setSrc: ", e); 
		}
	},

	doc: function(/*DOMNode*/iframeNode){
		//summary: Returns the document object associated with the iframe DOM Node argument.
		var doc = iframeNode.contentDocument || // W3
			(
				(iframeNode.contentWindow)&&(iframeNode.contentWindow.document)
			) ||  // IE
			(
				(iframeNode.name)&&(document.frames[iframeNode.name])&&
				(document.frames[iframeNode.name].document)
			) || null;
		return doc;
	},

	/*=====
	dojo.io.iframe.__ioArgs = function(kwArgs){
		//	summary:
		//		All the properties described in the dojo.__ioArgs type, apply
		//		to this type. The following additional properties are allowed
		//		for dojo.io.iframe.send():
		//	method: String?
		//		The HTTP method to use. "GET" or "POST" are the only supported
		//		values.  It will try to read the value from the form node's
		//		method, then try this argument. If neither one exists, then it
		//		defaults to POST.
		//	handleAs: String?
		//		Specifies what format the result data should be given to the
		//		load/handle callback. Valid values are: text, html, javascript,
		//		json. IMPORTANT: For all values EXCEPT html, The server
		//		response should be an HTML file with a textarea element. The
		//		response data should be inside the textarea element. Using an
		//		HTML document the only reliable, cross-browser way this
		//		transport can know when the response has loaded. For the html
		//		handleAs value, just return a normal HTML document.  NOTE: xml
		//		or any other XML type is NOT supported by this transport.
		//	content: Object?
		//		If "form" is one of the other args properties, then the content
		//		object properties become hidden form form elements. For
		//		instance, a content object of {name1 : "value1"} is converted
		//		to a hidden form element with a name of "name1" and a value of
		//		"value1". If there is not a "form" property, then the content
		//		object is converted into a name=value&name=value string, by
		//		using dojo.objectToQuery().
	}
	=====*/

	send: function(/*dojo.io.iframe.__ioArgs*/args){
		//summary: function that sends the request to the server.
		//This transport can only process one send() request at a time, so if send() is called
		//multiple times, it will queue up the calls and only process one at a time.
		if(!this["_frame"]){
			this._frame = this.create(this._iframeName, "dojo.io.iframe._iframeOnload();");
		}

		//Set up the deferred.
		var dfd = dojo._ioSetArgs(
			args,
			function(/*Deferred*/dfd){
				//summary: canceller function for dojo._ioSetArgs call.
				dfd.canceled = true;
				dfd.ioArgs._callNext();
			},
			function(/*Deferred*/dfd){
				//summary: okHandler function for dojo._ioSetArgs call.
				var value = null;
				try{
					var ioArgs = dfd.ioArgs;
					var dii = dojo.io.iframe;
					var ifd = dii.doc(dii._frame);
					var handleAs = ioArgs.handleAs;

					//Assign correct value based on handleAs value.
					value = ifd; //html
					if(handleAs != "html"){
						value = ifd.getElementsByTagName("textarea")[0].value; //text
						if(handleAs == "json"){
							value = dojo.fromJson(value); //json
						}else if(handleAs == "javascript"){
							value = dojo.eval(value); //javascript
						}
					}
				}catch(e){
					value = e;
				}finally{
					ioArgs._callNext();				
				}
				return value;
			},
			function(/*Error*/error, /*Deferred*/dfd){
				//summary: errHandler function for dojo._ioSetArgs call.
				dfd.ioArgs._hasError = true;
				dfd.ioArgs._callNext();
				return error;
			}
		);

		//Set up a function that will fire the next iframe request. Make sure it only
		//happens once per deferred.
		dfd.ioArgs._callNext = function(){
			if(!this["_calledNext"]){
				this._calledNext = true;
				dojo.io.iframe._currentDfd = null;
				dojo.io.iframe._fireNextRequest();
			}
		}

		this._dfdQueue.push(dfd);
		this._fireNextRequest();
		
		//Add it the IO watch queue, to get things like timeout support.
		dojo._ioWatch(
			dfd,
			function(/*Deferred*/dfd){
				//validCheck
				return !dfd.ioArgs["_hasError"];
			},
			function(dfd){
				//ioCheck
				return (!!dfd.ioArgs["_finished"]);
			},
			function(dfd){
				//resHandle
				if(dfd.ioArgs._finished){
					dfd.callback(dfd);
				}else{
					dfd.errback(new Error("Invalid dojo.io.iframe request state"));
				}
			}
		);

		return dfd;
	},

	_currentDfd: null,
	_dfdQueue: [],
	_iframeName: "dojoIoIframe",

	_fireNextRequest: function(){
		//summary: Internal method used to fire the next request in the bind queue.
		try{
			if((this._currentDfd)||(this._dfdQueue.length == 0)){ return; }
			var dfd = this._currentDfd = this._dfdQueue.shift();
			var ioArgs = dfd.ioArgs;
			var args = ioArgs.args;

			ioArgs._contentToClean = [];
			var fn = args["form"];
			var content = args["content"] || {};
			if(fn){
				if(content){
					// if we have things in content, we need to add them to the form
					// before submission
					for(var x in content){
						if(!fn[x]){
							var tn;
							if(dojo.isIE){
								tn = dojo.doc.createElement("<input type='hidden' name='"+x+"'>");
							}else{
								tn = dojo.doc.createElement("input");
								tn.type = "hidden";
								tn.name = x;
							}
							tn.value = content[x];
							fn.appendChild(tn);
							ioArgs._contentToClean.push(x);
						}else{
							fn[x].value = content[x];
						}
					}
				}
				//IE requires going through getAttributeNode instead of just getAttribute in some form cases, 
				//so use it for all.  See #2844
				var actnNode = fn.getAttributeNode("action");
				var mthdNode = fn.getAttributeNode("method");
				var trgtNode = fn.getAttributeNode("target");
				if(args["url"]){
					ioArgs._originalAction = actnNode ? actnNode.value : null;
					if(actnNode){
						actnNode.value = args.url;
					}else{
						fn.setAttribute("action",args.url);
					}
				}
				if(!mthdNode || !mthdNode.value){
					if(mthdNode){
						mthdNode.value= (args["method"]) ? args["method"] : "post";
					}else{
						fn.setAttribute("method", (args["method"]) ? args["method"] : "post");
					}
				}
				ioArgs._originalTarget = trgtNode ? trgtNode.value: null;
				if(trgtNode){
					trgtNode.value = this._iframeName;
				}else{
					fn.setAttribute("target", this._iframeName);
				}
				fn.target = this._iframeName;
				fn.submit();
			}else{
				// otherwise we post a GET string by changing URL location for the
				// iframe
				var tmpUrl = args.url + (args.url.indexOf("?") > -1 ? "&" : "?") + ioArgs.query;
				this.setSrc(this._frame, tmpUrl, true);
			}
		}catch(e){
			dfd.errback(e);
		}
	},

	_iframeOnload: function(){
		var dfd = this._currentDfd;
		if(!dfd){
			this._fireNextRequest();
			return;
		}

		var ioArgs = dfd.ioArgs;
		var args = ioArgs.args;
		var fNode = args.form;
	
		if(fNode){
			// remove all the hidden content inputs
			var toClean = ioArgs._contentToClean;
			for(var i = 0; i < toClean.length; i++) {
				var key = toClean[i];
				if(dojo.isSafari < 3){
					//In Safari (at least 2.0.3), can't use form[key] syntax to find the node,
					//for nodes that were dynamically added.
					for(var j = 0; j < fNode.childNodes.length; j++){
						var chNode = fNode.childNodes[j];
						if(chNode.name == key){
							dojo._destroyElement(chNode);
							break;
						}
					}
				}else{
					dojo._destroyElement(fNode[key]);
					fNode[key] = null;
				}
			}
	
			// restore original action + target
			if(ioArgs["_originalAction"]){
				fNode.setAttribute("action", ioArgs._originalAction);
			}
			if(ioArgs["_originalTarget"]){
				fNode.setAttribute("target", ioArgs._originalTarget);
				fNode.target = ioArgs._originalTarget;
			}
		}

		ioArgs._finished = true;
	}
}

}

if(!dojo._hasResource["dijit.form.CheckBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.CheckBox"] = true;
dojo.provide("dijit.form.CheckBox");



dojo.declare(
	"dijit.form.CheckBox",
	dijit.form.ToggleButton,
	{
		// summary:
		// 		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		// User interacts with real html inputs.
		// On onclick (which occurs by mouse click, space-bar, or
		// using the arrow keys to switch the selected radio button),
		// we update the state of the checkbox/radio.
		//
		// There are two modes:
		//   1. High contrast mode
		//   2. Normal mode
		// In case 1, the regular html inputs are shown and used by the user.
		// In case 2, the regular html inputs are invisible but still used by
		// the user. They are turned quasi-invisible and overlay the background-image.

		templateString:"<fieldset class=\"dijitReset dijitInline\" waiRole=\"presentation\"\r\n\t><input\r\n\t \ttype=\"${type}\" name=\"${name}\"\r\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\r\n\t\tdojoAttachPoint=\"inputNode,focusNode\"\r\n\t \tdojoAttachEvent=\"onmouseover:_onMouse,onmouseout:_onMouse,onclick:_onClick\"\r\n/></fieldset>\r\n",

		baseClass: "dijitCheckBox",

		//	Value of "type" attribute for <input>
		type: "checkbox",

		// value: Value
		//	equivalent to value field on normal checkbox (if checked, the value is passed as
		//	the value when form is submitted)
		value: "on",

		postCreate: function(){
			dojo.setSelectable(this.inputNode, false);
			this.setChecked(this.checked);
			this.inherited(arguments);
		},

		setChecked: function(/*Boolean*/ checked){
			if(dojo.isIE){
				if(checked){ this.inputNode.setAttribute('checked', 'checked'); }
				else{ this.inputNode.removeAttribute('checked'); }
			}else{ this.inputNode.checked = checked; }
			this.inherited(arguments);
		},

		setValue: function(/*String*/ value){
			if(value == null){ value = ""; }
			this.inputNode.value = value;
			dijit.form.CheckBox.superclass.setValue.call(this,value);
		}
	}
);

dojo.declare(
	"dijit.form.RadioButton",
	dijit.form.CheckBox,
	{
		// summary:
		// 		Same as an HTML radio, but with fancy styling.
		//
		// description:
		// Implementation details
		//
		// Specialization:
		// We keep track of dijit radio groups so that we can update the state
		// of all the siblings (the "context") in a group based on input
		// events. We don't rely on browser radio grouping.

		type: "radio",
		baseClass: "dijitRadio",

		// This shared object keeps track of all widgets, grouped by name
		_groups: {},

		postCreate: function(){
			// add this widget to _groups
			(this._groups[this.name] = this._groups[this.name] || []).push(this);

			this.inherited(arguments);
		},

		uninitialize: function(){
			// remove this widget from _groups
			dojo.forEach(this._groups[this.name], function(widget, i, arr){
				if(widget === this){
					arr.splice(i, 1);
					return;
				}
			}, this);
		},

		setChecked: function(/*Boolean*/ checked){
			// If I am being checked then have to deselect currently checked radio button
			if(checked){
				dojo.forEach(this._groups[this.name], function(widget){
					if(widget != this && widget.checked){
						widget.setChecked(false);
					}
				}, this);
			}
			this.inherited(arguments);			
		},

		_clicked: function(/*Event*/ e){
			if(!this.checked){
				this.setChecked(true);
			}
		}
	}
);

}

if(!dojo._hasResource["dojo.data.util.filter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.filter"] = true;
dojo.provide("dojo.data.util.filter");

dojo.data.util.filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	//	summary:  
	//		Helper function to convert a simple pattern to a regular expression for matching.
	//	description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:  
	//			ca*   -> /^ca.*$/
	//			*ca*  -> /^.*ca.*$/
	//			*c\*a*  -> /^.*c\*a.*$/
	//			*c\*a?*  -> /^.*c\*a..*$/
	//			and so on.
	//
	//	pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//			* Means match anything, so ca* means match anything starting with ca
	//			? Means match single character.  So, b?b will match to bob and bab, and so on.
	//      	\ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//				To use a \ as a character in the string, it must be escaped.  So in the pattern it should be 
	//				represented by \\ to be treated as an ordinary \ character instead of an escape.
	//
	//	ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch (c) {
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"i"); //RegExp
	}else{
		return new RegExp(rxp); //RegExp
	}
	
};

}

if(!dojo._hasResource["dojo.data.util.sorter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.sorter"] = true;
dojo.provide("dojo.data.util.sorter");

dojo.data.util.sorter.basicComparator = function(	/*anything*/ a, 
													/*anything*/ b){
	//	summary:  
	//		Basic comparision function that compares if an item is greater or less than another item
	//	description:  
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		undefined values are treated as larger values so that they're pushed to the end of the list.

	var ret = 0;
	if(a > b || typeof a === "undefined" || a === null){
		ret = 1;
	}else if(a < b || typeof b === "undefined" || b === null){
		ret = -1;
	}
	return ret; //int, {-1,0,1}
};

dojo.data.util.sorter.createSortFunction = function(	/* attributes array */sortSpec,
														/*dojo.data.core.Read*/ store){
	//	summary:  
	//		Helper function to generate the sorting function based off the list of sort attributes.
	//	description:  
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	//
	//	sortSpec: array
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	//		{
	//			attribute: "attributeName-string" || attribute,
	//			descending: true|false;   // Default is false.
	//		}
	//	store: object
	//		The datastore object to look up item values from.
	//
	var sortFunctions=[];   

	function createSortFunction(attr, dir){
		return function(itemA, itemB){
			var a = store.getValue(itemA, attr);
			var b = store.getValue(itemB, attr);
			//See if we have a override for an attribute comparison.
			var comparator = null;
			if(store.comparatorMap){
				if(typeof attr !== "string"){
					 attr = store.getIdentity(attr);
				}
				comparator = store.comparatorMap[attr]||dojo.data.util.sorter.basicComparator;
			}
			comparator = comparator||dojo.data.util.sorter.basicComparator; 
			return dir * comparator(a,b); //int
		};
	}

	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		if(sortAttribute.attribute){
			var direction = (sortAttribute.descending) ? -1 : 1;
			sortFunctions.push(createSortFunction(sortAttribute.attribute, direction));
		}
	}

	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int  
	};  //  Function
};

}

if(!dojo._hasResource["dojo.data.util.simpleFetch"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.simpleFetch"] = true;
dojo.provide("dojo.data.util.simpleFetch");


dojo.data.util.simpleFetch.fetch = function(/* Object? */ request){
	//	summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.  
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems() 
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch 
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems() 
	//		method takes three arguments, the keywordArgs object that was passed 
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.  
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters 
	//		(such as includeChildren).  The _fetchItems() method should create an array of 
	//		result items and pass it to the fetchHandler along with the original request object 
	//		-- or, the _fetchItems() method may, if it wants to, create an new request object 
	//		with other specifics about the request that are specific to the datastore and pass 
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo.data.api.Read.fetch()
	request = request || {};
	if(!request.store){
		request.store = this;
	}
	var self = this;

	var _errorHandler = function(errorData, requestObject){
		if(requestObject.onError){
			var scope = requestObject.scope || dojo.global;
			requestObject.onError.call(scope, errorData, requestObject);
		}
	};

	var _fetchHandler = function(items, requestObject){
		var oldAbortFunction = requestObject.abort || null;
		var aborted = false;

		var startIndex = requestObject.start?requestObject.start:0;
		var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;

		requestObject.abort = function(){
			aborted = true;
			if(oldAbortFunction){
				oldAbortFunction.call(requestObject);
			}
		};

		var scope = requestObject.scope || dojo.global;
		if(!requestObject.store){
			requestObject.store = self;
		}
		if(requestObject.onBegin){
			requestObject.onBegin.call(scope, items.length, requestObject);
		}
		if(requestObject.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
		}
		if(requestObject.onItem){
			for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
				var item = items[i];
				if(!aborted){
					requestObject.onItem.call(scope, item, requestObject);
				}
			}
		}
		if(requestObject.onComplete && !aborted){
			var subset = null;
			if (!requestObject.onItem) {
				subset = items.slice(startIndex, endIndex);
			}
			requestObject.onComplete.call(scope, subset, requestObject);   
		}
	};
	this._fetchItems(request, _fetchHandler, _errorHandler);
	return request;	// Object
};

}

if(!dojo._hasResource["dojo.data.ItemFileReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.ItemFileReadStore"] = true;
dojo.provide("dojo.data.ItemFileReadStore");





dojo.declare("dojo.data.ItemFileReadStore", null,{
	//	summary:
	//		The ItemFileReadStore implements the dojo.data.api.Read API and reads
	//		data from JSON files that have contents in this format --
	//		{ items: [
	//			{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	//			{ name:'Fozzie Bear', wears:['hat', 'tie']},
	//			{ name:'Miss Piggy', pets:'Foo-Foo'}
	//		]}
	//		Note that it can also contain an 'identifer' property that specified which attribute on the items 
	//		in the array of items that acts as the unique identifier for that item.
	//
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {data: jsonObject}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the 
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}
	
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo.date.stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
											type: Date,
											deserialize: function(value){
												return dojo.date.stamp.fromISOString(value);
											}
										};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S";  // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._loadInProgress = false;	//Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];
	},
	
	url: "",	// use "" rather than undefined for the benefit of the parser (#3539)

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		//	item: 
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){ 
			throw new Error("dojo.data.ItemFileReadStore: Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute: 
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){ 
			throw new Error("dojo.data.ItemFileReadStore: Invalid attribute argument.");
		}
	},

	getValue: function(	/* item */ item, 
						/* attribute-name-string */ attribute, 
						/* value? */ defaultValue){
		//	summary: 
		//		See dojo.data.api.Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* item */ item, 
						/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return item[attribute] || []; // Array
	},

	getAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute) {
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		return this.getValues(item, attribute).length > 0;
	},

	containsValue: function(/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* anything */ value){
		//	summary: 
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = dojo.data.util.filter.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item, 
								/* attribute-name-string */ attribute, 
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary: 
		//		Internal function for looking at the values contained by the item.
		//	description: 
		//		Internal function for looking at the values contained by the item.  This 
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//	
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:	
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return dojo.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !dojo.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			}else if(value === possibleValue){
				return true; // Boolean
			}
		});
	},

	isItem: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		//	summary: 
		//		See dojo.data.api.Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ findCallback, 
							/* Function */ errorCallback){
		//	summary: 
		//		See dojo.data.util.simpleFetch.fetch()
		var self = this;
		var filter = function(requestArgs, arrayOfItems){
			var items = [];
			if(requestArgs.query){
				var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 

				//See if there are any string values that can be regexp parsed first to avoid multiple regexp gens on the
				//same value for each item examined.  Much more efficient.
				var regexpList = {};
				for(var key in requestArgs.query){
					var value = requestArgs.query[key];
					if(typeof value === "string"){
						regexpList[key] = dojo.data.util.filter.patternToRegExp(value, ignoreCase);
					}
				}

				for(var i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						for(var key in requestArgs.query) {
							var value = requestArgs.query[key];
							if (!self._containsValue(candidateItem, key, value, regexpList[key])){
								match = false;
							}
						}
					}
					if(match){
						items.push(candidateItem);
					}
				}
				findCallback(items, requestArgs);
			}else{
				// We want a copy to pass back in case the parent wishes to sort the array. 
				// We shouldn't allow resort of the internal list, so that multiple callers 
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(var i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			}
		};

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{

			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be 
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional"
						};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							
							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojo.data.ItemFileReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		}
	},

	_handleQueuedFetches: function(){
		//	summary: 
		//		Internal function to execute delayed request in the store.
		//Execute any deferred fetches now.
		if (this._queuedFetches.length > 0) {
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i];
				var delayedQuery = fData.args;
				var delayedFilter = fData.filter;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions)); 
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		//	summary: 
		//		Internal function to determine which list of items to search over.
		//	queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep) {
			return this._arrayOfAllItems; 
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		 //	summary: 
		 //		See dojo.data.api.Read.close()
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.
		
		// First, we define a couple little utility functions...
		
		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'May 14, 1802'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			var isItem = (
				(aValue != null) &&
				(typeof aValue == "object") &&
				(!dojo.isArray(aValue)) &&
				(!dojo.isFunction(aValue)) &&
				(aValue.constructor == Object) &&
				(typeof aValue._reference == "undefined") && 
				(typeof aValue._type == "undefined") && 
				(typeof aValue._value == "undefined")
			);
			return isItem;
		}
		
		var self = this;
		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(dojo.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i;
		var item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items, 
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		// 
		// We also store the attribute names so we can validate our store  
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {};
		var key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if (key !== this._rootItemPropName)
				{
					var value = item[key];
					if(value !== null){
						if(!dojo.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is 
		// the name of an attribute that holds the identity of each item. 
		// If this data file specified an identifier attribute, then build a 
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!this._itemsByIdentity[identity]){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojo.data.ItemFileReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojo.data.ItemFileReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties 
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] } 
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'July 18, 1918'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date('July 18, 1918')) } 
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j) {
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(value._type && value._value){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){ 
								throw new Error("dojo.data.ItemFileReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(dojo.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(dojo.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojo.data.ItemFileReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(dojo.isString(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._itemsByIdentity[referenceDescription];
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k];
									var found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){ 
											found = false; 
										}
									}
									if(found){ 
										arrayOfValues[j] = candidateItem; 
									}
								}
							}
						}
					}
				}
			}
		}
	},

	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object || String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		if(!this._loadFinished){
			var self = this;
			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional"
					};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							var item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				var item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
					keywordArgs.onItem.call(scope, item);
				}
			} 
		}else{
			// Already loaded.  We can just look it up and call back.
			var item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		//	summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			item = this._itemsByIdentity[identity];
		}else{
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentifierAttributes()
		 
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo.data.api.Identity
			// spec says we need to return null if the identity is not composed 
			// of attributes 
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},
	
	_forceLoad: function(){
		//	summary: 
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.  
		var self = this;
		if(this._jsonFileUrl){
				var getArgs = {
					url: self._jsonFileUrl, 
					handleAs: "json-comment-optional",
					sync: true
				};
			var getHandler = dojo.xhrGet(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently 
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if (self._loadInProgress !== true && !self._loadFinished) {
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		} 
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(dojo.data.ItemFileReadStore,dojo.data.util.simpleFetch);

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		dojo.requireLocalization() or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (var foo = { bar: dojo.i18n.getLocalization() }).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in dojo.adOnLoad()), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in RFC 3066.
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		tests/hello/nls/en-us/salutations.js is loaded as the object
	//		tests.hello.nls.salutations.en_us={...} The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			if(targetLocale.indexOf(flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(djConfig.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = djConfig.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = djConfig.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormWidget,
	{
		// summary:
		//		A generic textbox field.
		//		Serves as a base class to derive more specialized functionality in subclasses.

		//	trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		//	uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		//	lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		//	propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		templateString:"<input class=\"dojoTextBox\" dojoAttachPoint='textbox,focusNode' name=\"${name}\"\r\n\tdojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress'\r\n\tautocomplete=\"off\" type=\"${type}\"\r\n\t/>\r\n",
		baseClass: "dijitTextBox",

		attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
			{maxLength:"focusNode"}),

		getDisplayedValue: function(){
			return this.filter(this.textbox.value);
		},

		getValue: function(){
			return this.parse(this.getDisplayedValue(), this.constraints);
		},

		setValue: function(value, /*Boolean, optional*/ priorityChange, /*String, optional*/ formattedValue){
			var filteredValue = this.filter(value);
			if((typeof filteredValue == typeof value) && (formattedValue == null || formattedValue == undefined)){
				formattedValue = this.format(filteredValue, this.constraints);
			}
			if(formattedValue != null && formattedValue != undefined){
				this.textbox.value = formattedValue;
			}
			dijit.form.TextBox.superclass.setValue.call(this, filteredValue, priorityChange);
		},

		setDisplayedValue: function(/*String*/value){
			this.textbox.value = value;
			this.setValue(this.getValue(), true);
		},

		forWaiValuenow: function(){
			return this.getDisplayedValue();
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a value to a properly formatted string
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a formatted string to a value
			return value;
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.getDisplayedValue());
			this.inherited('postCreate', arguments);

			if(this.srcNodeRef){
				dojo.style(this.textbox, "cssText", this.style);
				this.textbox.className += " " + this["class"];
			}
			this._layoutHack();
		},

		_layoutHack: function(){
			// summary: work around table sizing bugs on FF2 by forcing redraw
			if(dojo.isFF == 2 && this.domNode.tagName=="TABLE"){
				var node=this.domNode;
				var old = node.style.opacity;
				node.style.opacity = "0.999";
				setTimeout(function(){
					node.style.opacity = old;
				}, 0);
			}			
		},

		filter: function(val){
			// summary: Apply various filters to textbox value
			if(val == undefined || val == null){ return ""; }
			else if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		// event handlers, you can over-ride these in your own subclasses
		_onBlur: function(){
			this.setValue(this.getValue(), (this.isValid ? this.isValid() : true));
		},

		onkeyup: function(){
			// TODO: it would be nice to massage the value (ie: automatic uppercase, etc) as the user types
			// but this messes up the cursor position if you are typing into the middle of a word, and
			// also trimming doesn't work correctly (it prevents spaces between words too!)
			// this.setValue(this.getValue());
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ValidationTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ValidationTextBox"] = true;
dojo.provide("dijit.form.ValidationTextBox");








dojo.declare(
	"dijit.form.ValidationTextBox",
	dijit.form.TextBox,
	{
		// summary:
		//		A subclass of TextBox.
		//		Over-ride isValid in subclasses to perform specific kinds of validation.

		templateString:"<table style=\"display: -moz-inline-stack;\" class=\"dijit dijitReset dijitInlineTable\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitInputField\" width=\"100%\"\r\n\t\t\t><input dojoAttachPoint='textbox,focusNode' dojoAttachEvent='onfocus,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress' autocomplete=\"off\"\r\n\t\t\ttype='${type}' name='${name}'\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t></tr\r\n></table>\r\n",
		baseClass: "dijitTextBox",

		// default values for new subclass properties
		// required: Boolean
		//		Can be true or false, default is false.
		required: false,
		// promptMessage: String
		//		Hint string
		promptMessage: "",
		// invalidMessage: String
		// 		The message to display if value is invalid.
		invalidMessage: "$_unset_$", // read from the message file if not overridden
		// constraints: Object
		//		user-defined object needed to pass parameters to the validator functions
		constraints: {},
		// regExp: String
		//		regular expression string used to validate the input
		//		Do not specify both regExp and regExpGen
		regExp: ".*",
		// regExpGen: Function
		//		user replaceable function used to generate regExp when dependent on constraints
		//		Do not specify both regExp and regExpGen
		regExpGen: function(constraints){ return this.regExp; },
		
		// state: String
		//		Shows current state (ie, validation result) of input (Normal, Warning, or Error)
		state: "",

		setValue: function(){
			this.inherited('setValue', arguments);
			this.validate(false);
		},

		validator: function(value,constraints){
			// summary: user replaceable function used to validate the text input against the regular expression.
			return (new RegExp("^(" + this.regExpGen(constraints) + ")"+(this.required?"":"?")+"$")).test(value) &&
				(!this.required || !this._isEmpty(value)) &&
				(this._isEmpty(value) || this.parse(value, constraints) !== null);
		},

		isValid: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.validator(this.textbox.value, this.constraints);
		},

		_isEmpty: function(value){
			// summary: Checks for whitespace
			return /^\s*$/.test(value); // Boolean
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			// summary: return an error message to show if appropriate
			return this.invalidMessage;
		},

		getPromptMessage: function(/* Boolean*/ isFocused){
			// summary: return a hint to show if appropriate
			return this.promptMessage;
		},

		validate: function(/* Boolean*/ isFocused){
			// summary:
			//		Called by oninit, onblur, and onkeypress.
			// description:
			//		Show missing or invalid messages if appropriate, and highlight textbox field.
			var message = "";
			var isValid = this.isValid(isFocused);
			var isEmpty = this._isEmpty(this.textbox.value);
			this.state = (isValid || (!this._hasBeenBlurred && isEmpty)) ? "" : "Error";
			this._setStateClass();
			dijit.setWaiState(this.focusNode, "invalid", (isValid? "false" : "true"));
			if(isFocused){
				if(isEmpty){
					message = this.getPromptMessage(true);
				}
				if(!message && !isValid){
					message = this.getErrorMessage(true);
				}
			}
			this._displayMessage(message);
		},

		// currently displayed message
		_message: "",

		_displayMessage: function(/*String*/ message){
			if(this._message == message){ return; }
			this._message = message;
			this.displayMessage(message);
		},

		displayMessage: function(/*String*/ message){
			// summary:
			//		User overridable method to display validation errors/hints.
			//		By default uses a tooltip.
			if(message){
				dijit.showTooltip(message, this.domNode);
			}else{
				dijit.hideTooltip(this.domNode);
			}
		},

		_hasBeenBlurred: false,

		_onBlur: function(evt){
			this._hasBeenBlurred = true;
			this.validate(false);
			this.inherited('_onBlur', arguments);
		},

		onfocus: function(evt){
			// TODO: change to _onFocus?
			this.validate(true);
			this._onMouse(evt);	// update CSS classes
		},

		onkeyup: function(evt){
			this.onfocus(evt);
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.constraints = {};
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			this.constraints.locale=this.lang;
			this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
			if(this.invalidMessage == "$_unset_$"){ this.invalidMessage = this.messages.invalidMessage; }
			var p = this.regExpGen(this.constraints);
			this.regExp = p;
			// make value a string for all types so that form reset works well
		}
	}
);

dojo.declare(
	"dijit.form.MappedTextBox",
	dijit.form.ValidationTextBox,
	{
		// summary:
		//		A subclass of ValidationTextBox.
		//		Provides a hidden input field and a serialize method to override

		serialize: function(val, /*Object?*/options){
			// summary: user replaceable function used to convert the getValue() result to a String
			return (val.toString ? val.toString() : "");
		},

		toString: function(){
			// summary: display the widget as a printable string using the widget's value
			var val = this.filter(this.getValue());
			return (val!=null) ? ((typeof val == "string") ? val : this.serialize(val, this.constraints)) : "";
		},

		validate: function(){
			this.valueNode.value = this.toString();
			this.inherited('validate', arguments);
		},

		postCreate: function(){
			var textbox = this.textbox;
			var valueNode = (this.valueNode = document.createElement("input"));
			valueNode.setAttribute("type", textbox.type);
			valueNode.setAttribute("value", this.toString());
			dojo.style(valueNode, "display", "none");
			valueNode.name = this.textbox.name;
			this.textbox.name = "_" + this.textbox.name + "_displayed_";
			this.textbox.removeAttribute("name");
			dojo.place(valueNode, textbox, "after");

			this.inherited('postCreate', arguments);
		}
	}
);

dojo.declare(
	"dijit.form.RangeBoundTextBox",
	dijit.form.MappedTextBox,
	{
		// summary:
		//		A subclass of MappedTextBox.
		//		Tests for a value out-of-range
		/*===== contraints object:
		// min: Number
		//		Minimum signed value.  Default is -Infinity
		min: undefined,
		// max: Number
		//		Maximum signed value.  Default is +Infinity
		max: undefined,
		=====*/

		// rangeMessage: String
		//		The message to display if value is out-of-range
		rangeMessage: "",

		compare: function(val1, val2){
			// summary: compare 2 values
			return val1 - val2;
		},

		rangeCheck: function(/* Number */ primitive, /* Object */ constraints){
			// summary: user replaceable function used to validate the range of the numeric input value
			var isMin = (typeof constraints.min != "undefined");
			var isMax = (typeof constraints.max != "undefined");
			if(isMin || isMax){
				return (!isMin || this.compare(primitive,constraints.min) >= 0) &&
					(!isMax || this.compare(primitive,constraints.max) <= 0);
			}else{ return true; }
		},

		isInRange: function(/* Boolean*/ isFocused){
			// summary: Need to over-ride with your own validation code in subclasses
			return this.rangeCheck(this.getValue(), this.constraints);
		},

		isValid: function(/* Boolean*/ isFocused){
			return this.inherited('isValid', arguments) &&
				((this._isEmpty(this.textbox.value) && !this.required) || this.isInRange(isFocused));
		},

		getErrorMessage: function(/* Boolean*/ isFocused){
			if(dijit.form.RangeBoundTextBox.superclass.isValid.call(this, false) && !this.isInRange(isFocused)){ return this.rangeMessage; }
			else{ return this.inherited('getErrorMessage', arguments); }
		},

		postMixInProperties: function(){
			this.inherited('postMixInProperties', arguments);
			if(!this.rangeMessage){
				this.messages = dojo.i18n.getLocalization("dijit.form", "validate", this.lang);
				this.rangeMessage = this.messages.rangeMessage;
			}
		},

		postCreate: function(){
			this.inherited('postCreate', arguments);
			if(typeof this.constraints.min != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemin", this.constraints.min);
			}
			if(typeof this.constraints.max != "undefined"){
				dijit.setWaiState(this.focusNode, "valuemax", this.constraints.max);
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.form.ComboBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboBox"] = true;
dojo.provide("dijit.form.ComboBox");





dojo.declare(
	"dijit.form.ComboBoxMixin",
	null,
	{
		// summary:
		//		Auto-completing text box, and base class for FilteringSelect widget.
		//
		//		The drop down box's values are populated from an class called
		//		a data provider, which returns a list of values based on the characters
		//		that the user has typed into the input box.
		//
		//		Some of the options to the ComboBox are actually arguments to the data
		//		provider.
		//
		//		You can assume that all the form widgets (and thus anything that mixes
		//		in ComboBoxMixin) will inherit from _FormWidget and thus the "this"
		//		reference will also "be a" _FormWidget.

		// item: Object
		//		This is the item returned by the dojo.data.store implementation that
		//		provides the data for this cobobox, it's the currently selected item.
		item: null,

		// pageSize: Integer
		//		Argument to data provider.
		//		Specifies number of search results per page (before hitting "next" button)
		pageSize: Infinity,

		// store: Object
		//		Reference to data provider object used by this ComboBox
		store: null,

		// query: Object
		//		A query that can be passed to 'store' to initially filter the items,
		//		before doing further filtering based on searchAttr and the key.
		query: {},

		// autoComplete: Boolean
		//		If you type in a partial string, and then tab out of the <input> box,
		//		automatically copy the first entry displayed in the drop down list to
		//		the <input> field
		autoComplete: true,

		// searchDelay: Integer
		//		Delay in milliseconds between when user types something and we start
		//		searching based on that value
		searchDelay: 100,

		// searchAttr: String
		//		Searches pattern match against this field
		searchAttr: "name",

		// ignoreCase: Boolean
		//		Set true if the ComboBox should ignore case when matching possible items
		ignoreCase: true,

		// hasDownArrow: Boolean
		//		Set this textbox to have a down arrow button.
		//		Defaults to true.
		hasDownArrow:true,

		// _hasFocus: Boolean
		//		Represents focus state of the textbox
		// TODO: get rid of this; it's unnecessary (but currently referenced in FilteringSelect)
		_hasFocus:false,

		templateString:"<table class=\"dijit dijitReset dijitInlineTable dijitLeft\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\" dojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse\" waiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class='dijitReset dijitStretch dijitInputField' width=\"100%\"\r\n\t\t\t><input type=\"text\" autocomplete=\"off\" name=\"${name}\"\r\n\t\t\tdojoAttachEvent=\"onkeypress, onkeyup, onfocus, compositionend\"\r\n\t\t\tdojoAttachPoint=\"textbox,focusNode\" waiRole=\"combobox\"\r\n\t\t/></td\r\n\t\t><td class=\"dijitReset dijitValidationIconField\" width=\"0%\"\r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div\r\n\t\t\t><div class='dijitValidationIconText'>&Chi;</div\r\n\t\t></td\r\n\t\t><td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton' width=\"0%\"\r\n\t\t\tdojoAttachPoint=\"downArrowNode\"\r\n\t\t\tdojoAttachEvent=\"onmousedown:_onArrowMouseDown,onmouseup:_onMouse,onmouseenter:_onMouse,onmouseleave:_onMouse\"\r\n\t\t\t><div class=\"dijitDownArrowButtonInner\" waiRole=\"presentation\"\r\n\t\t\t\t><div class=\"dijitDownArrowButtonChar\">&#9660;</div\r\n\t\t\t></div\r\n\t\t></td\t\r\n\t></tr\r\n></table>\r\n",

		baseClass:"dijitComboBox",

		_lastDisplayedValue: "",

		getValue:function(){
			// don't get the textbox value but rather the previously set hidden value
			return dijit.form.TextBox.superclass.getValue.apply(this, arguments);
		},

		setDisplayedValue:function(/*String*/ value){
			this._lastDisplayedValue = value;
			this.setValue(value, true);
		},

		_getCaretPos: function(/*DomNode*/ element){
			// khtml 3.5.2 has selection* methods as does webkit nightlies from 2005-06-22
			if(typeof(element.selectionStart)=="number"){
				// FIXME: this is totally borked on Moz < 1.3. Any recourse?
				return element.selectionStart;
			}else if(dojo.isIE){
				// in the case of a mouse click in a popup being handled,
				// then the document.selection is not the textarea, but the popup
				// var r = document.selection.createRange();
				// hack to get IE 6 to play nice. What a POS browser.
				var tr = document.selection.createRange().duplicate();
				var ntr = element.createTextRange();
				tr.move("character",0);
				ntr.move("character",0);
				try{
					// If control doesnt have focus, you get an exception.
					// Seems to happen on reverse-tab, but can also happen on tab (seems to be a race condition - only happens sometimes).
					// There appears to be no workaround for this - googled for quite a while.
					ntr.setEndPoint("EndToEnd", tr);
					return String(ntr.text).replace(/\r/g,"").length;
				}catch(e){
					return 0; // If focus has shifted, 0 is fine for caret pos.
				}
			}
		},

		_setCaretPos: function(/*DomNode*/ element, /*Number*/ location){
			location = parseInt(location);
			this._setSelectedRange(element, location, location);
		},

		_setSelectedRange: function(/*DomNode*/ element, /*Number*/ start, /*Number*/ end){
			if(!end){
				end = element.value.length;
			}  // NOTE: Strange - should be able to put caret at start of text?
			// Mozilla
			// parts borrowed from http://www.faqts.com/knowledge_base/view.phtml/aid/13562/fid/130
			if(element.setSelectionRange){
				dijit.focus(element);
				element.setSelectionRange(start, end);
			}else if(element.createTextRange){ // IE
				var range = element.createTextRange();
				with(range){
					collapse(true);
					moveEnd('character', end);
					moveStart('character', start);
					select();
				}
			}else{ //otherwise try the event-creation hack (our own invention)
				// do we need these?
				element.value = element.value;
				element.blur();
				dijit.focus(element);
				// figure out how far back to go
				var dist = parseInt(element.value.length)-end;
				var tchar = String.fromCharCode(37);
				var tcc = tchar.charCodeAt(0);
				for(var x = 0; x < dist; x++){
					var te = document.createEvent("KeyEvents");
					te.initKeyEvent("keypress", true, true, null, false, false, false, false, tcc, tcc);
					element.dispatchEvent(te);
				}
			}
		},

		onkeypress: function(/*Event*/ evt){
			// summary: handles keyboard events

			//except for pasting case - ctrl + v(118)
			if(evt.altKey || (evt.ctrlKey && evt.charCode != 118)){
				return;
			}
			var doSearch = false;
			this.item = null; // #4872
			if(this._isShowingNow){this._popupWidget.handleKey(evt);}
			switch(evt.keyCode){
				case dojo.keys.PAGE_DOWN:
				case dojo.keys.DOWN_ARROW:
					if(!this._isShowingNow||this._prev_key_esc){
						this._arrowPressed();
						doSearch=true;
					}else{
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.PAGE_UP:
				case dojo.keys.UP_ARROW:
					if(this._isShowingNow){
						this._announceOption(this._popupWidget.getHighlightedOption());
					}
					dojo.stopEvent(evt);
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				case dojo.keys.ENTER:
					// prevent submitting form if user presses enter
					// also prevent accepting the value if either Next or Previous are selected
					var highlighted;
					if(this._isShowingNow&&(highlighted=this._popupWidget.getHighlightedOption())){
						// only stop event on prev/next
						if(highlighted==this._popupWidget.nextButton){
							this._nextSearch(1);
							dojo.stopEvent(evt);
							break;
						}
						else if(highlighted==this._popupWidget.previousButton){
							this._nextSearch(-1);
							dojo.stopEvent(evt);
							break;
						}
					}else{
						this.setDisplayedValue(this.getDisplayedValue());
					}
					// default case:
					// prevent submit, but allow event to bubble
					evt.preventDefault();
					// fall through

				case dojo.keys.TAB:
					var newvalue=this.getDisplayedValue();
					// #4617: if the user had More Choices selected fall into the _onBlur handler
					if(this._popupWidget &&
						(newvalue == this._popupWidget._messages["previousMessage"] ||
							newvalue == this._popupWidget._messages["nextMessage"])){
						break;
					}
					if(this._isShowingNow){
						this._prev_key_backspace = false;
						this._prev_key_esc = false;
						if(this._popupWidget.getHighlightedOption()){
							this._popupWidget.setValue({target:this._popupWidget.getHighlightedOption()}, true);
						}
						this._hideResultList();
					}
					break;

				case dojo.keys.SPACE:
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(this._isShowingNow && this._popupWidget.getHighlightedOption()){
						dojo.stopEvent(evt);
						this._selectOption();
						this._hideResultList();
					}else{
						doSearch = true;
					}
					break;

				case dojo.keys.ESCAPE:
					this._prev_key_backspace = false;
					this._prev_key_esc = true;
					this._hideResultList();
					if(this._lastDisplayedValue != this.getDisplayedValue()){
						this.setDisplayedValue(this._lastDisplayedValue);
						dojo.stopEvent(evt);
					}else{
						this.setValue(this.getValue(), false);
					}
					break;

				case dojo.keys.DELETE:
				case dojo.keys.BACKSPACE:
					this._prev_key_esc = false;
					this._prev_key_backspace = true;
					doSearch = true;
					break;

				case dojo.keys.RIGHT_ARROW: // fall through

				case dojo.keys.LEFT_ARROW: // fall through
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					break;

				default:// non char keys (F1-F12 etc..)  shouldn't open list
					this._prev_key_backspace = false;
					this._prev_key_esc = false;
					if(dojo.isIE || evt.charCode != 0){
						doSearch=true;
					}
			}
			if(this.searchTimer){
				clearTimeout(this.searchTimer);
			}
			if(doSearch){
				// need to wait a tad before start search so that the event bubbles through DOM and we have value visible
				this.searchTimer = setTimeout(dojo.hitch(this, this._startSearchFromInput), this.searchDelay);
			}
		},

		_autoCompleteText: function(/*String*/ text){
			// summary:
			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			// IE7: clear selection so next highlight works all the time
			this._setSelectedRange(this.focusNode, this.focusNode.value.length, this.focusNode.value.length);
			// does text autoComplete the value in the textbox?
			// #3744: escape regexp so the user's input isn't treated as a regular expression.
			// Example: If the user typed "(" then the regexp would throw "unterminated parenthetical."
			// Also see #2558 for the autocompletion bug this regular expression fixes.
			if(new RegExp("^"+escape(this.focusNode.value), this.ignoreCase ? "i" : "").test(escape(text))){
				var cpos = this._getCaretPos(this.focusNode);
				// only try to extend if we added the last character at the end of the input
				if((cpos+1) > this.focusNode.value.length){
					// only add to input node as we would overwrite Capitalisation of chars
					// actually, that is ok
					this.focusNode.value = text;//.substr(cpos);
					// visually highlight the autocompleted characters
					this._setSelectedRange(this.focusNode, cpos, this.focusNode.value.length);
					dijit.setWaiState(this.focusNode, "valuenow", text);
				}
			}else{
				// text does not autoComplete; replace the whole value and highlight
				this.focusNode.value = text;
				this._setSelectedRange(this.focusNode, 0, this.focusNode.value.length);
				dijit.setWaiState(this.focusNode, "valuenow", text);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			if(this.disabled || dataObject.query[this.searchAttr] != this._lastQuery){
				return;
			}
			this._popupWidget.clearResultList();
			if(!results.length){
				this._hideResultList();
				return;
			}

			// Fill in the textbox with the first item from the drop down list, and
			// highlight the characters that were auto-completed.   For example, if user
			// typed "CA" and the drop down list appeared, the textbox would be changed to
			// "California" and "ifornia" would be highlighted.

			var zerothvalue=new String(this.store.getValue(results[0], this.searchAttr));
			if(zerothvalue && this.autoComplete && !this._prev_key_backspace &&
			// when the user clicks the arrow button to show the full list,
			// startSearch looks for "*".
			// it does not make sense to autocomplete
			// if they are just previewing the options available.
				(dataObject.query[this.searchAttr] != "*")){
				this._autoCompleteText(zerothvalue);
				// announce the autocompleted value
				dijit.setWaiState(this.focusNode || this.domNode, "valuenow", zerothvalue);
			}
			this._popupWidget.createOptions(results, dataObject, dojo.hitch(this, this._getMenuLabelFromItem));

			// show our list (only if we have content, else nothing)
			this._showResultList();

			// #4091: tell the screen reader that the paging callback finished by shouting the next choice
			if(dataObject.direction){
				if(dataObject.direction==1){
					this._popupWidget.highlightFirstOption();
				}else if(dataObject.direction==-1){
					this._popupWidget.highlightLastOption();
				}
				this._announceOption(this._popupWidget.getHighlightedOption());
			}
		},

		_showResultList: function(){
			this._hideResultList();
			var items = this._popupWidget.getItems(),
				visibleCount = Math.min(items.length,this.maxListLength);
			this._arrowPressed();
			// hide the tooltip
			this._displayMessage("");
			
			// Position the list and if it's too big to fit on the screen then
			// size it to the maximum possible height
			// Our dear friend IE doesnt take max-height so we need to calculate that on our own every time
			// TODO: want to redo this, see http://trac.dojotoolkit.org/ticket/3272, http://trac.dojotoolkit.org/ticket/4108
			with(this._popupWidget.domNode.style){
				// natural size of the list has changed, so erase old width/height settings,
				// which were hardcoded in a previous call to this function (via dojo.marginBox() call) 
				width="";
				height="";
			}
			var best=this.open();
			// #3212: only set auto scroll bars if necessary
			// prevents issues with scroll bars appearing when they shouldn't when node is made wider (fractional pixels cause this)
			var popupbox=dojo.marginBox(this._popupWidget.domNode);
			this._popupWidget.domNode.style.overflow=((best.h==popupbox.h)&&(best.w==popupbox.w))?"hidden":"auto";
			// #4134: borrow TextArea scrollbar test so content isn't covered by scrollbar and horizontal scrollbar doesn't appear
			var newwidth=best.w;
			if(best.h<this._popupWidget.domNode.scrollHeight){newwidth+=16;}
			dojo.marginBox(this._popupWidget.domNode, {h:best.h,w:Math.max(newwidth,this.domNode.offsetWidth)});
		},

		_hideResultList: function(){
			if(this._isShowingNow){
				dijit.popup.close(this._popupWidget);
				this._arrowIdle();
				this._isShowingNow=false;
			}
		},

		_onBlur: function(){
			// summary: called magically when focus has shifted away from this widget and it's dropdown
			this._hasFocus=false;
			this._hasBeenBlurred = true;
			this._hideResultList();
			this._arrowIdle();
			// if the user clicks away from the textbox OR tabs away, set the value to the textbox value
			// #4617: if value is now more choices or previous choices, revert the value
			var newvalue=this.getDisplayedValue();
			if(this._popupWidget&&(newvalue==this._popupWidget._messages["previousMessage"]||newvalue==this._popupWidget._messages["nextMessage"])){
				this.setValue(this._lastValueReported, true);
			}else{
				this.setDisplayedValue(newvalue);
			}
		},

		onfocus:function(/*Event*/ evt){
			this._hasFocus=true;
			
			// update styling to reflect that we are focused
			this._onMouse(evt);
		},

		_announceOption: function(/*Node*/ node){
			// summary:
			//	a11y code that puts the highlighted option in the textbox
			//	This way screen readers will know what is happening in the menu

			if(node==null){return;}
			// pull the text value from the item attached to the DOM node
			var newValue;
			if(node==this._popupWidget.nextButton||node==this._popupWidget.previousButton){
				newValue=node.innerHTML;
			}else{
				newValue=this.store.getValue(node.item, this.searchAttr);
			}
			// get the text that the user manually entered (cut off autocompleted text)
			this.focusNode.value=this.focusNode.value.substring(0, this._getCaretPos(this.focusNode));
			// autocomplete the rest of the option to announce change
			this._autoCompleteText(newValue);
		},

		_selectOption: function(/*Event*/ evt){
			var tgt = null;
			if(!evt){
				evt ={ target: this._popupWidget.getHighlightedOption()};
			}
				// what if nothing is highlighted yet?
			if(!evt.target){
				// handle autocompletion where the the user has hit ENTER or TAB
				this.setDisplayedValue(this.getDisplayedValue());
				return;
			// otherwise the user has accepted the autocompleted value
			}else{
				tgt = evt.target;
			}
			if(!evt.noHide){
				this._hideResultList();
				this._setCaretPos(this.focusNode, this.store.getValue(tgt.item, this.searchAttr).length);
			}
			this._doSelect(tgt);
		},

		_doSelect: function(tgt){
			this.item = tgt.item;
			this.setValue(this.store.getValue(tgt.item, this.searchAttr), true);
		},

		_onArrowMouseDown: function(evt){
			// summary: callback when arrow is clicked
			if(this.disabled){
				return;
			}
			dojo.stopEvent(evt);
			this.focus();
			if(this._isShowingNow){
				this._hideResultList();
			}else{
				// forces full population of results, if they click
				// on the arrow it means they want to see more options
				this._startSearch("");
			}
		},

		_startSearchFromInput: function(){
			this._startSearch(this.focusNode.value);
		},

		_startSearch: function(/*String*/ key){
			if(!this._popupWidget){
				this._popupWidget = new dijit.form._ComboBoxMenu({
					onChange: dojo.hitch(this, this._selectOption)
				});
			}
			// create a new query to prevent accidentally querying for a hidden value from FilteringSelect's keyField
			var query=this.query;
			this._lastQuery=query[this.searchAttr]=key+"*";
			var dataObject=this.store.fetch({queryOptions:{ignoreCase:this.ignoreCase, deep:true}, query: query, onComplete:dojo.hitch(this, "_openResultList"), start:0, count:this.pageSize});
			function nextSearch(dataObject, direction){
				dataObject.start+=dataObject.count*direction;
				// #4091: tell callback the direction of the paging so the screen reader knows which menu option to shout
				dataObject.direction=direction;
				dataObject.store.fetch(dataObject);
			}
			this._nextSearch=this._popupWidget.onPage=dojo.hitch(this, nextSearch, dataObject);
		},

		_getValueField:function(){
			return this.searchAttr;
		},

		/////////////// Event handlers /////////////////////

		_arrowPressed: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.addClass(this.downArrowNode, "dijitArrowButtonActive");
			}
		},

		_arrowIdle: function(){
			if(!this.disabled&&this.hasDownArrow){
				dojo.removeClass(this.downArrowNode, "dojoArrowButtonPushed");
			}
		},

		compositionend: function(/*Event*/ evt){
			// summary: When inputting characters using an input method, such as Asian
			// languages, it will generate this event instead of onKeyDown event
			// Note: this event is only triggered in FF (not in IE)
			this.onkeypress({charCode:-1});
		},

		//////////// INITIALIZATION METHODS ///////////////////////////////////////
		constructor: function(){
			this.query={};
		},

		postMixInProperties: function(){
			if(!this.hasDownArrow){
				this.baseClass = "dijitTextBox";
			}
			if(!this.store){
				// if user didn't specify store, then assume there are option tags
				var items = this.srcNodeRef ? dojo.query("> option", this.srcNodeRef).map(function(node){
					node.style.display="none";
					return { value: node.getAttribute("value"), name: String(node.innerHTML) };
				}) : {};
				this.store = new dojo.data.ItemFileReadStore({data: {identifier:this._getValueField(), items:items}});

				// if there is no value set and there is an option list,
				// set the value to the first value to be consistent with native Select
				if(items && items.length && !this.value){
					// For <select>, IE does not let you set the value attribute of the srcNodeRef (and thus dojo.mixin does not copy it).
					// IE does understand selectedIndex though, which is automatically set by the selected attribute of an option tag
					this.value = items[this.srcNodeRef.selectedIndex != -1 ? this.srcNodeRef.selectedIndex : 0]
						[this._getValueField()];
				}
			}
		},

		uninitialize:function(){
			if(this._popupWidget){
				this._hideResultList();
				this._popupWidget.destroy()
			};
		},

		_getMenuLabelFromItem:function(/*Item*/ item){
			return {html:false, label:this.store.getValue(item, this.searchAttr)};
		},

		open:function(){
			this._isShowingNow=true;
			return dijit.popup.open({
				popup: this._popupWidget,
				around: this.domNode,
				parent: this
			});
		}
	}
);

dojo.declare(
	"dijit.form._ComboBoxMenu",
	[dijit._Widget, dijit._Templated],

	{
		// summary:
		//	Focus-less div based menu for internal use in ComboBox

		templateString:"<div class='dijitMenu' dojoAttachEvent='onmousedown,onmouseup,onmouseover,onmouseout' tabIndex='-1' style='overflow:\"auto\";'>"
				+"<div class='dijitMenuItem dijitMenuPreviousButton' dojoAttachPoint='previousButton'></div>"
				+"<div class='dijitMenuItem dijitMenuNextButton' dojoAttachPoint='nextButton'></div>"
			+"</div>",
		_messages:null,

		postMixInProperties:function(){
			this._messages = dojo.i18n.getLocalization("dijit.form", "ComboBox", this.lang);
			this.inherited("postMixInProperties", arguments);
		},

		setValue:function(/*Object*/ value){
			this.value=value;
			this.onChange(value);
		},

		onChange:function(/*Object*/ value){},
		onPage:function(/*Number*/ direction){},

		postCreate:function(){
			// fill in template with i18n messages
			this.previousButton.innerHTML=this._messages["previousMessage"];
			this.nextButton.innerHTML=this._messages["nextMessage"];
			this.inherited("postCreate", arguments);
		},

		onClose:function(){
			this._blurOptionNode();
		},

		_createOption:function(/*Object*/ item, labelFunc){
			// summary: creates an option to appear on the popup menu
			// subclassed by FilteringSelect

			var labelObject=labelFunc(item);
			var menuitem = document.createElement("div");
			if(labelObject.html){menuitem.innerHTML=labelObject.label;}
			else{menuitem.appendChild(document.createTextNode(labelObject.label));}
			// #3250: in blank options, assign a normal height
			if(menuitem.innerHTML==""){
				menuitem.innerHTML="&nbsp;"
			}
			menuitem.item=item;
			return menuitem;
		},

		createOptions:function(results, dataObject, labelFunc){
			//this._dataObject=dataObject;
			//this._dataObject.onComplete=dojo.hitch(comboBox, comboBox._openResultList);
			// display "Previous . . ." button
			this.previousButton.style.display=dataObject.start==0?"none":"";
			// create options using _createOption function defined by parent ComboBox (or FilteringSelect) class
			// #2309: iterate over cache nondestructively
			var _this=this;
			dojo.forEach(results, function(item){
				var menuitem=_this._createOption(item, labelFunc);
				menuitem.className = "dijitMenuItem";
				_this.domNode.insertBefore(menuitem, _this.nextButton);
			});
			// display "Next . . ." button
			this.nextButton.style.display=dataObject.count==results.length?"":"none";
		},

		clearResultList:function(){
			// keep the previous and next buttons of course
			while(this.domNode.childNodes.length>2){
				this.domNode.removeChild(this.domNode.childNodes[this.domNode.childNodes.length-2]);
			}
		},

		// these functions are called in showResultList
		getItems:function(){
			return this.domNode.childNodes;
		},

		getListLength:function(){
			return this.domNode.childNodes.length-2;
		},

		onmousedown:function(/*Event*/ evt){
			dojo.stopEvent(evt);
		},

		onmouseup:function(/*Event*/ evt){
			if(evt.target === this.domNode){
				return;
			}else if(evt.target==this.previousButton){
				this.onPage(-1);
			}else if(evt.target==this.nextButton){
				this.onPage(1);
			}else{
				var tgt=evt.target;
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
				this.setValue({target:tgt}, true);
			}
		},

		onmouseover:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			var tgt=evt.target;
			if(!(tgt==this.previousButton||tgt==this.nextButton)){
				// while the clicked node is inside the div
				while(!tgt.item){
					// recurse to the top
					tgt=tgt.parentNode;
				}
			}
			this._focusOptionNode(tgt);
		},

		onmouseout:function(/*Event*/ evt){
			if(evt.target === this.domNode){ return; }
			this._blurOptionNode();
		},

		_focusOptionNode:function(/*DomNode*/ node){
			// summary:
			//	does the actual highlight
			if(this._highlighted_option != node){
				this._blurOptionNode();
				this._highlighted_option = node;
				dojo.addClass(this._highlighted_option, "dijitMenuItemHover");
			}
		},

		_blurOptionNode:function(){
			// summary:
			//	removes highlight on highlighted option
			if(this._highlighted_option){
				dojo.removeClass(this._highlighted_option, "dijitMenuItemHover");
				this._highlighted_option = null;
			}
		},

		_highlightNextOption:function(){
			// because each press of a button clears the menu,
			// the highlighted option sometimes becomes detached from the menu!
			// test to see if the option has a parent to see if this is the case.
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.firstChild.style.display=="none"?this.domNode.firstChild.nextSibling:this.domNode.firstChild);
			}else if(this._highlighted_option.nextSibling&&this._highlighted_option.nextSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.nextSibling);
			}
			// scrollIntoView is called outside of _focusOptionNode because in IE putting it inside causes the menu to scroll up on mouseover
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightFirstOption:function(){
			// highlight the non-Previous choices option
			this._focusOptionNode(this.domNode.firstChild.nextSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		highlightLastOption:function(){
			// highlight the noon-More choices option
			this._focusOptionNode(this.domNode.lastChild.previousSibling);
			dijit.scrollIntoView(this._highlighted_option);
		},

		_highlightPrevOption:function(){
			// if nothing selected, highlight last option
			// makes sense if you select Previous and try to keep scrolling up the list
			if(!this.getHighlightedOption()){
				this._focusOptionNode(this.domNode.lastChild.style.display=="none"?this.domNode.lastChild.previousSibling:this.domNode.lastChild);
			}else if(this._highlighted_option.previousSibling&&this._highlighted_option.previousSibling.style.display!="none"){
				this._focusOptionNode(this._highlighted_option.previousSibling);
			}
			dijit.scrollIntoView(this._highlighted_option);
		},

		_page:function(/*Boolean*/ up){
			var scrollamount=0;
			var oldscroll=this.domNode.scrollTop;
			var height=parseInt(dojo.getComputedStyle(this.domNode).height);
			// if no item is highlighted, highlight the first option
			if(!this.getHighlightedOption()){this._highlightNextOption();}
			while(scrollamount<height){
				if(up){
					// stop at option 1
					if(!this.getHighlightedOption().previousSibling||this._highlighted_option.previousSibling.style.display=="none"){break;}
					this._highlightPrevOption();
				}else{
					// stop at last option
					if(!this.getHighlightedOption().nextSibling||this._highlighted_option.nextSibling.style.display=="none"){break;}
					this._highlightNextOption();
				}
				// going backwards
				var newscroll=this.domNode.scrollTop;
				scrollamount+=(newscroll-oldscroll)*(up ? -1:1);
				oldscroll=newscroll;
			}
		},

		pageUp:function(){
			this._page(true);
		},

		pageDown:function(){
			this._page(false);
		},

		getHighlightedOption:function(){
			// summary:
			//	Returns the highlighted option.
			return this._highlighted_option&&this._highlighted_option.parentNode ? this._highlighted_option : null;
		},

		handleKey:function(evt){
			switch(evt.keyCode){
				case dojo.keys.DOWN_ARROW:
					this._highlightNextOption();
					break;
				case dojo.keys.PAGE_DOWN:
					this.pageDown();
					break;	
				case dojo.keys.UP_ARROW:
					this._highlightPrevOption();
					break;
				case dojo.keys.PAGE_UP:
					this.pageUp();
					break;	
			}
		}
	}
);

dojo.declare(
	"dijit.form.ComboBox",
	[dijit.form.ValidationTextBox, dijit.form.ComboBoxMixin],
	{
		postMixInProperties: function(){
			dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
			dijit.form.ValidationTextBox.prototype.postMixInProperties.apply(this, arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.form._Spinner"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._Spinner"] = true;
dojo.provide("dijit.form._Spinner");



dojo.declare(
	"dijit.form._Spinner",
	dijit.form.RangeBoundTextBox,
	{

		// summary: Mixin for validation widgets with a spinner
		// description: This class basically (conceptually) extends dijit.form.ValidationTextBox.
		//	It modifies the template to have up/down arrows, and provides related handling code.

		// defaultTimeout: Number
		//	  number of milliseconds before a held key or button becomes typematic
		defaultTimeout: 500,

		// timeoutChangeRate: Number
		//	  fraction of time used to change the typematic timer between events
		//	  1.0 means that each typematic event fires at defaultTimeout intervals
		//	  < 1.0 means that each typematic event fires at an increasing faster rate
		timeoutChangeRate: 0.90,

		// smallDelta: Number
		//	  adjust the value by this much when spinning using the arrow keys/buttons
		smallDelta: 1,
		// largeDelta: Number
		//	  adjust the value by this much when spinning using the PgUp/Dn keys
		largeDelta: 10,

		templateString:"<table class=\"dijit dijitReset dijitInlineTable dijitLeft\" cellspacing=\"0\" cellpadding=\"0\"\r\n\tid=\"widget_${id}\" name=\"${name}\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onkeypress:_onKeyPress\"\r\n\twaiRole=\"presentation\"\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td rowspan=\"2\" class=\"dijitReset dijitStretch dijitInputField\" width=\"100%\"\r\n\t\t\t><input dojoAttachPoint=\"textbox,focusNode\" type=\"${type}\" dojoAttachEvent=\"onfocus,onkeyup\"\r\n\t\t\t\twaiRole=\"spinbutton\" autocomplete=\"off\" name=\"${name}\"\r\n\t\t></td\r\n\t\t><td rowspan=\"2\" class=\"dijitReset dijitValidationIconField\" width=\"0%\" \r\n\t\t\t><div dojoAttachPoint='iconNode' class='dijitValidationIcon'></div\r\n\t\t></td\r\n\t\t><td class=\"dijitReset dijitRight dijitButtonNode dijitUpArrowButton\" width=\"0%\"\r\n\t\t\t\tdojoAttachPoint=\"upArrowNode\"\r\n\t\t\t\tdojoAttachEvent=\"onmousedown:_handleUpArrowEvent,onmouseup:_handleUpArrowEvent,onmouseover:_handleUpArrowEvent,onmouseout:_handleUpArrowEvent\"\r\n\t\t\t\tstateModifier=\"UpArrow\"\r\n\t\t\t><div class=\"dijitA11yUpArrow\">&#9650;</div\r\n\t\t></td\r\n\t></tr\r\n\t><tr class=\"dijitReset\"\r\n\t\t><td class=\"dijitReset dijitRight dijitButtonNode dijitDownArrowButton\" width=\"0%\"\r\n\t\t\t\tdojoAttachPoint=\"downArrowNode\"\r\n\t\t\t\tdojoAttachEvent=\"onmousedown:_handleDownArrowEvent,onmouseup:_handleDownArrowEvent,onmouseover:_handleDownArrowEvent,onmouseout:_handleDownArrowEvent\"\r\n\t\t\t\tstateModifier=\"DownArrow\"\r\n\t\t\t><div class=\"dijitA11yDownArrow\">&#9660;</div\r\n\t\t></td\r\n\t></tr\r\n></table>\r\n\r\n",
		baseClass: "dijitSpinner",

		adjust: function(/* Object */ val, /*Number*/ delta){
			// summary: user replaceable function used to adjust a primitive value(Number/Date/...) by the delta amount specified
			// the val is adjusted in a way that makes sense to the object type
			return val;
		},

		_handleUpArrowEvent : function(/*Event*/ e){
			this._onMouse(e, this.upArrowNode);
		},

		_handleDownArrowEvent : function(/*Event*/ e){
			this._onMouse(e, this.downArrowNode);
		},


		_arrowPressed: function(/*Node*/ nodePressed, /*Number*/ direction){
			if(this.disabled){ return; }
			dojo.addClass(nodePressed, "dijitSpinnerButtonActive");
			this.setValue(this.adjust(this.getValue(), direction*this.smallDelta), false);
		},

		_arrowReleased: function(/*Node*/ node){
			if(this.disabled){ return; }
			this._wheelTimer = null;
			dijit.focus(this.textbox);
			dojo.removeClass(node, "dijitSpinnerButtonActive");
		},

		_typematicCallback: function(/*Number*/ count, /*DOMNode*/ node, /*Event*/ evt){
			if(node == this.textbox){ node = (evt.keyCode == dojo.keys.UP_ARROW) ? this.upArrowNode : this.downArrowNode; }
			if(count == -1){ this._arrowReleased(node); }
			else{ this._arrowPressed(node, (node == this.upArrowNode) ? 1 : -1); }
		},

		_wheelTimer: null,
		_mouseWheeled: function(/*Event*/ evt){
			dojo.stopEvent(evt);
			var scrollAmount = 0;
			if(typeof evt.wheelDelta == 'number'){ // IE
				scrollAmount = evt.wheelDelta;
			}else if(typeof evt.detail == 'number'){ // Mozilla+Firefox
				scrollAmount = -evt.detail;
			}
			if(scrollAmount > 0){
				var node = this.upArrowNode;
				var dir = +1;
			}else if(scrollAmount < 0){
				var node = this.downArrowNode;
				var dir = -1;
			}else{ return; }
			this._arrowPressed(node, dir);
			if(this._wheelTimer != null){
				clearTimeout(this._wheelTimer);
			}
			var _this = this;
			this._wheelTimer = setTimeout(function(){_this._arrowReleased(node);}, 50);
		},

		postCreate: function(){
			this.inherited('postCreate', arguments);

			// extra listeners
			this.connect(this.textbox, dojo.isIE ? "onmousewheel" : 'DOMMouseScroll', "_mouseWheeled");
			dijit.typematic.addListener(this.upArrowNode, this.textbox, {keyCode:dojo.keys.UP_ARROW,ctrlKey:false,altKey:false,shiftKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout);
			dijit.typematic.addListener(this.downArrowNode, this.textbox, {keyCode:dojo.keys.DOWN_ARROW,ctrlKey:false,altKey:false,shiftKey:false}, this, "_typematicCallback", this.timeoutChangeRate, this.defaultTimeout);
		}
});

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

//	return str.replace(/([\f\b\n\t\r[\^$|?*+(){}])/gm, "\\$1"); // string
	return str.replace(/([\.$?*!=:|{}\(\)\[\]\\\/^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
}

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression. 
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
}

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. 
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
}

}

if(!dojo._hasResource["dojo.number"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.number"] = true;
dojo.provide("dojo.number");







/*=====
dojo.number.__formatOptions = function(kwArgs){
	//	pattern: String?
	//		override formatting pattern with this string (see
	//		dojo.number._applyPattern)
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	places: Number?
	//		fixed number of decimal places to show.  This overrides any
	//		information in the provided pattern.
	//	round: NUmber?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
	//	currency: String?
	//		iso4217 currency code
	//	symbol: String?
	//		localized currency symbol
	//	locale: String?
	//		override the locale used to determine formatting rules
}
=====*/

dojo.number.format = function(/*Number*/value, /*dojo.number.__formatOptions?*/options){
	// summary:
	//		Format a Number as a String, using locale-specific settings
	// description:
	//		Create a string from a Number using a known localized pattern.
	//		Formatting patterns appropriate to the locale are chosen from the
	//		CLDR http://unicode.org/cldr as well as the appropriate symbols and
	//		delimiters.  See http://www.unicode.org/reports/tr35/#Number_Elements
	// value:
	//		the number to be formatted.  If not a valid JavaScript number,
	//		return null.

	options = dojo.mixin({}, options || {});
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	options.customs = bundle;
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
	if(isNaN(value)){ return null; } // null
	return dojo.number._applyPattern(value, pattern, options); // String
};

//dojo.number._numberPatternRE = /(?:[#0]*,?)*[#0](?:\.0*#*)?/; // not precise, but good enough
dojo.number._numberPatternRE = /[#0,]*[#0](?:\.0*#*)?/; // not precise, but good enough

dojo.number._applyPattern = function(/*Number*/value, /*String*/pattern, /*dojo.number.__formatOptions?*/options){
	// summary:
	//		Apply pattern to format value as a string using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted.
	// pattern:
	//		a pattern string as described in
	//		http://www.unicode.org/reports/tr35/#Number_Format_Patterns
	// options: dojo.number.__formatOptions?
	//		_applyPattern is usually called via dojo.number.format() which
	//		populates an extra property in the options parameter, "customs".
	//		The customs object specifies group and decimal parameters if set.

	//TODO: support escapes
	options = options || {};
	var group = options.customs.group;
	var decimal = options.customs.decimal;

	var patternList = pattern.split(';');
	var positivePattern = patternList[0];
	pattern = patternList[(value < 0) ? 1 : 0] || ("-" + positivePattern);

	//TODO: only test against unescaped
	if(pattern.indexOf('%') != -1){
		value *= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		value *= 1000; // per mille
	}else if(pattern.indexOf('\u00a4') != -1){
		group = options.customs.currencyGroup || group;//mixins instead?
		decimal = options.customs.currencyDecimal || decimal;// Should these be mixins instead?
		pattern = pattern.replace(/\u00a4{1,3}/, function(match){
			var prop = ["symbol", "currency", "displayName"][match.length-1];
			return options[prop] || options.currency || "";
		});
	}else if(pattern.indexOf('E') != -1){
		throw new Error("exponential notation not supported");
	}
	
	//TODO: support @ sig figs?
	var numberPatternRE = dojo.number._numberPatternRE;
	var numberPattern = positivePattern.match(numberPatternRE);
	if(!numberPattern){
		throw new Error("unable to find a number expression in pattern: "+pattern);
	}
	return pattern.replace(numberPatternRE,
		dojo.number._formatAbsolute(value, numberPattern[0], {decimal: decimal, group: group, places: options.places}));
}

dojo.number.round = function(/*Number*/value, /*Number*/places, /*Number?*/multiple){
	//	summary:
	//		Rounds the number at the given number of places
	//	value:
	//		the number to round
	//	places:
	//		the number of decimal places where rounding takes place
	//	multiple:
	//		rounds next place to nearest multiple

	var pieces = String(value).split(".");
	var length = (pieces[1] && pieces[1].length) || 0;
	if(length > places){
		var factor = Math.pow(10, places);
		if(multiple > 0){factor *= 10/multiple;places++;} //FIXME
		value = Math.round(value * factor)/factor;

		// truncate to remove any residual floating point values
		pieces = String(value).split(".");
		length = (pieces[1] && pieces[1].length) || 0;
		if(length > places){
			pieces[1] = pieces[1].substr(0, places);
			value = Number(pieces.join("."));
		}
	}
	return value; //Number
}

/*=====
dojo.number.__formatAbsoluteOptions = function(kwArgs){
	//	decimal: String?
	//		the decimal separator
	//	group: String?
	//		the group separator
	//	places: Integer?
	//		number of decimal places
	//	round: Number?
	//		5 rounds to nearest .5; 0 rounds to nearest whole (default). -1
	//		means don't round.
}
=====*/

dojo.number._formatAbsolute = function(/*Number*/value, /*String*/pattern, /*dojo.number.__formatAbsoluteOptions?*/options){
	// summary: 
	//		Apply numeric pattern to absolute value using options. Gives no
	//		consideration to local customs.
	// value:
	//		the number to be formatted, ignores sign
	// pattern:
	//		the number portion of a pattern (e.g. #,##0.00)
	options = options || {};
	if(options.places === true){options.places=0;}
	if(options.places === Infinity){options.places=6;} // avoid a loop; pick a limit

	var patternParts = pattern.split(".");
	var maxPlaces = (options.places >= 0) ? options.places : (patternParts[1] && patternParts[1].length) || 0;
	if(!(options.round < 0)){
		value = dojo.number.round(value, maxPlaces, options.round);
	}

	var valueParts = String(Math.abs(value)).split(".");
	var fractional = valueParts[1] || "";
	if(options.places){
		valueParts[1] = dojo.string.pad(fractional.substr(0, options.places), options.places, '0', true);
	}else if(patternParts[1] && options.places !== 0){
		// Pad fractional with trailing zeros
		var pad = patternParts[1].lastIndexOf("0") + 1;
		if(pad > fractional.length){
			valueParts[1] = dojo.string.pad(fractional, pad, '0', true);
		}

		// Truncate fractional
		var places = patternParts[1].length;
		if(places < fractional.length){
			valueParts[1] = fractional.substr(0, places);
		}
	}else{
		if(valueParts[1]){ valueParts.pop(); }
	}

	// Pad whole with leading zeros
	var patternDigits = patternParts[0].replace(',', '');
	pad = patternDigits.indexOf("0");
	if(pad != -1){
		pad = patternDigits.length - pad;
		if(pad > valueParts[0].length){
			valueParts[0] = dojo.string.pad(valueParts[0], pad);
		}

		// Truncate whole
		if(patternDigits.indexOf("#") == -1){
			valueParts[0] = valueParts[0].substr(valueParts[0].length - pad);
		}
	}

	// Add group separators
	var index = patternParts[0].lastIndexOf(',');
	var groupSize, groupSize2;
	if(index != -1){
		groupSize = patternParts[0].length - index - 1;
		var remainder = patternParts[0].substr(0, index);
		index = remainder.lastIndexOf(',');
		if(index != -1){
			groupSize2 = remainder.length - index - 1;
		}
	}
	var pieces = [];
	for(var whole = valueParts[0]; whole;){
		var off = whole.length - groupSize;
		pieces.push((off > 0) ? whole.substr(off) : whole);
		whole = (off > 0) ? whole.slice(0, off) : "";
		if(groupSize2){
			groupSize = groupSize2;
			delete groupSize2;
		}
	}
	valueParts[0] = pieces.reverse().join(options.group || ",");

	return valueParts.join(options.decimal || ".");
};

/*=====
dojo.number.__regexpOptions = function(kwArgs){
	//	pattern: String?
	//		override pattern with this string.  Default is provided based on
	//		locale.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	locale: String?
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default
	//	places: Number|String?
	//		number of decimal places to accept: Infinity, a positive number, or
	//		a range "n,m".  By default, defined by pattern.
}
=====*/
dojo.number.regexp = function(/*dojo.number.__regexpOptions?*/options){
	//	summary:
	//		Builds the regular needed to parse a number
	//	description:
	//		Returns regular expression with positive and negative match, group
	//		and decimal separators
	return dojo.number._parseInfo(options).regexp; // String
}

dojo.number._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.i18n.getLocalization("dojo.cldr", "number", locale);
	var pattern = options.pattern || bundle[(options.type || "decimal") + "Format"];
//TODO: memoize?
	var group = bundle.group;
	var decimal = bundle.decimal;
	var factor = 1;

	if(pattern.indexOf('%') != -1){
		factor /= 100;
	}else if(pattern.indexOf('\u2030') != -1){
		factor /= 1000; // per mille
	}else{
		var isCurrency = pattern.indexOf('\u00a4') != -1;
		if(isCurrency){
			group = bundle.currencyGroup || group;
			decimal = bundle.currencyDecimal || decimal;
		}
	}

	//TODO: handle quoted escapes
	var patternList = pattern.split(';');
	if(patternList.length == 1){
		patternList.push("-" + patternList[0]);
	}

	var re = dojo.regexp.buildGroupRE(patternList, function(pattern){
		pattern = "(?:"+dojo.regexp.escapeString(pattern, '.')+")";
		return pattern.replace(dojo.number._numberPatternRE, function(format){
			var flags = {
				signed: false,
				separator: options.strict ? group : [group,""],
				fractional: options.fractional,
				decimal: decimal,
				exponent: false};
			var parts = format.split('.');
			var places = options.places;
			if(parts.length == 1 || places === 0){flags.fractional = false;}
			else{
				if(typeof places == "undefined"){ places = parts[1].lastIndexOf('0')+1; }
				if(places && options.fractional == undefined){flags.fractional = true;} // required fractional, unless otherwise specified
				if(!options.places && (places < parts[1].length)){ places += "," + parts[1].length; }
				flags.places = places;
			}
			var groups = parts[0].split(',');
			if(groups.length>1){
				flags.groupSize = groups.pop().length;
				if(groups.length>1){
					flags.groupSize2 = groups.pop().length;
				}
			}
			return "("+dojo.number._realNumberRegexp(flags)+")";
		});
	}, true);

	if(isCurrency){
		// substitute the currency symbol for the placeholder in the pattern
		re = re.replace(/(\s*)(\u00a4{1,3})(\s*)/g, function(match, before, target, after){
			var prop = ["symbol", "currency", "displayName"][target.length-1];
			var symbol = dojo.regexp.escapeString(options[prop] || options.currency || "");
			before = before ? "\\s" : "";
			after = after ? "\\s" : "";
			if(!options.strict){
				if(before){before += "*";}
				if(after){after += "*";}
				return "(?:"+before+symbol+after+")?";
			}
			return before+symbol+after;
		});
	}

//TODO: substitute localized sign/percent/permille/etc.?

	// normalize whitespace and return
	return {regexp: re.replace(/[\xa0 ]/g, "[\\s\\xa0]"), group: group, decimal: decimal, factor: factor}; // Object
}

/*=====
dojo.number.__parseOptions = function(kwArgs){
	//	pattern: String
	//		override pattern with this string.  Default is provided based on
	//		locale.
	//	type: String?
	//		choose a format type based on the locale from the following:
	//		decimal, scientific, percent, currency. decimal by default.
	//	locale: String
	//		override the locale used to determine formatting rules
	//	strict: Boolean?
	//		strict parsing, false by default
	//	currency: Object
	//		object with currency information
}
=====*/
dojo.number.parse = function(/*String*/expression, /*dojo.number.__parseOptions?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Number, using
	//		locale-specific settings.
	// description:
	//		Create a Number from a string using a known localized pattern.
	//		Formatting patterns are chosen appropriate to the locale.
	//		Formatting patterns are implemented using the syntax described at
	//		*URL*
	// expression:
	//		A string representation of a Number
	var info = dojo.number._parseInfo(options);
	var results = (new RegExp("^"+info.regexp+"$")).exec(expression);
	if(!results){
		return NaN; //NaN
	}
	var absoluteMatch = results[1]; // match for the positive expression
	if(!results[1]){
		if(!results[2]){
			return NaN; //NaN
		}
		// matched the negative pattern
		absoluteMatch =results[2];
		info.factor *= -1;
	}

	// Transform it to something Javascript can parse as a number.  Normalize
	// decimal point and strip out group separators or alternate forms of whitespace
	absoluteMatch = absoluteMatch.
		replace(new RegExp("["+info.group + "\\s\\xa0"+"]", "g"), "").
		replace(info.decimal, ".");
	// Adjust for negative sign, percent, etc. as necessary
	return Number(absoluteMatch) * info.factor; //Number
};

/*=====
dojo.number.__realNumberRegexpFlags = function(kwArgs){
	//	places: Number?
	//		The integer number of decimal places or a range given as "n,m".  If
	//		not given, the decimal part is optional and the number of places is
	//		unlimited.
	//	decimal: String?
	//		A string for the character used as the decimal point.  Default
	//		is ".".
	//	fractional: Boolean|Array?
	//		Whether decimal places are allowed.  Can be true, false, or [true,
	//		false].  Default is [true, false]
	//	exponent: Boolean|Array?
	//		Express in exponential notation.  Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if the
	//		exponential part is present are not).
	//	eSigned: Boolean|Array?
	//		The leading plus-or-minus sign on the exponent.  Can be true,
	//		false, or [true, false].  Default is [true, false], (i.e. will
	//		match if it is signed or unsigned).  flags in regexp.integer can be
	//		applied.
}
=====*/

dojo.number._realNumberRegexp = function(/*dojo.number.__realNumberRegexpFlags?*/flags){
	// summary:
	//		Builds a regular expression to match a real number in exponential
	//		notation
	// flags:
	//		An object

	// assign default values to missing paramters
	flags = flags || {};
	if(typeof flags.places == "undefined"){ flags.places = Infinity; }
	if(typeof flags.decimal != "string"){ flags.decimal = "."; }
	if(typeof flags.fractional == "undefined" || /^0/.test(flags.places)){ flags.fractional = [true, false]; }
	if(typeof flags.exponent == "undefined"){ flags.exponent = [true, false]; }
	if(typeof flags.eSigned == "undefined"){ flags.eSigned = [true, false]; }

	// integer RE
	var integerRE = dojo.number._integerRegexp(flags);

	// decimal RE
	var decimalRE = dojo.regexp.buildGroupRE(flags.fractional,
		function(q){
			var re = "";
			if(q && (flags.places!==0)){
				re = "\\" + flags.decimal;
				if(flags.places == Infinity){ 
					re = "(?:" + re + "\\d+)?"; 
				}else{
					re += "\\d{" + flags.places + "}"; 
				}
			}
			return re;
		},
		true
	);

	// exponent RE
	var exponentRE = dojo.regexp.buildGroupRE(flags.exponent,
		function(q){ 
			if(q){ return "([eE]" + dojo.number._integerRegexp({ signed: flags.eSigned}) + ")"; }
			return ""; 
		}
	);

	// real number RE
	var realRE = integerRE + decimalRE;
	// allow for decimals without integers, e.g. .25
	if(decimalRE){realRE = "(?:(?:"+ realRE + ")|(?:" + decimalRE + "))";}
	return realRE + exponentRE; // String
};

/*=====
dojo.number.__integerRegexpFlags = function(kwArgs){
	//	signed: Boolean?
	//		The leading plus-or-minus sign. Can be true, false, or [true,
	//		false]. Default is [true, false], (i.e. will match if it is signed
	//		or unsigned).
	//	separator: String?
	//		The character used as the thousands separator. Default is no
	//		separator. For more than one symbol use an array, e.g. [",", ""],
	//		makes ',' optional.
	//	groupSize: Number?
	//		group size between separators
	//	flags.groupSize2: Number?
	//		second grouping (for India)
}
=====*/

dojo.number._integerRegexp = function(/*dojo.number.__integerRegexpFlags?*/flags){
	// summary: 
	//		Builds a regular expression that matches an integer
	// flags: 
	//		An object

	// assign default values to missing paramters
	flags = flags || {};
	if(typeof flags.signed == "undefined"){ flags.signed = [true, false]; }
	if(typeof flags.separator == "undefined"){
		flags.separator = "";
	}else if(typeof flags.groupSize == "undefined"){
		flags.groupSize = 3;
	}
	// build sign RE
	var signRE = dojo.regexp.buildGroupRE(flags.signed,
		function(q) { return q ? "[-+]" : ""; },
		true
	);

	// number RE
	var numberRE = dojo.regexp.buildGroupRE(flags.separator,
		function(sep){
			if(!sep){
				return "(?:0|[1-9]\\d*)";
			}

			sep = dojo.regexp.escapeString(sep);
			if(sep == " "){ sep = "\\s"; }
			else if(sep == "\xa0"){ sep = "\\s\\xa0"; }

			var grp = flags.groupSize, grp2 = flags.groupSize2;
			if(grp2){
				var grp2RE = "(?:0|[1-9]\\d{0," + (grp2-1) + "}(?:[" + sep + "]\\d{" + grp2 + "})*[" + sep + "]\\d{" + grp + "})";
				return ((grp-grp2) > 0) ? "(?:" + grp2RE + "|(?:0|[1-9]\\d{0," + (grp-1) + "}))" : grp2RE;
			}
			return "(?:0|[1-9]\\d{0," + (grp-1) + "}(?:[" + sep + "]\\d{" + grp + "})*)";
		},
		true
	);

	// integer RE
	return signRE + numberRE; // String
}

}

if(!dojo._hasResource["dijit.form.NumberTextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.NumberTextBox"] = true;
dojo.provide("dijit.form.NumberTextBox");




dojo.declare(
	"dijit.form.NumberTextBoxMixin",
	null,
	{
		// summary:
		//		A mixin for all number textboxes
		regExpGen: dojo.number.regexp,

		format: function(/*Number*/ value, /*Object*/ constraints){
			if(isNaN(value)){ return ""; }
			return dojo.number.format(value, constraints);
		},

		parse: dojo.number.parse,

		filter: function(/*Number*/ value){
			if(typeof value == "string"){ return this.inherited('filter', arguments); }
			return (isNaN(value) ? '' : value);
		},

		value: NaN
	}
);

dojo.declare(
	"dijit.form.NumberTextBox",
	[dijit.form.RangeBoundTextBox,dijit.form.NumberTextBoxMixin],
	{
		// summary:
		//		A validating, serializable, range-bound text box.
		// constraints object: min, max, places
	}
);

}

if(!dojo._hasResource["dijit.form.NumberSpinner"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.NumberSpinner"] = true;
dojo.provide("dijit.form.NumberSpinner");




dojo.declare(
"dijit.form.NumberSpinner",
[dijit.form._Spinner, dijit.form.NumberTextBoxMixin],
{
	// summary: Number Spinner
	// description: This widget is the same as NumberTextBox but with up/down arrows added

	required: true,

	adjust: function(/* Object */ val, /*Number*/ delta){
		// summary: change Number val by the given amount
		var newval = val+delta;
		if(isNaN(val) || isNaN(newval)){ return val; }
		if((typeof this.constraints.max == "number") && (newval > this.constraints.max)){
			newval = this.constraints.max;
		}
		if((typeof this.constraints.min == "number") && (newval < this.constraints.min)){
			newval = this.constraints.min;
		}
		return newval;
	}
});

}

if(!dojo._hasResource["dijit.form.FilteringSelect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.FilteringSelect"] = true;
dojo.provide("dijit.form.FilteringSelect");



dojo.declare(
	"dijit.form.FilteringSelect",
	[dijit.form.MappedTextBox, dijit.form.ComboBoxMixin],
	{
		/*
		 * summary
		 *	Enhanced version of HTML's <select> tag.
		 *
		 *	Similar features:
		 *	  - There is a drop down list of possible values.
		 *	- You can only enter a value from the drop down list.  (You can't enter an arbitrary value.)
		 *	- The value submitted with the form is the hidden value (ex: CA),
		 *	  not the displayed value a.k.a. label (ex: California)
		 *
		 *	Enhancements over plain HTML version:
		 *	- If you type in some text then it will filter down the list of possible values in the drop down list.
		 *	- List can be specified either as a static list or via a javascript function (that can get the list from a server)
		 */

		// searchAttr: String
		//		Searches pattern match against this field

		// labelAttr: String
		//		Optional.  The text that actually appears in the drop down.
		//		If not specified, the searchAttr text is used instead.
		labelAttr: "",

		// labelType: String
		//		"html" or "text"
		labelType: "text",

		_isvalid:true,

		isValid:function(){
			return this._isvalid;
		},

		_callbackSetLabel: function(/*Array*/ result, /*Object*/ dataObject, /*Boolean, optional*/ priorityChange){
			// summary
			//	Callback function that dynamically sets the label of the ComboBox

			// setValue does a synchronous lookup,
			// so it calls _callbackSetLabel directly,
			// and so does not pass dataObject
			// dataObject==null means do not test the lastQuery, just continue
			if(dataObject&&dataObject.query[this.searchAttr]!=this._lastQuery){return;}
			if(!result.length){
				//#3268: do nothing on bad input
				//this._setValue("", "");
				//#3285: change CSS to indicate error
				if(!this._hasFocus){ this.valueNode.value=""; }
				dijit.form.TextBox.superclass.setValue.call(this, undefined, !this._hasFocus);
				this._isvalid=false;
				this.validate(this._hasFocus);
			}else{
				this._setValueFromItem(result[0], priorityChange);
			}
		},

		_openResultList: function(/*Object*/ results, /*Object*/ dataObject){
			// #3285: tap into search callback to see if user's query resembles a match
			if(dataObject.query[this.searchAttr]!=this._lastQuery){return;}
			this._isvalid=results.length!=0;
			this.validate(true);
			dijit.form.ComboBoxMixin.prototype._openResultList.apply(this, arguments);
		},

		getValue:function(){
			// don't get the textbox value but rather the previously set hidden value
			return this.valueNode.value;
		},

		_getValueField:function(){
			// used for option tag selects
			return "value";
		},

		_setValue:function(/*String*/ value, /*String*/ displayedValue, /*Boolean, optional*/ priorityChange){
			this.valueNode.value = value;
			dijit.form.FilteringSelect.superclass.setValue.call(this, value, priorityChange, displayedValue);
			this._lastDisplayedValue = displayedValue;
		},

		setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
			// summary
			//	Sets the value of the select.
			//	Also sets the label to the corresponding value by reverse lookup.

			//#3347: fetchItemByIdentity if no keyAttr specified
			var self=this;
			var handleFetchByIdentity = function(item, priorityChange){
				if(item){
					if(self.store.isItemLoaded(item)){
						self._callbackSetLabel([item], undefined, priorityChange);
					}else{
						self.store.loadItem({item:item, onItem: function(result, dataObject){self._callbackSetLabel(result, dataObject, priorityChange)}});
					}
				}else{
					self._isvalid=false;
					// prevent errors from Tooltip not being created yet
					self.validate(false);
				}
			}
			this.store.fetchItemByIdentity({identity: value, onItem: function(item){handleFetchByIdentity(item, priorityChange)}});
		},

		_setValueFromItem: function(/*item*/ item, /*Boolean, optional*/ priorityChange){
			// summary
			//	Set the displayed valued in the input box, based on a selected item.
			//	Users shouldn't call this function; they should be calling setDisplayedValue() instead
			this._isvalid=true;
			this._setValue(this.store.getIdentity(item), this.labelFunc(item, this.store), priorityChange);
		},

		labelFunc: function(/*item*/ item, /*dojo.data.store*/ store){
			// summary: Event handler called when the label changes
			// returns the label that the ComboBox should display
			return store.getValue(item, this.searchAttr);
		},

		onkeyup: function(/*Event*/ evt){
			// summary: internal function
			// FilteringSelect needs to wait for the complete label before committing to a reverse lookup
			//this.setDisplayedValue(this.textbox.value);
		},

		_doSelect: function(/*Event*/ tgt){
			// summary:
			//	ComboBox's menu callback function
			//	FilteringSelect overrides this to set both the visible and hidden value from the information stored in the menu
			this.item = tgt.item;
			this._setValueFromItem(tgt.item, true);
		},

		setDisplayedValue:function(/*String*/ label){
			// summary:
			//	Set textbox to display label
			//	Also performs reverse lookup to set the hidden value
			//	Used in InlineEditBox

			if(this.store){
				var query={};
				this._lastQuery=query[this.searchAttr]=label;
				// if the label is not valid, the callback will never set it,
				// so the last valid value will get the warning textbox
				// set the textbox value now so that the impending warning will make sense to the user
				this.textbox.value=label;
				this._lastDisplayedValue=label;
				this.store.fetch({query:query, queryOptions:{ignoreCase:this.ignoreCase, deep:true}, onComplete: dojo.hitch(this, this._callbackSetLabel)});
			}
		},

		_getMenuLabelFromItem:function(/*Item*/ item){
			// internal function to help ComboBoxMenu figure out what to display
			if(this.labelAttr){return {html:this.labelType=="html", label:this.store.getValue(item, this.labelAttr)};}
			else{
				// because this function is called by ComboBoxMenu, this.inherited tries to find the superclass of ComboBoxMenu
				return dijit.form.ComboBoxMixin.prototype._getMenuLabelFromItem.apply(this, arguments);
			}
		},

		postMixInProperties: function(){
			dijit.form.ComboBoxMixin.prototype.postMixInProperties.apply(this, arguments);
			dijit.form.MappedTextBox.prototype.postMixInProperties.apply(this, arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");




dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer.
		//		Widgets which mixin this code must define layout() to lay out the children

		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under document.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }
			this._started=true;

			if(this.getChildren){
				dojo.forEach(this.getChildren(), function(child){ child.startup(); });
			}

			// If I am a top level widget
			if(!this.getParent || !this.getParent()){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// since my parent isn't a layout container, and my style is width=height=100% (or something similar),
				// then I need to watch when the window resizes, and size myself accordingly
				// (passing in no argument to resize means that it has to glean the size itself)
				this.connect(window, 'onresize', function(){this.resize();});
			}
		},

		resize: function(args){
			// summary:
			//		Explicitly set this widget's size (in pixels),
			//		and then call layout() to resize contents (and maybe adjust child widgets)
			//	
			// args: Object?
			//		{w: int, h: int, l: int, t: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(args){
				dojo.marginBox(node, args);

				// set offset of the node
				if(args.t){ node.style.top = args.t + "px"; }
				if(args.l){ node.style.left = args.l + "px"; }
			}
			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = dojo.mixin(dojo.marginBox(node), args||{});

			// Save the size of my content box.
			this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

			// Callback for widget to adjust size of it's children
			this.layout();
		},

		layout: function(){
			//	summary
			//		Widgets override this method to size & position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return it's content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me=dojo._getMarginExtents(node, cs);
	var pb=dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		/**
		 * summary
		 *		Layout a bunch of child dom nodes within a parent dom node
		 * container:
		 *		parent node
		 * dim:
		 *		{l, t, w, h} object specifying dimensions of container into which to place children
		 * children:
		 *		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]
		 */

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect it's height.
			if(pos=="top" || pos=="bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos=="top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos=="left" || pos=="right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos=="left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos=="client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dijit.layout.StackContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackContainer"] = true;
dojo.provide("dijit.layout.StackContainer");





dojo.declare(
	"dijit.layout.StackContainer",
	dijit.layout._LayoutWidget,

	// summary
	//	A container that has multiple children, but shows only
	//	one child at a time (like looking at the pages in a book one by one).
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//
	//	Can be base class for container, Wizard, Show, etc.
{
	// doLayout: Boolean
	//  if true, change the size of my currently displayed child to match my size
	doLayout: true,

	_started: false,

	// selectedChildWidget: Widget
	//	References the currently selected child widget, if any

	postCreate: function(){
		dijit.setWaiRole((this.containerNode || this.domNode), "tabpanel");
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},
	
	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel
		dojo.forEach(children, this._setupChild, this);

		// Figure out which child to initially display
		dojo.some(children, function(child){
			if(child.selected){
				this.selectedChildWidget = child;
			}
			return child.selected;
		}, this);

		var selected = this.selectedChildWidget;

		// Default to the first child
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}
		if(selected){
			this._showChild(selected);
		}

		// Now publish information about myself so any StackControllers can initialize..
		dojo.publish(this.id+"-startup", [{children: children, selected: selected}]);
		this.inherited("startup",arguments);
		this._started = true;
	},

	_setupChild: function(/*Widget*/ page){
		// Summary: prepare the given child

		page.domNode.style.display = "none";

		// since we are setting the width/height of the child elements, they need
		// to be position:relative, or IE has problems (See bug #2033)
		page.domNode.style.position = "relative";

		return page; // dijit._Widget
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		// summary: Adds a widget to the stack
		 
		dijit._Container.prototype.addChild.apply(this, arguments);
		child = this._setupChild(child);

		if(this._started){
			// in case the tab titles have overflowed from one line to two lines
			this.layout();

			dojo.publish(this.id+"-addChild", [child, insertIndex]);

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*Widget*/ page){
		// summary: Removes the pane from the stack

		dijit._Container.prototype.removeChild.apply(this, arguments);

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id+"-removeChild", [page]);

			// in case the tab titles now take up one line instead of two lines
			this.layout();
		}

		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}
	},

	selectChild: function(/*Widget*/ page){
		// summary:
		//	Show the given widget (which must be one of my children)

		page = dijit.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			this._transition(page, this.selectedChildWidget);
			this.selectedChildWidget = page;
			dojo.publish(this.id+"-selectChild", [page]);
		}
	},

	_transition: function(/*Widget*/newWidget, /*Widget*/oldWidget){
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// page must be visible for resizing to work
		if(this.doLayout && newWidget.resize){
			newWidget.resize(this._containerContentBox || this._contentBox);
		}
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary: Gets the next/previous child widget in this container from the current selection
		var children = this.getChildren();
		var index = dojo.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// Summary: advance to next page
		this.selectChild(this._adjacent(true));
	},

	back: function(){
		// Summary: go back to previous page
		this.selectChild(this._adjacent(false));
	},

	_onKeyPress: function(e){
		dojo.publish(this.id+"-containerKeyPress", [{ e: e, page: this}]);
	},

	layout: function(){
		if(this.doLayout && this.selectedChildWidget && this.selectedChildWidget.resize){
			this.selectedChildWidget.resize(this._contentBox);
		}
	},

	_showChild: function(/*Widget*/ page){
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;

		page.domNode.style.display="";
		if(page._loadCheck){
			page._loadCheck(); // trigger load in ContentPane
		}
		if(page.onShow){
			page.onShow();
		}
	},

	_hideChild: function(/*Widget*/ page){
		page.selected=false;
		page.domNode.style.display="none";
		if(page.onHide){
			page.onHide();
		}
	},

	closeChild: function(/*Widget*/ page){
		// summary
		//	callback when user clicks the [X] to remove a page
		//	if onClose() returns true then remove and destroy the childd
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroy();
		}
	},

	destroy: function(){
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	}
});

dojo.declare(
	"dijit.layout.StackController",
	[dijit._Widget, dijit._Templated, dijit._Container],
	{
		// summary:
		//	Set of buttons to select a page in a page list.
		//	Monitors the specified StackContainer, and whenever a page is
		//	added, deleted, or selected, updates itself accordingly.

		templateString: "<span wairole='tablist' dojoAttachEvent='onkeypress' class='dijitStackController'></span>",

		// containerId: String
		//	the id of the page container that I point to
		containerId: "",

		// buttonWidget: String
		//	the name of the button widget to create to correspond to each page
		buttonWidget: "dijit.layout._StackButton",

		postCreate: function(){
			dijit.setWaiRole(this.domNode, "tablist");

			this.pane2button = {};		// mapping from panes to buttons
			this._subscriptions=[
				dojo.subscribe(this.containerId+"-startup", this, "onStartup"),
				dojo.subscribe(this.containerId+"-addChild", this, "onAddChild"),
				dojo.subscribe(this.containerId+"-removeChild", this, "onRemoveChild"),
				dojo.subscribe(this.containerId+"-selectChild", this, "onSelectChild"),
				dojo.subscribe(this.containerId+"-containerKeyPress", this, "onContainerKeyPress")
			];
		},

		onStartup: function(/*Object*/ info){
			// summary: called after StackContainer has finished initializing
			dojo.forEach(info.children, this.onAddChild, this);
			this.onSelectChild(info.selected);
		},

		destroy: function(){
			dojo.forEach(this._subscriptions, dojo.unsubscribe);
			this.inherited("destroy",arguments);
		},

		onAddChild: function(/*Widget*/ page, /*Integer?*/ insertIndex){
			// summary:
			//   Called whenever a page is added to the container.
			//   Create button corresponding to the page.

			// add a node that will be promoted to the button widget
			var refNode = document.createElement("span");
			this.domNode.appendChild(refNode);
			// create an instance of the button widget
			var cls = dojo.getObject(this.buttonWidget);
			var button = new cls({label: page.title, closeButton: page.closable}, refNode);
			this.addChild(button, insertIndex);
			this.pane2button[page] = button;
			page.controlButton = button;	// this value might be overwritten if two tabs point to same container
			
			dojo.connect(button, "onClick", dojo.hitch(this,"onButtonClick",page));
			dojo.connect(button, "onClickCloseButton", dojo.hitch(this,"onCloseButtonClick",page));
			
			if(!this._currentChild){ // put the first child into the tab order
				button.focusNode.setAttribute("tabIndex","0");
				this._currentChild = page;
			}
		},

		onRemoveChild: function(/*Widget*/ page){
			// summary:
			//   Called whenever a page is removed from the container.
			//   Remove the button corresponding to the page.
			if(this._currentChild === page){ this._currentChild = null; }
			var button = this.pane2button[page];
			if(button){
				// TODO? if current child { reassign }
				button.destroy();
			}
			this.pane2button[page] = null;
		},

		onSelectChild: function(/*Widget*/ page){
			// summary:
			//	Called when a page has been selected in the StackContainer, either by me or by another StackController

			if(!page){ return; }

			if(this._currentChild){
				var oldButton=this.pane2button[this._currentChild];
				oldButton.setChecked(false);
				oldButton.focusNode.setAttribute("tabIndex", "-1");
			}

			var newButton=this.pane2button[page];
			newButton.setChecked(true);
			this._currentChild = page;
			newButton.focusNode.setAttribute("tabIndex", "0");
		},

		onButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons is pressed in an attempt to select a page
			var container = dijit.byId(this.containerId);	// TODO: do this via topics?
			container.selectChild(page); 
		},

		onCloseButtonClick: function(/*Widget*/ page){
			// summary:
			//   Called whenever one of my child buttons [X] is pressed in an attempt to close a page
			var container = dijit.byId(this.containerId);
			container.closeChild(page);
			var b = this.pane2button[this._currentChild];
			if(b){
				dijit.focus(b.focusNode || b.domNode);
			}
		},
		
		// TODO: this is a bit redundant with forward, back api in StackContainer
		adjacent: function(/*Boolean*/ forward){
			// find currently focused button in children array
			var children = this.getChildren();
			var current = dojo.indexOf(children, this.pane2button[this._currentChild]);
			// pick next button to focus on
			var offset = forward ? 1 : children.length - 1;
			return children[ (current + offset) % children.length ]; // dijit._Widget
		},

		onkeypress: function(/*Event*/ e){
			// summary:
			//   Handle keystrokes on the page list, for advancing to next/previous button
			//   and closing the current page if the page is closable.

			if(this.disabled || e.altKey ){ return; }
			var forward = true;
			if(e.ctrlKey || !e._djpage){
				var k = dojo.keys;
				switch(e.keyCode){
					case k.LEFT_ARROW:
					case k.UP_ARROW:
					case k.PAGE_UP:
						forward = false;
						// fall through
					case k.RIGHT_ARROW:
					case k.DOWN_ARROW:
					case k.PAGE_DOWN:
						this.adjacent(forward).onClick();
						dojo.stopEvent(e);
						break;
					case k.DELETE:
						if(this._currentChild.closable){
							this.onCloseButtonClick(this._currentChild);
						}
						dojo.stopEvent(e);
						break;
					default:
						if(e.ctrlKey){
							if(e.keyCode == k.TAB){
								this.adjacent(!e.shiftKey).onClick();
								dojo.stopEvent(e);
							}else if(e.keyChar == "w"){
								if(this._currentChild.closable){
									this.onCloseButtonClick(this._currentChild);
								}
								dojo.stopEvent(e); // avoid browser tab closing.
							}
						}
				}
			}
		},

		onContainerKeyPress: function(/*Object*/ info){
			info.e._djpage = info.page;
			this.onkeypress(info.e);
		}
});

dojo.declare("dijit.layout._StackButton",
	dijit.form.ToggleButton,
	{
	// summary
	//	Internal widget used by StackContainer.
	//	The button-like or tab-like object you click to select or delete a page
	
	tabIndex: "-1", // StackContainer buttons are not in the tab order by default
	
	postCreate: function(/*Event*/ evt){
		dijit.setWaiRole((this.focusNode || this.domNode), "tab");
		this.inherited("postCreate", arguments);
	},
	
	onClick: function(/*Event*/ evt){
		// summary: This is for TabContainer where the tabs are <span> rather than button,
		// 	so need to set focus explicitly (on some browsers)
		dijit.focus(this.focusNode);

		// ... now let StackController catch the event and tell me what to do
	},

	onClickCloseButton: function(/*Event*/ evt){
		// summary
		//	StackContainer connects to this function; if your widget contains a close button
		//	then clicking it should call this function.
		evt.stopPropagation();
	}
});

// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// title: String
	//		Title of this widget.  Used by TabContainer to the name the tab, etc.
	title: "",

	// selected: Boolean
	//		Is this child currently selected?
	selected: false,

	// closable: Boolean
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,	// true if user can close this tab pane

	onClose: function(){
		// summary: Callback if someone tries to close the child, child will be closed if func returns true
		return true;
	}
});

}

if(!dojo._hasResource["dijit.layout.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabContainer"] = true;
dojo.provide("dijit.layout.TabContainer");




dojo.declare("dijit.layout.TabContainer",
	[dijit.layout.StackContainer, dijit._Templated],
	{	
	// summary: 
	//	A Container with Title Tabs, each one pointing at a pane in the container.
	// description:
	//	A TabContainer is a container that has multiple panes, but shows only
	//	one pane at a time.  There are a set of tabs corresponding to each pane,
	//	where each tab has the title (aka title) of the pane, and optionally a close button.
	//
	//	Publishes topics <widgetId>-addChild, <widgetId>-removeChild, and <widgetId>-selectChild
	//	(where <widgetId> is the id of the TabContainer itself.
	//
	// tabPosition: String
	//   Defines where tabs go relative to tab content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	templateString: null,	// override setting in StackContainer
	templateString:"<div class=\"dijitTabContainer\">\r\n\t<div dojoAttachPoint=\"tablistNode\"></div>\r\n\t<div class=\"dijitTabPaneWrapper\" dojoAttachPoint=\"containerNode\"></div>\r\n</div>\r\n",

	postCreate: function(){	
		dijit.layout.TabContainer.superclass.postCreate.apply(this, arguments);
		// create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = new dijit.layout.TabController(
			{
				id: this.id + "_tablist",
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id
			}, this.tablistNode);		
	},

	_setupChild: function(/* Widget */tab){
		dojo.addClass(tab.domNode, "dijitTabPane");
		this.inherited("_setupChild",arguments);
		return tab; // Widget
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();
		this.inherited("startup",arguments);

		if(dojo.isSafari){
			// sometimes safari 3.0.3 miscalculates the height of the tab labels, see #4058
			setTimeout(dojo.hitch(this, "layout"), 0);
		}
	},

	layout: function(){
		// Summary: Configure the content pane to take up all the space except for where the tabs are
		if(!this.doLayout){ return; }

		// position and size the titles and the container node
		var titleAlign=this.tabPosition.replace(/-h/,"");
		var children = [
			{domNode: this.tablist.domNode, layoutAlign: titleAlign},
			{domNode: this.containerNode, layoutAlign: "client"}
		];
		dijit.layout.layoutChildren(this.domNode, this._contentBox, children);

		// Compute size to make each of my children.
		// children[1] is the margin-box size of this.containerNode, set by layoutChildren() call above
		this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[1]);

		if(this.selectedChildWidget){
			this._showChild(this.selectedChildWidget);
			if(this.doLayout && this.selectedChildWidget.resize){
				this.selectedChildWidget.resize(this._containerContentBox);
			}
		}
	},

	destroy: function(){
		this.tablist.destroy();
		this.inherited("destroy",arguments);
	}
});

//TODO: make private?
dojo.declare("dijit.layout.TabController",
	dijit.layout.StackController,
	{
	// summary:
	// 	Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	// description:
	//	Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//	TabController also monitors the TabContainer, and whenever a pane is
	//	added or deleted updates itself accordingly.

	templateString: "<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'></div>",

	// tabPosition: String
	//   Defines where tabs go relative to the content.
	//   "top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	// doLayout: Boolean
	// 	TODOC: deprecate doLayout? not sure.
	doLayout: true,

	// buttonWidget: String
	//	the name of the tab widget to create to correspond to each page
	buttonWidget: "dijit.layout._TabButton",

	postMixInProperties: function(){
		this["class"] = "dijitTabLabels-" + this.tabPosition + (this.doLayout ? "" : " dijitTabNoLayout");
		this.inherited("postMixInProperties",arguments);
	}
});

dojo.declare("dijit.layout._TabButton",
	dijit.layout._StackButton,
	{
	// summary:
	//	A tab (the thing you click to select a pane).
	// description:
	//	Contains the title of the pane, and optionally a close-button to destroy the pane.
	//	This is an internal widget and should not be instantiated directly.

	baseClass: "dijitTab",

	templateString:"<div dojoAttachEvent='onclick:onClick,onmouseenter:_onMouse,onmouseleave:_onMouse'>\r\n    <div class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\r\n        <span dojoAttachPoint='containerNode,focusNode'>${!label}</span>\r\n        <span dojoAttachPoint='closeButtonNode' class='closeImage' dojoAttachEvent='onmouseenter:_onMouse, onmouseleave:_onMouse, onclick:onClickCloseButton' stateModifier='CloseButton'>\r\n            <span dojoAttachPoint='closeText' class='closeText'>x</span>\r\n        </span>\r\n    </div>\r\n</div>\r\n",

	postCreate: function(){
		if(this.closeButton){
			dojo.addClass(this.innerDiv, "dijitClosable");
		} else {
			this.closeButtonNode.style.display="none";
		}
		this.inherited("postCreate",arguments); 
		dojo.setSelectable(this.containerNode, false);
	}
});

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");








dojo.declare(
	"dijit.layout.ContentPane",
	dijit._Widget,
{
	// summary:
	//		A widget that acts as a Container for other widgets, and includes a ajax interface
	// description:
	//		A widget that can be used as a standalone widget
	//		or as a baseclass for other widgets
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .setContent('<b>new content</b>')
	//
	//		Or you can send it a NodeList, .setContent(dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .setHref('url')
	//
	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; see setHref();
	href: "",

	// extractContent: Boolean
	//	Extract visible content from inside of <body> .... </body>
	extractContent: false,

	// parseOnLoad: Boolean
	//	parse content and create the widgets, if any
	parseOnLoad:	true,

	// preventCache: Boolean
	//		Cache content retreived externally
	preventCache:	false,

	// preload: Boolean
	//	Force load of data even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//	Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>", 

	// errorMessage: String
	//	Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>", 

	// isLoaded: Boolean
	//	Tells loading status see onLoad|onUnload for event hooks
	isLoaded: false,

	// class: String
	//	Class name to apply to ContentPane dom nodes
	"class": "dijitContentPane",

	postCreate: function(){
		// remove the title attribute so it doesn't show up when i hover
		// over a node
		this.domNode.title = "";

		if(this.preload){
			this._loadCheck();
		}

		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// for programatically created ContentPane (with <span> tag), need to muck w/CSS
		// or it's as though overflow:visible is set
		dojo.addClass(this.domNode, this["class"]);
	},

	startup: function(){
		if(this._started){ return; }
		this._checkIfSingleChild();
		if(this._singleChild){
			this._singleChild.startup();
		}
		this._loadCheck();
		this._started = true;
	},

	_checkIfSingleChild: function(){
		// summary:
		// 	Test if we have exactly one widget as a child, and if so assume that we are a container for that widget,
		//	and should propogate startup() and resize() calls to it.
		var childNodes = dojo.query(">", this.containerNode || this.domNode),
			childWidgets = childNodes.filter("[widgetId]");

		if(childNodes.length == 1 && childWidgets.length == 1){
			this.isContainer = true;
			this._singleChild = dijit.byNode(childWidgets[0]);
		}else{
			delete this.isContainer;
			delete this._singleChild;
		}
	},

	refresh: function(){
		// summary:
		//	Force a refresh (re-download) of content, be sure to turn off cache

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad(true);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage
		this.href = href;

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad();
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we cant run refresh and clear content
		// refresh should only work if we downloaded the content
		if(!this._isDownloaded){
			this.href = "";
			this._onUnloadHandler();
		}

		this._setContent(data || "");

		this._isDownloaded = false; // must be set after _setContent(..), pathadjust in dojox.layout.ContentPane

		if(this.parseOnLoad){
			this._createSubWidgets();
		}

		this._checkIfSingleChild();
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}

		this._onLoadHandler();
	},

	cancel: function(){
		// summary:
		//		Cancels a inflight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect
	},

	destroy: function(){
		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		// make sure we call onUnload
		this._onUnloadHandler();
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	},

	resize: function(size){
		dojo.marginBox(this.domNode, size);

		// Compute content box size in case we [later] need to size child
		// If either height or width wasn't specified by the user, then query node for it.
		// But note that setting the margin box and then immediately querying dimensions may return
		// inaccurate results, so try not to depend on it.
		var node = this.containerNode || this.domNode,
			mb = dojo.mixin(dojo.marginBox(node), size||{});

		this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

		// If we have a single widget child then size it to fit snugly within my borders
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}
	},

	_prepareLoad: function(forceLoad){
		// sets up for a xhrLoad, load is deferred until widget onShow
		// cancels a inflight download
		this.cancel();
		this.isLoaded = false;
		this._loadCheck(forceLoad);
	},

	_loadCheck: function(forceLoad){
		// call this when you change onShow (onSelected) status when selected in parent container
		// it's used as a trigger for href download when this.domNode.display != 'none'

		// sequence:
		// if no href -> bail
		// forceLoad -> always load
		// this.preload -> load when download not in progress, domNode display doesn't matter
		// this.refreshOnShow -> load when download in progress bails, domNode display !='none' AND
		//						this.open !== false (undefined is ok), isLoaded doesn't matter
		// else -> load when download not in progress, if this.open !== false (undefined is ok) AND
		//						domNode display != 'none', isLoaded must be false

		var displayState = ((this.open !== false) && (this.domNode.style.display != 'none'));

		if(this.href &&	
			(forceLoad ||
				(this.preload && !this._xhrDfd) ||
				(this.refreshOnShow && displayState && !this._xhrDfd) ||
				(!this.isLoaded && displayState && !this._xhrDfd)
			)
		){
			this._downloadExternalContent();
		}
	},

	_downloadExternalContent: function(){
		this._onUnloadHandler();

		// display loading message
		this._setContent(
			this.onDownloadStart.call(this)
		);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs);

		hand.addCallback(function(html){
			try{
				self.onDownloadEnd.call(self);
				self._isDownloaded = true;
				self.setContent.call(self, html); // onload event is called from here
			}catch(err){
				self._onError.call(self, 'Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.cancelled){
				// show error message in the pane
				self._onError.call(self, 'Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});
	},

	_onLoadHandler: function(){
		this.isLoaded = true;
		try{
			this.onLoad.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code');
		}
	},

	_onUnloadHandler: function(){
		this.isLoaded = false;
		this.cancel();
		try{
			this.onUnload.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code');
		}
	},

	_setContent: function(cont){
		this.destroyDescendants();

		try{
			var node = this.containerNode || this.domNode;
			while(node.firstChild){
				dojo._destroyElement(node.firstChild);
			}
			if(typeof cont == "string"){
				// dijit.ContentPane does only minimal fixes,
				// No pathAdjustments, script retrieval, style clean etc
				// some of these should be available in the dojox.layout.ContentPane
				if(this.extractContent){
					match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
					if(match){ cont = match[1]; }
				}
				node.innerHTML = cont;
			}else{
				// domNode or NodeList
				if(cont.nodeType){ // domNode (htmlNode 1 or textNode 3)
					node.appendChild(cont);
				}else{// nodelist or array such as dojo.Nodelist
					dojo.forEach(cont, function(n){
						node.appendChild(n.cloneNode(true));
					});
				}
			}
		}catch(e){
			// check if a domfault occurs when we are appending this.errorMessage
			// like for instance if domNode is a UL and we try append a DIV
			var errMess = this.onContentError(e);
			try{
				node.innerHTML = errMess;
			}catch(e){
				console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
			}
		}
	},

	_onError: function(type, err, consoleText){
		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent.call(this, errText);
		}
	},

	_createSubWidgets: function(){
		// summary: scan my contents and create subwidgets
		var rootNode = this.containerNode || this.domNode;
		try{
			dojo.parser.parse(rootNode, true);
		}catch(e){
			this._onError('Content', e, "Couldn't create widgets in "+this.id
				+(this.href ? " from "+this.href : ""));
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(e){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
	},

	onUnload: function(e){
		// summary:
		//		Event hook, is called before old content is cleared
	},

	onDownloadStart: function(){
		// summary:
		//		called before download starts
		//		the string returned by this function will be the html
		//		that tells the user we are loading something
		//		override with your own function if you want to change text
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		called on DOM faults, require fault etc in content
		//		default is to display errormessage inside pane
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs, default is to display
		//		errormessage inside pane. Overide function to change that.
		//		The string returned by this function will be the html
		//		that tells the user a error happend
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		called when download is finished
	}
});

}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd._copyKey = navigator.appVersion.indexOf("Macintosh") < 0 ? "ctrlKey" : "metaKey";

dojo.dnd.getCopyKeyState = function(e) {
	// summary: abstracts away the difference between selection on Mac and PC,
	//	and returns the state of the "copy" key to be pressed.
	// e: Event: mouse event
	return e[dojo.dnd._copyKey];	// Boolean
};

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = "dojoUnique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary: returns true, if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");

dojo.dnd.getViewport = function(){
	// summary: returns a viewport size (visible part of the window)

	// FIXME: need more docs!!
	var d = dojo.doc, dd = d.documentElement, w = window, b = dojo.body();
	if(dojo.isMozilla){
		return {w: dd.clientWidth, h: w.innerHeight};	// Object
	}else if(!dojo.isOpera && w.innerWidth){
		return {w: w.innerWidth, h: w.innerHeight};		// Object
	}else if (!dojo.isOpera && dd && dd.clientWidth){
		return {w: dd.clientWidth, h: dd.clientHeight};	// Object
	}else if (b.clientWidth){
		return {w: b.clientWidth, h: b.clientHeight};	// Object
	}
	return null;	// Object
};

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event:
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.dnd.getViewport(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event:
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo._abs(n, true);
				// console.debug(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				b.l += t.x + n.scrollLeft;
				b.t += t.y + n.scrollTop;
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2), 
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - b.l, ry = e.pageY - b.t, dx = 0, dy = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
					}else if(rx > b.w - w){
						dx = dojo.dnd.H_AUTOSCROLL_VALUE;
					}
				}
				//console.debug("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
					}else if(ry > b.h - h){
						dy = dojo.dnd.V_AUTOSCROLL_VALUE;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				// if(dx || dy){ console.debug(oldLeft + ", " + oldTop + "\n" + dx + ", " + dy + "\n" + n.scrollLeft + ", " + n.scrollTop); }
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary: an object, which makes a node follow the mouse, 
		//	used as a default mover, and as a base class for custom movers
		// node: Node: a node (or node's id) to be moved
		// e: Event: a mouse event, which started the move;
		//	only pageX and pageY properties are used
		// host: Object?: object which implements the functionality of the move,
		//	 and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = this.host = host, d = node.ownerDocument, 
			firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo, "stopEvent"),
			dojo.connect(d, "onselectstart", dojo, "stopEvent"),
			firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary: event processor for onmousemove
		// e: Event: mouse event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY});
	},
	onMouseUp: function(e){
		if(this.mouseButton == e.button){
			this.destroy();
		}
	},
	// utilities
	onFirstMove: function(){
		// summary: makes the node absolute; it is meant to be called only once
		this.node.style.position = "absolute";	// enforcing the absolute mode
		var m = dojo.marginBox(this.node);
		m.l -= this.marginBox.l;
		m.t -= this.marginBox.t;
		this.marginBox = m;
		this.host.onFirstMove(this);
		dojo.disconnect(this.events.pop());
	},
	destroy: function(){
		// summary: stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		handle: Node: a node (or node's id), which is used as a mouse handle
		//			if omitted, the node itself is used as a handle
		//		delay: Number: delay move by this number of pixels
		//		skip: Boolean: skip move of form elements
		//		mover: Object: a constructor of custom Mover
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary: stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary: event processor for onmousedown, creates a Mover for the node
		// e: Event: mouse event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(dojo.connect(this.handle, "onmousemove", this, "onMouseMove"));
			this.events.push(dojo.connect(this.handle, "onmouseup", this, "onMouseUp"));
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			new this.mover(this.node, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary: event processor for onmousemove, used only for delayed drags
		// e: Event: mouse event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			new this.mover(this.node, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary: event processor for onmouseup, used only for delayed delayed drags
		// e: Event: mouse event
		dojo.disconnect(this.events.pop());
		dojo.disconnect(this.events.pop());
	},
	onSelectStart: function(e){
		// summary: event processor for onselectevent and ondragevent
		// e: Event: mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary: called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove"); 
		dojo.addClass(this.node, "dojoMoveItem"); 
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary: called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary: called during the very first move notification,
		//	can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called during every move notification,
		//	should actually move the node, can be overwritten.
		this.onMoving(mover, leftTop);
		dojo.marginBox(mover.node, leftTop);
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called before every incremental move,
		//	can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called after every incremental move,
		//	can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		constraints: Function: a function, which calculates a constraint box,
		//			it is called in a context of the moveable object.
		//		within: Boolean: restrict move within boundaries.
		//	the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary: called during the very first move notification,
		//	can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover), m = mover.marginBox;
		c.r = c.l + c.w - (this.within ? m.w : 0);
		c.b = c.t + c.h - (this.within ? m.h : 0);
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called during every move notification,
		//	should actually move the node, can be overwritten.
		var c = this.constraintBox;
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		dojo.marginBox(mover.node, leftTop);
	}
});

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		box: Object: a constraint box
		//	the rest is passed to the base class
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		area: String: a parent's area to restrict the move,
		//			can be "margin", "border", "padding", or "content".
		//	the rest is passed to the base class
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode, 
				s = dojo.getComputedStyle(n), 
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// WARNING: below are obsolete objects, instead of custom movers use custom moveables (above)

dojo.dnd.move.constrainedMover = function(fun, within){
	// summary: returns a constrained version of dojo.dnd.Mover
	// description: this function produces n object, which will put a constraint on 
	//	the margin box of dragged object in absolute coordinates
	// fun: Function: called on drag, and returns a constraint box
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	var mover = function(node, e, notifier){
		dojo.dnd.Mover.call(this, node, e, notifier);
	};
	dojo.extend(mover, dojo.dnd.Mover.prototype);
	dojo.extend(mover, {
		onMouseMove: function(e){
			// summary: event processor for onmousemove
			// e: Event: mouse event
			dojo.dnd.autoScroll(e);
			var m = this.marginBox, c = this.constraintBox,
				l = m.l + e.pageX, t = m.t + e.pageY;
			l = l < c.l ? c.l : c.r < l ? c.r : l;
			t = t < c.t ? c.t : c.b < t ? c.b : t;
			this.host.onMove(this, {l: l, t: t});
		},
		onFirstMove: function(){
			// summary: called once to initialize things; it is meant to be called only once
			dojo.dnd.Mover.prototype.onFirstMove.call(this);
			var c = this.constraintBox = fun.call(this), m = this.marginBox;
			c.r = c.l + c.w - (within ? m.w : 0);
			c.b = c.t + c.h - (within ? m.h : 0);
		}
	});
	return mover;	// Object
};

dojo.dnd.move.boxConstrainedMover = function(box, within){
	// summary: a specialization of dojo.dnd.constrainedMover, which constrains to the specified box
	// box: Object: a constraint box (l, t, w, h)
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	return dojo.dnd.move.constrainedMover(function(){ return box; }, within);	// Object
};

dojo.dnd.move.parentConstrainedMover = function(area, within){
	// summary: a specialization of dojo.dnd.constrainedMover, which constrains to the parent node
	// area: String: "margin" to constrain within the parent's margin box, "border" for the border box,
	//	"padding" for the padding box, and "content" for the content box; "content" is the default value.
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	var fun = function(){
		var n = this.node.parentNode, 
			s = dojo.getComputedStyle(n), 
			mb = dojo._getMarginBox(n, s);
		if(area == "margin"){
			return mb;	// Object
		}
		var t = dojo._getMarginExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "border"){
			return mb;	// Object
		}
		t = dojo._getBorderExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "padding"){
			return mb;	// Object
		}
		t = dojo._getPadExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		return mb;	// Object
	};
	return dojo.dnd.move.constrainedMover(fun, within);	// Object
};

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");









dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary: the thing that grays out the screen behind the dialog

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class=dijitDialogUnderlayWrapper id='${id}_underlay'><div class=dijitDialogUnderlay dojoAttachPoint='node'></div></div>",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		layout: function(){
			// summary
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.

			var viewport = dijit.getViewport();
			var is = this.node.style,
				os = this.domNode.style;

			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";

			// process twice since the scroll bar may have been removed
			// by the previous resizing
			var viewport2 = dijit.getViewport();
			if(viewport.w != viewport2.w){ is.width = viewport2.w + "px"; }
			if(viewport.h != viewport2.h){ is.height = viewport2.h + "px"; }
		},

		show: function(){
			this.domNode.style.display = "block";
			this.layout();
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "block";
			}
			this._resizeHandler = this.connect(window, "onresize", "layout");
		},

		hide: function(){
			this.domNode.style.display = "none";
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "none";
			}
			this.disconnect(this._resizeHandler);
		},

		uninitialize: function(){
			if(this.bgIframe){
				this.bgIframe.destroy();
			}
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)

		templateString: null,
		templateString:"<div class=\"dijitDialog\">\r\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n\t</span>\r\n\t</div>\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\r\n\t<span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",

		// open: Boolean
		//		is True or False depending on state of dialog
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: 400,

		_lastFocusItem:null,

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{title: "titleBar"}),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.inherited("postCreate",arguments);
			this.domNode.style.display="none";
			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
		},

		onLoad: function(){
			// summary: 
			//		when href is specified we need to reposition the dialog after the data is loaded
			this._position();
			this.inherited("onLoad",arguments);
		},

		_setup: function(){
			// summary:
			//		stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons)

			this._modalconnects = [];

			if(this.titleBar){
				this._moveable = new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
			}

			this._underlay = new dijit.DialogUnderlay();

			var node = this.domNode;
			this._fadeIn = dojo.fx.combine(
				[dojo.fadeIn({
					node: node,
					duration: this.duration
				 }),
				 dojo.fadeIn({
					node: this._underlay.domNode,
					duration: this.duration,
					onBegin: dojo.hitch(this._underlay, "show")
				 })
				]
			);

			this._fadeOut = dojo.fx.combine(
				[dojo.fadeOut({
					node: node,
					duration: this.duration,
					onEnd: function(){
						node.style.display="none";
					}
				 }),
				 dojo.fadeOut({
					node: this._underlay.domNode,
					duration: this.duration,
					onEnd: dojo.hitch(this._underlay, "hide")
				 })
				]
			);
		},

		uninitialize: function(){
			if(this._underlay){
				this._underlay.destroy();
			}
		},

		_position: function(){
			// summary: position modal dialog in center of screen
			
			if(dojo.hasClass(dojo.body(),"dojoMove")){ return; }
			var viewport = dijit.getViewport();
			var mb = dojo.marginBox(this.domNode);

			var style = this.domNode.style;
			style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
			style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary:  called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to titleBar

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.titleBar.focus();
		},

		_onKey: function(/*Event*/ evt){
			if(evt.keyCode){
				var node = evt.target;
				// see if we are shift-tabbing from titleBar
				if(node == this.titleBar && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
					if(this._lastFocusItem){
						this._lastFocusItem.focus(); // send focus to last item in dialog if known
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode){
							if(evt.keyCode == dojo.keys.ESCAPE){
								this.hide(); 
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.keyCode != dojo.keys.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if (!dojo.isOpera){
						try{
							this.titleBar.focus();
						}catch(e){/*squelch*/}
					}
				}
			}
		},

		show: function(){
			// summary: display the dialog

			// first time we show the dialog, there's some initialization stuff to do			
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(document.documentElement, "onkeypress", this, "_onKey"));

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this._modalconnects.push(dojo.connect(this.containerNode, ev, this, "_findLastFocus"));

			dojo.style(this.domNode, "opacity", 0);
			this.domNode.style.display="block";
			this.open = true;
			this._loadCheck(); // lazy load trigger

			this._position();

			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);

			// set timeout to allow the browser to render dialog
			setTimeout(dojo.hitch(this, function(){
				dijit.focus(this.titleBar);
			}), 50);
		},

		hide: function(){
			// summary
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return		
			if(!this._alreadyInitialized){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}
			this._fadeOut.play();

			if (this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			this.connect(this._fadeOut,"onEnd",dojo.hitch(this,function(){
				dijit.focus(this._savedFocus);
			}));
			this.open = false;
		},

		layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
				this._underlay.layout();
				this._position();
			}
		}
	}
);

dojo.declare(
	"dijit.TooltipDialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a dialog that appears like a Tooltip
		// title: String
		// 		Description of tooltip dialog (required for a11Y)
		title: "",

		_lastFocusItem: null,

		templateString: null,
		templateString:"<div class=\"dijitTooltipDialog\" >\r\n\t<div class=\"dijitTooltipContainer\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"0\" waiRole=\"dialog\"></div>\r\n\t</div>\r\n\t<span dojoAttachPoint=\"tabEnd\" tabindex=\"0\" dojoAttachEvent=\"focus:_cycleFocus\"></span>\r\n\t<div class=\"dijitTooltipConnector\" ></div>\r\n</div>\r\n",

		postCreate: function(){
			this.inherited("postCreate",arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this.connect(this.containerNode, ev, "_findLastFocus");
			this.containerNode.title=this.title;
		},

		orient: function(/*Object*/ corner){
			// summary: configure widget to be displayed in given position relative to the button
			this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above");
		},

		onOpen: function(/*Object*/ pos){
			// summary: called when dialog is displayed
			this.orient(pos.corner);
			this._loadCheck(); // lazy load trigger
			this.containerNode.focus();
		},

		_onKey: function(/*Event*/ evt){
			// summary: keep keyboard focus in dialog; close dialog on escape key
			if(evt.keyCode == dojo.keys.ESCAPE){
				this.onCancel();
			}else if(evt.target == this.containerNode && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
				if (this._lastFocusItem){
					this._lastFocusItem.focus();
				}
				dojo.stopEvent(evt);
			}else if(evt.keyCode == dojo.keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary: called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to containerNode

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.containerNode.focus();
		}
	}	
);


}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");





dojo.declare(
	"dijit.Menu",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	constructor: function() {
		this._bindings = [];
	},

	templateString:
			'<table class="dijit dijitMenu dijitReset dijitMenuTable" waiRole="menu" dojoAttachEvent="onkeypress:_onKeyPress">' +
				'<tbody class="dijitReset" dojoAttachPoint="containerNode"></tbody>'+
			'</table>',

	// targetNodeIds: String[]
	//	Array of dom node ids of nodes to attach to.
	//	Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: Boolean
	//	if true, right clicking anywhere on the window will cause this context menu to open;
	//	if false, must specify targetNodeIds
	contextMenuForWindow: false,

	// parentMenu: Widget
	// pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//	number of milliseconds before hovering (without clicking) causes the popup to automatically open
	popupDelay: 500,

	// _contextMenuWithMouse: Boolean
	//	used to record mouse and keyboard events to determine if a context
	//	menu is being opened with the keyboard or the mouse
	_contextMenuWithMouse: false,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();
	},

	onExecute: function(){
		// summary: attach point for notification about when a menu item has been executed
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary: attach point for notification about when the user cancels the current menu
	},

	_moveToPopup: function(/*Event*/ evt){
		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary
		//	Handle keyboard based menu navigation.
		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.keyCode){
			case dojo.keys.RIGHT_ARROW:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case dojo.keys.LEFT_ARROW:
				if(this.parentMenu){
					this.onCancel(false);
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		this.focusChild(item);

		if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
			this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
		}
	},

	_onChildBlur: function(item){
		// Close all popups that are open and descendants of this menu
		dijit.popup.close(item.popup);
		item._blur();
		this._stopPopupTimer();
	},

	onItemUnhover: function(/*MenuItem*/ item){
	},

	_stopPopupTimer: function(){
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_getTopMenu: function(){
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*Widget*/ item){
		// summary: user defined function to handle clicks on an item
		// summary: internal function for clicks
		if(item.disabled){ return false; }

		if(item.popup){
			if(!this.is_open){
				this._openPopup();
			}
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick();
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el) {
		//	summary
		//	returns the window reference of the passed iframe
		var win = dijit.getDocumentWindow(dijit.Menu._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			dijit.Menu._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && document.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		//	summary
		//	returns a reference to the document object inside iframe_el
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
			dojo.connect(cn, "oncontextmenu", this, "_openMyself"),
			dojo.connect(cn, "onkeydown", this, "_contextKey"),
			dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},

	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
				window.setTimeout(dojo.hitch(this, function(){ this._openMyself(_e); }), 1);
			}
		}
	},

	_contextMouse: function(e){
		this._contextMenuWithMouse = true;
	},

	_openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	},

	onOpen: function(/*Event*/ e){
		// summary
		//		Open menu relative to the mouse
		this.isShowingNow = true;
	},

	onClose: function(){
		// summary: callback when this menu is closed
		this._stopPopupTimer();
		this.parentMenu = null;
		this.isShowingNow = false;
		this.currentPopup = null;
		if(this.focusedChild){
			this._onChildBlur(this.focusedChild);
			this.focusedChild = null;
		}
	},

	_openPopup: function(){
		// summary: open the popup to the side of the current menu item
		this._stopPopupTimer();
		var from_item = this.focusedChild;
		var popup = from_item.popup;

		if(popup.isShowingNow){ return; }
		popup.parentMenu = this;
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.arrowCell,
			orient: this.isLeftToRight() ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'},
			onCancel: function(){
				// called when the child menu is canceled
				dijit.popup.close(popup);
				from_item.focus();	// put focus back on my node
				self.currentPopup = null;
			}
		});

		this.currentPopup = popup;

		if(popup.focus){
			popup.focus();
		}
	}
}
);

dojo.declare(
	"dijit.MenuItem",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line item in a Menu2

	// Make 3 columns
	//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
	templateString:
		 '<tr class="dijitReset dijitMenuItem"'
		+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">'
		+'<td class="dijitReset"><div class="dijitMenuItemIcon ${iconClass}" dojoAttachPoint="iconNode" ></div></td>'
		+'<td tabIndex="-1" class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode" waiRole="menuitem"></td>'
		+'<td class="dijitReset" dojoAttachPoint="arrowCell">'
			+'<div class="dijitMenuExpand" dojoAttachPoint="expand" style="display:none">'
			+'<span class="dijitInline dijitArrowNode dijitMenuExpandInner">+</span>'
			+'</div>'
		+'</td>'
		+'</tr>',

	// label: String
	//	menu text
	label: '',

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	// disabled: Boolean
	//  if true, the menu item is disabled
	//  if false, the menu item is enabled
	disabled: false,

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
		this.setDisabled(this.disabled);
		if(this.label){
			this.containerNode.innerHTML=this.label;
		}
	},

	_onHover: function(){
		// summary: callback when mouse is moved onto menu item
		this.getParent().onItemHover(this);
	},

	_onUnhover: function(){
		// summary: callback when mouse is moved off of menu item
		// if we are unhovering the currently selected item
		// then unselect it
		this.getParent().onItemUnhover(this);
	},

	_onClick: function(evt){
		this.getParent().onItemClick(this);
		dojo.stopEvent(evt);
	},

	onClick: function() {
		// summary
		//	User defined function to handle clicks
	},

	focus: function(){
		dojo.addClass(this.domNode, 'dijitMenuItemHover');
		try{
			dijit.focus(this.containerNode);
		}catch(e){
			// this throws on IE (at least) in some scenarios
		}
	},

	_blur: function(){
		dojo.removeClass(this.domNode, 'dijitMenuItemHover');
	},

	setDisabled: function(/*Boolean*/ value){
		// summary: enable or disable this menu item
		this.disabled = value;
		dojo[value ? "addClass" : "removeClass"](this.domNode, 'dijitMenuItemDisabled');
		dijit.setWaiState(this.containerNode, 'disabled', value ? 'true' : 'false');
	}
});

dojo.declare(
	"dijit.PopupMenuItem",
	dijit.MenuItem,
{
	_fillContent: function(){
		// my inner HTML contains both the menu item text and a popup widget, like
		// <div dojoType="dijit.PopupMenuItem">
		//		<span>pick me</span>
		//		<popup> ... </popup>
		// </div>
		// the first part holds the menu item text and the second part is the popup
		if(this.srcNodeRef){
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
		// land now.  move it to document.body.
		if(!this.popup){
			var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.popup = dijit.byNode(node);
		}
		dojo.body().appendChild(this.popup.domNode);

		this.popup.domNode.style.display="none";
		dojo.addClass(this.expand, "dijitMenuExpandEnabled");
		dojo.style(this.expand, "display", "");
		dijit.setWaiState(this.containerNode, "haspopup", "true");
	}
});

dojo.declare(
	"dijit.MenuSeparator",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line between two menu items

	templateString: '<tr class="dijitMenuSeparator"><td colspan=3>'
			+'<div class="dijitMenuSeparatorTop"></div>'
			+'<div class="dijitMenuSeparatorBottom"></div>'
			+'</td></tr>',

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
	},
	
	isFocusable: function(){
		// summary:
		//		over ride to always return false
		return false;
	}
});

}

if(!dojo._hasResource["dijit.form.Textarea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Textarea"] = true;
dojo.provide("dijit.form.Textarea");





dojo.declare(
	"dijit.form.Textarea",
	dijit.form._FormWidget,
{
	// summary
	//	A textarea that resizes vertically to contain the data.
	//	Takes nearly all the parameters (name, value, etc.) that a vanilla textarea takes.
	//	Cols is not supported and the width should be specified with style width.
	//	Rows is not supported since this widget adjusts the height.
	// usage:
	//	<textarea dojoType="dijit.form.TextArea">...</textarea>

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{style:"styleNode", 'class':"styleNode"}),

	templateString: (dojo.isIE || dojo.isSafari || dojo.isMozilla) ?
				((dojo.isIE || dojo.isSafari) ? '<fieldset id="${id}" class="dijitInline dijitInputField dijitTextArea" dojoAttachPoint="styleNode" waiRole="presentation"><div dojoAttachPoint="editNode,focusNode,eventNode" dojoAttachEvent="onpaste:_changing,oncut:_changing" waiRole="textarea" style="text-decoration:none;_padding-bottom:16px;display:block;overflow:auto;" contentEditable="true"></div>'
					: '<span id="${id}" class="dijitReset">'+
					'<iframe src="javascript:<html><head><title>${_iframeEditTitle}</title></head><body><script>var _postCreate=window.frameElement?window.frameElement.postCreate:null;if(_postCreate)_postCreate();</script></body></html>"'+
							' dojoAttachPoint="iframe,styleNode" dojoAttachEvent="onblur:_onIframeBlur" class="dijitInline dijitInputField dijitTextArea"></iframe>')
				+ '<textarea name="${name}" value="${value}" dojoAttachPoint="formValueNode" style="display:none;"></textarea>'
				+ ((dojo.isIE || dojo.isSafari) ? '</fieldset>':'</span>')
			: '<textarea id="${id}" name="${name}" value="${value}" dojoAttachPoint="formValueNode,editNode,focusNode,styleNode" class="dijitInputField dijitTextArea"></textarea>',

	focus: function(){
		// summary: Received focus, needed for the InlineEditBox widget
		if(!this.disabled){
			this._changing(); // set initial height
		}
		if(dojo.isMozilla){
			dijit.focus(this.iframe);
		}else{
			dijit.focus(this.focusNode);
		}
	},

	setValue: function(/*String*/ value, /*Boolean, optional*/ priorityChange){
		var editNode = this.editNode;
		if(typeof value == "string"){
			editNode.innerHTML = ""; // wipe out old nodes
			if(value.split){
				var _this=this;
				var isFirst = true;
				dojo.forEach(value.split("\n"), function(line){
					if(isFirst){ isFirst = false; }
					else {
						editNode.appendChild(document.createElement("BR")); // preserve line breaks
					}
					editNode.appendChild(document.createTextNode(line)); // use text nodes so that imbedded tags can be edited
				});
			}else{
				editNode.appendChild(document.createTextNode(value));
			}
		}else{
			// blah<BR>blah --> blah\nblah
			// <P>blah</P><P>blah</P> --> blah\nblah
			// <DIV>blah</DIV><DIV>blah</DIV> --> blah\nblah
			// &amp;&lt;&gt; -->&< >
			value = editNode.innerHTML;
			if(this.iframe){ // strip sizeNode
				value = value.replace(/<div><\/div>\r?\n?$/i,"");
			}
			value = value.replace(/\s*\r?\n|^\s+|\s+$|&nbsp;/g,"").replace(/>\s+</g,"><").replace(/<\/(p|div)>$|^<(p|div)[^>]*>/gi,"").replace(/([^>])<div>/g,"$1\n").replace(/<\/p>\s*<p[^>]*>|<br[^>]*>/gi,"\n").replace(/<[^>]*>/g,"").replace(/&amp;/gi,"\&").replace(/&lt;/gi,"<").replace(/&gt;/gi,">");
		}
		this.value = this.formValueNode.value = value;
		if(this.iframe){
			var sizeNode = document.createElement('div');
			editNode.appendChild(sizeNode);
			var newHeight = sizeNode.offsetTop;
			if(editNode.scrollWidth > editNode.clientWidth){ newHeight+=16; } // scrollbar space needed?
			if(this.lastHeight != newHeight){ // cache size so that we don't get a resize event because of a resize event
				if(newHeight == 0){ newHeight = 16; } // height = 0 causes the browser to not set scrollHeight
				dojo.contentBox(this.iframe, {h: newHeight});
				this.lastHeight = newHeight;
			}
			editNode.removeChild(sizeNode);
		}
		dijit.form.Textarea.superclass.setValue.call(this, this.getValue(), priorityChange);
	},

	getValue: function(){
		return this.formValueNode.value.replace(/\r/g,"");
	},

	postMixInProperties: function(){
		dijit.form.Textarea.superclass.postMixInProperties.apply(this,arguments);
		// don't let the source text be converted to a DOM structure since we just want raw text
		if(this.srcNodeRef && this.srcNodeRef.innerHTML != ""){
			this.value = this.srcNodeRef.innerHTML;
			this.srcNodeRef.innerHTML = "";
		}
		if((!this.value || this.value == "") && this.srcNodeRef && this.srcNodeRef.value){
			this.value = this.srcNodeRef.value;
		}
		if(!this.value){ this.value = ""; }
		this.value = this.value.replace(/\r\n/g,"\n").replace(/&gt;/g,">").replace(/&lt;/g,"<").replace(/&amp;/g,"&");
		if(dojo.isMozilla){
			// In the case of Firefox an iframe is used and when the text gets focus,
			// focus is fired from the document object.  There isn't a way to put a
			// waiRole on the document object and as a result screen readers don't
			// announce the role.  As a result screen reader users are lost.
			//
			// An additional problem is that the browser gives the document object a
			// very cryptic accessible name, e.g.
			// wysiwyg://13/http://archive.dojotoolkit.org/nightly/dojotoolkit/dijit/tests/form/test_InlineEditBox.html
			// When focus is fired from the document object, the screen reader speaks
			// the accessible name.  The cyptic accessile name is confusing.
			//
			// A workaround for both of these problems is to give the iframe's
			// document a title, the name of which is similar to a role name, i.e.
			// "edit area".  This will be used as the accessible name which will replace
			// the cryptic name and will also convey the role information to the user.
			// Because it is read directly to the user, the string must be localized.
			// In addition, since a <label> element can not be associated with an iframe, if 
			// this control has a label, insert the text into the title as well.
			var _nlsResources = dojo.i18n.getLocalization("dijit", "Textarea");
			this._iframeEditTitle = _nlsResources.iframeEditTitle;
			this._iframeFocusTitle = _nlsResources.iframeFocusTitle;
			var label=dojo.query('label[for="'+this.id+'"]');
			if(label.length){
				this._iframeEditTitle = label[0].innerHTML + " " + this._iframeEditTitle;
			}
			var body = this.focusNode = this.editNode = document.createElement('BODY');
			body.style.margin="0px";
			body.style.padding="0px";
			body.style.border="0px";
		}
	},

	postCreate: function(){
		if(dojo.isIE || dojo.isSafari){
			this.domNode.style.overflowY = 'hidden';
		}else if(dojo.isMozilla){
			var w = this.iframe.contentWindow;
			try { // #4715: peeking at the title can throw a security exception during iframe setup
				var title = this.iframe.contentDocument.title;
			} catch(e) { var title = ''; }
			if(!w || !title){
				this.iframe.postCreate = dojo.hitch(this, this.postCreate);
				return;
			}
			var d = w.document;
			d.getElementsByTagName('HTML')[0].replaceChild(this.editNode, d.getElementsByTagName('BODY')[0]);
			if(!this.isLeftToRight()){
				d.getElementsByTagName('HTML')[0].dir = "rtl";
			}			
			this.iframe.style.overflowY = 'hidden';
			this.eventNode = d;
			// this.connect won't destroy this handler cleanly since its on the iframe's window object
			// resize is a method of window, not document
			w.addEventListener("resize", dojo.hitch(this, this._changed), false); // resize is only on the window object
		}else{
			this.focusNode = this.domNode;
		}
		if(this.eventNode){
			this.connect(this.eventNode, "keypress", this._onKeyPress);
			this.connect(this.eventNode, "mousemove", this._changed);
			this.connect(this.eventNode, "focus", this._focused);
			this.connect(this.eventNode, "blur", this._blurred);
		}
		if(this.editNode){
			this.connect(this.editNode, "change", this._changed); // needed for mouse paste events per #3479
		}
		this.inherited('postCreate', arguments);
	},

	// event handlers, you can over-ride these in your own subclasses
	_focused: function(e){
		dojo.addClass(this.iframe||this.domNode, "dijitInputFieldFocused");
		this._changed(e);
	},

	_blurred: function(e){
		dojo.removeClass(this.iframe||this.domNode, "dijitInputFieldFocused");
		this._changed(e, true);
	},

	_onIframeBlur: function(){
		// Reset the title back to "edit area".
		this.iframe.contentDocument.title = this._iframeEditTitle;
	},

	_onKeyPress: function(e){
		if(e.keyCode == dojo.keys.TAB && !e.shiftKey && !e.ctrlKey && !e.altKey && this.iframe){
			// Pressing the tab key in the iframe (with designMode on) will cause the
			// entry of a tab character so we have to trap that here.  Since we don't
			// know the next focusable object we put focus on the iframe and then the
			// user has to press tab again (which then does the expected thing).
			// A problem with that is that the screen reader user hears "edit area"
			// announced twice which causes confusion.  By setting the
			// contentDocument's title to "edit area frame" the confusion should be
			// eliminated.
			this.iframe.contentDocument.title = this._iframeFocusTitle;
			// Place focus on the iframe. A subsequent tab or shift tab will put focus
			// on the correct control.
			// Note: Can't use this.focus() because that results in a call to
			// dijit.focus and if that receives an iframe target it will set focus
			// on the iframe's contentWindow.
			this.iframe.focus();  // this.focus(); won't work
			dojo.stopEvent(e);
		}else if(e.keyCode == dojo.keys.ENTER){
			e.stopPropagation();
		}else if(this.inherited("_onKeyPress", arguments) && this.iframe){
			// #3752:
			// The key press will not make it past the iframe.
			// If a widget is listening outside of the iframe, (like InlineEditBox)
			// it will not hear anything.
			// Create an equivalent event so everyone else knows what is going on.
			var te = document.createEvent("KeyEvents");
			te.initKeyEvent("keypress", true, true, null, e.ctrlKey, e.altKey, e.shiftKey, e.metaKey, e.keyCode, e.charCode);
			this.iframe.dispatchEvent(te);
		}
		this._changing();
	},

	_changing: function(e){
		// summary: event handler for when a change is imminent
		setTimeout(dojo.hitch(this, "_changed", e, false), 1);
	},

	_changed: function(e, priorityChange){
		// summary: event handler for when a change has already happened
		if(this.iframe && this.iframe.contentDocument.designMode != "on"){
			this.iframe.contentDocument.designMode="on"; // in case this failed on init due to being hidden
		}
		this.setValue(null, priorityChange);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.Version"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.Version"] = true;

dojo.provide("ibm.tivoli.tip.Version");

ibm.tivoli.tip.Version.getVersion = function() {
	  return( "1.1.0.1-2008/11/14 21:07" );
};
		

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartProperties"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartProperties"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: TIPChartPortlet/WebContent/js/ibm/tivoli/tip/charts/dijit/ChartProperties.js, tipcharts, tip2
 * Version:       1.18
 * Last Modified: 5/8/08
 *******************************************************************************/
dojo.provide("ibm.tivoli.tip.charts.dijit.ChartProperties");










  // also contains RadioButton






// NLS



dojo.declare("ibm.tivoli.tip.charts.dijit.ChartProperties",
            [ dijit._Widget, dijit._Templated ],
{
	templateString:"<div>\r\n<div dojoType=\"dijit.Dialog\"\r\n\tdojoAttachPoint=\"dialog\"\r\n\ttitle=\"${resources_.PREFERENCES}\"\r\n\tstyle=\"display:none;\"\r\n>\r\n<form name=\"${namespace}propertiesForm\" id=\"${namespace}propertiesForm\" action=\"${formURL}\" \r\n\t  dojoType=\"dijit.form.Form\" enctype=\"multipart/form-data\" \r\n\t  style=\"padding: 0px; margin:0px\"\r\n>\r\n\r\n\r\n\t<div id=\"${namespace}tabs\" dojoType=\"dijit.layout.TabContainer\" class=\"chartsDialog\" style=\"width: 550px; height: 350px;\" >\r\n\t\t<div id=\"${namespace}tab1\" dojoType=\"dijit.layout.ContentPane\" selected=\"true\" title=\"${resources_.GENERAL}\" \r\n\t\t\t class=\"chartsDialog\" style=\"padding: 15px\">\r\n\r\n\t\t    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n\t\t\t<tr valign=\"top\">\r\n\t\t\t       <td>\r\n\t\t\t\t \t<h2>\r\n\t\t\t\t     \t${resources_.CHART_NAME}\r\n\t\t\t\t   \t</h2>\r\n\t\t\t\t\t<label class=\"label\" for=\"${namespace}chartName\" title=\"${resources_.CHART_NAME}\">\r\n\t\t\t\t\t</label>\r\n\t\t\t\t\t<input id=\"${namespace}chartName\" type=\"text\" name=\"${namespace}chartName\" dojoType=\"dijit.form.ValidationTextBox\"\r\n\t\t\t\t\t\trequired=\"true\" />\r\n\t\t\t\t\t<br/>\r\n\r\n\t\t\t\t \t<h2>\r\n\t\t\t\t     \t${resources_.VIEW_AS}\r\n\t\t\t\t   \t</h2>\r\n\r\n\t\t\t\t\t  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType = \"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${namespace}tableOrChartGroup\"\r\n\t\t\t\t\t  id=\"${namespace}tableSelected\"\r\n\t\t\t\t\t  value=\"table\" />\r\n\t\t\t\t\t  <label class=\"label\" for=\"${namespace}tableSelected\">${resources_.TABLE}</label>\r\n\r\n\t\t\t\t\t  <br/>\r\n\t\t\t\t\t  \r\n\t\t\t\t\t  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType = \"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${namespace}tableOrChartGroup\"\r\n\t\t\t\t\t  id=\"${namespace}chartSelected\"\r\n  \t\t\t\t\t  value=\"chart\" />\r\n\t\t\t\t\t  <label class=\"label\" for=\"${namespace}chartSelected\">${resources_.CHART}</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t\r\n\t\t\t\t<td>\r\n\t\t\t\t  <h2>${resources_.DESCRIPTION}</h2>\r\n\t\t\t\t  <label class=\"h2\" for=\"${namespace}description\" title=\"${resources_.DESCRIPTION}\"></label>\r\n\t\t\t          <textarea id=\"${namespace}description\" \r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.Textarea\"\r\n\t\t\t\t\t   \t\tname=\"${namespace}description\" \r\n\t\t\t\t\t   \t\tstyle=\"width:200px; height:150px;\"\r\n\t\t\t\t\t   \t\trows=\"5\"\r\n\t\t\t\t\t\t    cols=\"30\" >\r\n\t\t\t\t  </textarea>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr valign=\"top\">\r\n\t\t\t\t<td>\r\n\t\t\t\t     \t<h2>\r\n\t\t\t\t     \t${resources_.OPTIONS}\r\n\t\t\t\t     \t</h2>\r\n\r\n\t\t\t\t\t\t<input type=\"checkbox\"\r\n\t\t\t\t\t\t\tname=\"${namespace}checkForUpdatesCB\"\r\n\t\t\t\t\t\t\tid=\"${namespace}checkForUpdatesCB\"\r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" />\r\n\t\t\t\t\t\t<label class=\"label\"\r\n\t\t\t              for=\"${namespace}checkForUpdatesCB\">${resources_.AUTO_UPDATE}</label>\r\n\t\t\t\t\t\t<br/>\r\n\r\n\t\t\t\t\t\t<input type=\"checkbox\"\r\n\t\t\t\t\t\tname=\"${namespace}showLegendCB\"\r\n\t\t\t\t\t\tid=\"${namespace}showLegendCB\"\r\n\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" />\r\n\t\t\t\t\t\t<label class=\"label\" for=\"${namespace}showLegendCB\">${resources_.SHOW_LEGEND}</label>\r\n\t\t\t\t\t\t<br/>\r\n\r\n\t\t\t\t\t\t<input type=\"checkbox\"\r\n\t\t\t\t\t\t\tname=\"${namespace}showNameInChart\"\r\n\t\t\t\t\t\t\tid=\"${namespace}showNameInChart\"\r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" />\r\n\t\t\t\t\t\t<label class=\"label\"\r\n\t\t\t              for=\"${namespace}showNameInChart\">${resources_.SHOW_NAME_IN_CHART}</label>\r\n\t\t\t\t\t\t<br/>\r\n\r\n\t\t\t\t\t\t<input type=\"checkbox\"\r\n\t\t\t\t\t\t\tname=\"${namespace}showLastUpdated\"\r\n\t\t\t\t\t\t\tid=\"${namespace}showLastUpdated\"\r\n\t\t\t\t\t\t\tdojoType=\"dijit.form.CheckBox\" />\r\n\t\t\t\t\t\t<label class=\"label\"\r\n\t\t\t              for=\"${namespace}showLastUpdated\">${resources_.SHOW_LAST_UPDATED}</label>\r\n\t\t\t\t\t\t<br/>\r\n\t\t\t\t\t\t\r\n\t\t\t\t</td>\r\n\t\t\t\t\r\n\t\t\t\t<td>\r\n\t\t\t\t  <table border=\"0\" cellpadding=\"2\" cellspacing=\"0\" style=\"display:none\">\r\n\t\t\t\t     <tr align=\"left\" valign=\"top\">\r\n\t\t\t\t\t<td>\r\n\t\t\t\t     \t  <h2>\r\n\t\t\t\t     \t  ${resources_.DIMENSIONS}\r\n\t\t\t\t     \t  </h2>\r\n\t\t\t\t     \t</td>\r\n\t\t\t\t     </tr>\r\n\t\t\t\t\t\r\n\t\t\t\t\t<tr align=\"left\" valign=\"top\">\r\n\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t<label class=\"label\" for=\"${namespace}height\" title=\"${resources_.HEIGHT}\">${resources_.HEIGHT}</label>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t<input type=\"text\"\r\n\t\t\t\t\t\t\t\t\tid=\"${namespace}height\"\r\n\t\t\t\t\t\t\t\t\tname=\"${namespace}height\"\r\n\t\t\t\t\t\t\t\t\tstyle=\"width: 8em;\"\r\n\t\t\t\t\t\t\t\t\tdojoType=\"dijit.form.TextBox\" />\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\r\n\t\t\t\t\t<tr align=\"left\" valign=\"top\">\r\n\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t<label class=\"label\" for=\"${namespace}width\" title=\"${resources_.WIDTH}\">${resources_.WIDTH}</label>\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t\t<td>\r\n\t\t\t\t\t\t\t<input type=\"text\"\r\n\t\t\t\t\t\t\t\t\tid=\"${namespace}width\"\r\n\t\t\t\t\t\t\t\t\tname=\"${namespace}width\"\r\n\t\t\t\t\t\t\t\t\tstyle=\"width: 8em;\"\r\n\t\t\t\t\t\t\t\t\tdojoType=\"dijit.form.TextBox\" />\r\n\t\t\t\t\t\t</td>\r\n\t\t\t\t\t</tr>\r\n\r\n\t\t\t\t</table>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t</table>\r\n\t\t</div>\r\n\r\n\r\n\t\t<div id=\"${namespace}tab3\" dojoType=\"dijit.layout.ContentPane\" title=\"${resources_.PARAMETERS}\"\r\n\t\t\t\tclass=\"chartsDialog\"  style=\"padding: 15px\">\r\n\t\t    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n\t\t    \t   <tr valign=\"top\" align=\"left\">\r\n\t\t    \t   \t<td>\r\n\t\t        \t    <h2>\r\n\t\t\t\t     \t${resources_.OPTIONS}\r\n\t\t\t\t     \t</h2>\r\n\t\t\t\t    </td>\r\n\t\t\t\t   </tr>\r\n\t\t    \r\n\t\t   \t\t   <tr align=\"left\" valign=\"top\">\r\n\t\t\t\t        <td>\r\n\t\t\t\t\t  <div class=\"label\">\r\n\t\t\t\t\t  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${namespace}whichParametersGroup\"\r\n\t\t\t\t\t  id=\"${namespace}contextualParametersSelected\"\r\n\t\t\t\t\t  value=\"true\"\r\n\t\t\t\t\t  />\r\n\t\t\t\t\t  <label for=\"${namespace}contextualParametersSelected\" title=\"${resources_.USE_CONTEXT}\">${resources_.USE_CONTEXT}</label>\r\n\t\t\t\t\t  </div>\r\n\t\t\t\t\t</td>\r\n\t\t\t\t    </tr>\r\n\r\n\t\t\t\t    <tr align=\"left\" valign=\"top\">\r\n\t\t\t\t        <td>\r\n\t\t\t\t\t<div class=\"label\">\t\t\t\t        \r\n\t\t\t\t\t  <input type=\"radio\"\r\n\t\t\t\t\t  dojoType=\"dijit.form.RadioButton\"\r\n\t\t\t\t\t  name=\"${namespace}whichParametersGroup\"\r\n\t\t\t\t\t  id=\"${namespace}defaultParametersSelected\"\r\n\t\t\t\t\t  value=\"false\"\r\n\t\t\t\t\t  checked />\r\n\t\t\t\t\t  <label for=\"${namespace}defaultParametersSelected\" title=\"${resources_.USE_DEFAULTS}\">${resources_.USE_DEFAULTS}</label>\r\n\t\t\t\t\t  </div>\r\n\t\t\t\t\t</td>\r\n\t\t\t\t    </tr>\r\n\t\t    </table>\r\n\r\n\t\t\t<br/>\r\n\r\n\t\t\t<div id=\"${namespace}parametersDivContainer\">\r\n\t\t\t</div>\r\n   \t\t</div>\r\n   \t\t\r\n   \t\t<div id=\"${namespace}tab4\" dojoType=\"dijit.layout.ContentPane\" title=\"${resources_.ABOUT}\"\r\n\t\t\t\tclass=\"chartsDialog\"  style=\"padding: 15px\">\r\n\t\t    <table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n\t\t\t\t<tr>\r\n\t\t\t\t\t<td>\r\n\t\t\t\t\t\t${chartingVersion}\r\n\t\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\t\t\t\t<tr>\r\n\t\t\t\t\t<td>\r\n\t\t\t\t\t\t${tipDojoVersion}\r\n\t\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\t\t\t\t<tr>\r\n\t\t\t\t\t<td>\r\n\t\t\t\t\t\t${dojoVersion}\r\n\t\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\t\t\t</table>\r\n\t\t</div>\r\n   \t\t\r\n\r\n\t  </div>\r\n\r\n\t<div dojoType=\"dijit.Toolbar\" style=\"padding: 0px\">\r\n\t\t<button id=\"${namespace}prefs_OK\" \r\n\t\t\t\t       value=\"${resources_.OK}\"\r\n\t\t\t\t       dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\t       dojoAttachEvent=\"onClick:onOK\" \r\n\t\t\t>${resources_.OK}</button>\r\n\t\t<button id=\"${namespace}prefs_Cancel\" \r\n\t\t\t\t\t   dojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\t\t   dojoAttachPoint=\"cancelButton\"\r\n\t\t\t\t\t   dojoAttachEvent=\"onClick:onCancel\"\r\n\t\t\t\t       value=\"${resources_.CANCEL}\"\r\n\t\t >${resources_.CANCEL}</button>\r\n\t</div>\t  \r\n\t\r\n</form>\r\n</div>\r\n</div>\r\n",
	widgetsInTemplate: true,
	namespace: null,
	contextPath: null,
	formURL: null,
	
	constructor: function() {
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	    this.connections = [];
	    this.chartingVersion = dojo.string.substitute(this.resources_.CHARTING_VERSION, { 0: ibm.tivoli.tip.charts.Version.getVersion() });
	    console.log ("chartingVersion: " + this.chartingVersion);
	    this.tipDojoVersion = dojo.string.substitute(this.resources_.TIPDOJO_VERSION, { 0: ibm.tivoli.tip.Version.getVersion() });
	    console.log ("tipDojoVersion: " + this.tipDojoVersion);
	    this.dojoVersion = dojo.string.substitute(this.resources_.DOJO_VERSION, { 0: dojo.version });
	    console.log ("dojoVersion: " + this.dojoVersion);
	},
	
	postCreate: function () {
		this.connections.push (dojo.connect (this.cancelButton, "onclick", this, "hide"));
		console.log ("connected cancel button with hide");
	},
	
//	postCreate: function () {
//		this.inherited (arguments);
//		var tabs = new dijit.layout.TabContainer();
//		var div = dojo.doc.createElement( 'DIV' );
//		var tab1 = new dijit.layout.ContentPane(
//			{	selected: true,
//				title: "GENERAL"
//			}, div);
//		tabs.addChild (tab1);
//		this.dialog.domNode.appendChild (tabs.domNode);
//	},
	
	show: function () {
		console.info ("enter ChartProperties");
		console.log ("this.dialog is: ");
		console.log (this.dialog);
		this.dialog.show();
		dijit.byId (this.namespace + "tabs").resize();
	},
	
	hide: function () {
		console.log("enter ChartProperties.hide");
		this.dialog.hide();
	},
	onOK: function () {
		dijit.byId (this.namespace + "chartWidget").dlgOKPressed();
	},
	onCancel: function () {
		this.hide();
		var chartWidget = dijit.byId (this.namespace + "chartWidget");
		
		var nameField = dojo.byId(this.namespace + "chartName");
		var heightField = dojo.byId(this.namespace + "height");
		var widthField = dojo.byId (this.namespace + "width");
		var showLegendField = dijit.byId (this.namespace + "showLegendCB");
		var descriptionField = dijit.byId(this.namespace + "description");
		var chartRadio = dijit.byId (this.namespace + "chartSelected");
		var tableRadio = dijit.byId (this.namespace + "tableSelected");
		var checkForUpdatesCB = dijit.byId (this.namespace + "checkForUpdatesCB");
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		var useDefaultContextRadio = dijit.byId(this.namespace + "defaultParametersSelected");
		var showLastUpdated = dijit.byId (this.namespace + "showLastUpdated");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");

		nameField.value = chartWidget.oldName_;
		descriptionField.setValue (chartWidget.oldDescription_);
		heightField.value = chartWidget.oldHeight_;
		widthField.value = chartWidget.oldWidth_;
		showLegendField.setChecked (chartWidget.oldShowLegend_);
		chartRadio.setChecked (chartWidget.oldChartRadio_);
		tableRadio.setChecked (!chartWidget.oldChartRadio_);
		checkForUpdatesCB.setChecked (chartWidget.oldCheckForUpdates_);
		useDefaultContextRadio.setChecked (!chartWidget.oldReceiveContext_);
		receiveContextRadio.setChecked (chartWidget.oldReceiveContext_);
		showLastUpdated.setChecked (chartWidget.oldShowLastUpdated_);
		showNameInChart.setChecked (chartWidget.oldShowNameInChart_);

		var paramDiv = dijit.byId (this.namespace + "parametersDiv");
		paramDiv.resetParameters();
	},
	destroy: function () {
		console.log ("enter ChartProperties.destroy");
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});
		
		this.dialog.destroy();

		this.inherited (arguments);
		console.log ("exit ChartProperties.destroy");
	}
	
});

	

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ParametersWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ParametersWidget"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ParametersWidget");





// NLS



// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.ParametersWidget", 
    [ dijit._Widget, dijit._Templated, dijit._Container ],
{
    namespace: "namespace1_",
    widgetsInTemplate: true,
    data: [],
    context: [],
    hasContext_: false,
	templateString:"<div id=\"${id}parametersWidgetDiv\">\r\n</div>\r\n",

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.resourceMessages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	},

	postCreate: function() { 
		this.inherited ("postCreate", arguments);
		console.info ("enter ParametersWidget:postCreate");
//		console.log ("looking for div: " + this.id);
//		var mydiv = dijit.byId (this.id);
//		console.info (mydiv);
		console.log ("ParametersWidget.domNode: ");
		console.log (this.domNode);
//		this.domNode.innerHTML = this.getContent(this.data);
		this.addParameters (this.data);
	},
	
	addParameters: function (model) {
		console.info ("enter ParametersWidget.addParameters");
		var param = null;
		var widget = null;

		for (var row in model) {
			if (row) {
				param = model[row];
				widget = new ibm.tivoli.tip.charts.dijit.Parameter ({
					parmid: this.namespace + "param_" + param.name,
					baseid: param.name,
					value: param.value,
					label: param.name
				});
				
				this.addChild (widget);
			}
		}
	},
	
	getContent: function(model) {
		console.info ("enter ParametersWidget.getContent");
		var innerHTML = "";

		innerHTML += "<table>";
		
		innerHTML += "<tr>";
		innerHTML += "<td><h2>";
		innerHTML += this.resourceMessages_.DEFAULT_PARAMETERS;
		innerHTML += "</h2></td>";
		innerHTML += "</tr>";
		
		var param = null;

		for (var row in model) {
			if (row) {
				param = model[row];

				innerHTML += "<tr>";
				innerHTML += "<td><div class=\"label\">";
				innerHTML += param.name;
			
				innerHTML += "</div></td>";
				innerHTML += "<td><div><input id=\"";
				innerHTML += this.namespace;
				innerHTML += "param_";
				innerHTML += param.name;
				innerHTML += "\" type=\"text\" name=\"";
				innerHTML += this.namespace;
				innerHTML += "param_";
				innerHTML += param.name;
				innerHTML += "\" value=\"";
				if (param.value) {
					innerHTML += param.value;
				}
				innerHTML += "\" dojoType=\"TextBox\">";
				innerHTML += "</input></div>";
				innerHTML += "</td>";
				innerHTML += "</tr>";
			}
		}
		
		innerHTML += "</table>";

		console.log ("return innerHTML:");
		console.log (innerHTML);
		return innerHTML;
	},
	
	isContextOn: function () {
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		console.log ("isContextOn: " + receiveContextRadio.checked);
		return receiveContextRadio.checked;
	},
	
	hasContext: function() {
		return this.hasContext_;
	},
	
	getParameterValue: function (parameterName) {
		console.info ("enter ParametersWidget.getParameterValue");
	
		var value = null;
		
		if (this.isContextOn() && this.hasContext()) {
			console.log ("context is on and we have context");
			if (this.context[parameterName]) {
				console.log ("from context w/ parameter: " + parameterName + " and value: " + this.context[parameterName]);
				value = this.context[parameterName];
			} else {
				console.log ("context value is null for name: " + parameterName);
				value = null;
			}
		} else {
			var textbox = dijit.byId (this.namespace + "param_" + parameterName);
			console.log ("getting param value from default value: ");
			console.log (textbox);
			value = textbox.getValue();
		}
		
		value = this.urlizeValue (value);
		
		return (value);
	},
	
	urlizeValue: function (value) {
		if (value !== null) {
			value = value.replace ("&", ":::");
		}
		
		return (value);
	},

	setParameterValue: function (parameterName, parameterValue) {
		console.info ("enter ParametersWidget.setParameterValue");
		var textbox = dijit.byId (this.namespace + "param_" + parameterName);
		console.log ("ParametersWidget.setParameterValue, textbox: ");
		console.log (textbox);
		textbox.setValue (parameterValue);
	},

	setAllContext: function (ctxt) {
		var params = this.getParameters();
		var param = null;
		var value = null;
		var x = null;
		
		for (x in params) {
			if (x) {
				param = params[x];
				value = eval ("ctxt." + param.name);
				if (value) {
					this.setContext (param.name, value);
				}
			}
		}
		
		this.hasContext_ = true;
	},
	
	setContext: function (parameterName, parameterValue) {
		this.context[parameterName] = parameterValue;
		this.hasContext_ = true;
	},
	
	getParameters: function () {
		return this.data;
	},
	
	updateInternalParametersData: function () {
		dojo.forEach (this.data, function (datum) {
			var param = dijit.byId (this.namespace + "param_" + datum.name);
			datum.value = param.getValue();
		}, this);
	},
	
	removeParameters: function () {
		console.log ("enter ParametersWidget.removeParameters");
		dojo.forEach (this.getChildren(), function (child) {
			this.removeChild (child);
			child.destroy();
		}, this);
		console.log ("exit ParametersWidget.removeParameters");
	},
	
	resetParameters: function () {
		console.log ("enter ParametersWidget.resetParameters");
		var param = null;
		var widget = null;
		for (var row in this.data) {
			if (row) {
				console.log ("resetting for row: " + row);
				param = this.data[row];
				widget = dijit.byId (this.namespace + "param_" + param.name);
				if (param.value === undefined) {
					param.value = "";
				}
				widget.setValue (param.value);
			}
		}

		console.log ("exit ParametersWidget.resetParameters");
	},
	
	destroy: function () {
		console.log ("Enter ParametersWidget.destroy");
//		this.removeParameters();
		dojo.forEach (this.data, function (datum) {
			var param = dijit.byId (this.namespace + "param_" + datum.name);
			if (param) {
				param.destroy();
			}
		}, this);
		
		this.data = null;
		this.destroyDescendants();
		this.inherited (arguments);
		console.log ("Exit ParametersWidget.destroy");
	}

});
	

// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.Parameter", 
    [ dijit._Widget, dijit._Templated ],
{
	parmid: null,
	baseid: null,
	label: null,
	description: null,
	value: null,
	widgetsInTemplate: true,
	templateString: '<div>' +
		'<label for="${id}">${label}</label> ' +
		'<input type="text" dojoType="dijit.form.TextBox" dojoAttachPoint="textbox" name="${parmid}" id="${parmid}" value="" />' +
		'</div><br/>',
	
	constructor: function () {
	},
	postCreate: function () {
		if (this.value) {
			this.textbox.setValue (this.value);
		}
	},
	setValue: function (value) {
		this.textbox.setValue (value);
	},
	getValue: function () {
		return this.textbox.getValue();
	},
	destroy: function () {
		console.log ("enter Parameter.destroy for");
		console.log (this);
//		if (dijit.byId(this.parmid)) {
//			console.log ("about to destroy: ");
//			console.log (dijit.byId (this.parmid));
//			dijit.byId (this.parmid).destroy();
//		} else {
//			console.log ("not found to destroy: " + this.parmid);
//		}

//		this.destroyDescendants();
		this.inherited (arguments);
		console.log ("exit Parameter.destroy");
	}
	
});


}

if(!dojo._hasResource["dojox.grid._grid.lib"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.lib"] = true;
dojo.provide("dojox.grid._grid.lib");

dojo.isNumber = function(v){
	return (typeof v == 'number') || (v instanceof Number);
}

// summary:
//	grid utility library

dojo.mixin(dojox.grid, {
	na: '...',
	nop: function() {
	},
	getTdIndex: function(td){
		return td.cellIndex >=0 ? td.cellIndex : dojo.indexOf(td.parentNode.cells, td);
	},
	getTrIndex: function(tr){
		return tr.rowIndex >=0 ? tr.rowIndex : dojo.indexOf(tr.parentNode.childNodes, tr);
	},
	getTr: function(rowOwner, index){
		return rowOwner && ((rowOwner.rows||0)[index] || rowOwner.childNodes[index]);
	},
	getTd: function(rowOwner, rowIndex, cellIndex){
		return (dojox.grid.getTr(inTable, rowIndex)||0)[cellIndex];
	},
	findTable: function(node){
		for (var n=node; n && n.tagName!='TABLE'; n=n.parentNode);
		return n;
	},
	ascendDom: function(inNode, inWhile){
		for (var n=inNode; n && inWhile(n); n=n.parentNode);
		return n;
	},
	makeNotTagName: function(inTagName){
		var name = inTagName.toUpperCase();
		return function(node){ return node.tagName != name; };
	},
	fire: function(ob, ev, args){
		var fn = ob && ev && ob[ev];
		return fn && (args ? fn.apply(ob, args) : ob[ev]());
	},
	// from lib.js
	setStyleText: function(inNode, inStyleText){
		if(inNode.style.cssText == undefined){
			inNode.setAttribute("style", inStyleText);
		}else{
			inNode.style.cssText = inStyleText;
		}
	},
	getStyleText: function(inNode, inStyleText){
		return (inNode.style.cssText == undefined ? inNode.getAttribute("style") : inNode.style.cssText);
	},
	setStyle: function(inElement, inStyle, inValue){
		if(inElement && inElement.style[inStyle] != inValue){
			inElement.style[inStyle] = inValue;
		}
	},
	setStyleHeightPx: function(inElement, inHeight){
		if(inHeight >= 0){
			dojox.grid.setStyle(inElement, 'height', inHeight + 'px');
		}
	},
	mouseEvents: [ 'mouseover', 'mouseout', /*'mousemove',*/ 'mousedown', 'mouseup', 'click', 'dblclick', 'contextmenu' ],
	keyEvents: [ 'keyup', 'keydown', 'keypress' ],
	funnelEvents: function(inNode, inObject, inMethod, inEvents){
		var evts = (inEvents ? inEvents : dojox.grid.mouseEvents.concat(dojox.grid.keyEvents));
		for (var i=0, l=evts.length; i<l; i++){
			dojo.connect(inNode, 'on' + evts[i], inObject, inMethod);
		}
	},
	removeNode: function(inNode){
		inNode = dojo.byId(inNode);
		inNode && inNode.parentNode && inNode.parentNode.removeChild(inNode);
		return inNode;
	},
	getScrollbarWidth: function(){
		if(this._scrollBarWidth){
			return this._scrollBarWidth;
		}
		this._scrollBarWidth = 18;
		try{
			var e = document.createElement("div");
			e.style.cssText = "top:0;left:0;width:100px;height:100px;overflow:scroll;position:absolute;visibility:hidden;";
			document.body.appendChild(e);
			this._scrollBarWidth = e.offsetWidth - e.clientWidth;
			document.body.removeChild(e);
			delete e;
		}catch (ex){}
		return this._scrollBarWidth;
	},
	// needed? dojo has _getProp
	getRef: function(name, create, context){
		var obj=context||dojo.global, parts=name.split("."), prop=parts.pop();
		for(var i=0, p; obj&&(p=parts[i]); i++){
			obj = (p in obj ? obj[p] : (create ? obj[p]={} : undefined));
		}
		return { obj: obj, prop: prop }; 
	},
	getProp: function(name, create, context){
		with(dojox.grid.getRef(name, create, context)){
			return (obj)&&(prop)&&(prop in obj ? obj[prop] : (create ? obj[prop]={} : undefined));
		}
	},
	indexInParent: function(inNode){
		var i=0, n, p=inNode.parentNode;
		while(n = p.childNodes[i++]){
			if(n == inNode){
				return i - 1;
			}
		}
		return -1;
	},
	cleanNode: function(inNode){
		if(!inNode){
			return;
		}
		var filter = function(inW){
			return inW.domNode && dojo.isDescendant(inW.domNode, inNode, true);
		}
		var ws = dijit.registry.filter(filter);
		for(var i=0, w; (w=ws[i]); i++){
			w.destroy();
		}
		delete ws;
	},
	getTagName: function(inNodeOrId){
		var node = dojo.byId(inNodeOrId);
		return (node && node.tagName ? node.tagName.toLowerCase() : '');
	},
	nodeKids: function(inNode, inTag){
		var result = [];
		var i=0, n;
		while(n = inNode.childNodes[i++]){
			if(dojox.grid.getTagName(n) == inTag){
				result.push(n);
			}
		}
		return result;
	},
	divkids: function(inNode){
		return dojox.grid.nodeKids(inNode, 'div');
	},
	focusSelectNode: function(inNode){
		try{
			dojox.grid.fire(inNode, "focus");
			dojox.grid.fire(inNode, "select");
		}catch(e){// IE sux bad
		}
	},
	whenIdle: function(/*inContext, inMethod, args ...*/){
		setTimeout(dojo.hitch.apply(dojo, arguments), 0);
	},
	arrayCompare: function(inA, inB){
		for(var i=0,l=inA.length; i<l; i++){
			if(inA[i] != inB[i]){return false;}
		}
		return (inA.length == inB.length);
	},
	arrayInsert: function(inArray, inIndex, inValue){
		if(inArray.length <= inIndex){
			inArray[inIndex] = inValue;
		}else{
			inArray.splice(inIndex, 0, inValue);
		}
	},
	arrayRemove: function(inArray, inIndex){
		inArray.splice(inIndex, 1);
	},
	arraySwap: function(inArray, inI, inJ){
		var cache = inArray[inI];
		inArray[inI] = inArray[inJ];
		inArray[inJ] = cache;
	},
	initTextSizePoll: function(inInterval) {
		var f = document.createElement("div");
		with (f.style) {
			top = "0px";
			left = "0px";
			position = "absolute";
			visibility = "hidden";
		}
		f.innerHTML = "TheQuickBrownFoxJumpedOverTheLazyDog";
		document.body.appendChild(f);
		var fw = f.offsetWidth;
		var job = function() {
			if (f.offsetWidth != fw) {
				fw = f.offsetWidth;
				dojox.grid.textSizeChanged();
			}
		}
		window.setInterval(job, inInterval||200);
		dojox.grid.initTextSizePoll = dojox.grid.nop;
	},
	textSizeChanged: function() {
	}
});

dojox.grid.jobs = {
	cancel: function(inHandle){
		if(inHandle){
			window.clearTimeout(inHandle);
		}
	},
	jobs: [],
	job: function(inName, inDelay, inJob){
		dojox.grid.jobs.cancelJob(inName);
		var job = function(){
			delete dojox.grid.jobs.jobs[inName];
			inJob();
		}
		dojox.grid.jobs.jobs[inName] = setTimeout(job, inDelay);
	},
	cancelJob: function(inName){
		dojox.grid.jobs.cancel(dojox.grid.jobs.jobs[inName]);
	}
}

}

if(!dojo._hasResource['dojox.grid._grid.scroller']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._grid.scroller'] = true;
dojo.provide('dojox.grid._grid.scroller');

dojo.declare('dojox.grid.scroller.base', null, {
	// summary:
	//	virtual scrollbox, abstract class
	//	Content must in /rows/
	//	Rows are managed in contiguous sets called /pages/
	//	There are a fixed # of rows per page
	//	The minimum rendered unit is a page
	constructor: function(){
		this.pageHeights = [];
		this.stack = [];
	},
	// specified
	rowCount: 0, // total number of rows to manage
	defaultRowHeight: 10, // default height of a row
	keepRows: 100, // maximum number of rows that should exist at one time
	contentNode: null, // node to contain pages
	scrollboxNode: null, // node that controls scrolling
	// calculated
	defaultPageHeight: 0, // default height of a page
	keepPages: 10, // maximum number of pages that should exists at one time
	pageCount: 0,
	windowHeight: 0,
	firstVisibleRow: 0,
	lastVisibleRow: 0,
	// private
	page: 0,
	pageTop: 0,
	// init
	init: function(inRowCount, inKeepRows, inRowsPerPage){
		switch(arguments.length){
			case 3: this.rowsPerPage = inRowsPerPage;
			case 2: this.keepRows = inKeepRows;
			case 1: this.rowCount = inRowCount;
		}
		this.defaultPageHeight = this.defaultRowHeight * this.rowsPerPage;
		//this.defaultPageHeight = this.defaultRowHeight * Math.min(this.rowsPerPage, this.rowCount);
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		this.keepPages = Math.max(Math.ceil(this.keepRows / this.rowsPerPage), 2);
		this.invalidate();
		if(this.scrollboxNode){
			this.scrollboxNode.scrollTop = 0;
			this.scroll(0);
			this.scrollboxNode.onscroll = dojo.hitch(this, 'onscroll');
		}
	},
	// updating
	invalidate: function(){
		this.invalidateNodes();
		this.pageHeights = [];
		this.height = (this.pageCount ? (this.pageCount - 1)* this.defaultPageHeight + this.calcLastPageHeight() : 0);
		this.resize();
	},
	updateRowCount: function(inRowCount){
		this.invalidateNodes();
		this.rowCount = inRowCount;
		// update page count, adjust document height
		oldPageCount = this.pageCount;
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		if(this.pageCount < oldPageCount){
			for(var i=oldPageCount-1; i>=this.pageCount; i--){
				this.height -= this.getPageHeight(i);
				delete this.pageHeights[i]
			}
		}else if(this.pageCount > oldPageCount){
			this.height += this.defaultPageHeight * (this.pageCount - oldPageCount - 1) + this.calcLastPageHeight();
		}
		this.resize();
	},
	// abstract interface
	pageExists: function(inPageIndex){
	},
	measurePage: function(inPageIndex){
	},
	positionPage: function(inPageIndex, inPos){
	},
	repositionPages: function(inPageIndex){
	},
	installPage: function(inPageIndex){
	},
	preparePage: function(inPageIndex, inPos, inReuseNode){
	},
	renderPage: function(inPageIndex){
	},
	removePage: function(inPageIndex){
	},
	pacify: function(inShouldPacify){
	},
	// pacification
	pacifying: false,
	pacifyTicks: 200,
	setPacifying: function(inPacifying){
		if(this.pacifying != inPacifying){
			this.pacifying = inPacifying;
			this.pacify(this.pacifying);
		}
	},
	startPacify: function(){
		this.startPacifyTicks = new Date().getTime();
	},
	doPacify: function(){
		var result = (new Date().getTime() - this.startPacifyTicks) > this.pacifyTicks;
		this.setPacifying(true);
		this.startPacify();
		return result;
	},
	endPacify: function(){
		this.setPacifying(false);
	},
	// default sizing implementation
	resize: function(){
		if(this.scrollboxNode){
			this.windowHeight = this.scrollboxNode.clientHeight;
		}
		dojox.grid.setStyleHeightPx(this.contentNode, this.height);
	},
	calcLastPageHeight: function(){
		if(!this.pageCount){
			return 0;
		}
		var lastPage = this.pageCount - 1;
		var lastPageHeight = ((this.rowCount % this.rowsPerPage)||(this.rowsPerPage)) * this.defaultRowHeight;
		this.pageHeights[lastPage] = lastPageHeight;
		return lastPageHeight;
	},
	updateContentHeight: function(inDh){
		this.height += inDh;
		this.resize();
	},
	updatePageHeight: function(inPageIndex){
		if(this.pageExists(inPageIndex)){
			var oh = this.getPageHeight(inPageIndex);
			var h = (this.measurePage(inPageIndex))||(oh);
			this.pageHeights[inPageIndex] = h;
			if((h)&&(oh != h)){
				this.updateContentHeight(h - oh)
				this.repositionPages(inPageIndex);
			}
		}
	},
	rowHeightChanged: function(inRowIndex){
		this.updatePageHeight(Math.floor(inRowIndex / this.rowsPerPage));
	},
	// scroller core
	invalidateNodes: function(){
		while(this.stack.length){
			this.destroyPage(this.popPage());
		}
	},
	createPageNode: function(){
		var p = document.createElement('div');
		p.style.position = 'absolute';
		//p.style.width = '100%';
		p.style.left = '0';
		return p;
	},
	getPageHeight: function(inPageIndex){
		var ph = this.pageHeights[inPageIndex];
		return (ph !== undefined ? ph : this.defaultPageHeight);
	},
	// FIXME: this is not a stack, it's a FIFO list
	pushPage: function(inPageIndex){
		return this.stack.push(inPageIndex);
	},
	popPage: function(){
		return this.stack.shift();
	},
	findPage: function(inTop){
		var i = 0, h = 0;
		for(var ph = 0; i<this.pageCount; i++, h += ph){
			ph = this.getPageHeight(i);
			if(h + ph >= inTop){
				break;
			}
		}
		this.page = i;
		this.pageTop = h;
	},
	buildPage: function(inPageIndex, inReuseNode, inPos){
		this.preparePage(inPageIndex, inReuseNode);
		this.positionPage(inPageIndex, inPos);
		// order of operations is key below
		this.installPage(inPageIndex);
		this.renderPage(inPageIndex);
		// order of operations is key above
		this.pushPage(inPageIndex);
	},
	needPage: function(inPageIndex, inPos){
		var h = this.getPageHeight(inPageIndex), oh = h;
		if(!this.pageExists(inPageIndex)){
			this.buildPage(inPageIndex, (this.keepPages)&&(this.stack.length >= this.keepPages), inPos);
			h = this.measurePage(inPageIndex) || h;
			this.pageHeights[inPageIndex] = h;
			if(h && (oh != h)){
				this.updateContentHeight(h - oh)
			}
		}else{
			this.positionPage(inPageIndex, inPos);
		}
		return h;
	},
	onscroll: function(){
		this.scroll(this.scrollboxNode.scrollTop);
	},
	scroll: function(inTop){
		this.startPacify();
		this.findPage(inTop);
		var h = this.height;
		var b = this.getScrollBottom(inTop);
		for(var p=this.page, y=this.pageTop; (p<this.pageCount)&&((b<0)||(y<b)); p++){
			y += this.needPage(p, y);
		}
		this.firstVisibleRow = this.getFirstVisibleRow(this.page, this.pageTop, inTop);
		this.lastVisibleRow = this.getLastVisibleRow(p - 1, y, b);
		// indicates some page size has been updated
		if(h != this.height){
			this.repositionPages(p-1);
		}
		this.endPacify();
	},
	getScrollBottom: function(inTop){
		return (this.windowHeight >= 0 ? inTop + this.windowHeight : -1);
	},
	// events
	processNodeEvent: function(e, inNode){
		var t = e.target;
		while(t && (t != inNode) && t.parentNode && (t.parentNode.parentNode != inNode)){
			t = t.parentNode;
		}
		if(!t || !t.parentNode || (t.parentNode.parentNode != inNode)){
			return false;
		}
		var page = t.parentNode;
		e.topRowIndex = page.pageIndex * this.rowsPerPage;
		e.rowIndex = e.topRowIndex + dojox.grid.indexInParent(t);
		e.rowTarget = t;
		return true;
	},
	processEvent: function(e){
		return this.processNodeEvent(e, this.contentNode);
	},
	dummy: 0
});

dojo.declare('dojox.grid.scroller', dojox.grid.scroller.base, {
	// summary:
	//	virtual scroller class, makes no assumption about shape of items being scrolled
	constructor: function(){
		this.pageNodes = [];
	},
	// virtual rendering interface
	renderRow: function(inRowIndex, inPageNode){
	},
	removeRow: function(inRowIndex){
	},
	// page node operations
	getDefaultNodes: function(){
		return this.pageNodes;
	},
	getDefaultPageNode: function(inPageIndex){
		return this.getDefaultNodes()[inPageIndex];
	},
	positionPageNode: function(inNode, inPos){
		inNode.style.top = inPos + 'px';
	},
	getPageNodePosition: function(inNode){
		return inNode.offsetTop;
	},
	repositionPageNodes: function(inPageIndex, inNodes){
		var last = 0;
		for(var i=0; i<this.stack.length; i++){
			last = Math.max(this.stack[i], last);
		}
		//
		var n = inNodes[inPageIndex];
		var y = (n ? this.getPageNodePosition(n) + this.getPageHeight(inPageIndex) : 0);
		//console.log('detected height change, repositioning from #%d (%d) @ %d ', inPageIndex + 1, last, y, this.pageHeights[0]);
		//
		for(var p=inPageIndex+1; p<=last; p++){
			n = inNodes[p];
			if(n){
				//console.log('#%d @ %d', inPageIndex, y, this.getPageNodePosition(n));
				if(this.getPageNodePosition(n) == y){
					return;
				}
				//console.log('placing page %d at %d', p, y);
				this.positionPage(p, y);
			}
			y += this.getPageHeight(p);
		}
	},
	invalidatePageNode: function(inPageIndex, inNodes){
		var p = inNodes[inPageIndex];
		if(p){
			delete inNodes[inPageIndex];
			this.removePage(inPageIndex, p);
			dojox.grid.cleanNode(p);
			p.innerHTML = '';
		}
		return p;
	},
	preparePageNode: function(inPageIndex, inReusePageIndex, inNodes){
		var p = (inReusePageIndex === null ? this.createPageNode() : this.invalidatePageNode(inReusePageIndex, inNodes));
		p.pageIndex = inPageIndex;
		p.id = 'page-' + inPageIndex;
		inNodes[inPageIndex] = p;
	},
	// implementation for page manager
	pageExists: function(inPageIndex){
		return Boolean(this.getDefaultPageNode(inPageIndex));
	},
	measurePage: function(inPageIndex){
		return this.getDefaultPageNode(inPageIndex).offsetHeight;
	},
	positionPage: function(inPageIndex, inPos){
		this.positionPageNode(this.getDefaultPageNode(inPageIndex), inPos);
	},
	repositionPages: function(inPageIndex){
		this.repositionPageNodes(inPageIndex, this.getDefaultNodes());
	},
	preparePage: function(inPageIndex, inReuseNode){
		this.preparePageNode(inPageIndex, (inReuseNode ? this.popPage() : null), this.getDefaultNodes());
	},
	installPage: function(inPageIndex){
		this.contentNode.appendChild(this.getDefaultPageNode(inPageIndex));
	},
	destroyPage: function(inPageIndex){
		var p = this.invalidatePageNode(inPageIndex, this.getDefaultNodes());
		dojox.grid.removeNode(p);
	},
	// rendering implementation
	renderPage: function(inPageIndex){
		var node = this.pageNodes[inPageIndex];
		for(var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
			this.renderRow(j, node);
		}
	},
	removePage: function(inPageIndex){
		for(var i=0, j=inPageIndex*this.rowsPerPage; i<this.rowsPerPage; i++, j++){
			this.removeRow(j);
		}
	},
	// scroll control
	getPageRow: function(inPage){
		return inPage * this.rowsPerPage;
	},
	getLastPageRow: function(inPage){
		return Math.min(this.rowCount, this.getPageRow(inPage + 1)) - 1;
	},
	getFirstVisibleRowNodes: function(inPage, inPageTop, inScrollTop, inNodes){
		var row = this.getPageRow(inPage);
		var rows = dojox.grid.divkids(inNodes[inPage]);
		for(var i=0,l=rows.length; i<l && inPageTop<inScrollTop; i++, row++){
			inPageTop += rows[i].offsetHeight;
		}
		return (row ? row - 1 : row);
	},
	getFirstVisibleRow: function(inPage, inPageTop, inScrollTop){
		if(!this.pageExists(inPage)){
			return 0;
		}
		return this.getFirstVisibleRowNodes(inPage, inPageTop, inScrollTop, this.getDefaultNodes());
	},
	getLastVisibleRowNodes: function(inPage, inBottom, inScrollBottom, inNodes){
		var row = this.getLastPageRow(inPage);
		var rows = dojox.grid.divkids(inNodes[inPage]);
		for(var i=rows.length-1; i>=0 && inBottom>inScrollBottom; i--, row--){
			inBottom -= rows[i].offsetHeight;
		}
		return row + 1;
	},
	getLastVisibleRow: function(inPage, inBottom, inScrollBottom){
		if(!this.pageExists(inPage)){
			return 0;
		}
		return this.getLastVisibleRowNodes(inPage, inBottom, inScrollBottom, this.getDefaultNodes());
	},
	findTopRowForNodes: function(inScrollTop, inNodes){
		var rows = dojox.grid.divkids(inNodes[this.page]);
		for(var i=0,l=rows.length,t=this.pageTop,h; i<l; i++){
			h = rows[i].offsetHeight;
			t += h;
			if(t >= inScrollTop){
				this.offset = h - (t - inScrollTop);
				return i + this.page * this.rowsPerPage;
			}
		}
		return -1;
	},
	findScrollTopForNodes: function(inRow, inNodes){
		var rowPage = Math.floor(inRow / this.rowsPerPage);
		var t = 0;
		for(var i=0; i<rowPage; i++){
			t += this.getPageHeight(i);
		}
		this.pageTop = t;
		this.needPage(rowPage, this.pageTop);
		var rows = dojox.grid.divkids(inNodes[rowPage]);
		var r = inRow - this.rowsPerPage * rowPage;
		for(var i=0,l=rows.length; i<l && i<r; i++){
			t += rows[i].offsetHeight;
		}
		return t;
	},
	findTopRow: function(inScrollTop){
		return this.findTopRowForNodes(inScrollTop, this.getDefaultNodes());
	},
	findScrollTop: function(inRow){
		return this.findScrollTopForNodes(inRow, this.getDefaultNodes());
	},
	dummy: 0
});

dojo.declare('dojox.grid.scroller.columns', dojox.grid.scroller, {
	// summary:
	//	Virtual scroller class that scrolls list of columns. Owned by grid and used internally
	//	for virtual scrolling.
	constructor: function(inContentNodes){
		this.setContentNodes(inContentNodes);
	},
	// nodes
	setContentNodes: function(inNodes){
		this.contentNodes = inNodes;
		this.colCount = (this.contentNodes ? this.contentNodes.length : 0);
		this.pageNodes = [];
		for(var i=0; i<this.colCount; i++){
			this.pageNodes[i] = [];
		}
	},
	getDefaultNodes: function(){
		return this.pageNodes[0] || [];
	},
	scroll: function(inTop) {
		if(this.colCount){
			dojox.grid.scroller.prototype.scroll.call(this, inTop);
		}
	},
	// resize
	resize: function(){
		if(this.scrollboxNode){
			this.windowHeight = this.scrollboxNode.clientHeight;
		}
		for(var i=0; i<this.colCount; i++){
			dojox.grid.setStyleHeightPx(this.contentNodes[i], this.height);
		}
	},
	// implementation for page manager
	positionPage: function(inPageIndex, inPos){
		for(var i=0; i<this.colCount; i++){
			this.positionPageNode(this.pageNodes[i][inPageIndex], inPos);
		}
	},
	preparePage: function(inPageIndex, inReuseNode){
		var p = (inReuseNode ? this.popPage() : null);
		for(var i=0; i<this.colCount; i++){
			this.preparePageNode(inPageIndex, p, this.pageNodes[i]);
		}
	},
	installPage: function(inPageIndex){
		for(var i=0; i<this.colCount; i++){
			this.contentNodes[i].appendChild(this.pageNodes[i][inPageIndex]);
		}
	},
	destroyPage: function(inPageIndex){
		for(var i=0; i<this.colCount; i++){
			dojox.grid.removeNode(this.invalidatePageNode(inPageIndex, this.pageNodes[i]));
		}
	},
	// rendering implementation
	renderPage: function(inPageIndex){
		var nodes = [];
		for(var i=0; i<this.colCount; i++){
			nodes[i] = this.pageNodes[i][inPageIndex];
		}
		//this.renderRows(inPageIndex*this.rowsPerPage, this.rowsPerPage, nodes);
		for(var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
			this.renderRow(j, nodes);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.drag"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.drag"] = true;
dojo.provide("dojox.grid._grid.drag");

// summary:
//	utility functions for dragging as used in grid.
// begin closure
(function(){

var dgdrag = dojox.grid.drag = {};

dgdrag.dragging = false;
dgdrag.hysteresis = 2;

dgdrag.capture = function(inElement) {
	//console.debug('dojox.grid.drag.capture');
	if (inElement.setCapture)
		inElement.setCapture();
	else {
		document.addEventListener("mousemove", inElement.onmousemove, true);
		document.addEventListener("mouseup", inElement.onmouseup, true);
		document.addEventListener("click", inElement.onclick, true);
	}
}

dgdrag.release = function(inElement) {
	//console.debug('dojox.grid.drag.release');
	if(inElement.releaseCapture){
		inElement.releaseCapture();
	}else{
		document.removeEventListener("click", inElement.onclick, true);
		document.removeEventListener("mouseup", inElement.onmouseup, true);
		document.removeEventListener("mousemove", inElement.onmousemove, true);
	}
}

dgdrag.start = function(inElement, inOnDrag, inOnEnd, inEvent, inOnStart){
	if(/*dgdrag.elt ||*/ !inElement || dgdrag.dragging){
		console.debug('failed to start drag: bad input node or already dragging');
		return;
	}
	dgdrag.dragging = true;
	dgdrag.elt = inElement;
	dgdrag.events = {
		drag: inOnDrag || dojox.grid.nop, 
		end: inOnEnd || dojox.grid.nop, 
		start: inOnStart || dojox.grid.nop, 
		oldmove: inElement.onmousemove, 
		oldup: inElement.onmouseup, 
		oldclick: inElement.onclick 
	};
	dgdrag.positionX = (inEvent && ('screenX' in inEvent) ? inEvent.screenX : false);
	dgdrag.positionY = (inEvent && ('screenY' in inEvent) ? inEvent.screenY : false);
	dgdrag.started = (dgdrag.position === false);
	inElement.onmousemove = dgdrag.mousemove;
	inElement.onmouseup = dgdrag.mouseup;
	inElement.onclick = dgdrag.click;
	dgdrag.capture(dgdrag.elt);
}

dgdrag.end = function(){
	//console.debug("dojox.grid.drag.end");
	dgdrag.release(dgdrag.elt);
	dgdrag.elt.onmousemove = dgdrag.events.oldmove;
	dgdrag.elt.onmouseup = dgdrag.events.oldup;
	dgdrag.elt.onclick = dgdrag.events.oldclick;
	dgdrag.elt = null;
	try{
		if(dgdrag.started){
			dgdrag.events.end();
		}
	}finally{
		dgdrag.dragging = false;
	}
}

dgdrag.calcDelta = function(inEvent){
	inEvent.deltaX = inEvent.screenX - dgdrag.positionX;
	inEvent.deltaY = inEvent.screenY - dgdrag.positionY;
}

dgdrag.hasMoved = function(inEvent){
	return Math.abs(inEvent.deltaX) + Math.abs(inEvent.deltaY) > dgdrag.hysteresis;
}

dgdrag.mousemove = function(inEvent){
	inEvent = dojo.fixEvent(inEvent);
	dojo.stopEvent(inEvent);
	dgdrag.calcDelta(inEvent);
	if((!dgdrag.started)&&(dgdrag.hasMoved(inEvent))){
		dgdrag.events.start(inEvent);
		dgdrag.started = true;
	}
	if(dgdrag.started){
		dgdrag.events.drag(inEvent);
	}
}

dgdrag.mouseup = function(inEvent){
	//console.debug("dojox.grid.drag.mouseup");
	dojo.stopEvent(dojo.fixEvent(inEvent));
	dgdrag.end();
}

dgdrag.click = function(inEvent){
	dojo.stopEvent(dojo.fixEvent(inEvent));
	//dgdrag.end();
}

})();
// end closure

}

if(!dojo._hasResource["dojox.grid._grid.builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.builder"] = true;
dojo.provide("dojox.grid._grid.builder");


dojo.declare("dojox.grid.Builder", null, {
	// summary:
	//	Base class to produce html for grid content.
	//	Also provide event decoration, providing grid related information inside the event object
	// passed to grid events.
	constructor: function(inView){
		this.view = inView;
		this.grid = inView.grid;
	},
	view: null,
	// boilerplate HTML
	_table: '<table class="dojoxGrid-row-table" border="0" cellspacing="0" cellpadding="0" role="wairole:presentation">',
	// generate starting tags for a cell
	generateCellMarkup: function(inCell, inMoreStyles, inMoreClasses, isHeader){
		var result = [], html;
		if (isHeader){
			html = [ '<th tabIndex="-1" role="wairole:columnheader"' ];
		}else{
			html = [ '<td tabIndex="-1" role="wairole:gridcell"' ];
		}
		inCell.colSpan && html.push(' colspan="', inCell.colSpan, '"');
		inCell.rowSpan && html.push(' rowspan="', inCell.rowSpan, '"');
		html.push(' class="dojoxGrid-cell ');
		inCell.classes && html.push(inCell.classes, ' ');
		inMoreClasses && html.push(inMoreClasses, ' ');
		// result[0] => td opener, style
		result.push(html.join(''));
		// SLOT: result[1] => td classes 
		result.push('');
		html = ['" idx="', inCell.index, '" style="'];
		html.push(inCell.styles, inMoreStyles||'');
		inCell.unitWidth && html.push('width:', inCell.unitWidth, ';');
		// result[2] => markup
		result.push(html.join(''));
		// SLOT: result[3] => td style 
		result.push('');
		html = [ '"' ];
		inCell.attrs && html.push(" ", inCell.attrs);
		html.push('>');
		// result[4] => td postfix
		result.push(html.join(''));
		// SLOT: result[5] => content
		result.push('');
		// result[6] => td closes
		result.push('</td>');
		return result;
	},
	// cell finding
	isCellNode: function(inNode){
		return Boolean(inNode && inNode.getAttribute && inNode.getAttribute("idx"));
	},
	getCellNodeIndex: function(inCellNode){
		return inCellNode ? Number(inCellNode.getAttribute("idx")) : -1;
	},
	getCellNode: function(inRowNode, inCellIndex){
		for(var i=0, row; row=dojox.grid.getTr(inRowNode.firstChild, i); i++){
			for(var j=0, cell; cell=row.cells[j]; j++){
				if(this.getCellNodeIndex(cell) == inCellIndex){
					return cell;
				}
			}
		}
	},
	findCellTarget: function(inSourceNode, inTopNode){
		var n = inSourceNode;
		while(n && (!this.isCellNode(n) || (dojox.grid.gridViewTag in n.offsetParent.parentNode && n.offsetParent.parentNode[dojox.grid.gridViewTag] != this.view)) && (n!=inTopNode)){
			n = n.parentNode;
		}
		return n!=inTopNode ? n : null 
	},
	// event decoration
	baseDecorateEvent: function(e){
		e.dispatch = 'do' + e.type;
		e.grid = this.grid;
		e.sourceView = this.view;
		e.cellNode = this.findCellTarget(e.target, e.rowNode);
		e.cellIndex = this.getCellNodeIndex(e.cellNode);
		e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
	},
	// event dispatch
	findTarget: function(inSource, inTag){
		var n = inSource;
		while(n && (n!=this.domNode) && (!(inTag in n) || (dojox.grid.gridViewTag in n && n[dojox.grid.gridViewTag] != this.view))){
			n = n.parentNode;
		}
		return (n != this.domNode) ? n : null; 
	},
	findRowTarget: function(inSource){
		return this.findTarget(inSource, dojox.grid.rowIndexTag);
	},
	isIntraNodeEvent: function(e){
		try{
			return (e.cellNode && e.relatedTarget && dojo.isDescendant(e.relatedTarget, e.cellNode));
		}catch(x){
			// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;
		}
	},
	isIntraRowEvent: function(e){
		try{
			var row = e.relatedTarget && this.findRowTarget(e.relatedTarget);
			return !row && (e.rowIndex==-1) || row && (e.rowIndex==row.gridRowIndex);			
		}catch(x){
			// e.relatedTarget on INPUT has permission problem in FF: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;
		}
	},
	dispatchEvent: function(e){
		if(e.dispatch in this){
			return this[e.dispatch](e);
		}
	},
	// dispatched event handlers
	domouseover: function(e){
		if(e.cellNode && (e.cellNode!=this.lastOverCellNode)){
			this.lastOverCellNode = e.cellNode;
			this.grid.onMouseOver(e);
		}
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(e.cellNode && (e.cellNode==this.lastOverCellNode) && !this.isIntraNodeEvent(e, this.lastOverCellNode)){
			this.lastOverCellNode = null;
			this.grid.onMouseOut(e);
			if(!this.isIntraRowEvent(e)){
				this.grid.onMouseOutRow(e);
			}
		}
	}
});

dojo.declare("dojox.grid.contentBuilder", dojox.grid.Builder, {
	// summary:
	//	Produces html for grid data content. Owned by grid and used internally 
	//	for rendering data. Override to implement custom rendering.
	update: function(){
		this.prepareHtml();
	},
	// cache html for rendering data rows
	prepareHtml: function(){
		var defaultGet=this.grid.get, rows=this.view.structure.rows;
		for(var j=0, row; (row=rows[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				cell.get = cell.get || (cell.value == undefined) && defaultGet;
				cell.markup = this.generateCellMarkup(cell, cell.cellStyles, cell.cellClasses, false);
			}
		}
	},
	// time critical: generate html using cache and data source
	generateHtml: function(inDataIndex, inRowIndex){
		var
			html = [ this._table ],
			v = this.view,
			obr = v.onBeforeRow,
			rows = v.structure.rows;
		obr && obr(inRowIndex, rows);
		for(var j=0, row; (row=rows[j]); j++){
			if(row.hidden || row.header){
				continue;
			}
			html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGrid-invisible">');
			for(var i=0, cell, m, cc, cs; (cell=row[i]); i++){
				m = cell.markup, cc = cell.customClasses = [], cs = cell.customStyles = [];
				// content (format can fill in cc and cs as side-effects)
				m[5] = cell.format(inDataIndex);
				// classes
				m[1] = cc.join(' ');
				// styles
				m[3] = cs.join(';');
				// in-place concat
				html.push.apply(html, m);
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');
	},
	decorateEvent: function(e){
		e.rowNode = this.findRowTarget(e.target);
		if(!e.rowNode){return false};
		e.rowIndex = e.rowNode[dojox.grid.rowIndexTag];
		this.baseDecorateEvent(e);
		e.cell = this.grid.getCell(e.cellIndex);
		return true;
	}
});

dojo.declare("dojox.grid.headerBuilder", dojox.grid.Builder, {
	// summary:
	//	Produces html for grid header content. Owned by grid and used internally 
	//	for rendering data. Override to implement custom rendering.
	bogusClickTime: 0,
	overResizeWidth: 4,
	minColWidth: 1,
	_table: '<table class="dojoxGrid-row-table" border="0" cellspacing="0" cellpadding="0" role="wairole:presentation"',
	update: function(){
		this.tableMap = new dojox.grid.tableMap(this.view.structure.rows);
	},
	generateHtml: function(inGetValue, inValue){
		var html = [this._table], rows = this.view.structure.rows;
		
		// render header with appropriate width, if possible so that views with flex columns are correct height
		if(this.view.viewWidth){
			html.push([' style="width:', this.view.viewWidth, ';"'].join(''));
		}
		html.push('>');
		dojox.grid.fire(this.view, "onBeforeRow", [-1, rows]);
		for(var j=0, row; (row=rows[j]); j++){
			if(row.hidden){
				continue;
			}
			html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGrid-invisible">');
			for(var i=0, cell, markup; (cell=row[i]); i++){
				cell.customClasses = [];
				cell.customStyles = [];
				markup = this.generateCellMarkup(cell, cell.headerStyles, cell.headerClasses, true);
				// content
				markup[5] = (inValue != undefined ? inValue : inGetValue(cell));
				// styles
				markup[3] = cell.customStyles.join(';');
				// classes
				markup[1] = cell.customClasses.join(' '); //(cell.customClasses ? ' ' + cell.customClasses : '');
				html.push(markup.join(''));
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');
	},
	// event helpers
	getCellX: function(e){
		var x = e.layerX;
		if(dojo.isMoz){
			var n = dojox.grid.ascendDom(e.target, dojox.grid.makeNotTagName("th"));
			x -= (n && n.offsetLeft) || 0;
			//x -= getProp(ascendDom(e.target, mkNotTagName("td")), "offsetLeft") || 0;
		}
		var n = dojox.grid.ascendDom(e.target, function(){
			if(!n || n == e.cellNode){
				return false;
			}
			// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
			// when parent has border, overflow: hidden, and is positioned
			// handle this problem here ... not a general solution!
			x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
			return true;
		});
		return x;
	},
	// event decoration
	decorateEvent: function(e){
		this.baseDecorateEvent(e);
		e.rowIndex = -1;
		e.cellX = this.getCellX(e);
		return true;
	},
	// event handlers
	// resizing
	prepareLeftResize: function(e){
		var i = dojox.grid.getTdIndex(e.cellNode);
		e.cellNode = (i ? e.cellNode.parentNode.cells[i-1] : null);
		e.cellIndex = (e.cellNode ? this.getCellNodeIndex(e.cellNode) : -1);
		return Boolean(e.cellNode);
	},
	canResize: function(e){
		if(!e.cellNode || e.cellNode.colSpan > 1){
			return false;
		}
		var cell = this.grid.getCell(e.cellIndex); 
		return !cell.noresize && !cell.isFlex();
	},
	overLeftResizeArea: function(e){
		return (e.cellIndex>0) && (e.cellX < this.overResizeWidth) && this.prepareLeftResize(e);
	},
	overRightResizeArea: function(e){
		return e.cellNode && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth);
	},
	domousemove: function(e){
		//console.log(e.cellIndex, e.cellX, e.cellNode.offsetWidth);
		var c = (this.overRightResizeArea(e) ? 'e-resize' : (this.overLeftResizeArea(e) ? 'w-resize' : ''));
		if(c && !this.canResize(e)){
			c = 'not-allowed';
		}
		e.sourceView.headerNode.style.cursor = c || ''; //'default';
	},
	domousedown: function(e){
		if(!dojox.grid.drag.dragging){
			if((this.overRightResizeArea(e) || this.overLeftResizeArea(e)) && this.canResize(e)){
				this.beginColumnResize(e);
			}
			//else{
			//	this.beginMoveColumn(e);
			//}
		}
	},
	doclick: function(e) {
		if (new Date().getTime() < this.bogusClickTime) {
			dojo.stopEvent(e);
			return true;
		}
	},
	// column resizing
	beginColumnResize: function(e){
		dojo.stopEvent(e);
		var spanners = [], nodes = this.tableMap.findOverlappingNodes(e.cellNode);
		for(var i=0, cell; (cell=nodes[i]); i++){
			spanners.push({ node: cell, index: this.getCellNodeIndex(cell), width: cell.offsetWidth });
			//console.log("spanner: " + this.getCellNodeIndex(cell));
		}
		var drag = {
			view: e.sourceView,
			node: e.cellNode,
			index: e.cellIndex,
			w: e.cellNode.clientWidth,
			spanners: spanners
		};
		//console.log(drag.index, drag.w);
		dojox.grid.drag.start(e.cellNode, dojo.hitch(this, 'doResizeColumn', drag), dojo.hitch(this, 'endResizeColumn', drag), e);
	},
	doResizeColumn: function(inDrag, inEvent){
		var w = inDrag.w + inEvent.deltaX;
		if(w >= this.minColWidth){
			for(var i=0, s, sw; (s=inDrag.spanners[i]); i++){
				sw = s.width + inEvent.deltaX;
				s.node.style.width = sw + 'px';
				inDrag.view.setColWidth(s.index, sw);
				//console.log('setColWidth', '#' + s.index, sw + 'px');
			}
			inDrag.node.style.width = w + 'px';
			inDrag.view.setColWidth(inDrag.index, w);
		}
		if(inDrag.view.flexCells && !inDrag.view.testFlexCells()){
			var t = dojox.grid.findTable(inDrag.node);
			t && (t.style.width = '');
		}
	},
	endResizeColumn: function(inDrag){
		this.bogusClickTime = new Date().getTime() + 30;
		setTimeout(dojo.hitch(inDrag.view, "update"), 50);
	}
});

dojo.declare("dojox.grid.tableMap", null, {
	// summary:
	//	Maps an html table into a structure parsable for information about cell row and col spanning.
	//	Used by headerBuilder
	constructor: function(inRows){
		this.mapRows(inRows);
	},
	map: null,
	// map table topography
	mapRows: function(inRows){
		//console.log('mapRows');
		// # of rows
		var rowCount = inRows.length;
		if(!rowCount){
			return;
		}
		// map which columns and rows fill which cells
		this.map = [ ];
		for(var j=0, row; (row=inRows[j]); j++){
			this.map[j] = [];
		}
		for(var j=0, row; (row=inRows[j]); j++){
			for(var i=0, x=0, cell, colSpan, rowSpan; (cell=row[i]); i++){
				while (this.map[j][x]){x++};
				this.map[j][x] = { c: i, r: j };
				rowSpan = cell.rowSpan || 1;
				colSpan = cell.colSpan || 1;
				for(var y=0; y<rowSpan; y++){
					for(var s=0; s<colSpan; s++){
						this.map[j+y][x+s] = this.map[j][x];
					}
				}
				x += colSpan;
			}
		}
		//this.dumMap();
	},
	dumpMap: function(){
		for(var j=0, row, h=''; (row=this.map[j]); j++,h=''){
			for(var i=0, cell; (cell=row[i]); i++){
				h += cell.r + ',' + cell.c + '   ';
			}
			console.log(h);
		}
	},
	// find node's map coords by it's structure coords
	getMapCoords: function(inRow, inCol){
		for(var j=0, row; (row=this.map[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				if(cell.c==inCol && cell.r == inRow){
					return { j: j, i: i };
				}
				//else{console.log(inRow, inCol, ' : ', i, j, " : ", cell.r, cell.c); };
			}
		}
		return { j: -1, i: -1 };
	},
	// find a node in inNode's table with the given structure coords
	getNode: function(inTable, inRow, inCol){
		var row = inTable && inTable.rows[inRow];
		return row && row.cells[inCol];
	},
	_findOverlappingNodes: function(inTable, inRow, inCol){
		var nodes = [];
		var m = this.getMapCoords(inRow, inCol);
		//console.log("node j: %d, i: %d", m.j, m.i);
		var row = this.map[m.j];
		for(var j=0, row; (row=this.map[j]); j++){
			if(j == m.j){ continue; }
			with(row[m.i]){
				//console.log("overlaps: r: %d, c: %d", r, c);
				var n = this.getNode(inTable, r, c);
				if(n){ nodes.push(n); }
			}
		}
		//console.log(nodes);
		return nodes;
	},
	findOverlappingNodes: function(inNode){
		return this._findOverlappingNodes(dojox.grid.findTable(inNode), dojox.grid.getTrIndex(inNode.parentNode), dojox.grid.getTdIndex(inNode));
	}
});

dojox.grid.rowIndexTag = "gridRowIndex";
dojox.grid.gridViewTag = "gridView";

}

if(!dojo._hasResource["dojox.grid._grid.view"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.view"] = true;
dojo.provide("dojox.grid._grid.view");




dojo.declare('dojox.GridView', [dijit._Widget, dijit._Templated], {
	// summary:
	//	A collection of grid columns. A grid is comprised of a set of views that stack horizontally.
	//	Grid creates views automatically based on grid's layout structure.
	//	Users should typically not need to access individual views directly.
	defaultWidth: "18em",
	// viewWidth: string
	// width for the view, in valid css unit
	viewWidth: "",
	templateString: '<div class="dojoxGrid-view"><div class="dojoxGrid-header" dojoAttachPoint="headerNode"><div style="width: 9000em"><div dojoAttachPoint="headerContentNode"></div></div></div><input type="checkbox" class="dojoxGrid-hidden-focus" dojoAttachPoint="hiddenFocusNode" /><input type="checkbox" class="dojoxGrid-hidden-focus" /><div class="dojoxGrid-scrollbox" dojoAttachPoint="scrollboxNode"><div class="dojoxGrid-content" dojoAttachPoint="contentNode" hidefocus="hidefocus"></div></div></div>',
	themeable: false,
	classTag: 'dojoxGrid',
	marginBottom: 0,
	rowPad: 2,
	postMixInProperties: function(){
		this.rowNodes = [];
	},
	postCreate: function(){
		dojo.connect(this.scrollboxNode, "onscroll", dojo.hitch(this, "doscroll"));
		dojox.grid.funnelEvents(this.contentNode, this, "doContentEvent", [ 'mouseover', 'mouseout', 'click', 'dblclick', 'contextmenu' ]);
		dojox.grid.funnelEvents(this.headerNode, this, "doHeaderEvent", [ 'dblclick', 'mouseover', 'mouseout', 'mousemove', 'mousedown', 'click', 'contextmenu' ]);
		this.content = new dojox.grid.contentBuilder(this);
		this.header = new dojox.grid.headerBuilder(this);
	},
	destroy: function(){
		dojox.grid.removeNode(this.headerNode);
		this.inherited("destroy", arguments);
	},
	// focus 
	focus: function(){
		if(dojo.isSafari || dojo.isOpera){
			this.hiddenFocusNode.focus();
		}else{
			this.scrollboxNode.focus();
		}
	},
	setStructure: function(inStructure){
		var vs = this.structure = inStructure;
		// FIXME: similar logic is duplicated in layout
		if(vs.width && dojo.isNumber(vs.width)){
			this.viewWidth = vs.width + 'em';
		}else{
			this.viewWidth = vs.width || this.viewWidth; //|| this.defaultWidth;
		}
		this.onBeforeRow = vs.onBeforeRow;
		this.noscroll = vs.noscroll;
		if(this.noscroll){
			this.scrollboxNode.style.overflow = "hidden";
		}
		// bookkeeping
		this.testFlexCells();
		// accomodate new structure
		this.updateStructure();
	},
	testFlexCells: function(){
		// FIXME: cheater, this function does double duty as initializer and tester
		this.flexCells = false;
		for(var j=0, row; (row=this.structure.rows[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				cell.view = this;
				this.flexCells = this.flexCells || cell.isFlex();
			}
		}
		return this.flexCells;
	},
	updateStructure: function(){
		// header builder needs to update table map
		this.header.update();
		// content builder needs to update markup cache
		this.content.update();
	},
	getScrollbarWidth: function(){
		return (this.noscroll ? 0 : dojox.grid.getScrollbarWidth());
	},
	getColumnsWidth: function(){
		return this.headerContentNode.firstChild.offsetWidth;
	},
	getWidth: function(){
		return this.viewWidth || (this.getColumnsWidth()+this.getScrollbarWidth()) +'px';
	},
	getContentWidth: function(){
		return Math.max(0, dojo._getContentBox(this.domNode).w - this.getScrollbarWidth()) + 'px';
	},
	render: function(){
		this.scrollboxNode.style.height = '';
		this.renderHeader();
	},
	renderHeader: function(){
		this.headerContentNode.innerHTML = this.header.generateHtml(this._getHeaderContent);
	},
	// note: not called in 'view' context
	_getHeaderContent: function(inCell){
		var n = inCell.name || inCell.grid.getCellName(inCell);
		if(inCell.index != inCell.grid.getSortIndex()){
			return n;
		}
		return [ '<div class="', inCell.grid.sortInfo > 0 ? 'dojoxGrid-sort-down' : 'dojoxGrid-sort-up', '">', n, '</div>' ].join(''); 
	},
	resize: function(){
		this.resizeHeight();
		this.resizeWidth();
	},
	hasScrollbar: function(){
		return (this.scrollboxNode.clientHeight != this.scrollboxNode.offsetHeight);
	},
	resizeHeight: function(){
		if(!this.grid.autoHeight){
			var h = this.domNode.clientHeight;
			if(!this.hasScrollbar()){ // no scrollbar is rendered
				h -= dojox.grid.getScrollbarWidth();
			}
			dojox.grid.setStyleHeightPx(this.scrollboxNode, h);
		}
	},
	resizeWidth: function(){
		if(this.flexCells){
			// the view content width
			this.contentWidth = this.getContentWidth();
			this.headerContentNode.firstChild.style.width = this.contentWidth;
		}
		// FIXME: it should be easier to get w from this.scrollboxNode.clientWidth, 
		// but clientWidth seemingly does not include scrollbar width in some cases
		var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
		w = Math.max(w, this.getColumnsWidth()) + 'px';
		with(this.contentNode){
			style.width = '';
			offsetWidth;
			style.width = w;
		}
	},
	setSize: function(w, h){
		with(this.domNode.style){
			if(w){
				width = w;
			}
			height = (h >= 0 ? h + 'px' : '');
		}
		with(this.headerNode.style){
			if(w){
				width = w;
			}
		}
	},
	renderRow: function(inRowIndex, inHeightPx){
		var rowNode = this.createRowNode(inRowIndex);
		this.buildRow(inRowIndex, rowNode, inHeightPx);
		this.grid.edit.restore(this, inRowIndex);
		return rowNode;
	},
	createRowNode: function(inRowIndex){
		var node = document.createElement("div");
		node.className = this.classTag + '-row';
		node[dojox.grid.gridViewTag] = this;
		node[dojox.grid.rowIndexTag] = inRowIndex;
		this.rowNodes[inRowIndex] = node;
		return node;
	},
	buildRow: function(inRowIndex, inRowNode){
		this.buildRowContent(inRowIndex, inRowNode);
		this.styleRow(inRowIndex, inRowNode);
	},
	buildRowContent: function(inRowIndex, inRowNode){
		inRowNode.innerHTML = this.content.generateHtml(inRowIndex, inRowIndex); 
		if(this.flexCells){
			// FIXME: accessing firstChild here breaks encapsulation
			inRowNode.firstChild.style.width = this.contentWidth;
		}
	},
	rowRemoved:function(inRowIndex){
		this.grid.edit.save(this, inRowIndex);
		delete this.rowNodes[inRowIndex];
	},
	getRowNode: function(inRowIndex){
		return this.rowNodes[inRowIndex];
	},
	getCellNode: function(inRowIndex, inCellIndex){
		var row = this.getRowNode(inRowIndex);
		if(row){
			return this.content.getCellNode(row, inCellIndex);
		}
	},
	// styling
	styleRow: function(inRowIndex, inRowNode){
		inRowNode._style = dojox.grid.getStyleText(inRowNode);
		this.styleRowNode(inRowIndex, inRowNode);
	},
	styleRowNode: function(inRowIndex, inRowNode){
		if(inRowNode){
			this.doStyleRowNode(inRowIndex, inRowNode);
		}
	},
	doStyleRowNode: function(inRowIndex, inRowNode){
		this.grid.styleRowNode(inRowIndex, inRowNode);
	},
	// updating
	updateRow: function(inRowIndex, inHeightPx, inPageNode){
		var rowNode = this.getRowNode(inRowIndex);
		if(rowNode){
			rowNode.style.height = '';
			this.buildRow(inRowIndex, rowNode);
		}
		return rowNode;
	},
	updateRowStyles: function(inRowIndex){
		this.styleRowNode(inRowIndex, this.getRowNode(inRowIndex));
	},
	// scrolling
	lastTop: 0,
	doscroll: function(inEvent){
		this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
		// 'lastTop' is a semaphore to prevent feedback-loop with setScrollTop below
		var top = this.scrollboxNode.scrollTop;
		if(top != this.lastTop){
			this.grid.scrollTo(top);
		}
	},
	setScrollTop: function(inTop){
		// 'lastTop' is a semaphore to prevent feedback-loop with doScroll above
		this.lastTop = inTop;
		this.scrollboxNode.scrollTop = inTop;
		return this.scrollboxNode.scrollTop;
	},
	// event handlers (direct from DOM)
	doContentEvent: function(e){
		if(this.content.decorateEvent(e)){
			this.grid.onContentEvent(e);
		}
	},
	doHeaderEvent: function(e){
		if(this.header.decorateEvent(e)){
			this.grid.onHeaderEvent(e);
		}
	},
	// event dispatch(from Grid)
	dispatchContentEvent: function(e){
		return this.content.dispatchEvent(e);
	},
	dispatchHeaderEvent: function(e){
		return this.header.dispatchEvent(e);
	},
	// column resizing
	setColWidth: function(inIndex, inWidth){
		this.grid.setCellWidth(inIndex, inWidth + 'px');
	},
	update: function(){
		var left = this.scrollboxNode.scrollLeft;
		this.content.update();
		this.grid.update();
		this.scrollboxNode.scrollLeft = left;
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.views"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.views"] = true;
dojo.provide("dojox.grid._grid.views");

dojo.declare('dojox.grid.views', null, {
	// summary:
	//	A collection of grid views. Owned by grid and used internally for managing grid views.
	//	Grid creates views automatically based on grid's layout structure.
	//	Users should typically not need to access individual views or the views collection directly.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	defaultWidth: 200,
	views: [],
	// operations
	resize: function(){
		this.onEach("resize");
	},
	render: function(){
		this.onEach("render");
		this.normalizeHeaderNodeHeight();
	},
	// views
	addView: function(inView){
		inView.idx = this.views.length;
		this.views.push(inView);
	},
	destroyViews: function(){
		for (var i=0, v; v=this.views[i]; i++)
			v.destroy();
		this.views = [];
	},
	getContentNodes: function(){
		var nodes = [];
		for(var i=0, v; v=this.views[i]; i++){
			nodes.push(v.contentNode);
		}
		return nodes;
	},
	forEach: function(inCallback){
		for(var i=0, v; v=this.views[i]; i++){
			inCallback(v, i);
		}
	},
	onEach: function(inMethod, inArgs){
		inArgs = inArgs || [];
		for(var i=0, v; v=this.views[i]; i++){
			if(inMethod in v){
				v[inMethod].apply(v, inArgs);
			}
		}
	},
	// layout
	normalizeHeaderNodeHeight: function(){
		var rowNodes = [];
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.headerContentNode.firstChild){
				rowNodes.push(v.headerContentNode)
			};
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	normalizeRowNodeHeights: function(inRowNodes){
		var h = 0; 
		for(var i=0, n, o; (n=inRowNodes[i]); i++){
			h = Math.max(h, (n.firstChild.clientHeight)||(n.firstChild.offsetHeight));
		}
		h = (h >= 0 ? h : 0);
		//
		var hpx = h + 'px';
		for(var i=0, n; (n=inRowNodes[i]); i++){
			if(n.firstChild.clientHeight!=h){
				n.firstChild.style.height = hpx;
			}
		}
		//
		//console.log('normalizeRowNodeHeights ', h);
		//
		// querying the height here seems to help scroller measure the page on IE
		if(inRowNodes&&inRowNodes[0]){
			inRowNodes[0].parentNode.offsetHeight;
		}
	},
	renormalizeRow: function(inRowIndex){
		var rowNodes = [];
		for(var i=0, v, n; (v=this.views[i])&&(n=v.getRowNode(inRowIndex)); i++){
			n.firstChild.style.height = '';
			rowNodes.push(n);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	getViewWidth: function(inIndex){
		return this.views[inIndex].getWidth() || this.defaultWidth;
	},
	measureHeader: function(){
		this.forEach(function(inView){
			inView.headerContentNode.style.height = '';
		});
		var h = 0;
		this.forEach(function(inView){
			//console.log('headerContentNode', inView.headerContentNode.offsetHeight, inView.headerContentNode.offsetWidth);
			h = Math.max(inView.headerNode.offsetHeight, h);
		});
		return h;
	},
	measureContent: function(){
		var h = 0;
		this.forEach(function(inView) {
			h = Math.max(inView.domNode.offsetHeight, h);
		});
		return h;
	},
	findClient: function(inAutoWidth){
		// try to use user defined client
		var c = this.grid.elasticView || -1;
		// attempt to find implicit client
		if(c < 0){
			for(var i=1, v; (v=this.views[i]); i++){
				if(v.viewWidth){
					for(i=1; (v=this.views[i]); i++){
						if(!v.viewWidth){
							c = i;
							break;
						}
					}
					break;
				}
			}
		}
		// client is in the middle by default
		if(c < 0){
			c = Math.floor(this.views.length / 2);
		}
		return c;
	},
	_arrange: function(l, t, w, h){
		var i, v, vw, len = this.views.length;
		// find the client
		var c = (w <= 0 ? len : this.findClient());
		// layout views
		var setPosition = function(v, l, t){
			with(v.domNode.style){
				left = l + 'px';
				top = t + 'px';
			}
			with(v.headerNode.style){
				left = l + 'px';
				top = 0;
			}
		}
		// for views left of the client
		for(i=0; (v=this.views[i])&&(i<c); i++){
			// get width
			vw = this.getViewWidth(i);
			// process boxes
			v.setSize(vw, h);
			setPosition(v, l, t);
			vw = v.domNode.offsetWidth;
			// update position
			l += vw;
		}
		// next view (is the client, i++ == c) 
		i++;
		// start from the right edge
		var r = w;
		// for views right of the client (iterated from the right)
		for(var j=len-1; (v=this.views[j])&&(i<=j); j--){
			// get width
			vw = this.getViewWidth(j);
			// set size
			v.setSize(vw, h);
			// measure in pixels
			vw = v.domNode.offsetWidth;
			// update position
			r -= vw;
			// set position
			setPosition(v, r, t);
		}
		if(c<len){
			v = this.views[c];
			// position the client box between left and right boxes	
			vw = Math.max(1, r-l);
			// set size
			v.setSize(vw + 'px', h);
			setPosition(v, l, t);
		}
		return l;
	},
	arrange: function(l, t, w, h){
		var w = this._arrange(l, t, w, h);
		this.resize();
		return w;
	},
	// rendering
	renderRow: function(inRowIndex, inNodes){
		var rowNodes = [];
		for(var i=0, v, n, rowNode; (v=this.views[i])&&(n=inNodes[i]); i++){
			rowNode = v.renderRow(inRowIndex);
			n.appendChild(rowNode);
			rowNodes.push(rowNode);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	rowRemoved: function(inRowIndex){
		this.onEach("rowRemoved", [ inRowIndex ]);
	},
	// updating
	updateRow: function(inRowIndex, inHeight){
		for(var i=0, v; v=this.views[i]; i++){
			v.updateRow(inRowIndex, inHeight);
		}
		this.renormalizeRow(inRowIndex);
	},
	updateRowStyles: function(inRowIndex){
		this.onEach("updateRowStyles", [ inRowIndex ]);
	},
	// scrolling
	setScrollTop: function(inTop){
		var top = inTop;
		for(var i=0, v; v=this.views[i]; i++){
			top = v.setScrollTop(inTop);
		}
		return top;
		//this.onEach("setScrollTop", [ inTop ]);
	},
	getFirstScrollingView: function(){
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.hasScrollbar()){
				return v;
			}
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.cell"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.cell"] = true;
dojo.provide("dojox.grid._grid.cell");

dojo.declare("dojox.grid.cell", null, {
	// summary:
	//	Respresents a grid cell and contains information about column options and methods
	//	for retrieving cell related information.
	//	Each column in a grid layout has a cell object and most events and many methods
	//	provide access to these objects.
	styles: '',
	constructor: function(inProps){
		dojo.mixin(this, inProps);
		if(this.editor){this.editor = new this.editor(this);}
	},
	// data source
	format: function(inRowIndex){
		// summary:
		//	provides the html for a given grid cell.
		// inRowIndex: int
		// grid row index
		// returns: html for a given grid cell
		var f, i=this.grid.edit.info, d=this.get ? this.get(inRowIndex) : this.value;
		if(this.editor && (this.editor.alwaysOn || (i.rowIndex==inRowIndex && i.cell==this))){
			return this.editor.format(d, inRowIndex);
		}else{
			return (f = this.formatter) ? f.call(this, d, inRowIndex) : d;
		}
	},
	// utility
	getNode: function(inRowIndex){
		// summary:
		//	gets the dom node for a given grid cell.
		// inRowIndex: int
		// grid row index
		// returns: dom node for a given grid cell
		return this.view.getCellNode(inRowIndex, this.index);
	},
	isFlex: function(){
		var uw = this.unitWidth;
		return uw && (uw=='auto' || uw.slice(-1)=='%');
	},
	// edit support
	applyEdit: function(inValue, inRowIndex){
		this.grid.edit.applyCellEdit(inValue, this, inRowIndex);
	},
	cancelEdit: function(inRowIndex){
		this.grid.doCancelEdit(inRowIndex);
	},
	_onEditBlur: function(inRowIndex){
		if(this.grid.edit.isEditCell(inRowIndex, this.index)){
			//console.log('editor onblur', e);
			this.grid.edit.apply();
		}
	},
	registerOnBlur: function(inNode, inRowIndex){
		if(this.commitOnBlur){
			dojo.connect(inNode, "onblur", function(e){
				// hack: if editor still thinks this editor is current some ms after it blurs, assume we've focused away from grid
				setTimeout(dojo.hitch(this, "_onEditBlur", inRowIndex), 250);
			});
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.layout"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.layout"] = true;
dojo.provide("dojox.grid._grid.layout");


dojo.declare("dojox.grid.layout", null, {
	// summary:
	//	Controls grid cell layout. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	// flat array of grid cells
	cells: null,
	// structured array of grid cells
	structure: null,
	// default cell width
	defaultWidth: '6em',
	// methods
	setStructure: function(inStructure){
		this.fieldIndex = 0;
		this.cells = [];
		var s = this.structure = [];
		for(var i=0, viewDef, rows; (viewDef=inStructure[i]); i++){
			s.push(this.addViewDef(viewDef));
		}
		this.cellCount = this.cells.length;
	},
	addViewDef: function(inDef){
		this._defaultCellProps = inDef.defaultCell || {};
		return dojo.mixin({}, inDef, {rows: this.addRowsDef(inDef.rows || inDef.cells)});
	},
	addRowsDef: function(inDef){
		var result = [];
		for(var i=0, row; inDef && (row=inDef[i]); i++){
			result.push(this.addRowDef(i, row));
		}
		return result;
	},
	addRowDef: function(inRowIndex, inDef){
		var result = [];
		for(var i=0, def, cell; (def=inDef[i]); i++){
			cell = this.addCellDef(inRowIndex, i, def);
			result.push(cell);
			this.cells.push(cell);
		}
		return result;
	},
	addCellDef: function(inRowIndex, inCellIndex, inDef){
		var w = 0;
		if(inDef.colSpan > 1){
			w = 0;
		}else if(!isNaN(inDef.width)){
			w = inDef.width + "em";
		}else{
			w = inDef.width || this.defaultWidth;
		}
		// fieldIndex progresses linearly from the last indexed field
		// FIXME: support generating fieldIndex based a text field name (probably in Grid)
		var fieldIndex = inDef.field != undefined ? inDef.field : (inDef.get ? -1 : this.fieldIndex);
		if((inDef.field != undefined) || !inDef.get){
			this.fieldIndex = (inDef.field > -1 ? inDef.field : this.fieldIndex) + 1; 
		}
		return new dojox.grid.cell(
			dojo.mixin({}, this._defaultCellProps, inDef, {
				grid: this.grid,
				subrow: inRowIndex,
				layoutIndex: inCellIndex,
				index: this.cells.length,
				fieldIndex: fieldIndex,
				unitWidth: w
			}));
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.rows"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.rows"] = true;
dojo.provide("dojox.grid._grid.rows");

dojo.declare("dojox.grid.rows", null, {
	//	Stores information about grid rows. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	linesToEms: 2,
	defaultRowHeight: 1, // lines
	overRow: -2,
	// metrics
	getHeight: function(inRowIndex){
		return '';
	},
	getDefaultHeightPx: function(){
		// summmary:
		// retrieves the default row height
		// returns: int, default row height
		return 32;
		//return Math.round(this.defaultRowHeight * this.linesToEms * this.grid.contentPixelToEmRatio);
	},
	// styles
	prepareStylingRow: function(inRowIndex, inRowNode){
		return {
			index: inRowIndex, 
			node: inRowNode,
			odd: Boolean(inRowIndex&1),
			selected: this.grid.selection.isSelected(inRowIndex),
			over: this.isOver(inRowIndex),
			customStyles: "",
			customClasses: "dojoxGrid-row"
		}
	},
	styleRowNode: function(inRowIndex, inRowNode){
		var row = this.prepareStylingRow(inRowIndex, inRowNode);
		this.grid.onStyleRow(row);
		this.applyStyles(row);
	},
	applyStyles: function(inRow){
		with(inRow){
			node.className = customClasses;
			var h = node.style.height;
			dojox.grid.setStyleText(node, customStyles + ';' + (node._style||''));
			node.style.height = h;
		}
	},
	updateStyles: function(inRowIndex){
		this.grid.updateRowStyles(inRowIndex);
	},
	// states and events
	setOverRow: function(inRowIndex){
		var last = this.overRow;
		this.overRow = inRowIndex;
		if((last!=this.overRow)&&(last >=0)){
			this.updateStyles(last);
		}
		this.updateStyles(this.overRow);
	},
	isOver: function(inRowIndex){
		return (this.overRow == inRowIndex);
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.focus"] = true;
dojo.provide("dojox.grid._grid.focus");

// focus management
dojo.declare("dojox.grid.focus", null, {
	// summary:
	//	Controls grid cell focus. Owned by grid and used internally for focusing.
	//	Note: grid cell actually receives keyboard input only when cell is being edited.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.cell = null;
		this.rowIndex = -1;
		dojo.connect(this.grid.domNode, "onfocus", this, "doFocus");
	},
	tabbingOut: false,
	focusClass: "dojoxGrid-cell-focus",
	focusView: null,
	initFocusView: function(){
		this.focusView = this.grid.views.getFirstScrollingView();
	},
	isFocusCell: function(inCell, inRowIndex){
		// summary:
		//	states if the given cell is focused
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		// returns:
		//	true of the given grid cell is focused
		return (this.cell == inCell) && (this.rowIndex == inRowIndex);
	},
	isLastFocusCell: function(){
		return (this.rowIndex == this.grid.rowCount-1) && (this.cell.index == this.grid.layout.cellCount-1);
	},
	isFirstFocusCell: function(){
		return (this.rowIndex == 0) && (this.cell.index == 0);
	},
	isNoFocusCell: function(){
		return (this.rowIndex < 0) || !this.cell;
	},
	_focusifyCellNode: function(inBork){
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){
			dojo.toggleClass(n, this.focusClass, inBork);
			this.scrollIntoView();
			try{
				if(!this.grid.edit.isEditing())
					dojox.grid.fire(n, "focus");
			}catch(e){}
		}
	},
	scrollIntoView: function() {
		if(!this.cell){
			return;
		}
		var 
			c = this.cell,
			s = c.view.scrollboxNode,
			sr = {
				w: s.clientWidth,
				l: s.scrollLeft,
				t: s.scrollTop,
				h: s.clientHeight
			},
			n = c.getNode(this.rowIndex),
			r = c.view.getRowNode(this.rowIndex),
			rt = this.grid.scroller.findScrollTop(this.rowIndex);
		// place cell within horizontal view
		if(n.offsetLeft + n.offsetWidth > sr.l + sr.w){
			s.scrollLeft = n.offsetLeft + n.offsetWidth - sr.w;
		}else if(n.offsetLeft < sr.l){
			s.scrollLeft = n.offsetLeft;
		}
		// place cell within vertical view
		if(rt + r.offsetHeight > sr.t + sr.h){
			this.grid.setScrollTop(rt + r.offsetHeight - sr.h);
		}else if(rt < sr.t){
			this.grid.setScrollTop(rt);
		}
	},
	styleRow: function(inRow){
		return;
	},
	setFocusIndex: function(inRowIndex, inCellIndex){
		// summary:
		//	focuses the given grid cell
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		this.setFocusCell(this.grid.getCell(inCellIndex), inRowIndex);
	},
	setFocusCell: function(inCell, inRowIndex){
		// summary:
		//	focuses the given grid cell
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(inCell && !this.isFocusCell(inCell, inRowIndex)){
			this.tabbingOut = false;
			this.focusGrid();
			this._focusifyCellNode(false);
			this.cell = inCell;
			this.rowIndex = inRowIndex;
			this._focusifyCellNode(true);
		}
		// even if this cell isFocusCell, the document focus may need to be rejiggered
		// call opera on delay to prevent keypress from altering focus
		if(dojo.isOpera){
			setTimeout(dojo.hitch(this.grid, 'onCellFocus', this.cell, this.rowIndex), 1);
		}else{
			this.grid.onCellFocus(this.cell, this.rowIndex);
		}
	},
	next: function(){
		// summary:
		//	focus next grid cell
		var row=this.rowIndex, col=this.cell.index+1, cc=this.grid.layout.cellCount-1, rc=this.grid.rowCount-1;
		if(col > cc){
			col = 0;
			row++;
		}
		if(row > rc){
			col = cc;
			row = rc;
		}
		this.setFocusIndex(row, col);
	},
	previous: function(){
		// summary:
		//	focus previous grid cell
		var row=(this.rowIndex || 0), col=(this.cell.index || 0) - 1;
		if(col < 0){
			col = this.grid.layout.cellCount-1;
			row--;
		}
		if(row < 0){
			row = 0;
			col = 0;
		}
		this.setFocusIndex(row, col);
	},
	move: function(inRowDelta, inColDelta) {
		// summary:
		//	focus grid cell based on position relative to current focus
		// inRowDelta: int
		// vertical distance from current focus
		// inColDelta: int
		// horizontal distance from current focus
		var
			rc = this.grid.rowCount-1,
			cc = this.grid.layout.cellCount-1,
			r = this.rowIndex,
			i = this.cell.index,
			row = Math.min(rc, Math.max(0, r+inRowDelta)),
			col = Math.min(cc, Math.max(0, i+inColDelta));
		this.setFocusIndex(row, col);
		if(inRowDelta){
			this.grid.updateRow(r);
		}
	},
	previousKey: function(e){
		if(this.isFirstFocusCell()){
			this.tabOut(this.grid.domNode);
		}else{
			dojo.stopEvent(e);
			this.previous();
		}
	},
	nextKey: function(e) {
		if(this.isLastFocusCell()){
			this.tabOut(this.grid.lastFocusNode);
		}else{
			dojo.stopEvent(e);
			this.next();
		}
	},
	tabOut: function(inFocusNode){
		this.tabbingOut = true;
		inFocusNode.focus();
	},
	focusGrid: function(){
		dojox.grid.fire(this.focusView, "focus");
		this._focusifyCellNode(true);
	},
	doFocus: function(e){
		// trap focus only for grid dom node
		if(e && e.target != e.currentTarget){
			return;
		}
		// do not focus for scrolling if grid is about to blur
		if(!this.tabbingOut && this.isNoFocusCell()){
			// establish our virtual-focus, if necessary
			this.setFocusIndex(0, 0);
		}
		this.tabbingOut = false;
	}
});

}

if(!dojo._hasResource['dojox.grid._grid.selection']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._grid.selection'] = true;
dojo.provide('dojox.grid._grid.selection');

dojo.declare("dojox.grid.selection", null, {
	// summary:
	//	Manages row selection for grid. Owned by grid and used internally 
	//	for selection. Override to implement custom selection.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.selected = [];
	},
	multiSelect: true,
	selected: null,
	updating: 0,
	selectedIndex: -1,
	onCanSelect: function(inIndex){
		return this.grid.onCanSelect(inIndex);
	},
	onCanDeselect: function(inIndex){
		return this.grid.onCanDeselect(inIndex);
	},
	onSelected: function(inIndex){
		return this.grid.onSelected(inIndex);
	},
	onDeselected: function(inIndex){
		return this.grid.onDeselected(inIndex);
	},
	//onSetSelected: function(inIndex, inSelect) { };
	onChanging: function(){
	},
	onChanged: function(){
		return this.grid.onSelectionChanged();
	},
	isSelected: function(inIndex){
		return this.selected[inIndex];
	},
	getFirstSelected: function(){
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},
	getNextSelected: function(inPrev){
		for(var i=inPrev+1, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},
	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(i);
			}
		}
		return result;
	},
	getSelectedCount: function(){
		var c = 0;
		for(var i=0; i<this.selected.length; i++){
			if(this.selected[i]){
				c++;
			}
		}
		return c;
	},
	beginUpdate: function(){
		if(this.updating == 0){
			this.onChanging();
		}
		this.updating++;
	},
	endUpdate: function(){
		this.updating--;
		if(this.updating == 0){
			this.onChanged();
		}
	},
	select: function(inIndex){
		this.unselectAll(inIndex);
		this.addToSelection(inIndex);
	},
	addToSelection: function(inIndex){
		inIndex = Number(inIndex);
		if(this.selected[inIndex]){
			this.selectedIndex = inIndex;
		}else{
			if(this.onCanSelect(inIndex) !== false){
				this.selectedIndex = inIndex;
				this.beginUpdate();
				this.selected[inIndex] = true;
				this.grid.onSelected(inIndex);
				//this.onSelected(inIndex);
				//this.onSetSelected(inIndex, true);
				this.endUpdate();
			}
		}
	},
	deselect: function(inIndex){
		inIndex = Number(inIndex);
		if(this.selectedIndex == inIndex){
			this.selectedIndex = -1;
		}
		if(this.selected[inIndex]){
			if(this.onCanDeselect(inIndex) === false){
				return;
			}
			this.beginUpdate();
			delete this.selected[inIndex];
			this.grid.onDeselected(inIndex);
			//this.onDeselected(inIndex);
			//this.onSetSelected(inIndex, false);
			this.endUpdate();
		}
	},
	setSelected: function(inIndex, inSelect){
		this[(inSelect ? 'addToSelection' : 'deselect')](inIndex);
	},
	toggleSelect: function(inIndex){
		this.setSelected(inIndex, !this.selected[inIndex])
	},
	insert: function(inIndex){
		this.selected.splice(inIndex, 0, false);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex++;
		}
	},
	remove: function(inIndex){
		this.selected.splice(inIndex, 1);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex--;
		}
	},
	unselectAll: function(inExcept){
		for(var i in this.selected){
			if((i!=inExcept)&&(this.selected[i]===true)){
				this.deselect(i);
			}
		}
	},
	shiftSelect: function(inFrom, inTo){
		var s = (inFrom >= 0 ? inFrom : inTo), e = inTo;
		if(s > e){
			e = s;
			s = inTo;
		}
		for(var i=s; i<=e; i++){
			this.addToSelection(i);
		}
	},
	clickSelect: function(inIndex, inCtrlKey, inShiftKey){
		this.beginUpdate();
		if(!this.multiSelect){
			this.select(inIndex);
		}else{
			var lastSelected = this.selectedIndex;
			if(!inCtrlKey){
				this.unselectAll(inIndex);
			}
			if(inShiftKey){
				this.shiftSelect(lastSelected, inIndex);
			}else if(inCtrlKey){
				this.toggleSelect(inIndex);
			}else{
				this.addToSelection(inIndex)
			}
		}
		this.endUpdate();
	},
	clickSelectEvent: function(e){
		this.clickSelect(e.rowIndex, e.ctrlKey, e.shiftKey);
	},
	clear: function(){
		this.beginUpdate();
		this.unselectAll();
		this.endUpdate();
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.edit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.edit"] = true;
dojo.provide("dojox.grid._grid.edit");

dojo.declare("dojox.grid.edit", null, {
	// summary:
	//	Controls grid cell editing process. Owned by grid and used internally for editing.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.connections = [];
		if(dojo.isIE){
			this.connections.push(dojo.connect(document.body, "onfocus", dojo.hitch(this, "_boomerangFocus")));
		}
	},
	info: {},
	destroy: function(){
		dojo.forEach(this.connections, function(c){
			dojo.disconnect(c);
		});
	},
	cellFocus: function(inCell, inRowIndex){
		// summary:
		//	invoke editing when cell is focused
		// inCell: cell object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(this.grid.singleClickEdit || this.isEditRow(inRowIndex)){
			// if same row or quick editing, edit
			this.setEditCell(inCell, inRowIndex);
		}else{
			// otherwise, apply any pending row edits
			this.apply();
		}
		// if dynamic or static editing...
		if(this.isEditing() || (inCell && (inCell.editor||0).alwaysOn)){
			// let the editor focus itself as needed
			this._focusEditor(inCell, inRowIndex);
		}
	},
	rowClick: function(e){
		if(this.isEditing() && !this.isEditRow(e.rowIndex)){
			this.apply();
		}
	},
	styleRow: function(inRow){
		if(inRow.index == this.info.rowIndex){
			inRow.customClasses += ' dojoxGrid-row-editing';
		}
	},
	dispatchEvent: function(e){
		var c = e.cell, ed = c && c.editor;
		return ed && ed.dispatchEvent(e.dispatch, e);
	},
	// Editing
	isEditing: function(){
		// summary:
		//	indicates editing state of the grid.
		// returns:
		//	 true if grid is actively editing
		return this.info.rowIndex !== undefined;
	},
	isEditCell: function(inRowIndex, inCellIndex){
		// summary:
		//	indicates if the given cell is being edited.
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		// returns:
		//	 true if given cell is being edited
		return (this.info.rowIndex === inRowIndex) && (this.info.cell.index == inCellIndex);
	},
	isEditRow: function(inRowIndex){
		// summary:
		//	indicates if the given row is being edited.
		// inRowIndex: int
		//	grid row index
		// returns:
		//	 true if given row is being edited
		return this.info.rowIndex === inRowIndex;
	},
	setEditCell: function(inCell, inRowIndex){
		// summary:
		//	set the given cell to be edited
		// inRowIndex: int
		//	grid row index
		// inCell: object
		//	grid cell object
		if(!this.isEditCell(inRowIndex, inCell.index)){
			this.start(inCell, inRowIndex, this.isEditRow(inRowIndex) || inCell.editor);
		}
	},
	_focusEditor: function(inCell, inRowIndex){
		dojox.grid.fire(inCell.editor, "focus", [inRowIndex]);
	},
	focusEditor: function(){
		if(this.isEditing()){
			this._focusEditor(this.info.cell, this.info.rowIndex);
		}
	},
	// implement fix for focus boomerang effect on IE
	_boomerangWindow: 500,
	_shouldCatchBoomerang: function(){
		return this._catchBoomerang > new Date().getTime();
	},
	_boomerangFocus: function(){
		//console.log("_boomerangFocus");
		if(this._shouldCatchBoomerang()){
			// make sure we don't utterly lose focus
			this.grid.focus.focusGrid();
			// let the editor focus itself as needed
			this.focusEditor();
			// only catch once
			this._catchBoomerang = 0;
		}
	},
	_doCatchBoomerang: function(){
		// give ourselves a few ms to boomerang IE focus effects
		if(dojo.isIE){this._catchBoomerang = new Date().getTime() + this._boomerangWindow;}
	},
	// end boomerang fix API
	start: function(inCell, inRowIndex, inEditing){
		this.grid.beginUpdate();
		this.editorApply();
		if(this.isEditing() && !this.isEditRow(inRowIndex)){
			this.applyRowEdit();
			this.grid.updateRow(inRowIndex);
		}
		if(inEditing){
			this.info = { cell: inCell, rowIndex: inRowIndex };
			this.grid.doStartEdit(inCell, inRowIndex); 
			this.grid.updateRow(inRowIndex);
		}else{
			this.info = {};
		}
		this.grid.endUpdate();
		// make sure we don't utterly lose focus
		this.grid.focus.focusGrid();
		// let the editor focus itself as needed
		this._focusEditor(inCell, inRowIndex);
		// give ourselves a few ms to boomerang IE focus effects
		this._doCatchBoomerang();
	},
	_editorDo: function(inMethod){
		var c = this.info.cell
		//c && c.editor && c.editor[inMethod](c, this.info.rowIndex);
		c && c.editor && c.editor[inMethod](this.info.rowIndex);
	},
	editorApply: function(){
		this._editorDo("apply");
	},
	editorCancel: function(){
		this._editorDo("cancel");
	},
	applyCellEdit: function(inValue, inCell, inRowIndex){
		this.grid.doApplyCellEdit(inValue, inRowIndex, inCell.fieldIndex);
	},
	applyRowEdit: function(){
		this.grid.doApplyEdit(this.info.rowIndex);
	},
	apply: function(){
		// summary:
		//	apply a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorApply();
			this.applyRowEdit();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},
	cancel: function(){
		// summary:
		//	cancel a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorCancel();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},
	save: function(inRowIndex, inView){
		// summary:
		//	save the grid editing state
		// inRowIndex: int
		//	grid row index
		// inView: object
		//	grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && (!inView || c.view==inView) && c.editor){
			c.editor.save(c, this.info.rowIndex);
		}
	},
	restore: function(inView, inRowIndex){
		// summary:
		//	restores the grid editing state
		// inRowIndex: int
		//	grid row index
		// inView: object
		//	grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && c.view == inView && c.editor){
			c.editor.restore(c, this.info.rowIndex);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.rowbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.rowbar"] = true;
dojo.provide("dojox.grid._grid.rowbar");


dojo.declare('dojox.GridRowView', dojox.GridView, {
	// summary:
	//	Custom grid view. If used in a grid structure, provides a small selectable region for grid rows.
	defaultWidth: "3em",
	noscroll: true,
	padBorderWidth: 2,
	buildRendering: function(){
		this.inherited('buildRendering', arguments);
		this.scrollboxNode.style.overflow = "hidden";
		this.headerNode.style.visibility = "hidden";
	},	
	getWidth: function(){
		return this.viewWidth || this.defaultWidth;
	},
	buildRowContent: function(inRowIndex, inRowNode){
		var w = this.contentNode.offsetWidth - this.padBorderWidth 
		inRowNode.innerHTML = '<table style="width:' + w + 'px;" role="wairole:presentation"><tr><td class="dojoxGrid-rowbar-inner"></td></tr></table>';
	},
	renderHeader: function(){
	},
	resize: function(){
		this.resizeHeight();
	},
	// styling
	doStyleRowNode: function(inRowIndex, inRowNode){
		var n = [ "dojoxGrid-rowbar" ];
		if(this.grid.rows.isOver(inRowIndex)){
			n.push("dojoxGrid-rowbar-over");
		}
		if(this.grid.selection.isSelected(inRowIndex)){
			n.push("dojoxGrid-rowbar-selected");
		}
		inRowNode.className = n.join(" ");
	},
	// event handlers
	domouseover: function(e){
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(!this.isIntraRowEvent(e)){
			this.grid.onMouseOutRow(e);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.publicEvents"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.publicEvents"] = true;
dojo.provide("dojox.grid._grid.publicEvents");

dojox.grid.publicEvents = {
	// summary:
	//	VirtualGrid mixin that provides default implementations for grid events.
	//	dojo.connect to events to retain default implementation or override them for custom handling.
	
	//cellOverClass: string
	// css class to apply to grid cells over which the cursor is placed.
	cellOverClass: "dojoxGrid-cell-over",
	// top level handlers (more specified handlers below)
	onKeyEvent: function(e){
		this.dispatchKeyEvent(e);
	},
	onContentEvent: function(e){
		this.dispatchContentEvent(e);
	},
	onHeaderEvent: function(e){
		this.dispatchHeaderEvent(e);
	},
	onStyleRow: function(inRow){
		// summary:
		//	Perform row styling on a given row. Called whenever row styling is updated.
		// inRow: object
		// Object containing row state information: selected, true if the row is selcted; over:
		// true of the mouse is over the row; odd: true if the row is odd. Use customClasses and
		// customStyles to control row css classes and styles; both properties are strings.
		with(inRow){
			customClasses += (odd?" dojoxGrid-row-odd":"") + (selected?" dojoxGrid-row-selected":"") + (over?" dojoxGrid-row-over":"");
		}
		this.focus.styleRow(inRow);
		this.edit.styleRow(inRow);
	},
	onKeyDown: function(e){
		// summary:
		// grid key event handler. By default enter begins editing and applies edits, escape cancels and edit,
		// tab, shift-tab, and arrow keys move grid cell focus.
		if(e.altKey || e.ctrlKey || e.metaKey ){
			return;
		}
		switch(e.keyCode){
			case dojo.keys.ESCAPE:
				this.edit.cancel();
				break;
			case dojo.keys.ENTER:
				if (!e.shiftKey) {
					var isEditing = this.edit.isEditing();
					this.edit.apply();
					if(!isEditing){
						this.edit.setEditCell(this.focus.cell, this.focus.rowIndex);
					}
				}
				break;
			case dojo.keys.TAB:
				this.focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
				break;
			case dojo.keys.LEFT_ARROW:
				if(!this.edit.isEditing()){
				this.focus.move(0, -1);
				}
				break;
			case dojo.keys.RIGHT_ARROW:
				if(!this.edit.isEditing()){
					this.focus.move(0, 1);
				}
				break;
			case dojo.keys.UP_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex != 0){
					dojo.stopEvent(e); 
					this.focus.move(-1, 0);
				}
				break;
			case dojo.keys.DOWN_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.model.count){
					dojo.stopEvent(e);
					this.focus.move(1, 0);
				}
				break;
			case dojo.keys.PAGE_UP: 
				if(!this.edit.isEditing() && this.focus.rowIndex != 0){ 
					dojo.stopEvent(e); 
					if(this.focus.rowIndex != this.scroller.firstVisibleRow+1){ 
						this.focus.move(this.scroller.firstVisibleRow-this.focus.rowIndex, 0); 
					}else{ 
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex-1)); 
						this.focus.move(this.scroller.firstVisibleRow-this.scroller.lastVisibleRow+1, 0); 
					} 
				}
				break; 
			case dojo.keys.PAGE_DOWN: 
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.model.count){  
					dojo.stopEvent(e); 
					if(this.focus.rowIndex != this.scroller.lastVisibleRow-1){ 
						this.focus.move(this.scroller.lastVisibleRow-this.focus.rowIndex-1, 0); 
					}else{ 
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex+1)); 
						this.focus.move(this.scroller.lastVisibleRow-this.scroller.firstVisibleRow-1, 0); 
					} 
				} 
				break; 
		}
	},
	onMouseOver: function(e){
		// summary:
		//	event fired when mouse is over the grid.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOver(e) : this.onCellMouseOver(e);
	},
	onMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of the grid.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOut(e) : this.onCellMouseOut(e);
	},
	onMouseOverRow: function(e){
		// summary:
		//	event fired when mouse is over any row (data or header).
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(!this.rows.isOver(e.rowIndex)){
			this.rows.setOverRow(e.rowIndex);
			e.rowIndex == -1 ? this.onHeaderMouseOver(e) : this.onRowMouseOver(e);
		}
	},
	onMouseOutRow: function(e){
		// summary:
		//	event fired when mouse moves out of any row (data or header).
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(this.rows.isOver(-1)){
			this.onHeaderMouseOut(e);
		}else if(!this.rows.isOver(-2)){
			this.rows.setOverRow(-2);
			this.onRowMouseOut(e);
		}
	},
	// cell events
	onCellMouseOver: function(e){
		// summary:
		//	event fired when mouse is over a cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.addClass(e.cellNode, this.cellOverClass);
	},
	onCellMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.removeClass(e.cellNode, this.cellOverClass);
	},
	onCellClick: function(e){
		// summary:
		//	event fired when a cell is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this._click[0] = this._click[1];
		this._click[1] = e;
		if(!this.edit.isEditCell(e.rowIndex, e.cellIndex)){
			this.focus.setFocusCell(e.cell, e.rowIndex);
		}
		this.onRowClick(e);
	},
	onCellDblClick: function(e){
		// summary:
		//	event fired when a cell is double-clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(dojo.isIE){
			this.edit.setEditCell(this._click[1].cell, this._click[1].rowIndex);
		}else if(this._click[0].rowIndex != this._click[1].rowIndex){
			this.edit.setEditCell(this._click[0].cell, this._click[0].rowIndex);
		}else{
			this.edit.setEditCell(e.cell, e.rowIndex);
		}
		this.onRowDblClick(e);
	},
	onCellContextMenu: function(e){
		// summary:
		//	event fired when a cell context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onRowContextMenu(e);
	},
	onCellFocus: function(inCell, inRowIndex){
		// summary:
		//	event fired when a cell receives focus.
		// inCell: object
		//	cell object containing properties of the grid column.
		// inRowIndex: int
		//	index of the grid row
		this.edit.cellFocus(inCell, inRowIndex);
	},
	// row events
	onRowClick: function(e){
		// summary:
		//	event fired when a row is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.edit.rowClick(e);
		this.selection.clickSelectEvent(e);
	},
	onRowDblClick: function(e){
		// summary:
		//	event fired when a row is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over a data row.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a data row.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowContextMenu: function(e){
		// summary:
		//	event fired when a row context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.stopEvent(e);
	},
	// header events
	onHeaderMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over the grid header.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of the grid header.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over a header cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.addClass(e.cellNode, this.cellOverClass);
	},
	onHeaderCellMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a header cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.removeClass(e.cellNode, this.cellOverClass);
	},
	onHeaderClick: function(e){
		// summary:
		//	event fired when the grid header is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellClick: function(e){
		// summary:
		//	event fired when a header cell is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.setSortIndex(e.cell.index);
		this.onHeaderClick(e);
	},
	onHeaderDblClick: function(e){
		// summary:
		//	event fired when the grid header is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellDblClick: function(e){
		// summary:
		//	event fired when a header cell is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onHeaderDblClick(e);
	},
	onHeaderCellContextMenu: function(e){
		// summary:
		//	event fired when a header cell context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onHeaderContextMenu(e);
	},
	onHeaderContextMenu: function(e){
		// summary:
		//	event fired when the grid header context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.stopEvent(e);
	},
	// editing
	onStartEdit: function(inCell, inRowIndex){
		// summary:
		//	event fired when editing is started for a given grid cell
		// inCell: object
		//	cell object containing properties of the grid column.
		// inRowIndex: int
		//	index of the grid row
	},
	onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		// summary:
		//	event fired when editing is applied for a given grid cell
		// inValue: string
		//	value from cell editor
		// inRowIndex: int
		//	index of the grid row
		// inFieldIndex: int
		//	index in the grid's data model
	},
	onCancelEdit: function(inRowIndex){
		// summary:
		//	event fired when editing is cancelled for a given grid cell
		// inRowIndex: int
		//	index of the grid row
	},
	onApplyEdit: function(inRowIndex){
		// summary:
		//	event fired when editing is applied for a given grid row
		// inRowIndex: int
		//	index of the grid row
	},
	onCanSelect: function(inRowIndex){
		// summary:
		//	event to determine if a grid row may be selected
		// inRowIndex: int
		//	index of the grid row
		// returns:
		//	true if the row can be selected
		return true // boolean;
	},
	onCanDeselect: function(inRowIndex){
		// summary:
		//	event to determine if a grid row may be deselected
		// inRowIndex: int
		//	index of the grid row
		// returns:
		//	true if the row can be deselected
		return true // boolean;
	},
	onSelected: function(inRowIndex){
		// summary:
		//	event fired when a grid row is selected
		// inRowIndex: int
		//	index of the grid row
		this.updateRowStyles(inRowIndex);
	},
	onDeselected: function(inRowIndex){
		// summary:
		//	event fired when a grid row is deselected
		// inRowIndex: int
		//	index of the grid row
		this.updateRowStyles(inRowIndex);
	},
	onSelectionChanged: function(){
	}
}

}

if(!dojo._hasResource["dojox.grid.VirtualGrid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.VirtualGrid"] = true;
dojo.provide("dojox.grid.VirtualGrid");












dojo.declare('dojox.VirtualGrid', 
	[ dijit._Widget, dijit._Templated ], 
{
	// summary:
	// 		A grid widget with virtual scrolling, cell editing, complex rows,
	// 		sorting, fixed columns, sizeable columns, etc.
	//
	//	description:
	//		VirtualGrid provides the full set of grid features without any
	//		direct connection to a data store.
	//
	//		The grid exposes a get function for the grid, or optionally
	//		individual columns, to populate cell contents.
	//
	//		The grid is rendered based on its structure, an object describing
	//		column and cell layout.
	//
	//	example:
	//		A quick sample:
	//		
	//		define a get function
	//	|	function get(inRowIndex){ // called in cell context
	//	|		return [this.index, inRowIndex].join(', ');
	//	|	}
	//		
	//		define the grid structure:
	//	|	var structure = [ // array of view objects
	//	|		{ cells: [// array of rows, a row is an array of cells
	//	|			[
	//	|				{ name: "Alpha", width: 6 }, 
	//	|				{ name: "Beta" }, 
	//	|				{ name: "Gamma", get: get }]
	//	|		]}
	//	|	];
	//		
	//	|	<div id="grid" 
	//	|		rowCount="100" get="get" 
	//	|		structure="structure" 
	//	|		dojoType="dojox.VirtualGrid"></div>

	templateString: '<div class="dojoxGrid" hidefocus="hidefocus" role="wairole:grid"><div class="dojoxGrid-master-header" dojoAttachPoint="headerNode"></div><div class="dojoxGrid-master-view" dojoAttachPoint="viewsNode"></div><span dojoAttachPoint="lastFocusNode" tabindex="0"></span></div>',
	// classTag: string
	// css class applied to the grid's domNode
	classTag: 'dojoxGrid',
	get: function(inRowIndex){
		/* summary: Default data getter. 
			description:
				Provides data to display in a grid cell. Called in grid cell context.
				So this.cell.index is the column index.
			inRowIndex: integer
				row for which to provide data
			returns:
				data to display for a given grid cell.
		*/
	},
	// settings
	// rowCount: int
	//	Number of rows to display. 
	rowCount: 5,
	// keepRows: int
	//	Number of rows to keep in the rendering cache.
	keepRows: 75, 
	// rowsPerPage: int
	//	Number of rows to render at a time.
	rowsPerPage: 25,
	// autoWidth: boolean
	//	If autoWidth is true, grid width is automatically set to fit the data.
	autoWidth: false,
	// autoHeight: boolean
	//	If autoHeight is true, grid height is automatically set to fit the data.
	autoHeight: false,
	// autoRender: boolean
	//	If autoRender is true, grid will render itself after initialization.
	autoRender: true,
	// defaultHeight: string
	//	default height of the grid, measured in any valid css unit.
	defaultHeight: '15em',
	// structure: object or string
	//	View layout defintion. Can be set to a layout object, or to the (string) name of a layout object.
	structure: '',
	// elasticView: int
	//	Override defaults and make the indexed grid view elastic, thus filling available horizontal space.
	elasticView: -1,
	// singleClickEdit: boolean
	//	Single-click starts editing. Default is double-click
	singleClickEdit: false,
	// Used to store the last two clicks, to ensure double-clicking occurs based on the intended row
	_click: null,
	// private
	sortInfo: 0,
	themeable: true,
	// initialization
	buildRendering: function(){
		this.inherited(arguments);
		// reset get from blank function (needed for markup parsing) to null, if not changed
		if(this.get == dojox.VirtualGrid.prototype.get){
			this.get = null;
		}
		if(!this.domNode.getAttribute('tabIndex')){
			this.domNode.tabIndex = "0";
		}
		this.createScroller();
		this.createLayout();
		this.createViews();
		this.createManagers();
		dojox.grid.initTextSizePoll();
		this.connect(dojox.grid, "textSizeChanged", "textSizeChanged");
		dojox.grid.funnelEvents(this.domNode, this, 'doKeyEvent', dojox.grid.keyEvents);
		this.connect(this, "onShow", "renderOnIdle");
	},
	postCreate: function(){
		// replace stock styleChanged with one that triggers an update
		this.styleChanged = this._styleChanged;
		this.setStructure(this.structure);
		this._click = [];
	},
	destroy: function(){
		this.domNode.onReveal = null;
		this.domNode.onSizeChange = null;
		this.edit.destroy();
		this.views.destroyViews();
		this.inherited(arguments);
	},
	styleChanged: function(){
		this.setStyledClass(this.domNode, '');
	},
	_styleChanged: function(){
		this.styleChanged();
		this.update();
	},
	textSizeChanged: function(){
		setTimeout(dojo.hitch(this, "_textSizeChanged"), 1);
	},
	_textSizeChanged: function(){
		if(this.domNode){
			this.views.forEach(function(v){
				v.content.update();
			});
			this.render();
		}
	},
	sizeChange: function(){
		dojox.grid.jobs.job(this.id + 'SizeChange', 50, dojo.hitch(this, "update"));
	},
	renderOnIdle: function() {
		setTimeout(dojo.hitch(this, "render"), 1);
	},
	// managers
	createManagers: function(){
		// summary:
		//	create grid managers for various tasks including rows, focus, selection, editing
		
		// row manager
		this.rows = new dojox.grid.rows(this);
		// focus manager
		this.focus = new dojox.grid.focus(this);
		// selection manager
		this.selection = new dojox.grid.selection(this);
		// edit manager
		this.edit = new dojox.grid.edit(this);
	},
	// virtual scroller
	createScroller: function(){
		this.scroller = new dojox.grid.scroller.columns();
		this.scroller.renderRow = dojo.hitch(this, "renderRow");
		this.scroller.removeRow = dojo.hitch(this, "rowRemoved");
	},
	// layout
	createLayout: function(){
		this.layout = new dojox.grid.layout(this);
	},
	// views
	createViews: function(){
		this.views = new dojox.grid.views(this);
		this.views.createView = dojo.hitch(this, "createView");
	},
	createView: function(inClass){
		var c = eval(inClass);
		var view = new c({ grid: this });
		this.viewsNode.appendChild(view.domNode);
		this.headerNode.appendChild(view.headerNode);
		this.views.addView(view);
		return view;
	},
	buildViews: function(){
		for(var i=0, vs; (vs=this.layout.structure[i]); i++){
			this.createView(vs.type || "dojox.GridView").setStructure(vs);
		}
		this.scroller.setContentNodes(this.views.getContentNodes());
	},
	setStructure: function(inStructure){
		// summary:
		//		Install a new structure and rebuild the grid.
		// inStructure: Object
		//		Structure object defines the grid layout and provides various
		//		options for grid views and columns
		//	description:
		//		A grid structure is an array of view objects. A view object can
		//		specify a view type (view class), width, noscroll (boolean flag
		//		for view scrolling), and cells. Cells is an array of objects
		//		corresponding to each grid column. The view cells object is an
		//		array of subrows comprising a single row. Each subrow is an
		//		array of column objects. A column object can have a name,
		//		width, value (default), get function to provide data, styles,
		//		and span attributes (rowSpan, colSpan).

		this.views.destroyViews();
		this.structure = inStructure;
		if((this.structure)&&(dojo.isString(this.structure))){
			this.structure=dojox.grid.getProp(this.structure);
		}
		if(!this.structure){
			this.structure=window["layout"];
		}
		if(!this.structure){
			return;
		}
		this.layout.setStructure(this.structure);
		this._structureChanged();
	},
	_structureChanged: function() {
		this.buildViews();
		if(this.autoRender){
			this.render();
		}
	},
	// sizing
	resize: function(){
		// summary:
		//		Update the grid's rendering dimensions and resize it
		
		// FIXME: If grid is not sized explicitly, sometimes bogus scrollbars 
		// can appear in our container, which may require an extra call to 'resize'
		// to sort out.
		
		// if we have set up everything except the DOM, we cannot resize
		if(!this.domNode.parentNode){
			return;
		}
		// useful measurement
		var padBorder = dojo._getPadBorderExtents(this.domNode);
		// grid height
		if(this.autoHeight){
			this.domNode.style.height = 'auto';
			this.viewsNode.style.height = '';
		}else if(this.flex > 0){
		}else if(this.domNode.clientHeight <= padBorder.h){
			if(this.domNode.parentNode == document.body){
				this.domNode.style.height = this.defaultHeight;
			}else{
				this.fitTo = "parent";
			}
		}
		if(this.fitTo == "parent"){
			var h = dojo._getContentBox(this.domNode.parentNode).h;
			dojo.marginBox(this.domNode, { h: Math.max(0, h) });
		}
		var h = dojo._getContentBox(this.domNode).h;
		if(h == 0){
			// We need to hide the header, since the Grid is essentially hidden.
			this.headerNode.style.display = "none";
		}else{
			// Otherwise, show the header and give it an appropriate height.
			this.headerNode.style.display = "block";
			// header height
			var t = this.views.measureHeader();
			this.headerNode.style.height = t + 'px';
		}
		// content extent
		var l = 1, h = (this.autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
		if(this.autoWidth){
			// grid width set to total width
			this.domNode.style.width = this.views.arrange(l, 0, 0, h) + 'px';
		}else{
			// views fit to our clientWidth
			var w = this.domNode.clientWidth || (this.domNode.offsetWidth - padBorder.w);
			this.views.arrange(l, 0, w, h);
		}
		// virtual scroller height
		this.scroller.windowHeight = h; 
		// default row height (FIXME: use running average(?), remove magic #)
		this.scroller.defaultRowHeight = this.rows.getDefaultHeightPx() + 1;
		this.postresize();
	},
	resizeHeight: function(){
		var t = this.views.measureHeader();
		this.headerNode.style.height = t + 'px';
		// content extent
		var h = (this.autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
		//this.views.arrange(0, 0, 0, h);
		this.views.onEach('setSize', [0, h]);
		this.views.onEach('resizeHeight');
		this.scroller.windowHeight = h; 
	},
	// render 
	render: function(){
		// summary:
		//	Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and 
		// scrolling states, see Update.

		if(!this.domNode){return;}
		//
		this.update = this.defaultUpdate;
		this.scroller.init(this.rowCount, this.keepRows, this.rowsPerPage);
		this.prerender();
		this.setScrollTop(0);
		this.postrender();
	},
	prerender: function(){
		this.views.render();
		this.resize();
	},
	postrender: function(){
		this.postresize();
		this.focus.initFocusView();
		// make rows unselectable
		dojo.setSelectable(this.domNode, false);
	},
	postresize: function(){
		// views are position absolute, so they do not inflate the parent
		if(this.autoHeight){
			this.viewsNode.style.height = this.views.measureContent() + 'px';
		}
	},
	// private, used internally to render rows
	renderRow: function(inRowIndex, inNodes){
		this.views.renderRow(inRowIndex, inNodes);
	},
	// private, used internally to remove rows
	rowRemoved: function(inRowIndex){
		this.views.rowRemoved(inRowIndex);
	},
	invalidated: null,
	updating: false,
	beginUpdate: function(){
		// summary:
		//	Use to make multiple changes to rows while queueing row updating.
		// NOTE: not currently supporting nested begin/endUpdate calls
		this.invalidated = [];
		this.updating = true;
	},
	endUpdate: function(){
		// summary:
		//	Use after calling beginUpdate to render any changes made to rows.
		this.updating = false;
		var i = this.invalidated;
		if(i.all){
			this.update();
		}else if(i.rowCount != undefined){
			this.updateRowCount(i.rowCount);
		}else{
			for(r in i){
				this.updateRow(Number(r));
			}
		}
		this.invalidated = null;
	},
	// update
	defaultUpdate: function(){
		// note: initial update calls render and subsequently this function.
		if(this.updating){
			this.invalidated.all = true;
			return;
		}
		//this.edit.saveState(inRowIndex);
		this.prerender();
		this.scroller.invalidateNodes();
		this.setScrollTop(this.scrollTop);
		this.postrender();
		//this.edit.restoreState(inRowIndex);
	},
	update: function(){
		// summary:
		//	Update the grid, retaining edit and scrolling states.
		this.render();
	},
	updateRow: function(inRowIndex){
		// summary:
		//	Render a single row.
		// inRowIndex: int
		//	index of the row to render
		inRowIndex = Number(inRowIndex);
		if(this.updating){
			this.invalidated[inRowIndex]=true;
			return;
		}
		this.views.updateRow(inRowIndex, this.rows.getHeight(inRowIndex));
		this.scroller.rowHeightChanged(inRowIndex);
	},
	updateRowCount: function(inRowCount){
		//summary: 
		//	Change the number of rows.
		// inRowCount: int
		//	Number of rows in the grid.
		if(this.updating){
			this.invalidated.rowCount = inRowCount;
			return;
		}
		this.rowCount = inRowCount;
		this.scroller.updateRowCount(inRowCount);
		this.setScrollTop(this.scrollTop);
		this.resize();
	},
	updateRowStyles: function(inRowIndex){
		// summary:
		//	Update the styles for a row after it's state has changed.
		
		this.views.updateRowStyles(inRowIndex);
	},
	rowHeightChanged: function(inRowIndex){
		// summary: 
		//	Update grid when the height of a row has changed. Row height is handled automatically as rows
		//	are rendered. Use this function only to update a row's height outside the normal rendering process.
		// inRowIndex: int
		// index of the row that has changed height
		
		this.views.renormalizeRow(inRowIndex);
		this.scroller.rowHeightChanged(inRowIndex);
	},
	// fastScroll: boolean
	//	flag modifies vertical scrolling behavior. Defaults to true but set to false for slower 
	//	scroll performance but more immediate scrolling feedback
	fastScroll: true,
	delayScroll: false,
	// scrollRedrawThreshold: int
	//	pixel distance a user must scroll vertically to trigger grid scrolling.
	scrollRedrawThreshold: (dojo.isIE ? 100 : 50),
	// scroll methods
	scrollTo: function(inTop){
		// summary:
		//	Vertically scroll the grid to a given pixel position
		// inTop: int
		//	vertical position of the grid in pixels
		if(!this.fastScroll){
			this.setScrollTop(inTop);
			return;
		}
		var delta = Math.abs(this.lastScrollTop - inTop);
		this.lastScrollTop = inTop;
		if(delta > this.scrollRedrawThreshold || this.delayScroll){
			this.delayScroll = true;
			this.scrollTop = inTop;
			this.views.setScrollTop(inTop);
			dojox.grid.jobs.job('dojoxGrid-scroll', 200, dojo.hitch(this, "finishScrollJob"));
		}else{
			this.setScrollTop(inTop);
		}
	},
	finishScrollJob: function(){
		this.delayScroll = false;
		this.setScrollTop(this.scrollTop);
	},
	setScrollTop: function(inTop){
		this.scrollTop = this.views.setScrollTop(inTop);
		this.scroller.scroll(this.scrollTop);
	},
	scrollToRow: function(inRowIndex){
		// summary:
		//	Scroll the grid to a specific row.
		// inRowIndex: int
		// grid row index
		this.setScrollTop(this.scroller.findScrollTop(inRowIndex) + 1);
	},
	// styling (private, used internally to style individual parts of a row)
	styleRowNode: function(inRowIndex, inRowNode){
		if(inRowNode){
			this.rows.styleRowNode(inRowIndex, inRowNode);
		}
	},
	// cells
	getCell: function(inIndex){
		// summary:
		//	retrieves the cell object for a given grid column.
		// inIndex: int
		// grid column index of cell to retrieve
		// returns:
		//	a grid cell
		return this.layout.cells[inIndex];
	},
	setCellWidth: function(inIndex, inUnitWidth) {
		this.getCell(inIndex).unitWidth = inUnitWidth;
	},
	getCellName: function(inCell){
		return "Cell " + inCell.index;
	},
	// sorting
	canSort: function(inSortInfo){
		// summary:
		//	determines if the grid can be sorted
		// inSortInfo: int
		//	Sort information, 1-based index of column on which to sort, positive for an ascending sort
		// and negative for a descending sort
		// returns:
		//	true if grid can be sorted on the given column in the given direction
	},
	sort: function(){
	},
	getSortAsc: function(inSortInfo){
		// summary:
		//	returns true if grid is sorted in an ascending direction.
		inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
		return Boolean(inSortInfo > 0);
	},
	getSortIndex: function(inSortInfo){
		// summary:
		//	returns the index of the column on which the grid is sorted
		inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
		return Math.abs(inSortInfo) - 1;
	},
	setSortIndex: function(inIndex, inAsc){
		// summary:
		// Sort the grid on a column in a specified direction
		// inIndex: int
		// Column index on which to sort.
		// inAsc: boolean
		// If true, sort the grid in ascending order, otherwise in descending order
		var si = inIndex +1;
		if(inAsc != undefined){
			si *= (inAsc ? 1 : -1);
		} else if(this.getSortIndex() == inIndex){
			si = -this.sortInfo;
		}
		this.setSortInfo(si);
	},
	setSortInfo: function(inSortInfo){
		if(this.canSort(inSortInfo)){
			this.sortInfo = inSortInfo;
			this.sort();
			this.update();
		}
	},
	// DOM event handler
	doKeyEvent: function(e){
		e.dispatch = 'do' + e.type;
		this.onKeyEvent(e);
	},
	// event dispatch
	//: protected
	_dispatch: function(m, e){
		if(m in this){
			return this[m](e);
		}
	},
	dispatchKeyEvent: function(e){
		this._dispatch(e.dispatch, e);
	},
	dispatchContentEvent: function(e){
		this.edit.dispatchEvent(e) || e.sourceView.dispatchContentEvent(e) || this._dispatch(e.dispatch, e);
	},
	dispatchHeaderEvent: function(e){
		e.sourceView.dispatchHeaderEvent(e) || this._dispatch('doheader' + e.type, e);
	},
	dokeydown: function(e){
		this.onKeyDown(e);
	},
	doclick: function(e){
		if(e.cellNode){
			this.onCellClick(e);
		}else{
			this.onRowClick(e);
		}
	},
	dodblclick: function(e){
		if(e.cellNode){
			this.onCellDblClick(e);
		}else{
			this.onRowDblClick(e);
		}
	},
	docontextmenu: function(e){
		if(e.cellNode){
			this.onCellContextMenu(e);
		}else{
			this.onRowContextMenu(e);
		}
	},
	doheaderclick: function(e){
		if(e.cellNode){
			this.onHeaderCellClick(e);
		}else{
			this.onHeaderClick(e);
		}
	},
	doheaderdblclick: function(e){
		if(e.cellNode){
			this.onHeaderCellDblClick(e);
		}else{
			this.onHeaderDblClick(e);
		}
	},
	doheadercontextmenu: function(e){
		if(e.cellNode){
			this.onHeaderCellContextMenu(e);
		}else{
			this.onHeaderContextMenu(e);
		}
	},
	// override to modify editing process
	doStartEdit: function(inCell, inRowIndex){
		this.onStartEdit(inCell, inRowIndex);
	},
	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
	},
	doCancelEdit: function(inRowIndex){
		this.onCancelEdit(inRowIndex);
	},
	doApplyEdit: function(inRowIndex){
		this.onApplyEdit(inRowIndex);
	},
	// row editing
	addRow: function(){
		// summary:
		//	add a row to the grid.
		this.updateRowCount(this.rowCount+1);
	},
	removeSelectedRows: function(){
		// summary:
		//	remove the selected rows from the grid.
		this.updateRowCount(Math.max(0, this.rowCount - this.selection.getSelected().length));
		this.selection.clear();
	}
});

dojo.mixin(dojox.VirtualGrid.prototype, dojox.grid.publicEvents);

}

if(!dojo._hasResource["dojox.grid._data.fields"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._data.fields"] = true;
dojo.provide("dojox.grid._data.fields");

dojo.declare("dojox.grid.data.Mixer", null, {
	// summary:
	//	basic collection class that provides a default value for items
	
	constructor: function(){
		this.defaultValue = {};
		this.values = [];
	},
	count: function(){
		return this.values.length;
	},
	clear: function(){
		this.values = [];
	},
	build: function(inIndex){
		var result = dojo.mixin({owner: this}, this.defaultValue);
		result.key = inIndex;
		this.values[inIndex] = result;
		return result;
	},
	getDefault: function(){
		return this.defaultValue;
	},
	setDefault: function(inField /*[, inField2, ... inFieldN] */){
		for(var i=0, a; (a = arguments[i]); i++){
			dojo.mixin(this.defaultValue, a);
		}
	},
	get: function(inIndex){
		return this.values[inIndex] || this.build(inIndex);
	},
	_set: function(inIndex, inField /*[, inField2, ... inFieldN] */){
		// each field argument can be a single field object of an array of field objects
		var v = this.get(inIndex);
		for(var i=1; i<arguments.length; i++){
			dojo.mixin(v, arguments[i]);
		}
		this.values[inIndex] = v;
	},
	set: function(/* inIndex, inField [, inField2, ... inFieldN] | inArray */){
		if(arguments.length < 1){
			return;
		}
		var a = arguments[0];
		if(!dojo.isArray(a)){
			this._set.apply(this, arguments);
		}else{
			if(a.length && a[0]["default"]){
				this.setDefault(a.shift());
			}
			for(var i=0, l=a.length; i<l; i++){
				this._set(i, a[i]);
			}
		}
	},
	insert: function(inIndex, inProps){
		if (inIndex >= this.values.length){
			this.values[inIndex] = inProps;
		}else{
			this.values.splice(inIndex, 0, inProps);
		}
	},
	remove: function(inIndex){
		this.values.splice(inIndex, 1);
	},
	swap: function(inIndexA, inIndexB){
		dojox.grid.arraySwap(this.values, inIndexA, inIndexB);
	},
	move: function(inFromIndex, inToIndex){
		dojox.grid.arrayMove(this.values, inFromIndex, inToIndex);
	}
});

dojox.grid.data.compare = function(a, b){
	return (a > b ? 1 : (a == b ? 0 : -1));
}

dojo.declare('dojox.grid.data.Field', null, {
	constructor: function(inName){
		this.name = inName;
		this.compare = dojox.grid.data.compare;
	},
	na: dojox.grid.na
});

dojo.declare('dojox.grid.data.Fields', dojox.grid.data.Mixer, {
	constructor: function(inFieldClass){
		var fieldClass = inFieldClass ? inFieldClass : dojox.grid.data.Field;
		this.defaultValue = new fieldClass();
	},
	indexOf: function(inKey){
		for(var i=0; i<this.values.length; i++){
			var v = this.values[i];
			if(v && v.key == inKey){return i;}
		}
		return -1;
	}
});

}

if(!dojo._hasResource['dojox.grid._data.model']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._data.model'] = true;
dojo.provide('dojox.grid._data.model');


dojo.declare("dojox.grid.data.Model", null, {
	// summary:
	//	Base abstract grid data model.
	//	Makes no assumptions about the structure of grid data.
	constructor: function(inFields, inData){
		this.observers = [];
		this.fields = new dojox.grid.data.Fields();
		if(inFields){
			this.fields.set(inFields);
		}
		this.setData(inData);
	},
	count: 0,
	updating: 0,
	// observers 
	observer: function(inObserver, inPrefix){
		this.observers.push({o: inObserver, p: inPrefix||'model' });
	},
	notObserver: function(inObserver){
		for(var i=0, m, o; (o=this.observers[i]); i++){
			if(o.o==inObserver){
				this.observers.splice(i, 1);
				return;
			}
		}
	},
	notify: function(inMsg, inArgs){
		if(!this.isUpdating()){
			var a = inArgs || [];
			for(var i=0, m, o; (o=this.observers[i]); i++){
				m = o.p + inMsg, o = o.o;
				(m in o)&&(o[m].apply(o, a));
			}
		}
	},
	// updates
	clear: function(){
		this.fields.clear();
		this.clearData();
	},
	beginUpdate: function(){
		this.updating++;
	},
	endUpdate: function(){
		if(this.updating){
			this.updating--;
		}
		/*if(this.updating){
			if(!(--this.updating)){
				this.change();
			}
		}
		}*/
	},
	isUpdating: function(){
		return Boolean(this.updating);
	},
	// data
	clearData: function(){
		this.setData(null);
	},
	// observer events
	change: function(){
		this.notify("Change", arguments);
	},
	insertion: function(/* index */){
		this.notify("Insertion", arguments);
		this.notify("Change", arguments);
	},
	removal: function(/* keys */){
		this.notify("Removal", arguments);
		this.notify("Change", arguments);
	},
	// insert
	insert: function(inData /*, index */){
		if(!this._insert.apply(this, arguments)){
			return false;
		}
		this.insertion.apply(this, dojo._toArray(arguments, 1));
		return true;
	},
	// remove
	remove: function(inData /*, index */){
		if(!this._remove.apply(this, arguments)){
			return false;
		}
		this.removal.apply(this, arguments);
		return true;
	},
	// sort
	canSort: function(/* (+|-)column_index+1, ... */){
		return this.sort != null;
	},
	makeComparator: function(inIndices){
		var idx, col, field, result = null;
		for(var i=inIndices.length-1; i>=0; i--){
			idx = inIndices[i];
			col = Math.abs(idx) - 1;
			if(col >= 0){
				field = this.fields.get(col);
				result = this.generateComparator(field.compare, field.key, idx > 0, result);
			}
		}
		return result;
	},
	sort: null,
	dummy: 0
});

dojo.declare("dojox.grid.data.Rows", dojox.grid.data.Model, {
	// observer events
	allChange: function(){
		this.notify("AllChange", arguments);
		this.notify("Change", arguments);
	},
	rowChange: function(){
		this.notify("RowChange", arguments);
	},
	datumChange: function(){
		this.notify("DatumChange", arguments);
	},
	// copyRow: function(inRowIndex); // abstract
	// update
	beginModifyRow: function(inRowIndex){
		if(!this.cache[inRowIndex]){
			this.cache[inRowIndex] = this.copyRow(inRowIndex);
		}
	},
	endModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			var data = this.getRow(inRowIndex);
			if(!dojox.grid.arrayCompare(cache, data)){
				this.update(cache, data, inRowIndex);
			}
			delete this.cache[inRowIndex];
		}
	},
	cancelModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			this.setRow(cache, inRowIndex);
			delete this.cache[inRowIndex];
		}
	},
	generateComparator: function(inCompare, inField, inTrueForAscend, inSubCompare){
		return function(a, b){
			var ineq = inCompare(a[inField], b[inField]);
			return ineq ? (inTrueForAscend ? ineq : -ineq) : inSubCompare && inSubCompare(a, b);
		}
	}
});

dojo.declare("dojox.grid.data.Table", dojox.grid.data.Rows, {
	// summary:
	//	Basic grid data model for static data in the form of an array of rows
	//	that are arrays of cell data
	constructor: function(){
		this.cache = [];
	},
	colCount: 0, // tables introduce cols
	data: null,
	cache: null,
	// morphology
	measure: function(){
		this.count = this.getRowCount();
		this.colCount = this.getColCount();
		this.allChange();
		//this.notify("Measure");
	},
	getRowCount: function(){
		return (this.data ? this.data.length : 0);
	},
	getColCount: function(){
		return (this.data && this.data.length ? this.data[0].length : this.fields.count());
	},
	badIndex: function(inCaller, inDescriptor){
		console.debug('dojox.grid.data.Table: badIndex');
	},
	isGoodIndex: function(inRowIndex, inColIndex){
		return (inRowIndex >= 0 && inRowIndex < this.count && (arguments.length < 2 || (inColIndex >= 0 && inColIndex < this.colCount)));
	},
	// access
	getRow: function(inRowIndex){
		return this.data[inRowIndex];
	},
	copyRow: function(inRowIndex){
		return this.getRow(inRowIndex).slice(0);
	},
	getDatum: function(inRowIndex, inColIndex){
		return this.data[inRowIndex][inColIndex];
	},
	get: function(){
		throw('Plain "get" no longer supported. Use "getRow" or "getDatum".');
	},
	setData: function(inData){
		this.data = (inData || []);
		this.allChange();
	},
	setRow: function(inData, inRowIndex){
		this.data[inRowIndex] = inData;
		this.rowChange(inData, inRowIndex);
		this.change();
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		this.data[inRowIndex][inColIndex] = inDatum;
		this.datumChange(inDatum, inRowIndex, inColIndex);
	},
	set: function(){
		throw('Plain "set" no longer supported. Use "setData", "setRow", or "setDatum".');
	},
	setRows: function(inData, inRowIndex){
		for(var i=0, l=inData.length, r=inRowIndex; i<l; i++, r++){
			this.setRow(inData[i], r);
		}
	},
	// update
	update: function(inOldData, inNewData, inRowIndex){
		//delete this.cache[inRowIndex];	
		//this.setRow(inNewData, inRowIndex);
		return true;
	},
	// insert
	_insert: function(inData, inRowIndex){
		dojox.grid.arrayInsert(this.data, inRowIndex, inData);
		this.count++;
		return true;
	},
	// remove
	_remove: function(inKeys){
		for(var i=inKeys.length-1; i>=0; i--){
			dojox.grid.arrayRemove(this.data, inKeys[i]);
		}
		this.count -= inKeys.length;
		return true;
	},
	// sort
	sort: function(/* (+|-)column_index+1, ... */){
		this.data.sort(this.makeComparator(arguments));
	},
	swap: function(inIndexA, inIndexB){
		dojox.grid.arraySwap(this.data, inIndexA, inIndexB);
		this.rowChange(this.getRow(inIndexA), inIndexA);
		this.rowChange(this.getRow(inIndexB), inIndexB);
		this.change();
	},
	dummy: 0
});

dojo.declare("dojox.grid.data.Objects", dojox.grid.data.Table, {
	constructor: function(inFields, inData, inKey){
		if(!inFields){
			this.autoAssignFields();
		}
	},
	autoAssignFields: function(){
		var d = this.data[0], i = 0;
		for(var f in d){
			this.fields.get(i++).key = f;
		}
	},
	getDatum: function(inRowIndex, inColIndex){
		return this.data[inRowIndex][this.fields.get(inColIndex).key];
	}
});

dojo.declare("dojox.grid.data.Dynamic", dojox.grid.data.Table, {
	// summary:
	//	Grid data model for dynamic data such as data retrieved from a server.
	//	Retrieves data automatically when requested and provides notification when data is received
	constructor: function(){
		this.page = [];
		this.pages = [];
	},
	page: null,
	pages: null,
	rowsPerPage: 100,
	requests: 0,
	bop: -1,
	eop: -1,
	// data
	clearData: function(){
		this.pages = [];
		this.bop = this.eop = -1;
		this.setData([]);
	},
	getRowCount: function(){
		return this.count;
	},
	getColCount: function(){
		return this.fields.count();
	},
	setRowCount: function(inCount){
		this.count = inCount;
		this.change();
	},
	// paging
	requestsPending: function(inBoolean){
	},
	rowToPage: function(inRowIndex){
		return (this.rowsPerPage ? Math.floor(inRowIndex / this.rowsPerPage) : inRowIndex);
	},
	pageToRow: function(inPageIndex){
		return (this.rowsPerPage ? this.rowsPerPage * inPageIndex : inPageIndex);
	},
	requestRows: function(inRowIndex, inCount){
		// summary:
		//		stub. Fill in to perform actual data row fetching logic. The
		//		returning logic must provide the data back to the system via
		//		setRow
	},
	rowsProvided: function(inRowIndex, inCount){
		this.requests--;
		if(this.requests == 0){
			this.requestsPending(false);
		}
	},
	requestPage: function(inPageIndex){
		var row = this.pageToRow(inPageIndex);
		var count = Math.min(this.rowsPerPage, this.count - row);
		if(count > 0){
			this.requests++;
			this.requestsPending(true);
			setTimeout(dojo.hitch(this, "requestRows", row, count), 1);
			//this.requestRows(row, count);
		}
	},
	needPage: function(inPageIndex){
		if(!this.pages[inPageIndex]){
			this.pages[inPageIndex] = true;
			this.requestPage(inPageIndex);
		}
	},
	preparePage: function(inRowIndex, inColIndex){
		if(inRowIndex < this.bop || inRowIndex >= this.eop){
			var pageIndex = this.rowToPage(inRowIndex);
			this.needPage(pageIndex);
			this.bop = pageIndex * this.rowsPerPage;
			this.eop = this.bop + (this.rowsPerPage || this.count);
		}
	},
	isRowLoaded: function(inRowIndex){
		return Boolean(this.data[inRowIndex]);
	},
	// removal
	removePages: function(inRowIndexes){
		for(var i=0, r; ((r=inRowIndexes[i]) != undefined); i++){
			this.pages[this.rowToPage(r)] = false;
		}
		this.bop = this.eop =-1;
	},
	remove: function(inRowIndexes){
		this.removePages(inRowIndexes);
		dojox.grid.data.Table.prototype.remove.apply(this, arguments);
	},
	// access
	getRow: function(inRowIndex){
		var row = this.data[inRowIndex];
		if(!row){
			this.preparePage(inRowIndex);
		}
		return row;
	},
	getDatum: function(inRowIndex, inColIndex){
		var row = this.getRow(inRowIndex);
		return (row ? row[inColIndex] : this.fields.get(inColIndex).na);
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		var row = this.getRow(inRowIndex);
		if(row){
			row[inColIndex] = inDatum;
			this.datumChange(inDatum, inRowIndex, inColIndex);
		}else{
			console.debug('[' + this.declaredClass + '] dojox.grid.data.dynamic.set: cannot set data on an non-loaded row');
		}
	},
	// sort
	canSort: function(){
		return false;
	}
});

// FIXME: deprecated: (included for backward compatibility only)
dojox.grid.data.table = dojox.grid.data.Table;
dojox.grid.data.dynamic = dojox.grid.data.Dyanamic;

// we treat dojo.data stores as dynamic stores because no matter how they got
// here, they should always fill that contract
dojo.declare("dojox.grid.data.DojoData", dojox.grid.data.Dynamic, {
	//	summary:
	//		A grid data model for dynamic data retreived from a store which
	//		implements the dojo.data API set. Retrieves data automatically when
	//		requested and provides notification when data is received
	//	description:
	//		This store subclasses the Dynamic grid data object in order to
	//		provide paginated data access support, notification and view
	//		updates for stores which support those features, and simple
	//		field/column mapping for all dojo.data stores.
	constructor: function(inFields, inData, args){
		this.count = 1;
		this._rowIdentities = {};
		this._currentlyProcessing = [];
		if(args){
			dojo.mixin(this, args);
		}
		if(this.store){
			var f = this.store.getFeatures();
			this._canNotify = f['dojo.data.api.Notification'];
			this._canWrite = f['dojo.data.api.Write'];
			this._canIdentify = f['dojo.data.api.Identity'];
			if(this._canNotify){
				dojo.connect(this.store, "onSet", this, "_storeDatumChange");
				dojo.connect(this.store, "onDelete", this, "_storeDatumDelete");
				dojo.connect(this.store, "onNew", this, "_storeDatumNew");
			}
			if(this._canWrite) {
				dojo.connect(this.store, "revert", this, "refresh");
			}
		}
	},
	markupFactory: function(args, node){
		return new dojox.grid.data.DojoData(null, null, args);
	},
	query: { name: "*" }, // default, stupid query
	store: null,
	_currentlyProcessing: null,
	_canNotify: false,
	_canWrite: false,
	_canIdentify: false,
	_rowIdentities: {},
	clientSort: false,
	sortFields: null,
	queryOptions: null,

	// data
	setData: function(inData){
		this.store = inData;
		this.data = [];
		this.allChange();
	},
	setRowCount: function(inCount){
		//console.debug("inCount:", inCount);
		this.count = inCount;
		this.allChange();
	},
	beginReturn: function(inCount){
		if(this.count != inCount){
			// this.setRowCount(0);
			// this.clear();
			// console.debug(this.count, inCount);
			this.setRowCount(inCount);
		}
	},
	_setupFields: function(dataItem){
		// abort if we already have setup fields
		if(this.fields._nameMaps){
			return;
		}
		// set up field/index mappings
		var m = {};
		//console.debug("setting up fields", m);
		var fields = dojo.map(this.store.getAttributes(dataItem),
			function(item, idx){ 
				m[item] = idx;
				m[idx+".idx"] = item;
				// name == display name, key = property name
				return { name: item, key: item };
			},
			this
		);
		this.fields._nameMaps = m;
		// console.debug("new fields:", fields);
		this.fields.set(fields);
		this.notify("FieldsChange");
	},
	_getRowFromItem: function(item){
		// gets us the row object (and row index) of an item
	},
	_createRow: function(item){
		var row = {}; 
		row.__dojo_data_item = item;
		dojo.forEach(this.fields.values, function(a){
			value = this.store.getValue(item, a.name);
			row[a.name] = (value === undefined || value === null)?"":value;
		}, this);
		return row;
	},
	processRows: function(items, request){
		// console.debug(arguments);
		if(!items || items.length == 0){ return; }
		this._setupFields(items[0]);
		dojo.forEach(items, function(item, idx){
			var row = this._createRow(item);
			this._setRowId(item, request.start, idx);
			this.setRow(row, request.start+idx);
		}, this);
		// FIXME: 
		//	Q: scott, steve, how the hell do we actually get this to update
		//		the visible UI for these rows?
		//	A: the goal is that Grid automatically updates to reflect changes
		//		in model. In this case, setRow -> rowChanged -> (observed by) Grid -> modelRowChange -> updateRow
	},
	// request data 
	requestRows: function(inRowIndex, inCount){
		var row  = inRowIndex || 0;
		var params = { 
			start: row,
			count: this.rowsPerPage,
			query: this.query,
			sort: this.sortFields,
			queryOptions: this.queryOptions,
			onBegin: dojo.hitch(this, "beginReturn"),
			onComplete: dojo.hitch(this, "processRows"), // add to deferred?
			onError: dojo.hitch(this, "processError")
		};
		this.store.fetch(params);
	},
	getDatum: function(inRowIndex, inColIndex){
		//console.debug("getDatum", inRowIndex, inColIndex);
		var row = this.getRow(inRowIndex);
		var field = this.fields.values[inColIndex];
		return row && field ? row[field.name] : field ? field.na : '?';
		//var idx = row && this.fields._nameMaps[inColIndex+".idx"];
		//return (row ? row[idx] : this.fields.get(inColIndex).na);
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		var n = this.fields._nameMaps[inColIndex+".idx"];
		// console.debug("setDatum:", "n:"+n, inDatum, inRowIndex, inColIndex);
		if(n){
			this.data[inRowIndex][n] = inDatum;
			this.datumChange(inDatum, inRowIndex, inColIndex);
		}
	},
	// modification, update and store eventing
	copyRow: function(inRowIndex){
		var row = {};
		var backstop = {};
		var src = this.getRow(inRowIndex);
		for(var x in src){
			if(src[x] != backstop[x]){
				row[x] = src[x];
			}
		}
		return row;
	},
	_attrCompare: function(cache, data){
		dojo.forEach(this.fields.values, function(a){
			if(cache[a.name] != data[a.name]){ return false; }
		}, this);
		return true;
	},
	endModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			var data = this.getRow(inRowIndex);
			if(!this._attrCompare(cache, data)){
				this.update(cache, data, inRowIndex);
			}
			delete this.cache[inRowIndex];
		}
	},
	cancelModifyRow: function(inRowIndex){
		// console.debug("cancelModifyRow", arguments);
		var cache = this.cache[inRowIndex];
		if(cache){
			this.setRow(cache, inRowIndex);
			delete this.cache[inRowIndex];
		}
	},
	_setRowId: function(item, offset, idx){
		// FIXME: where else do we need to keep this in sync?
		//Handle stores that implement identity and try to handle those that do not.
		if (this._canIdentify) {
			this._rowIdentities[this.store.getIdentity(item)] = {rowId: offset+idx, item: item};
		}else{
			var identity = dojo.toJson(this.query) + ":start:" + offset + ":idx:" + idx + ":sort:" + dojo.toJson(this.sortFields);
			this._rowIdentities[identity] = {rowId: offset+idx, item: item};
		}
	},
	_getRowId: function(item, isNotItem){
		//	summary:
		//		Function determine the row index for a particular item
		//	item:
		//		The store item to examine to determine row index.
		//	isNotItem:
		//		Boolean flag to indicate if the item passed is a store item or not.
		var rowId = null;
		//Handle identity and nonidentity capable stores.
		if(this._canIdentify && !isNotItem){
			rowId = this._rowIdentities[this.store.getIdentity(item)].rowId;
		}else{
			//Not efficient, but without identity support, 
			//not a better way to do it.  Basically, do our best to locate it
			//This may or may not work, but best we can do here.
			var id;
			for(id in this._rowIdentities){
				if(this._rowIdentities[id].item === item){
					rowId = this._rowIdentities[id].rowId;
					break;
				}
			}
		}
		return rowId;
	},
	_storeDatumChange: function(item, attr, oldVal, newVal){
		// the store has changed some data under us, need to update the display
		var rowId = this._getRowId(item);
		var row = this.getRow(rowId);
		if(row){
			row[attr] = newVal;
			var colId = this.fields._nameMaps[attr];
			this.notify("DatumChange", [ newVal, rowId, colId ]);
		}
	},
	_storeDatumDelete: function(item){
		if(dojo.indexOf(this._currentlyProcessing, item) != -1)
			return;
		// the store has deleted some item under us, need to remove that item from
		// the view if possible.  It may be the deleted item isn't even in the grid.
		var rowId = this._getRowId(item, true);
		if(rowId != null){
			this._removeItems([rowId]);
		}
	},
	_storeDatumNew: function(item){
		if(this._disableNew)
			return;
		// the store has added some item under us, need to add it to the view.
		this._insertItem(item, this.data.length);
	},
	insert: function(item, index){
		// Push the given item back to the store
		this._disableNew = true;
		var i = this.store.newItem(item);
		this._disableNew = false;
		this._insertItem(i, index);
	},
	_insertItem: function(storeItem, index){
		var row = this._createRow(storeItem);
		this._setRowId(storeItem, 0, index);
		dojox.grid.data.Dynamic.prototype.insert.apply(this, [row, index]);
	},
	datumChange: function(value, rowIdx, colIdx){
		if(this._canWrite){
			// we're chaning some data, which means we need to write back
			var row = this.getRow(rowIdx);
			var field = this.fields._nameMaps[colIdx+".idx"];
			this.store.setValue(row.__dojo_data_item, field, value);
			// we don't need to call DatumChange, an eventing store will tell
			// us about the row change events
		}else{
			// we can't write back, so just go ahead and change our local copy
			// of the data
			this.notify("DatumChange", arguments);
		}
	},
	insertion: function(/* index */){
		console.debug("Insertion", arguments);
		this.notify("Insertion", arguments);
		this.notify("Change", arguments);
	},
	removal: function(/* keys */){
		console.debug("Removal", arguments);
		this.notify("Removal", arguments);
		this.notify("Change", arguments);
	},
	remove: function(inRowIndexes){
		//	summary:
		//		Function to remove a set of items from the store based on the row index.
		//	inRowIndexes:
		//		An array of row indexes from the grid to remove from the store.
		/* Call delete on the store */ 
		for(var i=inRowIndexes.length-1; i>=0; i--){
			// Need to find the item, then remove each from the data store
			var item = this.data[inRowIndexes[i]].__dojo_data_item;
			this._currentlyProcessing.push(item);
			this.store.deleteItem(item);
		}
		/* Remove from internal data structure and the view */
		this._removeItems(inRowIndexes);
		this._currentlyProcessing = [];
	},
	_removeItems: function(inRowIndexes /*array*/){
		//	summary:
		//		Function to remove a set of items from the store based on the row index.
		//	inRowIndexes:
		//		An array of row indexes from the grid to remove from the store.
		dojox.grid.data.Dynamic.prototype.remove.apply(this, arguments);
		// Rebuild _rowIdentities
		this._rowIdentities = {};
		for (var i = 0; i < this.data.length; i++){
			this._setRowId(this.data[i].__dojo_data_item, 0, i);
		}
	},
	canSort: function(){
		// Q: Return true and re-issue the queries?
		// A: Return true only. Re-issue the query in 'sort'.
		// Note, above are original comments :)
		return true;
	},
	sort: function(colIndex){
		var col = Math.abs(colIndex) - 1;
		this.sortFields = [{'attribute': this.fields.values[col].name, 'descending': (colIndex>0)}];
		
		// Since we're relying on the data store to sort, we have to refresh our data.
		this.refresh();
	},
	refresh: function(){
		//	summary:
		//		Function to cause the model to re-query the store and rebuild the current viewport.
		this.clearData(true);
		this.requestRows();
	},
	clearData: function(/* boolean */ keepStore){
		this._rowIdentities = {};
		this.pages = [];
		this.bop = this.eop = -1;
		this.count = 0;
		this.setData((keepStore?this.store:[]));
	},
	processError: function(error, request){
		//	summary:
		//		Hook function to trap error messages from the store and emit them.  
		//		Intended for connecting to and handling the error object or at least reporting it.
		//
		//	error:
		//		The error object returned by the store when a problem occurred.
		//	request:
		//		The request object that caused the error.
		console.log(error);
	}
});

}

if(!dojo._hasResource["dojox.grid._data.editors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._data.editors"] = true;
dojo.provide("dojox.grid._data.editors");
dojo.provide("dojox.grid.editors");

dojo.declare("dojox.grid.editors.Base", null, {
	// summary:
	//	base grid editor class. Other grid editors should inherited from this class.
	constructor: function(inCell){
		this.cell = inCell;
	},
	//private
	_valueProp: "value",
	_formatPending: false,
	format: function(inDatum, inRowIndex){
		// summary:
		//	formats the cell for editing
		// inDatum: anything
		//	cell data to edit
		// inRowIndex: int
		//	grid row index
		// returns: string of html to place in grid cell
	},
	//protected
	needFormatNode: function(inDatum, inRowIndex){
		this._formatPending = true;
		dojox.grid.whenIdle(this, "_formatNode", inDatum, inRowIndex);
	},
	cancelFormatNode: function(){
		this._formatPending = false;
	},
	//private
	_formatNode: function(inDatum, inRowIndex){
		if(this._formatPending){
			this._formatPending = false;
			// make cell selectable
			dojo.setSelectable(this.cell.grid.domNode, true);
			this.formatNode(this.getNode(inRowIndex), inDatum, inRowIndex);
		}
	},
	//protected
	getNode: function(inRowIndex){
		return (this.cell.getNode(inRowIndex) || 0).firstChild || 0;
	},
	formatNode: function(inNode, inDatum, inRowIndex){
		// summary:
		//	format the editing dom node. Use when editor is a widget.
		// inNode: dom node
		// dom node for the editor
		// inDatum: anything
		//	cell data to edit
		// inRowIndex: int
		//	grid row index
		if(dojo.isIE){
			// IE sux bad
			dojox.grid.whenIdle(this, "focus", inRowIndex, inNode);
		}else{
			this.focus(inRowIndex, inNode);
		}
	},
	dispatchEvent: function(m, e){
		if(m in this){
			return this[m](e);
		}
	},
	//public
	getValue: function(inRowIndex){
		// summary:
		//	returns value entered into editor
		// inRowIndex: int
		// grid row index
		// returns:
		//	value of editor
		return this.getNode(inRowIndex)[this._valueProp];
	},
	setValue: function(inRowIndex, inValue){
		// summary:
		//	set the value of the grid editor
		// inRowIndex: int
		// grid row index
		// inValue: anything
		//	value of editor
		var n = this.getNode(inRowIndex);
		if(n){
			n[this._valueProp] = inValue
		};
	},
	focus: function(inRowIndex, inNode){
		// summary:
		//	focus the grid editor
		// inRowIndex: int
		// grid row index
		// inNode: dom node
		//	editor node
		dojox.grid.focusSelectNode(inNode || this.getNode(inRowIndex));
	},
	save: function(inRowIndex){
		// summary:
		//	save editor state
		// inRowIndex: int
		// grid row index
		this.value = this.value || this.getValue(inRowIndex);
		//console.log("save", this.value, inCell.index, inRowIndex);
	},
	restore: function(inRowIndex){
		// summary:
		//	restore editor state
		// inRowIndex: int
		// grid row index
		this.setValue(inRowIndex, this.value);
		//console.log("restore", this.value, inCell.index, inRowIndex);
	},
	//protected
	_finish: function(inRowIndex){
		// summary:
		//	called when editing is completed to clean up editor
		// inRowIndex: int
		// grid row index
		dojo.setSelectable(this.cell.grid.domNode, false);
		this.cancelFormatNode(this.cell);
	},
	//public
	apply: function(inRowIndex){
		// summary:
		//	apply edit from cell editor
		// inRowIndex: int
		// grid row index
		this.cell.applyEdit(this.getValue(inRowIndex), inRowIndex);
		this._finish(inRowIndex);
	},
	cancel: function(inRowIndex){
		// summary:
		//	cancel cell edit
		// inRowIndex: int
		// grid row index
		this.cell.cancelEdit(inRowIndex);
		this._finish(inRowIndex);
	}
});
dojox.grid.editors.base = dojox.grid.editors.Base; // back-compat

dojo.declare("dojox.grid.editors.Input", dojox.grid.editors.Base, {
	// summary
	// grid cell editor that provides a standard text input box
	constructor: function(inCell){
		this.keyFilter = this.keyFilter || this.cell.keyFilter;
	},
	// keyFilter: object
	// optional regex for disallowing keypresses
	keyFilter: null,
	format: function(inDatum, inRowIndex){
		this.needFormatNode(inDatum, inRowIndex);
		return '<input class="dojoxGrid-input" type="text" value="' + inDatum + '">';
	},
	formatNode: function(inNode, inDatum, inRowIndex){
		this.inherited(arguments);
		// FIXME: feels too specific for this interface
		this.cell.registerOnBlur(inNode, inRowIndex);
	},
	doKey: function(e){
		if(this.keyFilter){
			var key = String.fromCharCode(e.charCode);
			if(key.search(this.keyFilter) == -1){
				dojo.stopEvent(e);
			}
		}
	},
	_finish: function(inRowIndex){
		this.inherited(arguments);
		var n = this.getNode(inRowIndex);
		try{
			dojox.grid.fire(n, "blur");
		}catch(e){}
	}
});
dojox.grid.editors.input = dojox.grid.editors.Input; // back compat

dojo.declare("dojox.grid.editors.Select", dojox.grid.editors.Input, {
	// summary:
	// grid cell editor that provides a standard select
	// options: text of each item
	// values: value for each item
	// returnIndex: editor returns only the index of the selected option and not the value
	constructor: function(inCell){
		this.options = this.options || this.cell.options;
		this.values = this.values || this.cell.values || this.options;
	},
	format: function(inDatum, inRowIndex){
		this.needFormatNode(inDatum, inRowIndex);
		var h = [ '<select class="dojoxGrid-select">' ];
		for (var i=0, o, v; (o=this.options[i])&&(v=this.values[i]); i++){
			h.push("<option", (inDatum==o ? ' selected' : ''), /*' value="' + v + '"',*/ ">", o, "</option>");
		}
		h.push('</select>');
		return h.join('');
	},
	getValue: function(inRowIndex){
		var n = this.getNode(inRowIndex);
		if(n){
			var i = n.selectedIndex, o = n.options[i];
			return this.cell.returnIndex ? i : o.value || o.innerHTML;
		}
	}
});
dojox.grid.editors.select = dojox.grid.editors.Select; // back compat

dojo.declare("dojox.grid.editors.AlwaysOn", dojox.grid.editors.Input, {
	// summary:
	// grid cell editor that is always on, regardless of grid editing state
	// alwaysOn: boolean
	// flag to use editor to format grid cell regardless of editing state.
	alwaysOn: true,
	_formatNode: function(inDatum, inRowIndex){
		this.formatNode(this.getNode(inRowIndex), inDatum, inRowIndex);
	},
	applyStaticValue: function(inRowIndex){
		var e = this.cell.grid.edit;
		e.applyCellEdit(this.getValue(inRowIndex), this.cell, inRowIndex);
		e.start(this.cell, inRowIndex, true);
	}
});
dojox.grid.editors.alwaysOn = dojox.grid.editors.AlwaysOn; // back-compat

dojo.declare("dojox.grid.editors.Bool", dojox.grid.editors.AlwaysOn, {
	// summary:
	// grid cell editor that provides a standard checkbox that is always on
	_valueProp: "checked",
	format: function(inDatum, inRowIndex){
		return '<input class="dojoxGrid-input" type="checkbox"' + (inDatum ? ' checked="checked"' : '') + ' style="width: auto" />';
	},
	doclick: function(e){
		if(e.target.tagName == 'INPUT'){
			this.applyStaticValue(e.rowIndex);
		}
	}
});
dojox.grid.editors.bool = dojox.grid.editors.Bool; // back-compat

}

if(!dojo._hasResource["dojox.grid.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.Grid"] = true;
dojo.provide("dojox.grid.Grid");




dojo.declare('dojox.Grid', dojox.VirtualGrid, {
	//	summary:
	//		A grid widget with virtual scrolling, cell editing, complex rows,
	//		sorting, fixed columns, sizeable columns, etc.
	//	description:
	//		Grid is a subclass of VirtualGrid, providing binding to a data
	//		store.
	//	example:
	//		define the grid structure:
	//	|	var structure = [ // array of view objects
	//	|		{ cells: [// array of rows, a row is an array of cells
	//	|			[	{ name: "Alpha", width: 6 }, 
	//	|				{ name: "Beta" }, 
	//	|				{ name: "Gamma", get: formatFunction }
	//	|			]
	//	|		]}
	//	|	];
	//	  	
	//		define a grid data model
	//	|	var model = new dojox.grid.data.table(null, data);
	//	|
	//	|	<div id="grid" model="model" structure="structure" 
	//	|		dojoType="dojox.VirtualGrid"></div>
	//	

	//	model:
	//		string or object grid data model
	model: 'dojox.grid.data.Table',
	// life cycle
	postCreate: function(){
		if(this.model){
			var m = this.model;
			if(dojo.isString(m)){
				m = dojo.getObject(m);
			}
			this.model = (dojo.isFunction(m)) ? new m() : m;
			this._setModel(this.model);
		}
		this.inherited(arguments);
	},
	destroy: function(){
		this.setModel(null);
		this.inherited(arguments);
	},
	// structure
	_structureChanged: function() {
		this.indexCellFields();
		this.inherited(arguments);
	},
	// model
	_setModel: function(inModel){
		// if(!inModel){ return; }
		this.model = inModel;
		if(this.model){
			this.model.observer(this);
			this.model.measure();
			this.indexCellFields();
		}
	},
	setModel: function(inModel){
		// summary:
		//		set the grid's data model
		// inModel:
		//		model object, usually an instance of a dojox.grid.data.Model
		//		subclass
		if(this.model){
			this.model.notObserver(this);
		}
		this._setModel(inModel);
	},
	// data socket (called in cell's context)
	get: function(inRowIndex){
		return this.grid.model.getDatum(inRowIndex, this.fieldIndex);
	},
	// model modifications
	modelAllChange: function(){
		this.rowCount = (this.model ? this.model.getRowCount() : 0);
		this.updateRowCount(this.rowCount);
	},
	modelRowChange: function(inData, inRowIndex){
		this.updateRow(inRowIndex);
	},
	modelDatumChange: function(inDatum, inRowIndex, inFieldIndex){
		this.updateRow(inRowIndex);
	},
	modelFieldsChange: function() {
		this.indexCellFields();
		this.render();
	},
	// model insertion
	modelInsertion: function(inRowIndex){
		this.updateRowCount(this.model.getRowCount());
	},
	// model removal
	modelRemoval: function(inKeys){
		this.updateRowCount(this.model.getRowCount());
	},
	// cells
	getCellName: function(inCell){
		var v = this.model.fields.values, i = inCell.fieldIndex;
		return i>=0 && i<v.length && v[i].name || this.inherited(arguments);
	},
	indexCellFields: function(){
		var cells = this.layout.cells;
		for(var i=0, c; cells && (c=cells[i]); i++){
			if(dojo.isString(c.field)){
				c.fieldIndex = this.model.fields.indexOf(c.field);
			}
		}
	},
	// utility
	refresh: function(){
		// summary:
		//	re-render the grid, getting new data from the model
		this.edit.cancel();
		this.model.measure();
	},
	// sorting
	canSort: function(inSortInfo){
		var f = this.getSortField(inSortInfo);
		// 0 is not a valid sort field
		return f && this.model.canSort(f);
	},
	getSortField: function(inSortInfo){
		// summary:
		//	retrieves the model field on which to sort data.
		// inSortInfo: int
		//	1-based grid column index; positive if sort is ascending, otherwise negative
		var c = this.getCell(this.getSortIndex(inSortInfo));
		// we expect c.fieldIndex == -1 for non model fields
		// that yields a getSortField value of 0, which can be detected as invalid
		return (c.fieldIndex+1) * (this.sortInfo > 0 ? 1 : -1);
	},
	sort: function(){
		this.edit.apply();
		this.model.sort(this.getSortField());
	},
	// row editing
	addRow: function(inRowData, inIndex){
		this.edit.apply();
		var i = inIndex || -1;
		if(i<0){
			i = this.selection.getFirstSelected() || 0;
		}
		if(i<0){
			i = 0;
		}
		this.model.insert(inRowData, i);
		this.model.beginModifyRow(i);
		// begin editing row
		// FIXME: add to edit
		for(var j=0, c; ((c=this.getCell(j)) && !c.editor); j++){}
		if(c&&c.editor){
			this.edit.setEditCell(c, i);
			this.focus.setFocusCell(c, i);
		}else{
			this.focus.setFocusCell(this.getCell(0), i);
		}
	},
	removeSelectedRows: function(){
		this.edit.apply();
		var s = this.selection.getSelected();
		if(s.length){
			this.model.remove(s);
			this.selection.clear();
		}
	},
	//: protected
	// editing
	canEdit: function(inCell, inRowIndex){
		// summary: 
		//	determines if a given cell may be edited
		//	inCell: grid cell
		// inRowIndex: grid row index
		// returns: true if given cell may be edited
		return (this.model.canModify ? this.model.canModify(inRowIndex) : true);
	},
	doStartEdit: function(inCell, inRowIndex){
		var edit = this.canEdit(inCell, inRowIndex);
		if(edit){
			this.model.beginModifyRow(inRowIndex);
			this.onStartEdit(inCell, inRowIndex);
		}
		return edit;
	},
	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		this.model.setDatum(inValue, inRowIndex, inFieldIndex);
		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
	},
	doCancelEdit: function(inRowIndex){
		this.model.cancelModifyRow(inRowIndex);
		this.onCancelEdit.apply(this, arguments);
	},
	doApplyEdit: function(inRowIndex){
		this.model.endModifyRow(inRowIndex);
		this.onApplyEdit(inRowIndex);
	},
	// Perform row styling 
	styleRowState: function(inRow){
		if(this.model.getState){
			var states=this.model.getState(inRow.index), c='';
			for(var i=0, ss=["inflight", "error", "inserting"], s; s=ss[i]; i++){
				if(states[s]){
					c = ' dojoxGrid-row-' + s;
					break;
				}
			}
			inRow.customClasses += c;
		}
	},
	onStyleRow: function(inRow){
		this.styleRowState(inRow);
		this.inherited(arguments);
	},
	junk: 0
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/








// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableSearchField");

dojo.declare("ibm.tivoli.tip.dijit.TIPTableSearchField",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,
  
  /* The TIPTable that this search bar is attached to */
  table: null,
  
  _ALL_COLS_NAME: "QF_ALL_COLS",
  
  /** An array of column names that should participate in filtering **/
  columnsToFilterOn: { "QF_ALL_COLS": true },
  
  /** Keeps track of all topic subscriptions to cleanly destroy later **/
  topics: [],
  
  _qf_arrow_src: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/qf_arrow.gif"),
  
  /** Keeps track of all event connections **/
  objectConnections: [],
  
  /** Keeps track of all children widgets in dropdown **/
  childrenWidgets: [],
  
  templateString:"<div class=\"quick-filter\" align=\"right\">\r\n<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n<tbody>\r\n<tr>\r\n<td dojoAttachPoint=\"colFilter\" class=\"qf-lcap\" width=\"20px\" valign=\"top\" dojoAttachEvent='onmouseover:_onHover,onmouseout:_onUnHover'>\r\n</td>\r\n<td class=\"qf-mid\" width=\"170\" valign=\"middle\">\r\n<label title=\"${resources_.SEARCH_TEXT}\">\r\n<input dojoAttachPoint=\"searchField\" class=\"qf-textfield-label\" type=\"text\" valign=\"middle\" dojoAttachEvent='onfocus:_onFocus,onblur:_onBlur,onkeypress:_onKeyPress' />\r\n</label>\r\n</td>\r\n<td class=\"qf-rcap\" width=\"15\"/>\r\n</tr>\r\n</tbody>\r\n</table>\r\n</div>\r\n",

  constructor: function( parms ) 
  {
  	this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
  	this.lastColumns = null;
  },
  
  postCreate: function()
  {
    if ( this.table !== null )
    {
      this.setTable( this.table );
    }        
  },  
   
  /********************************************************************
  * This method must be called to set the underlying table that the 
  * search bar will filter on.  As soon as this method is called
  * the table is queried for its "structure" (columns) and the dropdown
  * column filter dialog is populated
  *********************************************************************/
  setTable: function( table )
  {
  	// Save a reference to the table
  	this.table = table;
  	
  	this.clear(); //ensure that we start with a fresh slate
  	  	
  	var div  = document.createElement( 'DIV' );
  	var div2 = document.createElement( 'DIV' );
  	div2.className="qf-col-filter-title";	  
  
  	div2.appendChild( document.createTextNode( this.resources_.FILTER_ON ) );
  	div.appendChild( div2 );

  	div2 = document.createElement( 'DIV' );
	div2.className="qf-col-chkbox-label-row";
  	
  	var checkbox = new dijit.form.CheckBox( { id     : table.id + this._ALL_COLS_NAME,
  											  pId    : table.id,
  	                                          name   : this._ALL_COLS_NAME,
  	                                          checked: true } );
//  	                                          checked: this.columnsToFilterOn[ this._ALL_COLS_NAME ] } );

	this.childrenWidgets[ checkbox.id ] = checkbox;
	
	checkbox.onColsFilter = function()
	{
		this.setChecked( false );
	};
	
	checkbox.onCheckedEvent = function ()
	{
  		if ( this.checked )
  		{
	  		dojo.publish( this.pId + "_filterOnAll", ["Filter on all rows"] );
  		}
	};  	           
	
  	this.topics[ this._ALL_COLS_NAME ] = dojo.subscribe( table.id + "_filterByCol", checkbox, "onColsFilter" );                                                                         
	this.objectConnections[ this._ALL_COLS_NAME ] = dojo.connect( checkbox, "setChecked", checkbox, "onCheckedEvent" );  	                                          

	var label = document.createElement( 'LABEL' );
	label.className = "qf-col-chkbox-label";
	label["for"] = checkbox.id;
	label.title = this.resources_.ALL_COLUMNS;
	
	label.appendChild( document.createTextNode( this.resources_.ALL_COLUMNS ) );
	
	div2.appendChild( checkbox.domNode );  	                                         
	div2.appendChild( label );  	                                         

	div.appendChild( div2 );

	var columns = [];
	var i = 0;
	var j = 0;
	
	table.views.forEach( function( view, i )
	{
		if ( view.structure.cells )
		{
			for ( i in view.structure.cells )
			{
				for ( j in view.structure.cells[i] )
				{
					 var v = view.structure.cells[i][j];				
		             if (v.field !== undefined) {
                         columns.push( v );
	                 }
				}			
			}
		}
	});  	
  	  	
  	for ( i in columns )
  	{
	  if ( columns[i].isSearchable === false ) {
	    continue;
	  }
  		div2 = document.createElement( 'DIV' );
  		div2.className="qf-col-chkbox-label-row";
  		
  		checkbox = new dijit.form.CheckBox( { id     : table.id + columns[i].name,
  											  pId    : table.id,
  											  columnID: columns[i].field,
  	                                          name   : columns[i].name,
  	                                          checked: this.columnsToFilterOn[ columns[i].name ] } );

		this.childrenWidgets[ checkbox.id ] = checkbox;
		
		checkbox.onAllColsFilter = function()
		{
			this.setChecked( false );
		};
		
		checkbox.onCheckedEvent = function ()
		{
	  		if ( this.checked )
	  		{
		  		dojo.publish( this.pId + "_filterByCol", ["Filter on individual columns"] );		
	  		}
		};  	           
		
		this.topics[ columns[i].name ] = dojo.subscribe( table.id + "_filterOnAll", checkbox, "onAllColsFilter" );                               
  	    
		this.objectConnections[ columns[i].name ] = dojo.connect( checkbox, "setChecked", checkbox, "onCheckedEvent" );  	                                          
  	                                          
		div2.appendChild( checkbox.domNode );  	                            

		label = document.createElement( 'LABEL' );
		label["for"] = checkbox.id;
		label.title = columns[i].name;
		label.className = "qf-col-chkbox-label";
		
		label.appendChild( document.createTextNode( columns[i].name ) );
		
		div2.appendChild( label );  	                                         		             
  		
  		div.appendChild( div2 );  		
  	}
  	  	
  	var button = new dijit.form.Button( { id: table.id + "close_btn",
                                        type: "submit", 
                                        onClick: dojo.hitch (this, "onFilterOK"),
  								       label: this.resources_.OK } );

	this.childrenWidgets[ button.id ] = button;	  								       
  								     
	div.appendChild( button.domNode );  								     
  	
  	var tooltipDialog = new dijit.TooltipDialog( { id: table.id + "gf_tooltipdialog" } );

	this.childrenWidgets[ tooltipDialog.id ] = tooltipDialog;	  								       
  	  	
  	tooltipDialog.containerNode.appendChild( div );
  	
  	/** OVERLOAD to allow our own styling to be added **/
  	tooltipDialog.orient = function(/*Object*/ corner)
  	{
		// summary: configure widget to be displayed in given position relative to the button
		this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above" ) + " qf-TooltipDialog";
	};  	
  	
  	tooltipDialog.execute = function( args )
  	{  		
  	    this.columnsToFilterOn = [];
  	    
  		for ( var i in args )
  		{
			this.columnsToFilterOn[i] = args[i][0] ? true : false ;
  		}  	
  	};
  	
	var btn = new dijit.form.DropDownButton( { 	id: table.id + "_coltrigger", 
												dropDown: tooltipDialog,
												label: "",
												baseClass: "qf-col-trigger" } );
												
	this.tooltip_dropdownhelp = new dijit.Tooltip({label:this.resources_.SEARCH_DROP_DOWN_HELP, connectId:[btn.id]}, document.createElement("div"));
												
	
	this.childrenWidgets[ btn.id ] = btn;	  								       

	this.colFilter.appendChild( btn.domNode );
		
	btn.startup();
	
	var downArrowSpan =  dojo.query("span.dijitA11yDownArrow", btn.domNode );	
	
	dojo.query("span.dijitButtonText", btn.domNode ).orphan();
	
	downArrowSpan[0].innerHTML = "";
	dojo.removeClass( downArrowSpan[0], "dijitA11yDownArrow" );
	downArrowSpan[0].style.padding="0px";
	
	var image = dojo.doc.createElement( 'IMG' );	
	image.src = this._qf_arrow_src;
	image.border = 0;
	image.style.padding="0px";
	image.style.margin="0px";
	image.alt = this.resources_.SEARCH_DROP_DOWN_HELP;
	
	downArrowSpan[0].appendChild( image );	
  },
  
  onFilterOK: function () {
  	console.log ("enter onFilterOK");
	var columns = this.getSelectedColumns();
	if (this.columnsEqual (columns, this.lastColumns)) {
		console.log ("columns match.  not calling search");
	} else {
		console.log ("calling search");
  		dojo.publish (this.id + "searchInvoked", [this, null]);
  		this.lastColumns = columns;
	}
  },
  
  columnsEqual: function (columns1, columns2) {
  	if ((columns1 === null) && (columns2 === null)) {
  		return (true);
  	} else if ((columns1 === null) || (columns2 === null)) {
  		return (false);
  	} else if (columns1.length !== columns2.length) {
  		return (false);
  	} else {
		for (var x in columns1) {
			if (columns1[x] !== columns2[x]) {
				return (false);
			}
  		}
  		
  		return (true);
  	}
  },
  
  _onFocus: function()
  {
	this.domNode.className="quick-filter-focus";  
  },
  
  _onBlur: function()
  {
	this.domNode.className="quick-filter";
  },
  
  _onKeyPress: function(event)
  {
  	if (event.keyCode == 13) {
	  	event.cancelBubble = true;
  		event.returnValue = false;
  		dojo.publish ("onEnter", [this, event]);
  		dojo.publish (this.id + "searchInvoked", [this, event]);
	}
  },
  
  _onHover: function()
  {
	this.colFilter.className="qf-lcap-over";
  },
  
  _onUnHover: function()
  {
	this.colFilter.className="qf-lcap";
  },
  
  _setFilterCol: function ()
  {  	
  },
  

  getSelectedColumns: function ()
  {
  	var result = [];
  	
  	if (this.childrenWidgets[this.table.id + this._ALL_COLS_NAME].checked) {
	  	for (var x in this.childrenWidgets) {
	  		if (x) {
	  			if (this.childrenWidgets[x].columnID !== undefined ) {
		  			result.push (this.childrenWidgets[x].columnID);
		  		}
  			}
  		}
  	} else {
	  	for (var y in this.childrenWidgets) {
	  		if (y) {
	  			if (this.childrenWidgets[y].checked) {
		  			if (this.childrenWidgets[y].columnID !== undefined ) {
		  				result.push (this.childrenWidgets[y].columnID);
		  			}
	  			}
	  		}
	  	}
  	}
  	
	return (result);
  },
  
  /*************************************************************
  * Clears the contents of the search drop down to be recreated
  *************************************************************/
  clear: function()
  {
  	/** Destroy all subscriptions **/
  	for ( var i in this.topics )
  	{
  		dojo.unsubscribe( this.topics[i] );
  	}
  	
  	delete this.topics;
  	this.topics = [];

  	/** Destroy all event connections **/
  	for ( i in this.objectConnections )
  	{
  		dojo.disconnect( this.objectConnections[i] );
  	}  	

  	delete this.objectConnections;
  	this.objectConnections = [];
  	
  	/** Destroy all children widgets **/
  	for ( i in this.childrenWidgets )
  	{
  		this.childrenWidgets[i].destroy();
  	}  
  	
  	if ( this.tooltip_dropdownhelp )
  	{
  		this.tooltip_dropdownhelp.destroy();
  	}

  	delete this.childrenWidgets;
  	this.childrenWidgets = [];
  	
  	delete this.columnsToFilterOn;
  	this.columnsToFilterOn = [];
  },
  
  /*************************************************************
  * Destroy this widget -- remove all subscriptions and event
  * connections
  *************************************************************/
  destroy: function()
  {  	
  	this.clear();
  	
  	/** call super **/
  	this.inherited( "destroy", arguments );
  }
  
});

}

if(!dojo._hasResource["dojox.layout.ResizeHandle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ResizeHandle"] = true;
dojo.provide("dojox.layout.ResizeHandle");
dojo.experimental("dojox.layout.ResizeHandle"); 


 
 

dojo.declare("dojox.layout.ResizeHandle", [dijit._Widget, dijit._Templated], {
	// summary
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.

	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: '',

	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null, 

	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ... 
	resizeAxis: "xy",

	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement, 
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node. 
	activeResizeClass: 'dojoxResizeHandleClone',

	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale" 
	// 	node to size, "chain" will alter width, then height
	animateMethod: 'chain',

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation 
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// resize handle template, fairly easy to override: 
	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	// private propteries and holders
	_isSizing: false,
	_connects: [],
	_activeResizeNode: null,	
	_activeResizeLastEvent: null,
	// defaults to match default resizeAxis. set resizeAxis variable to modify. 
	_resizeX: true,
	_resizeY: true,


	postCreate: function(){
		// summary: setup our one major listener upon creation
		dojo.connect(this.resizeHandle, "onmousedown", this, "_beginSizing");
		if(!this.activeResize){ 
			this._activeResizeNode = document.createElement('div');
			dojo.addClass(this._activeResizeNode,this.activeResizeClass); 
		}else{ this.animateSizing = false; } 	

		if (!this.minSize) { 
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false; 
		switch (this.resizeAxis.toLowerCase()) {
		case "xy" : 
			this._resizeX = this._resizeY = true; 
			// FIXME: need logic to determine NW or NE class to see
			// based on which [todo] corner is clicked
			dojo.addClass(this.resizeHandle,"dojoxResizeNW"); 
			break; 
		case "x" : 
			this._resizeX = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeW");
			break;
		case "y" : 
			this._resizeY = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeN");
			break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if (this._isSizing){ return false; }

		this.targetWidget = dijit.byId(this.targetId);

		// FIXME: resizing widgets does weird things, disable virtual resizing for now:
		if (this.targetWidget) { this.activeResize = true; } 

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
		if (this.targetContainer) { this.targetDomNode = this.targetContainer; } 
		if (!this.targetDomNode){ return; }

		if (!this.activeResize) {
			this.targetDomNode.appendChild(this._activeResizeNode); 
			dojo.fadeIn({ node: this._activeResizeNode, duration:120, 
				beforeBegin: dojo.hitch(this,function(){
					this._activeResizeNode.style.display=''; 
				})
			}).play(); 
		}

		this._isSizing = true;
		this.startPoint  = {'x':e.clientX, 'y':e.clientY};

		// FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
		var mb = (this.targetWidget) ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);  
		this.startSize  = { 'w':mb.w, 'h':mb.h };

		this._connects = []; 
		this._connects.push(dojo.connect(document,"onmousemove",this,"_updateSizing")); 
		this._connects.push(dojo.connect(document,"onmouseup", this, "_endSizing"));

		e.preventDefault();
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines 
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e);	
			if(tmp === false){ return; }
			dojo.style(this._activeResizeNode,"width",tmp.width+"px");
			dojo.style(this._activeResizeNode,"height",tmp.height+"px"); 
			this._activeResizeNode.style.display=''; 
		}
	},

	_getNewCoords: function(/* Event */ e){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e; 

		var dx = this.startPoint.x - e.clientX;
		var dy = this.startPoint.y - e.clientY;
		
		var newW = (this._resizeX) ? this.startSize.w - dx : this.startSize.w;
		var newH = (this._resizeY) ? this.startSize.h - dy : this.startSize.h;

		// minimum size check
		if(this.minSize){
			//var mb = dojo.marginBox(this.targetDomNode);
			if(newW < this.minSize.w){
				newW = this.minSize.w;
			}
			if(newH < this.minSize.h){
				newH = this.minSize.h;
			}
		}
		return {width:newW, height:newH};  // Object
	},
	
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		var tmp = this._getNewCoords(e);
		if(tmp===false){ return; }

		if(this.targetWidget && typeof this.targetWidget.resize == "function"){ 
			this.targetWidget.resize({ w: tmp.width, h: tmp.height });
		}else{
			if(this.animateSizing){
				var anim = dojo.fx[this.animateMethod]([
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							width: { start: this.startSize.w, end: tmp.width, unit:'px' } 
						},	
						duration: this.animateDuration
					}),
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							height: { start: this.startSize.h, end: tmp.height, unit:'px' }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				dojo.style(this.targetDomNode,"width",tmp.width+"px"); 
				dojo.style(this.targetDomNode,"height",tmp.height+"px");
			}
		}	
		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		dojo.forEach(this._connects,function(c){
			dojo.disconnect(c); 
		});
		if(!this.activeResize){
			dojo.fadeOut({ node:this._activeResizeNode, duration:250,
				onEnd: dojo.hitch(this,function(){
					this._activeResizeNode.style.display="none";
				})
			}).play();
			this._changeSizing(e);
		}
		this._isSizing = false;
	}

});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007, 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.TIPMessageDialog");

 





// NLS


// define the widget class
dojo.declare(
	"ibm.tivoli.tip.dijit.TIPMessageDialog", 
    [ dijit._Widget ],
{
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction
    resizeAxis: "xy",

	// type: String: defines the type of message dialog ("Error", "Warning", "Info", "Confirm")
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String: the displayable translated message
	message: null,

	// messageId: String: the displayable translated message's unique identifier
	messageId: null,
	
	// width: String: gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String: gives the pixel height of the dialog (Default: 200px)
	height: "225px",
	
	// buttons: String Array: a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function: a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String: a URL that links the message to an InfoCenter page
	url: null,
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,

	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	    
	    this.dialog_ = null;

	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
	    if (this.type.toLowerCase() == "error") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_critical.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_warning.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_confirm.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_validation.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_info.png");
	        className = "info";
	    }
	    
		var div = document.createElement('div'); 
//		div.style.width = this.width; 
//		div.style.height = this.height;
        // The following div settings are needed to ensure the dialog is seen when the bottom frames
        // are rendered due to refresh or re-submit during error scenarios (see main_ruleset.jsp).
        // Without the settings, the bottom frame was overlaying the error message. 
		//div.style.top = 0; 
		//div.style.left = 0;
		document.body.appendChild(div);

        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_
		                                 },
		                                 div); 

		var dojoAttachpoint = this.dialog_.containerNode;
		dojoAttachpoint.innerHTML = this.getContent(className);

        //Build and place resize handle       
        var resizeHandleDOM = dojo.doc.createElement( 'span' );
        resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
        dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );
    
        this.resizeHandle = new dojox.layout.ResizeHandle(
        { 
              targetId: this.dialog_.id, 
            resizeAxis: this.resizeAxis 
        }, resizeHandleDOM );   
	
        this.dialog_.domNode.style.width = this.width;	
        this.dialog_.domNode.style.height = this.height;	
        //this.dialog_.containerNode.style.overflow="auto";
        this.dialog_.domNode.style.overflow = "hidden";        
    
		for (var i = 0; i < this.buttons.length; i++) {
			var button = dojo.byId(this.id + "mmd_button_" + i);
			dojo.connect(button, 'onclick', this, 'onClick');    
		}
    
        this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	  this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );    
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onClick: function(/* HTMLEvent */e) {
		var index = this.id.length + "mmd_button_".length;
		this.buttonPressedId_ = parseInt(e.target.id.substring(index), 0);
	    console.log ("TIPMessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
		this.dialog_.show();
		this.customLayout();
		this.correctPNG();

	},

      customLayout: function() 
      {
		console.debug( dojo.byId( this.id + '_messagediv' ) );
		console.debug( dojo.contentBox( this.dialog_.domNode ) );
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

		var titleBarSize = dojo.contentBox( this.dialog_.titleBar );

		var newSize = {w:0,h:0};

		newSize.w = dialogSize.w - 75;
		if (this.showCheckbox === true) {
			var neverShowCoords = dojo.coords (this.id + "neverShowAgain");
			newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 90;
		} else {		
			newSize.h = dialogSize.h - titleBarSize.h - 90;
		}

	    dojo.contentBox( messageDiv, newSize );
	},
	
	getContent: function(className) {
		var innerHTML = "";

		innerHTML += "<table class=\"" + className + "\">";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";

		innerHTML += "<table>";
		innerHTML += "<tr>";
		innerHTML += "<td rowspan=\"2\">";
		innerHTML += "&nbsp;";
		innerHTML += "<img align=\"middle\" src=\"" + this.icon_ + "\"/>";
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";
		innerHTML += "<td class=\"message-id\" align=\"left\">";
		
		// add URL to infocenter
		if (this.url) {
			innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + '\')">';
			innerHTML += this.messageId;
			innerHTML += "</a>";
		}
		else {
			innerHTML += this.messageId;
		}
		
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td class=\"message-description\" align=\"left\">";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;'>";
		innerHTML += "<p>";
		innerHTML += this.message;
		innerHTML += "</p>";
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr>";
			innerHTML += "<td class=\"checkbox\" valign=\"bottom\">";
			innerHTML += "&nbsp;&nbsp;&nbsp;<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/><label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";

		innerHTML += "<div>";
		innerHTML += "<table width='100%'>";
		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";
		innerHTML += "<div align=\"right\">";
		for (var i = 0; i < this.buttons.length; i++) {
			innerHTML += "&nbsp;<button dojoType=\"ibm.tivoli.tip.dijit.TIPButton\" id=\"" + this.id + "mmd_button_" + i + "\" value=\"" + this.buttons[i] + "\">";
			innerHTML += this.buttons[i] + "</button>";
		}
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";

		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
        dojo.disconnect( this.onCloseListener );
        dojo.disconnect( this.onResizeListener );
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove (this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Global"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Global"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* xxxx-xxx 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/



dojo.provide("ibm.tivoli.tip.dojo.Global");
dojo.provide("ibm.tivoli.tip.dojo.Global.InvalidContextError");
dojo.provide("ibm.tivoli.tip.dojo._Global");

/****************************************************************************
* Global 
*
* (Q) What is this?
* (A) Since the TIP applications will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.dojo.Global.instances     = [];
ibm.tivoli.tip.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global getSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (optional: used only if global object does NOT exist) an object of 
*			properties that will get mixed into the Global object for storage
*
*  When creating the context, you create it like this:
*  var context = {
*                  controllerURL: "/SimpleExample/AjaxController",
*                  namespace: myNamespace,
*                  nlsResources : dojo.i18n.getLocalization( "ibm.tivoli.tbsm", "resources" )
*                  contextPath: "<%=rootContextPath%>"
*                };
*   controllerURL (required) - specifies the URL to use to send requests to.
*                 this is the URL which handles the requests usually using the
*                 TIP CommandHandler.
*   namespace (optional) - Should always be specified.  If not specified, then
*             this routine will set it on the context.
*   nlsResources (optional) - Must be specified if you intend to use the nls 
*                facility provided in this object
*   contextPath (optional) - Should always be specified but not required. 
*               Must be specified if you intend to use the helper
*               function "contextualURL".
*   ===========
*   any other properties you add into the context will be avaible to your app.
****************************************************************************/
ibm.tivoli.tip.dojo.Global.getSingleton = function( namespace, context )
{
  var singleton = ibm.tivoli.tip.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
  	//check to make sure a context object was passed before creating a Global object
  	//if the context object wasn't given then assume that an attempt was made to 
  	//access a Global object that hasn't been created -- throw an error
  	if ( context === undefined )
  	{
  		throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
  		"singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
  		"object was not provided on creation." );
  	}
  	
  	//make sure the namespace is saved on the context object
  	if ( context.namespace === undefined )
  	{
  		context.namespace = namespace;	
  	}  	
  	
    singleton = new ibm.tivoli.tip.dojo._Global( context );
   
   	//save the new singleton instance on the Global memory space
    ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton;
  }  
  
  return( singleton );	
};

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
****************************************************************************/
ibm.tivoli.tip.dojo.Global.setSingleton = function (namespace, context) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
   var singleton = new ibm.tivoli.tip.dojo._Global( context );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};


/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Global",null,
{	 
  _actionBroker:  null,
  _portletTitle: null,
	
	constructor: function( context )
	{
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );
      
      //create path to JSON controller servlet
      this.controller = context.controllerURL;
         
      // Create resource bundle for TIP based resource strings
      this.nlsTIPResources = dojo.i18n.getLocalization( "ibm.tivoli.tip.dojo", "resources" );
         
        
	},

  /************************************************************************
  * Get the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  getActionBroker : function() 
  {
    return( _actionBroker );
  },

  /************************************************************************
  * Set the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  setActionBroker : function( /* ibm.tivoli.tip.dojo.ActionBroker */ broker ) 
  {
    return( _actionBroker = broker );
  },
  
  getPortletTitle : function () {
  	return this._portletTitle;
  },
  
  setPortletTitle: function (/*String*/ title) {
  	this._portletTitle = title;
  },
	
	/************************************************************************
	* Generate Conextual URL
	************************************************************************/
	contextualURL : function( baseURL ) 
	{
  	return( this.contextPath + baseURL );
	},

	/************************************************************************
	* Generate namespaced ID from a BASE ID.
	************************************************************************/
	namespaceID: function( baseID ) 
	{
  	return( this.namespace + baseID );
	},	

  /************************************************************************
  * show error message dialog
  ************************************************************************/
  showErrorMessage: function( msgMixin )
  {
    var msgKW = {
              type:         "Error",
              showCheckbox: false,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );

    var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW );
    x.show();
  },

  /************************************************************************
  * Retrieve a specific NLS string from the TIP resource bundle.  Deployers
  * could create a method like this for their own product that calls nls
  * with their resource bundle for convenience.  This is meant to only
  * be called by TIP code.
  ************************************************************************/
  tipnls: function( nlsKey, inserts )
  {
     return this.nls( nlsKey, inserts, this.nlsTIPResources );
  },

  /************************************************************************
  * Retrieve a specific NLS string optionally performing substitution(s).
  * This is a fairly generic routine meant to be used by exploiters of 
  * the Global object.  If an exploiter specifies a resource bundle in 
  * the Global context via the name 'nlsResources' when creating the Global object
  * then a bundle need not be specified on the invocation of this method
  * because it will use nlsResources instead.
  ************************************************************************/
  nls: function( nlsKey, inserts, myBundle )
  {
    // attempt to retrieve the specified string via its key
    var s = null;
    
    // If a bundle has been specified, then use it, else
    // use the bundle specified on nlsResources.
    if (myBundle !== null)
    {
       s = myBundle[ nlsKey ];
    }
    else
    {
       s = this.nlsResources[ nlsKey ];
    }

    // substitute "key" if not able to resolve
    if ( ! s )
    {
      s = nlsKey;
    }

    // string insert(s) specified?
    if ( inserts ) 
    {
      // if not already a KW object -or- array 
      if ( ! ( typeof inserts == 'object' ) )
      {
        // convert single number/string/... to array
        inserts = [ inserts ];
      }

      // perform the substitutions
      s = dojo.string.substitute( s, inserts );
    }

    return( s );
  }
    
} );

//Declare a new Error type to throw when a Global context error is detected
dojo.declare("ibm.tivoli.tip.dojo.Global.InvalidContextError", Error, {});

}

if(!dojo._hasResource["dojo.date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date"] = true;
dojo.provide("dojo.date");

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
}

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
}

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year 
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string, 
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
}

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(Number(date1));
	date2 = new Date(Number(date2 || new Date()));

	if(typeof portion !== "undefined"){
		if(portion == "date"){
			// Ignore times and compare dates.
			date1.setHours(0, 0, 0, 0);
			date2.setHours(0, 0, 0, 0);
		}else if(portion == "time"){
			// Ignore dates and compare times.
			date1.setFullYear(0, 0, 0);
			date2.setFullYear(0, 0, 0);
		}
	}
	
	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(Number(date)); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but even this is not standard.  There are CLDR entries to localize this.
			var days, weeks;
			var adj = 0;
			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = 7 * weeks + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
		case "hour":
		case "minute":
		case "second":
		case "millisecond":
			property = "UTC" + interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojo.cldr.supplemental"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cldr.supplemental"] = true;
dojo.provide("dojo.cldr.supplemental");



dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,lb:6,ly:6,ma:6,om:6,qa:6,sa:6,
		sd:6,so:6,tn:6,ye:6,
		as:0,au:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,ie:0,il:0,is:0,jm:0,jp:0,kg:0,kr:0,la:0,
		mh:0,mo:0,mp:0,mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,vi:0,za:0,zw:0,
		et:0,mw:0,ng:0,tj:0,
		gb:0,
		sy:4
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (typeof dow == 'undefined') ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
        region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", hu:"hu", it:"it",
        ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
}

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		eg:5,il:5,sy:5,
		'in':0,
		ae:4,bh:4,dz:4,iq:4,jo:4,kw:4,lb:4,ly:4,ma:4,om:4,qa:4,sa:4,sd:4,tn:4,ye:4		
	};

	var weekendEnd = {/*default is 0=Sunday*/
		ae:5,bh:5,dz:5,iq:5,jo:5,kw:5,lb:5,ly:5,ma:5,om:5,qa:5,sa:5,sd:5,tn:5,ye:5,af:5,ir:5,
		eg:6,il:6,sy:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(typeof start == 'undefined'){start=6;}
	if(typeof end == 'undefined'){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

}

if(!dojo._hasResource["dojo.date.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.locale"] = true;
dojo.provide("dojo.date.locale");

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.







// Load the bundles containing localization information for
// names and formats


//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var pad;
			var widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							s = String(s); s = s.substr(s.length - 2);
							break;
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					var width;
					switch(l){
						case 1: case 2:
							s = m+1; pad = true;
							break;
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "L") ? "standalone" : "format";
						var prop = ["months",type,width].join("-");
						s = bundle[prop][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
				case 'e':
				case 'c': // REVIEW: don't see this in the spec?
					var d = dateObject.getDay();
					var width;
					switch(l){
						case 1: case 2:
							if(c == 'e'){
								var first = dojo.cldr.supplemental.getFirstDayOfWeek(options.locale);
								d = (d-first+7)%7;
							}
							if(c != 'c'){
								s = d+1; pad = true;
								break;
							}
							// else fallthrough...
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "c") ? "standalone" : "format";
						var prop = ["days",type,width].join("-");
						s = bundle[prop][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle[timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c) {
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.getTimezoneName(dateObject);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dateObject.getTimezoneOffset();
					var tz = [
						(offset<=0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset)/60), 2),
						dojo.string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.debug(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}

dojo.date.locale.format = function(/*Date*/dateObject, /*Object?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string}
	//		selector- choice of 'time','date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var str = [];
	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle);
	if(options.selector == "year"){
		// Special case as this is not yet driven by CLDR data
		var year = dateObject.getFullYear();
		if(locale.match(/^zh|^ja/)){
			year += "\u5E74";
		}
		return year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time
	return result; // String
};

dojo.date.locale.regexp = function(/*Object?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		locale- override the locale used to determine formatting rules

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];
	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = datePattern + ' ' + timePattern; //TODO: use locale-specific pattern to assemble date + time
	}

	var tokens = [];
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*Object?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/#Date_Format_Patterns
	//
	// value:
	//		A string representation of a date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules
	//		strict- strict parsing, off by default

	var info = dojo.date.locale._parseInfo(options);
	var tokens = info.tokens, bundle = info.bundle;
	var re = new RegExp("^" + info.regexp + "$");
	var match = re.exec(value);
	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'];
	//1972 is a leap year.  We want to avoid Feb 29 rolling over into Mar 1,
	//in the cases where the year is parsed after the month and day.
	var result = new Date(1972, 0);
	var expected = {};
	var amPm = "";
	dojo.forEach(match, function(v, i){
		if(!i){return;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2){
					//interpret year literally, so '5' would be 5 A.D.
					result.setFullYear(v);
					expected.year = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear();
						var century = year.substring(0, 2) * 100;
						var yearPart = Number(year.substring(2, 4));
						var cutoff = Math.min(yearPart + 20, 99);
						var num = (v < cutoff) ? century + v : century - 100 + v;
						result.setFullYear(num);
						expected.year = num;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return null;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result.setFullYear(v);
						expected.year = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.debug("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return null;
					}
				}else{
					v--;
				}
				result.setMonth(v);
				expected.month = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, "".toLowerCase);
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.debug("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return null;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'd':
				result.setDate(v);
				expected.date = v;
				break;
			case 'D':
				//FIXME: need to defer this until after the year is set for leap-year?
				result.setMonth(0);
				result.setDate(v);
				break;
			case 'a': //am/pm
				var am = options.am || bundle.am;
				var pm = options.pm || bundle.pm;
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.debug("dojo.date.locale.parse: Could not parse am/pm part.");
					return null;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v==24){v=0;}
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.debug("dojo.date.locale.parse: Illegal hours value");
					return null;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result.setHours(v);
				break;
			case 'm': //minutes
				result.setMinutes(v);
				break;
			case 's': //seconds
				result.setSeconds(v);
				break;
			case 'S': //milliseconds
				result.setMilliseconds(v);
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.debug("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
	});

	var hours = result.getHours();
	if(amPm === 'p' && hours < 12){
		result.setHours(hours + 12); //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result.setHours(0); //12am -> 0
	}

	//validate parse date fields versus input date fields
	if(expected.year && result.getFullYear() != expected.year){
//		console.debug("dojo.date.locale.parse: Parsed year: '" + result.getFullYear() + "' did not match input year: '" + expected.year + "'.");
		return null;
	}
	if(expected.month && result.getMonth() != expected.month){
//		console.debug("dojo.date.locale.parse: Parsed month: '" + result.getMonth() + "' did not match input month: '" + expected.month + "'.");
		return null;
	}
	if(expected.date && result.getDate() != expected.date){
//		console.debug("dojo.date.locale.parse: Parsed day of month: '" + result.getDate() + "' did not match input day of month: '" + expected.date + "'.");
		return null;
	}

	//TODO: implement a getWeekday() method in order to test 
	//validity of input strings containing 'EEE' or 'EEEE'...
	return result; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings) 
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g); 
	var literal = false;

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = dojo.regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s;
		var c = match.charAt(0);
		var l = match.length;
		var p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+' : p2+'[1-9]|1[0-2]';
				break;
			case 'D':
				s = p2+'[1-9]|'+p3+'[1-9][0-9]|[12][0-9][0-9]|3[0-5][0-9]|36[0-6]';
				break;
			case 'd':
				s = p2+'[1-9]|[12]\\d|3[01]';
				break;
			case 'w':
				s = p2+'[1-9]|[1-4][0-9]|5[0-3]';
				break;
		    case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = p2+'[1-9]|1[0-2]';
				break;
			case 'k': //hour (0-11)
				s = p2+'\\d|1[01]';
				break;
			case 'H': //hour (0-23)
				s = p2+'\\d|1\\d|2[0-3]';
				break;
			case 'K': //hour (1-24)
				s = p2+'[1-9]|1\\d|2[0-4]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle.am || 'AM';
				var pm = options.pm || bundle.pm || 'PM';
				if(options.strict){
					s = am + '|' + pm;
				}else{
					s = am + '|' + pm;
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
				}
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.debug("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}
})();

(function(){
var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo for the CLDR data: dateFormat-xxxx / timeFormat-xxxx
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.format for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = dojo.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};
})();

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/use, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item: 'months' || 'days'
	// type: 'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// use: 'standAlone' || 'format' (default)
	// locale: override locale used to find the names

	var label;
	var lookup = dojo.date.locale._getGregorianBundle(locale);
	var props = [item, use, type];
	if(use == 'standAlone'){
		label = lookup[props.join('-')];
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = dojo.cldr.supplemental.getWeekend(locale);
	var day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return dojo.date.difference(new Date(dateObject.getFullYear(), 0, 1), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay();
	var adj = (firstDayOfYear - firstDayOfWeek + 7) % 7;
	var week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.util"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.util");

ibm.tivoli.tip.dojo.util._stdDateFormat = { // selector:     'dateOnly', 
                                      formatLength: 'short' 
                                    };

/************************************************************************
* "apply" any GLOBAL-based data of interest.
************************************************************************/	
ibm.tivoli.tip.dojo.util.applyGlobal = function( global ) 
{
  ibm.tivoli.tip.dojo.util._stdDateFormat = global.l10n.date.stdTS;

  console.debug( "** util.applyGlobal **", dojo.toJson(ibm.tivoli.tip.dojo.util._stdDateFormat ) ); 
};

/************************************************************************
* Format a DATE with NLS rules
************************************************************************/	
ibm.tivoli.tip.dojo.util.formatDate = function( date ) 
{
	fDate = ( date == null )
          ? ""
          : dojo.date.locale.format( date, ibm.tivoli.tip.dojo.util._stdDateFormat );
 
	return( fDate );
};

/************************************************************************
* Standard formatter for a table column containing Date objects.
************************************************************************/	
ibm.tivoli.tip.dojo.util.dateColumnFormater = function( /* Date */ cellValue )
 {
   // console.debug( "cell: ", cellValue );

   return( ibm.tivoli.tip.dojo.util.formatDate( cellValue ) );
 };

/************************************************************************
* Refresh the page
************************************************************************/	
ibm.tivoli.tip.dojo.util.refreshPage = function()
{
	window.location.reload( false );
};

/*****************************************************************************
* Return a string of all the fields contained in an object.
*****************************************************************************/
ibm.tivoli.tip.dojo.util.fieldsOf = function( object )
{ 
  s = "";

  if ( object && ( object !== null ) ) 
  {
    for ( var field in object )
    {
      s += field + "; ";
    }
  }

  return( s );
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
* 
* Returns dojo.Deferred() -- must add Callback
*
* example usage:
* ibm.tivoli.tip.dojo.util.clearDataStore( store ).addCallback( function(){} );
************************************************************************/ 
ibm.tivoli.tip.dojo.util.clearDataStore = function( store )
{
  var deferred = new dojo.Deferred();
    
  var callback = 
  {  
        deferred: deferred,
           store: store,        
      onComplete: function( items, request ) 
                  {                    
                    for ( var i in items )
                    {
                      this.store.deleteItem( items[i] );
                    }

                    this.deferred.callback();
                  }
  };
  
  store.fetch( { onComplete: dojo.hitch( callback, "onComplete" ) } );
  
  return deferred;
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
************************************************************************/ 
ibm.tivoli.tip.dojo.util.addItemsToDataStore = function( store, items )
{
  for ( var i in items )
  {
    store.newItem( items[i] );
  }

  store.save();
};

/************************************************************************
* Find the default name for a specific value using a name/value array.
*
* [
*  { value: "-1", name: this.global.nls( "CBOX_CONFIG_DEFAULT", { config : config.maxSnapshotsDefault } ) }, 
*  { value:  "0", name: bundle.CBOX_DISABLED }, 
*  { value:  "1", name:  "1" }
* ]
************************************************************************/ 
ibm.tivoli.tip.dojo.util.findDefaultName = function( value, nameValueArray )
{
  name = nameValueArray[ 0 ].name;     // default to first element

  for ( var i in nameValueArray )
  {
    nv = nameValueArray[ i ];

    if ( nv.value == value ) 
    {
      name = nv.name;
      break;
    }
  }

  // console.debug( "value: " + value + ", name: " + name );

  return( name );
};


}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Server"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Server"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.Server");
dojo.provide("ibm.tivoli.tip.dojo._Server");

ibm.tivoli.tip.dojo.Server.anyException  = "*"; 

ibm.tivoli.tip.dojo.Server._singleton    = null;

/****************************************************************************
* Get the "singleton" Server instance.
****************************************************************************/
/* "ibm.tivoli.tip.dojo._Server" */
ibm.tivoli.tip.dojo.Server.getSingleton = function( /* required: ibm.tivoli.tip.dojo._Global */ global )
{
  if ( global === undefined ) {
  	throw new Error( "ibm.tivoli.tip.dojo.Server requires a 'global' object reference" );
  }
  		
  //if the singleton doesn't exist yet --> create it now
  //there must be a unique singleton per global instance
  if ( global._serverSingleton === undefined || global._serverSingleton === null )
  {
    global._serverSingleton = new ibm.tivoli.tip.dojo._Server( { global : global } );
  }  
  
  return global._serverSingleton;	
};

/****************************************************************************
* Provides I/O access to the controller servlet.
* Handles TIP protocol of returning server-side exceptions in a valid
* data response besides Dojo's error and timeout conditions.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Server", 
             null,
{	 
	_totalRequests:   0,        // total server requests made
  _activeRequests:  0,        // # server request(s) currently active
	
  /************************************************************************
  * Object constructor
  ************************************************************************/
	constructor: function( mixinData )
	{
    dojo.mixin( this, mixinData );
	},
	
	/************************************************************************
	* Perform a HTTP GET request.
	************************************************************************/
	/* void */ get: function( /* Object */ requestKW )
	{
    this._preRequest( requestKW );
	
    //console.debug( "Ready to xhrGet", requestKW );
    //console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(requestKW) );
    //console.debug( "Timeout", requestKW.timeout );

	  dojo.xhrGet( requestKW ); 
	},
	
	/************************************************************************
	* Perform a HTTP POST request.  Preferred for data forms that do
  * NOT contain any file-input fields...
	************************************************************************/
	/* void */ post: function( /* Object */ requestKW )
	{
    this._preRequest( requestKW );
	
	  dojo.xhrPost( requestKW ); 
	},

  /************************************************************************
  * Common PRE request processing of the request KW object.
  ************************************************************************/
  /* Object */ _preRequest: function( /* Object */ requestKW )
  {
    this._requestStarting( ( requestKW.showBusyDialog !== undefined ) ? requestKW.showBusyDialog : true );                 

    requestKW._server      = this;                 // save self reference for use in load/error handlers below   
    requestKW._requestID   = this._totalRequests;  // unofficial request ID

    requestKW.url          = requestKW.appendURL ? 
      this.global.controller + requestKW.appendURL :
      this.global.controller;

    requestKW.handleAs	   = "json-comment-filtered";  
    requestKW.mimetype     = "application/json";
    
    requestKW.content.requestType  = "global";     // indicates that the request was sent here

    requestKW.preventCache = true;     // fix IE cache bug

    // provide default timeout if none specified

    if ( ! requestKW.timeout ) {
           requestKW.timeout = 0; 
    }

    // our code handles it all!  NOTE: trying to set callbacks on "deferred" object did NOT work

    requestKW.load  = this._loadHandler;
    requestKW.error = this._errorHandler;

    return( requestKW ); 
  },
	
	/************************************************************************
	* Standard JSON request "load" handler.
  *
  * Use the appropriate "how to handle" directions from the request:
  * - "onExc"      for server-side errors returned in the 'exc' field of response
  * - "onResponse" for response data
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used (aka requestKW)
	************************************************************************/
  /* response */ _loadHandler: function( response, ioArgs ) 
	{
    //console.debug( "************************ Server._loadHandler ************************" );

//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(this) );      proved its requestKW
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(response) );
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );    ioArgs.args == requestKW

    //console.debug( "  ** response", dojo.toJson(response) );

    var server = this._server;

    server._requestCompleted();

	  // was an exception detected/reported from the server-side?

    var exc = response.exc;

	  if ( exc )
	  { 
      //console.debug( "    !!!!! handling EXC !!!!!" );

      // ISC session timeout?
      if ( exc == "ISCSessionTimeoutException" ) 
      {
        ibm.tivoli.tip.dojo.util.refreshPage();
        return;
      }

      var howTo  = this.onExc;

      if ( ! howTo ) {
        howTo = { showMsg : true };    // use default
      }

      //console.debug( "  ?? howTo",  dojo.toJson(howTo) );

      // Crop the exc message to maximum size
      server.cropExcMsg( response );

      //TODO: handle new NLS enabled exceptions...
      var msgKW = null;
      
      if (response.excID) {
	      msgKW = { 
                   messageId : response.excID,
                   message   : response.excMsg
                  };
      } else {
    	  msgKW = { 
                   messageId : server.global.tipnls( "BIND_ERROR_ID" ),
                   message   : server.global.tipnls( "BIND_ERROR_MSG", response )
                  };
	  }

      //console.debug( "  ?? msgKW",  dojo.toJson(msgKW) );

      if ( howTo.showMsg ) {
        server.global.showErrorMessage( msgKW );
      }

      if ( howTo.callback ) {
        howTo.callback( msgKW, this /* requestKW */ );
      }
	  }
	
	  // No exception --> forward to callback
	  else
	  {
      howTo = this.onResponse;

      if ( howTo && howTo.callback ) {
	      howTo.callback( response, this /* requestKW */ );
      }
	  }

    return( response ); // Dojo best practice 
	},

	/************************************************************************
	* Standard JSON request error handler
  *
  * How to handle directions are contained in the request's "onExc" field.
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used
  *
  * error.message:    specific error message, "too much recursion"
  * error.fileName:   "https://localhost:16316/ibm/console/secure/isclite/scripts/dojo/dojo.js"
  * error.lineNumber: 20
  * error.stack:      callback stack (can be very deep )
  * error.name:       "InternalError"
  *
  * error.dojoType:   "timeout" or "cancel", but other error types are possible.
  * dojoType=timeout message=timeout exceeded
  *
  * Error: bad http response code:404 message=bad http response code:404
  * 
  * console.error("HTTP status code: ", ioArgs.xhr.status); 
	************************************************************************/
	/* data */ _errorHandler: function( error, ioArgs ) 
	{
    //console.debug( "************************ Server._errorHandler ************************" );

    //console.debug( "  >> this",   ibm.tivoli.tip.dojo.util.fieldsOf(this) );
    //console.debug( "  >> error",  ibm.tivoli.tip.dojo.util.fieldsOf(error) );
    //console.debug( "  >> ioArgs", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );
    //console.debug( "  >> ioArgs.args", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs.args) );

    //console.debug( "  -- error",  dojo.toJson(error) );

    var server = this._server;

    server._requestCompleted();

    var howTo = this.onError;
    var msgKW = null;

    if ( ! howTo ) {
      howTo = { showMsg : true };    // use default
    }

    if ( error.dojoType && ( error.dojoType === "timeout" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "BIND_TIMEOUT_ID" ),
                message   : server.global.tipnls( "BIND_TIMEOUT_MSG", error )
              }; 
    }
    else if ( error.name && ( error.name === "InternalError" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "JS_ERROR_ID" ),
                message   : server.global.tipnls( "JS_ERROR_MSG", error )
              }; 
    }
    else
    {
      msgKW = { 
                messageId : server.global.tipnls( "XHR_ERROR_ID" ),
                message   : server.global.tipnls( "XHR_ERROR_MSG", error )
              }; 
    }

    if ( howTo.showMsg ) {
	    server.global.showErrorMessage( msgKW );
    }

    if ( howTo.callback ) {
      howTo.callback( msgKW, this /* requestKW */ );
    }

    return( error ); // Dojo best practice 
	},

  /************************************************************************
  * Starting a new server request.
  ************************************************************************/
  _requestStarting: function( showBusyDialog ) 
  {
    // increment total server requests 
    this._totalRequests++;

    // increment outstanding server requests 
    this._activeRequests++;

    // show wait dialog on first active request
    if ( showBusyDialog === true && this._activeRequests == 1 )
    {
      this.wait_timer = setInterval( dojo.hitch( this, "_showBusyPane" ), 100 );
    }
  },
  
  /************************************************************************
  * Show Busy Pane
  *    
  * Constructing a ibm.tivoli.tip.dijit.BusyPane and passing its ID on 
  * busyPaneID when constructing Server object will provide feedback
  * when requests are made
  ************************************************************************/
  _showBusyPane: function()
  {
    clearInterval( this.wait_timer );
    // in case of race condition, make sure that active requests > 0
    if ( this._activeRequests > 0 ) {
	    var busyPane = dijit.byId( this.busyPaneID );
	    if ( busyPane )
	    {
	      busyPane.show(true); // size to portlet
	    }
    }
  },

  /************************************************************************
  * Completing a server request.
  ************************************************************************/
  _requestCompleted: function() 
  {
	  //decrement the # outstanding requests 
	  this._activeRequests--;

	  // if no more active requests --> close wait dialog
	  if ( this._activeRequests === 0 )
	  {
		  clearInterval( this.wait_timer );
      
          var busyPane = dijit.byId( this.busyPaneID );
          if ( busyPane )
          {
            busyPane.hide();
          }
	  }  	
  },

  /***************************************************************************
  * Crop the message in the 'excMsg' field to the maximum allowed length.
  ***************************************************************************/
  cropExcMsg: function( response )
  {
    // provide a default exc message if field isn't already available
    if ( ! response.excMsg ) {
      response.excMsg = "-?-";
    }

    // access externalized MAX length
    var msgMax = this.global.nlsTIPResources.MAX_WIDTH_EXC_MSG;

    // exc message too long?
    if ( response.excMsg.length > msgMax ) 
    {
      response.excMsg = response.excMsg.substring( 0, msgMax ) + this.global.tipnls( "TEXT_CROPPED" );
    }
  }

} );

}

/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.require( "dijit.Menu" );

dojo.provide( "ibm.tivoli.tip.dijit.ContextMenuAdapter" );
/*******************************************************************************
* ContextMenuAdapter is responsible for constructing and displaying dynamic
* context menus.  That is it will ask for the items that should be displayed
* each time it is told to present the context menu. 
*
* It can handle calling a JSON Url to retreive the items for you.
*
* It also has a method that can be overloaded to provide context to the code
* building the menu. (For instance -- the row of a table that is selected)
*
* The menu model is as follows:
* 
* [ { 
*     id: "uniqueID", 
*     label: "label", 
*     iconClass: "iconClass", 
*     submenu_items: [ {},{} ] 
*     actionID: "actionToFire" 
    } 
* ... ]
*
* id, iconClass, and submenu_items are all optional
*
* NOTE:
* ! IF "_separator_" is passed as actionID then a menu separator is created !
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dijit.ContextMenuAdapter", null,
{
   /* [Optional] The JSON Url to request context menu items from */
   url: null,
   
   /* [Optional] The unique name of the context menu -- for lookup of items */
   name: null,
   
   /****************************************************************************
   * The constructor of the adapter.  Mixes in parameters.
   ****************************************************************************/
   constructor: function( args )
   {
     dojo.mixin( this, args );
     
     this.menuItems = [];
     this.subMenus = [];
   },
      
   /****************************************************************************
   * Ensures that only one context menu is used during the life of this adapter.
   * It destroys any previously created menus and recreates a new menu.
   ****************************************************************************/
   createContextMenu: function()
   {
     this._menu = new dijit.Menu();
     
     this._onCloseListener = dojo.connect( this._menu, "onClose", this, "_destroyMenu" );
     
     //populate the menu
     this.populateMenu( this._menu ); 
     
     return this._menu;         
   },
   
   /****************************************************************************
   * When the menu closes -- cleanly destroy it.  There is a bug in Dojo that
   * requires us to find all orphaned pieces of the menu to cleanly destroy
   ****************************************************************************/   
   _destroyMenu: function()
   {
     dojo.disconnect( this._onCloseListener );
     
     delete this._onCloseListener;

	 for ( var i in this.menuItems )
	 {
	   if (i) {
	 	this.menuItems[ i ].destroy();
	   }
	 }
     
	 for ( i in this.subMenus )
	 {
	   if (i) {
	 	this.subMenus[ i ].destroy();
	   }
	 }

     this._menu.destroyRecursive();

     dojo.query( ".dijitPopup" ).forEach( function( domNode ) { dojo._destroyElement( domNode ); } );       
     
     delete this._menu;            
   },
   
   /****************************************************************************
   * This method must be called with a mouse event to show the Context menu.
   * You could pass in an object with x:xPos and y:yPos for event to trick the 
   * context menu to open at any given point on screen.
   *
   * The context object is any object that you want to send back to the server
   * or pass to the getMenuItems method.
   ****************************************************************************/
   contextMenuOpen: function ( event, context )
   {
      this._context = context;
      
      var menu = this.createContextMenu();

      menu._contextMenuWithMouse = true;
      menu._openMyself( event ); 
   },
   
   /****************************************************************************
   * Asks the JSON URL or callback method for the menu items then recursively
   * builds the menu.
   ****************************************************************************/   
   populateMenu: function( menu )
   {
     var model = null;
     
     if ( this.url !== null )
     { 
       model = this._loadMenuItems(); 
     }
     else
     {
       model = this.getMenuItems( this._context );
     }
     
     if ( model !== undefined && model !== null )
     {
       this._addItemsToMenu( menu, model );
       
       menu.startup();
     }               
   },
   
   /****************************************************************************
   * Recursive function to construct menu from array of items
   ****************************************************************************/
   _addItemsToMenu:function( parent, items )
   {
      var params;
      var subMenuIndex;
      var menuItemIndex;
      
      for ( var i in items )
      {
         if ( items[i].submenu_items )
         {
           subMenuIndex = this.subMenus.length;
           this.subMenus[ subMenuIndex ] = new dijit.Menu();
 
           params = 
           {
               popup: this.subMenus[ subMenuIndex ]
           };

           dojo.mixin( params, items[i] );
        
           parent.addChild( new dijit.PopupMenuItem( params ) );  
           
           this._addItemsToMenu( this.subMenus[ subMenuIndex ], items[i].submenu_items );           
         }
         else
         {
           // [2/15/08] The following if clause added for backward compatibility
           // with the previous version of TIPContextMenu
           if (items[i].id && items[i].id.toLowerCase() === "separator")
           {
              // Previous TIPContextMenu users indicated separators via the id
              items[i].actionID = "_separator_";
           } 
           else if (items[i].id && !items[i].actionID) 
           {
              // Previous TIPContextMenu users didn't use actionID
              items[i].actionID = items[i].id;
           }

           if ( items[i].actionID === "_separator_" )
           {
             menuItemIndex = this.menuItems.length;
             
             this.menuItems[ menuItemIndex ]  = new dijit.MenuSeparator();
             
             parent.addChild( this.menuItems[ menuItemIndex ] );
           }
           else
           {
             params = {
               adapter: this,
               onClick: function() { this.adapter.onMenuItemClick( this ); }
             };
             
             dojo.mixin( params, items[i] );

             menuItemIndex = this.menuItems.length;
             
             this.menuItems[ menuItemIndex ] = new dijit.MenuItem( params );
             
             parent.addChild( this.menuItems[ menuItemIndex ] );  
           }
         }         
      }
   },
      
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem ){ /*required overload*/ },
   
   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   * Calls the provided url with the following:
   *    name: this.name (unique name of context menu to retrieve)
   *    content: this._context (the context object passed to contextMenuOpen)
   *
   * Theoretically this method could be overloaded to do something other than
   * dojo.xhrPost()
   ****************************************************************************/
   _loadMenuItems: function()
   {
     var kw = {
            sync:         true,
            url:          this.url,
            name:         this.name,
            content:      this._context,
            handleAs:     "json",
            mimetype:     "application/json",            
            load:         dojo.hitch( this, function( response ) { this._menuitems = response; } )       
          };
     
     //call synchronously                                                                       
     dojo.xhrGet( kw );
     
     return this._menuitems;
   },  

   /****************************************************************************
   * This method will be called if 'url' parameter is not set.  Provides a means
   * for javascript to build the menu items.
   ****************************************************************************/
   getMenuItems: function( context ){ /* must overload to provide menu items if URL is not used for JSON */ }
   
});

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenuV2"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenuV2"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/







dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuV2");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProviderV2");
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProviderV2");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProxyV2");


/**
 *  Provides dynamic context menu support.
 *  How do you use this?
 *  First, you create an instance of this context menu similar to:
 *     this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenuV2({cmProvider: this.cmProvider});
 *  Note: you pass in an instance of a context menu provider (see objects lower in this file)
 *
 *  Second, you latch whatever event you want to create the context menu.  For 
 *  Grids, you might do something like this:
 *     this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
 *  Don't forget to disconnect this listener when your object is destroyed.
 *
 *  Third, you create the method that gets called by the above connect, and call
 *  the contextMenuOpen on this context menu with whatever context you want 
 *  processed.  Again, in a Grid example:
 *   _onRowContextMenu: function( e )
 *   { 
 *     var row = e.grid.model.getRow( e.rowIndex );
 *     
 *     if ( row )
 *     {
 *       //select the row that the context menu was initiated on
 *       e.grid.selection.unselectAll( e.rowIndex );
 *       e.grid.selection.setSelected( e.rowIndex, true );
 *               
 *       //tell the context menu adapter to show the menu
 *       // no context is provided because we overload the _loadMenuItems method
 *       this._contextMenuAdapter.contextMenuOpen( e, {selectedData: this.getSelectionData()} );
 *     }
 *   },
 *
 *  
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuV2",
              ibm.tivoli.tip.dijit.ContextMenuAdapter,
{
   cmProvider: null,       // TIPContextMenuProvider
   url: "no_url_needed",   // Note we have to pass something to ContextMenuAdapter
                           // for the url so that we can override _loadMenuItems
   
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem )
   { 
      // console.debug("-------------- menuItem :",menuItem);
      this.cmProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
   },

   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   ****************************************************************************/
   _loadMenuItems: function()
   {
     return this.cmProvider.getMenuItems( this._context );
   }

});

/**
 *  What is this?
 *  This is a helper for the TIPContextMenu.  It allows you pass in DOM nodes 
 *  and it will set up the context menu action for those nodes.  You must call 
 *  destroy on this object when you are done with it.  You provide the following
 *  when you create it:
 *   - targetNodeIds (required): these are string IDs for the DOM nodes you want 
 *                               context menus on
 *   - getContextCallback (optional): this is a function that will return any 
 *                                    context that you want passed along on the 
 *                                    context menu open call.  It will get 
 *                                    passed to your provider.
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProxyV2",
              ibm.tivoli.tip.dijit.TIPContextMenuV2,
{

   targetNodeIds: null,

   getContextCallback: null,
   
   _bindings: null,
   
   constructor: function() 
   {
      this._bindings = [];
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.bindDomNode));
   },

   // Called when it is time to notify the user that something was clicked.
   _openContextMenu: function(event)
   {
      var result = null;
      if (this.getContextCallback !== null) 
      {
         result = this.getContextCallback(event);
      }
      this.contextMenuOpen( event, result );
   },
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
         // <kjp> Changed this next line to point to my _openContextMenu.
         // The line previously pointed to _openMyself
			dojo.connect(cn, "oncontextmenu", this, "_openContextMenu"),
			dojo.connect(cn, "onkeydown", this, "_contextKey")
         // <kjp> I removed the next line (and removed the comma from the previous)
			//dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

   // This is a complete copy from dijit.Menu
	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
            //<kjp> I changed the following line from calling this_openMyself to this._openContextMenu
				window.setTimeout(dojo.hitch(this, function(){ this._openContextMenu(_e); }), 1);
			}
		}
	},

   destroy: function()
   {
   	console.log ("enter TIPContextMenuV2.destroy");
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.unBindDomNode));
      if (this.cmProvider !== null)
      {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }
   }
});


/**
 *  You provide a context menu provider to do things like populate the context menu with
 *  items, and to handle the actions when one of those items are clicked.
 *  You provide:
 *  - getMenuItemsCallback (required)   (optional alternative: override getMenuItems)
 *      Parameters: (context)
 *                 context (that was passed on the contextMenuOpen call (in your code)
 *
 *      Summary: This callback provides the menu items to be inserted into the context menu.          
 *      The following defines the callback that is used to return menu items to appear
 *      in the list.  This method should return JSON data in the form:
 *       [ 
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon", 
 *            "actionID": "testActionID" 
 *       },
 *       { 
 *            "actionID": "_separator_" 
 *       },
 *       { 
 *            "label": "submenu", 
 *            "iconClass": "tipMenuIcon", 
 *            "submenu_items": 
 *            [ 
 *             { 
 *                  "label": "Delete", 
 *                 "iconClass": "tipMenuIcon tipDeleteIcon", 
 *                 "actionID": "testDeleteActionID" 
 *             }
 *            ] 
 *       } 
 *       ]
 *
 *  - getMenuCommandCallback (optional)  (optional alternative: override getMenuCommand)
 *      Parameters: (commandName, context, menuItem)
 *                 commandName - the command name of the context menu item.  Provided on the
 *                               'actionID' item above. 
 *                 context - (that was passed on the contextMenuOpen call (in your code)
 *                 menuItem - the actual menu item object.
 *      This callback provides the function needed to process a menu item click.
 *      The response depends largely on the action you wish to take.  A sample response:
 *
 *      {
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *      }
 *      Launch types include: JavaScript, WEB_URL, CLIENT_EVENT, PORTLET, PORTAL_PAGE, NOOP
 *
 *      The actions listed above may be included in the original information returned by 
 *      the getMenuItemsCallback.  If these actions are included, you need not specify
 *      a getMenuCommandCallback, and the provider will process these actions for you
 *      automatically.  For example:
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon",
 *            "actionID": "testActionID", 
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *       },
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProviderV2", null,
{
   iscPortletNamespace: null,

   getMenuItemsCallback: null,
   
   getMenuCommandCallback: null,

   menus_: null, // List of menus saved from getMenuItems

   constructor: function( args ) 
   {
      // summary:
      //      The base class for all providers that populate the TIPContextMenu
      dojo.mixin( this, args );

      this.id_ = "1_";
   },

   getMenuItems: function(context)
   {
      this.menus_ = this.getMenuItemsCallback(context);
      return this.menus_;
   },

   getItem: function(/*String*/ id) 
   {
      var submenus = [];
      var currentMenu = null;
      var x = null;
      submenus.push(this.menus_);
      
      currentMenu = submenus.pop();
      while (currentMenu)
      {
         x = currentMenu.pop();
         while (x)
         {
            if (x) 
            {
               if (x.submenu_items)
               {
                  submenus.push(x.submenu_items);
               }
               else
               {
                  if (x.id == id || x.actionID == id)
                  {
                     return x;
                  }
               }
            }
            x = currentMenu.pop();
         }
         currentMenu = submenus.pop();
      }
   },
   
   isArray: function (obj) 
   {
      if (obj.constructor.toString().indexOf("Array") == -1) 
      {
         return false;
      } else {
         return true;      
      }
   },

   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
      }
      else
      {
         result = this.getItem(commandName);
      }

      // console.debug("------------ handleMenuAction result",result);
      this.handleMenuAction(result);
   },

   handleMenuAction: function (data, ioArgs) 
   {
      // console.info ("handleMenuAction: ");
      // console.info (data);

      if (data.launchType == "JavaScript") {
         this.handleJavascript (data);

      } else if (data.launchType == "WEB_URL") {
         this.launchURL (data);

      } else if (data.launchType == "CLIENT_EVENT") {
         this.sendEvent (data);

      } else if (data.launchType == "PORTLET") {
         this.launchPortlet (data);

      } else if (data.launchType == "PORTAL_PAGE") {
         this.launchPage(data);

      } else if (data.launchType == "NOOP") {
         // Do nothing in the case of a no operation type

      } else {
         console.error ("unsupported launchtype: " + data.launchType);
      }
   },

   handleJavascript: function (data) 
   {
      // console.log ("enter handleJavascript");
      var fnc = new Function ("eval (\"" + data.uri + "\");");
      fnc();
   },

   launchURL: function(data) 
   {
      // console.log ("enter launchURL");
      window.open (data.uri);
   },

   launchPortlet: function (data) 
   {
      // console.log ("enter launchPortlet");
      var portletEvent = {
                            name: "http://ibm.com/isclite#launchPortlet",
                            portletName: data.portletName,
                            portletApplicationID: data.portletAppUID
                         };

      if (data.parameters) 
      {
         for (var j=0; j<data.parameters.length; j++) 
         {
            eval ("portletEvent." + data.parameters[j].name + "= '" + data.parameters[j].value + "'");
         }
      }

      try 
      {
         EventBroker.sendEvent (this.iscPortletNamespace, portletEvent);
      } catch (e) {
         console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
         alert (e);
      }
   },
   
   launchPage: function (data) 
   {
      var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: data.portletPageID,
                         pageInstanceRef: null,
                         switchPage: true
                      };

      if (data.parameters) 
      {
         for (var k=0; k<data.parameters.length; k++) 
         {
            eval ("pageEvent." + data.parameters[k].name + "= '" + data.parameters[k].value + "'");
         }
      }

      try 
      {
         EventBroker.sendEvent (this.iscPortletNamespace, pageEvent);
      } catch (ex) {
         console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
         alert (ex);
      }
   },
   
   sendEvent: function (data) 
   {
      var parms = data.parameters;
      var event = {
                     name: data.event
                  };

      if (data.parameters) 
      {
         for (var i=0; i<parms.length; i++) 
         {
            eval ("event." + parms[i].name + "= '" + parms[i].value + "'");
         }
      }

	  console.log ("TIPContextMenuV2: sending event: ");            
	  console.log (event);
      EventBroker.sendEvent (this.iscPortletNamespace, event);
   },

   destroy: function()
   {
      // Placeholder: for overrides.  Exploiters should call this as best practice. 
   }
      
});

/**
 *  The CMSMenuProviderV2 is a specialized menu provider for use with the 
 *  context menu service.  Certainly you may use this provider if not using CMS,
 *  but it returns extra data that is used for CMS.  The CMS provider will
 *  make requests to the server to obtain both the menu items as well as the
 *  menu commands.
 *
 *  The CMSMenuProviderV2 expects the following to be included in the context 
 *  passed to contextMenuOpen
 *  {
 *     selectedData: <the selection data>
 *  }
 */
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProviderV2", 
             ibm.tivoli.tip.dijit.TIPContextMenuProviderV2, 
{
   global: null,
   rcfItems: null, 
   menuName: null, 
   parentid: null,
   
   // When the following is true, then this provider will first look 
   // at the menu items for a launchType, and if one exists, it will
   // execute that action here instead of going back to the host.
   hasLocalActions: true, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,

   getIdPrefix: function() 
   {
       return this.id_;
   },
   
   setParentId: function (parentid) 
   {
      this.parentid = parentid;
   },
   
   setResourceContextFilters: function (rcfItems) 
   {
      this.rcfItems = rcfItems;
   },
   
   getRCFURL: function (kw) 
   {
      if (this.rcfItems) 
      {
         for (var i in this.rcfItems) 
         {
            if (i) 
            {
               for (var name in this.rcfItems[i]) 
               {
                  if (name) 
                  {
                       kw.content["rcf_" + name]=this.rcfItems[i][name];
                  }
               }
            }
         }
      }   
   },

   getMenuItems: function(context) 
   {
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              "getMenuItems",
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      this.dataToURL (context.selectedData,kw);
      
      this.getRCFURL(kw);
         
      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );

      return this.menus_;
   },
   
   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
         this.handleMenuAction(result);
      }
      else
      {
         this.getMenuCommandNoCallback(commandName, context, menuItem);
      }
   },

   getMenuCommandNoCallback: function (commandName, context, menuItem) 
   {
      var actionHandled = false;
      if (this.hasLocalActions === true)
      {
         // Check if the menu item contains a launchType.  If so,
         // handle it locally.
	 var result = this.getItem(commandName);
         if (result && result.launchType)
         {
             actionHandled = true;
             this.handleMenuAction(result);
         }
      }
      
      if (actionHandled === false)
      {
         this.getMenuCommandFromServer(commandName, context, menuItem);
      }
   },
   
   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: "getMenuCommand",
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       this.dataToURL (context.selectedData,kw);
       this.getRCFURL(kw);
       
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
   },   
   
   dataToURL: function (selectedData, kw) 
   {
      // if you get an exception here, it means you did not provide
      // selection data in your context.  I'll put a catch of the 
      // exception here because I bet this happens a lot.
      var columns = null; 
      try 
      {
         columns = selectedData.columns;
      } catch (ex) {
         console.error ("Caught javascript exception in cmsContextMenuProvider.dataToURL: " + ex);
         console.error ("You must provide 'selectedData' on your context.");
         alert (ex);
      }
      var rows = selectedData.rows;
      var row = null;
      if (this.isArray (rows)) {
         row = rows[0];
      } else {
         row = rows;
      }
      
      for (var i=0; i<columns.length; i++) 
      {
         var rowField = columns[i].field;
         var value = row[columns[i].field];

         if (!value) 
         {
            value = "";
         }

         if (columns[i].isNamingAttribute === "true") 
         {
            kw.content["rn_"+columns[i].field]=value;
         } else {
            kw.content["sv_"+columns[i].field]=value;
         }
      }
         
      if (row.id) 
      {
        kw.content.arg_rowId = row.id;
      }
   }
   
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPDojoData"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPDojoData"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPDojoData");

// dojox.grid._data.model is the module with dojox.grid.data.DojoData


dojo.declare("ibm.tivoli.tip.dojo.TIPDojoData",
	          dojox.grid.data.DojoData,
{
//	returnTimes: 0,
	realCount: 0,
	
    // custom override of requestRows()
    requestRows: function (inRowIndex, inCount) {
         // creates serverQuery-parameter
         var row  = inRowIndex || 0;
         var params = {
	                start: row,
                    count: inCount || this.rowsPerPage,
                    serverQuery: dojo.mixin(
                      { start: row,
                        count: inCount || this.rowsPerPage,
                        sort:(this.sortColumn || '')
                      },
                      this.query
                    ),
                    query: this.query,
                    onBegin: dojo.hitch(this, "beginReturn"),
                    onComplete: dojo.hitch(this, "processRows")
                }
          this.store.fetch(params);
    },

	beginReturn: function (inCount) {
		console.log ("in beginReturn with inCount: " + inCount);
		this.setRowCount (inCount);
		this.realCount = inCount;
		dojo.publish ("/ibm.tip/TIPDojoData#rowCountUpdated", [this.id, inCount]);
	},

	processRows: function (items, store) {
		console.log ("in processRows with items: ");
		console.log (items);
		console.log ("and store: ");
		console.log (store);
		if (!items || items.length===0){
		    console.log("not processing any rows");
		    return;
		}
		this.inherited (arguments);
	},

	getRealRowCount: function () {
		return (this.realCount);
	},
       
    getRowCount: function () {
    	return (this.rowsPerPage > this.count ? this.rowsPerPage : this.count);
    },
    
    // src: String
    setData: function(inData) {
//    	this.returnTimes = 0;
//    	this.setRowCount(this.rowsPerPage + 1);
          console.log("in setData() with data: ");
          console.log(inData);
          // edited not to reset the store
          this.data = [];
          this.allChange();
    },
    sort: function(colIndex) {
    	console.log ("enter sort for col: " + colIndex);
//    	this.returnTimes = 0;
        this.clearData();
        this.sortColumn = colIndex;
        this.requestRows(); 
    },
    
    getSortColumn: function () {
    	return (this.sortColumn);
    },
    
    // Overriding this method only to prevent the '?' from showing up
	getDatum: function(inRowIndex, inColIndex){
		//console.debug("getDatum", inRowIndex, inColIndex);
		var row = this.getRow(inRowIndex);
		var field = this.fields.values[inColIndex];
		return row && field ? row[field.name] : field ? field.na : '...';
		//var idx = row && this.fields._nameMaps[inColIndex+".idx"];
		//return (row ? row[idx] : this.fields.get(inColIndex).na);
	},
    canSort: function () {
          return true;
    }
});

}

if(!dojo._hasResource["dojox.data.QueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.QueryReadStore"] = true;
dojo.provide("dojox.data.QueryReadStore");



dojo.declare("dojox.data.QueryReadStore", null, {
	/*
	//	summary:
	//		This class provides a store that is mainly intended to be used
	//		for loading data dynamically from the server, used i.e. for
	//		retreiving chunks of data from huge data stores on the server (by server-side filtering!).
	//		Upon calling the fetch() method of this store the data are requested from
	//		the server if they are not yet loaded for paging (or cached).
	//
	//		For example used for a combobox which works on lots of data. It
	//		can be used to retreive the data partially upon entering the
	//		letters "ac" it returns only items like "action", "acting", etc.
	//
	//  note:
	//      The field name "id" in a query is reserved for looking up data
	//      by id. This is necessary as before the first fetch, the store
	//      has no way of knowing which field the server will declare as
	//      identifier.
	//
	//	examples:
	//		// The parameter "query" contains the data that are sent to the server.
	//		var store = new dojox.data.QueryReadStore({url:'/search.php'});
	//		store.fetch({query:{name:'a'}, queryOptions:{ignoreCase:false}});
	//
	//		// Since "serverQuery" is given, it overrules and those data are
	//		// sent to the server.
	//		var store = new dojox.data.QueryReadStore({url:'/search.php'});
	//		store.fetch({serverQuery:{name:'a'}, queryOptions:{ignoreCase:false}});
	//
	//	todo:
	//		- there is a bug in the paging, when i set start:2, count:5 after an initial fetch() and doClientPaging:true
	//		  it returns 6 elemetns, though count=5, try it in QueryReadStore.html
	//		- allow configuring if the paging shall takes place on the client or the server
	//		- add optional caching
	//		- when the first query searched for "a" and the next for a subset of
	//		  the first, i.e. "ab" then we actually dont need a server request, if
	//		  we have client paging, we just need to filter the items we already have
	//		  that might also be tooo much logic
	*/
	
	url:"",
	requestMethod:"get",
	//useCache:false,
	
	// We use the name in the errors, once the name is fixed hardcode it, may be.
	_className:"dojox.data.QueryReadStore",
	
	// This will contain the items we have loaded from the server.
	// The contents of this array is optimized to satisfy all read-api requirements
	// and for using lesser storage, so the keys and their content need some explaination:
	// 		this._items[0].i - the item itself 
	//		this._items[0].r - a reference to the store, so we can identify the item
	//			securly. We set this reference right after receiving the item from the
	//			server.
	_items:[],
	
	// Store the last query that triggered xhr request to the server.
	// So we can compare if the request changed and if we shall reload 
	// (this also depends on other factors, such as is caching used, etc).
	_lastServerQuery:null,
	
	
	// Store a hash of the last server request. Actually I introduced this
	// for testing, so I can check if no unnecessary requests were issued for
	// client-side-paging.
	lastRequestHash:null,
	
	// If this is false, every request is sent to the server.
	// If it's true a second request with the same query will not issue another
	// request, but use the already returned data. This assumes that the server
	// does not do the paging.
	doClientPaging:true,

	// If this is false, all the sorting is done serverside before the data is returned
	// which is the proper place to be doing it for really large datasets.
	doClientSorting:true,

	// Items by identify for Identify API
	_itemsByIdentity:null,
	
	// Identifier used
	_identifier:null,

	_features: {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true},

	_labelAttr: "label",
	
	constructor: function(/* Object */ params){
		dojo.mixin(this,params);
	},
	
	getValue: function(/* item */ item, /* attribute-name-string */ attribute, /* value? */ defaultValue){
		//	According to the Read API comments in getValue() and exception is
		//	thrown when an item is not an item or the attribute not a string!
		this._assertIsItem(item);
		if (!dojo.isString(attribute)) {
			throw new Error(this._className+".getValue(): Invalid attribute, string expected!");
		}
		if(!this.hasAttribute(item, attribute)){
			// read api says: return defaultValue "only if *item* does not have a value for *attribute*." 
			// Is this the case here? The attribute doesn't exist, but a defaultValue, sounds reasonable.
			if(defaultValue){
				return defaultValue;
			}
			console.log(this._className+".getValue(): Item does not have the attribute '"+attribute+"'.");
		}
		return item.i[attribute];
	},
	
	getValues: function(/* item */ item, /* attribute-name-string */ attribute){
		this._assertIsItem(item);
		var ret = [];
		if(this.hasAttribute(item, attribute)){
			ret.push(item.i[attribute]);
		}
		return ret;
	},
	
	getAttributes: function(/* item */ item){
		this._assertIsItem(item);
		var ret = [];
		for(var i in item.i){
			ret.push(i);
		}
		return ret;
	},

	hasAttribute: function(/* item */ item,	/* attribute-name-string */ attribute) {
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		return this.isItem(item) && typeof item.i[attribute]!="undefined";
	},
	
	containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
		var values = this.getValues(item, attribute);
		var len = values.length;
		for(var i=0; i<len; i++){
			if(values[i]==value){
				return true;
			}
		}
		return false;
	},
	
	isItem: function(/* anything */ something){
		// Some basic tests, that are quick and easy to do here.
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem("");
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem({});
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem(0);
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem({name:"me", label:"me too"});
		// false
		//
		if(something){
			return typeof something.r!="undefined" && something.r==this;
		}
		return false;
	},
	
	isItemLoaded: function(/* anything */ something) {
		// Currently we dont have any state that tells if an item is loaded or not
		// if the item exists its also loaded.
		// This might change when we start working with refs inside items ...
		return this.isItem(something);
	},

	loadItem: function(/* object */ args){
		if(this.isItemLoaded(args.item)){
			return;
		}
		// Actually we have nothing to do here, or at least I dont know what to do here ...
	},

	fetch:function(/* Object? */ request){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
		//		only the paging, since it happens on the server if doClientPaging is
		//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
		//		Would be nice to be able to use simpleFetch() to reduce copied code,
		//		but i dont know how yet. Ideas please!
		request = request || {};
		if(!request.store){
			request.store = this;
		}
		var self = this;
	
		var _errorHandler = function(errorData, requestObject){
			if(requestObject.onError){
				var scope = requestObject.scope || dojo.global;
				requestObject.onError.call(scope, errorData, requestObject);
			}
		};
	
		var _fetchHandler = function(items, requestObject, numRows){
			var oldAbortFunction = requestObject.abort || null;
			var aborted = false;
			
			var startIndex = requestObject.start?requestObject.start:0;
			if (self.doClientPaging==false) {
				// For client paging we dont need no slicing of the result.
				startIndex = 0;
			}
			var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;
	
			requestObject.abort = function(){
				aborted = true;
				if(oldAbortFunction){
					oldAbortFunction.call(requestObject);
				}
			};
	
			var scope = requestObject.scope || dojo.global;
			if(!requestObject.store){
				requestObject.store = self;
			}
			if(requestObject.onBegin){
				requestObject.onBegin.call(scope, numRows, requestObject);
			}
			if(requestObject.sort && this.doClientSorting){
				items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
			}
			if(requestObject.onItem){
				for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
					var item = items[i];
					if(!aborted){
						requestObject.onItem.call(scope, item, requestObject);
					}
				}
			}
			if(requestObject.onComplete && !aborted){
				var subset = null;
				if (!requestObject.onItem) {
					subset = items.slice(startIndex, endIndex);
				}
				requestObject.onComplete.call(scope, subset, requestObject);   
			}
		};
		this._fetchItems(request, _fetchHandler, _errorHandler);
		return request;	// Object
	},

	getFeatures: function(){
		return this._features;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		// I have no idea if this is really needed ... 
	},

	getLabel: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item, this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
        },
	
	_fetchItems: function(request, fetchHandler, errorHandler){
		//	summary:
		// 		The request contains the data as defined in the Read-API.
		// 		Additionally there is following keyword "serverQuery".
		//
		//	The *serverQuery* parameter, optional.
		//		This parameter contains the data that will be sent to the server.
		//		If this parameter is not given the parameter "query"'s
		//		data are sent to the server. This is done for some reasons:
		//		- to specify explicitly which data are sent to the server, they
		//		  might also be a mix of what is contained in "query", "queryOptions"
		//		  and the paging parameters "start" and "count" or may be even
		//		  completely different things.
		//		- don't modify the request.query data, so the interface using this
		//		  store can rely on unmodified data, as the combobox dijit currently
		//		  does it, it compares if the query has changed
		//		- request.query is required by the Read-API
		//
		// 		I.e. the following examples might be sent via GET:
		//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?name=abc
		//
		//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?q=abc&c=true
		//		  // The serverQuery-parameter has overruled the query-parameter
		//		  // but the query parameter stays untouched, but is not sent to the server!
		//		  // The serverQuery contains more data than the query, so they might differ!
		//

		var serverQuery = request.serverQuery || request.query || {};
		//Need to add start and count
		if(!this.doClientPaging){
			serverQuery.start = request.start || 0;
			// Count might not be sent if not given.
			if (request.count) {
				serverQuery.count = request.count;
			}
		}
		if(!this.doClientSorting){
			if(request.sort){
				var sort = request.sort[0];
				if(sort && sort.attribute){
					var sortStr = sort.attribute;
					if(sort.descending){
						sortStr = "-" + sortStr;
					}
					serverQuery.sort = sortStr;
				}
			}
		}
		// Compare the last query and the current query by simply json-encoding them,
		// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
		if(this.doClientPaging && this._lastServerQuery!==null &&
			dojo.toJson(serverQuery)==dojo.toJson(this._lastServerQuery)
			){
			fetchHandler(this._items, request);
		}else{
			var xhrFunc = this.requestMethod.toLowerCase()=="post" ? dojo.xhrPost : dojo.xhrGet;
			var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
			xhrHandler.addCallback(dojo.hitch(this, function(data){
				data = this._filterResponse(data);
				if (data.label){
					this._labelAttr = data.label;
				}
				var numRows = data.numRows || -1;

				this._items = [];
				// Store a ref to "this" in each item, so we can simply check if an item
				// really origins form here (idea is from ItemFileReadStore, I just don't know
				// how efficient the real storage use, garbage collection effort, etc. is).
				dojo.forEach(data.items,function(e){ 
					this._items.push({i:e, r:this}); 
				},this); 
				
				var identifier = data.identifier;
				this._itemsByIdentity = {};
				if(identifier){
					this._identifier = identifier;
					for(i = 0; i < this._items.length; ++i){
						var item = this._items[i].i;
						var identity = item[identifier];
						if(!this._itemsByIdentity[identity]){
							this._itemsByIdentity[identity] = item;
						}else{
							throw new Error(this._className+":  The json data as specified by: [" + this.url + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
						}
					}
				}else{
					this._identifier = Number;
					for(i = 0; i < this._items.length; ++i){
						this._items[i].n = i;
					}
				}
				
				// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
				// (does it really sanititze them) and store the data optimal. should we? for security reasons???
				numRows = (numRows === -1) ? this._items.length : numRows;
				fetchHandler(this._items, request, numRows);
			}));
			xhrHandler.addErrback(function(error){
				errorHandler(error, request);
			});
			// Generate the hash using the time in milliseconds and a randon number.
			// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
			// probably just for esthetic reasons :-).
			this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
			this._lastServerQuery = dojo.mixin({}, serverQuery);
		}
	},
	
	_filterResponse: function(data){
		//	summary:
		//		If the data from servers needs to be processed before it can be processed by this
		//		store, then this function should be re-implemented in subclass. This default 
		//		implementation just return the data unchanged.
		//	data:
		//		The data received from server
		return data;
	},

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		It throws an error if item is not valid, so you can call it in every method that needs to
		//		throw an error when item is invalid.
		//	item: 
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){
			throw new Error(this._className+": Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute: 
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){ 
			throw new Error(this._className+": Invalid attribute argument ('"+attribute+"').");
		}
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// See if we have already loaded the item with that id
		// In case there hasn't been a fetch yet, _itemsByIdentity is null
		// and thus a fetch will be triggered below.
		if(this._itemsByIdentity){
			var item = this._itemsByIdentity[keywordArgs.identity];
			if(!(item === undefined)){
				if(keywordArgs.onItem){
					var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
					keywordArgs.onItem.call(scope, {i:item, r:this});
				}
				return;
			}
		}

		// Otherwise we need to go remote
		// Set up error handler
		var _errorHandler = function(errorData, requestObject){
			var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		};
		
		// Set up fetch handler
		var _fetchHandler = function(items, requestObject){
			var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
			try{
				// There is supposed to be only one result
				var item = null;
				if(items && items.length == 1){
					item = items[0];
				}
				
				// If no item was found, item is still null and we'll
				// fire the onItem event with the null here
				if(keywordArgs.onItem){
					keywordArgs.onItem.call(scope, item);
				}
			}catch(error){
				if(keywordArgs.onError){
					keywordArgs.onError.call(scope, error);
				}
			}
		};
		
		// Construct query
		var request = {serverQuery:{id:keywordArgs.identity}};
		
		// Dispatch query
		this._fetchItems(request, _fetchHandler, _errorHandler);
	},
	
	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = null;
		if(this._identifier === Number){
			identifier = item.n; // Number
		}else{
			identifier = item.i[this._identifier];
		}
		return identifier;
	},
	
	getIdentityAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentityAttributes()
		return [this._identifier];
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPQueryReadStore");




// Override QueryReadStore to make it work with paging
// See changes between: Begin Change - JSA
//                      End Change - JSA

dojo.declare("ibm.tivoli.tip.dojo.TIPQueryReadStore",
	          dojox.data.QueryReadStore,
{
    global: null,
    tableID: null,
    loadCompleteCallback: null, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	
    
    setLoadCompleteCallback: function(aFunction){
        this.loadCompleteCallback = aFunction;
    },
    
	fetch:function(/* Object? */ request){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
		//		only the paging, since it happens on the server if doClientPaging is
		//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
		//		Would be nice to be able to use simpleFetch() to reduce copied code,
		//		but i dont know how yet. Ideas please!
		console.log ("enter fetch w/ request:");
		console.log (request);
		request = request || {};
		if(!request.store){
			request.store = this;
		}
		var self = this;
	
		var _errorHandler = function(errorData, requestObject){
			if(requestObject.onError){
				var scope = requestObject.scope || dojo.global;
				requestObject.onError.call(scope, errorData, requestObject);
			}
		};
	
	    
		// Begin Change - JSA
		var _fetchHandler = function(items, requestObject, data){
		// End Change - JSA
			var oldAbortFunction = requestObject.abort || null;
			var aborted = false;
			
			var startIndex = requestObject.start?requestObject.start:0;
			if (self.doClientPaging==false) {
				// For client paging we dont need no slicing of the result.
				startIndex = 0;
			}
			var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;
	
			requestObject.abort = function(){
				aborted = true;
				if(oldAbortFunction){
					oldAbortFunction.call(requestObject);
				}
			};
	
			var scope = requestObject.scope || dojo.global;
			if(!requestObject.store){
				requestObject.store = self;
			}
			if(requestObject.onBegin){
				// Begin Change - JSA 
				requestObject.onBegin.call(scope, data.rowCount, requestObject);
				// End Change - JSA
			}
			if(requestObject.sort){
				items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
			}
			if(requestObject.onItem){
				for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
					var item = items[i];
					if(!aborted){
						requestObject.onItem.call(scope, item, requestObject);
					}
				}
			}
			if(requestObject.onComplete && !aborted){
				var subset = null;
				if (!requestObject.onItem) {
					subset = items.slice(startIndex, endIndex);
				}
				requestObject.onComplete.call(scope, subset, requestObject);   
			}
		};
		
		
		this._fetchItems(request, _fetchHandler, _errorHandler);
		return request;	// Object
	},
	
	_fetchItems: function(request, fetchHandler, errorHandler){
		//	summary:
		// 		The request contains the data as defined in the Read-API.
		// 		Additionally there is following keyword "serverQuery".
		//
		//	The *serverQuery* parameter, optional.
		//		This parameter contains the data that will be sent to the server.
		//		If this parameter is not given the parameter "query"'s
		//		data are sent to the server. This is done for some reasons:
		//		- to specify explicitly which data are sent to the server, they
		//		  might also be a mix of what is contained in "query", "queryOptions"
		//		  and the paging parameters "start" and "count" or may be even
		//		  completely different things.
		//		- don't modify the request.query data, so the interface using this
		//		  store can rely on unmodified data, as the combobox dijit currently
		//		  does it, it compares if the query has changed
		//		- request.query is required by the Read-API
		//
		// 		I.e. the following examples might be sent via GET:
		//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?name=abc
		//
		//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?q=abc&c=true
		//		  // The serverQuery-parameter has overruled the query-parameter
		//		  // but the query parameter stays untouched, but is not sent to the server!
		//		  // The serverQuery contains more data than the query, so they might differ!
		//

		var serverQuery = request.serverQuery || request.query || {};
		//Need to add start and count
		if(!this.doClientPaging){
			serverQuery.start = request.start || 0;
			// Count might not be sent if not given.
			if (request.count) {
				serverQuery.count = request.count;
			}
		}
		// Compare the last query and the current query by simply json-encoding them,
		// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
		if(this.doClientPaging && this._lastServerQuery!==null &&
			dojo.toJson(serverQuery)==dojo.toJson(this._lastServerQuery)
			){
			fetchHandler(this._items, request);
		}else{
			// var xhrFunc = this.requestMethod.toLowerCase()=="post" ? dojo.xhrPost : dojo.xhrGet;
			// var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
			var kResponseHandler = dojo.hitch(this, function(data){
				data=this._filterResponse(data);
				this._items = [];
				// Store a ref to "this" in each item, so we can simply check if an item
				// really origins form here (idea is from ItemFileReadStore, I just don't know
				// how efficient the real storage use, garbage collection effort, etc. is).
				dojo.forEach(data.items,function(e){ 
					this._items.push({i:e, r:this}); 
				},this); 
				
				var identifier = data.identifier;
				this._itemsByIdentity = {};
				if(identifier){
					this._identifier = identifier;
					for(i = 0; i < this._items.length; ++i){
						var item = this._items[i].i;
						var identity = item[identifier];
						if(!this._itemsByIdentity[identity]){
							this._itemsByIdentity[identity] = item;
						}else{
							throw new Error("dojo.data.QueryReadStore:  The json data as specified is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
						}
					}
				}else{
					this._identifier = Number;
					for(i = 0; i < this._items.length; ++i){
						this._items[i].n = i;
					}
				}
				
				// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
				// (does it really sanititze them) and store the data optimal. should we? for security reasons???
				// Begin Change - JSA
				fetchHandler(this._items, request, data);
				
				// If a callback is registered, then call it now.  Recall this is used to help
				// smoothly replace models on a refresh to avoid flashing.
				if (this.loadCompleteCallback) {
				    this.loadCompleteCallback();
				}
				// End Change - JSA
			});
			
			var kErrorHandler = function(error){
				errorHandler(error, request);
			};
			
			// Generate the hash using the time in milliseconds and a randon number.
			// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
			// probably just for esthetic reasons :-).
			this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
			this._lastServerQuery = serverQuery;

            var kw = {
                    content:   {
                                  command: "getTableRows",
                                  arg_fqTableId: this.tableID,
                                  arg_portletNamespace: this.global.namespace
                               },
                    timeout:   this.requestTimeout,
                    onResponse:{
                                  callback: kResponseHandler
                               },
                    onError:{
                                  callback: kErrorHandler
                               }
            };
            
            // Add server query stuff
            kw.content = dojo.mixin(kw.content,serverQuery);
            
            if (this.requestMethod.toLowerCase()=="post") {
                ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
            } else {
                ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
            }
			
		}
	},
	
	_filterResponse: function(data){
		//	summary:
		//		If the data from servers needs to be processed before it can be processed by this
		//		store, then this function should be re-implemented in subclass. This default 
		//		implementation just return the data unchanged.
		//	data:
		//		The data received from server
		return data;
	}
	
	
});
	

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/





//









// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTable");
dojo.provide("ibm.tivoli.tip.dijit.TIPTableContainer");
/******************************************************************************
* This container places the search bar and toolbar around a given 
* dojox.grid.Grid 
*
* Authors: Joseph Firebaugh, Joshua Allen, Ken Parzygnat
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableContainer",
            [ dijit._Widget, dijit._Templated ],
{
	templateString:"<div>\r\n<table dojoAttachPoint=\"tableContainer\" class=\"tableToolbar\" id=\"${id}tableContainer\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\r\n\t<tr dojoAttachPoint=\"wipeNode\">\r\n\t\t<td class=\"TIPTableToolbarArea\">\r\n\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t</td>\r\n\t    <td class=\"TIPTableSearchFieldArea\">\r\n\t\t    <div dojoType=\"ibm.tivoli.tip.dijit.TIPTableSearchField\" dojoAttachPoint=\"searchField\"></div>\r\n\t    </td>\r\n\t</tr>\r\n\t<tr class=\"TIPTableToggle\">\r\n\t\t<td colspan=\"2\">\r\n\t\t\t<div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' tabIndex=\"0\" dojoAttachEvent=\"onclick:toggleToolbar,onkeypress:onToggleKeyPress\"></div>\r\n\t\t</td>\r\n\t</tr>\r\n<tr>\r\n  <td id=\"${id}gridContainerCell\" dojoAttachPoint=\"gridContainerCell\" colspan=\"2\" height=\"200px\" valign=\"top\">\r\n\t<div id=\"${id}gridContainer\" dojoAttachPoint=\"gridContainer\" ></div>\r\n  </td>\r\n</tr>\r\n<tr>\r\n  <td colspan=\"2\" valign=\"top\">\r\n\t<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t\t<tbody>\r\n\t\t\t<tr class=\"isc-table-footer\" dir=\"ltr\">\r\n\t\t\t\t<td class=\"isc-table-footer-cap-l\" valign=\"bottom\" nowrap=\"nowrap\" align=\"left\">\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"footerToolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t\t\t</td>\r\n\t\t\t\t\r\n\t\t\t\t<td class=\"isc-table-footer-data\" nowrap=\"nowrap\" align=\"right\"> \r\n\t\t\t\t\t<div id=\"${id}footer\" dojoAttachPoint=\"footer\" ></div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td class=\"isc-table-footer-cap-r\" valign=\"bottom\" nowrap=\"nowrap\" align=\"right\">\r\n\t\t\t\t</td>\r\n\t\t\t</tr>  \r\n\t\t</tbody>\r\n\t</table>\r\n  </td>\r\n</tr>\r\n</table>\r\n</div>\r\n",	
	widgetsInTemplate: true,
	
	/* Do you want the default buttons added to the toolbar? */
	makeDefaultButtons: true,	

	/* Reference to the dojox.grid.Grid */
	grid: null,
	
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	
	multiSelect: true,
	rowDblClick: null,
	
	// open: Boolean
	//    Whether toolbar is opened or closed on first render.
	open: false,

	// duration: Integer
	//    Time in milliseconds to wipe in/wipe out toolbar
	duration: 300,
	
	// null means auto width
	// 100% takes all available width
	width: "100%",
	
	height: "200px",

	constructor: function () 
	{
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.tooltips = [];
		this.connections = [];
		this.topics = [];
	},
	
	/**************************************************************************
	* Called after template has been processed.
	*
	* Lets check to see if a dojox.grid.Grid was passed in on params.  If so,
	* add it to the DOM tree and setup.
	**************************************************************************/
	postCreate: function ()
	{
	    if( !this.open ) {
			this.wipeNode.style.display = "none";
	    }
		this._updateToggleClass();    
	
		//call super
		this.inherited( "postCreate", arguments );
		
		//if grid passed on params then set it
		if ( this.grid !== null )
		{
			this.setGrid( this.grid );
		}
		
		//initialize the toolbar
		if ( this.makeDefaultButtons === true ) 
		{ 
			this.initializeToolbar(); 
		}
		
		// set width
		if (this.width !== null) {
			this.setWidth (this.width);
		}
		
		// set height
		if (this.height !== null) {
			this.setHeight (this.height);
		}
		
		this.topics.push (dojo.subscribe (this.searchField.id + "searchInvoked", this, "onSearch"));
		this.setupAnimation();
		this.dojoNLSFix();
	},
	
	setupAnimation: function () {
	    // setup open/close animations
		var wipeNode = this.wipeNode;
        
	    this._wipeIn = dojo.fx.wipeIn({
	      node: this.wipeNode,
	      duration: this.duration,
	      beforeBegin: function(){
	      }
	    });
    
	    this._wipeOut = dojo.fx.wipeOut({
	      node: this.wipeNode,
	      duration: this.duration,
	      onEnd: function(){
	      }
	    });
	},
	
	/**************************************************************************
	* Can be called at any time to place a grid within this "container"
	**************************************************************************/
	setGrid: function( grid, append )
	{
		this.grid = grid;
		
		this.searchField.setTable( grid );

		if (append === false) 
		{
		} 
		else 
		{
		  this.gridContainer.appendChild( grid.domNode );
		}
		
		this.grid.selection.multiSelect = this.multiSelect;
	},
	
	setWidth: function (width) 
	{
		this.width = width;
		//dojo.byId (this.id + "tableContainer").width = this.width;
		if (this.tableContainer && this.tableContainer.width) {
			this.tableContainer.width = width;
		}
	},
	
	setHeight: function (height)
	{
		this.height = height;
//		this.gridContainerCell.height = this.height;
//		dojo.byId (this.id + "gridContainerCell").height = this.height;
	},
	
	/**************************************************************************
	* Get the dojox.grid.Grid object
	**************************************************************************/
	getGrid: function()
	{
		return this.grid;
	},
	
	/**************************************************************************
	* Get the dijit.Toolbar attached to the table
	**************************************************************************/
	getToolbar: function() 
	{
		return this.toolbar;
	},
	
	getFooterToolbar: function () 
	{
		return this.footerToolbar;
	},
	
	/**************************************************************************
	* Initialize the toolbar with default buttons (like refresh)
	**************************************************************************/
	initializeToolbar: function() 
	{
		var button = new ibm.tivoli.tip.dijit.TIPButton(
			{	label: this.resources_.REFRESH, 
				id: this.id + "refreshButton",
				showLabel: false, 
				iconClass: "refreshIcon",
			 	onClick: dojo.hitch (this, this.onRefresh) 
			});
		this.toolbar.addChild(button);
		
		if (this.multiSelect) {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.SELECT_ALL, 
					showLabel: false, 
					iconClass: "selectAllIcon",
				 	onClick: dojo.hitch (this, this.selectAll) 
				});
			this.toolbar.addChild(button);
			
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.DESELECT_ALL, 
					showLabel: false, 
					iconClass: "deselectAllIcon",
				 	onClick: dojo.hitch (this, this.deselectAll) 
				});
			this.toolbar.addChild(button);
		}
	},
	
	/**************************************************************************
	* This method fires a widgetID#Refresh event when the refresh buttons is
	* pressed.  OR you can overload this method instead. If you overload this
	* method you should call inherited so the table is cleared of contents.
	**************************************************************************/
	onRefresh: function() 
	{
//		this.setLoading();
		
		//create empty model
		this._model = new dojox.grid.data.DojoData(null, null);
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
	
	setLoading: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "refreshIcon");
			dojo.addClass (btn, "loadingIcon");
		}
	},
	
	setSteady: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "loadingIcon");
			dojo.addClass (btn, "refreshIcon");
		}
	},
	
	selectAll: function () 
	{
		var count = 0;
		if (this._model.getRealRowCount) {
			count = this._model.getRealRowCount();
		} else {
			count = this._model.getRowCount();
		}
		
		for (var i=0; i<count; i++) {
			this.grid.selection.addToSelection(i);
		}
		
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	deselectAll: function () 
	{
		this.grid.selection.unselectAll();
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	onSearch: function (tipSearchField, event) 
	{
		console.log ("enter onSearch with search value: " + tipSearchField.searchField.value);
		
		if (this.search) 
		{
			var columns = this.searchField.getSelectedColumns();
			this.search (columns, tipSearchField.searchField.value);
		}
	},
	onToggleKeyPress: function (event) {
	  	if (event.keyCode == 13) {
		  	event.cancelBubble = true;
	  		event.returnValue = false;
	  		console.log ("in onToggleKeyPress: pressed enter");
	  		this.toggleToolbar();
	  	}
	},
	
  /*****************************************************************************
  * The toggle open/close button div was clicked -- toggle open/close state
  *****************************************************************************/
  toggleToolbar: function()
  {
    dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
      if(animation.status() == "playing"){
        animation.stop();
      }
    });

    this[this.open ? "_wipeOut" : "_wipeIn"].play();
    this.open =! this.open;    
    this._updateToggleClass();    
	dojo.publish (this.id + "#ToggleToolbar", [ "ToggleToolbar" ]);
  },
  openToolbar: function () {
  	this.open = false;
  	this.toggleToolbar();
  },
  closeToolbar: function () {
  	this.open = true;
  	this.toggleToolbar();
  },
  
  /*****************************************************************************
  * based on the open/close state of widget change CSS to update button
  *****************************************************************************/
  _updateToggleClass: function()
  {
    if ( this.open === true )
    {
      dojo.removeClass( this.domNode, "close"  );
         dojo.addClass( this.domNode, "open" );          
    }
    else
    {
      dojo.removeClass( this.domNode, "open" );
         dojo.addClass( this.domNode, "close"  );          
    }  
  },
  
  resize: function () {
  	// placeholder... do nothing for now
  },
  
 	/************************************************************************
	* A nasty hack to fix TIPTable when using i.e. Arabic language . It's an issue from
	* Dojo code so as soon as they'll fix it this method ought to be deleted.
	* It isn't a good solution, I know that.
	  This solutions is borrowed from TCR. Thanks!
	************************************************************************/
	dojoNLSFix: function(context)
	{
		console.log("ibm.tivoli.tip.dijit.TIPTable.dojoNLSFix ENTRY");

		if(dojo.isFF) {
			dojo.query("#page-0", this.domNode).style("position","");
		} else if(dojo.isIE) {
			dojo.query("#dojox_GridView_0", this.domNode).style("position","relative");
		}

		console.log("ibm.tivoli.tip.dijit.TIPTable.dojoNLSFix EXIT");
	}
  
	
});

/******************************************************************************
* TIP Based Table that creates the underlying dojox.grid.Grid for you based on
* a given data store or Global that uses defined JSON returns for model data
* and column structure.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTable",
              ibm.tivoli.tip.dijit.TIPTableContainer,
{
	global: null,
	tableID: null,
	cmProvider: null,
	store: null,
	tableArgs: null,
	gridParms: {autoWidth: false, autoHeight:false, rowsPerPage: "10", rowCount: "5" },
	query: null,
	_contextMenu: null,
	_model: null,
   _onRowContextMenuListener: null,
   _contextMenuAdapter : null,

   clientViewStructure : null, // Provide if you want to provide view structure from client
   clientRfcItems : null,      // Provide if you are providing above view structure.
   autoSavePreferences : false,

  
	constructor: function() {
		this.lastPreferences = null;
	},
	
	postCreate: function () {
		this.inherited (arguments);
		this.topics.push (dojo.subscribe ("/ibm.tip/TIPDojoData#rowCountUpdated", this, "updateFooter"));
	},

	startup: function() {
		console.info ("enter TIPTable.startup");
		try {
			console.info (this.grid);
			if (!this.grid) {
				this.makeGrid();
			}

			if (this.store) {
				this.configureTableModel();
				this.afterViewLoad();
			} else if (this.global) {
				this.getView();
				this.getStore();
//  		dojo.connect (this.grid, 'onmouseover', this, 'getSelectedData');
			}
			
			this.grid.startup();
			if (this.autoSavePreferences) {
				this.connections.push (dojo.connect (this.grid, "resize", this, "onResize"));
			}
		} catch (ex) {
			console.error ("Caught exception in TIPTable.startup(): ");
			console.error (ex);
		}
		
		console.info ("exit TIPTable.startup");
	},
	
	afterViewLoad: function (rcfItems) {
		console.log ("Enter TIPTable.afterViewLoad");
		if (this.cmProvider) {
			this.configureContextMenu(rcfItems);  			
	  	}	  	
	},

	makeGrid: function () {
		console.info ("enter makeGrid");
		this.setGrid( new dojox.Grid(this.gridParms, dojo.byId(this.id + "gridContainer")), false);
		this.connections.push (dojo.connect (this.grid, "onRowClick", this, "onRowClick"));
		this.connections.push (dojo.connect (this.grid, "onRowDblClick", this, "onRowDblClick"));
	},
  
   getContextMenuAdapter: function()
   {
      if (this._contextMenuAdapter === null) {
         // Create the contextMenuAdapter.
         this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenuV2({cmProvider: this.cmProvider});
      }
      return this._contextMenuAdapter;
   },
  
    /************************************************************************
    * This method is called each time a context menu is requred on a row
    ************************************************************************/
    _onRowContextMenu: function( e )
    { 
      var row = e.grid.model.getRow( e.rowIndex );
      
      if ( row )
      {
        //select the row that the context menu was initiated on
        e.grid.selection.unselectAll( e.rowIndex );
        e.grid.selection.setSelected( e.rowIndex, true );
                
        //tell the context menu adapter to show the menu
        // no context is provided because we overload the _loadMenuItems method
        this.getContextMenuAdapter().contextMenuOpen( e, {selectedData: this.getSelectionData()} );
      }
    },

	configureContextMenu: function (rcfItems) {
		if ((this.cmProvider.parentid === null) || (this.cmProvider.parentid === "")) {
			this.cmProvider.setParentId (this.id);
		}
		if ((this.cmProvider.rcfItems === null) && (rcfItems)) {
			this.cmProvider.rcfItems = rcfItems;
		}

      this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
      	 //FIXES CONTEXT MENU BUG with selected table rows IN DOJO 1.0.2
		 this.grid.focus.styleRow = function(inRow){};
      
	},
	
	getView: function () {
      if (this.clientViewStructure === null) {
          var kw = {
                      content:   {
                                    command: "getTableColumns",
                                    arg_fqTableId: this.id,
                                    arg_portletNamespace: this.global.namespace
                                 },
                      timeout:   this.requestTimeout,
                      onResponse:{
                                    callback: dojo.hitch(this, this.getViewCallback)
                                 }
                   };

          if (this.tableArgs) {
             for (var key in this.tableArgs) {
                if (key) {
                      kw.content[key] = this.tableArgs[key];
                }	
             }
          }

          ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
      }
      else
      {
         this.setupView( this.clientViewStructure, this.clientRfcItems );
      }
	},
	
	handleError: function (response, ioArgs) {
		console.error ("HTTP status code: ", ioArgs.xhr.status);
		return response;
	},
	
	handleTimeout: function (response, ioArgs) {
		console.error ("Timeout for request: " + ioArgs.xhr);
		return response;
	},
	
	setupView: function (structure, rcfItems) {
		this.grid.setStructure( structure );	
		this.searchField.setTable( this.grid );
 	    this.afterViewLoad(rcfItems);
	},
  
	getViewCallback: function (data, ioArgs) {
      var view1 = { cells: [ data.cells ] };
      // a grid layout is an array of views.
      var layout = [ view1 ];
      this.setupView( layout, data.rcfItems );
	},
  
	getStore: function () {
	  	console.info ("enter getStore");
		// TODO: Change doClientPaging:false when paging is fixed
	  	this.store = new ibm.tivoli.tip.dojo.TIPQueryReadStore ({ global: this.global, tableID: this.id, jsId: "jsonStore", doClientPaging:false, requestTimeout: this.requestTimeout });
  		console.log ("got the store: ");
	  	console.log (this.store);
//	  	this.store.fetch();
	  	this.configureTableModel();
  	},
  	
	onRefresh: function() {
//		this.setLoading();
	
		//create a new model to force a refresh
		//this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: true });

		//clear table
		//this.grid.setModel( this._model );
		
		//post publish event
		//dojo.publish (this.id + "#Refresh", [ "Refresh" ]);

        // What is going on here?
        // Previously, the preceding three lines were used in this function to refresh
        // the table.  Create a new model, set the model, and issue the event.  However,
        // the problem is that TIPTable gets its data from the server, so the new model
        // was set prior to it having any data.  This resulted in the table 
        // "flashing".  That is the table would go all blank, then turn around and fill
        // with the data from the server.  This didn't look good, so the below code fixes
        // this.  A callback was added to the store so that when it finishes handling
        // data from the server, it will call the callback.  This enables us to first 
        // create the model, and load its first page, then swap it in to the table.
        
        // Create the callback function that will swap in the new model when it has some data
        var cb = dojo.hitch(this,function(){  console.debug("----- Inside the refresh callback ----");
                                              this.store.setLoadCompleteCallback(null);
                                              this._model=this.my_new_model; 
                                              this.grid.setModel(this.my_new_model);
                                              // I found that the following update is needed when only
                                              // one row is in the model.
                                              this.grid.update();
                                              this.my_new_model = null; 
                                              this.updateFooter(this._model.id);
                                              dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
                                            });
                                            
        this.store.setLoadCompleteCallback(cb);  // set the callback

		this.my_new_model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: false });
		// preserve sort on a refresh
		this.my_new_model.sortColumn = this._model.sortColumn;
		
		// Now need to set the row count equal to the rowsPerPage and make a request for the first page
		// in order to pre-load the model.  Notice that when this request completes, our callback above
		// will be invoked.
		this.my_new_model.setRowCount(20);
		this.my_new_model.getRow(0);
		
	},
	
	refreshWithArgs: function (args) {
		this.tableArgs = args;
		var dataQuery = {query:this.query};
	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
	  				dataQuery[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}


		//create a new model to force a refresh
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:dataQuery, clientSort: false });
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
  	
  	configureTableModel: function () {
  		console.info ("enter TIPTable.configureTableModel");
  		this.query = '';
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: false });
		this.grid.setModel( this._model );
		console.info ("exit TIPTable.configureTableModel");
	},
	
	search: function (columns, value) {
		console.log ("enter search with columns: ");
		console.log (columns);
		console.log ("and value: ");
		console.log (value);
		this.query = '';
		for (var x in columns) {
			if (this.query === '') {
				this.query += columns[x];
			} else {
				this.query += "," + columns[x];
			}
		}
		this.query += ":" + value;
		console.log ("in search with query: " + this.query);
		// preserve sort
		var sort = this._model.sortColumn;
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: false });
		this._model.sortColumn = sort;
		
		//clear table
		this.grid.setModel( this._model );
	},

	getSelectedRowIndexes: function () {
		return this.grid.selection.getSelected();
	},
	
	getValueAt: function (row, column) {
		return this.grid.model.getDatum (row, column);
	},
	
	getViewColumnsIDs: function () {
	  	var columns = this.grid.structure[0].cells[0];
	  	var columnIDs = [];
	  	for ( var i in columns ) {
	  		if (i) {
				columnIDs.push (columns[i].field);
			}
		}
		
		return (columnIDs);
	},
	
	getAllColumnIndexes: function () {
		var count = this.grid.model.getColCount();
		var columnIDs = [];
		for (var i=0; i<count; i++) {
			columnIDs.push (i);
		}
		
		return (columnIDs);
	},
	
	isNamingAttribute: function (field) {
	  	var columns = this.grid.structure[0].cells[0];
	  	if (columns[field]) {
		  	return columns[field].isNamingAttribute;
		} else {
			return (false);
		}
	},

	getModelData: function () {
		var selectedRows = this.getSelectedRowIndexes ();
		var row = selectedRows[0];
		var columnIDs = this.getAllColumnIndexes();

		for (var column in columnIDs) {
			if (column) {
				var data = this.grid.model.getDatum (row, column);
				console.log ("cell value for row: " + row + ", and column: " + column + " : " + data);
			}
		}
	},
	
	getIdentifier: function () {
//	  	var columns = this.grid.structure[0].cells[0];
	  	return (this.grid.model.getColCount() - 1);
	},

	getSelectionData: function () {
		this.getModelData();
		var selectedRows = this.getSelectedRowIndexes ();
		console.log ("selectedRows: " + selectedRows);

		var rowData = null;
		var columnIDs = this.getAllColumnIndexes();
		var identifier = this.getIdentifier();
		var rowJSON = "{ ";
		
		dojo.forEach (selectedRows,
			function (row) {
				var firstTime = true;
				rowJSON += '"id": "' + this.getValueAt (row, identifier) + '", ';
			
				for (var column in columnIDs) {
					if (column) {
						console.log ("getting cell value for row: " + row + " and column: " + column);
						var cellValue = this.getValueAt (row, column);
						if (!firstTime) {
							rowJSON += ', ';
						} else {
							firstTime = false;
						}
						rowJSON += '"' + column + '": "' + cellValue + '"';
					}
				}
			}, this);

		rowJSON += "}";
		
		var columnsJSON = "";
		var firstLoop = true;
		for (var c in columnIDs) {
			if (c) {
				if (!firstLoop) {
					columnsJSON += ', ';
				} else {
					firstLoop = false;
				}
				columnsJSON += '{ "field": "' + c + '", "isNamingAttribute": "' + this.isNamingAttribute(c) + '" }';
			}
		}
		
		var resultStr = "result = {";
		resultStr += "columns: [ ";
		resultStr += columnsJSON;
		resultStr += " ], rows: ";
		resultStr += rowJSON;
		resultStr += "};";
		
		console.log ("resultStr: " + resultStr);

		var result = eval (resultStr);
		
//		var sample = { 
//		columns: [{ field: "srcToken", hidden: true },
//				  { field: "ManagedSystemName", label: "Managed System Name" },
//				  { field: "column_2", label: "baz" }],
//		
//		 rows: { srcToken: "1234", ManagedSystemName: "bar", column_2: "bang" }
//		 		};
		
		return (result);
	},

	onRowClick: function (evt) {
		console.log ("enter TIPTable.onRowClick");
		this.updateFooter (this._model.id);
	},

    // The following processes double clicks on a row.  The user will add the double
    // click context on construction such as:
    // var dblClickContext = { content: { command: "myDblClickCommand" },
    //                         callback: dojo.hitch(this, this.myDblClickAction)  
    //                       };
    // and include it on the declaration of TIPTable with "rowDblClick: dlbClickContext".
    // Acceptable attributes:
    //     content: (optional) content to be sent to the callback
    //     callback: (required) a javascript function to be called.  It can have the form:
    //             myfunction: function(command, table)
    //                   where command is the command passed in the context,
    //                         table is a reference to this table
    
	onRowDblClick: function (evt) { 
		console.log ("enter TIPTable.onRowDblClick");
		if (this.rowDblClick) {
		    if (this.rowDblClick.callback) {
		        this.rowDblClick.callback(this, this.rowDblClick.content);
		    }
		}
	},

	updateFooter: function (modelID) {
		if (modelID == this._model.id) {
			var numSelected = 0;
			var total = this._model.getRealRowCount();
			var selected = this.getSelectedRowIndexes();
			if (selected) {
				numSelected = selected.length; 
			}
			var footerText = dojo.string.substitute(this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: total });
			this.footer.innerHTML = '<p class="footerLabel">' + footerText +  '</p>';
		}
	},
	
	getPreferences: function () {
		console.log ("enter TIPTable.getPreferences");
		var preferences = null;
		
		if (this.grid.structure) {
			var ids = this.getViewColumnsIDs();
			preferences = '{ "columns" : [ ';
		
			for (var x in ids) {
				if (x) {
					preferences += ' {"id": "' + x + '", "width":"' + this.grid.getCell(x).unitWidth + '"} ';
					if (x < ids.length - 1) {
						preferences += ", ";
					}
				}
			}
			preferences += " ] }";
			console.log ("cell widths: ");
			console.log (preferences);
		} else {
			console.log ("grid has no structure yet");
		}
				
		return (preferences);
	},
	
	onResize: function () {
		console.log ("enter TIPTable.onResize");
		// make sure that the grid has structure already.
		var preferences = this.getPreferences();
		
		if (this.autoSavePreferences && 
			(preferences !== this.lastPreferences)) {
		         var kw = {
                      content:   {
                                    command: "saveTablePreferences",
                                    arg_fqTableId: this.id,
                                    arg_portletNamespace: this.global.namespace,
                                    arg_preferences: preferences
                                 },
                      timeout:   this.requestTimeout
                   };

   	      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
          this.lastPreferences = preferences;
        }
	},
	
	
	
	destroy: function () {
		console.info ("enter TIPTable.destroy");
		dojo.forEach (this.tooltips, function (tt) {
			tt.destroy();
		});
		dojo.forEach (this.topics, function (topic) {
			dojo.unsubscribe (topic);
		});
		dojo.forEach (this.connections, function (conn) {
			dojo.disconnect (conn);
		});
		
		// remove the context menu because it lives outside of this domNode		
		if (this._contextMenu) {
			this._contextMenu.removeAllItems();
			this._contextMenu.destroy();
			dijit.registry.remove (this._contextMenu.id);
		} else {
			console.info ("no context menu");
		}

      if (this._onRowContextMenuListener) {
         dojo.disconnect(this._onRowContextMenuListener);
      }
      this._contextMenuAdapter = null;
      if (this.cmProvider) {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }

		this.inherited (arguments);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
/*******************************************************************************
 * CMVC Location: %W%
 * Version:       %I%
 * Last Modified: %G%
 *******************************************************************************/

dojo.provide("ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget");













// NLS




// define the widget class
dojo.declare(
	"ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget", 
    [ dijit._Widget, dijit._Templated ],
{
    divToChange_ : "connectionManagerDiv",
    namespace_ : "namespace1_",
    iscPortletNamespace : null,
    callback_ : null,
    global: null,
  	// widgetsInTemplate Boolean:
	//		should we parse the template to find widgets that might be
	//		declared in markup inside it? false by default.
	widgetsInTemplate: true,
	templateString:"<div id=\"${id}connectionManager\" class=\"ConnectionManager\">\r\n\r\n\t<div id=\"${id}listConnectionsDiv\" class=\"tableContainer\">\r\n\t\t<label for=\"${id}tbl_listConnections\" title=\"${resources_.SELECT_DATA_SOURCE}\">\r\n\t\t</label>\r\n\t\t<table id=\"${id}tbl_listConnections\" class=\"tipTable\"></table>\r\n\t</div>\r\n\r\n\t<div id=\"${id}connectionProperties\" \r\n\t\t dojoType=\"dijit.Dialog\"\r\n\t\t dojoAttachPoint=\"dialog\" \r\n\t\t title=\"${resources_.NEW_CONNECTION}\"\r\n\t\t style=\"width: 450px;\"\r\n\t >\r\n\t\t<table cellpadding=\"3\" cellspacing=\"0\" border=\"0px\" class=\"chartsDialog\" style=\"padding: 15px\">\r\n\t\t\t<tr>\r\n\t\t\t\t<td colspan=\"2\">\r\n\t\t\t\t\t<p class=\"descriptionNoIndent\">\r\n\t\t\t\t\t\t${resources_.NEW_CONNECTION_DESC}\r\n\t\t\t\t\t</p>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}name\" title=\"${resources_.NAME}\">\r\n\t\t\t\t\t${resources_.NAME}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<span>\r\n\t\t\t\t\t<input id=\"${id}name\" type=\"text\" dojoType=\"dijit.form.ValidationTextBox\"\r\n\t\t\t\t\t\t\trequired=\"true\" />\r\n\t\t\t\t\t</span>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}protocol\" title=\"${resources_.PROTOCOL}\">\r\n\t\t\t\t\t${resources_.PROTOCOL}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<select id=\"${id}protocol\" name=\"${id}protocol\">\r\n\t\t\t\t\t\t<option value=\"http\">${resources_.HTTP}</option>\r\n\t\t\t\t\t\t<option value=\"https\">${resources_.HTTPS}</option>\r\n\t\t\t\t\t</select>\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}hostname\" title=\"${resources_.HOSTNAME}\">\r\n\t\t\t\t\t${resources_.HOSTNAME}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<input id=\"${id}hostname\" type=\"text\" dojoType=\"dijit.form.ValidationTextBox\"\r\n\t\t\t\t\t\t\trequired=\"true\" />\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}port\" title=\"${resources_.PORT}\">\r\n\t\t\t\t\t${resources_.PORT}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<input id=\"${id}port\" type=\"text\" dojoType=\"dijit.form.NumberTextBox\"\r\n              constraints=\"{min:1024,max:65000,places:0,strict: true,pattern: '#,######0.0#'}\"\r\n              invalidMessage=\"${messages_.TIPCH3009E}\"\r\n              promptMessage=\"${resources_.ITM_PORT_RANGE_MESSAGE}\"\r\n\t\t\t\t\t\t\trequired=\"true\" />\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}service_name\" title=\"${resources_.SERVICE_NAME}\">\r\n\t\t\t\t\t${resources_.SERVICE_NAME}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<input id=\"${id}service_name\" type=\"text\" dojoType=\"dijit.form.ValidationTextBox\"\r\n\t\t\t\t\t\t\trequired=\"true\" />\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t\r\n<!-- // Remove for now, not used as all apps within TIP should be SSO\r\n     // ITM is only exception and usr/pwd handled by installer.\r\n\t // May be introduced at later point.\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}username\" title=\"${resources_.USERNAME}\">\r\n\t\t\t\t\t${resources_.USERNAME}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<input id=\"${id}username\" type=\"text\" dojoType=\"dijit.form.TextBox\" />\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n\r\n\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<label class=\"label\" for=\"${id}password\" title=\"${resources_.PASSWORD}\">\r\n\t\t\t\t\t${resources_.PASSWORD}\r\n\t\t\t\t\t</label>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<input id=\"${id}password\" type=\"password\" dojoType=\"dijit.form.TextBox\" />\r\n\t\t\t\t</td>\r\n\t\t\t</tr>\r\n-->\r\n\t\t</table>\r\n\r\n\t\t<div dojoType=\"dijit.Toolbar\" style=\"padding: 0px\">\r\n\t\t\t<button \r\n\t\t\t\tid=\"${id}createBtn\" \r\n\t\t\t\tclass=\"tipButton\" \r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\" \r\n\t\t\t\tvalue=\"Create\"\r\n\t\t\t\tlabel=\"${resources_.CREATE}\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\r\n\t\t\t<button id=\"${id}cancelBtn\" \r\n\t\t\t\tclass=\"tipButton\" \r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\" \r\n\t\t\t\tvalue=\"Cancel\"\r\n\t\t\t\tlabel=\"${resources_.CANCEL}\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\t\t</div>\r\n\r\n\t</div>\r\n\t\r\n</div>\r\n",

	constructor: function() {
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_messages");
	    
	    this.connections = [];
	},
	initialize: function() { 
	},
		
	postCreate: function() {
		this.inherited ("postCreate", arguments);
		this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		var createButton = dojo.byId (this.id + "createBtn");
        this.connections.push (dojo.connect(createButton, 'onclick', this, 'onCreateClick'));

		var cancelButton = dojo.byId (this.id + "cancelBtn");
        this.connections.push (dojo.connect(cancelButton, 'onclick', this, 'onCancelClick'));

        this.createConnectionsTable ();
	},
	onCreateClick: function() {
		var name = dojo.byId(this.id + "name").value;
//		var protocol = dojo.byId(this.id + "protocol").value;
		var protocolSelect = dojo.byId (this.id + "protocol"); 
		var protocol = protocolSelect.options[protocolSelect.selectedIndex].value; 
		
		var hostname = dojo.byId(this.id + "hostname").value;
    var portField = dijit.byId(this.id + "port");
		var port = portField.getValue();
		var serviceName = dojo.byId(this.id + "service_name").value;
		// Remove for now, not used as all apps within TIP should be SSO
		// ITM is only exception and usr/pwd handled by installer.
		// May be introduced at later point.
		//var username = dojo.byId(this.id + "username").value;
		//var password = dojo.byId(this.id + "password").value;		
		var username = "";
		var password = "";
	console.debug("--isInrange="+portField.isInRange()+" isValid="+portField.isValid()+" port="+port);	
		if ((!name) || (!protocol) || (!hostname) || (!serviceName)) {
			this.showRequiredFieldsDialog();
    } else if ( !isNaN(port) && !portField.isInRange() ) {
      this.showFieldValueOutOfRangeDialog(); 
    } else if ( !portField.isValid() ) {
      this.showInvalidFieldValueDialog(); 
		} else {
			var kw = {
	             content:   {
                        command: "saveConnection",
						arg_name: encodeURI(name),
						arg_protocol: protocol,
						arg_hostname: encodeURI(hostname),
						arg_port: port,
						arg_service_name: encodeURI(serviceName),
						arg_username: encodeURI(username),
						arg_password: password
                        },
            	 onResponse:{
                         callback: dojo.hitch(this, this.updateConnectionsTable)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
			this.showListConnections();
//			dojo.publish("/com.ibm.tivoli.tip.charts", [ "saveConnection" ]);
		}
	},
	showRequiredFieldsDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
        var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3006E,
	        	                                      messageId: "TIPCH3006E"
	                    	                        }); 
		modalWindow.show();
	},
  showInvalidFieldValueDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3009E,
	        	                                      messageId: "TIPCH3009E"
	                    	                        }); 
		modalWindow.show();
  },
  showFieldValueOutOfRangeDialog: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
		                                              portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3010E,
	        	                                      messageId: "TIPCH3010E"
	                    	                        }); 
		modalWindow.show();
  },
	onCancelClick: function () {
		dojo.byId(this.id + "name").value = "";
		var protocolSelect = dojo.byId (this.id + "protocol"); 
		protocolSelect.selectedIndex = 0;
//		dojo.byId(this.id + "protocol").value = "";
		dojo.byId(this.id + "hostname").value = "";
		dojo.byId(this.id + "port").value = "";
		dojo.byId(this.id + "service_name").value = "";
		//dojo.byId(this.id + "username").value = "";
		//dojo.byId(this.id + "password").value = "";		

		this.showListConnections();
//		dojo.publish("/com.ibm.tivoli.tip.charts", [ "cancelConnection" ]);
	},
	newConnection: function () {
//		var listConns = dojo.byId (this.id + "listConnectionsDiv");
//		listConns.style.display = 'none';
//		var newConn = dojo.byId (this.id + "connectionProperties");
//		newConn.style.display = '';
		this.dialog.show();
		
//		dojo.publish("/com.ibm.tivoli.tip.charts", [ "newConnection" ]);
	},
	deleteConnection: function () {
		var connectionName = this.getSelectedDataSourceName();

		if (connectionName) {
			var kw = {
	             content:   {
                        command: "deleteConnection",
						arg_connectionName: encodeURI(connectionName)
                        },
            	 onResponse:{
                         callback: dojo.hitch(this, this.updateConnectionsTable)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    	    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
													  portletTitle: helper.getPortletTitle(this.domNode),
	    	                                          message: this.messages_.TIPCH3007E,
	        	                                      messageId: "TIPCH3007E"
	                    	                        }); 
			modalWindow.show();
		}
	},
	showListConnections: function () {
//		var listConns = dojo.byId (this.id + "listConnectionsDiv");
//		listConns.style.display = '';
//		var newConn = dojo.byId (this.id + "connectionProperties");
//		newConn.style.display = 'none';
		this.dialog.hide();
	},
	selectConnection: function (connectionName) {
		console.info ("enter selectionConnection with name: " + connectionName);
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		console.info ("got tbl: ");
		console.info (tbl);
		var row = null;

		loop:
		for (var rowIndex=0; rowIndex < tbl.getModel().getRowCount(); rowIndex++) {
			console.info ("checking row: " + rowIndex);
			var tblConnectionName = tbl.getValueAt (rowIndex, 0);
			if (tblConnectionName == connectionName) {
		    	console.info ("found connectionName in table... selecting row: " + rowIndex);
		    	tbl.getGrid().selection.select (rowIndex);
		    	break loop;
		    }
		}
		
		console.info ("exit selectConnection");
	},
	getSelectedDataSourceName: function () {
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		var selectedRow = tbl.getSelectedRowIndexes();
		console.log ("ConnectionManagerWidget.getSelectedDataSourceName. selectedRows: " + selectedRow);
		if ((selectedRow) && (selectedRow.length > 0)) {
			var connectionName = tbl.getValueAt (selectedRow, 0);
			console.log ("ConnectionManagerWidget.getSelectedDataSourceName.  connectionName: " + connectionName);
			return (connectionName);
		} else {
			console.log ("ConnectionManagerWidget.getSelectedDataSourceName: no row selected");
			return (null);
		}
	},
	//  Update the filtering table identified by view with fresh data from the backend.
	createConnectionsTable: function () {
		var global = ibm.tivoli.tip.dojo.Global.getSingleton (this.namespace_);
		
		var cmProvider = new ibm.tivoli.tip.dijit.CMSMenuProviderV2(
				{ global: global,
				  menuName: "listConnectionsMenu",
				  parentid: this.id,
				  hasLocalActions: false,
				  iscPortletNamespace: this.iscPortletNamespace
				});
	
		var tipTable = new ibm.tivoli.tip.dijit.TIPTable({
					global: global,
					tableID: "tbl_listConnections",
					multiSelect: false,
					cmProvider: cmProvider,
					onModelAllChange: this.callback_,
					width: "100%",
					gridParms: {autoWidth: false, autoHeight:false, rowsPerPage: "10", rowCount: "5" },
					id: this.id + "tbl_listConnections"
				}, 
				dojo.byId (this.id + "tbl_listConnections"));
				
		tipTable.startup();
//		var footerToolbar = tipTable.getFooterToolbar().getToolbar();
		var footerToolbar = tipTable.getFooterToolbar();

		var back = new ibm.tivoli.tip.dijit.TIPButton ({
							label: this.resources_.BACK,
							showLabel: true,
							onClick: dojo.hitch (this, this.onBack)
						});
		footerToolbar.addChild (back);
		
		var next = new ibm.tivoli.tip.dijit.TIPButton ({
							label: this.resources_.NEXT,
							showLabel: true,
							onClick: dojo.hitch (this, this.onNext)
						});
		footerToolbar.addChild (next);
		
		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		console.log ("got context:");
		console.log (context);
		if ( context.roles.admin ) {
			console.log ("user is a chartAdministrator. constructing new/delete connections buttons");
			var toolbar = tipTable.getToolbar();

			var button = new ibm.tivoli.tip.dijit.TIPButton(
			{	
				iconClass: "newIcon",
				label: this.resources_.NEW,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.newConnection) 
			});
			toolbar.addChild(button);
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
			{	
				iconClass: "deleteIcon",
				label: this.resources_.DELETE,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.deleteConnection) 
			});
			toolbar.addChild(button);
		} else {
			console.log ("user is not a chartAdministrator");
		}
		
		dojo.subscribe (tipTable.id + "#LoadComplete", this, "onTableLoad");
		this.resize();
	},
	onTableLoad: function () {
		// We have to do this to accommodate a bug in TIPTable where toolbars and footers 
		// don't appear until after the data is loaded and resize is called again
		console.log ("enter ConnectionManagerWidget.onTableLoad");
		this.resize();
		
		// We have to refresh to data again to accommodate a bug in TIPTable selection
		dijit.byId (this.id + "tbl_listConnections").onRefresh();
		console.log ("exit ConnectionManagerWidget.onTableLoad");
	},
	resize: function () {
		var portletSize = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
		var size = { w: portletSize.available.w, h: portletSize.available.h };
		if (size.w === undefined) {
			// guess
			size.w = 850;
		} else if (isNaN(size.w)) {
			// guess
			size.w = portletSize.current.w;
		} 
		if (size.h === undefined) {
			// guess
			size.h = 200;
		} else if (isNaN(size.h)) {
			// guess
			size.h = 200;
		} 
		
		console.log ("ConnectionManagerWidget.resize: portlet size: ");
		console.log (portletSize);
		console.log ("size: ");
		console.log (size);
		//dijit.byId (this.namespace_ + "mainChartNavigatorDiv").resize(size);
		dijit.byId (this.id + "tbl_listConnections").resize (size);
	},
	onBack: function () {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showChartNavigator();
	},
	onNext: function () {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showCMChartSelector();
	},
	updateConnectionsTable: function (data, ioArgs) {
		var tbl = dijit.byId (this.id + "tbl_listConnections");
		tbl.onRefresh();
	},
	destroy: function () {
		console.log ("enter ConnectionManagerWidget.destroy");
	
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});
		this.dialog.destroy();
		
//		this.destroyDescendants();
		this.inherited ("destroy", arguments);
		console.log ("exit ConnectionManagerWidget.destroy");
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dojo.Global"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dojo.Global"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/

dojo.provide("ibm.tivoli.tip.charts.dojo.Global");

dojo.require ("ibm.tivoli.tip.dijit.TIPMessageDialog");
// NLS


/****************************************************************************
* Based on TCR ibm.tivoli.tcr.Global
* Global 
*
* (Q) What is this?
* (A) Since the TIP application will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.charts.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.charts.dojo.Global.stdDateFormat = "%c %z";
ibm.tivoli.tip.charts.dojo.Global.instances     = [];
ibm.tivoli.tip.charts.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
****************************************************************************/
ibm.tivoli.tip.charts.dojo.Global.setSingleton = function (namespace, context) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.charts.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.charts.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
   var singleton = new ibm.tivoli.tip.charts.dojo._Global( context );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};

ibm.tivoli.tip.charts.dojo.Global.getSingleton = function( namespace, context )
{
  var singleton = ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.charts.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.charts.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
    singleton = new ibm.tivoli.tip.charts.dojo._Global( context );
   
    //save the new singleton instance on the Global memory space
    ibm.tivoli.tip.charts.dojo.Global.instances[ namespace ] = singleton; 
  }  
  
  return( singleton ); 
};

/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.charts.dojo._Global",null,
{  
 requests: 0,
 queuedRequests: 0,
 
 constructor: function( context )
 {
      //add all functions, fields, objects inside context to this object
   dojo.mixin( this, context );
   
   this.controller = this.contextualURL( "/TIPChartController" );
   this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
 },
 
 /************************************************************************
 * Generate Conextual URL
 ************************************************************************/
 contextualURL : function( baseURL ) 
 {
    return( this.contextPath + baseURL );
 },

 /************************************************************************
 * Generate Namespaced ID from a BASE ID.
 ************************************************************************/
 namespaceID: function( baseID ) 
 {
    return( this.namespace + baseID );
 }, 
 
 /************************************************************************
 * Funnel all AJAX requests through global object
 ************************************************************************/
 submitRequest: function( requestKW )
 {
   //total requests made
   this.requests++;
   
   //current running requests
   this.queuedRequests++;
   
   //show wait dialog
   if ( this.queuedRequests == 1 )
   {
    //this.wait_timer = setInterval( dojo.lang.hitch( this, "showWaitDialog" ), 100 );
   }
   
   if ( requestKW.appendURL )
   {
     requestKW.url          = this.controller + requestKW.appendURL;
   }
   else
   {
  requestKW.url          = this.controller;
   }
   
   requestKW.handleAs    = "json";
   
   requestKW.mimetype       = "application/json";
 
   requestKW.timeoutSeconds = 0;  // TODO:  0 = no timeout
   requestKW.timeout        = dojo.hitch( this, this.requestTimeoutHandler );
 
   requestKW.error          = dojo.hitch( this, this.requestErrorHandler ); 
 
   requestKW.loadCallback   = requestKW.load; 
   requestKW.load           = dojo.hitch( this, this.requestLoadHandler ); 
   
   requestKW.preventCache   = true; //fix IE cache bug
 
   if ( requestKW.transport && ( requestKW.transport == ibm.tivoli.tip.charts.dojo.Global.IFrame ) ) 
   {
     this.IFrameRequestKW = requestKW;
   }
 
   dojo.xhrGet( requestKW ); 
 },
 
 /************************************************************************
 * Standard JSON request "load" handler - check for server-side errors
 * returned in the 'exc' field.  Forward to caller's "load" handler if none.
 ************************************************************************/
    requestLoadHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
   
   var kw = obj.args;

   if ( this.queuedRequests === 0 )
   {
  //clearInterval( this.wait_timer );
    //hide_wait_dialog( this.context.nameSpace );
   }   
   
   // check for availability of request KW; IFrame-based transport handled specially
   // see above method
   if ( ! kw )
   {
     if ( this.IFrameRequestKW )
     {
         kw = this.IFrameRequestKW;
       delete this.IFrameRequestKW;
     }
   }
 
   // was an exception detected and reported?
   if ( obj.exc )
   { 
     var exc = obj.exc;
 
     // ISC session timeout?
     if ( exc == "ISCSessionTimeoutException" ) 
     {
       this.refreshPage();
 
       return;
     }
     
     // some other server-side processing exception :-(  Exception handler/list defined?
     if ( kw && kw.excHandler && kw.excHandled )
     {
       if ( ( kw.excHandled == ibm.tivoli.tip.charts.dojo.Global.anyException ) || // handle ANY exc  
           ( kw.excHandled.indexOf( exc ) != -1 ) )     // handle this exc?
       {
         kw.excHandler( exc, obj, kw );  
 
         return;
       }
     }
 
     // exc message available?
     var excMsg = obj.excMsg ? obj.excMsg : "",
 
         msgMax = 500;
 
     if ( excMsg.length > msgMax ) 
     {
       excMsg = excMsg.substring( 0, msgMax ) + "...";
     }
 
     var mKW = { 
                messageId :                        this.nlsResources.BIND_ERROR_ID,
                message   : dojo.string.substitute(this.nlsResources.BIND_ERROR_MSG, { 0 : exc, 1 : excMsg } )
               } ;
 
     this.showErrorMessage( mKW );                                         
   }
 
   // No exception --> forward to originally specified "load" callback
   else
   {
     kw.loadCallback( data, obj );
   }
 },
 
 /************************************************************************
 * show error message dialog
 ************************************************************************/
 showErrorMessage: function( kwMessage )
 {
   //var mb = new TcrMsgBox( kwMessage );
 
   //mb.showError();
   
   console.debug( "Error occured" );
   console.debug( kwMessage );
 },

 /************************************************************************
 * Standard JSON request error handler
 ************************************************************************/
 requestErrorHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
 
   if ( this.queuedRequests === 0 )
   {
    //clearInterval( this.wait_timer );
    //hide_wait_dialog( this.context.nameSpace );
   }   
 
   this.showErrorMessage( 
                          { 
                            messageId :                        this.nlsResources.BIND_ERROR_ID,
                            message   : dojo.string.substitute(this.nlsResources.BIND_ERROR_MSG, { 0 : obj.message } )
                          } 
                        );                                         
 },

 /************************************************************************
 * Standard JSON request timeout handler
 ************************************************************************/
 requestTimeoutHandler: function( data, obj ) 
 {
   //decrement the total queued requests -- if 0 then close wait dialog
   this.queuedRequests--;
 
   if ( this.queuedRequests === 0 )
   {
     //clearInterval( this.wait_timer );   
    //hide_wait_dialog( this.context.nameSpace );
   }   
 
   this.showErrorMessage( 
                          { 
                           messageId : this.nlsResources.BIND_TIMEOUT_ID,
                           message   : this.nlsResources.BIND_TIMEOUT_MSG
                          } 
                        );                                         
 },
 
 /************************************************************************
 * Format a DATE with NLS rules
 ************************************************************************/ 
 formatDate: function( date ) 
 {
    var formattedDate = dojo.date.strftime( date, ibm.tivoli.tip.charts.dojo.Global.stdDateFormat );
  
    return( formattedDate );
 },
 
 /************************************************************************
 * Refresh the page
 ************************************************************************/ 
 refreshPage: function()
 {
    window.location.reload( false );
 },
 
 handleServerException: function (exception) 
 {
 	console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
	console.error ("with server stack trace: ");
	console.error (exception.stackTrace);
	this.showMessageDialog (exception.type, exception.messageID, exception.messageString, false);
 },
 
 showMessageDialog: function (type, msgID, msg, showCheckbox) {
	var closeText = this.resources_.CLOSE;
	var callback = null;
	if (!showCheckbox) {
		showCheckbox = false;
		callback = null;
	} else {
		callback = dojo.hitch (this, this._onShowMessageClose);
	}
	
    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
	        	                                      showCheckbox: showCheckbox,
	        	                                      callback: callback,
	            	                                  buttons: [ closeText ]
	                    	                        }); 
	modalWindow.show();
},

_onShowMessageClose: function (button, messageID, checked) {
	if (checked) {
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=setMessagePreference&arg_messageID=" + messageID + "&arg_neverShow=" + checked + "&arg_userName=" + this.username;
	    dojo.xhrGet({
                       url: url,
					    handleAs: "json-comment-filtered",
	          			error: dojo.hitch (this, this.handleError),
			  			timeout: dojo.hitch (this, this.handleTimeout)
                    });
    }
},

getAncestorByName: function (element, elementName) {
	var result=null;
	try {
		while(result===null && element!==null) {
			element=element.parentNode;
			if(element.nodeName.toLowerCase()==elementName.toLowerCase()) {
				result=element;
			}
		}
	} catch(e) {
		//no node
	}
	//console.log ("Nodename = " + result);
	return(result);
},
getAncestorByClass: function (element, className) {
	var result=null;
	try {
		while(result===null && element!==null) {
			element=element.parentNode;
			if(dojo.hasClass (element, className)) {
				result=element;
			}
		}
	} catch(e) {
		//no node
	}
	//console.log ("Nodename = " + result);
	return(result);
}

 
  
});

//Declare a new Error type to throw when a Global context error is detected
dojo.declare("ibm.tivoli.tip.charts.dojo.Global.InvalidContextError", Error, {});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbBar");
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbSeparator");






dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
//		this.destroyDescendants();
//		this.inherited (arguments);
	}
}
);

dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbSeparator",
	[ dijit._Widget, dijit._Templated ],
{
	// summary
	//	A line between two menu items
	templateString: '<div class="breadcrumbDivider dijitInline">&gt;</div>',
	postCreate: function(){ dojo.setSelectable(this.domNode, false); },
	isFocusable: function(){ return false; }
});

dojo.declare(
	"ibm.tivoli.tip.dijit.Breadcrumb",
	[ dijit.form.Button ],
{
	templateString:"<div class=\"dijit dijitLeft dijitInline breadcrumb\"\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",

	constructor: function () {
	},
	postCreate: function() {
		this.inherited (arguments);
	},
	destroy: function () {
//		this.destroyDescendants();
//		this.inherited (arguments);
	}
});


}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartSelectorWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartSelectorWidget"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartSelectorWidget");












// NLS



// define the widget class
dojo.declare (
	"ibm.tivoli.tip.charts.dijit.ChartSelectorWidget", 
    [ dijit._Widget, dijit._Templated ],
{
    datasource_ : "",
    namespace_ : "namespace1_",
    groupID_ : "",
    chartID_ : "",
    populateGroups: true,
    global: null,
    widgetsInTemplate: true,
    templateString:"<div>\r\n<div id=\"${id}chartSelectorDiv\">\r\n     <div dojoType=\"ibm.tivoli.tip.dijit.BreadcrumbBar\">\r\n \t\t  <div dojoType=\"ibm.tivoli.tip.dijit.Breadcrumb\"\r\n\t\t    \tlabel=\"${resources_.CHART_SOURCE_TYPE}\"\r\n\t\t    \tdojoAttachEvent=\"onClick:showChartNavigator\"\r\n\t\t    >\r\n\t\t  </div>\r\n\t  \t  <div dojoType=\"ibm.tivoli.tip.dijit.BreadcrumbSeparator\">\r\n\t\t  </div>\r\n\t  \t  <div dojoType=\"ibm.tivoli.tip.dijit.Breadcrumb\"\r\n\t\t    \tlabel=\"${resources_.CONNECTION_MANAGER}\"\r\n\t\t    \tdojoAttachEvent=\"onClick:showConnectionManager\"\r\n\t\t    >\r\n\t\t  </div>\r\n\t</div>\r\n\r\n  \t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" class=\"contentHeader\" width=\"100%\" style=\"padding:10px;\">\r\n\t    \t  \r\n  \t  <tr valign=\"top\" align=\"left\">\r\n  \t  <td>\r\n  \t      <span class=\"importChartBig\">\r\n  \t      </span>\r\n  \t  </td>\r\n  \t  <td align=\"left\" width=\"100%\">\r\n\t  \t  <span class=\"contentHeaderTitle\">\r\n\t     \t${resources_.IMPORT_CHART}\t\t \r\n\t  \t  </span>\r\n\t      <p class=\"descriptionNoIndent\">\r\n\t\t     ${resources_.SELECT_CHART_TO_IMPORT_DESCRIPTION}\r\n    \t  </p>\r\n      </td>\r\n\t  </tr>\r\n\t</table>\r\n\r\n\r\n  \t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width=\"100%\" style=\"padding:8px;\" class=\"chartsDialog\" >\r\n  \t<tr>\r\n  \t<td>\r\n\r\n  \t<div dojoType=\"dijit.Toolbar\" style=\"padding-left:5px\">\r\n\t  <label for=\"${id}groupSelector\" title=\"${resources_.GROUPS}\">\r\n\t  ${resources_.GROUPS}\r\n\t  </label>\r\n\t  <br/>\r\n\t  <select \r\n\t\t  id=\"${id}groupSelector\" \r\n\t\t  name=\"${id}groupSelector\" \r\n\t\t  size=\"1\"\r\n\t  >\r\n\t  </select>\r\n\t  <button \r\n\t\tid=\"${id}groupRefresh\"\r\n\t\tname=\"${id}groupRefresh\" \r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:onRefreshGroup\"\r\n\t\t\t\ticonClass=\"refreshIcon\"\r\n\t\t\t\tlabel=\"${resources_.REFRESH}\"\r\n\t\t\t\tshowLabel=\"false\"\r\n\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\t>\r\n\t  </button>\t\r\n\t</div>\r\n\t\r\n\t</td>\r\n\t</tr>\r\n\t</table>\r\n\r\n\t<label for=\"${id}tbl_applicationCharts\" title=\"${resources_.CHARTS}\">\r\n\t</label>\r\n\t\r\n\t<div id=\"${id}tbl_applicationCharts\">\r\n\t</div>\r\n\t\r\n\t<div style=\"display:none\">\r\n\t\t<input dojoAttachPoint=\"hChartID\" type=\"text\" name=\"${id}chartSelector\" />\r\n\t</div>\r\n</div>\r\n</div>\r\n",

	constructor: function() {
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");	
	    this.chartsTable = null;
	    this.connections = [];
	},
	initialize: function() { 
	},
	postCreate: function() {
		this.inherited(arguments);
		this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		
//		var selectGroupButton = dojo.byId (this.id + "selectGroupBtn");
//        dojo.connect(selectGroupButton, 'onclick', this, 'onSelectGroup');
		this.connections.push (dojo.connect(dojo.byId(this.id + 'groupSelector'),'onchange', this, 'onSelectGroup'));

		this.makeChartsTable();        
        
        if (this.populateGroups) {
	        this.populateSelectGroup ();
	    }
	},
	setDataSource: function (datasource) {
		if (this.datasource_ != datasource) {
			this.datasource_ = datasource;
			this.clearSelectChart();
			this.clearSelectGroup();
			this.populateSelectGroup();
		}
	},
	showConnectionManager: function () {
		dojo.publish ("/com.ibm.tivoli.tip.charts/" + this.namespace_ + "#showConnectionManager", []);
	},
	showCursorWait: function() {
		if (dojo.byId (this.id + "chartSelectorDiv")) {
			dojo.byId (this.id + "chartSelectorDiv").style.cursor = "wait";
//			dojo.byId (this.id + "selectGroupBtn").style.cursor = "wait";
//			dojo.byId (this.id + "selectChartBtn").style.cursor = "wait";
			dojo.byId (this.id + "tbl_applicationCharts").style.cursor = "wait";
			dojo.byId (this.id + "groupSelector").style.cursor = "wait";
			dojo.byId (this.id + "groupRefresh").style.cursor = "wait";
		}
	},
	showCursorDefault: function() {
		if (dojo.byId (this.id + "chartSelectorDiv")) {
			dojo.byId (this.id + "chartSelectorDiv").style.cursor = "default";
//			dojo.byId (this.id + "selectGroupBtn").style.cursor = "default";
//			dojo.byId (this.id + "selectChartBtn").style.cursor = "default";
			dojo.byId (this.id + "tbl_applicationCharts").style.cursor = "default";
			dojo.byId (this.id + "groupSelector").style.cursor = "default";
			dojo.byId (this.id + "groupRefresh").style.cursor = "default";
		}
	},
	
	
	populateSelectGroup: function() {
		this.showCursorWait();
		var howTo = { showMsg : true, callback: dojo.hitch (this, this.onError) };
   
		var kw = {
             content:   {
                         command: "listGroups",
                         arg_datasource: encodeURI(this.datasource_)
                        },
             onExc: howTo,
             onResponse:{
                         callback: dojo.hitch(this, this.populateSelectGroupCallback)
                        }
         };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	onError: function () {
		console.log ("enter ChartSelectorWidget.onError");
		this.showCursorDefault();
	},
	clearSelectGroup: function () {
		var groupSelector = dojo.byId (this.id + "groupSelector");
		groupSelector.options.length = 0;
	},
	populateSelectGroupCallback: function(data, ioArgs) {
		console.log ("populating selectgroup.  groupID: " + this.groupID_);

		if (data.objectType === "serverException") {
			var context = ibm.tivoli.tip.charts.dojo.Global.getSingleton(this.namespace_);
			context.handleServerException (data);
		}
				
		var groupSelector = dojo.byId (this.id + "groupSelector");
		var selected = false;
		var val = null;
		this.clearSelectGroup();

		groupSelector.options[0] = new Option(this.resources_.SELECT_GROUP, this.resources_.SELECT_GROUP);
		for (var i=1; i<data.length+1; i++) {
			groupSelector.options[groupSelector.options.length] = new Option (data[i-1].label, data[i-1].id);
			if (data[i-1].id == this.groupID_) {
				selected = true;
			}
		}
		this.showCursorDefault();

		console.log ("populateSelectGroupCallback. selected: " + selected);
		console.log ("populateSelectGroupCallback. groupSelector entries: " + groupSelector.options.length);
		if ((!selected) && (groupSelector.options.length > 0)) {
			console.log ("populateSelectGroupCallback. selecting");
			groupSelector.options[0].selected = true;
			selected = true;
		}
				
		if (selected) {
			this.onSelectGroup();
		}
	},
	onSelectGroup: function() {
		var group = dojo.byId(this.id + "groupSelector").value;
		// set the value for submit so Portlet Prefs can pick it up.
		dojo.byId (this.namespace_ + "groupIDH").value = group;
		this.updateChartsTable(group);
	},
	onRefreshGroup: function() {
		this.showCursorWait();
		var howTo = { showMsg : true, callback: dojo.hitch (this, this.onError) };
   
		var kw = {
             content:   {
                         command: "listGroups",
                         arg_datasource: encodeURI(this.datasource_),
						 arg_clearcache: "true"
                        },
             onExc: howTo,
             onResponse:{
                         callback: dojo.hitch(this, this.populateSelectGroupCallback)
                        }
         };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},	
	clearSelectChart: function () {
		var args = [];
		args.clear = true;
	
		this.chartsTable.refreshWithArgs(args);
		dojo.byId (this.namespace_ + "chartIDH").value = "";
//		var chartSelector = dojo.byId (this.id + "chartSelector");
//		chartSelector.options.length = 0;
	},
//	populateSelectChartCallback: function (data, ioArgs) {
//		console.log ("enter populateSelectChartCallback; chartID: " + this.chartID_);
//		if (data.objectType === "serverException") {
//			var context = ibm.tivoli.tip.charts.dojo.Global.getSingleton(this.namespace_);
//			context.handleServerException (data);
//		}
		
//		var chartSelector = dojo.byId (this.id + "chartSelector");
//		var selected = false;

//		console.log ("in populateSelectChartCallback. data length: " + data.length);
//		this.clearSelectChart();
		
//		for (var i=0; i<data.length; i++) {
//			console.log ("adding chart: " + data[i].id);
//			chartSelector.options[chartSelector.options.length] = new Option (data[i].label, data[i].id);
//			if (data[i].id == this.chartID_) {
//				chartSelector.options[i].selected = true;
//				console.log ("selected chart: " + data[i].id);
//				selected = true;
//			} else {
//				console.log ("chart: " + data[i].id + " does not match " + this.chartID_);
//			}
//		}
//		this.showCursorDefault();
//
//		if (selected) {
//			this.onSelectChart();
//		}
//	},
	onSelectChart: function() {
		console.log ("enter onSelectChart");		
		var group = dojo.byId(this.id + "groupSelector").value;
		var selectedRow = this.chartsTable.getSelectedRowIndexes();
		if ((selectedRow) && (selectedRow.length > 0)) {
			var chartID = this.chartsTable.getValueAt (selectedRow, 3);
			var chartName = this.chartsTable.getValueAt (selectedRow, 1);
			console.log ("calling getChartDefinition with group: " + group + ", chartID: " + chartID);
			// copy the chart ID to a hidden field so it can be read by the server on a post
			dojo.byId (this.namespace_ + "chartIDH").value = chartID;
	        this.getChartDefinition(group, chartID, chartName);
	    }
	},
	getChartDefinition: function (group, chartID, chartName) {
		this.showCursorWait();
//		var chartWidget = dijit.byId (this.namespace_ + "chartWidget");
//		var errorHandler = { showMsg : true, callback: dojo.hitch (chartWidget, chartWidget.onError) };
		var howTo = { showMsg : true, callback: dojo.hitch (this, this.onError) };


		var kw = {
             content:   {
                         command: "getChartDefinition",
                         arg_datasource: encodeURI(this.datasource_),
						 arg_groupID: encodeURI(group),
						 arg_chartName: encodeURI(chartName),
						 arg_chartID: encodeURI(chartID)
                        },
	         onExc: howTo,
             onResponse:{
                         callback: dojo.hitch(this, this.populateGetChartDefinitionCallback)
                        }
         };

        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	populateGetChartDefinitionCallback: function (data, ioArgs) {
//		var group = dojo.byId(this.id + "groupSelector").value;
		this.onWebServiceChartSelection(data[0].chartSourceType, data[0].connectionName, data[0].groupID, data[0].id);
	},
	onWebServiceChartSelection: function (chartSrcType, connectionName, groupID, chartID) {
		var clearPreferences = true;
		this.showCursorDefault();		
		var selectedRow = this.chartsTable.getSelectedRowIndexes();
		var chartName = null;
		if ((selectedRow) && (selectedRow.length > 0)) {
			chartName = this.chartsTable.getValueAt (selectedRow, 1);
		}
		console.log ("in onWebServiceChartSelection with chartName: " + chartName);
		
		dijit.byId (this.namespace_ + "chartWidget").onReportSelection (
			chartSrcType,
			connectionName,
			groupID,
			chartID,
			chartName,
			clearPreferences,
			true);		
	},
	showConnectionManager: function () {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showConnectionManager();
	},
	showChartNavigator: function() {
		dijit.byId (this.namespace_ + "mainChartNavigatorDiv").showChartNavigator();
	},
	makeChartsTable: function () {
		console.log ("enter makeChartsTable");
		
		var global = ibm.tivoli.tip.dojo.Global.getSingleton (this.namespace_);
		this.chartsTable = new ibm.tivoli.tip.dijit.TIPTable({
					global: global,
					tableID: "tbl_applicationCharts",
					multiSelect: false,
					cmProvider: null,
					width: "100%",
					gridParms: {autoWidth: false, autoHeight:false, rowsPerPage: "10", rowCount: "5" },
					id: this.id + "tbl_applicationCharts"
				}, 
				dojo.byId (this.id + "tbl_applicationCharts"));
				
		this.chartsTable.startup();
		this.contributeToolbar();
		this.contributeFooter();
		dojo.subscribe (this.chartsTable.id + "#LoadComplete", this, "onTableLoad");
		this.resize();
		console.log ("exit makeChartsTable");
	},
	onTableLoad: function () {
		console.log ("enter ConnectionManagerWidget.onTableLoad");
		this.resize();
	},
	resize: function () {
		console.log ("enter ChartSelectorWidget.resize");
		var portletSize = iscPortlet.getPortletSize(this.global.iscPortletNamespace);
		var size = { w: portletSize.available.w, h: portletSize.available.h };
		if (size.w === undefined) {
			// guess
			size.w = 800;
		} else if (isNaN(size.w)) {
			// guess
			size.w = portletSize.current.w;
		} 
		if (size.h === undefined) {
			// guess
			size.h = 200;
		} else if (isNaN(size.h)) {
			// guess
			size.h = 200;
		} 
		
		console.log ("size: ");
		console.log (size);
		dijit.byId (this.id + "tbl_applicationCharts").resize (size);
	},
	contributeToolbar: function () {
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		if (context.roles.admin) {
			var toolbar = this.chartsTable.getToolbar();
			var button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.CLEAR_CACHE, 
					id: this.id + "clearCacheButton",
					showLabel: true, 
				 	onClick: dojo.hitch (this, this.clearDesignCache) 
				});
			toolbar.addChild(button);
		}
	},
	contributeFooter: function () {
		var toolbar = this.chartsTable.getFooterToolbar(); //.getToolbar();

		var button = new ibm.tivoli.tip.dijit.TIPButton ({
			label: this.resources_.BACK,
			showLabel: true,
			onClick: dojo.hitch (this, this.showConnectionManager)
		});
		toolbar.addChild (button);

		button = new ibm.tivoli.tip.dijit.TIPButton ({
			label: this.resources_.FINISH,
			showLabel: true,
			onClick: dojo.hitch (this, this.onSelectChart)
		});
		toolbar.addChild (button);
	},
	clearDesignCache: function () {
		var kw = {
             content:   {
	             command: "clearCache",
	             arg_connectionID: encodeURI(this.datasource_)
                        },
             onResponse:{
                 callback: dojo.hitch(this, this.clearDesignCacheCallback)
                        }
         };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	clearDesignCacheCallback: function () {
		console.log ("design cache cleared");
	},
	updateChartsTable: function (group) {
		// TODO need a callback chartsTable.refresh so we can set the cursor back to normal
//		this.showCursorWait();
		if (group != this.resources_.SELECT_GROUP) {
			console.log ("enter updateChartsTable.  ds: " + this.datasource_ + ", group: " + group);
			var args = [];
			args.dataSource = encodeURI(this.datasource_);
			args.groupName = encodeURI(group);
		
			console.log ("in updateChartsTable. args: ");
			console.log (args);
		
			this.chartsTable.refreshWithArgs (args);
		}
	},
	destroy: function () {
		console.log ("enter ChartSelectorWidget.destroy");
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});

//		this.destroyDescendants();
		this.inherited (arguments);
		console.log ("exit ChartSelectorWidget.destroy");
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartNavigator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartNavigator"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartNavigator");















// NLS




// define the widget class
dojo.declare(
	"ibm.tivoli.tip.charts.dijit.ChartNavigator", 
    [ dijit._Widget, dijit._Templated ],
{
    namespace_ : "namespace1_",
    chartSrcType_ : "",
    connectionName_ : "",
    groupID_ : "",
    chartID_ : "",
    global : null,
    widgetsInTemplate: true,
    CHART_SOURCE_TYPE_PAGE: "CHART_SOURCE_TYPE_PAGE",
    CONNECTION_MGR_PAGE: "CONNECTION_MGR_PAGE",
    WS_CHART_SELECTOR_PAGE: "WS_CHART_SELECTOR_PAGE",
    CUSTOM_CHART_SELECTOR_PAGE: "CUSTOM_CHART_SELECTOR_PAGE",
    CHART_PAGE: "CHART_PAGE",
    TABLE_PAGE: "TABLE_PAGE",
    templateString:"<div id=\"${id}chartNavigatorDiv\">\r\n\r\n\t<div id=\"${id}uploadDiv\" dojoType=\"dijit.Dialog\" dojoAttachPoint=\"uploadDialog\" title=\"${resources_.UPLOAD_CHART}\" \r\n\t\t\tstyle=\"padding:0px;display:none;\">\t\r\n\t\t<form id=\"${id}filenameForm\" name=\"${id}filenameForm\" \r\n\t\t\tdojoType=\"dijit.form.Form\" enctype=\"multipart/form-data\" action=\"#\" method=\"post\"\r\n\t\t\tstyle=\"padding:0px;\"\r\n\t\t>\r\n\t\t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" class=\"chartsDialog\" width=\"400px\">\r\n\t\t<tr>\r\n\t\t<td>\r\n\t\t\t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" style=\"padding:15px;\">\r\n\t\t\t<tr>\r\n\t\t\t<td>\r\n\t\t\t\t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\">\r\n\t\t\t\t<tr>\r\n\t\t\t\t<td>\r\n\t\t\t\t<span class=\"descriptionNoIndent\">\r\n\t\t\t\t\t${resources_.UPLOAD_CHART_DESCRIPTION}\r\n\t\t\t\t</span>\r\n\t\t\t\t</td>\r\n\t\t\t\t</tr>\r\n\t\t\t\t</table>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t\t<tr valign=\"middle\">\r\n\t\t\t<td>\r\n\t\t\t\t<p></p>\r\n\t\t\t\t<p></p>\r\n\t\t\t\t\r\n\t\t\t\t<label id=\"${id}uploadChartStep\" for=\"${id}filename\" title=\"${resources_.UPLOAD_CHART_STEP}\">\r\n\t\t\t\t\t${resources_.FILENAME}\r\n\t\t\t\t</label>\r\n\t\t\t\t<br/>\r\n\r\n\t\t\t\t<input type=\"file\" id=\"${id}filename\" name=\"${id}filename\" size=\"60\"> \r\n\t\t\t\t</input>\r\n\r\n\t\t\t\t<p></p>\r\n\t\t\t\t<p></p>\r\n\t\t\t\t<p></p>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</table>\r\n\t\t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width=\"100%\" >\r\n\t\t\t<tr valign=\"bottom\">\r\n\t\t\t<td>\r\n\t\t\t\t<div dojoType=\"dijit.Toolbar\">\r\n\t\t\t\t\t<button\r\n\t\t\t\t\t\tid=\"${id}uploadButton\"\r\n\t\t\t\t\t\tname=\"${id}uploadButton\"\r\n\t\t\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\t\t\tclass=\"tipButton\">\r\n\t\t\t\t\t\t${resources_.UPLOAD}\r\n\t\t\t\t\t</button>\r\n\t\t\t\t\t<button\r\n\t\t\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\t\t\tdojoAttachEvent=\"onClick:hideUploadDialog\"\r\n\t\t\t\t\t\t>\r\n\t\t\t\t\t\t${resources_.CANCEL}\r\n\t\t\t\t\t</button>\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t\t</tr>\r\n\t\t</table>\r\n\t\t</td>\r\n\t\t</tr>\r\n\t\t</table>\r\n\t\t</form>\r\n\t</div>\r\n\r\n\r\n\r\n\r\n\t<div id=\"${id}chartSourceTypeDiv\" style=\"padding:15px 15px 15px 15px\">\r\n\t  <span class=\"contentHeaderTitle\">\r\n\t  \t${resources_.SELECT_CHART_SOURCE_TYPE}\r\n\t  </span>\r\n\t  <p></p>\r\n\r\n\r\n\t<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n\t<tr valign=\"top\">\r\n  \t<td>\r\n\t\t<span id=\"${id}homeImportChartIcon\" class=\"importChartBig\"></span>\r\n\t</td>\r\n\t<td style=\"padding-right: 17px\">\r\n\t\t<label id=\"${id}homeImportChartLink\" for=\"${id}tivoliApp\" title=\"${resources_.TIV_APP}\"\r\n\t\t>\r\n\t\t\t<a class=\"link\">\r\n\t\t\t\t${resources_.TIV_APP}\t\t \r\n\t\t\t</a>\r\n\t\t</label>\r\n\t\t<p class=\"descriptionNoIndent\">${resources_.TIV_APP_DESC}\r\n\t\t</p>\r\n  \t</td>\r\n  </tr>\r\n\r\n  <tr valign=\"top\">\r\n  <td>\r\n\t<span id=\"${id}homeCustomChartIcon\" class=\"customChartBig\"></span>\r\n  </td>\r\n  <td style=\"padding-right: 17px\">\r\n\t<label id=\"${id}homeCustomChartLink\" for=\"${id}tipServer\" title=\"${resources_.TIP_FILE_STORE}\"\r\n\t>\r\n\t\t<a class=\"link\">\r\n\t\t\t${resources_.TIP_FILE_STORE}\t\t \r\n\t\t</a>\r\n\t</label>\r\n\t<p class=\"descriptionNoIndent\">${resources_.TIP_FILE_STORE_DESC}\r\n  </p>\r\n  </td>\r\n  </tr>\r\n\r\n<!-- \r\n  <tr valign=\"top\">\r\n  <td>\r\n  </td>\r\n  <td>\r\n\t  <span id=\"${id}homeDownloadBIRTIcon\" class=\"downloadBIRTBig\"></span>\r\n  </td>\r\n  <td>\r\n  \t\t<a\r\n  \t\t\tid=\"${id}homeDownloadBIRTLink\"\r\n\t\t\tclass=\"downloadLink\"\r\n\t\t >\r\n\t\t\t${resources_.DOWNLOAD_BIRT}\r\n\t\t</a>\r\n\t\t<br/>\r\n\t\t<span class=\"descriptionNoIndent\">\r\n\t\t\t${resources_.DOWNLOAD_BIRT_DESCRIPTION}\r\n\t\t</span>\r\n  </td>\r\n  \t\r\n  </tr>\r\n -->    \r\n  </table>\r\n\r\n  </div>\r\n\r\n\r\n  <div id=\"${id}tipServerDiv\" style=\"display:none; \" >\r\n     <div dojoType=\"ibm.tivoli.tip.dijit.BreadcrumbBar\">\r\n \t\t  <div dojoType=\"ibm.tivoli.tip.dijit.Breadcrumb\"\r\n\t\t    \tlabel=\"${resources_.CHART_SOURCE_TYPE}\"\r\n\t\t    \tdojoAttachEvent=\"onClick:showChartNavigator\"\r\n\t\t    >\r\n\t\t  </div>\r\n\t</div>\r\n  \r\n  \t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" class=\"contentHeader\" style=\"padding:10px;\">\r\n  \t  <tr valign=\"top\">\r\n  \t  <td>\r\n  \t      <span class=\"customChartBig\">\r\n  \t      </span>\r\n  \t  </td>\r\n  \t  <td>\r\n\t  \t  <span class=\"contentHeaderTitle\">\r\n\t  \t\t${resources_.TIP_FILE_STORE}\t\t \r\n\t  \t  </span>\r\n          \r\n          \r\n          \r\n\t      <p class=\"descriptionNoIndent\">\r\n          \r\n          <table cellpadding=\"5px\" cellspacing=\"0\" border=\"0\" align=\"right\">\r\n  \t  <tr>\r\n  \t  <td><div id=\"${id}downloadBIRTDiv\" >\r\n\t\t  \t<span id=\"${id}downloadBIRTButton\" class=\"downloadBIRTBig\"></span>\r\n\t\t  \t<a\r\n\t\t  \t\tstyle=\"display:none;\"\r\n\t\t\t\tid=\"${id}downloadBIRTLink\"\r\n\t\t\t\tclass=\"downloadLink\" >\r\n\t\t\t\t${resources_.DOWNLOAD_BIRT}\r\n\t\t\t</a>\r\n\t\t  </div></td></tr></table>\r\n          \r\n\t\t      ${resources_.CUSTOM_CHARTS_DESCRIPTION}\r\n    \t  </p></td>\r\n\t  </tr>\r\n\t</table>\r\n\r\n\r\n\t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" width=\"100%\">\r\n\t<tr><td>\r\n\t\t<div id=\"${id}toolbar\" dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\">\r\n\t\t\t<button \r\n\t\t\t\tid=\"${id}refreshButton\"\r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:refreshCustomCharts\"\r\n\t\t\t\ticonClass=\"refreshIcon\"\r\n\t\t\t\tlabel=\"${resources_.REFRESH}\"\r\n\t\t\t\tshowLabel=\"false\"\r\n\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\t\t\t<button \r\n\t\t\t\tid=\"${id}uploadDialogButton\"\r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:showUploadDialog\"\r\n\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\tlabel=\"${resources_.UPLOAD}\"\r\n\t\t\t\tshowLabel=\"false\"\r\n\t\t\t\ticonClass=\"uploadIcon\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\t\t\t<button \r\n\t\t\t\tid=\"${id}deleteCustomChartButton\"\r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:deleteCustomChart\"\r\n\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\tlabel=\"${resources_.DELETE}\"\r\n\t\t\t\tshowLabel=\"false\"\r\n\t\t\t\ticonClass=\"deleteIcon\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\t\t\t<button \r\n\t\t\t\tid=\"${id}exportCustomChartButton\"\r\n\t\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\t\tdojoAttachEvent=\"onClick:exportCustomChart\"\r\n\t\t\t\tclass=\"tipButton\"\r\n\t\t\t\tlabel=\"${resources_.SAVE_AS_FILE}\"\r\n\t\t\t\tshowLabel=\"false\"\r\n\t\t\t\ticonClass=\"saveToFileIcon\"\r\n\t\t\t\t>\r\n\t\t\t</button>\r\n\t\t</div>\r\n\t</td>\r\n\t</tr>\r\n\r\n\t<tr>\r\n\t<td>\r\n\t\t<label id=\"${id}selectChartStep\" for=\"${id}tipServerChartSelector\" title=\"${resources_.SELECT_CHART_STEP}\">\r\n\t\t</label>\r\n\t\t<select \r\n\t\t\tid=\"${id}tipServerChartSelector\" \r\n\t\t\tname=\"${id}tipServerChartSelector\" \r\n\t\t\tstyle=\"width:100%;\"\r\n\t\t\tsize=\"10\" >\r\n\t\t</select>\r\n\t</td>\r\n\t</tr>\r\n\t</table>\r\n\r\n\t<div dojoType=\"dijit.Toolbar\">\r\n\t\t<button\r\n\t\t\tid=\"${id}selectChartBack\"\r\n\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\tclass=\"tipButton\" >\r\n\t\t\t${resources_.BACK}\r\n\t\t</button>\r\n\t\r\n\t\t<button\r\n\t\t\tid=\"${id}tipServerChartSelectorBtn\"\r\n\t\t\tdojoType=\"ibm.tivoli.tip.dijit.TIPButton\"\r\n\t\t\tclass=\"tipButton\">\r\n\t\t\t${resources_.FINISH}\r\n\t\t</button>\r\n\t</div>\r\n\t\r\n\t\r\n  </div>\r\n  \r\n  <div id=\"${id}connectionMgrDiv\" style=\"display:none; \"> \r\n    <div dojoType=\"ibm.tivoli.tip.dijit.BreadcrumbBar\">\r\n \t\t  <div dojoType=\"ibm.tivoli.tip.dijit.Breadcrumb\"\r\n\t\t    \tlabel=\"${resources_.CHART_SOURCE_TYPE}\"\r\n\t\t    \tdojoAttachEvent=\"onClick:showChartNavigator\"\r\n\t\t    >\r\n\t\t  </div>\r\n\t</div>\r\n  \r\n  \t<table cellpadding=\"0\" cellspacing=\"0\" border=\"0\" class=\"contentHeader\" style=\"padding:10px;\">\r\n  \t  <tr valign=\"top\">\r\n  \t  <td>\r\n  \t      <span class=\"importChartBig\">\r\n  \t      </span>\r\n  \t  </td>\r\n  \t  <td>\r\n\t  \t  <span class=\"contentHeaderTitle\">\r\n\t  \t\t  ${resources_.TIV_APP}\t\t \r\n\t  \t  </span>\r\n\t      <p class=\"descriptionNoIndent\">\r\n\t\t      ${resources_.TIV_APP_PAGE_DESC}\r\n    \t  </p>\r\n      </td>\r\n\t  </tr>\r\n\t</table>\r\n\r\n\r\n\t  <div id=\"${id}connectionMgrWidget\">\r\n\t  </div>\r\n\t  \r\n  </div>\r\n  \r\n  \r\n  <div id=\"${id}mainChartSelectorDiv\" style=\"display:none; \" >\r\n\t  <div id=\"${id}chartSelectorDiv\" style=\"display:none; \" >\r\n\t  </div>\r\n  </div>\r\n  \r\n  \r\n  \r\n</div>\r\n\r\n\r\n",

	constructor: function() {
		this.connectionMgr_ = null;
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
   	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_messages");
   	    this.tooltips = [];
   	    this.connections = [];
   	    this.topics = [];
   	    this.currentPage = this.CHART_SOURCE_TYPE_PAGE;
	},
	initialize: function() { 
	},
	postCreate: function() {
		this.inherited ("postCreate", arguments);
		this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		
		var btn2 = dojo.byId (this.id + "tipServerChartSelectorBtn");
        this.connections.push (dojo.connect(btn2, 'onclick', this, 'onTipServerChartSelection'));
        var btn6 = dojo.byId (this.id + "downloadBIRTLink");
        this.connections.push (dojo.connect(btn6, 'onclick', this, 'downloadBIRTDesigner'));
        this.connections.push (dojo.connect(dojo.byId (this.id + "downloadBIRTButton"), 'onclick', this, 'downloadBIRTDesigner'));
        var btn7 = dojo.byId (this.id + "uploadButton");
        this.connections.push (dojo.connect(btn7, 'onclick', this, 'uploadChart'));
        
        this.connections.push (dojo.connect (dojo.byId (this.id + "homeImportChartIcon"), "onclick", this, "onImportChartType"));
        this.connections.push (dojo.connect (dojo.byId (this.id + "homeImportChartLink"), "onclick", this, "onImportChartType"));

        this.connections.push (dojo.connect (dojo.byId (this.id + "homeCustomChartIcon"), "onclick", this, "onCustomChartType"));
        this.connections.push (dojo.connect (dojo.byId (this.id + "homeCustomChartLink"), "onclick", this, "onCustomChartType"));

//        dojo.connect (dojo.byId (this.id + "homeDownloadBIRTIcon"), "onclick", this, "downloadBIRTDesigner");
//        dojo.connect (dojo.byId (this.id + "homeDownloadBIRTLink"), "onclick", this, "downloadBIRTDesigner");

        this.connections.push (dojo.connect (dojo.byId (this.id + "selectChartBack"), "onclick", this, 'showChartNavigator'));
        
        this.topics.push (dojo.subscribe ("/com.ibm.tivoli.tip.charts/" + this.namespace_ + "#showConnectionManager", this, "showConnectionManager"));
        
        this.handleRole();
        
        this.getPortletPreferences();
	},
	setCurrentPage: function (page) {
		this.currentPage = page;
	},
	getCurrentPage: function () {
		return this.currentPage;
	},
	getPortletPreferences: function () {
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
		var startNode = dojo.byId(this.id);
		var chartWidget = dijit.byId (this.namespace_ + "chartWidget");
		var errorHandler = { showMsg : true, callback: dojo.hitch (chartWidget, chartWidget.onError) };
		

		var kw = {
            content:   {
                  command: "getPreferences",
                  arg_portletNamespace: this.namespace_,
                  arg_chartProfileID: this.global.chartProfileID,
                  arg_pageName: helper.getPortletPageName(this.domNode),
                  arg_windowID: helper.getPortletWindowID(this.domNode),
                  arg_pageID: helper.getPortletPageID(this.domNode),
                  arg_load: "true"
            },
         	onExc: errorHandler,
       	    onResponse:{
                  callback: dojo.hitch(this, this.handleSavedPreferences)
       	    }
    	 };

		console.log ("making request getPreferences");	
		ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
	},
	handleRole: function () {
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace_);
		if (context.roles.admin || context.roles.creator) {
			console.log ("administrator or creator role. enabling chart upload");
			dojo.byId(this.id + "downloadBIRTDiv").style.display = "";
		} else {
			console.log ("no administrator or creator role. disabling chart upload");
			// hide download/upload
			dojo.byId(this.id + "downloadBIRTDiv").style.display = "none";
			dojo.byId(this.id + "selectChartStep").innerHTML = this.resources_.SELECT_CHART;
			dojo.byId(this.id + "selectChartStep").title = this.resources_.SELECT_CHART;
			dojo.byId(this.id + "uploadDialogButton").style.display = "none";
			dojo.byId(this.id + "deleteCustomChartButton").style.display = "none";
			dojo.byId(this.id + "exportCustomChartButton").style.display = "none";
		}
	},
	downloadBIRTDesigner: function () {
		var url = "/ibm/TIPChartPortlet/birt/birt-designer.zip";
		window.open (url, "BIRTDesigner");
	},
	handleSavedPreferences: function (data, ioArgs) {
		console.log ("Enter handleSavedPreferences");
		this.chartSrcType_ = data.CHART_CONNECTION_TYPE;
		this.connectionName_ = data.CONNECTION_NAME;
		this.groupID_ = data.GROUP_ID;
		this.chartID_ = data.REPORT_NAME;
		var chartName = data.CHART_NAME;
		console.log (" with namespace: " + this.namespace_ + ", chartSrcType_: " + this.chartSrcType_ + ", connectionName: " + this.connectionName_ + ", groupID: " + this.groupID_ + ", chartID: " + this.chartID_);
		var doDefault = true;
		var srcTypeDiv = dojo.byId (this.id + "chartSourceTypeDiv");
		var webServiceRadio = dojo.byId (this.namespace_ + "tivoliApp");
		
		if (this.chartSrcType_ && (this.chartSrcType_ !== "") && (this.chartSrcType_ !== "null")) {
			if (this.chartSrcType_ == "TIPServer") {
				if (this.chartID_ && (this.chartID_ !== "") && (this.chartID_ !== "null")) {
					var tipServerRadio = dojo.byId (this.namespace_ + "tipServer");
					tipServerRadio.checked = true;
					srcTypeDiv.style.display = "none";
					this.populateTipServerChartSelector(true);
					var tipServerDiv = dojo.byId (this.id + "tipServerDiv");
					tipServerDiv.style.display = "none";
					doDefault = false;
				} else {
					doDefault = true;
				}
			} else if (this.chartSrcType_ == "WebService") {
				console.log ("rendering WebService:connectionName: " + this.connectionName_ + ", groupID: " + this.groupID_ + ", chartID: " + this.chartID_);
				webServiceRadio.checked = true;
				srcTypeDiv.style.display = "none";
				var selectorDiv = dojo.byId (this.namespace_ + "reportSelectorDiv");
				selectorDiv.style.display = "";
//				this.populateWebServicePath();
				this.handleWebServicePath(chartName);
				this.setWebServiceSubmitValues();
				doDefault = false;
			} else if (this.chartSrcType_ == "TivoliCommonReporting") {
				webServiceRadio.checked = true;
				srcTypeDiv.style.display = "none";
				this.populateTCRPath();
				doDefault = false;
			} else {
				console.error ("unknown chartSrcType: " + this.chartSrcType_);
				doDefault = true;
			}
		} else {
			console.log ("no chartSrcType: " + this.chartSrcType_);
		}
		
		if (doDefault) {
			var chartNavDiv = dojo.byId (this.namespace_ + "reportSelectorDiv");
			chartNavDiv.style.display = "";
		}
		
		console.log ("exit handleSavedPreferences");
	},
	setWebServiceSubmitValues: function () {
		if (this.connectionName_) {
			dojo.byId (this.namespace_ + "connectionNameH").value = this.connectionName_;
		}
		if (this.groupID_) {
			dojo.byId (this.namespace_ + "groupIDH").value = this.groupID_;
		}
		if (this.chartID_) {
			dojo.byId (this.namespace_ + "chartIDH").value = this.chartID_;
		}
	},
	handleWebServicePath: function (chartName) {
		var chartWidget = dijit.byId (this.namespace_ + "chartWidget");
		var errorHandler = { showMsg : true, callback: dojo.hitch (chartWidget, chartWidget.onError) };
	
		var kw = {
             content:   {
                         command: "getChartDefinition",
                         arg_datasource: encodeURI(this.connectionName_),
						 arg_groupID: encodeURI(this.groupID_),
						 arg_chartID: encodeURI(this.chartID_)
                        },
	         onExc: errorHandler,
             onResponse:{
                         callback: dojo.hitch(this, this.populateGetChartDefinitionCallback)
                        }
         };
         
         if ((chartName !== null) && (chartName !== undefined)) {
			kw.content.arg_chartName = encodeURI(chartName);
         }
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	populateGetChartDefinitionCallback: function (data, ioArgs) {
		var clearPreferences = false;
		dijit.byId (this.namespace_ + "chartWidget").onReportSelection (
			data[0].chartSourceType, 
			data[0].connectionName, 
			this.groupID_,
			data[0].id,
			data[0].label,
			clearPreferences,
			false);
	},
	populateTCRPath: function () {
		 // TODO Implement this
	},
	populateWebServicePath: function () {
		this.showConnectionManager();
	},
	showChartNavigator: function () {
		// clear hidden parameters
		dojo.byId (this.namespace_ + "groupIDH").value = "";
		dojo.byId (this.namespace_ + "chartIDH").value = "";
		dojo.byId (this.namespace_ + "connectionNameH").value = "";
		dojo.byId (this.namespace_ + "descriptionH").value = "";	
		dojo.byId (this.namespace_ + "chartNameH").value = "";
	
		// show/hide divs
		dojo.byId (this.id + "chartSourceTypeDiv").style.display = "";
		
		dojo.byId (this.id + "tipServerDiv").style.display = "none";
		dojo.byId (this.id + "connectionMgrDiv").style.display = "none";
		dojo.byId (this.id + "mainChartSelectorDiv").style.display = "none";
		
		var chartNavDiv = dojo.byId (this.namespace_ + "reportSelectorDiv");
		chartNavDiv.style.display = "";
		
		this.setCurrentPage (this.CHART_SOURCE_TYPE_PAGE);
	},
	isNavigatorVisible: function () {
		return (
			(dojo.byId (this.id + "chartSourceTypeDiv").style.display === "") ||
			(dojo.byId (this.id + "tipServerDiv").style.display === "") ||
			(dojo.byId (this.id + "connectionMgrDiv").style.display === "") ||
			(dojo.byId (this.id + "mainChartSelectorDiv").style.display === "")
		);
	},
//	showConnectionManager: function () {
//		dojo.byId (this.id + "connectionMgrDiv").style.display = "";
//
//		dojo.byId (this.id + "chartSourceTypeDiv").style.display = "none";
//		dojo.byId (this.id + "tipServerDiv").style.display = "none";
//		dojo.byId (this.id + "mainChartSelectorDiv").style.display = "none";
//	},
	onCustomChartType: function () {
		dojo.byId(this.namespace_ + 'tipServer').checked=true;
		this.onChartSourceTypeSelection();
	},
	onImportChartType: function () {
		dojo.byId(this.namespace_ + 'tivoliApp').checked=true;
		this.onChartSourceTypeSelection();
	},
	onChartSourceTypeSelection: function () {
		var srcTypeDiv = dojo.byId (this.id + "chartSourceTypeDiv");

//		var tcrRadio = dojo.byId (this.id + "tcrRepository");
		var tipServerRadio = dojo.byId (this.namespace_ + "tipServer");
		var tivAppRadio = dojo.byId (this.namespace_ + "tivoliApp");
		
//		if (tcrRadio.checked) {
//			this.showTCRNavigator();
		if (tipServerRadio.checked) {
			srcTypeDiv.style.display = "none";
			this.setCurrentPage (this.CUSTOM_CHART_SELECTOR_PAGE);
			this.populateTipServerChartSelector();		
		} else if (tivAppRadio.checked) {
			srcTypeDiv.style.display = "none";
			this.showConnectionManager();
		}
	},
	getChartSourceType: function () {
//		var tcrRadio = dojo.byId (this.id + "tcrRepository");
		var tipServerRadio = dojo.byId (this.namespace_ + "tipServer");
		var tivAppRadio = dojo.byId (this.namespace_ + "tivoliApp");
	
//		if (tcrRadio.checked) {
//			return ("TivoliCommonReporting");
		if (tipServerRadio.checked) {
			return ("TIPServer");
		} else if (tivAppRadio.checked) {
			return ("WebService");
		} else {
			return (null);
		}
	},
	showTCRNavigator: function() {
		alert ("not implemented yet");
	},
	populateTipServerChartSelector: function (force) {
		var tipServerDiv = dojo.byId (this.id + "tipServerDiv");
		tipServerDiv.style.display = "";
		
	
		if (force || (dojo.byId (this.id + "tipServerChartSelector").options.length < 1)) {
	       var kw = {
                    content:   {
                                  command: "getReports"
                               },
                    onResponse:{
                                  callback: dojo.hitch(this, this.populateTipServerChartSelectorCallback)
                                }
                 };
			ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
    	 }
	},
	/*******************************************************************************
	* This method will return a serialized object of all info inside the dialog form
	* You must not call this method while form is in use -- it will break dojo stuff!
	*******************************************************************************/
	getFormValuesForJSON: function(formID)
	{
		console.log ("Enter getFormValues for formID: " + formID);
	   var form = dijit.byId(formID);
	   return dojo.json.serialize(form.getValues()) ;	   	   
	},
	uploadChart: function () {
		console.log ("Enter uploadChart");
		var formID = this.id + "filenameForm";
		var filenameID = this.id + "filename";
		var filename = dojo.byId( filenameID );
		if (filename.value.length === 0 || filename.value === null) {
			// Do nothing, wait for user input
		} else {
 			dojo.io.iframe.send({
			  url: "/ibm/TIPChartPortlet/TIPChartController?command=uploadReport",
			  form: dojo.byId (formID),
		  	  handleAs: "json",
         	  error: dojo.hitch (this, this.handleError),
		  	  timeout: dojo.hitch (this, this.handleTimeout),
		   	  load: dojo.hitch (this, this.onUploadComplete)
		    });	
		    // TODO: remove this line once we upgrade to a dojo that has a fix for this.
		    // There is a bug in dojo 1.0 where the iframe becomes visible when you call
		    // send.
		    dojo.style(dojo.byId('dojoIoIframe'),"display","none"); 
		
		    this.uploadDialog.hide();
		}
	},
	handleError: function (response, ioArgs) {
//		console.error ("HTTP status code: ", ioArgs.xhr.status);
		console.error ("Error from server");
		console.error (response);
		console.error (ioArgs);
		return response;
	},
	
	handleTimeout: function (response, ioArgs) {
		console.error ("Timeout for request: " + ioArgs.xhr);
		return response;
	},
	
	onUploadComplete: function (data, ioArgs) {
		console.log ("data is: ");
		console.log (data);
		var modalWindow = null;
		var helper = new ibm.tivoli.tip.dojo.TIPHelper();
		
		if (data.id === null) {
			modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog ( 
		                                          { 
			                                          type: "info",
													  portletTitle: helper.getPortletTitle(this.domNode),			                                          
	    	                                          message: this.messages_.TIPCH3008E,
	        	                                      messageId: "TIPCH3008E",
	            	                                  buttons: [ this.resources_.CLOSE ]
	                    	                        });
			modalWindow.show();
		} else {
			var chartSelector = dojo.byId (this.id + "tipServerChartSelector");
			var index = -1;
			if ((index = this.selectItemExists (chartSelector, data.id)) > -1) {
				// already exists so just select it
				chartSelector.options[index].selected = true;				
			} else {
				// doesn't exist so add it
				chartSelector.options[chartSelector.options.length] = new Option (data.label, data.id);
				chartSelector.options[chartSelector.options.length - 1].selected = true;
			}
		
			modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog ( 
		                                          { 
			                                          type: "info",
													  portletTitle: helper.getPortletTitle(this.domNode),			                                          
	    	                                          message: this.messages_.TIPCH3003I,
	        	                                      messageId: "TIPCH3003I",
	            	                                  buttons: [ this.resources_.CLOSE ]
	                    	                        });
			modalWindow.show();
		}
	},
	selectItemExists: function (select, item) {
		for (var i=0; i<select.options.length; i++) {
			if (select.options[i].text == item) {
				return (i);
			}
		}
		
		return (-1);
	},
	populateTipServerChartSelectorCallback: function (data, ioArgs) {
		var chartSelector = dojo.byId (this.id + "tipServerChartSelector");
		var selected = false;
		chartSelector.options.length = 0;
		
		for (var i=0; i<data.length; i++) {
			chartSelector.options[chartSelector.options.length] = new Option (data[i].label, data[i].id);
			if (data[i].id == this.chartID_) {
				chartSelector.options[i].selected = true;
				selected = true;
			}
		}
		
		console.log ("chartSrcType: " + this.chartSrcType_);
		console.log ("chartID: "+ this.chartID_);
		if (selected) {
			this.onTipServerChartSelection(true);
		} else {
			if ((this.chartSrcType_ == "TIPServer") &&
				this.chartID_ && 
				(this.chartID_ !== "") && 
				(this.chartID_ !== "null")) {
				// This is the case where we have a predefined
				// chart but it has been deleted and no longer shows 
				// up in the list
				this.handleDeletedCustomChart();
			}
		}
	},
	handleDeletedCustomChart: function () {
		console.log ("enter handleDeletedCustomChart");
		this.chartID_ = null;
		this.showChartNavigator();
		console.log ("exit handleDeletedCustomChart");
	},
	onTipServerChartSelection: function (autoSelected) {
		var clearPreferences = true;
		var persist = true;
		if (autoSelected === true) {
			clearPreferences = false;
			persist = false;
		}
				
		var selector = dojo.byId (this.id + "tipServerChartSelector"); 
		var reportName = selector.options[selector.selectedIndex].text; 
		dojo.byId (this.namespace_ + "chartIDH").value = reportName;
		var chartSrcType = this.getChartSourceType();
		var groupID = null;
		var connectionName = null; //this.connectionMgr_ === null ? null : this.connectionMgr_.getSelectedDataSourceName();
		
		dijit.byId (this.namespace_ + "chartWidget").onReportSelection (
			chartSrcType,
			connectionName,
			groupID,
			reportName,
			reportName,
			clearPreferences,
			persist);		
	},
	refreshCustomCharts: function () {
		console.log ("enter refreshCustomCharts");
		this.populateTipServerChartSelector(true);
	},
	exportCustomChart: function () {
		var selector = dojo.byId (this.id + "tipServerChartSelector"); 
		var selected = selector.options[selector.selectedIndex];
		if (selected) {
			var chartID = selector.options[selector.selectedIndex].text; 
      // arg_chartID_UTF8 is the UTF-8 encoded string and
      // arg_chartID is the browser's default encoded string.
      var url = "/ibm/TIPChartPortlet/TIPChartController?command=exportChart&arg_chartID_UTF8="+ encodeURI(chartID).replace(/%/g, "%25")+"&arg_chartID="+chartID;

			window.open (url, 'ExportChart');
		}
	},
	deleteCustomChart: function () {
		var selector = dojo.byId (this.id + "tipServerChartSelector"); 
		var selected = selector.options[selector.selectedIndex];
		if (selected) {
			var reportName = selector.options[selector.selectedIndex].text; 

			var kw = {
             content:   {
                  command: "deleteChart",
                  arg_chartID: encodeURI(reportName),
                  arg_chartSrcType: this.getChartSourceType(),
                  arg_connectionName: null,
                  arg_groupID: null
                 },
            	 onResponse:{
                         callback: dojo.hitch(this, this.refreshCustomCharts)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			console.log ("In deleteCustomChart. No chart selected");
		}
	},
	showConnectionManager: function () {
		console.info ("enter showConnectionManager");
		//dojoType="ibm.tivoli.tip.dojo:ConnectionManagerWidget" namespace_="<portlet:namespace/>"
		var div = dojo.byId (this.id + "connectionMgrWidget");
		div.style.display = "";
		if (this.connectionMgr_ === null) {
			console.log ("creating connectionMgr_");
			this.connectionMgr_ = new ibm.tivoli.tip.charts.dijit.ConnectionManagerWidget (
				{
					namespace_:this.namespace_,
					callback_: dojo.hitch (this, this.onCMRealize)
				}, 
				div);
		} else {
			console.log ("this.connectionMgr_ already exists");
		}
	
		dojo.byId (this.id + "mainChartSelectorDiv").style.display = "none";
		dojo.byId (this.id + "connectionMgrDiv").style.display = "";
		this.setCurrentPage (this.CONNECTION_MGR_PAGE);
		
		console.log ("exit showConnectionManager");
	},
	onCMRealize: function () {
		if (this.connectionName_ && (this.connectionName_ !== null) && (this.connectionName_ !== "null")) {
			console.log ("In onCMRealize with connectionName_: " + this.connectionName_);
			this.connectionMgr_.selectConnection (this.connectionName_);
			this.showCMChartSelector();
		} else {
			console.log ("In onCMRealize with no connectionName: " + this.connectionName_);
		}
	},
	showCMChartSelector: function () {
		console.log ("Enter showCMChartSelector");
		if (this.connectionMgr_.getSelectedDataSourceName()) {
			var dataSourceName = this.connectionMgr_.getSelectedDataSourceName();
			console.log ("Using data source: " + dataSourceName);
			dojo.byId (this.namespace_ + "connectionNameH").value = dataSourceName;

			var chartSelector = dijit.byId (this.id + "chartSelectorDiv");
			if (chartSelector) {
				// chart selector already created, so don't create again
				// just set the data source
				chartSelector.setDataSource (dataSourceName);
			} else {			
				var div = dojo.byId (this.id + "chartSelectorDiv");
	
				chartSelector= new ibm.tivoli.tip.charts.dijit.ChartSelectorWidget (
					{
						datasource_:dataSourceName,
						namespace_:this.namespace_
						// TODO: Uncomment these to make auto selection work
//						groupID_:this.groupID_,
//						chartID_:this.chartID_
					},
					div);
			}
	
			dojo.byId (this.id + "connectionMgrDiv").style.display = "none";
			dojo.byId (this.id + "mainChartSelectorDiv").style.display = "";
			this.setCurrentPage (this.WS_CHART_SELECTOR_PAGE);
		} else {
			var helper = new ibm.tivoli.tip.dojo.TIPHelper();
    	    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: "error",
													  portletTitle: helper.getPortletTitle(this.domNode),			                                          
	    	                                          message: this.messages_.TIPCH3004E,
	        	                                      messageId: "TIPCH3004E",
	            	                                  buttons: [ this.resources_.CLOSE ]
	                    	                        });
			modalWindow.show();
		}
	},
	showConnectionEditor: function () {
	},
	showUploadDialog: function () {
		this.uploadDialog.show();
	},
	hideUploadDialog: function () {
		this.uploadDialog.hide();
	},
	resize: function (width, height) {
		if (this.getCurrentPage() == this.CONNECTION_MGR_PAGE) {
			this.connectionMgr_.resize(width, height);
		}
	},
	destroyDescendants: function(){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.

		// TODO: should I destroy in the reverse order, to go bottom up?
		dojo.forEach(this.getDescendants(), function(widget){ 
			if (widget.id != this.id) { 
				console.log ("in ChartNavigator.destroyDescendants. about to destroy: ");
				console.log (widget);
				widget.destroy(); 
			}
		}, this);
	},
	destroy: function () {
		console.log ("enter ChartNavigator.destroy");
		dojo.forEach (this.topics, function (topic) {
			dojo.unsubscribe (topic);
		});
		dojo.forEach (this.connections, function (conn) {
			dojo.disconnect (conn);
		});
		dojo.forEach (this.tooltips, function (tt) {
			tt.destroy();
		});
		
		console.log ("calling ChartNavigator.inherited.destroy");
		this.inherited ("destroy", arguments);
		console.log ("exit ChartNavigator.destroy");
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPHelper");

dojo.declare("ibm.tivoli.tip.dojo.TIPHelper",
	          null,
{
	constructor: function () {
	},
	// childNode must be a node within your portlet
	// padding is the amount of padding you want around your portlet
	//   e.g., "0px"
	setPortletPadding: function (childNode, padding) {
		var node = this.getAncestorByClass (childNode, "isc-portlet-bkg");
		dojo.style (node, "padding", padding);
	    var nextnode = dojo.query( 'table[id$="_portlet_content"]', node )[0];
        if (nextnode) {
        	dojo.style( nextnode, "padding", padding );
        }
	},
	getAncestorByClass: function (element, className) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				if(dojo.hasClass (element, className)) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByName: function (element, name) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementname = element.getAttribute ("name");
				if (elementname == name) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByID: function (element, id) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementid = element.getAttribute ("id");
				if (elementid == id) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
		
	},
	getPortletOID: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
			console.log ("get portlet data: ");
			console.log (node.modelObject);
			var result = node.modelObject.portletData.oid;
			return (result);
		}
	},
	getPortletWindowID: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
			var result = node.modelObject.portletData.portletName;
			return (result);
		}
	},
	getPortletPageID: function (element) {
		var node = this.getAncestorByName (element, "IscPage");
		if (node) {
			var result = node.modelObject.pageData.uniquePageName;
			return (result);
		}
	},
	getPortletPageName: function (element) {
		var parent = this.getAncestorByName (element, "IscPage");
		if (parent) {
		var node = dojo.query ("td.isc-layout-titletext", parent);
//		var node = dojo.query ("td.isc-tab-sel-l", frames["taskbar"].document);
		if (node) {
			if (node.length == 1) {
//				var result = node[0].innerHTML.alt;
				var result = node[0].childNodes[0].nodeValue;
				if (result) {
					// strip leading white space
					result = result.replace(/^\s+/,"");
					// strip trailing white space
					result = result.replace(/\s+$/,"");
				}
				console.log ("portlet page name: ");
				console.log (result);
				return (result);
			} else if (node.length === 0) {
				console.log ("could not find node for portlet page name");
			} else {
				console.log ("found multiple nodes for portlet page names");
				console.log (node);
			}
		} else {
			console.log ("could not find node for portlet page name");
		}
		} else {
			console.log ("no parent node to get portlet page name");
		}
	},
	getTopPortletNode: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		return (node);		
	},
	getPortletTitle: function (element) {
		var oid = this.getPortletOID (element);
		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				return node.innerHTML;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("cannot find oid for element: ");
			console.log (element);
		}
	},
	setPortletTitle: function (element, title) {
		console.log ("enter setPortletTitle: " + title);
		var oid = this.getPortletOID (element);

		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				node.innerHTML = title;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("cannot find oid for element: ");
			console.log (element);
		}
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.charts.dijit.ChartWidget"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.charts.dijit.ChartWidget");































// NLS



// define the widget class
dojo.declare(
	"ibm.tivoli.tip.charts.dijit.ChartWidget", 
    [ dijit._Widget, dijit._Templated ],
{
	namespace: "namespace1_",
	global: null,
	portletActionURL: null,
	iscPortletNamespace: "iscPortletNamespace1",
	contextPath: null,
	templateString:"<div>\r\n\r\n<div style=\"z-index:0;width: 100%;\" dojoAttachPoint=\"mainDiv\">\r\n<form name=\"${namespace}form\" id=\"${namespace}form\" action=\"${portletActionURL}\" \r\n\t  dojoType=\"dijit.form.Form\" enctype=\"multipart/form-data\" \r\n\t  style=\"padding: 0px; margin:0px\"\r\n>\r\n\r\n<div id=\"${namespace}msgDiv\" style=\"display:none\">\r\n</div>\r\n\r\n<div id=\"${namespace}reportSelectorDiv\" dojoAttachPoint=\"reportSelectorDiv\" style=\"display:none\" >\r\n\t<div id=\"${namespace}mainChartNavigatorDiv\">\r\n\t</div>\r\n</div>\r\n\r\n<div id=\"${namespace}chartDiv\" dojoAttachPoint=\"chartDiv\" style=\"display:none; \">\r\n<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\r\n\t<tr>\r\n\t\t<td align=\"left\">\r\n\t\t\t<div id=\"${namespace}mapDiv\" style=\"display:none; \">\r\n\t\t\t</div>\r\n\t\t\t<div id=\"${namespace}toolbarDiv\" dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbarDiv\" style=\"display:none; \">\r\n\t\t\t</div>\r\n\t\t</td>\r\n\t</tr>\r\n\t<tr>\r\n\t\t<td align=\"center\">\r\n\t\t\t<div id=\"${id}nub\" dojoAttachPoint=\"nub\" class=\"nubClosed\" tabindex=\"0\">\r\n\t\t\t</div>\r\n\t\t</td>\r\n\t</tr>\r\n\t<tr>\r\n\t\t\t<td align=\"left\">\r\n\t\t\t\t<div id=\"${id}chartImageDiv\" tabindex=\"0\">\r\n\t\t\t\t\t<a class=\"imagemap\"> \r\n\t\t\t\t\t\t<img id=\"${namespace}chartImage\" dojoAttachPoint=\"chartImage\" border=\"0\" ISMAP USEMAP=\"#chartmap\" /> \r\n\t\t\t\t\t</a>\t\t\t\r\n\t\t\t\t</div>\r\n\t\t\t</td>\r\n\t\t</tr>\r\n</table>\r\n</div>\r\n\r\n<div id=\"${namespace}tableDiv\" dojoAttachPoint=\"tableDiv\" class=\"tableContainer\" style=\"display:none;\">\r\n\t<table id=\"${namespace}tbl_tipChartData\" width=\"100%\"></table>\r\n</div>\r\n\r\n\r\n<div id=\"${namespace}metaDataDiv\" dojoAttachPoint=\"metaDiv\">\r\n</div>\r\n\r\n\t<div id=\"${namespace}hiddenProperties\" style=\"display:none; \">\r\n\t\t<input type=\"text\" name=\"${namespace}chartNameH\" id=\"${namespace}chartNameH\" />\r\n\t\t<input type=\"text\" name=\"${namespace}descriptionH\" id=\"${namespace}descriptionH\" />\r\n\t\t<input type=\"text\" name=\"${namespace}connectionNameH\" id=\"${namespace}connectionNameH\" />\r\n\t\t<input type=\"text\" name=\"${namespace}groupIDH\" id=\"${namespace}groupIDH\" />\r\n\t\t<input type=\"text\" name=\"${namespace}chartIDH\" id=\"${namespace}chartIDH\" />\r\n\t\t<input type=\"radio\"\r\n\t\t\t name=\"${namespace}chartSourceTypeGroup\"\r\n\t\t\t value=\"WebService\"\r\n\t\t\t id=\"${namespace}tivoliApp\" />\r\n\t\t<input type=\"radio\"\r\n\t\t\t name=\"${namespace}chartSourceTypeGroup\"\r\n\t\t\t value=\"TIPServer\"\r\n\t\t\t id=\"${namespace}tipServer\" />\r\n\t</div>\t\r\n\r\n\t\r\n<div id=\"${namespace}prefs_dialog\" >\r\n</div>\r\n\r\n</form>\r\n</div>\r\n\r\n</div>\r\n",
	widgetsInTemplate: true,
	tablesCreated: false,

  runReportStartTime: 0,
  runReportResponseTime: 5,
  updateMultiplier: 10,
  updateMinInterval: 30000,
  updateMaxInterval: 180000,
	chartSrcType_: null,
	connectionName_: null,
	groupID_: null,
	reportName: null,
	counter: 0,
	interval: null,
	oldChartType_: "line",
	oldParameterFragment_: "",
	paramWidget_: null,
	oldName_: null,
	oldDescription_: null,
	oldWidth_: null,
	oldHeight_: null,
	oldShowLegend_: null,
	oldChartRadio_: null,
	oldCheckForUpdates_: null,
	oldUpdateMultiplier_: null,
	oldReceiveContext_: null,
	oldChartSourceType_: null,
	oldShowNameInChart_: null,
	oldShowLastUpdated_: null,

	mssName_: null,
	prefsDialog: null,

	actualImgHeight_: null,
	actualImgWidth_: null,
	allowSubmit_: true,
	topics_: [],
	toggleSubscription: null,
	chartingTitle_: null,

	constructor: function() {
	//	dojo.byId (this.namespace + "console").innerHTML += "enter ChartWidget.constructor<br/>";
		console.log ("enter ChartWidget.constructor");
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_resources");
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.charts.dijit", "chart_messages");

		this.connections = [];
		this.tooltips = [];
		
		this.requestQueue = [];
		
		this.tipHelper_ = new ibm.tivoli.tip.dojo.TIPHelper();
		this.lastPage_ = null;
		this.lastPortletSize_ = null;
		
		this.firstRender = true;
		
		console.log ("exit ChartWidget.constructor");
	},

	postCreate: function() {
		this.inherited(arguments);
		console.log ("enter ChartWidget.postCreate");

		try {
			this.global = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		    console.log ("got global: ");
		    console.log (this.global);

		    // store the original title so we can restore it if 
		    // the chart becomes unconfigured.  Because it comes
		    // from the server we don't have a client resource bundle
		    // so we must store it.
		    this.chartingTitle_ = this.tipHelper_.getPortletTitle (this.domNode);
		    console.log ("chartingTitle before change: " + this.chartingTitle_);

		    this.toolbarHidden_ = (this.global.mode == "view") ? true : false;
			this.mode = this.global.mode;

			this.initCounter();
			this.addButtonsToChartToolbar(this.toolbarDiv);
    
		    // subscribe to enter key press events for table
		    this.topics_.push (dojo.subscribe ("onEnter", this, "handleOnEnter"));
		    
		    // make context menu
		    this.makeChartContextMenu();

			// connect toggle event to nub
			this.connections.push (dojo.connect (this.nub, "onclick", this, "toggleToolbar"));
			this.connections.push (dojo.connect (this.nub, "onkeypress", this, "onNubKeyPress"));
			
			// Create dialog
			this.makeChartProperties();

			console.log ("exit ChartWidget.postCreate");
		} catch (ex) {
			console.error (ex);
		}
	},
	createOrUpdateTable: function (update) {
		console.info ("enter createOrUpdateTable");
		if (!this.tablesCreated) {
			this.createTIPTable(this.namespace + "tbl_tipChartData");
			this.tablesCreated = true;
		} else {
			if (update) {
		        this.updateTIPTable (this.namespace + "tbl_tipChartData");
		    }
	    }
	},
	createTIPTable: function(view) {
		var cmProvider = new ibm.tivoli.tip.dijit.CMSMenuProviderV2(
				{ global: this.global,
				  menuName: "",
				  hasLocalActions: false,
				  iscPortletNamespace: this.iscPortletNamespace
				});
	
		var tipTable = new ibm.tivoli.tip.dijit.TIPTable({
					global: this.global,
					tableID: view,
					cmProvider: cmProvider,
					width: "100%",
					open: !this.toolbarHidden_,
					makeDefaultButtons: false,
					id: view,
					autoSavePreferences : true
				},
				dojo.byId (view));
		tipTable.startup();
		this.addButtonsToTableToolbar (tipTable);

		// subscribe to table toolbar toggle, so we can also toggle the chart toolbar			
		this.toggleSubscription = dojo.subscribe (tipTable.id + "#ToggleToolbar", this, "toggleChartToolbar");
		this.topics_.push (dojo.subscribe (tipTable.id + "#LoadComplete", this, "onTableLoad"));
		
		this.resizeTable();
	},
	onTableLoad: function () {
		// We have to refresh to data again to accommodate a bug in TIPTable selection
//		dijit.byId (this.namespace + "tbl_tipChartData").onRefresh();
	},
	updateTIPTable: function(view) {
		var tbl = dijit.byId (view);
		tbl.onRefresh();
	},
	addButtonsToTableToolbar: function (table) {
		var tblToolbar = table.getToolbar();

		var ddMenu = new dijit.Menu ();
		var mi = new dijit.MenuItem ({
				label: this.resources_.REFRESH
			});
		mi.onClick = dojo.hitch (this, "refreshChart");
		ddMenu.addChild (mi);
		
		mi = new dijit.MenuItem ({
				label: this.resources_.RELOAD
			});
		mi.onClick = dojo.hitch (this, "reloadChart");
		ddMenu.addChild (mi);
		
		var button = new dijit.form.ComboButton (
			{	label: this.resources_.REFRESH,
				showLabel: false,
				iconClass: "refreshIcon",
				onClick: dojo.hitch (this, this.refreshChart),
				dropDown: ddMenu
			});
		tblToolbar.addChild (button);
		tblToolbar.addChild (new dijit.ToolbarSeparator());

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.id + "tblPreferences",
				iconClass: "propertiesIcon",
				label: this.resources_.PREFERENCES,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.showPrefs) 
			});
		tblToolbar.addChild (button);
//		this.tooltips.push (new dijit.Tooltip({label:this.resources_.PREFERENCES, connectId:[this.id + "tblPreferences"]}, document.createElement("div")));

		// Only add toggle to chart if it is also a chart
	    var chartRadio = dijit.byId(this.namespace + "chartSelected");
		if (!chartRadio.disabled) {
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	id: this.id + "tblToggle", 
					iconClass: "chartGenericIcon",
					label: this.resources_.TOGGLE_TABLE_CHART,
					showLabel: false,
				 	onClick: dojo.hitch (this, this.toggleTableChart) 
				});
			tblToolbar.addChild (button);
		}
//		this.tooltips.push (new dijit.Tooltip({label:this.resources_.TOGGLE_TABLE_CHART, connectId:[this.id + "tblToggle"]}, document.createElement("div")));

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.id + "tblReturnToSelector", 
				iconClass: "backIcon",
				label: this.resources_.RETURN_TO_CHART_SELECTOR,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.showChartSelector) 
			});
		tblToolbar.addChild (button);
//		this.tooltips.push (new dijit.Tooltip({label:this.resources_.RETURN_TO_CHART_SELECTOR, connectId:[this.id + "tblReturnToSelector"]}, document.createElement("div")));

		tblToolbar.addChild (new dijit.ToolbarSeparator());

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.id + "tblPrintPreview", 
				iconClass: "printPreviewIcon",
				label: this.resources_.PRINT_PREVIEW,				
				showLabel: false,
			 	onClick: dojo.hitch (this, this.printPreviewTable) 
			});
		tblToolbar.addChild (button);
//		this.tooltips.push (new dijit.Tooltip({label:this.resources_.PRINT_PREVIEW, connectId:[this.id + "tblPrintPreview"]}, document.createElement("div")));

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.id + "tblSaveAs", 
				iconClass: "saveToFileIcon",
				label: this.resources_.SAVE_AS_FILE,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.saveTableAsFile) 
			});
		tblToolbar.addChild (button);
		
		
		ddMenu = new dijit.Menu ();
		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		if (context.roles.admin || context.roles.creator) {
			mi = new dijit.MenuItem ({
				label: this.resources_.SAVE_FOR_OTHERS
			});
			mi.onClick = dojo.hitch (this, "onSave");
			ddMenu.addChild (mi);
		
			mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_DEFAULT_PREFERENCES
			});
			mi.onClick = dojo.hitch (this, "onReset");
			ddMenu.addChild (mi);
		}
		
		mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_MY_PREFERENCES
			});
		mi.onClick = dojo.hitch (this, "onResetMyPreferences");
		ddMenu.addChild (mi);
		
		button = new dijit.form.DropDownButton (
			{	label: this.resources_.PUBLISH,
				showLabel: false,
				iconClass: "editIcon",
				dropDown: ddMenu
			});
		tblToolbar.addChild (button);
//		this.tooltips.push (new dijit.Tooltip({label:this.resources_.SAVE_AS_FILE, connectId:[this.id + "tblSaveAs"]}, document.createElement("div")));
	},
	addButtonsToChartToolbar: function (toolbar) {
		var ddMenu = new dijit.Menu ();
		var mi = new dijit.MenuItem ({
				label: this.resources_.REFRESH
			});
		mi.onClick = dojo.hitch (this, "refreshChart");
		ddMenu.addChild (mi);
		
		mi = new dijit.MenuItem ({
				label: this.resources_.RELOAD
			});
		mi.onClick = dojo.hitch (this, "reloadChart");
		ddMenu.addChild (mi);
		
		var button = new dijit.form.ComboButton (
			{	label: this.resources_.REFRESH,
				showLabel: false,
				iconClass: "refreshIcon",
				onClick: dojo.hitch (this, this.refreshChart),
				dropDown: ddMenu
			});
		toolbar.addChild (button);
		toolbar.addChild (new dijit.ToolbarSeparator());

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.namespace + "prefButton",
				iconClass: "propertiesIcon",
				label: this.resources_.PREFERENCES,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.showPrefs) 
			});
		toolbar.addChild (button);

		button = new ibm.tivoli.tip.dijit.TIPButton(
				{	id: this.namespace + "toggleButton", 
					iconClass: "tableIcon",
					label: this.resources_.TOGGLE_TABLE_CHART,
					showLabel: false,
				 	onClick: dojo.hitch (this, this.toggleTableChart) 
				});
		toolbar.addChild (button);

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.namespace + "returnButton", 
				iconClass: "backIcon",
				label: this.resources_.RETURN_TO_CHART_SELECTOR,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.showChartSelector) 
			});
		toolbar.addChild (button);

		toolbar.addChild (new dijit.ToolbarSeparator());

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.namespace + "printPreviewChartButton", 
				iconClass: "printPreviewIcon",
				label: this.resources_.PRINT_PREVIEW,				
				showLabel: false,
			 	onClick: dojo.hitch (this, this.printPreviewChart) 
			});
		toolbar.addChild (button);

		button = new ibm.tivoli.tip.dijit.TIPButton(
			{	id: this.id + "saveChartAsFileButton", 
				iconClass: "saveChartIcon",
				label: this.resources_.SAVE_AS_FILE,
				showLabel: false,
			 	onClick: dojo.hitch (this, this.saveChartAsFile) 
			});
		toolbar.addChild (button);
		
		
		ddMenu = new dijit.Menu ();
		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		if (context.roles.admin || context.roles.creator) {
			mi = new dijit.MenuItem ({
				label: this.resources_.SAVE_FOR_OTHERS
			});
			mi.onClick = dojo.hitch (this, "onSave");
			ddMenu.addChild (mi);
		
			mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_DEFAULT_PREFERENCES
			});
			mi.onClick = dojo.hitch (this, "onReset");
			ddMenu.addChild (mi);
		}
		
		mi = new dijit.MenuItem ({
				label: this.resources_.CLEAR_MY_PREFERENCES
			});
		mi.onClick = dojo.hitch (this, "onResetMyPreferences");
		ddMenu.addChild (mi);
		
		button = new dijit.form.DropDownButton (
			{	label: this.resources_.PUBLISH,
				showLabel: false,
				iconClass: "editIcon",
				dropDown: ddMenu
			});
		toolbar.addChild (button);
	},
	toggleTableChart: function() {
	    var tableRadio = dijit.byId(this.namespace + "tableSelected");
	    var chartRadio = dijit.byId(this.namespace + "chartSelected");
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");

	    if (tableRadio.checked) {
    	  	tableRadio.setChecked (false);
	       	chartRadio.setChecked (true);
			// Hide table
	        this.tableDiv.style.display = "none";
	        // Show chart toolbar
       		if (!this.toolbarHidden_) {
		        this.toolbarDiv.domNode.style.display = "";
		    }
			// Show chart
			this.chartDiv.style.display = "";
			chartNavigator.setCurrentPage (chartNavigator.CHART_PAGE);
	    } else {
	       	tableRadio.setChecked (true);
	       	chartRadio.setChecked (false);
			// Show table
	        this.tableDiv.style.display = "";
			// Hide chart
			this.chartDiv.style.display = "none";
	        // Hide chart toolbar
	        this.toolbarDiv.domNode.style.display = "none";
			chartNavigator.setCurrentPage (chartNavigator.TABLE_PAGE);
			this.createOrUpdateTable(false);
		}
		
		this.saveUserPrefs();
	},
	showChartSelector: function () {
    if (this.interval !== null) {
      // if auto update is on, cancel it
			clearTimeout (this.interval);
			this.interval = null;
		}
		this.onResetMyPreferences(false);
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		chartNavigator.setCurrentPage (chartNavigator.CHART_SOURCE_TYPE_PAGE);
		
		this.clearWidthHeight();
	
		this.chartDiv.style.display = "none";
		this.tableDiv.style.display = "none";

		this.cleanupForChartChange();
			
		var tbl2 = document.createElement("table");
		tbl2.setAttribute ("id", this.namespace + "tbl_tipChartData");
		tbl2.setAttribute ("class", "tipTable");
		this.tableDiv.appendChild (tbl2);
		
		dojo.style (this.metaDiv, "display", "none");
		if (!chartNavigator.isNavigatorVisible()) {
			chartNavigator.showChartNavigator();
		}
		
		this.reportSelectorDiv.style.display = "";
	
		this.handleNotPrimedWebService();
	},
	cleanupForChartChange: function () {
		// remove parametersDiv
		var paramDiv = dijit.byId (this.namespace + "parametersDiv");
		if (paramDiv) {
			paramDiv.destroy();
		}
	
		// remove old table
		this.destroyTable();
				
		// restore portlet title to translated Charting
		this.tipHelper_.setPortletTitle (this.domNode, this.chartingTitle_);
		this.global.setPortletTitle (this.tipHelper_.getPortletTitle (this.domNode));
	},
	destroyTable: function () {
		// remove old table
		var tbl = dijit.byId (this.namespace + "tbl_tipChartData");
		if (tbl) {
			// remove from DOM
			this.tableDiv.removeChild (dojo.byId (this.namespace + "tbl_tipChartData"));

			console.log ("dijit.registry: ");
			console.log (dijit.registry);
			// remove from dijit registry
			tbl.destroyRecursive();
		}
	},
	handleNotPrimedWebService: function () {
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		if ((chartNavigator.getChartSourceType() == "WebService") &&
			(chartNavigator.connectionMgr_ === null)) {
			chartNavigator.showChartNavigator();
		}
	},
	printPreviewTable: function () {
		var view = this.namespace + "tbl_tipChartData";
		var nameField = dojo.byId(this.namespace + "chartName");
		var name = encodeURI(nameField.value);
		
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=printTable&arg_view="+view+"&print=true&arg_portletNamespace=" + this.namespace + "&arg_name=" + name;

		window.open (url, 'TIPChartPrintPreview');
	},
	saveTableAsFile: function () {
		var view = this.namespace + "tbl_tipChartData";
		var nameField = dojo.byId(this.namespace + "chartName");
		var name = encodeURI(nameField.value);
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=printTable&arg_view="+view+"&arg_portletNamespace=" + this.namespace + "&arg_name=" + name;

		window.open (url, 'TIPChartPrintPreview');
	},
	printPreviewChart: function () {
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=" + this.namespace + "&print=true";

		window.open (url, 'TIPChartPrintPreview');
	},
	saveChartAsFile: function () {
		var url = "/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=" + this.namespace;

		window.open (url, 'TIPChartPrintPreview');
	},
	onNubKeyPress: function (event) {
	  	if (event.keyCode == 13) {
		  	event.cancelBubble = true;
	  		event.returnValue = false;
	  		console.log ("in onNubKeyPress: pressed enter");
	  		this.toggleToolbar();
	  	}
	},
	toggleToolbar: function () {
		this.toggleTableToolbar();
		this.toggleChartToolbar();
	},
	toggleTableToolbar: function () {
		console.log ("enter toggle table toolbar");
		
		var tipTable = dijit.byId (this.namespace + "tbl_tipChartData");
		if (tipTable) {
			dojo.unsubscribe (this.toggleSubscription);
			this.toggleSubscription = null;
			tipTable.toggleToolbar();	
			// subscribe to table toolbar toggle, so we can also toggle the chart toolbar			
			this.toggleSubscription = dojo.subscribe (tipTable.id + "#ToggleToolbar", this, "toggleChartToolbar");
		}
	},
	toggleChartToolbar: function () {
		console.log ("enter toggle toolbar. existing state: " + this.toolbarHidden_);
		if (this.toolbarHidden_) {
			this.openChartToolbar();
		} else {
			this.closeChartToolbar();
		}
	},
	openChartToolbar: function () {
		this.toolbarHidden_ = false;
		console.log ("showing toolbar");
		this.animateToolbarOpen();
		dojo.addClass (this.nub, "nubOpen");
		dojo.removeClass (this.nub, "nubClosed");
	},
	closeChartToolbar: function () {
		this.toolbarHidden_ = true;
		console.log ("hiding toolbar");
		this.animateToolbarClose();
		dojo.addClass (this.nub, "nubClosed");
		dojo.removeClass (this.nub, "nubOpen");
	},
	animateToolbarOpen: function () {
		dojo.fx.wipeIn ({
				node: dojo.byId (this.namespace + "toolbarDiv"),
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                })
			}).play();
	},
	animateToolbarClose: function () {
		dojo.fx.wipeOut ({
				node: dojo.byId (this.namespace + "toolbarDiv"),
		        duration: 500,
                onEnd: dojo.hitch(this, function() {
                })
			}).play();
	},
	dlgOKCallback: function () {
		var ct = dojo.byId(this.namespace + "chartType");
		var tableRadio = dijit.byId(this.namespace + "tableSelected");
		var chartRadio = dijit.byId(this.namespace + "chartSelected");
        var showTable = dijit.byId(this.namespace + "showTableCB");
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");

        if (tableRadio.checked) {
			// Show table
            this.tableDiv.style.display = "";
			// Hide chart
			this.chartDiv.style.display = "none";
			// Set the current page
			chartNavigator.setCurrentPage (chartNavigator.TABLE_PAGE);
			// Update table
//            this.createOrUpdateTable (true);
        } else {
			// Hide table
            this.tableDiv.style.display = "none";
			// Show chart
			this.chartDiv.style.display = "";
			// Set the current page
			chartNavigator.setCurrentPage (chartNavigator.TABLE_PAGE);
        }
            
		if (this.propertiesChanged()) {
          	this.updateProperties(true);
        } 
            
//	    this.configAutoUpdate();
	},
	propertiesChanged: function () {
		console.log ("enter propertiesChanged");
		var nameField = dojo.byId(this.namespace + "chartName");
		var descriptionField = dijit.byId(this.namespace + "description");
		var heightField = dojo.byId(this.namespace + "height");
		var widthField = dojo.byId (this.namespace + "width");
		var showLegendField = dijit.byId (this.namespace + "showLegendCB");
	    var chartRadio = dojo.byId(this.namespace + "chartSelected");
		var checkForUpdatesCB = dijit.byId (this.namespace + "checkForUpdatesCB");
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		var chartTypeSelector = dijit.byId(this.namespace + "mainChartNavigatorDiv");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");
		var showLastUpdated = dijit.byId (this.namespace + "showLastUpdated");
	
		var paramFragment = this.getParametersAsURLFragment();
	
		var result = (nameField.value != this.oldName_) ||
				(heightField.value != this.oldHeight_) ||
				(widthField.value != this.oldWidth_) ||
				(showLegendField.checked != this.oldShowLegend_) ||
				(descriptionField.getValue() != this.oldDescription_) ||
				(chartRadio.checked != this.oldChartRadio_) ||
				(checkForUpdatesCB.checked != this.oldCheckForUpdates_) ||
				(receiveContextRadio.checked != this.oldReceiveContext_) ||
				(chartTypeSelector.getChartSourceType() != this.oldChartSourceType_) ||
				(showNameInChart.checked != this.oldShowNameInChart_) ||
				(showLastUpdated.checked != this.oldShowLastUpdated_) ||
				(this.oldParameterFragment_ != paramFragment);
		console.log ("exit propertiesChanged: " + result);			
	
		return (result);
	},
	updateProperties: function (persist) {
		console.log ("enter updateProperties");
		var shouldRunReport = this.shouldRunReport();

		var nameField = dojo.byId(this.namespace + "chartName");
		var descriptionField = dijit.byId(this.namespace + "description");
		var showLegendField = dijit.byId (this.namespace + "showLegendCB");
	    var tableRadio = dijit.byId(this.namespace + "tableSelected");
	    var chartRadio = dijit.byId(this.namespace + "chartSelected");
		var checkForUpdatesCB = dijit.byId (this.namespace + "checkForUpdatesCB");
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		var useDefaultContextRadio = dijit.byId(this.namespace + "defaultParametersSelected");
		var chartNavigator = dijit.byId(this.namespace + "mainChartNavigatorDiv");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");
		var showLastUpdated = dijit.byId (this.namespace + "showLastUpdated");
	
		var urlFrag = {};

		if (nameField.value != this.oldName_) {
			urlFrag.arg_newName = encodeURI(nameField.value);
			dojo.byId(this.namespace + "chartNameH").value = nameField.value;
			this.oldName_ = nameField.value;
		}
		if (descriptionField.getValue() != this.oldDescription_) {
			urlFrag.arg_description = encodeURI(descriptionField.getValue());
			dojo.byId(this.namespace + "descriptionH").value = descriptionField.getValue();
			this.oldDescription_ = descriptionField.getValue();
		}
		if (showLegendField.checked != this.oldShowLegend_) {
			urlFrag.arg_showLegend = showLegendField.checked;
			this.oldShowLegend_ = showLegendField.checked;
		}
		if (tableRadio.checked) {
			urlFrag.arg_viewType = "table";
			this.oldChartRadio_ = false;
		} else if (chartRadio.checked) {
			urlFrag.arg_viewType = "chart";
			this.oldChartRadio_ = true;
		}
	
		console.log ("checkForUpdatesCB: ");
		console.log (checkForUpdatesCB);
		
		if (checkForUpdatesCB.checked != this.oldCheckForUpdates_) {
			urlFrag.arg_autoUpdateActive = checkForUpdatesCB.checked;
			this.oldCheckForUpdates_ = checkForUpdatesCB.checked;
		}
		
		if (showNameInChart.checked != this.oldShowNameInChart_) {
			urlFrag.arg_showNameInChart = showNameInChart.checked;
			this.oldShowNameInChart_ = showNameInChart.checked;
		}
		
		if (showLastUpdated.checked != this.oldShowLastUpdated_) {
			urlFrag.arg_showLastUpdated = showLastUpdated.checked;
			this.oldShowLastUpdated_ = showLastUpdated.checked;
		}
	
		if (receiveContextRadio.checked != this.oldReceiveContext_) {
			this.oldReceiveContext_ = receiveContextRadio.checked;
		}
		
		if (chartNavigator.getChartSourceType() != this.oldChartSourceType_) {
			this.oldChartSourceType_ = chartNavigator.getChartSourceType();
		}
	
		urlFrag.arg_chartSourceType = chartNavigator.getChartSourceType();
		urlFrag.arg_receiveContextActive = receiveContextRadio.checked;

		var paramFragment = this.getParametersAsURLFragment();
		this.oldParameterFragment_ = paramFragment;

		urlFrag.command = "modifyReport";
		urlFrag.arg_portletNamespace = this.namespace;
		urlFrag.arg_reportName = encodeURI(this.reportName);
		urlFrag.arg_chartSrcType = this.chartSrcType_;
		urlFrag.arg_connectionName = encodeURI(this.connectionName_);
		urlFrag.arg_groupID = this.groupID_;
		urlFrag.arg_dataSource = encodeURI(this.connectionName_);
		urlFrag.arg_chartID = encodeURI(this.reportName);
		urlFrag.arg_persist = persist;
		var size = this.getChartSize();
		urlFrag.arg_width = size.available.w;
		urlFrag.arg_height = size.available.h;
//		this.clearWidthHeight();
		
		var params = this.getParametersAsJSObject();
		for (var i in params) {
			if (i) {
				urlFrag[i] = params[i];
			}
		}
		
		// Update the internal data store used by parameters widget with the latest 
		// values from the text boxes
		dijit.byId (this.namespace + "parametersDiv").updateInternalParametersData();
	
		var handler = null;
	
	    if (shouldRunReport) {
			urlFrag.shouldRunReport = "true";
			handler = dojo.hitch (this, this.runReportCallback);
		}

		var kw = {
             content: urlFrag,
           	 onResponse:{
                      callback: handler
   	                }
   	     };

		this.makeRequestPrimitive (kw);
                    
		this.setVisibilityPostRunReport ();                    
	},
	shouldRunReport: function () {
		console.log ("enter shouldRunReport");
		var nameField = dojo.byId(this.namespace + "chartName");
		var descriptionField = dijit.byId(this.namespace + "description");
		var heightField = dojo.byId(this.namespace + "height");
		var widthField = dojo.byId (this.namespace + "width");
		var showLegendField = dijit.byId (this.namespace + "showLegendCB");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");
	
		var serverPropsChanged = (nameField.value != this.oldName_) ||
				(heightField.value != this.oldHeight_) ||
				(widthField.value != this.oldWidth_) ||
				(showNameInChart.checked != this.oldShowNameInChart_) ||
				(showLegendField.checked != this.oldShowLegend_) ||
				(descriptionField.getValue() != this.oldDescription_);

		var result = (this.updateMultiplier > 1) ||
           	(this.getParametersAsURLFragment() != this.oldParameterFragment_) ||
           	serverPropsChanged;
           	
	    console.log ("exit shouldRunReport: " + result);
	    return (result);
	},
	dlgOKPressed: function() {
	    var ds = dojo.byId(this.namespace + "dataSource");
	    var ct = dojo.byId(this.namespace + "chartType");
		var checkForUpdates = dijit.byId(this.namespace + "checkForUpdatesCB");

		if (checkForUpdates.checked) {
            
			if (this.interval !== null) {
				clearTimeout (this.interval);
				this.interval = null;
			}
		} else {
			if (this.interval !== null) {
				clearTimeout (this.interval);
				this.interval = null;
			}
		}

	    // Close dialog
	    var prefsDialog = dijit.byId(this.namespace + "prefs_dialog");
	    prefsDialog.hide();
	    this.dlgOKCallback();
	},
	showImage: function () {
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		chartNavigator.setCurrentPage (chartNavigator.CHART_PAGE);
	
		if (this.chartImage.src) {
			this.animateChartChange();
		} else {
			// just load the image
			this.chartImage.src = '/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=' + this.namespace + '&arg_count=' + this.counter;
		}
	},
	animateChartChange: function () {
		dojo.fx.chain([ 
	       dojo.fadeOut({ 
	       	   node: this.namespace + 'chartImage', 
	       	   duration: 300,
               onEnd: dojo.hitch(this, function() {
                   // load the new image
					if (this.actualImgWidth_ && this.actualImgHeight_) {
						this.chartImage.width = this.actualImgWidth_;
						this.chartImage.height = this.actualImgHeight_;
						this.chartImage.src = '/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=' + this.namespace + '&arg_count=' + this.counter;
					} else {
						this.chartImage.src = '/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=' + this.namespace + '&arg_count=' + this.counter;
					}
				})
			}),
	 	    dojo.fadeIn({ 
	 	   		node: this.namespace + 'chartImage', 
	 	   		duration: 300,
	 	   		onEnd: dojo.hitch (this, function () {
					// do nothing for now
	 	   		})
	 	    })
		]).play();	
	},
	setMap: function (data) {
	    var mapDiv = dojo.byId (this.namespace + "mapDiv");
		var html = '<map name="chartmap"> ' + data.map + ' </map>';
	  	mapDiv.innerHTML = html;
	},
	loadImage: function () {
		// load the new image
		var newImage = new Image();
		this.counter++;
		newImage.src = '/ibm/TIPChartPortlet/TIPChartController?command=getChart&arg_portletNamespace=' + this.namespace + '&arg_count=' + this.counter;
	// wait 250 ms... this should give the image time to load, then update the DOM to display it
	//	setTimeout(<portlet:namespace/>showImage, 250);
		this.showImage();
	},
	runLatestReport: function () {
		this.runReport(
				this.chartSrcType_,
				this.connectionName_,
				this.groupID_,
				this.reportName,
				false,
				false,
				false,    
				false);   // show processing indicator
	},
	configAutoUpdate: function () {
		console.log ("enter configAutoUpdate: updateMultiplier: " + this.updateMultiplier + 
			", interval: " + this.interval + 
			", and checkForUpdates: " + dijit.byId (this.namespace + "checkForUpdatesCB").checked);
		
		if ((this.interval === null) && (this.updateMultiplier > 0) &&
			dijit.byId (this.namespace + "checkForUpdatesCB").checked) {
			// equivalent of sleep, then run function
//    	    var fnc = 'this.runReport("' + this.chartSrcType_ + '", "' + this.connectionName_ + '", "' + this.groupID_ + '", "' + this.reportName +'")';
			var fnc = dojo.hitch (this, this.runLatestReport);
		    var nextUpdate = this.getUpdateInterval();
		    this.interval = setTimeout(fnc, nextUpdate);
		} else {
            if ((this.interval !== null) && (!dijit.byId (this.namespace + "checkForUpdatesCB").checked)) {
                clearTimeout (this.interval);
                this.interval = null;
            }
		}
		console.log ("exit configAutoUpdate");
	},
  getUpdateInterval: function() {
      var nextUpdateInterval = this.runReportResponseTime * this.updateMultiplier;
console.log("responsetime="+(this.runReportResponseTime/1000)+"s next update="+(nextUpdateInterval/1000)+"s  update minimum interval="+(this.updateMinInterval/1000)+"s update maximum interval="+(this.updateMaxInterval/1000)+"s");
      if ( nextUpdateInterval > this.updateMaxInterval ) {
        nextUpdateInterval = this.updateMaxInterval;
      } else if ( nextUpdateInterval < this.updateMinInterval ) {
        nextUpdateInterval = this.updateMinInterval;
      }
      return nextUpdateInterval;
  },
	runReportCallback: function (data, ioArgs) {
		console.log ("Enter runReportCallback with data");
		console.log (data);
		// get the time taken to run report
		this.runReportResponseTime = new Date().getTime() - this.runReportStartTime;
		try {
		this.reportSelectorDiv.style.display = "none";
		
	   	if (data.chartRendered.toString().toLowerCase() == "true") {
   			console.log ("chartRendered true");
	   		this.setMap(data);
	   		this.loadImage();
   			this.showChartAndTable();
	   	} else {
	   		console.log ("chartRendered false");
	   		this.showTableOnly();
	   	}
		   	
		this.updateMetaData();
			
	    var chartDiv = dojo.byId(this.namespace + "chartDiv");
	    var tableDiv = dojo.byId(this.namespace + "tableDiv");
	    var tableRadio = dijit.byId(this.namespace + "tableSelected");
    
	    if (tableRadio.checked) {
			// Show table
	        tableDiv.style.display = "";
			// Hide chart
			chartDiv.style.display = "none";
	    } else {
			// Hide table
	        tableDiv.style.display = "none";
			// Show chart
			chartDiv.style.display = "";
		}
		
		// Clear the old timeout so we can start the new one.
		if (this.interval !== null) {
			clearTimeout (this.interval);
			this.interval = null;
		}
		
		console.log ("ChartWidget.runReportCallback: requestQueue length before removing current request: " + this.requestQueue.length);
		if (this.requestQueue.length == 1) {
			console.log ("ChartWidget.runReportCallback: removing current request. no more requests to handle");
			// this was the only request in the queue, so remove it
			this.requestQueue.shift();
			// check to see if auto update should be turned back on
			this.configAutoUpdate();
		} else if (this.requestQueue.length < 1) {
			console.log ("ChartWidget.runReportCallback: queue length is less than 1... nothing to remove or process");
			// this shouldn't happen, but if nothing is in the queue, 
			// we should check to see if auto update needs to be re-enabled.
			this.configAutoUpdate();
		} else {  // length > 1
			console.log ("ChartWidget.runReportCallback: queue length is greater than 1... removing current request and processing next request");
			// pop off the most recent request which was just run
			this.requestQueue.shift();
			// now get the current request to invoke
			var requestContext = this.requestQueue.shift();
			ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( requestContext );			
		}

		} catch (ex) {
			console.error ("Caught javascript exception in runReportCallback");
			console.error (ex);
		}
	},
	updateMetaData: function () {
		var lastUpdated = new Date();
		var msg = dojo.string.substitute(this.resources_.LAST_UPDATED, { 0: lastUpdated.toLocaleString() });
		this.metaDiv.innerHTML = "<p class=\"lastUpdated\">" + msg + "</p>";
	},
	getParametersAsJSObject: function () {
		var params = {};

		if (this.paramWidget_ !== null) {
			var model = this.paramWidget_.getParameters();
			var param = null;
			var value = null;
		
			for (var row in model) {
				if (row) {
					param = model[row];
					value = this.paramWidget_.getParameterValue(param.name);
			
					if ((value !== null) && (value !== "")) {
						params["param_" + param.name] = encodeURI (value);
					}
				}
			}
		}

	//	alert ("urlFragment: " + urlFragment);
		console.log ("paramsAsJSObject: ");	
		console.log (params);
		return (params);
	},
	getParametersAsURLFragment: function () {
		var urlFragment = "";

		if (this.paramWidget_ !== null) {
			var model = this.paramWidget_.getParameters();
			var param = null;
			var value = null;
		
			for (var row in model) {
				if (row) {
					param = model[row];
					value = this.paramWidget_.getParameterValue(param.name);
			
					if ((value !== null) && (value !== "")) {
						if (urlFragment !== "") {
							urlFragment += "&";
						}
						urlFragment += "param_";
						urlFragment += param.name;
						urlFragment += "=";
						urlFragment += value;
					}
				}
			}
		}

	//	alert ("urlFragment: " + urlFragment);
		console.log ("urlFragment: " + urlFragment);	
		return (urlFragment);
	},
	refreshChart: function () {
		this.runReport (this.chartSrcType_, this.connectionName_, this.groupID_, this.reportName, false, false, false);	
	},
	reloadChart: function () {
	    // Pass along another param to tell runReport to grab new one
  		this.runReport (this.chartSrcType_, this.connectionName_, this.groupID_, this.reportName, false, true, false);	
	},

	clearWidthHeight: function () {
		console.log ("clearing image width and height");
		this.actualImgHeight_ = null;
		this.actualImgWidth_ = null;
		dijit.registry.remove (this.namespace + "chartImage");
		delete this.chartImage;
		dojo.byId (this.id + "chartImageDiv").innerHTML = '<a class="imagemap">' +
					'<img id="' + this.namespace + 'chartImage" dojoAttachPoint="chartImage" border="0" ISMAP USEMAP="#chartmap" /> ' +
					'</a>';
		this.chartImage = dojo.byId (this.namespace + 'chartImage');
//		this.chartDiv.innerHTML = '';
	},
	saveWidthHeight: function () {
		var img = dojo.byId (this.namespace + "chartImage");
		if (img) {
			console.log ("setting image height: " + img.height + " and width: " + img.width);
			this.actualImgHeight_ = img.height;
			this.actualImgWidth_ = img.width;
		} else {
			console.log ("no existing image for height & width");
		}
	},
	runReport: function (chartSrcType, connectionName, groupID, report, clearPreferences, reloadChart, persist, showProcessing) {
		console.log ("enter ChartWidget.runReport");
		this.saveWidthHeight ();
		this.reportName = report;
		this.chartSrcType_ = chartSrcType;
		this.connectionName_ = connectionName;
		this.groupID_ = groupID;
		
		var urlFragment = "";
		var content = {};
		
		if (!clearPreferences) {
			try { 
				urlFragment = this.getParametersAsURLFragment();
				content = this.getParametersAsJSObject();
			} catch (e) {
				// eat
			}
		}
		
		this.oldParameterFragment_ = urlFragment;
		
		content.command = "modifyReport";
		var size = this.getChartSize();
		content.arg_width = size.available.w;
		content.arg_height = size.available.h;		
		
		content.arg_portletNamespace = this.namespace;
		content.arg_reportName = encodeURI(report);
		content.arg_chartSrcType = chartSrcType;
		content.arg_connectionName = encodeURI(connectionName);
		content.arg_datasource = encodeURI(connectionName);
		content.arg_chartID = encodeURI(report);
		content.arg_groupID = encodeURI(groupID);
		content.shouldRunReport = true;
		
		if (clearPreferences) {
			content.clearPreferences="true";
		} 
		if (reloadChart) {
			content.reloadChart="true";
		}
		if (persist) {
			content.arg_persist = "true";
		}

		var errorHandler = { showMsg : true, callback: dojo.hitch (this, this.onError) };

		var kw = {
			showBusyDialog: showProcessing,
             content: content,
             onExc: errorHandler,
           	 onResponse:{
                      callback: dojo.hitch (this, this.runReportCallback)
   	                }
   	     };

		this.makeRequestPrimitive (kw);
		
		this.setVisibilityPostRunReport ();
		console.log ("exit ChartWidget.runReport");
	},
	setVisibilityPostRunReport: function () {
		// set the chart name in case it changed
		var nameField = dojo.byId(this.namespace + "chartName");
		if ((nameField.value !== undefined) && (nameField.value !== "")) {
			this.tipHelper_.setPortletTitle (this.domNode, nameField.value);
			this.global.setPortletTitle (this.tipHelper_.getPortletTitle (this.domNode));
		}
		
		if (this.toolbarHidden_) {
			dojo.byId (this.namespace + "toolbarDiv").style.display = "none";
		} else {
			dojo.byId (this.namespace + "toolbarDiv").style.display = "";
		}
	
		if (dijit.byId (this.namespace + "showLastUpdated").checked) {
			dojo.style (this.metaDiv, "display", "");
		} else {
			dojo.style (this.metaDiv, "display", "none");
		}
	},
	onReportSelection: function (chartSrcType, connectionName, groupID, chartID, chartName, clearPreferences, persist) {
		this.tablesCreated = false;
		this.oldName_ = chartName;
		this.reportName = chartID;
		this.chartSrcType_ = chartSrcType;
		this.connectionName_ = connectionName;
		this.groupID_ = groupID;
	
	    this.getChartInfo(chartSrcType, connectionName, groupID, chartID, chartName, persist);
		this.runReport (chartSrcType, connectionName, groupID, chartID, clearPreferences, false, persist);
	},
	createParamWidget: function (reportName) {
	//	var data = [{"helpText":"Hello World","displayName":"Hostname","name":"Hostname","type":"0","promptText":"Enter a hostname"}];
				var kw = {
	             content:   {
                        command: "readParameters",
                        arg_portletNamespace: this.namespace,
                        arg_reportName: encodeURI(reportName),
                        arg_chartSrcType: this.chartSrcType_,
                        arg_connectionName: encodeURI(this.connectionName_),
                        arg_groupID: encodeURI(this.groupID_)
                        },
            	 onResponse:{
                         callback: dojo.hitch(this, this.createParamWidgetCallback)
       	                }
    	     };
	
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	createParamWidgetCallback: function (data, ioArgs) {
		// remove existing parametersDiv
		var paramDiv = dijit.byId (this.namespace + "parametersDiv");
		if (paramDiv) {
			paramDiv.destroy();
		}
	
		var div = dojo.doc.createElement( 'div' );
		div.id = this.namespace + "parametersDiv";
    	dojo.place(  div, dojo.byId (this.namespace + "parametersDivContainer"), "after"  );

		this.paramWidget_=new ibm.tivoli.tip.charts.dijit.ParametersWidget (
			{
				namespace: this.namespace,
				data: data
			}, 
			dojo.byId(this.namespace + "parametersDiv")
		);
		this.paramWidget_.startup();
	
		this.oldParameterFragment_ = this.getParametersAsURLFragment();
	},
	showTableOnly : function () {
	    var tableRadio = dijit.byId(this.namespace + "tableSelected");
	    var chartRadio = dijit.byId(this.namespace + "chartSelected");
	
		tableRadio.setChecked (true);
		tableRadio.setDisabled (true);
		chartRadio.setChecked (false);
		chartRadio.setDisabled (true);
		var showLegend = dijit.byId (this.namespace + "showLegendCB");
		showLegend.setDisabled(true);
		
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");
		showNameInChart.setDisabled(true);
	
		// Show table
    	this.tableDiv.style.display = "";
		// Hide chart and empty the contents
		dojo.byId (this.id + "chartImageDiv").innerHTML = "";
		this.chartDiv.style.display = "none";
	
		console.log ("showTableOnly. creating table");
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		chartNavigator.setCurrentPage (chartNavigator.TABLE_PAGE);

		this.createOrUpdateTable(true);
	},
	showChartAndTable: function() {
	    var tableRadio = dijit.byId(this.namespace + "tableSelected");
	    var chartRadio = dijit.byId(this.namespace + "chartSelected");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");

		showNameInChart.setDisabled(false);
		tableRadio.setDisabled (false);
		chartRadio.setDisabled (false);
		var showLegend = dijit.byId (this.namespace + "showLegendCB");
		showLegend.setDisabled(false);
	
		// Show toggle button
		var button = dojo.byId (this.namespace + "toggleButton");
		button.style.display = "";
	//	var toolbarDiv = dojo.byId (this.namespace + "toolbarDiv");
	//	toolbarDiv.removeChild (button);
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");

		if (chartRadio.checked) {	
			// Hide table
	    	this.tableDiv.style.display = "none";
			// Show chart
			this.chartDiv.style.display = "";
			chartNavigator.setCurrentPage (chartNavigator.CHART_PAGE);
		} else {
			console.log ("showChartAndTable: showTable & hideChart");
			// Show table
		    this.tableDiv.style.display = "";
			// Hide chart
			this.chartDiv.style.display = "none";
			chartNavigator.setCurrentPage (chartNavigator.TABLE_PAGE);
			this.createOrUpdateTable(true);
		}
	},
	makeChartProperties: function () {
		if (this.prefsDialog === null) {
			console.log ("making chart properties");
		    var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		
			this.prefsDialog = new ibm.tivoli.tip.charts.dijit.ChartProperties(
				{ 	namespace: this.namespace,
					contextPath: context.contextPath,
					formURL: context.portletActionURL
				},
				dojo.byId (this.namespace + "prefs_dialog")
			);
			this.prefsDialog.startup();
			console.log ("tipserverradio: ");
			console.log (dojo.byId (this.namespace + "tipServer"));
		} else {
			console.log ("chart properties already made");
		}
	},
	getChartInfoCallback: function (data, ioArgs) {
		console.log ("enter getChartInfoCallback with data");
		console.log (data);
		
		var nameField = dojo.byId(this.namespace + "chartName");
		var heightField = dojo.byId(this.namespace + "height");
		var widthField = dojo.byId (this.namespace + "width");
		var showLegendField = dijit.byId (this.namespace + "showLegendCB");
		var descriptionField = dijit.byId(this.namespace + "description");
		var chartRadio = dijit.byId (this.namespace + "chartSelected");
		var tableRadio = dijit.byId (this.namespace + "tableSelected");
		var checkForUpdatesCB = dijit.byId (this.namespace + "checkForUpdatesCB");
		var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		var useDefaultContextRadio = dijit.byId(this.namespace + "defaultParametersSelected");
		var showNameInChart = dijit.byId (this.namespace + "showNameInChart");
		var showLastUpdated = dijit.byId (this.namespace + "showLastUpdated");
		
		nameField.value = data.name;
		dojo.byId(this.namespace + "chartNameH").value = data.name;
		descriptionField.setValue(data.description);
		dojo.byId(this.namespace + "descriptionH").value = data.description;
		heightField.value = data.height;
		widthField.value = data.width;
		if ((data.showLegend === true) || (data.showLegend === "true")) {
			showLegendField.setChecked (true);
		} else {
			showLegendField.setChecked (false);
		}
	
		if ((data.viewType === null) || (data.viewType === "") || (data.viewType === "chart")) {
			chartRadio.setChecked (true);
			tableRadio.setChecked (false);
		} else {
			chartRadio.setChecked (false);
			tableRadio.setChecked (true);
		}
	
		if ((data.autoUpdateActive === true) || (data.autoUpdateActive === "true")) {
			checkForUpdatesCB.setChecked (true);
			
		} else {
			checkForUpdatesCB.setChecked (false);
		}
		
		if ((data.showNameInChart === true) || (data.showNameInChart === "true")) {
			showNameInChart.setChecked (true);
		} else {
			showNameInChart.setChecked (false);
		}
		
		if ((data.showLastUpdated === true) || (data.showLastUpdated === "true")) {
			showLastUpdated.setChecked (true);
		} else {
			showLastUpdated.setChecked (false);
		}

		this.mssName_ = data.mssName;
		if (data.updateMultiplier) {
			this.updateMultiplier = data.updateMultiplier;
    }
    if ( data.updateMinInterval ) {
      this.updateMinInterval = data.updateMinInterval;
    }
    if ( data.updateMaxInterval ) {
      this.updateMaxInterval = data.updateMaxInterval;
    }
  
		
		if ((data.receiveContextActive === true) || (data.receiveContextActive === "true")) {
			receiveContextRadio.setChecked (true);
			useDefaultContextRadio.setChecked  (false);
		} else {
			receiveContextRadio.setChecked (false);
			useDefaultContextRadio.setChecked (true);
		}
		
		this.oldName_ = nameField.value;
		this.oldDescription_ = descriptionField.getValue();
		this.oldHeight_ = heightField.value;
		this.oldWidth_ = widthField.value;
		this.oldShowLegend_ = showLegendField.checked;
		this.oldChartRadio_ = chartRadio.checked;
		this.oldCheckForUpdates_ = checkForUpdatesCB.checked;
		this.oldReceiveContext_ = receiveContextRadio.checked;
		this.oldShowNameInChart_ = showNameInChart.checked;
		this.oldShowLastUpdated_ = showLastUpdated.checked;
		
    	this.createParamWidget (this.reportName);
    	
    	// set the portlet title to the chart name
    	this.tipHelper_.setPortletTitle (this.domNode, data.name);
		this.global.setPortletTitle (this.tipHelper_.getPortletTitle (this.domNode));
    	
    	if (showLastUpdated.checked) {
    		dojo.style (this.metaDiv, "display", "");
    	}
	},
	getChartInfo: function (sourceType, connectionName, groupID, chartID, chartName, persist) {
		var errorHandler = { showMsg : true, callback: dojo.hitch (this, this.onError) };
		var kw = {
        	content:   {
        		command: "getChartInfo",
	        	arg_chartID: encodeURI(chartID),
    	    	arg_portletNamespace: this.namespace,
        		arg_chartSrcType: sourceType,
	        	arg_connectionName: encodeURI(connectionName),
    	    	arg_groupID: encodeURI(groupID),
    	    	arg_persist: persist
                   },
            onExc: errorHandler,
	        onResponse:{
	            callback: dojo.hitch(this, this.getChartInfoCallback)
       	           }
  	     };
  	     
  	     if (chartName !== null) {
  	     	kw.content.arg_chartName = encodeURI(chartName);
  	     }
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
	},
	serializeParameters: function (params) {
		console.log ("enter serializeParameters with params: ");
		console.log (params);
		var result = "";
		for (var p in params) {
			if (p) {
				result += "&" + p + "=" + params[p];			
			}
		}
	
		return (result);
	},
	getHiddenParameters: function () {
		var urlfrag = this.namespace + "chartNameH=" + dojo.byId (this.namespace + "chartNameH").value;
		urlfrag += "&" + this.namespace + "descriptionH=" + dojo.byId (this.namespace + "descriptionH").value;
		urlfrag += "&" + this.namespace + "connectionNameH=" + dojo.byId (this.namespace + "connectionNameH").value;
		urlfrag += "&" + this.namespace + "groupIDH=" + dojo.byId (this.namespace + "groupIDH").value;
		urlfrag += "&" + this.namespace + "chartIDH=" + dojo.byId (this.namespace + "chartIDH").value;
		var webservice = dojo.byId (this.namespace + "tivoliApp");
		var custom = dojo.byId (this.namespace + "tipServer");
		var type = "";
		if (webservice.checked) {
			type = webservice.value;
		} else if (custom.checked) {
			type = custom.value;
		}
		
		urlfrag += "&" + this.namespace + "chartSourceTypeGroup=" + type;

		return (urlfrag);	
	},
	saveUserPrefs: function () {
		this.savePrimitive ("saveUser", false);
	},
	onSave: function () {
		this.savePrimitive ("saveShared", true);
	},
	savePrimitive: function (action, showCompletion) {
		var callback = null;
		console.log ("enter savePrimitive with args: " + action + ", " + showCompletion);
		if (showCompletion) {
			callback = dojo.hitch (this, this.onSaveComplete);
		} 
		
		if (this.allowSubmit_) {
			var urlFrag = this.getParametersAsURLFragment();
			var form = dojo.byId (this.namespace + "form");
			var propertiesDialogForm = dijit.byId (this.namespace + "propertiesForm");
			var hiddenparams = this.getHiddenParameters();
			var url = "?" + hiddenparams + "&" + urlFrag + this.serializeParameters (propertiesDialogForm.getValues());

			var tbl = dijit.byId (this.namespace + "tbl_tipChartData");
			if (tbl) {
				var tablePrefs = tbl.getPreferences();
				if (tablePrefs !== null) {
					url += "&arg_tablePreferences=" + tablePrefs;
				}
			}
			
			var kw = {
				 appendURL: url,
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		} else {
			this.allowSubmit_ = true;
		}
		
		// do not submit
		return (false);
	},
	onResetPrimitive: function (action, reloadPrefs) {
		var callback = null;
		
		if (reloadPrefs === false) {
			callback = null;
		} else {
			callback = dojo.hitch(this, this.afterReset);
		}
		
		var kw = {
	             content:   {
                        command: "savePreferences",
                        arg_portletNamespace: this.namespace,
                        action: action
                        },
            	 onResponse:{
                         callback: callback
       	                }
    	     };
	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );			
		
		// do not submit
		return (false);
	},
	onReset: function () {
		this.onResetPrimitive ("resetDefaults");
	},
	onResetMyPreferences: function (reloadDefaultPrefs) {
		this.onResetPrimitive ("resetMine", reloadDefaultPrefs);
	},
	afterReset: function () {
		var chartNav = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		chartNav.showChartNavigator();
		this.showChartSelector();
		chartNav.getPortletPreferences();
	},
	onSaveComplete: function (data, ioArgs) {
		console.info ("enter onSaveComplete");

		var msgID = "TIPCH3005I";
		var msg = this.messages_.TIPCH3005I;
		var context = ibm.tivoli.tip.charts.dojo.Global.getSingleton(this.namespace);
		context.showMessageDialog("info", msgID, msg, false);
	},
	showPrefs: function () {
		var prefsDialog = dijit.byId (this.namespace + "prefs_dialog");
		if (prefsDialog) {
			try {
				prefsDialog.show();
			} catch (e) {
				// alert ("caught error: " + e);
				console.error ("caught error in showPrefs: ");
				console.error (e);
				// handle IE error and try to show again.
				prefsDialog.show();
			}
		}
	},
	getSourceMSSName: function () {
		//	e.g., "ibm-cdm:///CDMMSS/Hostname=pico.raleigh.ibm.com,ManufacturerName=IBM,ProductName=IBM Tivoli Monitoring"
		return this.mssName_;
	},
	isInteger: function (value) {
	//    return ((value+'').match(/^(+|-)?\d+$/) != null);
		return (true);
	},
	initCounter: function () {
		this.counter = Math.random();
	},
	handleOnEnter: function () {
		console.log ("enter ChartWidget.handleOnEnter");
		this.allowSubmit_ = false;
	},
	processEvent: function (event) {
		try {
			console.log ("enter processEvent");
			console.log (event);
			var receiveContextRadio = dijit.byId(this.namespace + "contextualParametersSelected");
		
			if (event.name == "http://ibm.com/isclite#portletResize") {
				if (this.shouldResize()) {
					var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
					var myportletsize = this.getChartSize();
					var newWidth = myportletsize.available.w;
					var newHeight = myportletsize.available.h;

					if (chartNavigator.getCurrentPage() == chartNavigator.CHART_PAGE) {
						this.resizeChart (newWidth, newHeight);
					} else if (chartNavigator.getCurrentPage() == chartNavigator.CONNECTION_MGR_PAGE) {
						chartNavigator.resize (newWidth, newHeight);
					} else if (chartNavigator.getCurrentPage() == chartNavigator.WS_CHART_SELECTOR_PAGE) {
						var wsChartSelector = dijit.byId (chartNavigator.id + "chartSelectorDiv");
						wsChartSelector.resize();
					} else if (chartNavigator.getCurrentPage() == chartNavigator.TABLE_PAGE) {
						this.resizeTable();
					} else {
						console.log ("current page is not resizable... so not invoking resize");
					}
				}
			} else if (event.name == "http://ibm.com/tip#NodeClickedOn") {
				if (receiveContextRadio && receiveContextRadio.checked) {
					var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());		
		//			alert ("matchingMSSIndex: " + matchingMSSIndex)
					if (matchingMSSIndex < 0) {
						// no matching MSS found, so call to transform
						event = this.transformEvent (event);
					} else {
						// matching MSS found, so no need to transform, 
						// just invoke post transform processing
						this.onTransformEnd (event);
					}
				} else {
					console.log ("receive context is off");
				}
			} else if (event.name == "http://ibm.com/isclite#launchPortlet") {
				console.log ("handling launchPortlet event");
			    if (event.connectionType && event.chartID) {
					if (event.connectionName) {
					    var reportSelectorDiv = dojo.byId(this.namespace + "reportSelectorDiv");
						reportSelectorDiv.style.display = "none";
				    
						var selector = new ibm.tivoli.tip.charts.dijit.ChartSelectorWidget ({datasource_: event.connectionName, populateGroups: false, namespace_: this.namespace });
						selector.getChartDefinition (event.groupID, event.chartID);
					} else {
						console.log ("handleEvent: run report");
						this.onReportSelection (event.connectionType, null, event.groupID, event.chartID, null, null, true);
					}
				}
			} else {
				console.log ("received unknown event: ");
				console.log (event);
			}
		} catch (e) {
			console.error ("caught exception in processEvent");
			console.error (e);
		}
	},
	shouldResize: function () {
		var portletSize = iscPortlet.getPortletSize(this.iscPortletNamespace);
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");

		if (this.lastPortletSize_ &&
		    this.lastPage_ &&
		    (portletSize.available.w == this.lastPortletSize_.available.w) &&
		    (portletSize.available.h == this.lastPortletSize_.available.h) &&
		    (this.lastPage_ == chartNavigator.getCurrentPage())) {
			console.log ("shouldResize. return false.");
			return (false);
		} else {
			this.lastPage_ = chartNavigator.getCurrentPage();
			this.lastPortletSize_ = portletSize;
			console.log ("shouldResize. return true.");
			return (true);
		}
	},
	getMatchingMSSIndex : function (event, srcMssName) {
		console.log ("src MSS: " + srcMssName);
		var index = -1;
		
		if (this.equalsIgnoreCase (event.MSSName, srcMssName)) {
			index = 0;
		} else {
			var i = 1;
			console.log ("entering for loop");
			forloop:
			for (i=1; ; i++) {
				console.log ("in for loop with index: " + i);
				var value = event["MSSName" + i];
				console.log ("event.MSSName" + i + ": " + value);
				if (value) {
					if (this.equalsIgnoreCase (value, srcMssName)) {
						index = i;
						break forloop;
					}
				} else {
					break forloop;
				}
			}
		}
		
		console.log ("returning matching mss index: " + index);
		return (index);
	},
	equalsIgnoreCase: function (str1, str2) {
		if ((str1 === null) && (str2 === null)) {
			return (true);
		} else if ((str1 === null) || (str2 === null)) {
			return (false);
		} else {
			return (str1.toLowerCase() == str2.toLowerCase());
		}
	},
	getMatchingMSSIndexOld : function (event, srcMssName) {
		console.log ("src MSS: " + srcMssName);
		var index = -1;
	
		forloop:
		for (var i=1; ; i++) {
			var value = eval ("event.R" + i + "_mssName");
			console.log ("event.R" + i + "_mssName: " + value);
			if (value) {
				if (value == srcMssName) {
					index = i;
					break forloop;
				}
			} else {
				break forloop;
			}
		}
		
		console.log ("returning matching mss index: " + index);
		return (index);
	},
	
	transformEvent: function (event) {
		console.log ("enter transformEvent with event:");
		console.log (event);
		var transformInfo = {
                              "uniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "locationUniqueName": "com.ibm.tivoli.tip.TIPTransformation",
                              "eventObj": event,
                              "srcPortletNamespace": this.iscPortletNamespace,
                              "handler": dojo.hitch (this, this.onTransformEnd)
                            };
        EventBroker.executeTransform(transformInfo);
	},
	onTransformEnd: function (event) {
		console.log ("enter onTransformEnd with event: ");
		console.log (event);
		var matchingMSSIndex = this.getMatchingMSSIndex (event, this.getSourceMSSName());
		if (matchingMSSIndex > 0) {
			console.log("MSSName" + event["MSSName" + matchingMSSIndex]);
			console.log ("SourceToken" + event["SourceToken" + matchingMSSIndex]);

			event.MSSName = event["MSSName" + matchingMSSIndex];
			event.SourceToken = event["SourceToken" + matchingMSSIndex];
			console.log ("event: ");
			console.log (event);
		}

		this.paramWidget_.setAllContext (event);
		// updateProperties will also run a report to regenerate the graph (but only if the properties changed).
		this.updateProperties(false);
	},
	getMenuItems: function () {
		var menuItems = [
			{ id: this.namespace + "RefreshChartMenuItem", label: this.resources_.REFRESH, iconClass: "refreshIcon", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').refreshChart();", enabled: true },
			{ id: this.namespace + "ReloadChartMenuItem", label: this.resources_.RELOAD, iconClass: "refreshIcon", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').reloadChart();", enabled: true },
			{ id: "separator" },
			{ id: this.namespace + "PreferencesMenuItem", label: this.resources_.PREFERENCES, iconClass: "propertiesIcon", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').showPrefs();", enabled: true },
			{ id: this.namespace + "ToggleTableChartMenuItem", label: this.resources_.TOGGLE_TABLE_CHART, iconClass: "toggleIcon", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').toggleTableChart();", enabled: true },
			{ id: this.namespace + "ReturnToChartSelectorMenuItem", label: this.resources_.RETURN_TO_CHART_SELECTOR, iconClass: "backIcon", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').showChartSelector();", enabled: true },
			{ id: "separator" },
			{ id: this.namespace + "ShowHideToolbarMenuItem", label: this.resources_.SHOW_HIDE_TOOLBAR, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').toggleToolbar();", enabled: true },
			{ id: "separator" },
			{ id: this.namespace + "PrintPreviewMenuItem", label: this.resources_.PRINT_PREVIEW, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').printPreviewChart();", enabled: true },
			{ id: this.namespace + "SaveAsFileMenuItem", label: this.resources_.SAVE_AS_FILE, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').saveChartAsFile();", enabled: true },
			{ id: "separator" }
		];
		
		var context = ibm.tivoli.tip.dojo.Global.getSingleton(this.namespace);
		if (context.roles.admin || context.roles.creator) {
			menuItems.push ({ id: this.namespace + "SaveForAll", label: this.resources_.SAVE_FOR_OTHERS, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').onSave();", enabled: true });
			menuItems.push ({ id: this.namespace + "ResetForAll", label: this.resources_.CLEAR_DEFAULT_PREFERENCES, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').onReset();", enabled: true });
		}
		
		menuItems.push ({ id: this.namespace + "ResetForMe", label: this.resources_.CLEAR_MY_PREFERENCES, icon: "", launchType: "JavaScript", uri: "dijit.byId ('" + this.namespace + "chartWidget').onResetMyPreferences();", enabled: true });
		
		return (menuItems);
	},
	getSelectedData: function () {
	},
	makeChartContextMenu: function () {
		var provider = new ibm.tivoli.tip.dijit.TIPContextMenuProviderV2(
					{ iscPortletNamespace: this.iscPortletNamespace,
					  getMenuItemsCallback: dojo.hitch (this, this.getMenuItems)
					});
		console.log ("after make provider");

		var chartMenu = new ibm.tivoli.tip.dijit.TIPContextMenuProxyV2(
			{cmProvider: provider, 
			 targetNodeIds: [this.id + "chartImageDiv"],
			 getContextCallback: dojo.hitch (this, this.getSelectedData),
			 id: this.namespace + "chartMenu"
			});
		console.log ("after create context menu");
//		chartMenu.startup();
	},
	resizeChart: function (newWidth, newHeight) {
		console.log ("enter resizeChart with width: " + newWidth + ", and height: " + newHeight);
		this.actualImgHeight_ = newHeight;
		this.actualImgWidth_ = newWidth;
		
		var heightField = dojo.byId(this.namespace + "height");
		var widthField = dojo.byId (this.namespace + "width");
		
		heightField.value = newHeight;
		widthField.value = newWidth;
		
		var context = {};
		context.command = "modifyReport";
		context.arg_portletNamespace = this.namespace;
		context.arg_reportName = encodeURI(this.reportName);
		context.arg_chartSrcType = this.chartSrcType_;
		context.arg_connectionName = encodeURI(this.connectionName_);
		context.arg_groupID = encodeURI(this.groupID_);
		context.arg_dataSource = encodeURI(this.connectionName_);
		context.arg_chartID = encodeURI(this.reportName);
		context.arg_persist = false;
		context.shouldRunReport = "true";
		
		context.arg_height = newHeight;
		context.arg_width = newWidth;

		var kw = {
             content: context,
           	 onResponse:{
                      callback: dojo.hitch (this, this.runReportCallback)
   	         }
   	     };

		this.makeRequestPrimitive (kw);
	},
	makeRequestPrimitive: function (requestContext) {
		console.log ("enter ChartWidget.makeRequestPrimitive with context: ");
		console.log (requestContext);
		
		if (requestContext.content.command == "modifyReport") {
			// check to see if a modifyReport request is already in process.  
			// If so, we need to queue this one or ignore it
			console.log ("ChartWidget.makeRequestPrimitive: queue length: " + this.requestQueue.length);
		    // start the count for run report response time here
		    this.runReportStartTime = new Date().getTime();
			if (this.requestQueue.length > 0) {
				console.log ("ChartWidget.makeRequestPrimitive: adding request to queue for future processing");
				this.requestQueue.push (requestContext);
			} else {
				console.log ("ChartWidget.makeRequestPrimitive: processing request now");
				this.requestQueue.push (requestContext);
				ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( requestContext );
			}
		} else {
			console.log ("ChartWidget.makeRequestPrimitive: not a request we need to queue.");
	        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( requestContext );
	    }
	},
	resizeTable: function () {
		var portletSize = iscPortlet.getPortletSize(this.iscPortletNamespace);
		var size = { w: portletSize.available.w, h: portletSize.available.h };
		dijit.byId (this.namespace + "tbl_tipChartData").resize (size);
	},
	getChartSize: function () {
		var size = iscPortlet.getPortletSize(this.iscPortletNamespace);
		console.log ("got portlet size, available.w: " + size.available.w + ", h: " + size.available.h);
		// temporary hack for IE
		if ((size.available.w === undefined) ||
			isNaN(size.available.w)) {
			// guess
			size.available.w = 450;
		}
		
		var coords = null;
		
		var node = dojo.byId (this.namespace + "toolbarDiv");
		if (node) {
			// subtract size of toolbar 
			coords = dojo.coords (node);
			size.available.h = size.available.h - coords.h;
			console.log ("removing toolbar size: " + coords.h);
		} 
		
		node = dojo.byId (this.id + "nub");
		if (node) {
			// subtract size of toolbar nub
			coords = dojo.coords (node);
			size.available.h = size.available.h - coords.h;
		} else {
			// nub is 8 pixels
			size.available.h = size.available.h - 8;
			console.log ("removing fixed nub size");
		}
		
		node = this.metaDiv;
		var showLastUpdated = dijit.byId (this.namespace + "showLastUpdated");
		if (node && showLastUpdated && showLastUpdated.checked) {
			// subtract size of date 
			coords = dojo.coords (node);
			if (coords.h < 16) {
				coords.h = 16;
			}
			size.available.h = size.available.h - coords.h;
			console.log ("removed meta data size: " + coords.h);
		} else if (this.firstRender) {
			console.log ("firstRender and showLastUpdated is false.");
			// 1st render so those are not available yet, so use fixed size
			size.available.h = size.available.h - 16;
			this.firstRender = false;
		}
		
		// allow for extra space b/c we still get scroll bars on fixed size pages
		if (dojo.isIE) {
			size.available.h = size.available.h - 30;
			size.available.w = size.available.w - 18;
		} else {
			size.available.h = size.available.h - 12;
			size.available.w = size.available.w - 5;
		}
			
		// minimum height
		if (size.available.h <= 25) {
			size.available.h = 25;
		}
		
		console.log ("return chart size (after subtracting other UI elements): ");
		console.log (size);
		
		return (size);
	},
	onError: function (msg, request) {
		console.log ("enter ChartWidget.onError with msg: " + msg);
		console.log ("request was: ");
		console.log (request);
		// Show the chart div so we get the toolbar
		var chartNavigator = dijit.byId (this.namespace + "mainChartNavigatorDiv");
		console.log ("chartNav visible: " + chartNavigator.isNavigatorVisible());
		console.log ("chartDiv visiable: " + this.chartDiv.style.display);
		console.log ("tableDiv visiable: " + this.tableDiv.style.display);
		var cleanup = true;
		
		if (request.content.command == "modifyReport") {
			// pop off the most recent request which was just run
			this.requestQueue.shift();
			this.reportSelectorDiv.style.display = "none";
			this.chartDiv.style.display = "";
			this.openChartToolbar();
			cleanup = false;
		} 
		
		if ((!chartNavigator.isNavigatorVisible()) &&
			(this.chartDiv.style.display == "none") &&
			(this.tableDiv.style.display == "none")) {
			this.chartDiv.style.display = "";
			this.openChartToolbar();
		} 
			
		if (cleanup) {
			// Clean up after error to avoid any div errors on subsequent requests
			this.cleanupForChartChange();
		}
	},
	destroy: function () {
		console.log ("enter ChartWidget.destroy");

		var paramDiv = dijit.byId (this.namespace + "parametersDiv");
		if (paramDiv) {
			paramDiv.destroy();
		}
	
		dojo.forEach (this.connections, function (item) {
			dojo.disconnect (item);
		});
		dojo.forEach (this.topics_, function (topic) {
			dojo.unsubscribe (topic);
		});
		// remove tooltips
		dojo.forEach (this.tooltips, function (tt) {
			tt.destroy();
		});
		
		if (this.toggleSubscription) {
			dojo.unsubscribe (this.toggleSubscription);
			this.toggleSubscription = null;
		}
		
		// remove auto-refresh
		if (this.interval !== null) {
			clearTimeout (this.interval);
			delete this.interval;
		}

//		this.destroyDescendants();
		this.inherited ("destroy", arguments);

		console.log ("exit ChartWidget.destroy");
	}
});

}


dojo.i18n._preloadLocalizations("dojo.nls.tipchart", ["es-es", "es", "hu", "it-it", "de", "pt-br", "pl", "fr-fr", "zh-cn", "pt", "en-us", "zh", "ru", "xx", "fr", "zh-tw", "it", "cs", "en-gb", "de-de", "ja-jp", "ko-kr", "ko", "en", "ROOT", "ja"]);
