/*
	Copyright (c) 2004-2007, The Dojo Foundation
	All Rights Reserved.

	Licensed under the Academic Free License version 2.1 or above OR the
	modified BSD license. For more information on Dojo licensing, see:

		http://dojotoolkit.org/book/dojo-book-0-9/introduction/licensing
*/

/*
	This is a compiled version of Dojo, built for deployment and not for
	development. To get an editable version, please visit:

		http://dojotoolkit.org

	for documentation and information on getting the source.
*/

if(!dojo._hasResource["dojox.gfx._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx._base"] = true;
dojo.provide("dojox.gfx._base");

// candidates for dojox.style (work on VML and SVG nodes)
dojox.gfx._hasClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary:
	//		Returns whether or not the specified classes are a portion of the
	//		class list currently applied to the node. 
	// return (new RegExp('(^|\\s+)'+classStr+'(\\s+|$)')).test(node.className)	// Boolean
	return ((" "+node.getAttribute("className")+" ").indexOf(" "+classStr+" ") >= 0);  // Boolean
}

dojox.gfx._addClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary:
	//		Adds the specified classes to the end of the class list on the
	//		passed node.
	var cls = node.getAttribute("className");
	if((" "+cls+" ").indexOf(" "+classStr+" ") < 0){
		node.setAttribute("className", cls + (cls ? ' ' : '') + classStr);
	}
}

dojox.gfx._removeClass = function(/*DomNode*/node, /*String*/classStr){
	//	summary: Removes classes from node.
	node.setAttribute("className", node.getAttribute("className").replace(new RegExp('(^|\\s+)'+classStr+'(\\s+|$)'), "$1$2"));
}


// candidate for dojox.html.metrics (dynamic font resize handler is not implemented here)

//	derived from Morris John's emResized measurer
dojox.gfx._base._getFontMeasurements = function(){
	//	summary
	//	Returns an object that has pixel equivilents of standard font size values.
	var heights = {
		'1em':0, '1ex':0, '100%':0, '12pt':0, '16px':0, 'xx-small':0, 'x-small':0,
		'small':0, 'medium':0, 'large':0, 'x-large':0, 'xx-large':0
	};

	if(dojo.isIE){
		//	we do a font-size fix if and only if one isn't applied already.
		//	NOTE: If someone set the fontSize on the HTML Element, this will kill it.
		dojo.doc.documentElement.style.fontSize="100%";
	}

	//	set up the measuring node.
	var div=dojo.doc.createElement("div");
	div.style.position="absolute";
	div.style.left="-100px";
	div.style.top="0";
	div.style.width="30px";
	div.style.height="1000em";
	div.style.border="0";
	div.style.margin="0";
	div.style.padding="0";
	div.style.outline="0";
	div.style.lineHeight="1";
	div.style.overflow="hidden";
	dojo.body().appendChild(div);

	//	do the measurements.
	for(var p in heights){
		div.style.fontSize = p;
		heights[p] = Math.round(div.offsetHeight * 12/16) * 16/12 / 1000;
	}
	
	dojo.body().removeChild(div);
	div = null;
	return heights; 	//	object
};

dojox.gfx._base._fontMeasurements = null;

dojox.gfx._base._getCachedFontMeasurements = function(recalculate){
	if(recalculate || !dojox.gfx._base._fontMeasurements){
		dojox.gfx._base._fontMeasurements = dojox.gfx._base._getFontMeasurements();
	}
	return dojox.gfx._base._fontMeasurements;
};

// candidate for dojo.dom

dojox.gfx._base._uniqueId = 0;
dojox.gfx._base._getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = "dojoUnique" + (++dojox.gfx._base._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.mixin(dojox.gfx, {
	// summary: defines constants, prototypes, and utility functions
	
	// default shapes, which are used to fill in missing parameters
	defaultPath:     {type: "path",     path: ""},
	defaultPolyline: {type: "polyline", points: []},
	defaultRect:     {type: "rect",     x: 0, y: 0, width: 100, height: 100, r: 0},
	defaultEllipse:  {type: "ellipse",  cx: 0, cy: 0, rx: 200, ry: 100},
	defaultCircle:   {type: "circle",   cx: 0, cy: 0, r: 100},
	defaultLine:     {type: "line",     x1: 0, y1: 0, x2: 100, y2: 100},
	defaultImage:    {type: "image",    x: 0, y: 0, width: 0, height: 0, src: ""},
	defaultText:     {type: "text",     x: 0, y: 0, text: "",
		align: "start", decoration: "none", rotated: false, kerning: true },
	defaultTextPath: {type: "textpath", text: "",
		align: "start", decoration: "none", rotated: false, kerning: true },

	// default geometric attributes
	defaultStroke: {type: "stroke", color: "black", style: "solid", width: 1, cap: "butt", join: 4},
	defaultLinearGradient: {type: "linear", x1: 0, y1: 0, x2: 100, y2: 100, 
		colors: [{offset: 0, color: "black"}, {offset: 1, color: "white"}]},
	defaultRadialGradient: {type: "radial", cx: 0, cy: 0, r: 100, 
		colors: [{offset: 0, color: "black"}, {offset: 1, color: "white"}]},
	defaultPattern: {type: "pattern", x: 0, y: 0, width: 0, height: 0, src: ""},
	defaultFont: {type: "font", style: "normal", variant: "normal", weight: "normal", 
		size: "10pt", family: "serif"},

	normalizeColor: function(/*Color*/ color){
		// summary: converts any legal color representation to normalized dojo.Color object
		return (color instanceof dojo.Color) ? color : new dojo.Color(color); // dojo.Color
	},
	normalizeParameters: function(existed, update){
		// summary: updates an existing object with properties from an "update" object
		// existed: Object: the "target" object to be updated
		// update:  Object: the "update" object, whose properties will be used to update the existed object
		if(update){
			var empty = {};
			for(var x in existed){
				if(x in update && !(x in empty)){
					existed[x] = update[x];
				}
			}
		}
		return existed;	// Object
	},
	makeParameters: function(defaults, update){
		// summary: copies the original object, and all copied properties from the "update" object
		// defaults: Object: the object to be cloned before updating
		// update:   Object: the object, which properties are to be cloned during updating
		if(!update) return dojo.clone(defaults);
		var result = {};
		for(var i in defaults){
			if(!(i in result)){
				result[i] = dojo.clone((i in update) ? update[i] : defaults[i]);
			}
		}
		return result; // Object
	},
	formatNumber: function(x, addSpace){
		// summary: converts a number to a string using a fixed notation
		// x:			Number:		number to be converted
		// addSpace:	Boolean?:	if it is true, add a space before a positive number
		var val = x.toString();
		if(val.indexOf("e") >= 0){
			val = x.toFixed(4);
		}else{
			var point = val.indexOf(".");
			if(point >= 0 && val.length - point > 5){
				val = x.toFixed(4);
			}
		}
		if(x < 0){
			return val; // String
		}
		return addSpace ? " " + val : val; // String
	},
	// font operations
	makeFontString: function(font){
		// summary: converts a font object to a CSS font string
		// font:	Object:	font object (see dojox.gfx.defaultFont)
		return font.style + " " + font.variant + " " + font.weight + " " + font.size + " " + font.family; // Object
	},
	splitFontString: function(str){
		// summary: converts a CSS font string to a font object
		// str:		String:	a CSS font string
		var font = dojo.clone(dojox.gfx.defaultFont);
		var t = str.split(/\s+/);
		do{
			if(t.length < 5){ break; }
			font.style  = t[0];
			font.varian = t[1];
			font.weight = t[2];
			var i = t[3].indexOf("/");
			font.size = i < 0 ? t[3] : t[3].substring(0, i);
			var j = 4;
			if(i < 0){
				if(t[4] == "/"){
					j = 6;
					break;
				}
				if(t[4].substr(0, 1) == "/"){
					j = 5;
					break;
				}
			}
			if(j + 3 > t.length){ break; }
			font.size = t[j];
			font.family = t[j + 1];
		}while(false);
		return font;	// Object
	},
	// length operations
	cm_in_pt: 72 / 2.54,	// Number: centimeters per inch
	mm_in_pt: 7.2 / 2.54,	// Number: millimeters per inch
	px_in_pt: function(){
		// summary: returns a number of pixels per point
		return dojox.gfx._base._getCachedFontMeasurements()["12pt"] / 12;	// Number
	},
	pt2px: function(len){
		// summary: converts points to pixels
		// len: Number: a value in points
		return len * dojox.gfx.px_in_pt();	// Number
	},
	px2pt: function(len){
		// summary: converts pixels to points
		// len: Number: a value in pixels
		return len / dojox.gfx.px_in_pt();	// Number
	},
	normalizedLength: function(len) {
		// summary: converts any length value to pixels
		// len: String: a length, e.g., "12pc"
		if(len.length == 0) return 0;
		if(len.length > 2){
			var px_in_pt = dojox.gfx.px_in_pt();
			var val = parseFloat(len);
			switch(len.slice(-2)){
				case "px": return val;
				case "pt": return val * px_in_pt;
				case "in": return val * 72 * px_in_pt;
				case "pc": return val * 12 * px_in_pt;
				case "mm": return val / dojox.gfx.mm_in_pt * px_in_pt;
				case "cm": return val / dojox.gfx.cm_in_pt * px_in_pt;
			}
		}
		return parseFloat(len);	// Number
	},
	
	// a constant used to split a SVG/VML path into primitive components
	pathVmlRegExp: /([A-Za-z]+)|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	pathSvgRegExp: /([A-Za-z])|(\d+(\.\d+)?)|(\.\d+)|(-\d+(\.\d+)?)|(-\.\d+)/g,
	
	equalSources: function(a, b){
		// summary: compares event sources, returns true if they are equal
		return a && b && a == b;
	}
});

}

if(!dojo._hasResource["dojox.gfx.shape"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.shape"] = true;
dojo.provide("dojox.gfx.shape");



dojo.declare("dojox.gfx.Shape", null, {
	// summary: a Shape object, which knows how to apply 
	// graphical attributes and transformations
	
	constructor: function(){
		// rawNode: Node: underlying node
		this.rawNode = null;
		
		// shape: Object: an abstract shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		this.shape = null;
		
		// matrix: dojox.gfx.Matrix2D: a transformation matrix
		this.matrix = null;
		
		// fillStyle: Object: a fill object 
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)
		this.fillStyle = null;
		
		// strokeStyle: Object: a stroke object 
		//	(see dojox.gfx.defaultStroke) 
		this.strokeStyle = null;
		
		// bbox: dojox.gfx.Rectangle: a bounding box of this shape
		//	(see dojox.gfx.defaultRect)
		this.bbox = null;
		
		// virtual group structure
		
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		this.parent = null;
		
		// parentMatrix: dojox.gfx.Matrix2D
		//	a transformation matrix inherited from the parent
		this.parentMatrix = null;
	},
	
	// trivial getters
	
	getNode: function(){
		// summary: returns the current DOM Node or null
		return this.rawNode; // Node
	},
	getShape: function(){
		// summary: returns the current shape object or null
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		return this.shape; // Object
	},
	getTransform: function(){
		// summary: returns the current transformation matrix or null
		return this.matrix;	// dojox.gfx.Matrix2D
	},
	getFill: function(){
		// summary: returns the current fill object or null
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)
		return this.fillStyle;	// Object
	},
	getStroke: function(){
		// summary: returns the current stroke object or null
		//	(see dojox.gfx.defaultStroke) 
		return this.strokeStyle;	// Object
	},
	getParent: function(){
		// summary: returns the parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		return this.parent;	// Object
	},
	getBoundingBox: function(){
		// summary: returns the bounding box or null
		//	(see dojox.gfx.defaultRect)
		return this.bbox;	// dojox.gfx.Rectangle
	},
	getTransformedBoundingBox: function(){
		// summary: returns an array of four points or null
		//	four points represent four corners of the untransformed bounding box
		var b = this.getBoundingBox();
		if(!b){
			return null;	// null
		}
		var m = this._getRealMatrix();
		var r = [];
		var g = dojox.gfx.matrix;
		r.push(g.multiplyPoint(m, b.x, b.y));
		r.push(g.multiplyPoint(m, b.x + b.width, b.y));
		r.push(g.multiplyPoint(m, b.x + b.width, b.y + b.height));
		r.push(g.multiplyPoint(m, b.x, b.y + b.height));
		return r;	// Array
	},
	getEventSource: function(){
		// summary: returns a Node, which is used as 
		//	a source of events for this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this.rawNode;	// Node
	},
	
	// empty settings
	
	setShape: function(shape){
		// summary: sets a shape object
		//	(the default implementation simply ignores it)
		// shape: Object: a shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.shape = dojox.gfx.makeParameters(this.shape, shape);
		this.bbox = null;
		return this;	// self
	},
	setFill: function(fill){
		// summary: sets a fill object
		//	(the default implementation simply ignores it)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!fill){
			// don't fill
			this.fillStyle = null;
			return this;	// self
		}
		var f = null;
		if(typeof(fill) == "object" && "type" in fill){
			// gradient or pattern
			switch(fill.type){
				case "linear":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					break;
				case "radial":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					break;
				case "pattern":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					break;
			}
		}else{
			// color object
			f = dojox.gfx.normalizeColor(fill);
		}
		this.fillStyle = f;
		return this;	// self
	},
	setStroke: function(stroke){
		// summary: sets a stroke object
		//	(the default implementation simply ignores it)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke) 

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			return this;	// self
		}
		// normalize the stroke
		if(typeof stroke == "string"){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		return this;	// self
	},
	setTransform: function(matrix){
		// summary: sets a transformation matrix
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		this.matrix = dojox.gfx.matrix.clone(matrix ? dojox.gfx.matrix.normalize(matrix) : dojox.gfx.matrix.identity);
		return this._applyTransform();	// self
	},
	
	_applyTransform: function(){
		// summary: physically sets a matrix

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return this;	// self
	},
	
	// z-index
	
	moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToFront(this);
			this._moveToFront();	// execute renderer-specific action
		}
		return this;	// self
	},
	moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes
		var p = this.getParent();
		if(p){
			p._moveChildToBack(this);
			this._moveToBack();	// execute renderer-specific action
		}
		return this;
	},
	_moveToFront: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},
	_moveToBack: function(){
		// summary: renderer-specific hook, see dojox.gfx.shape.Shape.moveToFront()

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!
	},

	// apply left & right transformation
	
	applyRightTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on right side
		//	(this.matrix * matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	applyLeftTransform: function(matrix){
		// summary: multiplies the existing matrix with an argument on left side
		//	(matrix * this.matrix)
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([matrix, this.matrix]) : this;	// self
	},
	applyTransform: function(matrix){
		// summary: a shortcut for dojox.gfx.Shape.applyRightTransform
		// matrix: dojox.gfx.Matrix2D: a matrix or a matrix-like object
		//	(see an argument of dojox.gfx.Matrix2D 
		//	constructor for a list of acceptable arguments)
		return matrix ? this.setTransform([this.matrix, matrix]) : this;	// self
	},
	
	// virtual group methods
	
	removeShape: function(silently){
		// summary: removes the shape from its parent's list of shapes
		// silently: Boolean?: if true, do not redraw a picture yet
		if(this.parent){
			this.parent.remove(this, silently);
		}
		return this;	// self
	},
	_setParent: function(parent, matrix){
		// summary: sets a parent
		// parent: Object: a parent or null
		//	(see dojox.gfx.Surface,
		//	dojox.gfx.shape.VirtualGroup,
		//	or dojox.gfx.Group)
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parent = parent;
		return this._updateParentMatrix(matrix);	// self
	},
	_updateParentMatrix: function(matrix){
		// summary: updates the parent matrix with new matrix
		// matrix: dojox.gfx.Matrix2D:
		//	a 2D matrix or a matrix-like object
		this.parentMatrix = matrix ? dojox.gfx.matrix.clone(matrix) : null;
		return this._applyTransform();	// self
	},
	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix
		var m = this.matrix;
		var p = this.parent;
		while(p){
			if(p.matrix){
				m = dojox.gfx.matrix.multiply(p.matrix, m);
			}
			p = p.parent;
		}
		return m;	// dojox.gfx.Matrix2D
	}
});

dojox.gfx.shape._eventsProcessing = {
	connect: function(name, object, method){
		// summary: connects a handler to an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		return arguments.length > 2 ?	// Object
			dojo.connect(this.getEventSource(), name, object, method) :
			dojo.connect(this.getEventSource(), name, object);
	},
	disconnect: function(token){
		// summary: connects a handler by token from an event on this shape

		// COULD BE RE-IMPLEMENTED BY THE RENDERER!

		dojo.disconnect(token);
	}
};

dojo.extend(dojox.gfx.Shape, dojox.gfx.shape._eventsProcessing);

dojox.gfx.shape.Container = {
	// summary: a container of shapes, which can be used 
	//	as a foundation for renderer-specific groups, or as a way 
	//	to logically group shapes (e.g, to propagate matricies)
	
	_init: function() {
		// children: Array: a list of children
		this.children = [];
	},
	
	// group management
	
	add: function(shape){
		// summary: adds a shape to the list
		// shape: dojox.gfx.Shape: a shape
		var oldParent = shape.getParent();
		if(oldParent){
			oldParent.remove(shape, true);
		}
		this.children.push(shape);
		return shape._setParent(this, this._getRealMatrix());	// self
	},
	remove: function(shape, silently){
		// summary: removes a shape from the list
		// silently: Boolean?: if true, do not redraw a picture yet
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				if(silently){
					// skip for now
				}else{
					shape._setParent(null, null);
				}
				this.children.splice(i, 1);
				break;
			}
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		this.children = [];
		return this;	// self
	},
	
	// moving child nodes
	
	_moveChildToFront: function(shape){
		// summary: moves a shape to front of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.push(shape);
				break;
			}
		}
		return this;	// self
	},
	_moveChildToBack: function(shape){
		// summary: moves a shape to back of the list of shapes
		for(var i = 0; i < this.children.length; ++i){
			if(this.children[i] == shape){
				this.children.splice(i, 1);
				this.children.unshift(shape);
				break;
			}
		}
		return this;	// self
	}
};

dojo.declare("dojox.gfx.shape.Surface", null, {
	// summary: a surface object to be used for drawings
	constructor: function(){
		// underlying node
		this.rawNode = null;
	},
	getEventSource: function(){
		// summary: returns a node, which can be used to attach event listeners
		return this.rawNode; // Node
	},
	_getRealMatrix: function(){
		// summary: always returns the identity matrix
		return null;	// dojox.gfx.Matrix2D
	}
});

dojo.extend(dojox.gfx.shape.Surface, dojox.gfx.shape._eventsProcessing);

dojo.declare("dojox.gfx.Point", null, {
	// summary: a hypothetical 2D point to be used for drawings - {x, y}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2}.
});

dojo.declare("dojox.gfx.Rectangle", null, {
	// summary: a hypothetical rectangle - {x, y, width, height}
	// description: This object is defined for documentation purposes.
	//	You should use the naked object instead: {x: 1, y: 2, width: 100, height: 200}.
});

dojo.declare("dojox.gfx.shape.Rect", dojox.gfx.Shape, {
	// summary: a generic rectangle
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultRect);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Ellipse", dojox.gfx.Shape, {
	// summary: a generic ellipse
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultEllipse);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.rx, y: shape.cy - shape.ry, 
				width: 2 * shape.rx, height: 2 * shape.ry};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Circle", dojox.gfx.Shape, {
	// summary: a generic circle
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultCircle);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {x: shape.cx - shape.r, y: shape.cy - shape.r, 
				width: 2 * shape.r, height: 2 * shape.r};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Line", dojox.gfx.Shape, {
	// summary: a generic line
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultLine);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox){
			var shape = this.shape;
			this.bbox = {
				x:		Math.min(shape.x1, shape.x2),
				y:		Math.min(shape.y1, shape.y2),
				width:	Math.abs(shape.x2 - shape.x1),
				height:	Math.abs(shape.y2 - shape.y1)
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Polyline", dojox.gfx.Shape, {
	// summary: a generic polyline/polygon
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultPolyline);
		this.rawNode = rawNode;
	},
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object
		// points: Object: a polyline/polygon shape object
		// closed: Boolean: close the polyline to make a polygon
		if(points && points instanceof Array){
			// points: Array: an array of points
			dojox.gfx.Shape.prototype.setShape.call(this, {points: points});
			if(closed && this.shape.points.length){ 
				this.shape.points.push(this.shape.points[0]);
			}
		}else{
			dojox.gfx.Shape.prototype.setShape.call(this, points);
		}
		return this;	// self
	},
	getBoundingBox: function(){
		// summary: returns the bounding box
		if(!this.bbox && this.shape.points.length){
			var p = this.shape.points;
			var l = p.length;
			var t = p[0];
			var bbox = {l: t.x, t: t.y, r: t.x, b: t.y};
			for(var i = 1; i < l; ++i){
				t = p[i];
				if(bbox.l > t.x) bbox.l = t.x;
				if(bbox.r < t.x) bbox.r = t.x;
				if(bbox.t > t.y) bbox.t = t.y;
				if(bbox.b < t.y) bbox.b = t.y;
			}
			this.bbox = {
				x:		bbox.l, 
				y:		bbox.t, 
				width:	bbox.r - bbox.l, 
				height:	bbox.b - bbox.t
			};
		}
		return this.bbox;	// dojox.gfx.Rectangle
	}
});

dojo.declare("dojox.gfx.shape.Image", dojox.gfx.Shape, {
	// summary: a generic image
	//	(this is a helper object, which is defined for convenience)
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.shape = dojo.clone(dojox.gfx.defaultImage);
		this.rawNode = rawNode;
	},
	getBoundingBox: function(){
		// summary: returns the bounding box (its shape in this case)
		return this.shape;	// dojox.gfx.Rectangle
	},
	setStroke: function(){
		// summary: ignore setting a stroke style
		return this;	// self
	},
	setFill: function(){
		// summary: ignore setting a fill style
		return this;	// self
	}
});

dojo.declare("dojox.gfx.shape.Text", dojox.gfx.Shape, {
	// summary: a generic text
	constructor: function(rawNode) {
		// rawNode: Node: a DOM Node
		this.fontStyle = null;
		this.shape = dojo.clone(dojox.gfx.defaultText);
		this.rawNode = rawNode;
	},
	setFont: function(newFont){
		// summary: sets a font for text
		// newFont: Object: a font object (see dojox.gfx.defaultFont) or a font string
		this.fontStyle = typeof newFont == "string" ? dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

dojox.gfx.shape.Creator = {
	// summary: shape creators
	createShape: function(shape){
		// summary: creates a shape object based on its type; it is meant to be used
		//	by group-like objects
		// shape: Object: a shape descriptor object
		switch(shape.type){
			case dojox.gfx.defaultPath.type:		return this.createPath(shape);
			case dojox.gfx.defaultRect.type:		return this.createRect(shape);
			case dojox.gfx.defaultCircle.type:		return this.createCircle(shape);
			case dojox.gfx.defaultEllipse.type:		return this.createEllipse(shape);
			case dojox.gfx.defaultLine.type:		return this.createLine(shape);
			case dojox.gfx.defaultPolyline.type:	return this.createPolyline(shape);
			case dojox.gfx.defaultImage.type:		return this.createImage(shape);
			case dojox.gfx.defaultText.type:		return this.createText(shape);
			case dojox.gfx.defaultTextPath.type:	return this.createTextPath(shape);
		}
		return null;
	},
	createGroup: function(){
		// summary: creates an SVG group shape
		return this.createObject(dojox.gfx.Group);	// dojox.gfx.Group
	},
	createRect: function(rect){
		// summary: creates an SVG rectangle shape
		// rect: Object: a path object (see dojox.gfx.defaultRect)
		return this.createObject(dojox.gfx.Rect, rect);	// dojox.gfx.Rect
	},
	createEllipse: function(ellipse){
		// summary: creates an SVG ellipse shape
		// ellipse: Object: an ellipse object (see dojox.gfx.defaultEllipse)
		return this.createObject(dojox.gfx.Ellipse, ellipse);	// dojox.gfx.Ellipse
	},
	createCircle: function(circle){
		// summary: creates an SVG circle shape
		// circle: Object: a circle object (see dojox.gfx.defaultCircle)
		return this.createObject(dojox.gfx.Circle, circle);	// dojox.gfx.Circle
	},
	createLine: function(line){
		// summary: creates an SVG line shape
		// line: Object: a line object (see dojox.gfx.defaultLine)
		return this.createObject(dojox.gfx.Line, line);	// dojox.gfx.Line
	},
	createPolyline: function(points){
		// summary: creates an SVG polyline/polygon shape
		// points: Object: a points object (see dojox.gfx.defaultPolyline)
		//	or an Array of points
		return this.createObject(dojox.gfx.Polyline, points);	// dojox.gfx.Polyline
	},
	createImage: function(image){
		// summary: creates an SVG image shape
		// image: Object: an image object (see dojox.gfx.defaultImage)
		return this.createObject(dojox.gfx.Image, image);	// dojox.gfx.Image
	},
	createText: function(text){
		// summary: creates an SVG text shape
		// text: Object: a text object (see dojox.gfx.defaultText)
		return this.createObject(dojox.gfx.Text, text);	// dojox.gfx.Text
	},
	createPath: function(path){
		// summary: creates an SVG path shape
		// path: Object: a path object (see dojox.gfx.defaultPath)
		return this.createObject(dojox.gfx.Path, path);	// dojox.gfx.Path
	},
	createTextPath: function(text){
		// summary: creates an SVG text shape
		// text: Object: a textpath object (see dojox.gfx.defaultTextPath)
		return this.createObject(dojox.gfx.TextPath, {}).setText(text);	// dojox.gfx.TextPath
	},
	createObject: function(shapeType, rawShape){
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method

		// SHOULD BE RE-IMPLEMENTED BY THE RENDERER!

		return null;	// dojox.gfx.Shape
	}
};

}

if(!dojo._hasResource["dojox.gfx.path"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.path"] = true;
dojo.provide("dojox.gfx.path");



dojo.declare("dojox.gfx.path.Path", dojox.gfx.Shape, {
	// summary: a generalized path shape
	
	constructor: function(rawNode){
		// summary: a path constructor
		// rawNode: Node: a DOM node to be used by this path object
		this.shape = dojo.clone(dojox.gfx.defaultPath);
		this.segments = [];
		this.absolute = true;
		this.last = {};
		this.rawNode = rawNode;
	},
	
	// mode manipulations
	setAbsoluteMode: function(mode){
		// summary: sets an absolute or relative mode for path points
		// mode: Boolean: true/false or "absolute"/"relative" to specify the mode
		this.absolute = typeof mode == "string" ? (mode == "absolute") : mode;
		return this; // self
	},
	getAbsoluteMode: function(){
		// summary: returns a current value of the absolute mode
		return this.absolute; // Boolean
	},
	
	getBoundingBox: function(){
		// summary: returns the bounding box {x, y, width, height} or null
		return (this.bbox && ("l" in this.bbox)) ? {x: this.bbox.l, y: this.bbox.t, width: this.bbox.r - this.bbox.l, height: this.bbox.b - this.bbox.t} : null; // dojox.gfx.Rectangle
	},
	
	getLastPosition: function(){
		// summary: returns the last point in the path, or null
		return "x" in this.last ? this.last : null; // Object
	},
	
	// segment interpretation
	_updateBBox: function(x, y){
		// summary: updates the bounding box of path with new point
		// x: Number: an x coordinate
		// y: Number: a y coordinate
		
		// we use {l, b, r, t} representation of a bbox
		if(this.bbox && ("l" in this.bbox)){
			if(this.bbox.l > x) this.bbox.l = x;
			if(this.bbox.r < x) this.bbox.r = x;
			if(this.bbox.t > y) this.bbox.t = y;
			if(this.bbox.b < y) this.bbox.b = y;
		}else{
			this.bbox = {l: x, b: y, r: x, t: y};
		}
	},
	_updateWithSegment: function(segment){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		var n = segment.args, l = n.length;
		// update internal variables: bbox, absolute, last
		switch(segment.action){
			case "M":
			case "L":
			case "C":
			case "S":
			case "Q":
			case "T":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(n[i], n[i + 1]);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "H":
				for(var i = 0; i < l; ++i){
					this._updateBBox(n[i], this.last.y);
				}
				this.last.x = n[l - 1];
				this.absolute = true;
				break;
			case "V":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, n[i]);
				}
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "m":
				var start = 0;
				if(!("x" in this.last)){
					this._updateBBox(this.last.x = n[0], this.last.y = n[1]);
					start = 2;
				}
				for(var i = start; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1]);
				}
				this.absolute = false;
				break;
			case "l":
			case "t":
				for(var i = 0; i < l; i += 2){
					this._updateBBox(this.last.x += n[i], this.last.y += n[i + 1]);
				}
				this.absolute = false;
				break;
			case "h":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x += n[i], this.last.y);
				}
				this.absolute = false;
				break;
			case "v":
				for(var i = 0; i < l; ++i){
					this._updateBBox(this.last.x, this.last.y += n[i]);
				}
				this.absolute = false;
				break;
			case "c":
				for(var i = 0; i < l; i += 6){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1]);
					this._updateBBox(this.last.x + n[i + 2], this.last.y + n[i + 3]);
					this._updateBBox(this.last.x += n[i + 4], this.last.y += n[i + 5]);
				}
				this.absolute = false;
				break;
			case "s":
			case "q":
				for(var i = 0; i < l; i += 4){
					this._updateBBox(this.last.x + n[i], this.last.y + n[i + 1]);
					this._updateBBox(this.last.x += n[i + 2], this.last.y += n[i + 3]);
				}
				this.absolute = false;
				break;
			case "A":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(n[i + 5], n[i + 6]);
				}
				this.last.x = n[l - 2];
				this.last.y = n[l - 1];
				this.absolute = true;
				break;
			case "a":
				for(var i = 0; i < l; i += 7){
					this._updateBBox(this.last.x += n[i + 5], this.last.y += n[i + 6]);
				}
				this.absolute = false;
				break;
		}
		// add an SVG path segment
		var path = [segment.action];
		for(var i = 0; i < l; ++i){
			path.push(dojox.gfx.formatNumber(n[i], true));
		}
		if(typeof this.shape.path == "string"){
			this.shape.path += path.join("");
		}else{
			var l = path.length, a = this.shape.path;
			for(var i = 0; i < l; ++i){
				a.push(path[i]);
			}
		}
	},
	
	// a dictionary, which maps segment type codes to a number of their argemnts
	_validSegments: {m: 2, l: 2, h: 1, v: 1, c: 6, s: 4, q: 4, t: 2, a: 7, z: 0},
	
	_pushSegment: function(action, args){
		// summary: adds a segment
		// action: String: valid SVG code for a segment's type
		// args: Array: a list of parameters for this segment
		var group = this._validSegments[action.toLowerCase()];
		if(typeof group == "number"){
			if(group){
				if(args.length >= group){
					var segment = {action: action, args: args.slice(0, args.length - args.length % group)};
					this.segments.push(segment);
					this._updateWithSegment(segment);
				}
			}else{
				var segment = {action: action, args: []};
				this.segments.push(segment);
				this._updateWithSegment(segment);
			}
		}
	},
	
	_collectArgs: function(array, args){
		// summary: converts an array of arguments to plain numeric values
		// array: Array: an output argument (array of numbers)
		// args: Array: an input argument (can be values of Boolean, Number, dojox.gfx.Point, or an embedded array of them)
		for(var i = 0; i < args.length; ++i){
			var t = args[i];
			if(typeof t == "boolean"){
				array.push(t ? 1 : 0);
			}else if(typeof t == "number"){
				array.push(t);
			}else if(t instanceof Array){
				this._collectArgs(array, t);
			}else if("x" in t && "y" in t){
				array.push(t.x, t.y);
			}
		}
	},

	// segments	
	moveTo: function(){
		// summary: formes a move segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "M" : "m", args);
		return this; // self
	},
	lineTo: function(){
		// summary: formes a line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "L" : "l", args);
		return this; // self
	},
	hLineTo: function(){
		// summary: formes a horizontal line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "H" : "h", args);
		return this; // self
	},
	vLineTo: function(){
		// summary: formes a vertical line segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "V" : "v", args);
		return this; // self
	},
	curveTo: function(){
		// summary: formes a curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "C" : "c", args);
		return this; // self
	},
	smoothCurveTo: function(){
		// summary: formes a smooth curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "S" : "s", args);
		return this; // self
	},
	qCurveTo: function(){
		// summary: formes a quadratic curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "Q" : "q", args);
		return this; // self
	},
	qSmoothCurveTo: function(){
		// summary: formes a quadratic smooth curve segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "T" : "t", args);
		return this; // self
	},
	arcTo: function(){
		// summary: formes an elliptic arc segment
		var args = [];
		this._collectArgs(args, arguments);
		this._pushSegment(this.absolute ? "A" : "a", args);
		return this; // self
	},
	closePath: function(){
		// summary: closes a path
		this._pushSegment("Z", []);
		return this; // self
	},
	
	// setShape
	_setPath: function(path){
		// summary: forms a path using an SVG path string
		// path: String: an SVG path string
		var p = dojo.isArray(path) ? path : path.match(dojox.gfx.pathSvgRegExp);
		this.segments = [];
		this.absolute = true;
		this.bbox = {};
		this.last = {};
		if(!p) return;
		// create segments
		var action = "",	// current action
			args = [],		// current arguments
			l = p.length;
		for(var i = 0; i < l; ++i){
			var t = p[i], x = parseFloat(t);
			if(isNaN(x)){
				if(action){
					this._pushSegment(action, args);
				}
				args = [];
				action = t;
			}else{
				args.push(x);
			}
		}
		this._pushSegment(action, args);
	},
	setShape: function(newShape){
		// summary: forms a path using a shape
		// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
		dojox.gfx.Shape.prototype.setShape.call(this, typeof newShape == "string" ? {path: newShape} : newShape);
		var path = this.shape.path;
		// switch to non-updating version of path building
		this.shape.path = [];
		this._setPath(path);
		// switch back to the string path
		this.shape.path = this.shape.path.join("");
		return this; // self
	},
	
	// useful constant for descendants
	_2PI: Math.PI * 2
});

dojo.declare("dojox.gfx.path.TextPath", dojox.gfx.path.Path, {
	// summary: a generalized TextPath shape

	constructor: function(rawNode){
		// summary: a TextPath shape constructor
		// rawNode: Node: a DOM node to be used by this TextPath object
		if(!("text" in this)){
			this.text = dojo.clone(dojox.gfx.defaultTextPath);
		}
		if(!("fontStyle" in this)){
			this.fontStyle = dojo.clone(dojox.gfx.defaultFont);
		}
	},
	setText: function(newText){
		// summary: sets a text to be drawn along the path
		this.text = dojox.gfx.makeParameters(this.text, 
			typeof newText == "string" ? {text: newText} : newText);
		this._setText();
		return this;	// self
	},
	setFont: function(newFont){
		// summary: sets a font for text
		this.fontStyle = typeof newFont == "string" ? 
			dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	}
});

}

if(!dojo._hasResource["dojox.gfx.svg"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.svg"] = true;
dojo.provide("dojox.gfx.svg");





dojox.gfx.svg.xmlns = {
	xlink: "http://www.w3.org/1999/xlink",
	svg:   "http://www.w3.org/2000/svg"
};

dojox.gfx.svg.getRef = function(name){
	// summary: returns a DOM Node specified by the name argument or null
	// name: String: an SVG external reference 
	if(!name || name == "none") return null;
	if(name.match(/^url\(#.+\)$/)){
		return dojo.byId(name.slice(5, -1));	// Node
	}
	// alternative representation of a reference
	if(name.match(/^#dojoUnique\d+$/)){
		// we assume here that a reference was generated by dojox.gfx
		return dojo.byId(name.slice(1));	// Node
	}
	return null;	// Node
};

dojox.gfx.svg.dasharray = {
	solid:				"none",
	shortdash:			[4, 1],
	shortdot:			[1, 1],
	shortdashdot:		[4, 1, 1, 1],
	shortdashdotdot:	[4, 1, 1, 1, 1, 1],
	dot:				[1, 3],
	dash:				[4, 3],
	longdash:			[8, 3],
	dashdot:			[4, 3, 1, 3],
	longdashdot:		[8, 3, 1, 3],
	longdashdotdot:		[8, 3, 1, 3, 1, 3]
};

dojo.extend(dojox.gfx.Shape, {
	// summary: SVG-specific implementation of dojox.gfx.Shape methods
	
	setFill: function(fill){
		// summary: sets a fill object (SVG)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)

		if(!fill){
			// don't fill
			this.fillStyle = null;
			this.rawNode.setAttribute("fill", "none");
			this.rawNode.setAttribute("fill-opacity", 0);
			return this;
		}
		var f;
		// FIXME: slightly magical. We're using the outer scope's "f", but setting it later
		var setter = function(x){
				// we assume that we're executing in the scope of the node to mutate
				this.setAttribute(x, f[x].toFixed(8));
			};
		if(typeof(fill) == "object" && "type" in fill){
			// gradient
			switch(fill.type){
				case "linear":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill);
					var gradient = this._setFillObject(f, "linearGradient");
					dojo.forEach(["x1", "y1", "x2", "y2"], setter, gradient);
					break;
				case "radial":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					var gradient = this._setFillObject(f, "radialGradient");
					dojo.forEach(["cx", "cy", "r"], setter, gradient);
					break;
				case "pattern":
					f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					var pattern = this._setFillObject(f, "pattern");
					dojo.forEach(["x", "y", "width", "height"], setter, pattern);
					break;
			}
			this.fillStyle = f;
			return this;
		}
		// color object
		var f = dojox.gfx.normalizeColor(fill);
		this.fillStyle = f;
		this.rawNode.setAttribute("fill", f.toCss());
		this.rawNode.setAttribute("fill-opacity", f.a);
		this.rawNode.setAttribute("fill-rule", "evenodd");
		return this;	// self
	},

	setStroke: function(stroke){
		// summary: sets a stroke object (SVG)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke) 
	
		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			this.rawNode.setAttribute("stroke", "none");
			this.rawNode.setAttribute("stroke-opacity", 0);
			return this;
		}
		// normalize the stroke
		if(typeof stroke == "string"){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		// generate attributes
		var rn = this.rawNode;
		if(s){
			rn.setAttribute("stroke", s.color.toCss());
			rn.setAttribute("stroke-opacity", s.color.a);
			rn.setAttribute("stroke-width",   s.width);
			rn.setAttribute("stroke-linecap", s.cap);
			if(typeof s.join == "number"){
				rn.setAttribute("stroke-linejoin",   "miter");
				rn.setAttribute("stroke-miterlimit", s.join);
			}else{
				rn.setAttribute("stroke-linejoin",   s.join);
			}
			var da = s.style.toLowerCase();
			if(da in dojox.gfx.svg.dasharray){ da = dojox.gfx.svg.dasharray[da]; }
			if(da instanceof Array){
				da = dojo.clone(da);
				for(var i = 0; i < da.length; ++i){
					da[i] *= s.width;
				}
				if(s.cap != "butt"){
					for(var i = 0; i < da.length; i += 2){
						da[i] -= s.width;
						if(da[i] < 1){ da[i] = 1; }
					}
					for(var i = 1; i < da.length; i += 2){
						da[i] += s.width;
					}
				}
				da = da.join(",");
			}
			rn.setAttribute("stroke-dasharray", da);
			rn.setAttribute("dojoGfxStrokeStyle", s.style);
		}
		return this;	// self
	},
	
	_getParentSurface: function(){
		var surface = this.parent;
		for(; surface && !(surface instanceof dojox.gfx.Surface); surface = surface.parent);
		return surface;
	},

	_setFillObject: function(f, nodeType){
		var svgns = dojox.gfx.svg.xmlns.svg;
		this.fillStyle = f;
		var surface = this._getParentSurface();
		var defs = surface.defNode;
		var fill = this.rawNode.getAttribute("fill");
		var ref  = dojox.gfx.svg.getRef(fill);
		if(ref){
			fill = ref;
			if(fill.tagName.toLowerCase() != nodeType.toLowerCase()){
				var id = fill.id;
				fill.parentNode.removeChild(fill);
				fill = document.createElementNS(svgns, nodeType);
				fill.setAttribute("id", id);
				defs.appendChild(fill);
			}else{
				while(fill.childNodes.length){
					fill.removeChild(fill.lastChild);
				}
			}
		}else{
			fill = document.createElementNS(svgns, nodeType);
			fill.setAttribute("id", dojox.gfx._base._getUniqueId());
			defs.appendChild(fill);
		}
		if(nodeType == "pattern"){
			if(dojo.isSafari){
				fill.setAttributeNS(null, "patternUnits", "userSpaceOnUse");
			}else{
				fill.setAttribute("patternUnits", "userSpaceOnUse");
			}
			var img = document.createElementNS(svgns, "image");
			img.setAttribute("x", 0);
			img.setAttribute("y", 0);
			img.setAttribute("width",  f.width .toFixed(8));
			img.setAttribute("height", f.height.toFixed(8));
			img.setAttributeNS(dojox.gfx.svg.xmlns.xlink, "href", f.src);
			fill.appendChild(img);
		}else{
			if(dojo.isSafari){
				fill.setAttributeNS(null, "gradientUnits", "userSpaceOnUse");
			}else{
				fill.setAttribute("gradientUnits", "userSpaceOnUse");
			}
			for(var i = 0; i < f.colors.length; ++i){
				var c = f.colors[i], t = document.createElementNS(svgns, "stop"),
					cc = c.color = dojox.gfx.normalizeColor(c.color);
				t.setAttribute("offset",       c.offset.toFixed(8));
				t.setAttribute("stop-color",   cc.toCss());
				t.setAttribute("stop-opacity", cc.a);
				fill.appendChild(t);
			}
		}
		this.rawNode.setAttribute("fill", "url(#" + fill.getAttribute("id") +")");
		this.rawNode.removeAttribute("fill-opacity");
		this.rawNode.setAttribute("fill-rule", "evenodd");
		return fill;
	},
	
	_applyTransform: function() {
		var matrix = this.matrix;
		if(matrix){
			var tm = this.matrix;
			this.rawNode.setAttribute("transform", "matrix(" +
				tm.xx.toFixed(8) + "," + tm.yx.toFixed(8) + "," +
				tm.xy.toFixed(8) + "," + tm.yy.toFixed(8) + "," +
				tm.dx.toFixed(8) + "," + tm.dy.toFixed(8) + ")");
		}else{
			this.rawNode.removeAttribute("transform");
		}
		return this;
	},

	setRawNode: function(rawNode){
		// summary:
		//	assigns and clears the underlying node that will represent this
		//	shape. Once set, transforms, gradients, etc, can be applied.
		//	(no fill & stroke by default)
		var r = this.rawNode = rawNode;
		r.setAttribute("fill", "none");
		r.setAttribute("fill-opacity", 0);
		r.setAttribute("stroke", "none");
		r.setAttribute("stroke-opacity", 0);
		r.setAttribute("stroke-width", 1);
		r.setAttribute("stroke-linecap", "butt");
		r.setAttribute("stroke-linejoin", "miter");
		r.setAttribute("stroke-miterlimit", 4);
	},
	
	setShape: function(newShape){
		// summary: sets a shape object (SVG)
		// newShape: Object: a shape object
		//	(see dojox.gfx.defaultPath,
		//	dojox.gfx.defaultPolyline,
		//	dojox.gfx.defaultRect,
		//	dojox.gfx.defaultEllipse,
		//	dojox.gfx.defaultCircle,
		//	dojox.gfx.defaultLine,
		//	or dojox.gfx.defaultImage)
		this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		for(var i in this.shape){
			if(i != "type"){ this.rawNode.setAttribute(i, this.shape[i]); }
		}
		return this;	// self
	},

	// move family

	_moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes (SVG)
		this.rawNode.parentNode.appendChild(this.rawNode);
		return this;	// self
	},
	_moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes (SVG)
		this.rawNode.parentNode.insertBefore(this.rawNode, this.rawNode.parentNode.firstChild);
		return this;	// self
	}
});

dojo.declare("dojox.gfx.Group", dojox.gfx.Shape, {
	// summary: a group shape (SVG), which can be used 
	//	to logically group shapes (e.g, to propagate matricies)
	constructor: function(){
		dojox.gfx.svg.Container._init.call(this);
	},
	setRawNode: function(rawNode){
		// summary: sets a raw SVG node to be used by this shape
		// rawNode: Node: an SVG node
		this.rawNode = rawNode;
	}
});
dojox.gfx.Group.nodeType = "g";

dojo.declare("dojox.gfx.Rect", dojox.gfx.shape.Rect, {
	// summary: a rectangle shape (SVG)
	setShape: function(newShape){
		// summary: sets a rectangle shape object (SVG)
		// newShape: Object: a rectangle shape object
		this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		for(var i in this.shape){
			if(i != "type" && i != "r"){ this.rawNode.setAttribute(i, this.shape[i]); }
		}
		if(this.shape.r){
			this.rawNode.setAttribute("ry", this.shape.r);
			this.rawNode.setAttribute("rx", this.shape.r);
		}
		return this;	// self
	}
});
dojox.gfx.Rect.nodeType = "rect";

dojox.gfx.Ellipse = dojox.gfx.shape.Ellipse;
dojox.gfx.Ellipse.nodeType = "ellipse";

dojox.gfx.Circle = dojox.gfx.shape.Circle;
dojox.gfx.Circle.nodeType = "circle";

dojox.gfx.Line = dojox.gfx.shape.Line;
dojox.gfx.Line.nodeType = "line";

dojo.declare("dojox.gfx.Polyline", dojox.gfx.shape.Polyline, {
	// summary: a polyline/polygon shape (SVG)
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object (SVG)
		// points: Object: a polyline/polygon shape object
		if(points && points instanceof Array){
			// branch
			// points: Array: an array of points
			this.shape = dojox.gfx.makeParameters(this.shape, { points: points });
			if(closed && this.shape.points.length){ 
				this.shape.points.push(this.shape.points[0]);
			}
		}else{
			this.shape = dojox.gfx.makeParameters(this.shape, points);
		}
		this.box = null;
		var attr = [];
		var p = this.shape.points;
		for(var i = 0; i < p.length; ++i){
			if(typeof p[i] == "number"){
				attr.push(p[i].toFixed(8));
			}else{
				attr.push(p[i].x.toFixed(8));
				attr.push(p[i].y.toFixed(8));
			}
		}
		this.rawNode.setAttribute("points", attr.join(" "));
		return this;	// self
	}
});
dojox.gfx.Polyline.nodeType = "polyline";

dojo.declare("dojox.gfx.Image", dojox.gfx.shape.Image, {
	// summary: an image (SVG)
	setShape: function(newShape){
		// summary: sets an image shape object (SVG)
		// newShape: Object: an image shape object
		this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var rawNode = this.rawNode;
		for(var i in this.shape){
			if(i != "type" && i != "src"){ rawNode.setAttribute(i, this.shape[i]); }
		}
		rawNode.setAttributeNS(dojox.gfx.svg.xmlns.xlink, "href", this.shape.src);
		return this;	// self
	}
});
dojox.gfx.Image.nodeType = "image";

dojo.declare("dojox.gfx.Text", dojox.gfx.shape.Text, {
	// summary: an anchored text (SVG)
	setShape: function(newShape){
		// summary: sets a text shape object (SVG)
		// newShape: Object: a text shape object
		this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var r = this.rawNode;
		var s = this.shape;
		r.setAttribute("x", s.x);
		r.setAttribute("y", s.y);
		r.setAttribute("text-anchor", s.align);
		r.setAttribute("text-decoration", s.decoration);
		r.setAttribute("rotate", s.rotated ? 90 : 0);
		r.setAttribute("kerning", s.kerning ? "auto" : 0);
		r.setAttribute("text-rendering", "optimizeLegibility");
		r.textContent = s.text;
		return this;	// self
	},
	getTextWidth: function(){ 
		// summary: get the text width in pixels 
		var rawNode = this.rawNode; 
		var oldParent = rawNode.parentNode; 
		var _measurementNode = rawNode.cloneNode(true); 
		_measurementNode.style.visibility = "hidden"; 

		// solution to the "orphan issue" in FF 
		var _width = 0; 
		var _text = _measurementNode.firstChild.nodeValue; 
		oldParent.appendChild(_measurementNode); 

		// solution to the "orphan issue" in Opera 
		// (nodeValue == "" hangs firefox) 
		if(_text!=""){ 
			while(!_width){ 
				_width = parseInt(_measurementNode.getBBox().width); 
			} 
		} 
		oldParent.removeChild(_measurementNode); 
		return _width; 
	} 
});
dojox.gfx.Text.nodeType = "text";

dojo.declare("dojox.gfx.Path", dojox.gfx.path.Path, {
	// summary: a path shape (SVG)
	_updateWithSegment: function(segment){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		dojox.gfx.Path.superclass._updateWithSegment.apply(this, arguments);
		if(typeof(this.shape.path) == "string"){
			this.rawNode.setAttribute("d", this.shape.path);
		}
	},
	setShape: function(newShape){
		// summary: forms a path using a shape (SVG)
		// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
		dojox.gfx.Path.superclass.setShape.apply(this, arguments);
		this.rawNode.setAttribute("d", this.shape.path);
		return this;	// self
	}
});
dojox.gfx.Path.nodeType = "path";

dojo.declare("dojox.gfx.TextPath", dojox.gfx.path.TextPath, {
	// summary: a textpath shape (SVG)
	_updateWithSegment: function(segment){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		dojox.gfx.Path.superclass._updateWithSegment.apply(this, arguments);
		this._setTextPath();
	},
	setShape: function(newShape){
		// summary: forms a path using a shape (SVG)
		// newShape: Object: an SVG path string or a path object (see dojox.gfx.defaultPath)
		dojox.gfx.Path.superclass.setShape.apply(this, arguments);
		this._setTextPath();
		return this;	// self
	},
	_setTextPath: function(){
		if(typeof this.shape.path != "string"){ return; }
		var r = this.rawNode;
		if(!r.firstChild){
			var tp = document.createElementNS(dojox.gfx.svg.xmlns.svg, "textPath");
			var tx = document.createTextNode("");
			tp.appendChild(tx);
			r.appendChild(tp);
		}
		var ref  = r.firstChild.getAttributeNS(dojox.gfx.svg.xmlns.xlink, "href");
		var path = ref && dojox.gfx.svg.getRef(ref);
		if(!path){
			var surface = this._getParentSurface();
			if(surface){
				var defs = surface.defNode;
				path = document.createElementNS(dojox.gfx.svg.xmlns.svg, "path");
				var id = dojox.gfx._base._getUniqueId();
				path.setAttribute("id", id);
				defs.appendChild(path);
				r.firstChild.setAttributeNS(dojox.gfx.svg.xmlns.xlink, "href", "#" + id);
			}
		}
		if(path){
			path.setAttribute("d", this.shape.path);
		}
	},
	_setText: function(){
		var r = this.rawNode;
		if(!r.firstChild){
			var tp = document.createElementNS(dojox.gfx.svg.xmlns.svg, "textPath");
			var tx = document.createTextNode("");
			tp.appendChild(tx);
			r.appendChild(tp);
		}
		r = r.firstChild;
		var t = this.text;
		r.setAttribute("alignment-baseline", "middle");
		switch(t.align){
			case "middle":
				r.setAttribute("text-anchor", "middle");
				r.setAttribute("startOffset", "50%");
				break;
			case "end":
				r.setAttribute("text-anchor", "end");
				r.setAttribute("startOffset", "100%");
				break;
			default:
				r.setAttribute("text-anchor", "start");
				r.setAttribute("startOffset", "0%");
				break;
		}
		//r.parentNode.setAttribute("alignment-baseline", "central");
		//r.setAttribute("dominant-baseline", "central");
		r.setAttribute("baseline-shift", "0.5ex");
		r.setAttribute("text-decoration", t.decoration);
		r.setAttribute("rotate", t.rotated ? 90 : 0);
		r.setAttribute("kerning", t.kerning ? "auto" : 0);
		r.firstChild.data = t.text;
	}
});
dojox.gfx.TextPath.nodeType = "text";

dojo.declare("dojox.gfx.Surface", dojox.gfx.shape.Surface, {
	// summary: a surface object to be used for drawings (SVG)
	constructor: function(){
		dojox.gfx.svg.Container._init.call(this);
	},
	setDimensions: function(width, height){
		// summary: sets the width and height of the rawNode
		// width: String: width of surface, e.g., "100px"
		// height: String: height of surface, e.g., "100px"
		if(!this.rawNode){ return this; }
		this.rawNode.setAttribute("width",  width);
		this.rawNode.setAttribute("height", height);
		return this;	// self
	},
	getDimensions: function(){
		// summary: returns an object with properties "width" and "height"
		return this.rawNode ? {width: this.rawNode.getAttribute("width"), height: this.rawNode.getAttribute("height")} : null; // Object
	}
});

dojox.gfx.createSurface = function(parentNode, width, height){
	// summary: creates a surface (SVG)
	// parentNode: Node: a parent node
	// width: String: width of surface, e.g., "100px"
	// height: String: height of surface, e.g., "100px"

	var s = new dojox.gfx.Surface();
	s.rawNode = document.createElementNS(dojox.gfx.svg.xmlns.svg, "svg");
	s.rawNode.setAttribute("width",  width);
	s.rawNode.setAttribute("height", height);

	var node = document.createElementNS(dojox.gfx.svg.xmlns.svg, "defs"); 
	s.rawNode.appendChild(node);
	s.defNode = node;
	
	dojo.byId(parentNode).appendChild(s.rawNode);
	return s;	// dojox.gfx.Surface
};

// Extenders

dojox.gfx.svg.Font = {
	_setFont: function(){
		// summary: sets a font object (SVG)
		var f = this.fontStyle;
		// next line doesn't work in Firefox 2 or Opera 9
		//this.rawNode.setAttribute("font", dojox.gfx.makeFontString(this.fontStyle));
		this.rawNode.setAttribute("font-style", f.style);
		this.rawNode.setAttribute("font-variant", f.variant);
		this.rawNode.setAttribute("font-weight", f.weight);
		this.rawNode.setAttribute("font-size", f.size);
		this.rawNode.setAttribute("font-family", f.family);
	}
};

dojox.gfx.svg.Container = {
	_init: function(){
		dojox.gfx.shape.Container._init.call(this);
	},
	add: function(shape){
		// summary: adds a shape to a group/surface
		// shape: dojox.gfx.Shape: an VML shape object
		if(this != shape.getParent()){
			this.rawNode.appendChild(shape.rawNode);
			//dojox.gfx.Group.superclass.add.apply(this, arguments);
			//this.inherited(arguments);
			dojox.gfx.shape.Container.add.apply(this, arguments);
		}
		return this;	// self
	},
	remove: function(shape, silently){
		// summary: remove a shape from a group/surface
		// shape: dojox.gfx.Shape: an VML shape object
		// silently: Boolean?: if true, regenerate a picture
		if(this == shape.getParent()){
			if(this.rawNode == shape.rawNode.parentNode){
				this.rawNode.removeChild(shape.rawNode);
			}
			//dojox.gfx.Group.superclass.remove.apply(this, arguments);
			//this.inherited(arguments);
			dojox.gfx.shape.Container.remove.apply(this, arguments);
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		var r = this.rawNode;
		while(r.lastChild){
			r.removeChild(r.lastChild);
		}
		//return this.inherited(arguments);	// self
		return dojox.gfx.shape.Container.clear.apply(this, arguments);
	},
	_moveChildToFront: dojox.gfx.shape.Container._moveChildToFront,
	_moveChildToBack:  dojox.gfx.shape.Container._moveChildToBack
};

dojo.mixin(dojox.gfx.shape.Creator, {
	// summary: SVG shape creators
	createObject: function(shapeType, rawShape){
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method
		if(!this.rawNode){ return null; }
		var shape = new shapeType();
		var node = document.createElementNS(dojox.gfx.svg.xmlns.svg, shapeType.nodeType); 
		shape.setRawNode(node);
		this.rawNode.appendChild(node);
		shape.setShape(rawShape);
		this.add(shape);
		return shape;	// dojox.gfx.Shape
	}
});

dojo.extend(dojox.gfx.Text, dojox.gfx.svg.Font);
dojo.extend(dojox.gfx.TextPath, dojox.gfx.svg.Font);

dojo.extend(dojox.gfx.Group, dojox.gfx.svg.Container);
dojo.extend(dojox.gfx.Group, dojox.gfx.shape.Creator);

dojo.extend(dojox.gfx.Surface, dojox.gfx.svg.Container);
dojo.extend(dojox.gfx.Surface, dojox.gfx.shape.Creator);

}

if(!dojo._hasResource["dojox.gfx.matrix"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.matrix"] = true;
dojo.provide("dojox.gfx.matrix");

(function(){
	var m = dojox.gfx.matrix;
	
	// candidates for dojox.math:
	m._degToRad = function(degree){ return Math.PI * degree / 180; };
	m._radToDeg = function(radian){ return radian / Math.PI * 180; };

	m.Matrix2D = function(arg){
		// summary: a 2D matrix object
		// description: Normalizes a 2D matrix-like object. If arrays is passed, 
		//		all objects of the array are normalized and multiplied sequentially.
		// arg: Object
		//		a 2D matrix-like object, a number, or an array of such objects
		if(arg){
			if(typeof arg == "number"){
				this.xx = this.yy = arg;
			}else if(arg instanceof Array){
				if(arg.length > 0){
					var matrix = m.normalize(arg[0]);
					// combine matrices
					for(var i = 1; i < arg.length; ++i){
						var l = matrix, r = dojox.gfx.matrix.normalize(arg[i]);
						matrix = new m.Matrix2D();
						matrix.xx = l.xx * r.xx + l.xy * r.yx;
						matrix.xy = l.xx * r.xy + l.xy * r.yy;
						matrix.yx = l.yx * r.xx + l.yy * r.yx;
						matrix.yy = l.yx * r.xy + l.yy * r.yy;
						matrix.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
						matrix.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
					}
					dojo.mixin(this, matrix);
				}
			}else{
				dojo.mixin(this, arg);
			}
		}
	};

	// the default (identity) matrix, which is used to fill in missing values
	dojo.extend(m.Matrix2D, {xx: 1, xy: 0, yx: 0, yy: 1, dx: 0, dy: 0});

	dojo.mixin(m, {
		// summary: class constants, and methods of dojox.gfx.matrix
		
		// matrix constants
		
		// identity: dojox.gfx.matrix.Matrix2D
		//		an identity matrix constant: identity * (x, y) == (x, y)
		identity: new m.Matrix2D(),
		
		// flipX: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at x = 0 line: flipX * (x, y) == (-x, y)
		flipX:    new m.Matrix2D({xx: -1}),
		
		// flipY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at y = 0 line: flipY * (x, y) == (x, -y)
		flipY:    new m.Matrix2D({yy: -1}),
		
		// flipXY: dojox.gfx.matrix.Matrix2D
		//		a matrix, which reflects points at the origin of coordinates: flipXY * (x, y) == (-x, -y)
		flipXY:   new m.Matrix2D({xx: -1, yy: -1}),
		
		// matrix creators
		
		translate: function(a, b){
			// summary: forms a translation matrix
			// description: The resulting matrix is used to translate (move) points by specified offsets.
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			if(arguments.length > 1){
				return new m.Matrix2D({dx: a, dy: b}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies offsets for both dimensions
			// b: null
			return new m.Matrix2D({dx: a.x, dy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		scale: function(a, b){
			// summary: forms a scaling matrix
			// description: The resulting matrix is used to scale (magnify) points by specified offsets.
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			if(arguments.length > 1){
				return new m.Matrix2D({xx: a, yy: b}); // dojox.gfx.matrix.Matrix2D
			}
			if(typeof a == "number"){
				// branch
				// a: Number: a uniform scaling factor used for the both coordinates
				// b: null
				return new m.Matrix2D({xx: a, yy: a}); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: dojox.gfx.Point: a point-like object, which specifies scale factors for both dimensions
			// b: null
			return new m.Matrix2D({xx: a.x, yy: a.y}); // dojox.gfx.matrix.Matrix2D
		},
		rotate: function(angle){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points 
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an angle of rotation in radians (>0 for CW)
			var c = Math.cos(angle);
			var s = Math.sin(angle);
			return new m.Matrix2D({xx: c, xy: -s, yx: s, yy: c}); // dojox.gfx.matrix.Matrix2D
		},
		rotateg: function(degree){
			// summary: forms a rotating matrix
			// description: The resulting matrix is used to rotate points
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.rotate() for comparison.
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			return m.rotate(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewX: function(angle) {
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({xy: -Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewXg: function(degree){
			// summary: forms an x skewing matrix
			// description: The resulting matrix is used to skew points in the x dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewX() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewX(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		skewY: function(angle){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified angle.
			// angle: Number: an skewing angle in radians
			return new m.Matrix2D({yx: Math.tan(angle)}); // dojox.gfx.matrix.Matrix2D
		},
		skewYg: function(degree){
			// summary: forms a y skewing matrix
			// description: The resulting matrix is used to skew points in the y dimension
			//		around the origin of coordinates (0, 0) by specified degree.
			//		See dojox.gfx.matrix.skewY() for comparison.
			// degree: Number: an skewing angle in degrees
			return m.skewY(m._degToRad(degree)); // dojox.gfx.matrix.Matrix2D
		},
		reflect: function(a, b){
			// summary: forms a reflection matrix
			// description: The resulting matrix is used to reflect points around a vector, 
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of reflection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = 2 * a * b / n2;
			return new m.Matrix2D({xx: 2 * a2 / n2 - 1, xy: xy, yx: xy, yy: 2 * b2 / n2 - 1}); // dojox.gfx.matrix.Matrix2D
		},
		project: function(a, b){
			// summary: forms an orthogonal projection matrix
			// description: The resulting matrix is used to project points orthogonally on a vector, 
			//		which goes through the origin.
			// a: dojox.gfx.Point: a point-like object, which specifies a vector of projection
			// b: null
			if(arguments.length == 1){
				b = a.y;
				a = a.x;
			}
			// branch
			// a: Number: an x coordinate value
			// b: Number: a y coordinate value
			
			// make a unit vector
			var a2 = a * a, b2 = b * b, n2 = a2 + b2, xy = a * b / n2;
			return new m.Matrix2D({xx: a2 / n2, xy: xy, yx: xy, yy: b2 / n2}); // dojox.gfx.matrix.Matrix2D
		},
		
		// ensure matrix 2D conformance
		normalize: function(matrix){
			// summary: converts an object to a matrix, if necessary
			// description: Converts any 2D matrix-like object or an array of
			//		such objects to a valid dojox.gfx.matrix.Matrix2D object.
			// matrix: Object: an object, which is converted to a matrix, if necessary
			return (matrix instanceof m.Matrix2D) ? matrix : new m.Matrix2D(matrix); // dojox.gfx.matrix.Matrix2D
		},
		
		// common operations
		
		clone: function(matrix){
			// summary: creates a copy of a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be cloned
			var obj = new m.Matrix2D();
			for(var i in matrix){
				if(typeof(matrix[i]) == "number" && typeof(obj[i]) == "number" && obj[i] != matrix[i]) obj[i] = matrix[i];
			}
			return obj; // dojox.gfx.matrix.Matrix2D
		},
		invert: function(matrix){
			// summary: inverts a 2D matrix
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object to be inverted
			var M = m.normalize(matrix),
				D = M.xx * M.yy - M.xy * M.yx,
				M = new m.Matrix2D({
					xx: M.yy/D, xy: -M.xy/D, 
					yx: -M.yx/D, yy: M.xx/D, 
					dx: (M.xy * M.dy - M.yy * M.dx) / D, 
					dy: (M.yx * M.dx - M.xx * M.dy) / D
				});
			return M; // dojox.gfx.matrix.Matrix2D
		},
		_multiplyPoint: function(matrix, x, y){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// x: Number: an x coordinate of a point
			// y: Number: a y coordinate of a point
			return {x: matrix.xx * x + matrix.xy * y + matrix.dx, y: matrix.yx * x + matrix.yy * y + matrix.dy}; // dojox.gfx.Point
		},
		multiplyPoint: function(matrix, /* Number||Point */ a, /* Number, optional */ b){
			// summary: applies a matrix to a point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: Number: an x coordinate of a point
			// b: Number: a y coordinate of a point
			var M = m.normalize(matrix);
			if(typeof a == "number" && typeof b == "number"){
				return m._multiplyPoint(M, a, b); // dojox.gfx.Point
			}
			// branch
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix object to be applied
			// a: dojox.gfx.Point: a point
			// b: null
			return m._multiplyPoint(M, a.x, a.y); // dojox.gfx.Point
		},
		multiply: function(matrix){
			// summary: combines matrices by multiplying them sequentially in the given order
			// matrix: dojox.gfx.matrix.Matrix2D...: a 2D matrix-like object, 
			//		all subsequent arguments are matrix-like objects too
			var M = m.normalize(matrix);
			// combine matrices
			for(var i = 1; i < arguments.length; ++i){
				var l = M, r = m.normalize(arguments[i]);
				M = new m.Matrix2D();
				M.xx = l.xx * r.xx + l.xy * r.yx;
				M.xy = l.xx * r.xy + l.xy * r.yy;
				M.yx = l.yx * r.xx + l.yy * r.yx;
				M.yy = l.yx * r.xy + l.yy * r.yy;
				M.dx = l.xx * r.dx + l.xy * r.dy + l.dx;
				M.dy = l.yx * r.dx + l.yy * r.dy + l.dy;
			}
			return M; // dojox.gfx.matrix.Matrix2D
		},
		
		// high level operations
		
		_sandwich: function(matrix, x, y){
			// summary: applies a matrix at a centrtal point
			// matrix: dojox.gfx.matrix.Matrix2D: a 2D matrix-like object, which is applied at a central point
			// x: Number: an x component of the central point
			// y: Number: a y component of the central point
			return m.multiply(m.translate(x, y), matrix, m.translate(-x, -y)); // dojox.gfx.matrix.Matrix2D
		},
		scaleAt: function(a, b, c, d){
			// summary: scales a picture using a specified point as a center of scaling
			// description: Compare with dojox.gfx.matrix.scale().
			// a: Number: a scaling factor used for the x coordinate
			// b: Number: a scaling factor used for the y coordinate
			// c: Number: an x component of a central point
			// d: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) uniform scale factor, Point
			//	2) uniform scale factor, x, y
			//	3) x scale, y scale, Point
			//	4) x scale, y scale, x, y
			
			switch(arguments.length){
				case 4:
					// a and b are scale factor components, c and d are components of a point
					return m._sandwich(m.scale(a, b), c, d); // dojox.gfx.matrix.Matrix2D
				case 3:
					if(typeof c == "number"){
						// branch
						// a: Number: a uniform scaling factor used for both coordinates
						// b: Number: an x component of a central point
						// c: Number: a y component of a central point
						// d: null
						return m._sandwich(m.scale(a), b, c); // dojox.gfx.matrix.Matrix2D
					}
					// branch
					// a: Number: a scaling factor used for the x coordinate
					// b: Number: a scaling factor used for the y coordinate
					// c: dojox.gfx.Point: a central point
					// d: null
					return m._sandwich(m.scale(a, b), c.x, c.y); // dojox.gfx.matrix.Matrix2D
			}
			// branch
			// a: Number: a uniform scaling factor used for both coordinates
			// b: dojox.gfx.Point: a central point
			// c: null
			// d: null
			return m._sandwich(m.scale(a), b.x, b.y); // dojox.gfx.matrix.Matrix2D
		},
		rotateAt: function(angle, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotate().
			// angle: Number: an angle of rotation in radians (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) rotation angle in radians, Point
			//	2) rotation angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.rotate(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}
			
			// branch
			// angle: Number: an angle of rotation in radians (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotate(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		rotategAt: function(degree, a, b){
			// summary: rotates a picture using a specified point as a center of rotation
			// description: Compare with dojox.gfx.matrix.rotateg().
			// degree: Number: an angle of rotation in degrees (>0 for CW)
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) rotation angle in degrees, Point
			//	2) rotation angle in degrees, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.rotateg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an angle of rotation in degrees (>0 for CCW)
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.rotateg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXAt: function(angle, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewX().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.skewX(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewX(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewXgAt: function(degree, a, b){
			// summary: skews a picture along the x axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewXg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewXg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewXg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYAt: function(angle, a, b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewY().
			// angle: Number: an skewing angle in radians
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in radians, Point
			//	2) skew angle in radians, x, y
			
			if(arguments.length > 2){
				return m._sandwich(m.skewY(angle), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// angle: Number: an skewing angle in radians
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewY(angle), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		},
		skewYgAt: function(/* Number */ degree, /* Number||Point */ a, /* Number, optional */ b){
			// summary: skews a picture along the y axis using a specified point as a center of skewing
			// description: Compare with dojox.gfx.matrix.skewYg().
			// degree: Number: an skewing angle in degrees
			// a: Number: an x component of a central point
			// b: Number: a y component of a central point
			
			// accepts several signatures:
			//	1) skew angle in degrees, Point
			//	2) skew angle in degrees, x, y

			if(arguments.length > 2){
				return m._sandwich(m.skewYg(degree), a, b); // dojox.gfx.matrix.Matrix2D
			}

			// branch
			// degree: Number: an skewing angle in degrees
			// a: dojox.gfx.Point: a central point
			// b: null
			return m._sandwich(m.skewYg(degree), a.x, a.y); // dojox.gfx.matrix.Matrix2D
		}
		
		//TODO: rect-to-rect mapping, scale-to-fit (isotropic and anisotropic versions)
		
	});
})();

// propagate Matrix2D up
dojox.gfx.Matrix2D = dojox.gfx.matrix.Matrix2D;

}

if(!dojo._hasResource["dojox.gfx.arc"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.arc"] = true;
dojo.provide("dojox.gfx.arc");



(function(){
	var m = dojox.gfx.matrix,
		unitArcAsBezier = function(alpha){
			// summary: return a start point, 1st and 2nd control points, and an end point of
			//		a an arc, which is reflected on the x axis
			// alpha: Number: angle in radians, the arc will be 2 * angle size
			var cosa  = Math.cos(alpha), sina  = Math.sin(alpha),
				p2 = {x: cosa + (4 / 3) * (1 - cosa), y: sina - (4 / 3) * cosa * (1 - cosa) / sina};
			return {	// Object
				s:  {x: cosa, y: -sina},
				c1: {x: p2.x, y: -p2.y},
				c2: p2,
				e:  {x: cosa, y: sina}
			};
		},
		twoPI = 2 * Math.PI, pi4 = Math.PI / 4, pi8 = Math.PI / 8, 
		pi48 = pi4 + pi8, curvePI4 = unitArcAsBezier(pi8);

	dojo.mixin(dojox.gfx.arc, {
		unitArcAsBezier: unitArcAsBezier,
		curvePI4: curvePI4,
		arcAsBezier: function(last, rx, ry, xRotg, large, sweep, x, y){
			// summary: calculates an arc as a series of Bezier curves
			//	given the last point and a standard set of SVG arc parameters,
			//	it returns an array of arrays of parameters to form a series of
			//	absolute Bezier curves.
			// last: Object: a point-like object as a start of the arc
			// rx: Number: a horizontal radius for the virtual ellipse
			// ry: Number: a vertical radius for the virtual ellipse
			// xRotg: Number: a rotation of an x axis of the virtual ellipse in degrees
			// large: Boolean: which part of the ellipse will be used (the larger arc if true)
			// sweep: Boolean: direction of the arc (CW if true)
			// x: Number: the x coordinate of the end point of the arc
			// y: Number: the y coordinate of the end point of the arc
			
			// calculate parameters
			large = Boolean(large);
			sweep = Boolean(sweep);
			var xRot = m._degToRad(xRotg),
				rx2 = rx * rx, ry2 = ry * ry,
				pa = m.multiplyPoint(
					m.rotate(-xRot), 
					{x: (last.x - x) / 2, y: (last.y - y) / 2}
				),
				pax2 = pa.x * pa.x, pay2 = pa.y * pa.y,
				c1 = Math.sqrt((rx2 * ry2 - rx2 * pay2 - ry2 * pax2) / (rx2 * pay2 + ry2 * pax2));
			if(isNaN(c1)){ c1 = 0; }
			var	ca = {
					x:  c1 * rx * pa.y / ry,
					y: -c1 * ry * pa.x / rx
				};
			if(large == sweep){
				ca = {x: -ca.x, y: -ca.y};
			}
			// the center
			var c = m.multiplyPoint(
				[
					m.translate(
						(last.x + x) / 2,
						(last.y + y) / 2
					),
					m.rotate(xRot)
				], 
				ca
			);
			// calculate the elliptic transformation
			var elliptic_transform = m.normalize([
				m.translate(c.x, c.y),
				m.rotate(xRot),
				m.scale(rx, ry)
			]);
			// start, end, and size of our arc
			var inversed = m.invert(elliptic_transform),
				sp = m.multiplyPoint(inversed, last),
				ep = m.multiplyPoint(inversed, x, y),
				startAngle = Math.atan2(sp.y, sp.x),
				endAngle   = Math.atan2(ep.y, ep.x),
				theta = startAngle - endAngle;	// size of our arc in radians
			if(sweep){ theta = -theta; }
			if(theta < 0){
				theta += twoPI;
			}else if(theta > twoPI){
				theta -= twoPI;
			}
							
			// draw curve chunks
			var alpha = pi8, curve = curvePI4, step  = sweep ? alpha : -alpha,
				result = [];
			for(var angle = theta; angle > 0; angle -= pi4){
				if(angle < pi48){
					alpha = angle / 2;
					curve = unitArcAsBezier(alpha);
					step  = sweep ? alpha : -alpha;
					angle = 0;	// stop the loop
				}
				var c1, c2, e,
					M = m.normalize([elliptic_transform, m.rotate(startAngle + step)]);
				if(sweep){
					c1 = m.multiplyPoint(M, curve.c1);
					c2 = m.multiplyPoint(M, curve.c2);
					e  = m.multiplyPoint(M, curve.e );
				}else{
					c1 = m.multiplyPoint(M, curve.c2);
					c2 = m.multiplyPoint(M, curve.c1);
					e  = m.multiplyPoint(M, curve.s );
				}
				// draw the curve
				result.push([c1.x, c1.y, c2.x, c2.y, e.x, e.y]);
				startAngle += 2 * step;
			}
			return result;	// Object
		}
	});
})();

}

if(!dojo._hasResource["dojox.gfx.vml"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx.vml"] = true;
dojo.provide("dojox.gfx.vml");






// dojox.gfx.vml.xmlns: String: a VML's namespace
dojox.gfx.vml.xmlns = "urn:schemas-microsoft-com:vml";

// dojox.gfx.vml.text_alignment: Object: mapping from SVG alignment to VML alignment
dojox.gfx.vml.text_alignment = {start: "left", middle: "center", end: "right"};

dojox.gfx.vml._parseFloat = function(str) {
	// summary: a helper function to parse VML-specific floating-point values
	// str: String: a representation of a floating-point number
	return str.match(/^\d+f$/i) ? parseInt(str) / 65536 : parseFloat(str);	// Number
};

dojox.gfx.vml._bool = {"t": 1, "true": 1};

dojo.extend(dojox.gfx.Shape, {
	// summary: VML-specific implementation of dojox.gfx.Shape methods

	setFill: function(fill){
		// summary: sets a fill object (VML)
		// fill: Object: a fill object
		//	(see dojox.gfx.defaultLinearGradient, 
		//	dojox.gfx.defaultRadialGradient, 
		//	dojox.gfx.defaultPattern, 
		//	or dojo.Color)

		if(!fill){
			// don't fill
			this.fillStyle = null;
			this.rawNode.filled = "f";
			return this;
		}
		if(typeof fill == "object" && "type" in fill){
			// gradient
			switch(fill.type){
				case "linear":
					var f = dojox.gfx.makeParameters(dojox.gfx.defaultLinearGradient, fill),
						s = [], a = f.colors, matrix = this._getRealMatrix(), m = dojox.gfx.matrix;
					this.fillStyle = f;
					dojo.forEach(a, function(v, i, a){
						a[i].color = dojox.gfx.normalizeColor(v.color);
					});
					if(a[0].offset > 0){
						s.push("0 " + a[0].color.toHex());
					}
					for(var i = 0; i < a.length; ++i){
						s.push(a[i].offset.toFixed(8) + " " + a[i].color.toHex());
					}
					var i = a.length - 1;
					if(a[i].offset < 1){
						s.push("1 " + a[i].color.toHex());
					}
					var fo = this.rawNode.fill;
					fo.colors.value = s.join(";");
					fo.method = "sigma";
					fo.type = "gradient";
					var fc1 = matrix ? m.multiplyPoint(matrix, f.x1, f.y1) : {x: f.x1, y: f.y1},
						fc2 = matrix ? m.multiplyPoint(matrix, f.x2, f.y2) : {x: f.x2, y: f.y2};
					fo.angle = (m._radToDeg(Math.atan2(fc2.x - fc1.x, fc2.y - fc1.y)) + 180) % 360;
					fo.on = true;
					break;
				case "radial":
					var f = dojox.gfx.makeParameters(dojox.gfx.defaultRadialGradient, fill);
					this.fillStyle = f;
					var l = parseFloat(this.rawNode.style.left),
						t = parseFloat(this.rawNode.style.top),
						w = parseFloat(this.rawNode.style.width),
						h = parseFloat(this.rawNode.style.height),
						c = isNaN(w) ? 1 : 2 * f.r / w,
						a = new Array(f.colors.length);
					// massage colors
					dojo.forEach(f.colors, function(v, i){
						a[i] = {offset: 1 - v.offset * c, color: dojox.gfx.normalizeColor(v.color)};
					});
					var i = a.length - 1;
					while(i >= 0 && a[i].offset < 0){ --i; }
					if(i < a.length - 1){
						// correct excessive colors
						var q = a[i], p = a[i + 1];
						p.color = dojo.blendColors(q.color, p.color, q.offset / (q.offset - p.offset));
						p.offset = 0;
						while(a.length - i > 2) a.pop();
					}
					// set colors
					var i = a.length - 1, s = [];
					if(a[i].offset > 0){
						s.push("0 " + a[i].color.toHex());
					}
					for(; i >= 0; --i){
						s.push(a[i].offset.toFixed(8) + " " + a[i].color.toHex());
					}
					if(a[0].offset < 1){
						s.push("1 " + a[0].color.toHex());
					}
					var fo = this.rawNode.fill;
					fo.colors.value = s.join(";");
					fo.method = "sigma";
					fo.type = "gradientradial";
					if(isNaN(w) || isNaN(h) || isNaN(l) || isNaN(t)){
						fo.focusposition = "0.5 0.5";
					}else{
						fo.focusposition = ((f.cx - l) / w).toFixed(8) + " " + ((f.cy - t) / h).toFixed(8);
					}
					fo.focussize = "0 0";
					fo.on = true;
					break;
				case "pattern":
					var f = dojox.gfx.makeParameters(dojox.gfx.defaultPattern, fill);
					this.fillStyle = f;
					var fo = this.rawNode.fill;
					fo.type = "tile";
					fo.src = f.src;
					if(f.width && f.height){
						// in points
						fo.size.x = dojox.gfx.px2pt(f.width);
						fo.size.y = dojox.gfx.px2pt(f.height);
					}
					fo.alignShape = "f";
					fo.position.x = 0;
					fo.position.y = 0;
					fo.origin.x = f.width  ? f.x / f.width  : 0;
					fo.origin.y = f.height ? f.y / f.height : 0;
					fo.on = true;
					break;
			}
			this.rawNode.fill.opacity = 1;
			return this;
		}
		// color object
		this.fillStyle = dojox.gfx.normalizeColor(fill);
		this.rawNode.fillcolor = this.fillStyle.toHex();
		this.rawNode.fill.opacity = this.fillStyle.a;
		this.rawNode.filled = true;
		return this;	// self
	},

	setStroke: function(stroke){
		// summary: sets a stroke object (VML)
		// stroke: Object: a stroke object
		//	(see dojox.gfx.defaultStroke) 
	
		if(!stroke){
			// don't stroke
			this.strokeStyle = null;
			this.rawNode.stroked = "f";
			return this;
		}
		// normalize the stroke
		if(typeof stroke == "string"){
			stroke = {color: stroke};
		}
		var s = this.strokeStyle = dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		s.color = dojox.gfx.normalizeColor(s.color);
		// generate attributes
		var rn = this.rawNode;
		rn.stroked = true;
		rn.strokecolor = s.color.toCss();
		rn.strokeweight = s.width + "px";	// TODO: should we assume that the width is always in pixels?
		if(rn.stroke) {
			rn.stroke.opacity = s.color.a;
			rn.stroke.endcap = this._translate(this._capMap, s.cap);
			if(typeof s.join == "number") {
				rn.stroke.joinstyle = "miter";
				rn.stroke.miterlimit = s.join;
			}else{
				rn.stroke.joinstyle = s.join;
				// rn.stroke.miterlimit = s.width;
			}
			rn.stroke.dashstyle = s.style == "none" ? "Solid" : s.style;
		}
		return this;	// self
	},
	
	_capMap: { butt: 'flat' },
	_capMapReversed: { flat: 'butt' },
	
	_translate: function(dict, value) {
		return (value in dict) ? dict[value] : value;
	},
	
	_applyTransform: function() {
		if(this.fillStyle && this.fillStyle.type == "linear"){
			this.setFill(this.fillStyle);
		}
		var matrix = this._getRealMatrix();
		if(!matrix) return this;
		var skew = this.rawNode.skew;
		if(typeof skew == "undefined"){
			for(var i = 0; i < this.rawNode.childNodes.length; ++i){
				if(this.rawNode.childNodes[i].tagName == "skew"){
					skew = this.rawNode.childNodes[i];
					break;
				}
			}
		}
		if(skew){
			skew.on = "f";
			var mt = matrix.xx.toFixed(8) + " " + matrix.xy.toFixed(8) + " " + 
				matrix.yx.toFixed(8) + " " + matrix.yy.toFixed(8) + " 0 0",
				offset = Math.floor(matrix.dx).toFixed() + "px " + Math.floor(matrix.dy).toFixed() + "px",
				s = this.rawNode.style,
				l = parseFloat(s.left),
				t = parseFloat(s.top),
				w = parseFloat(s.width),
				h = parseFloat(s.height);
			if(isNaN(l)) l = 0;
			if(isNaN(t)) t = 0;
			if(isNaN(w)) w = 1;
			if(isNaN(h)) h = 1;
			var origin = (-l / w - 0.5).toFixed(8) + " " + (-t / h - 0.5).toFixed(8);
			skew.matrix =  mt;
			skew.origin = origin;
			skew.offset = offset;
			skew.on = true;
		}
		return this;
	},

	setRawNode: function(rawNode){
		// summary:
		//	assigns and clears the underlying node that will represent this
		//	shape. Once set, transforms, gradients, etc, can be applied.
		//	(no fill & stroke by default)
		rawNode.stroked = "f";
		rawNode.filled  = "f";
		this.rawNode = rawNode;
	},
	
	// move family

	_moveToFront: function(){
		// summary: moves a shape to front of its parent's list of shapes (VML)
		this.rawNode.parentNode.appendChild(this.rawNode);
		return this;
	},
	_moveToBack: function(){
		// summary: moves a shape to back of its parent's list of shapes (VML)
		var r = this.rawNode, p = r.parentNode, n = p.firstChild;
		p.insertBefore(r, n);
		if(n.tagName == "rect"){
			// surface has a background rectangle, which position should be preserved
			n.swapNode(r);
		}
		return this;
	},

	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix
		return this.parentMatrix ? new dojox.gfx.Matrix2D([this.parentMatrix, this.matrix]) : this.matrix;	// dojox.gfx.Matrix2D
	}
});

dojo.declare("dojox.gfx.Group", dojox.gfx.Shape, {
	// summary: a group shape (VML), which can be used 
	//	to logically group shapes (e.g, to propagate matricies)
	constructor: function(){
		dojox.gfx.vml.Container._init.call(this);
	},
	// apply transformation
	_applyTransform: function(){
		// summary: applies a transformation matrix to a group
		var matrix = this._getRealMatrix();
		for(var i = 0; i < this.children.length; ++i){
			this.children[i]._updateParentMatrix(matrix);
		}
		return this;	// self
	}
});
dojox.gfx.Group.nodeType = "group";

dojo.declare("dojox.gfx.Rect", dojox.gfx.shape.Rect, {
	// summary: a rectangle shape (VML)
	setShape: function(newShape){
		// summary: sets a rectangle shape object (VML)
		// newShape: Object: a rectangle shape object
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var style = this.rawNode.style;
		style.left   = shape.x.toFixed();
		style.top    = shape.y.toFixed();
		style.width  = (typeof shape.width == "string" && shape.width.indexOf("%") >= 0)  ? shape.width  : shape.width.toFixed();
		style.height = (typeof shape.width == "string" && shape.height.indexOf("%") >= 0) ? shape.height : shape.height.toFixed();
		var r = Math.min(1, (shape.r / Math.min(parseFloat(shape.width), parseFloat(shape.height)))).toFixed(8);
		// a workaround for the VML's arcsize bug: cannot read arcsize of an instantiated node
		var parent = this.rawNode.parentNode, before = null;
		if(parent){
			if(parent.lastChild != this.rawNode){
				for(var i = 0; i < parent.childNodes.length; ++i){
					if(parent.childNodes[i] == this.rawNode){
						before = parent.childNodes[i+1];
						break;
					}
				}
			}
			parent.removeChild(this.rawNode);
		}
		this.rawNode.arcsize = r;
		if(parent){
			if(before){
				parent.insertBefore(this.rawNode, before);
			}else{
				parent.appendChild(this.rawNode);
			}
		}
		// set all necessary styles, which are lost by VML (yes, it's a VML's bug)
		return this.setTransform(this.matrix).setFill(this.fillStyle).setStroke(this.strokeStyle);	// self
	}
});
dojox.gfx.Rect.nodeType = "roundrect"; // use a roundrect so the stroke join type is respected

dojo.declare("dojox.gfx.Ellipse", dojox.gfx.shape.Ellipse, {
	// summary: an ellipse shape (VML)
	setShape: function(newShape){
		// summary: sets an ellipse shape object (VML)
		// newShape: Object: an ellipse shape object
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var style = this.rawNode.style;
		style.left   = (shape.cx - shape.rx).toFixed();
		style.top    = (shape.cy - shape.ry).toFixed();
		style.width  = (shape.rx * 2).toFixed();
		style.height = (shape.ry * 2).toFixed();
		return this.setTransform(this.matrix);	// self
	}
});
dojox.gfx.Ellipse.nodeType = "oval";

dojo.declare("dojox.gfx.Circle", dojox.gfx.shape.Circle, {
	// summary: a circle shape (VML)
	setShape: function(newShape){
		// summary: sets a circle shape object (VML)
		// newShape: Object: a circle shape object
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var style = this.rawNode.style;
		style.left   = (shape.cx - shape.r).toFixed();
		style.top    = (shape.cy - shape.r).toFixed();
		style.width  = (shape.r * 2).toFixed();
		style.height = (shape.r * 2).toFixed();
		return this;	// self
	}
});
dojox.gfx.Circle.nodeType = "oval";

dojo.declare("dojox.gfx.Line", dojox.gfx.shape.Line, {
	// summary: a line shape (VML)
	constructor: function(rawNode){
		if(rawNode) rawNode.setAttribute("dojoGfxType", "line");
	},
	setShape: function(newShape){
		// summary: sets a line shape object (VML)
		// newShape: Object: a line shape object
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		this.rawNode.path.v = "m" + shape.x1.toFixed() + " " + shape.y1.toFixed() +
			"l" + shape.x2.toFixed() + " " + shape.y2.toFixed() + "e";
		return this.setTransform(this.matrix);	// self
	}
});
dojox.gfx.Line.nodeType = "shape";

dojo.declare("dojox.gfx.Polyline", dojox.gfx.shape.Polyline, {
	// summary: a polyline/polygon shape (VML)
	constructor: function(rawNode){
		if(rawNode) rawNode.setAttribute("dojoGfxType", "polyline");
	},
	setShape: function(points, closed){
		// summary: sets a polyline/polygon shape object (VML)
		// points: Object: a polyline/polygon shape object
		// closed: Boolean?: if true, close the polyline explicitely
		if(points && points instanceof Array){
			// branch
			// points: Array: an array of points
			this.shape = dojox.gfx.makeParameters(this.shape, { points: points });
			if(closed && this.shape.points.length) this.shape.points.push(this.shape.points[0]);
		}else{
			this.shape = dojox.gfx.makeParameters(this.shape, points);
		}
		this.bbox = null;
		var attr = [], p = this.shape.points;
		if(p.length > 0){
			attr.push("m");
			var k = 1;
			if(typeof p[0] == "number"){
				attr.push(p[0].toFixed());
				attr.push(p[1].toFixed());
				k = 2;
			}else{
				attr.push(p[0].x.toFixed());
				attr.push(p[0].y.toFixed());
			}
			if(p.length > k){
				attr.push("l");
				for(var i = k; i < p.length; ++i){
					if(typeof p[i] == "number"){
						attr.push(p[i].toFixed());
					}else{
						attr.push(p[i].x.toFixed());
						attr.push(p[i].y.toFixed());
					}
				}
			}
		}
		attr.push("e");
		this.rawNode.path.v = attr.join(" ");
		return this.setTransform(this.matrix);	// self
	}
});
dojox.gfx.Polyline.nodeType = "shape";

dojo.declare("dojox.gfx.Image", dojox.gfx.shape.Image, {
	// summary: an image (VML)
	constructor: function(rawNode){
		if(rawNode) rawNode.setAttribute("dojoGfxType", "image");
	},
	getEventSource: function() {
		// summary: returns a Node, which is used as 
		//	a source of events for this shape
		return this.rawNode ? this.rawNode.firstChild : null;	// Node
	},
	setShape: function(newShape){
		// summary: sets an image shape object (VML)
		// newShape: Object: an image shape object
		var shape = this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var firstChild = this.rawNode.firstChild;
        firstChild.src = shape.src;
        if(shape.width || shape.height){
			firstChild.style.width  = shape.width;
			firstChild.style.height = shape.height;
        }
		return this.setTransform(this.matrix);	// self
	},
	_applyTransform: function() {
		var matrix = this._getRealMatrix();
		if(!matrix) return this;
		matrix = dojox.gfx.matrix.multiply(matrix, {dx: this.shape.x, dy: this.shape.y});
		var f = this.rawNode.filters["DXImageTransform.Microsoft.Matrix"];
		f.M11 = matrix.xx;
		f.M12 = matrix.xy;
		f.M21 = matrix.yx;
		f.M22 = matrix.yy;
		f.Dx  = matrix.dx;
		f.Dy  = matrix.dy;
		return this;
	}
});
dojox.gfx.Image.nodeType = "div";

dojo.declare("dojox.gfx.Text", dojox.gfx.shape.Text, {
	// summary: an anchored text (VML)
	constructor: function(rawNode){
		if(rawNode){rawNode.setAttribute("dojoGfxType", "text");}
		this.fontStyle = null;
	},
	_alignment: {start: "left", middle: "center", end: "right"},
	setShape: function(newShape){
		// summary: sets a text shape object (VML)
		// newShape: Object: a text shape object
		this.shape = dojox.gfx.makeParameters(this.shape, newShape);
		this.bbox = null;
		var r = this.rawNode, s = this.shape, x = s.x, y = s.y.toFixed();
		switch(s.align){
			case "middle":
				x -= 5;
				break;
			case "end":
				x -= 10;
				break;
		}
		this.rawNode.path.v = "m" + x.toFixed() + "," + y + 
			"l" + (x + 10).toFixed() + "," + y + "e";
		// find path and text path
		var p = null, t = null, c = r.childNodes;
		for(var i = 0; i < c.length; ++i){
			var tag = c[i].tagName;
			if(tag == "path"){
				p = c[i];
				if(t) break;
			}else if(tag == "textpath"){
				t = c[i];
				if(p) break;
			}
		}
		if(!p){
			p = this.rawNode.ownerDocument.createElement("v:path");
			r.appendChild(p);
		}
		if(!t){
			t = this.rawNode.ownerDocument.createElement("v:textpath");
			r.appendChild(t);
		}
		p.textPathOk = true;
		t.on = true;
		var a = dojox.gfx.vml.text_alignment[s.align];
		t.style["v-text-align"] = a ? a : "left";
		t.style["text-decoration"] = s.decoration;
		t.style["v-rotate-letters"] = s.rotated;
		t.style["v-text-kern"] = s.kerning;
		t.string = s.text;
		return this.setTransform(this.matrix);	// self
	},
	_setFont: function(){
		// summary: sets a font object (VML)
		var f = this.fontStyle, c = this.rawNode.childNodes;
		for(var i = 0; i < c.length; ++i){
			if(c[i].tagName == "textpath"){
				c[i].style.font = dojox.gfx.makeFontString(f);
				break;
			}
		}
		this.setTransform(this.matrix);
	},
	_getRealMatrix: function(){
		// summary: returns the cumulative ("real") transformation matrix
		//	by combining the shape's matrix with its parent's matrix;
		//	it makes a correction for a font size
		var matrix = dojox.gfx.Shape.prototype._getRealMatrix.call(this);
		// It appears that text is always aligned vertically at a middle of x-height (???).
		// It is impossible to obtain these metrics from VML => I try to approximate it with 
		// more-or-less util value of 0.7 * FontSize, which is typical for European fonts.
		if(matrix){
			matrix = dojox.gfx.matrix.multiply(matrix, 
				{dy: -dojox.gfx.normalizedLength(this.fontStyle ? this.fontStyle.size : "10pt") * 0.35});
		}
		return matrix;	// dojox.gfx.Matrix2D
	},
	getTextWidth: function(){ 
		// summary: get the text width, in px 
		var rawNode = this.rawNode, _display = rawNode.style.display; 
		rawNode.style.display = "inline"; 
		var _width = dojox.gfx.pt2px(parseFloat(rawNode.currentStyle.width)); 
		rawNode.style.display = _display; 
		return _width; 
	} 
});
dojox.gfx.Text.nodeType = "shape";

dojox.gfx.path._calcArc = function(alpha){
	// return a start point, 1st and 2nd control points, and an end point
	var cosa  = Math.cos(alpha), sina  = Math.sin(alpha),
		p2 = {x: cosa + (4 / 3) * (1 - cosa), y: sina - (4 / 3) * cosa * (1 - cosa) / sina};
	return {
		s:  {x: cosa, y: -sina},
		c1: {x: p2.x, y: -p2.y},
		c2: p2,
		e:  {x: cosa, y: sina}
	};
};

dojo.declare("dojox.gfx.Path", dojox.gfx.path.Path, {
	// summary: a path shape (VML)
	constructor: function(rawNode){
		if(rawNode && !rawNode.getAttribute("dojoGfxType")){
			rawNode.setAttribute("dojoGfxType", "path");
		}
		this.vmlPath = "";
		this.lastControl = {};
	},
	_updateWithSegment: function(segment){
		// summary: updates the bounding box of path with new segment
		// segment: Object: a segment
		var last = dojo.clone(this.last);
		dojox.gfx.Path.superclass._updateWithSegment.apply(this, arguments);
		// add a VML path segment
		var path = this[this.renderers[segment.action]](segment, last);
		if(typeof this.vmlPath == "string"){
			this.vmlPath += path.join("");
			this.rawNode.path.v = this.vmlPath + " r0,0 e";
		}else{
			this.vmlPath = this.vmlPath.concat(path);
		}
	},
	setShape: function(newShape){
		// summary: forms a path using a shape (VML)
		// newShape: Object: an VML path string or a path object (see dojox.gfx.defaultPath)
		this.vmlPath = [];
		this.lastControl = {};
		dojox.gfx.Path.superclass.setShape.apply(this, arguments);
		this.vmlPath = this.vmlPath.join("");
		this.rawNode.path.v = this.vmlPath + " r0,0 e";
		return this;
	},
	_pathVmlToSvgMap: {m: "M", l: "L", t: "m", r: "l", c: "C", v: "c", qb: "Q", x: "z", e: ""},
	// VML-specific segment renderers
	renderers: {
		M: "_moveToA", m: "_moveToR", 
		L: "_lineToA", l: "_lineToR", 
		H: "_hLineToA", h: "_hLineToR", 
		V: "_vLineToA", v: "_vLineToR", 
		C: "_curveToA", c: "_curveToR", 
		S: "_smoothCurveToA", s: "_smoothCurveToR", 
		Q: "_qCurveToA", q: "_qCurveToR", 
		T: "_qSmoothCurveToA", t: "_qSmoothCurveToR", 
		A: "_arcTo", a: "_arcTo", 
		Z: "_closePath", z: "_closePath"
	},
	_addArgs: function(path, args, from, upto){
		if(typeof upto == "undefined"){
			upto = args.length;
		}
		if(typeof from == "undefined"){
			from = 0;
		}
		for(var i = from; i < upto; ++i){
			path.push(" ");
			path.push(args[i].toFixed());
		}
	},
	_addArgsAdjusted: function(path, last, args, from, upto){
		if(typeof upto == "undefined"){
			upto = args.length;
		}
		if(typeof from == "undefined"){
			from = 0;
		}
		for(var i = from; i < upto; i += 2){
			path.push(" ");
			path.push((last.x + args[i]).toFixed());
			path.push(" ");
			path.push((last.y + args[i + 1]).toFixed());
		}
	},
	_moveToA: function(segment){
		var p = [" m"], n = segment.args, l = n.length;
		if(l == 2){
			this._addArgs(p, n);
		}else{
			this._addArgs(p, n, 0, 2);
			p.push(" l");
			this._addArgs(p, n, 2);
		}
		this.lastControl = {};
		return p;
	},
	_moveToR: function(segment, last){
		var p = ["x" in last ? " t" : " m"], n = segment.args, l = n.length;
		if(l == 2){
			this._addArgs(p, n);
		}else{
			this._addArgs(p, n, 0, 2);
			p.push(" r");
			this._addArgs(p, n, 2);
		}
		this.lastControl = {};
		return p;
	},
	_lineToA: function(segment){
		var p = [" l"];
		this._addArgs(p, segment.args);
		this.lastControl = {};
		return p;
	},
	_lineToR: function(segment){
		var p = [" r"];
		this._addArgs(p, segment.args);
		this.lastControl = {};
		return p;
	},
	_hLineToA: function(segment, last){
		var p = [" l"], n = segment.args, l = n.length, y = " " + last.y.toFixed();
		for(var i = 0; i < l; ++i){
			p.push(" ");
			p.push(n[i].toFixed());
			p.push(y);
		}
		this.lastControl = {};
		return p;
	},
	_hLineToR: function(segment){
		var p = [" r"], n = segment.args, l = n.length;
		for(var i = 0; i < l; ++i){
			p.push(" ");
			p.push(n[i].toFixed());
			p.push(" 0");
		}
		this.lastControl = {};
		return p;
	},
	_vLineToA: function(segment, last){
		var p = [" l"], n = segment.args, l = n.length, x = " " + last.x.toFixed();
		for(var i = 0; i < l; ++i){
			p.push(x);
			p.push(" ");
			p.push(n[i].toFixed());
		}
		this.lastControl = {};
		return p;
	},
	_vLineToR: function(segment){
		var p = [" r"], n = segment.args, l = n.length;
		for(var i = 0; i < l; ++i){
			p.push(" 0 ");
			p.push(n[i].toFixed());
		}
		this.lastControl = {};
		return p;
	},
	_curveToA: function(segment){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 6){
			p.push(" c");
			this._addArgs(p, n, i, i + 6);
		}
		this.lastControl = {x: n[l - 4], y: n[l - 3], type: "C"};
		return p;
	},
	_curveToR: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 6){
			p.push(" v");
			this._addArgs(p, n, i, i + 6);
			this.lastControl = {x: last.x + n[i + 2], y: last.y + n[i + 3]};
			last.x += n[i + 4];
			last.y += n[i + 5];
		}
		this.lastControl.type = "C";
		return p;
	},
	_smoothCurveToA: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 4){
			p.push(" c");
			if(this.lastControl.type == "C"){
				this._addArgs(p, [
					2 * last.x - this.lastControl.x, 
					2 * last.y - this.lastControl.y
				]);
			}else{
				this._addArgs(p, [last.x, last.y]);
			}
			this._addArgs(p, n, i, i + 4);
		}
		this.lastControl = {x: n[l - 4], y: n[l - 3], type: "C"};
		return p;
	},
	_smoothCurveToR: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 4){
			p.push(" v");
			if(this.lastControl.type == "C"){
				this._addArgs(p, [
					last.x - this.lastControl.x, 
					last.y - this.lastControl.y
				]);
			}else{
				this._addArgs(p, [0, 0]);
			}
			this._addArgs(p, n, i, i + 4);
			this.lastControl = {x: last.x + n[i], y: last.y + n[i + 1]};
			last.x += n[i + 2];
			last.y += n[i + 3];
		}
		this.lastControl.type = "C";
		return p;
	},
	_qCurveToA: function(segment){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 4){
			p.push(" qb");
			this._addArgs(p, n, i, i + 4);
		}
		this.lastControl = {x: n[l - 4], y: n[l - 3], type: "Q"};
		return p;
	},
	_qCurveToR: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 4){
			p.push(" qb");
			this._addArgsAdjusted(p, last, n, i, i + 4);
			this.lastControl = {x: last.x + n[i], y: last.y + n[i + 1]};
			last.x += n[i + 2];
			last.y += n[i + 3];
		}
		this.lastControl.type = "Q";
		return p;
	},
	_qSmoothCurveToA: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 2){
			p.push(" qb");
			if(this.lastControl.type == "Q"){
				this._addArgs(p, [
					this.lastControl.x = 2 * last.x - this.lastControl.x, 
					this.lastControl.y = 2 * last.y - this.lastControl.y
				]);
			}else{
				this._addArgs(p, [
					this.lastControl.x = last.x, 
					this.lastControl.y = last.y
				]);
			}
			this._addArgs(p, n, i, i + 2);
		}
		this.lastControl.type = "Q";
		return p;
	},
	_qSmoothCurveToR: function(segment, last){
		var p = [], n = segment.args, l = n.length;
		for(var i = 0; i < l; i += 2){
			p.push(" qb");
			if(this.lastControl.type == "Q"){
				this._addArgs(p, [
					this.lastControl.x = 2 * last.x - this.lastControl.x, 
					this.lastControl.y = 2 * last.y - this.lastControl.y
				]);
			}else{
				this._addArgs(p, [
					this.lastControl.x = last.x, 
					this.lastControl.y = last.y
				]);
			}
			this._addArgsAdjusted(p, last, n, i, i + 2);
		}
		this.lastControl.type = "Q";
		return p;
	},
	_arcTo: function(segment, last){
		var p = [], n = segment.args, l = n.length, relative = segment.action == "a";
		for(var i = 0; i < l; i += 7){
			var x1 = n[i + 5], y1 = n[i + 6];
			if(relative){
				x1 += last.x;
				y1 += last.y;
			}
			var result = dojox.gfx.arc.arcAsBezier(
				last, n[i], n[i + 1], n[i + 2], 
				n[i + 3] ? 1 : 0, n[i + 4] ? 1 : 0,
				x1, y1
			);
			for(var j = 0; j < result.length; ++j){
				p.push(" c");
				this._addArgs(p, result[j]);
			}
			last = {x: x1, y: y1};
		}
		this.lastControl = {};
		return p;
	},
	_closePath: function(){
		this.lastControl = {};
		return ["x"];
	}
});
dojox.gfx.Path.nodeType = "shape";

dojo.declare("dojox.gfx.TextPath", dojox.gfx.Path, {
	// summary: a textpath shape (VML)
	constructor: function(rawNode){
		if(rawNode){rawNode.setAttribute("dojoGfxType", "textpath");}
		this.fontStyle = null;
		if(!("text" in this)){
			this.text = dojo.clone(dojox.gfx.defaultTextPath);
		}
		if(!("fontStyle" in this)){
			this.fontStyle = dojo.clone(dojox.gfx.defaultFont);
		}
	},
	setText: function(newText){
		// summary: sets a text to be drawn along the path
		this.text = dojox.gfx.makeParameters(this.text, 
			typeof newText == "string" ? {text: newText} : newText);
		this._setText();
		return this;	// self
	},
	setFont: function(newFont){
		// summary: sets a font for text
		this.fontStyle = typeof newFont == "string" ? 
			dojox.gfx.splitFontString(newFont) :
			dojox.gfx.makeParameters(dojox.gfx.defaultFont, newFont);
		this._setFont();
		return this;	// self
	},

	_setText: function(){
		// summary: sets a text shape object (VML)
		this.bbox = null;
		var r = this.rawNode, s = this.text,
			// find path and text path
			p = null, t = null, c = r.childNodes;
		for(var i = 0; i < c.length; ++i){
			var tag = c[i].tagName;
			if(tag == "path"){
				p = c[i];
				if(t) break;
			}else if(tag == "textpath"){
				t = c[i];
				if(p) break;
			}
		}
		if(!p){
			p = this.rawNode.ownerDocument.createElement("v:path");
			r.appendChild(p);
		}
		if(!t){
			t = this.rawNode.ownerDocument.createElement("v:textpath");
			r.appendChild(t);
		}
		p.textPathOk = true;
		t.on = true;
		var a = dojox.gfx.vml.text_alignment[s.align];
		t.style["v-text-align"] = a ? a : "left";
		t.style["text-decoration"] = s.decoration;
		t.style["v-rotate-letters"] = s.rotated;
		t.style["v-text-kern"] = s.kerning;
		t.string = s.text;
	},
	_setFont: function(){
		// summary: sets a font object (VML)
		var f = this.fontStyle, c = this.rawNode.childNodes;
		for(var i = 0; i < c.length; ++i){
			if(c[i].tagName == "textpath"){
				c[i].style.font = dojox.gfx.makeFontString(f);
				break;
			}
		}
	}
});
dojox.gfx.TextPath.nodeType = "shape";

dojo.declare("dojox.gfx.Surface", dojox.gfx.shape.Surface, {
	// summary: a surface object to be used for drawings (VML)
	constructor: function(){
		dojox.gfx.vml.Container._init.call(this);
	},
	setDimensions: function(width, height){
		// summary: sets the width and height of the rawNode
		// width: String: width of surface, e.g., "100px"
		// height: String: height of surface, e.g., "100px"
		this.width  = dojox.gfx.normalizedLength(width);	// in pixels
		this.height = dojox.gfx.normalizedLength(height);	// in pixels
		if(!this.rawNode) return this;
		var cs = this.clipNode.style, 
			r = this.rawNode, rs = r.style,
			bs = this.bgNode.style;
		cs.width  = width;
		cs.height = height;
		cs.clip = "rect(0 " + width + " " + height + " 0)";
		rs.width = width;
		rs.height = height;
		r.coordsize = width + " " + height;
		bs.width = width;
		bs.height = height;
		return this;	// self
	},
	getDimensions: function(){
		// summary: returns an object with properties "width" and "height"
		var t = this.rawNode ? {
			width:  dojox.gfx.normalizedLength(this.rawNode.style.width), 
			height: dojox.gfx.normalizedLength(this.rawNode.style.height)} : null;
		if(t.width  <= 0){ t.width  = this.width; }
		if(t.height <= 0){ t.height = this.height; }
		return t;	// Object
	}
});

dojox.gfx.createSurface = function(parentNode, width, height){
	// summary: creates a surface (VML)
	// parentNode: Node: a parent node
	// width: String: width of surface, e.g., "100px"
	// height: String: height of surface, e.g., "100px"

	if(!width){ width = "100%"; }
	if(!height){ height = "100%"; }
	var s = new dojox.gfx.Surface(), p = dojo.byId(parentNode),
		c = s.clipNode = p.ownerDocument.createElement("div"),
		r = s.rawNode = p.ownerDocument.createElement("v:group"),
		cs = c.style, rs = r.style;
		
	p.style.width  = width;
	p.style.height = height;
		
	cs.width  = width;
	cs.height = height;
	cs.clip = "rect(0 " + width + " " + height + " 0)";
	cs.position = "absolute";
	rs.width  = width;
	rs.height = height;
	r.coordsize = (width == "100%" ? width : parseFloat(width)) + " " +
		(height == "100%" ? height : parseFloat(height));
	r.coordorigin = "0 0";
	
	// create a background rectangle, which is required to show all other shapes
	var b = s.bgNode = r.ownerDocument.createElement("v:rect"), bs = b.style;
	bs.left = bs.top = 0;
	bs.width  = rs.width;
	bs.height = rs.height;
	b.filled = b.stroked = "f";

	r.appendChild(b);
	c.appendChild(r);
	p.appendChild(c);
	
	s.width  = dojox.gfx.normalizedLength(width);	// in pixels
	s.height = dojox.gfx.normalizedLength(height);	// in pixels

	return s;	// dojox.gfx.Surface
};

// Extenders

dojox.gfx.vml.Container = {
	_init: function(){
		dojox.gfx.shape.Container._init.call(this);
	},
	add: function(shape){
		// summary: adds a shape to a group/surface
		// shape: dojox.gfx.Shape: an VML shape object
		if(this != shape.getParent()){
			this.rawNode.appendChild(shape.rawNode);
			//dojox.gfx.Group.superclass.add.apply(this, arguments);
			//this.inherited(arguments);
			dojox.gfx.shape.Container.add.apply(this, arguments);
		}
		return this;	// self
	},
	remove: function(shape, silently){
		// summary: remove a shape from a group/surface
		// shape: dojox.gfx.Shape: an VML shape object
		// silently: Boolean?: if true, regenerate a picture
		if(this == shape.getParent()){
			if(this.rawNode == shape.rawNode.parentNode){
				this.rawNode.removeChild(shape.rawNode);
			}
			//dojox.gfx.Group.superclass.remove.apply(this, arguments);
			//this.inherited(arguments);
			dojox.gfx.shape.Container.remove.apply(this, arguments);
		}
		return this;	// self
	},
	clear: function(){
		// summary: removes all shapes from a group/surface
		var r = this.rawNode;
		while(r.firstChild != r.lastChild){
			if(r.firstChild != this.bgNode){
				r.removeChild(r.firstChild);
			}
			if(r.lastChild != this.bgNode){
				r.removeChild(r.lastChild);
			}
		}
		//return this.inherited(arguments);	// self
		return dojox.gfx.shape.Container.clear.apply(this, arguments);
	},
	_moveChildToFront: dojox.gfx.shape.Container._moveChildToFront,
	_moveChildToBack:  dojox.gfx.shape.Container._moveChildToBack
};

dojo.mixin(dojox.gfx.shape.Creator, {
	// summary: VML shape creators
	createGroup: function(){
		// summary: creates a VML group shape
		var g = this.createObject(dojox.gfx.Group, null);	// dojox.gfx.Group
		// create a background rectangle, which is required to show all other shapes
		var r = g.rawNode.ownerDocument.createElement("v:rect");
		r.style.left = r.style.top = 0;
		r.style.width  = g.rawNode.style.width;
		r.style.height = g.rawNode.style.height;
		r.filled = r.stroked = "f";
		g.rawNode.appendChild(r);
		g.bgNode = r;
		return g;	// dojox.gfx.Group
	},
	createImage: function(image){
		// summary: creates a VML image shape
		// image: Object: an image object (see dojox.gfx.defaultImage)
		if(!this.rawNode) return null;
		var shape = new dojox.gfx.Image(), node = this.rawNode.ownerDocument.createElement('div');
		node.style.position = "absolute";
		node.style.width  = this.rawNode.style.width;
		node.style.height = this.rawNode.style.height;
		node.style.filter = "progid:DXImageTransform.Microsoft.Matrix(M11=1, M12=0, M21=0, M22=1, Dx=0, Dy=0)";
		var img  = this.rawNode.ownerDocument.createElement('img');
		node.appendChild(img);
		shape.setRawNode(node);
		this.rawNode.appendChild(node);
		shape.setShape(image);
		this.add(shape);
		return shape;	// dojox.gfx.Image
	},
	createObject: function(shapeType, rawShape) {
		// summary: creates an instance of the passed shapeType class
		// shapeType: Function: a class constructor to create an instance of
		// rawShape: Object: properties to be passed in to the classes "setShape" method
		// overrideSize: Boolean: set the size explicitly, if true
		if(!this.rawNode) return null;
		var shape = new shapeType(),
			node = this.rawNode.ownerDocument.createElement('v:' + shapeType.nodeType);
		shape.setRawNode(node);
		this.rawNode.appendChild(node);
		switch(shapeType){
			case dojox.gfx.Group:
			case dojox.gfx.Line:
			case dojox.gfx.Polyline:
			case dojox.gfx.Text:
			case dojox.gfx.Path:
			case dojox.gfx.TextPath:
				this._overrideSize(node);
		}
		shape.setShape(rawShape);
		this.add(shape);
		return shape;	// dojox.gfx.Shape
	},
	_overrideSize: function(node){
		var p = this;
		for(; p && !(p instanceof dojox.gfx.Surface); p = p.parent);
		node.style.width  = p.width;
		node.style.height = p.height;
		node.coordsize = p.width + " " + p.height;
	}
});

dojo.extend(dojox.gfx.Group, dojox.gfx.vml.Container);
dojo.extend(dojox.gfx.Group, dojox.gfx.shape.Creator);

dojo.extend(dojox.gfx.Surface, dojox.gfx.vml.Container);
dojo.extend(dojox.gfx.Surface, dojox.gfx.shape.Creator);

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become actived/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example:  Combobutton --> Menu -->
//		MenuItem.   The onBlur event for Combobutton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		Combobutton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit,
{
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary: tests whether the current selection is empty
		var _window = dojo.global;
		var _document = dojo.doc;
		if(_document.selection){ // IE
			return !_document.selection.createRange().text; // Boolean
		}else if(_window.getSelection){
			var selection = _window.getSelection();
			if(dojo.isString(selection)){ // Safari
				return !selection; // Boolean
			}else{ // Mozilla/W3
				return selection.isCollapsed || !selection.toString(); // Boolean
			}
		}
	},

	getBookmark: function(){
		// summary: Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bookmark, selection = dojo.doc.selection;
		if(selection){ // IE
			var range = selection.createRange();
			if(selection.type.toUpperCase()=='CONTROL'){
				bookmark = range.length ? dojo._toArray(range) : null;
			}else{
				bookmark = range.getBookmark();
			}
		}else{
			if(dojo.global.getSelection){
				selection = dojo.global.getSelection();
				if(selection){
					var range = selection.getRangeAt(0);
					bookmark = range.cloneRange();
				}
			}else{
				console.debug("No idea how to store the current selection for this browser!");
			}
		}
		return bookmark; // Array
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary: Moves current selection to a bookmark
		// bookmark: this should be a returned object from dojo.html.selection.getBookmark()
		var _document = dojo.doc;
		if(_document.selection){ // IE
			var range;
			if(dojo.isArray(bookmark)){
				range = _document.body.createControlRange();
				dojo.forEach(bookmark, range.addElement);
			}else{
				range = _document.selection.createRange();
				range.moveToBookmark(bookmark);
			}
			range.select();
		}else{ //Moz/W3C
			var selection = dojo.global.getSelection && dojo.global.getSelection();
			if(selection && selection.removeAllRanges){
				selection.removeAllRanges();
				selection.addRange(bookmark);
			}else{
				console.debug("No idea how to restore selection for this browser!");
			}
		}
	},

	getFocus: function(/*Widget*/menu, /*Window*/ openedForWindow){
		// summary:
		//	Returns the current focus and selection.
		//	Called when a popup appears (either a top level menu or a dialog),
		//	or when a toolbar/menubar receives focus
		//
		// menu:
		//	the menu that's being opened
		//
		// openedForWindow:
		//	iframe in which menu was opened
		//
		// returns:
		//	a handle to restore focus/selection

		return {
			// Node to return focus to
			node: menu && dojo.isDescendant(dijit._curFocus, menu.domNode) ? dijit._prevFocus : dijit._curFocus,

			// Previously selected text
			bookmark:
				!dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed) ?
				dojo.withGlobal(openedForWindow||dojo.global, dijit.getBookmark) :
				null,

			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase()=="iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}			
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item)
		if(bookmark && dojo.withGlobal(openedForWindow||dojo.global, dijit.isCollapsed)){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow||dojo.global, moveToBookmark, null, [bookmark]);
			}catch(e){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerWin: function(/*Window?*/targetWindow){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe) to detect when the user has clicked somewhere.
		//		Anyone that creates an iframe should call this function.

		if(!targetWindow){
			targetWindow = window;
		}

		dojo.connect(targetWindow.document, "onmousedown", null, function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			dijit._onTouchNode(evt.target||evt.srcElement);
		});
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's body
		var body = targetWindow.document.body || targetWindow.document.getElementsByTagName("body")[0];
		if(body){
			if(dojo.isIE){
				body.attachEvent('onactivate', function(evt){
					if(evt.srcElement.tagName.toLowerCase() != "body"){
						dijit._onFocusNode(evt.srcElement);
					}
				});
				body.attachEvent('ondeactivate', function(evt){ dijit._onBlurNode(evt.srcElement); });
			}else{
				body.addEventListener('focus', function(evt){ dijit._onFocusNode(evt.target); }, true);
				body.addEventListener('blur', function(evt){ dijit._onBlurNode(evt.target); }, true);
			}
		}
		body = null;	// prevent memory leak (apparent circular reference via closure)
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = false;
			w._setStateClass();
		}
		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer; dijit._setStack([]); }, 100);
	},

	_onTouchNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused or mouse-downed

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				if(node.dijitPopupParent){
					node=dijit.byId(node.dijitPopupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase()=="body"){
					// is this the root of the document or just the root of an iframe?
					if(node===dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead) and continue tracing up the document
					node=dojo.query("iframe").filter(function(iframe){ return iframe.contentDocument.body===node; })[0];
				}else{
					var id = node.getAttribute && node.getAttribute("widgetId");
					if(id){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary
		//		Callback when node is focused
		if(node && node.tagName && node.tagName.toLowerCase() == "body"){
			return;
		}
		dijit._onTouchNode(node);
		if(node==dijit._curFocus){ return; }
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);

		// handle focus/blur styling
		var w = dijit.getEnclosingWidget(node);
		if (w && w._setStateClass){
			w._focused = true;
			w._setStateClass();
		}
	},

	_setStack: function(newStack){
		// summary
		//	The stack of active widgets has changed.  Send out appropriate events and record new stack

		var oldStack = dijit._activeStack;		
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			var widget = dijit.byId(oldStack[i]);
			if(widget){
				dojo.publish("widgetBlur", [widget]);
				if(widget._onBlur){
					widget._onBlur();
				}
			}
		}

		// for all element that have come into focus, send focus event
		for(var i=nCommon; i<newStack.length; i++){
			var widget = dijit.byId(newStack[i]);
			if(widget){
				dojo.publish("widgetFocus", [widget]);
				if(widget._onFocus){
					widget._onFocus();
				}
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(dijit.registerWin);

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	constructor: function(){
		// summary:
		//	A set of widgets indexed by id
		this._hash={};
	},

	add: function(/*Widget*/ widget){
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id]=widget;
	},

	remove: function(/*String*/ id){
		delete this._hash[id];
	},

	forEach: function(/*Function*/ func){
		for(var id in this._hash){
			func(this._hash[id]);
		}
	},

	filter: function(/*Function*/ filter){
		var res = new dijit.WidgetSet();
		this.forEach(function(widget){
			if(filter(widget)){ res.add(widget); }
		});
		return res;		// dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		return this._hash[id];
	},

	byClass: function(/*String*/ cls){
		return this.filter(function(widget){ return widget.declaredClass==cls; });	// dijit.WidgetSet
	}
	});

// registry: list of all widgets on page
dijit.registry = new dijit.WidgetSet();

dijit._widgetTypeCtr = {};

dijit.getUniqueId = function(/*String*/widgetType){
	// summary
	//	Generates a unique id for a given widgetType

	var id;
	do{
		id = widgetType + "_" +
			(dijit._widgetTypeCtr[widgetType] !== undefined ?
				++dijit._widgetTypeCtr[widgetType] : dijit._widgetTypeCtr[widgetType] = 0);
	}while(dijit.byId(id));
	return id; // String
};


if(dojo.isIE){
	// Only run this for IE because we think it's only necessary in that case,
	// and because it causes problems on FF.  See bug #3531 for details.
	dojo.addOnUnload(function(){
		dijit.registry.forEach(function(widget){ widget.destroy(); });
	});
}

dijit.byId = function(/*String|Widget*/id){
	// summary:
	//		Returns a widget by its id, or if passed a widget, no-op (like dojo.byId())
	return (dojo.isString(id)) ? dijit.registry.byId(id) : id; // Widget
};

dijit.byNode = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget as referenced by node
	return dijit.registry.byId(node.getAttribute("widgetId")); // Widget
};

dijit.getEnclosingWidget = function(/* DOMNode */ node){
	// summary:
	//		Returns the widget whose dom tree contains node or null if
	//		the node is not contained within the dom tree of any widget
	while(node){
		if(node.getAttribute && node.getAttribute("widgetId")){
			return dijit.registry.byId(node.getAttribute("widgetId"));
		}
		node = node.parentNode;
	}
	return null;
};

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");

// ported from dojo.html.util

dijit.getViewport = function(){
	//	summary
	//	Returns the dimensions and scroll position of the viewable area of a browser window

	var _window = dojo.global;
	var _document = dojo.doc;

	// get viewport size
	var w = 0, h = 0;
	if(dojo.isMozilla){
		// mozilla
		// _window.innerHeight includes the height taken by the scroll bar
		// clientHeight is ideal but has DTD issues:
		// #4539: FF reverses the roles of body.clientHeight/Width and documentElement.clientHeight/Width based on the DTD!
		// check DTD to see whether body or documentElement returns the viewport dimensions using this algorithm:
		var minw, minh, maxw, maxh;
		if(_document.body.clientWidth>_document.documentElement.clientWidth){
			minw = _document.documentElement.clientWidth;
			maxw = _document.body.clientWidth;
		}else{
			maxw = _document.documentElement.clientWidth;
			minw = _document.body.clientWidth;
		}
		if(_document.body.clientHeight>_document.documentElement.clientHeight){
			minh = _document.documentElement.clientHeight;
			maxh = _document.body.clientHeight;
		}else{
			maxh = _document.documentElement.clientHeight;
			minh = _document.body.clientHeight;
		}
		w = (maxw > _window.innerWidth) ? minw : maxw;
		h = (maxh > _window.innerHeight) ? minh : maxh;
	}else if(!dojo.isOpera && _window.innerWidth){
		//in opera9, dojo.body().clientWidth should be used, instead
		//of window.innerWidth/document.documentElement.clientWidth
		//so we have to check whether it is opera
		w = _window.innerWidth;
		h = _window.innerHeight;
	}else if(dojo.isIE && _document.documentElement && _document.documentElement.clientHeight){
		w = _document.documentElement.clientWidth;
		h = _document.documentElement.clientHeight;
	}else if(dojo.body().clientWidth){
		// IE5, Opera
		w = dojo.body().clientWidth;
		h = dojo.body().clientHeight;
	}

	// get scroll position
	var scroll = dojo._docScroll();

	return { w: w, h: h, l: scroll.x, t: scroll.y };	//	object
};

dijit.placeOnScreen = function(
	/* DomNode */	node,
	/* Object */		pos,
	/* Object */		corners,
	/* boolean? */		tryOnly){
	//	summary:
	//		Keeps 'node' in the visible area of the screen while trying to
	//		place closest to pos.x, pos.y. The input coordinates are
	//		expected to be the desired document position.
	//
	//		Set which corner(s) you want to bind to, such as
	//		
	//			placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])
	//		
	//		The desired x/y will be treated as the topleft(TL)/topright(TR) or
	//		BottomLeft(BL)/BottomRight(BR) corner of the node. Each corner is tested
	//		and if a perfect match is found, it will be used. Otherwise, it goes through
	//		all of the specified corners, and choose the most appropriate one.
	//		
	//		NOTE: node is assumed to be absolutely or relatively positioned.

	var choices = dojo.map(corners, function(corner){ return { corner: corner, pos: pos }; });

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dijit.getViewport();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to document.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best=null;
	for(var i=0; i<choices.length; i++){
		var corner = choices[i].corner;
		var pos = choices[i].pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(corner);
		}

		// get node's size
		var oldDisplay = node.style.display;
		var oldVis = node.style.visibility;
		node.style.visibility = "hidden";
		node.style.display = "";
		var mb = dojo.marginBox(node);
		node.style.display = oldDisplay;
		node.style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = (corner.charAt(1)=='L' ? pos.x : Math.max(view.l, pos.x - mb.w)),
			startY = (corner.charAt(0)=='T' ? pos.y : Math.max(view.t, pos.y -  mb.h)),
			endX = (corner.charAt(1)=='L' ? Math.min(view.l+view.w, startX+mb.w) : pos.x),
			endY = (corner.charAt(0)=='T' ? Math.min(view.t+view.h, startY+mb.h) : pos.y),
			width = endX-startX,
			height = endY-startY,
			overflow = (mb.w-width) + (mb.h-height);

		if(best==null || overflow<best.overflow){
			best = {
				corner: corner,
				aroundCorner: choices[i].aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		if(overflow==0){
			break;
		}
	}

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	return best;
}

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	//	summary
	//	Like placeOnScreen, except it accepts aroundNode instead of x,y
	//	and attempts to place node around it.  Uses margin box dimensions.
	//
	//	aroundCorners
	//		specify Which corner of aroundNode should be
	//		used to place the node => which corner(s) of node to use (see the
	//		corners parameter in dijit.placeOnScreen)
	//		e.g. {'TL': 'BL', 'BL': 'TL'}
	//
	//	layoutNode: Function(node, orient)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.


	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodeW = aroundNode.offsetWidth; //mb.w;
	var aroundNodeH = aroundNode.offsetHeight; //mb.h;
	var aroundNodePos = dojo.coords(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: aroundNodePos.x + (nodeCorner.charAt(1)=='L' ? 0 : aroundNodeW),
				y: aroundNodePos.y + (nodeCorner.charAt(0)=='T' ? 0 : aroundNodeH)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
}

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");

dijit.getDocumentWindow = function(doc){
	//	summary
	// 	Get window object associated with document doc

	// With Safari, there is not way to retrieve the window from the document, so we must fix it.
	if(dojo.isSafari && !doc._parentWindow){
		/*
			This is a Safari specific function that fix the reference to the parent
			window from the document object.
		*/
		var fix=function(win){
			win.document._parentWindow=win;
			for(var i=0; i<win.frames.length; i++){
				fix(win.frames[i]);
			}
		}
		fix(window.top);
	}

	//In some IE versions (at least 6.0), document.parentWindow does not return a
	//reference to the real window object (maybe a copy), so we must fix it as well
	//We use IE specific execScript to attach the real window reference to
	//document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow && !doc._parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc._parentWindow || doc.parentWindow || doc.defaultView;	//	Window
}

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





dijit.popup = new function(){
	// summary:
	//		This class is used to show/hide widgets as popups.
	//

	var stack = [],
		beginZIndex=1000,
		idGen = 1;

	this.open = function(/*Object*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// args: Object
		//		popup: Widget
		//			widget to display,
		//		parent: Widget
		//			the button etc. that is displaying this popup
		//		around: DomNode
		//			DOM node (typically a button); place popup relative to this node
		//		orient: Object
		//			structure specifying possible positions of popup relative to "around" node
		//		onCancel: Function
		//			callback when user has canceled the popup by
		//				1. hitting ESC or
		//				2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
		//				   ie: whenever popupWidget.onCancel() is called, args.onCancel is called
		//		onClose: Function
		//			callback whenever this popup is closed
		//		onExecute: Function
		//			callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
		//
		// examples:
		//		1. opening at the mouse position
		//			dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//		2. opening the widget as a dropdown
		//			dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}  });
		//
		//	Note that whatever widget called dijit.popup.open() should also listen to it's own _onBlur callback
		//	(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var widget = args.popup,
			orient = args.orient || {'BL':'TL', 'TL':'BL'},
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+idGen++);

		// make wrapper div to hold widget and possibly hold iframe behind it.
		// we can't attach the iframe as a child of the widget.domNode because
		// widget.domNode might be a <table>, <ul>, etc.
		var wrapper = dojo.doc.createElement("div");
		wrapper.id = id;
		wrapper.className="dijitPopup";
		wrapper.style.zIndex = beginZIndex + stack.length;
		wrapper.style.visibility = "hidden";
		if(args.parent){
			wrapper.dijitPopupParent=args.parent.id;
		}
		dojo.body().appendChild(wrapper);

		widget.domNode.style.display="";
		wrapper.appendChild(widget.domNode);

		var iframe = new dijit.BackgroundIframe(wrapper);

		// position the wrapper node
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR']);

		wrapper.style.visibility = "visible";
		// TODO: use effects to fade in wrapper

		var handlers = [];

		// Compute the closest ancestor popup that's *not* a child of another popup.
		// Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		function getTopPopup(){
			for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--);
			return stack[pi];
		}

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.keyCode == dojo.keys.ESCAPE && args.onCancel){
				args.onCancel();
			}else if(evt.keyCode == dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", null, args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", null, function(){
			var topPopup = getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			wrapper: wrapper,
			iframe: iframe,
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			widget.onOpen(best);
		}

		return best;
	};

	this.close = function(/*Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				wrapper = top.wrapper,
				iframe = top.iframe,
				widget = top.widget,
				onClose = top.onClose;
	
			if(widget.onClose){
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);
	
			// #2685: check if the widget still has a domNode so ContentPane can change its URL without getting an error
			if(!widget||!widget.domNode){ return; }
			dojo.style(widget.domNode, "display", "none");
			dojo.body().appendChild(widget.domNode);
			iframe.destroy();
			dojo._destroyElement(wrapper);
	
			if(onClose){
				onClose();
			}
		}
	};
}();

dijit._frames = new function(){
	// summary: cache of iframes
	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE){
				var html="<iframe src='javascript:\"\"'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	var iframe = dojo.doc.createElement("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didnt work.
			dojo.body().appendChild(iframe);
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="";
		if(dojo.isIE){
			iframe.style.removeExpression("width");
			iframe.style.removeExpression("height");
		}
		queue.push(iframe);
	}
}();

// fill the queue
if(dojo.isIE && dojo.isIE < 7){
	dojo.addOnLoad(function(){
		var f = dijit._frames;
		dojo.forEach([f.pop()], f.push);
	});
}


dijit.BackgroundIframe = function(/* DomNode */node){
	//	summary:
	//		For IE z-index schenanigans. id attribute is required.
	//
	//	description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if((dojo.isIE && dojo.isIE < 7) || (dojo.isFF && dojo.isFF < 3 && dojo.hasClass(dojo.body(), "dijit_a11y"))){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE){
			iframe.style.setExpression("width", "document.getElementById('" + node.id + "').offsetWidth");
			iframe.style.setExpression("height", "document.getElementById('" + node.id + "').offsetHeight");
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	destroy: function(){
		//	summary: destroy the iframe
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");

dijit.scrollIntoView = function(/* DomNode */node){
	//	summary
	//	Scroll the passed node into view, if it is not.

	// don't rely on that node.scrollIntoView works just because the function is there
	// it doesnt work in Konqueror or Opera even though the function is there and probably
	// not safari either
	// dont like browser sniffs implementations but sometimes you have to use it
	if(dojo.isIE){
		//only call scrollIntoView if there is a scrollbar for this menu,
		//otherwise, scrollIntoView will scroll the window scrollbar
		if(dojo.marginBox(node.parentNode).h <= node.parentNode.scrollHeight){ //PORT was getBorderBox
			node.scrollIntoView(false);
		}
	}else if(dojo.isMozilla){
		node.scrollIntoView(false);
	}else{
		var parent = node.parentNode;
		var parentBottom = parent.scrollTop + dojo.marginBox(parent).h; //PORT was getBorderBox
		var nodeBottom = node.offsetTop + dojo.marginBox(node).h;
		if(parentBottom < nodeBottom){
			parent.scrollTop += (nodeBottom - parentBottom);
		}else if(parent.scrollTop > node.offsetTop){
			parent.scrollTop -= (parent.scrollTop - node.offsetTop);
		}
	}
};

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
dojo.provide("dijit._base.sniff");

// ported from dojo.html.applyBrowserClass (style.js)

//	summary:
//		Applies pre-set class names based on browser & version to the
//		top-level HTML node.  Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.
(function(){
	var d = dojo;
	var ie = d.isIE;
	var opera = d.isOpera;
	var maj = Math.floor;
	var classes = {
		dj_ie: ie,
//		dj_ie55: ie == 5.5,
		dj_ie6: maj(ie) == 6,
		dj_ie7: maj(ie) == 7,
		dj_iequirks: ie && d.isQuirks,
// NOTE: Opera not supported by dijit
		dj_opera: opera,
		dj_opera8: maj(opera) == 8,
		dj_opera9: maj(opera) == 9,
		dj_khtml: d.isKhtml,
		dj_safari: d.isSafari,
		dj_gecko: d.isMozilla
	}; // no dojo unsupported browsers

	for(var p in classes){
		if(classes[p]){
			var html = dojo.doc.documentElement; //TODO browser-specific DOM magic needed?
			if(html.className){
				html.className += " " + p;
			}else{
				html.className = p;
			}
		}
	}
})();

}

if(!dojo._hasResource["dijit._base.bidi"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.bidi"] = true;
dojo.provide("dijit._base.bidi");

// summary: applies a class to the top of the document for right-to-left stylesheet rules

dojo.addOnLoad(function(){
	if(!dojo._isBodyLtr()){
		dojo.addClass(dojo.body(), "dijitRtl");
	}
});

}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//	These functions are used to repetitively call a user specified callback
	//	method when a specific key or mouse click over a specific DOM node is
	//	held down for a specific amount of time.
	//	Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		this._currentTimeout = (this._currentTimeout < 0) ? this._initialDelay : ((this._subsequentDelay > 1) ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay));
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /* Object */ _this, /*DOMNode*/ node, /* Function */ callback, /* Object */ obj, /* Number */ subsequentDelay, /* Number */ initialDelay){
		// summary:
		//      Start a timed, repeating callback sequence.
		//      If already started, the function call is ignored.
		//      This method is not normally called by the user but can be
		//      when the normal listener code is insufficient.
		//	Parameters:
		//	evt: key or mouse event object to pass to the user callback
		//	_this: pointer to the user's widget space.
		//	node: the DOM node object to pass the the callback function
		//	callback: function to call until the sequence is stopped called with 3 parameters:
		//		count: integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		//		node: the DOM node object passed in
		//		evt: key or mouse event object
		//	obj: user space object used to uniquely identify each typematic sequence
		//	subsequentDelay: if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		//	initialDelay: the number of milliseconds until the 2nd event occurs, default=500ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
		}
	},

	stop: function(){
		// summary:
		//	  Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key.
		//	keyObject: an object defining the key to listen for.
		//		key: (mandatory) the keyCode (number) or character (string) to listen for.
		//		ctrlKey: desired ctrl key state to initiate the calback sequence:
		//			pressed (true)
		//			released (false)
		//			either (unspecified)
		//		altKey: same as ctrlKey but for the alt key
		//		shiftKey: same as ctrlKey but for the shift key
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.keyCode == keyObject.keyCode && (!keyObject.charCode || keyObject.charCode == evt.charCode) &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.ctrlKey) &&
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.ctrlKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(keyObject, _this, node, callback, keyObject, subsequentDelay, initialDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a typematic mouse click.
		//	See the trigger method for other parameters.
		//	Returns an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				dojo.stopEvent(evt);
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay);
					setTimeout(dijit.typematic.stop, 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay){
		// summary: Start listening for a specific typematic key and mouseclick.
		//	This is a thin wrapper to addKeyListener and addMouseListener.
		//	mouseNode: the DOM node object to listen on for mouse events.
		//	keyNode: the DOM node object to listen on for key events.
		//	See the addMouseListener and addKeyListener methods for other parameters.
		//	Returns an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Function that detects if we are in high-contrast mode or not,
		//		and sets up a timer to periodically confirm the value.
		//		figure out the background-image style property
		//		and apply that to the image.src property.
		// description:
		//		This must be a named function and not an anonymous
		//		function, so that the widget parsing code can make sure it
		//		registers its onload function after this function.
		//		DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = document.createElement("div");
		div.id = "a11yTestNode";
		div.style.cssText = 'border: 1px solid;'
			+ 'border-color:red green;'
			+ 'position: absolute;'
			+ 'height: 5px;'
			+ 'top: -999px;'
			+ 'background-image: url("' + dojo.moduleUrl("dijit", "form/templates/blank.gif") + '");';
		dojo.body().appendChild(div);

		// test it
		function check(){
			var cs = dojo.getComputedStyle(div);
			if(cs){
				var bkImg = cs.backgroundImage;
				var needsA11y = (cs.borderTopColor==cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
				dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			}
		}
		check();
		if(dojo.isIE){
			setInterval(check, 4000);
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit,
{
	hasWaiRole: function(/*Element*/ elem){
		// Summary: Return true if elem has a role attribute and false if not.
		if(elem.hasAttribute){
			return elem.hasAttribute("role");
		}else{
			return elem.getAttribute("role") ? true : false;
		}
	},

	getWaiRole: function(/*Element*/ elem){
		// Summary: Return the role of elem or an empty string if
		//		elem does not have a role.
		var value = elem.getAttribute("role");
		if(value){
			var prefixEnd = value.indexOf(":");
			return prefixEnd == -1 ? value : value.substring(prefixEnd+1);
		}else{
			return "";
		}
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// Summary: Set the role on elem. On Firefox 2 and below, "wairole:" is
		//		prepended to the provided role value.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttribute("role", "wairole:"+role);
		}else{
			elem.setAttribute("role", role);
		}
	},

	removeWaiRole: function(/*Element*/ elem){
		// Summary: Removes the role attribute from elem.
		elem.removeAttribute("role");
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return true if elem has a value for the given state and
		//		false if it does not.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.hasAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			if(elem.hasAttribute){
				return elem.hasAttribute("aria-"+state);
			}else{
				return elem.getAttribute("aria-"+state) ? true : false;
			}
		}
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Return the value of the requested state on elem
		//		or an empty string if elem has no value for state.
		//		On Firefox 2 and below, we check for an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we check for an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			return elem.getAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			var value =  elem.getAttribute("aria-"+state);
			return value ? value : "";
		}
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// Summary: Set state on elem to value.
		//		On Firefox 2 and below, we set an attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we set an attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.setAttributeNS("http://www.w3.org/2005/07/aaa",
				"aaa:"+state, value);
		}else{
			elem.setAttribute("aria-"+state, value);
		}
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// Summary: Removes the given state from elem.
		//		On Firefox 2 and below, we remove the attribute in namespace
		//		"http://www.w3.org/2005/07/aaa" with a name of the given state.
		//		On all other browsers, we remove the attribute called
		//		"aria-"+state.
		if(dojo.isFF && dojo.isFF < 3){
			elem.removeAttributeNS("http://www.w3.org/2005/07/aaa", state);
		}else{
			elem.removeAttribute("aria-"+state);
		}
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");












}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");



dojo.declare("dijit._Widget", null, {
	// summary:
	//		The foundation of dijit widgets. 	
	//
	// id: String
	//		a unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: String
	//	Language to display this widget in (like en-us).
	//	Defaults to brower's specified preferred language (typically the language of the OS)
	lang: "",

	// dir: String
	//  Bi-directional support, as defined by the HTML DIR attribute. Either left-to-right "ltr" or right-to-left "rtl".
	dir: "",

	// class: String
	// HTML class attribute
	"class": "",

	// style: String
	// HTML style attribute
	style: "",

	// title: String
	// HTML title attribute
	title: "",

	// srcNodeRef: DomNode
	//		pointer to original dom node
	srcNodeRef: null,

	// domNode: DomNode
	//		this is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPonit syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// attributeMap: Object
	//		A map of attributes and attachpoints -- typically standard HTML attributes -- to set
	//		on the widget's dom, at the "domNode" attach point, by default.
	//		Other node references can be specified as properties of 'this'
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},  // TODO: add on* handlers?

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(params, srcNodeRef){
		this.create(params, srcNodeRef);
	},

	create: function(params, srcNodeRef){
		// summary:
		//		To understand the process by which widgets are instantiated, it
		//		is critical to understand what other methods create calls and
		//		which of them you'll want to override. Of course, adventurous
		//		developers could override create entirely, but this should
		//		only be done as a last resort.
		//
		//		Below is a list of the methods that are called, in the order
		//		they are fired, along with notes about what they do and if/when
		//		you should over-ride them in your widget:
		//			
		//			postMixInProperties:
		//				a stub function that you can over-ride to modify
		//				variables that may have been naively assigned by
		//				mixInProperties
		//			# widget is added to manager object here
		//			buildRendering
		//				Subclasses use this method to handle all UI initialization
		//				Sets this.domNode.  Templated widgets do this automatically
		//				and otherwise it just uses the source dom node.
		//			postCreate
		//				a stub function that you can over-ride to modify take
		//				actions once the widget has been placed in the UI

		// store pointer to original dom tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects=[];

		// _attaches: String[]
		// 		names of all our dojoAttachPoint variables
		this._attaches=[];

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.
		if(!this.id){
			this.id=dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
		// The placement of these attributes is according to the property mapping in attributeMap.
		// Note special handling for 'style' and 'class' attributes which are lists and can
		// have elements from both old and new structures, and some attributes like "type"
		// cannot be processed this way as they are not mutable.
		if(this.domNode){
			for(var attr in this.attributeMap){
				var mapNode = this[this.attributeMap[attr] || "domNode"];
				var value = this[attr];
				if(typeof value != "object" && (value !== "" || (params && params[attr]))){
					switch(attr){
					case "class":
						dojo.addClass(mapNode, value);
						break;
					case "style":
						if(mapNode.style.cssText){
							mapNode.style.cssText += "; " + value;// FIXME: Opera
						}else{
							mapNode.style.cssText = value;
						}
						break;
					default:
						mapNode.setAttribute(attr, value);
					}
				}
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}	
	},

	postMixInProperties: function(){
		// summary
		//	Called after the parameters to the widget have been read-in,
		//	but before the widget template is instantiated.
		//	Especially useful to set properties that are referenced in the widget template.
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode.
		//		Most widgets will mixin TemplatedWidget, which overrides this method.
		this.domNode = this.srcNodeRef || dojo.doc.createElement('div');
	},

	postCreate: function(){
		// summary:
		//		Called after a widget's dom has been setup
	},

	startup: function(){
		// summary:
		//		Called after a widget's children, and other widgets on the page, have been created.
		//		Provides an opportunity to manipulate any children before they are displayed
		//		This is useful for composite widgets that need to control or layout sub-widgets
		//		Many layout widgets can use this as a wiring phase
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget and it's descendants. This is the generic
		// 		"destructor" function that all widget users should call to
		// 		cleanly discard with a widget. Once a widget is destroyed, it's
		// 		removed from the manager object.
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		this.destroyDescendants();
		this.destroy();
	},

	destroy: function(/*Boolean*/ finalize){
		// summary:
		// 		Destroy this widget, but not its descendants
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?
		this.uninitialize();
		dojo.forEach(this._connects, function(array){
			dojo.forEach(array, dojo.disconnect);
		});
		this.destroyRendering(finalize);
		dijit.registry.remove(this.id);
	},

	destroyRendering: function(/*Boolean*/ finalize){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// finalize: Boolean
		//		is this function being called part of global environment
		//		tear-down?

		if(this.bgIframe){
			this.bgIframe.destroy();
			delete this.bgIframe;
		}

		if(this.domNode){
			dojo._destroyElement(this.domNode);
			delete this.domNode;
		}

		if(this.srcNodeRef){
			dojo._destroyElement(this.srcNodeRef);
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.

		// TODO: should I destroy in the reverse order, to go bottom up?
		dojo.forEach(this.getDescendants(), function(widget){ widget.destroy(); });
	},

	uninitialize: function(){
		// summary:
		//		stub function. Over-ride to implement custom widget tear-down
		//		behavior.
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	toString: function(){
		// summary:
		//		returns a string that represents the widget. When a widget is
		//		cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversable
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//	return all the descendant widgets
		var list = dojo.query('[widgetId]', this.domNode);
		return list.map(dijit.byNode);		// Array
	},

	nodesWithKeyClick : ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String*/ event,
			/*String|Function*/ method){

		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		//		Special event: "ondijitclick" triggers on a click or enter-down or space-up
		//		Similar to dojo.connect() but takes three arguments rather than four.
		var handles =[];
		if(event == "ondijitclick"){
			var w = this;
			// add key based click activation for unsupported nodes.
			if(!this.nodesWithKeyClick[obj.nodeName]){
				handles.push(dojo.connect(obj, "onkeydown", this,
					function(e){
						if(e.keyCode == dojo.keys.ENTER){
							return (dojo.isString(method))?
								w[method](e) : method.call(w, e);
						}else if(e.keyCode == dojo.keys.SPACE){
							// stop space down as it causes IE to scroll
							// the browser window
							dojo.stopEvent(e);
						}
			 		}));
				handles.push(dojo.connect(obj, "onkeyup", this,
					function(e){
						if(e.keyCode == dojo.keys.SPACE){
							return dojo.isString(method) ?
								w[method](e) : method.call(w, e);
						}
			 		}));
			}
			event = "onclick";
		}
		handles.push(dojo.connect(obj, event, this, method));

		// return handles for FormElement and ComboBox
		this._connects.push(handles);
		return handles;
	},

	disconnect: function(/*Object*/ handles){
		// summary:
		//		Disconnects handle created by this.connect.
		//		Also removes handle from this widget's list of connects
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i]==handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Checks the DOM to for the text direction for bi-directional support
		// description:
		//		This method cannot be used during widget construction because the widget
		//		must first be connected to the DOM tree.  Parent nodes are searched for the
		//		'dir' attribute until one is found, otherwise left to right mode is assumed.
		//		See HTML spec, DIR attribute for more information.

		if(typeof this._ltr == "undefined"){
			this._ltr = dojo.getComputedStyle(this.domNode).direction != "rtl";
		}
		return this._ltr; //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	}
});

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Contained",
	null,
	{
		// summary
		//		Mixin for widgets that are children of a container widget

		getParent: function(){
			// summary:
			//		returns the parent widget of this widget, assuming the parent
			//		implements dijit._Container
			for(var p=this.domNode.parentNode; p; p=p.parentNode){
				var id = p.getAttribute && p.getAttribute("widgetId");
				if(id){
					var parent = dijit.byId(id);
					return parent.isContainer ? parent : null;
				}
			}
			return null;
		},

		_getSibling: function(which){
			var node = this.domNode;
			do{
				node = node[which+"Sibling"];
			}while(node && node.nodeType != 1);
			if(!node){ return null; } // null
			var id = node.getAttribute("widgetId");
			return dijit.byId(id);
		},

		getPreviousSibling: function(){
			// summary:
			//		returns null if this is the first child of the parent,
			//		otherwise returns the next element sibling to the "left".

			return this._getSibling("previous");
		},

		getNextSibling: function(){
			// summary:
			//		returns null if this is the last child of the parent,
			//		otherwise returns the next element sibling to the "right".

			return this._getSibling("next");
		}
	}
);

dojo.declare("dijit._Container",
	null,
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer

		isContainer: true,

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Process the given child widget, inserting it's dom node as
			//		a child of our dom node

			if(insertIndex === undefined){
				insertIndex = "last";
			}
			var refNode = this.containerNode || this.domNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = dojo.query("> [widgetid]", refNode);
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1]; insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget*/ widget){
			// summary:
			//		removes the passed widget instance from this widget but does
			//		not destroy it
			var node = widget.domNode;
			node.parentNode.removeChild(node);	// detach but don't destroy
		},

		_nextElement: function(node){
			do{
				node = node.nextSibling;
			}while(node && node.nodeType != 1);
			return node;
		},

		_firstElement: function(node){
			node = node.firstChild;
			if(node && node.nodeType != 1){
				node = this._nextElement(node);
			}
			return node;
		},

		getChildren: function(){
			// summary:
			//		Returns array of children widgets
			return dojo.query("> [widgetId]", this.containerNode || this.domNode).map(dijit.byNode); // Array
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children
			var cn = this.containerNode || this.domNode;
			return !!this._firstElement(cn); // Boolean
		},

		_getSiblingOfChild: function(/*Widget*/ child, /*int*/ dir){
			// summary:
			//		get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			var node = child.domNode;
			var which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node ? dijit.byNode(node) : null;
		}
	}
);

dojo.declare("dijit._KeyNavContainer",
	[dijit._Container],
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().

/*=====
		// focusedChild: Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*Array*/ prevKeyCodes, /*Array*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: Array
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: Array
			//		Key codes for navigating to the next child.

			var keyCodes = this._keyNavCodes = {};
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			if(dojo.isIE){
				this.connect(this.domNode, "onactivate", "_onContainerFocus");
				this.connect(this.domNode, "ondeactivate", "_onContainerBlur");
			}else{
				this.connect(this.domNode, "onfocus", "_onContainerFocus");
				this.connect(this.domNode, "onblur", "_onContainerBlur");
			}
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_setTabIndexMinusOne"));
		},

		addChild: function(/*Widget*/ widget, /*int?*/ insertIndex){
			// summary: Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._setTabIndexMinusOne(widget);
		},

		focus: function(){
			// summary: Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary: Focus the first focusable child in the container.
			this.focusChild(this._getFirstFocusableChild());
		},

		focusNext: function(){
			// summary: Focus the next widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasNextFocalNode
					&& this.focusedChild.hasNextFocalNode()){
				this.focusedChild.focusNext();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			if(child.getFocalNodes){
				this.focusChild(child, child.getFocalNodes()[0]);
			}else{
				this.focusChild(child);
			}
		},

		focusPrev: function(){
			// summary: Focus the previous widget or focal node (for widgets
			//		with multiple focal nodes) within this container.
			if(this.focusedChild && this.focusedChild.hasPrevFocalNode
					&& this.focusedChild.hasPrevFocalNode()){
				this.focusedChild.focusPrev();
				return;
			}
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			if(child.getFocalNodes){
				var nodes = child.getFocalNodes();
				this.focusChild(child, nodes[nodes.length-1]);
			}else{
				this.focusChild(child);
			}
		},

		focusChild: function(/*Widget*/ widget, /*Node?*/ node){
			// summary: Focus widget. Optionally focus 'node' within widget.
			if(widget){
				if(this.focusedChild && widget !== this.focusedChild){
					this._onChildBlur(this.focusedChild);
				}
				this.focusedChild = widget;
				if(node && widget.focusFocalNode){
					widget.focusFocalNode(node);
				}else{
					widget.focus();
				}
			}
		},

		_setTabIndexMinusOne: function(/*Widget*/ widget){
			if(widget.getFocalNodes){
				dojo.forEach(widget.getFocalNodes(), function(node){
					node.setAttribute("tabIndex", -1);
				});
			}else{
				(widget.focusNode || widget.domNode).setAttribute("tabIndex", -1);
			}
		},

		_onContainerFocus: function(evt){
			this.domNode.setAttribute("tabIndex", -1);
			if(evt.target === this.domNode){
				this.focusFirstChild();
			}else{
				var widget = dijit.getEnclosingWidget(evt.target);
				if(widget && widget.isFocusable()){
					this.focusedChild = widget;
				}
			}
		},

		_onContainerBlur: function(evt){
			if(this.tabIndex){
				this.domNode.setAttribute("tabIndex", this.tabIndex);
			}
		},

		_onContainerKeypress: function(evt){
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.keyCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
		},

		_getFirstFocusableChild: function(){
			return this._getNextFocusableChild(null, 1);
		},

		_getNextFocusableChild: function(child, dir){
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;
				}
				child = this._getSiblingOfChild(child, dir);
			}
		}
	}
);

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

dojo.string.pad = function(/*String*/text, /*int*/size, /*String?*/ch, /*boolean?*/end){
	// summary:
	//		Pad a string to guarantee that it is at least 'size' length by
	//		filling with the character 'c' at either the start or end of the
	//		string. Pads at the start, by default.
	// text: the string to pad
	// size: length to provide padding
	// ch: character to pad, defaults to '0'
	// end: adds padding at the end if true, otherwise pads at start

	var out = String(text);
	if(!ch){
		ch = '0';
	}
	while(out.length < size){
		if(end){
			out += ch;
		}else{
			out = ch + out;
		}
	}
	return out;	// String
};

dojo.string.substitute = function(	/*String*/template, 
									/*Object or Array*/map, 
									/*Function?*/transform, 
									/*Object?*/thisObject){
	// summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	// description:
	//		For example,
	//		|	dojo.string.substitute("File '${0}' is not found in directory '${1}'.",["foo.html","/temp"]);
	//		|	dojo.string.substitute("File '${name}' is not found in directory '${info.dir}'.",{name: "foo.html", info: {dir: "/temp"}});
	//		both return
	//			"File 'foo.html' is not found in directory '/temp'."
	// template: 
	//		a string with expressions in the form ${key} to be replaced or
	//		${key:format} which specifies a format function.  NOTE syntax has
	//		changed from %{key}
	// map: where to look for substitutions
	// transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. dojo.string.encodeXML
	// thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g, function(match, key, format){
		var value = dojo.getObject(key,false,map);
		if(format){ value = dojo.getObject(format,false,thisObject)(value);}
		if(transform){ value = transform(value, key); }
		return value.toString();
	}); // string
};

dojo.string.trim = function(/*String*/ str){
	// summary: trims whitespaces from both sides of the string
	// description:
	//	This version of trim() was taken from Steven Levithan's blog: 
	//	http://blog.stevenlevithan.com/archives/faster-trim-javascript.
	//	The short yet good-performing version of this function is 
	//	dojo.trim(), which is part of the base.
	str = str.replace(/^\s+/, '');
	for(var i = str.length - 1; i > 0; i--){
		if(/\S/.test(str.charAt(i))){
			str = str.substring(0, i + 1);
			break;
		}
	}
	return str;	// String
};

}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		RFC3339 (http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified by http://www.w3.org/TR/NOTE-datetime
	//		The following combinations are valid:
	// 			* dates only
	//				yyyy
	//				yyyy-MM
	//				yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//				THH:mm
	//				THH:mm:ss
	//				THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString);
	var result = null;

	if(match){
		match.shift();
		match[1] && match[1]--; // Javascript Date months are 0-based
		match[6] && (match[6] *= 1000); // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}).forEach(function(value, index){
				if(match[index] === undefined){
					match[index] = value;
				}
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0);

		var offset = 0;
		var zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
}

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*Object?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows RFC3339 (http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.
	//
	//	dateObject:
	//		A Date object
	//
	//	object {selector: string, zulu: boolean, milliseconds: boolean}
	//		selector- "date" or "time" for partial formatting of the Date object.
	//			Both date and time will be formatted by default.
	//		zulu- if true, UTC/GMT is used for a timezone
	//		milliseconds- if true, output milliseconds

	var _ = function(n){ return (n < 10) ? "0" + n : n; }
	options = options || {};
	var formattedDate = [];
	var getter = options.zulu ? "getUTC" : "get";
	var date = "";
	if(options.selector != "time"){
		date = [dateObject[getter+"FullYear"](), _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") + 
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
}

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");


dojo.parser = new function(){

	var d = dojo;

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".  interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }"). 
					//  Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value.search(/[^\w\.]+/i) != -1){
						// TODO: "this" here won't work
						value = d.parser._nameAnonFunc(new Function(value), this);
					}
					return d.getObject(value, false);
				}catch(e){ return new Function(); }
			case "array":
				return value.split(/\s*,\s*/);
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};
	
	function getClassInfo(/*String*/ className){
		// className:
		//		fully qualified name (like "dijit.Button")
		// returns:
		//		structure like
		//			{ 
		//				cls: dijit.Button, 
		//				params: { label: "string", disabled: "boolean"}
		//			}

		if(!instanceClasses[className]){
			// get pointer to widget class
			var cls = d.getObject(className);
			if(!d.isFunction(cls)){
				throw new Error("Could not load class '" + className +
					"'. Did you spell the name correctly and use a full path, like 'dijit.form.Button'?");
			}
			var proto = cls.prototype;
	
			// get table of parameter names & types
			var params={};
			for(var name in proto){
				if(name.charAt(0)=="_"){ continue; } 	// skip internal properties
				var defVal = proto[name];
				params[name]=val2type(defVal);
			}

			instanceClasses[className] = { cls: cls, params: params };
		}
		return instanceClasses[className];
	}

	this._functionFromScript = function(script){
		var preamble = "";
		var suffix = "";
		var argsStr = script.getAttribute("args");
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	}

	this.instantiate = function(/* Array */nodes){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a layout method to allow them to connect with
		//		any children		
		var thelist = [];
		d.forEach(nodes, function(node){
			if(!node){ return; }
			var type = node.getAttribute("dojoType");
			if((!type)||(!type.length)){ return; }
			var clsInfo = getClassInfo(type);
			var clazz = clsInfo.cls;
			var ps = clazz._noScript||clazz.prototype._noScript;

			// read parameters (ie, attributes).
			// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
			var params = {};
			var attributes = node.attributes;
			for(var name in clsInfo.params){
				var item = attributes.getNamedItem(name);
				if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
				var value = item.value;
				// Deal with IE quirks for 'class' and 'style'
				switch(name){
				case "class":
					value = node.className;
					break;
				case "style":
					value = node.style && node.style.cssText; // FIXME: Opera?
				}
				var _type = clsInfo.params[name];
				params[name] = str2obj(value, _type);
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			if(!ps){
				var connects = [],	// functions to connect after instantiation
					calls = [];		// functions to call after instantiation

				d.query("> script[type^='dojo/']", node).orphan().forEach(function(script){
					var event = script.getAttribute("event"),
						type = script.getAttribute("type"),
						nf = d.parser._functionFromScript(script);
					if(event){
						if(type == "dojo/connect"){
							connects.push({event: event, func: nf});
						}else{
							params[event] = nf;
						}
					}else{
						calls.push(nf);
					}
				});
			}

			var markupFactory = clazz["markupFactory"];
			if(!markupFactory && clazz["prototype"]){
				markupFactory = clazz.prototype["markupFactory"];
			}
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			var jsname = node.getAttribute("jsId");
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			if(!ps){
				dojo.forEach(connects, function(connect){
					dojo.connect(instance, connect.event, null, connect.func);
				});
				dojo.forEach(calls, function(func){
					func.call(instance);
				});
			}
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		d.forEach(thelist, function(instance){
			if(	instance  && 
				(instance.startup) && 
				((!instance.getParent) || (!instance.getParent()))
			){
				instance.startup();
			}
		});
		return thelist;
	};

	this.parse = function(/*DomNode?*/ rootNode){
		// summary:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them Searches for
		//		dojoType="qualified.class.name"
		var list = d.query('[dojoType]', rootNode);
		// go build the object instances
		var instances = this.instantiate(list);
		return instances;
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){ 
		if(djConfig["parseOnLoad"] == true){
			dojo.parser.parse(); 
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.exists("dijit.wai.onload") && (dijit.wai.onload === dojo._loaders[0])){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

//TODO: ported from 0.4.x Dojo.  Can we reduce this?
dojo.parser._anonCtr = 0;
dojo.parser._anon = {}; // why is this property required?
dojo.parser._nameAnonFunc = function(/*Function*/anonFuncPtr, /*Object*/thisObj){
	// summary:
	//		Creates a reference to anonFuncPtr in thisObj with a completely
	//		unique name. The new name is returned as a String. 
	var jpn = "$joinpoint";
	var nso = (thisObj|| dojo.parser._anon);
	if(dojo.isIE){
		var cn = anonFuncPtr["__dojoNameCache"];
		if(cn && nso[cn] === anonFuncPtr){
			return anonFuncPtr["__dojoNameCache"];
		}
	}
	var ret = "__"+dojo.parser._anonCtr++;
	while(typeof nso[ret] != "undefined"){
		ret = "__"+dojo.parser._anonCtr++;
	}
	nso[ret] = anonFuncPtr;
	return ret; // String
}

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		mixin for widgets that are instantiated from a template

		// templateNode: DomNode
		//		a node that represents the widget template. Pre-empts both templateString and templatePath.
		templateNode: null,

		// templateString String:
		//		a string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		templateString: null,

		// templatePath: String
		//	Path to template (HTML file) for this widget
		templatePath: null,

		// widgetsInTemplate Boolean:
		//		should we parse the template to find widgets that might be
		//		declared in markup inside it? false by default.
		widgetsInTemplate: false,

		// containerNode DomNode:
		//		holds child elements. "containerNode" is generally set via a
		//		dojoAttachPoint assignment and it designates where children of
		//		the src dom node will be placed
		containerNode: null,

		// skipNodeCache Boolean:
		//		if using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				var className = this.declaredClass, _this = this;
				// Cache contains a string because we need to do property replacement
				// do the property replacement
				var tstr = dojo.string.substitute(cached, this, function(value, key){
					if(key.charAt(0) == '!'){ value = _this[key.substr(1)]; }
					if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
					if(!value){ return ""; }

					// Substitution keys beginning with ! will skip the transform step,
					// in case a user wishes to insert unescaped markup, e.g. ${!foo}
					return key.charAt(0) == "!" ? value :
						// Safer substitution, see heading "Attribute values" in
						// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
						value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
				}, this);

				node = dijit._Templated._createNodesFromText(tstr)[0];
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			// recurse through the node, looking for, and attaching to, our
			// attachment points which should be defined on the template node.
			this._attachTemplateNodes(node);

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(node, source);
			}

			this.domNode = node;
			if(this.widgetsInTemplate){
				var childWidgets = dojo.parser.parse(node);
				this._attachTemplateNodes(childWidgets, function(n,p){
					return n[p];
				});
			}

			this._fillContent(source);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		relocate source contents to templated container node
			//		this.containerNode must be able to receive children, or exceptions will be thrown
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent	
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x=dojo.isArray(rootNode)?0:-1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode,'dojoType')){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while(point=points.shift()){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while(event=events.shift()){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}

			}
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String
	//		the URL to get the template from. dojo.uri.Uri is often passed as well.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL
	// Returns:
	//	Either string (if there are ${} variables that need to be replaced) or just
	//	a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		return cached;
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dijit._Templated._sanitizeTemplateString(dojo._getText(templatePath));
	}

	templateString = dojo.string.trim(templateString);

	if(templateString.match(/\$\{([^\}]+)\}/g) || alwaysUseString){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		return (tmplts[key] = dijit._Templated._createNodesFromText(templateString)[0]); //Node
	}
};

dijit._Templated._sanitizeTemplateString = function(/*String*/tString){
	// summary: 
	//		Strips <?xml ...?> declarations so that external SVG and XML
	// 		documents can be added to a document without worry. Also, if the string
	//		is an HTML document, only the part inside the body tag is returned.
	if(tString){
		tString = tString.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
		var matches = tString.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
		if(matches){
			tString = matches[1];
		}
	}else{
		tString = "";
	}
	return tString; //String
};


if(dojo.isIE){
	dojo.addOnUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(!isNaN(value.nodeType)){ // isNode equivalent
				dojo._destroyElement(value);
			}
			delete cache[key];
		}
	});
}

(function(){
	var tagMap = {
		cell: {re: /^<t[dh][\s\r\n>]/i, pre: "<table><tbody><tr>", post: "</tr></tbody></table>"},
		row: {re: /^<tr[\s\r\n>]/i, pre: "<table><tbody>", post: "</tbody></table>"},
		section: {re: /^<(thead|tbody|tfoot)[\s\r\n>]/i, pre: "<table>", post: "</table>"}
	};

	// dummy container node used temporarily to hold nodes being created
	var tn;

	dijit._Templated._createNodesFromText = function(/*String*/text){
		// summary:
		//	Attempts to create a set of nodes based on the structure of the passed text.

		if(!tn){
			tn = dojo.doc.createElement("div");
			tn.style.display="none";
			dojo.body().appendChild(tn);
		}
		var tableType = "none";
		var rtext = text.replace(/^\s+/, "");
		for(var type in tagMap){
			var map = tagMap[type];
			if(map.re.test(rtext)){
				tableType = type;
				text = map.pre + text + map.post;
				break;
			}
		}

		tn.innerHTML = text;
		if(tn.normalize){
			tn.normalize();
		}

		var tag = { cell: "tr", row: "tbody", section: "table" }[tableType];
		var _parent = (typeof tag != "undefined") ?
						tn.getElementsByTagName(tag)[0] :
						tn;

		var nodes = [];
		while(_parent.firstChild){
			nodes.push(_parent.removeChild(_parent.firstChild));
		}
		tn.innerHTML="";
		return nodes;	//	Array
	}
})();

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
})

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");




dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated],
{
	/*
	Summary:
		FormElement widgets correspond to native HTML elements such as <input> or <button> or <select>.
		Each FormElement represents a single input value, and has a (possibly hidden) <input> element,
		to which it serializes its input value, so that form submission (either normal submission or via FormBind?)
		works as expected.

		All these widgets should have these attributes just like native HTML input elements.
		You can set them during widget construction, but after that they are read only.

		They also share some common methods.
	*/

	// baseClass: String
	//		Root CSS class of the widget (ex: dijitTextBox), used to add CSS classes of widget
	//		(ex: "dijitTextBox dijitTextBoxInvalid dijitTextBoxFocused dijitTextBoxInvalidFocused")
	//		See _setStateClass().
	baseClass: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// id: String
	//		Corresponds to the native HTML <input> element's attribute.
	//		Also becomes the id for the widget.
	id: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically 
	// require the 'name' attribute at element creation time.
	attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
		{id:"focusNode", tabIndex:"focusNode", alt:"focusNode"}),

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Set disabled state of widget.

		this.domNode.disabled = this.disabled = disabled;
		if(this.focusNode){
			this.focusNode.disabled = disabled;
		}
		if(disabled){
			//reset those, because after the domNode is disabled, we can no longer receive
			//mouse related events, see #4200
			this._hovering = false;
			this._active = false;
		}
		dijit.setWaiState(this.focusNode || this.domNode, "disabled", disabled);
		this._setStateClass();
	},


	_onMouse : function(/*Event*/ event){
		// summary:
		//	Sets _hovering, _active, and stateModifier properties depending on mouse state,
		//	then calls setStateClass() to set appropriate CSS classes for this.domNode.
		//
		//	To get a different CSS class for hover, send onmouseover and onmouseout events to this method.
		//	To get a different CSS class while mouse button is depressed, send onmousedown to this method.

		var mouseNode = event.target;
		if(mouseNode && mouseNode.getAttribute){
			this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
		}

		if(!this.disabled){
			switch(event.type){
				case "mouseenter" :	
				case "mouseover" :
					this._hovering = true;
					break;

				case "mouseout" :	
				case "mouseleave" :	
					this._hovering = false;
					break;

				case "mousedown" :
					this._active = true;
					// set a global event to handle mouseup, so it fires properly
					//	even if the cursor leaves the button
					var self = this;
					// #2685: use this.connect and disconnect so destroy works properly
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						self._active = false;
						self._setStateClass();
						self.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	isFocusable: function(){
		return !this.disabled && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		dijit.focus(this.focusNode);
	},

	_setStateClass: function(){
		// summary
		//	Update the visual state of the widget by setting the css classes on this.domNode
		//  (or this.stateNode if defined) by combining this.baseClass with
		//	various suffixes that represent the current widget state(s).
		//
		//	In the case where a widget has multiple
		//	states, it sets the class based on all possible
		//  combinations.  For example, an invalid form widget that is being hovered
		//	will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//	For complex widgets with multiple regions, there can be various hover/active states,
		//	such as "Hover" or "CloseButtonHover" (for tab buttons).
		//	This is controlled by a stateModifier="CloseButton" attribute on the close button node.
		//
		//	The widget may have one or more of the following states, determined
		//	by this.state, this.checked, this.valid, and this.selected:
		//		Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//		Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//		Selected - ex: currently selected tab will have this.selected==true
		//
		//	In addition, it may have at most one of the following states,
		//	based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//		Disabled	- if the widget is disabled
		//		Active		- if the mouse (or space/enter key?) is being pressed down
		//		Focused		- if the widget has focus
		//		Hover		- if the mouse is over the widget
		//
		//	(even if multiple af the above conditions are true we only pick the first matching one)


		// Get original (non state related, non baseClass related) class specified in template
		if(!("staticClass" in this)){
			this.staticClass = (this.stateNode||this.domNode).className;
		}

		// Compute new set of classes
		var classes = [ this.baseClass ];

		function multiply(modifier){
			classes=classes.concat(dojo.map(classes, function(c){ return c+modifier; }));
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		// Only one of these four can be applied.
		// Active trumps Focused, Focused trumps Hover, and Disabled trumps all.
		if(this.disabled){
			multiply("Disabled");
		}else if(this._active){
			multiply(this.stateModifier+"Active");
		}else{
			if(this._focused){
				multiply("Focused");
			}
			if((this.stateModifier || !this._focused) && this._hovering){
				multiply(this.stateModifier+"Hover");
			}
		}
		(this.stateNode || this.domNode).className = this.staticClass + " " + classes.join(" ");
	},

	onChange: function(newValue){
		// summary: callback when value is changed
	},

	postCreate: function(){
		this.setValue(this.value, null); // null reserved for initial value
		this.setDisabled(this.disabled);
		this._setStateClass();
	},

	setValue: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary: set the value of the widget.
		this._lastValue = newValue;
		dijit.setWaiState(this.focusNode || this.domNode, "valuenow", this.forWaiValuenow());
		if(priorityChange === undefined){ priorityChange = true; } // setValue with value only should fire onChange
		if(this._lastValueReported == undefined && priorityChange === null){ // don't report the initial value
			this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange) && 
			((newValue && newValue.toString)?newValue.toString():newValue) !== ((this._lastValueReported && this._lastValueReported.toString)?this._lastValueReported.toString():this._lastValueReported)){
			this._lastValueReported = newValue;
			this.onChange(newValue);
		}
	},

	getValue: function(){
		// summary: get the value of the widget.
		return this._lastValue;
	},

	undo: function(){
		// summary: restore the value to the last value passed to onChange
		this.setValue(this._lastValueReported, false);
	},

	_onKeyPress: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !e.shiftKey && !e.ctrlKey && !e.altKey){
			var v = this.getValue();
			var lv = this._lastValueReported;
			// Equality comparison of objects such as dates are done by reference so
			// two distinct objects are != even if they have the same data. So use
			// toStrings in case the values are objects.
			if((typeof lv != "undefined") && ((v!==null && v.toString)?v.toString():null) !== lv.toString()){	
				this.undo();
				dojo.stopEvent(e);
				return false;
			}
		}
		return true;
	},

	forWaiValuenow: function(){
		// summary: returns a value, reflecting the current state of the widget,
		//		to be used for the ARIA valuenow.
		// 		This method may be overridden by subclasses that want
		// 		to use something other than this.getValue() for valuenow
		return this.getValue();
	}
});

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");




dojo.declare("dijit.form.Button", dijit.form._FormWidget, {
/*
 * usage
 *	<button dojoType="button" onClick="...">Hello world</button>
 *
 *  var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
 *	dojo.body().appendChild(button1.domNode);
 */
	// summary
	//	Basically the same thing as a normal HTML button, but with special styling.

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \r\n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \r\n\t\t\t></span\r\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\r\n\t\t></button\r\n\t></div\r\n></div>\r\n",

	// TODO: set button's title to this.containerNode.innerText

	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel == false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
		}
		this.inherited(arguments);
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel == false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	}		
});

/*
 * usage
 *	<button dojoType="DropDownButton" label="Hello world"><div dojotype=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
 *	dojo.body().appendChild(button1);
 */
dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container], {
	// summary
	//		push the button and a menu shows up

	baseClass : "dijitDropDownButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline\"\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse,onclick:_onDropDownClick,onkeydown:_onDropDownKeydown,onblur:_onDropDownBlur,onkeypress:_onKey\"\r\n\t><div class='dijitRight'>\r\n\t<button class=\"dijitStretch dijitButtonNode dijitButtonContents\" type=\"${type}\"\r\n\t\tdojoAttachPoint=\"focusNode,titleNode\" waiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\r\n\t\t><div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div\r\n\t\t><span class=\"dijitButtonText\" \tdojoAttachPoint=\"containerNode,popupStateNode\"\r\n\t\tid=\"${id}_label\">${label}</span\r\n\t\t><span class='dijitA11yDownArrow'>&#9660;</span>\r\n\t</button>\r\n</div></div>\r\n",

	_fillContent: function(){
		// my inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.
		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dojo.body().appendChild(this.dropDown.domNode);
		this.dropDown.domNode.style.display="none";
	},

	_onArrowClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks on menu popup node
		if(this.disabled){ return; }
		this._toggleDropDown();
	},

	_onDropDownClick: function(/*Event*/ e){
		// on Firefox 2 on the Mac it is possible to fire onclick
		// by pressing enter down on a second element and transferring
		// focus to the DropDownButton;
		// we want to prevent opening our menu in this situation
		// and only do so if we have seen a keydown on this button;
		// e.detail != 0 means that we were fired by mouse
		var isMacFFlessThan3 = dojo.isFF && dojo.isFF < 3
			&& navigator.appVersion.indexOf("Macintosh") != -1;
		if(!isMacFFlessThan3 || e.detail != 0 || this._seenKeydown){
			this._onArrowClick(e);
		}
		this._seenKeydown = false;
	},

	_onDropDownKeydown: function(/*Event*/ e){
		this._seenKeydown = true;
	},

	_onDropDownBlur: function(/*Event*/ e){
		this._seenKeydown = false;
	},

	_onKey: function(/*Event*/ e){
		// summary: callback when the user presses a key on menu popup node
		if(this.disabled){ return; }
		if(e.keyCode == dojo.keys.DOWN_ARROW){
			if(!this.dropDown || this.dropDown.domNode.style.display=="none"){
				dojo.stopEvent(e);
				return this._toggleDropDown();
			}
		}
	},

	_onBlur: function(){
		// summary: called magically when focus has shifted away from this widget and it's dropdown
		this._closeDropDown();
		// don't focus on button.  the user has explicitly focused on something else.
	},

	_toggleDropDown: function(){
		// summary: toggle the drop-down widget; if it is up, close it, if not, open it
		if(this.disabled){ return; }
		dijit.focus(this.popupStateNode);
		var dropDown = this.dropDown;
		if(!dropDown){ return false; }
		if(!dropDown.isShowingNow){
			// If there's an href, then load that first, so we don't get a flicker
			if(dropDown.href && !dropDown.isLoaded){
				var self = this;
				var handler = dojo.connect(dropDown, "onLoad", function(){
					dojo.disconnect(handler);
					self._openDropDown();
				});
				dropDown._loadCheck(true);
				return;
			}else{
				this._openDropDown();
			}
		}else{
			this._closeDropDown();
		}
	},

	_openDropDown: function(){
		var dropDown = this.dropDown;
		var oldWidth=dropDown.domNode.style.width;
		var self = this;

		dijit.popup.open({
			parent: this,
			popup: dropDown,
			around: this.domNode,
			orient: this.isLeftToRight() ? {'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'}
				: {'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'},
			onExecute: function(){
				self._closeDropDown(true);
			},
			onCancel: function(){
				self._closeDropDown(true);
			},
			onClose: function(){
				dropDown.domNode.style.width = oldWidth;
				self.popupStateNode.removeAttribute("popupActive");
				this._opened = false;
			}
		});
		if(this.domNode.offsetWidth > dropDown.domNode.offsetWidth){
			var adjustNode = null;
			if(!this.isLeftToRight()){
				adjustNode = dropDown.domNode.parentNode;
				var oldRight = adjustNode.offsetLeft + adjustNode.offsetWidth;
			}
			// make menu at least as wide as the button
			dojo.marginBox(dropDown.domNode, {w: this.domNode.offsetWidth});
			if(adjustNode){
				adjustNode.style.left = oldRight - this.domNode.offsetWidth + "px";
			}
		}
		this.popupStateNode.setAttribute("popupActive", "true");
		this._opened=true;
		if(dropDown.focus){
			dropDown.focus();
		}
		// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
	},
	
	_closeDropDown: function(/*Boolean*/ focus){
		if(this._opened){
			dijit.popup.close(this.dropDown);
			if(focus){ this.focus(); }
			this._opened = false;			
		}
	}
});

/*
 * usage
 *	<button dojoType="ComboButton" onClick="..."><span>Hello world</span><div dojoType=dijit.Menu>...</div></button>
 *
 *  var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
 *	dojo.body().appendChild(button1.domNode);
 */
dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary
	//		left side is normal button, right side displays menu
	templateString:"<table class='dijit dijitReset dijitInline dijitLeft'\r\n\tcellspacing='0' cellpadding='0'\r\n\tdojoAttachEvent=\"onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\">\r\n\t<tr>\r\n\t\t<td\tclass=\"dijitStretch dijitButtonContents dijitButtonNode\"\r\n\t\t\ttabIndex=\"${tabIndex}\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"  dojoAttachPoint=\"titleNode\"\r\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\">\r\n\t\t\t<div class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\"></div>\r\n\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t</td>\r\n\t\t<td class='dijitReset dijitRight dijitButtonNode dijitDownArrowButton'\r\n\t\t\tdojoAttachPoint=\"popupStateNode,focusNode\"\r\n\t\t\tdojoAttachEvent=\"ondijitclick:_onArrowClick, onkeypress:_onKey\"\r\n\t\t\tstateModifier=\"DownArrow\"\r\n\t\t\ttitle=\"${optionsTitle}\" name=\"${name}\"\r\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\r\n\t\t><div waiRole=\"presentation\">&#9660;</div>\r\n\t</td></tr>\r\n</table>\r\n",

	attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
		{id:"", name:""}),

	// optionsTitle: String
	//  text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	_focusedNode: null,

	postCreate: function(){
		this.inherited(arguments);
		this._focalNodes = [this.titleNode, this.popupStateNode];
		dojo.forEach(this._focalNodes, dojo.hitch(this, function(node){
			if(dojo.isIE){
				this.connect(node, "onactivate", this._onNodeFocus);
			}else{
				this.connect(node, "onfocus", this._onNodeFocus);
			}
		}));
	},

	focusFocalNode: function(node){
		// summary: Focus the focal node node.
		this._focusedNode = node;
		dijit.focus(node);
	},

	hasNextFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node following the focused one.
		//		False is returned if the last node has focus.
		return this._focusedNode !== this.getFocalNodes()[1];
	},

	focusNext: function(){
		// summary: Focus the focal node following the current node with focus
		//		or the first one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 1 : 0];
		dijit.focus(this._focusedNode);
	},

	hasPrevFocalNode: function(){
		// summary: Returns true if this widget has no node currently
		//		focused or if there is a node before the focused one.
		//		False is returned if the first node has focus.
		return this._focusedNode !== this.getFocalNodes()[0];
	},

	focusPrev: function(){
		// summary: Focus the focal node before the current node with focus
		//		or the last one if no node currently has focus.
		this._focusedNode = this.getFocalNodes()[this._focusedNode ? 0 : 1];
		dijit.focus(this._focusedNode);
	},

	getFocalNodes: function(){
		// summary: Returns an array of focal nodes for this widget.
		return this._focalNodes;
	},

	_onNodeFocus: function(evt){
		this._focusedNode = evt.currentTarget;
	},

	_onBlur: function(evt){
		this.inherited(arguments);
		this._focusedNode = null;
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary
	//	A button that can be in two states (checked or not).
	//	Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	_clicked: function(/*Event*/ evt){
		this.setChecked(!this.checked);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary
		//	Programatically deselect the button
		this.checked = checked;
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", this.checked);
		this._setStateClass();		
		this.onChange(checked);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.Breadcrumb"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.Breadcrumb");
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbBar");
dojo.provide("ibm.tivoli.tip.dijit.BreadcrumbSeparator");






dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbBar",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
		'</div>',

	tabIndex: "0",

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	},
	
	destroy: function () {
//		this.destroyDescendants();
//		this.inherited (arguments);
	}
}
);

dojo.declare(
	"ibm.tivoli.tip.dijit.BreadcrumbSeparator",
	[ dijit._Widget, dijit._Templated ],
{
	// summary
	//	A line between two menu items
	templateString: '<div class="breadcrumbDivider dijitInline">&gt;</div>',
	postCreate: function(){ dojo.setSelectable(this.domNode, false); },
	isFocusable: function(){ return false; }
});

dojo.declare(
	"ibm.tivoli.tip.dijit.Breadcrumb",
	[ dijit.form.Button ],
{
	templateString:"<div class=\"dijit dijitLeft dijitInline breadcrumb\"\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",

	constructor: function () {
	},
	postCreate: function() {
		this.inherited (arguments);
	},
	destroy: function () {
//		this.destroyDescendants();
//		this.inherited (arguments);
	}
});


}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");





dojo.declare(
	"dijit.Menu",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	constructor: function() {
		this._bindings = [];
	},

	templateString:
			'<table class="dijit dijitMenu dijitReset dijitMenuTable" waiRole="menu" dojoAttachEvent="onkeypress:_onKeyPress">' +
				'<tbody class="dijitReset" dojoAttachPoint="containerNode"></tbody>'+
			'</table>',

	// targetNodeIds: String[]
	//	Array of dom node ids of nodes to attach to.
	//	Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: Boolean
	//	if true, right clicking anywhere on the window will cause this context menu to open;
	//	if false, must specify targetNodeIds
	contextMenuForWindow: false,

	// parentMenu: Widget
	// pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//	number of milliseconds before hovering (without clicking) causes the popup to automatically open
	popupDelay: 500,

	// _contextMenuWithMouse: Boolean
	//	used to record mouse and keyboard events to determine if a context
	//	menu is being opened with the keyboard or the mouse
	_contextMenuWithMouse: false,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		this.connectKeyNavHandlers([dojo.keys.UP_ARROW], [dojo.keys.DOWN_ARROW]);
	},

	startup: function(){
		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();
	},

	onExecute: function(){
		// summary: attach point for notification about when a menu item has been executed
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary: attach point for notification about when the user cancels the current menu
	},

	_moveToPopup: function(/*Event*/ evt){
		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary
		//	Handle keyboard based menu navigation.
		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.keyCode){
			case dojo.keys.RIGHT_ARROW:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case dojo.keys.LEFT_ARROW:
				if(this.parentMenu){
					this.onCancel(false);
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		this.focusChild(item);

		if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
			this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
		}
	},

	_onChildBlur: function(item){
		// Close all popups that are open and descendants of this menu
		dijit.popup.close(item.popup);
		item._blur();
		this._stopPopupTimer();
	},

	onItemUnhover: function(/*MenuItem*/ item){
	},

	_stopPopupTimer: function(){
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_getTopMenu: function(){
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*Widget*/ item){
		// summary: user defined function to handle clicks on an item
		// summary: internal function for clicks
		if(item.disabled){ return false; }

		if(item.popup){
			if(!this.is_open){
				this._openPopup();
			}
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick();
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el) {
		//	summary
		//	returns the window reference of the passed iframe
		var win = dijit.getDocumentWindow(dijit.Menu._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			dijit.Menu._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && document.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		//	summary
		//	returns a reference to the document object inside iframe_el
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && document.frames[iframe_el.name] && document.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
			dojo.connect(cn, "oncontextmenu", this, "_openMyself"),
			dojo.connect(cn, "onkeydown", this, "_contextKey"),
			dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},

	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
				window.setTimeout(dojo.hitch(this, function(){ this._openMyself(_e); }), 1);
			}
		}
	},

	_contextMouse: function(e){
		this._contextMenuWithMouse = true;
	},

	_openMyself: function(/*Event*/ e){
		// summary:
		//		Internal function for opening myself when the user
		//		does a right-click or something similar

		dojo.stopEvent(e);

		// Get coordinates.
		// if we are opening the menu with the mouse or on safari open
		// the menu at the mouse cursor
		// (Safari does not have a keyboard command to open the context menu
		// and we don't currently have a reliable way to determine
		// _contextMenuWithMouse on Safari)
		var x,y;
		if(dojo.isSafari || this._contextMenuWithMouse){
			x=e.pageX;
			y=e.pageY;
		}else{
			// otherwise open near e.target
			var coords = dojo.coords(e.target, true);
			x = coords.x + 10;
			y = coords.y + 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		}
	},

	onOpen: function(/*Event*/ e){
		// summary
		//		Open menu relative to the mouse
		this.isShowingNow = true;
	},

	onClose: function(){
		// summary: callback when this menu is closed
		this._stopPopupTimer();
		this.parentMenu = null;
		this.isShowingNow = false;
		this.currentPopup = null;
		if(this.focusedChild){
			this._onChildBlur(this.focusedChild);
			this.focusedChild = null;
		}
	},

	_openPopup: function(){
		// summary: open the popup to the side of the current menu item
		this._stopPopupTimer();
		var from_item = this.focusedChild;
		var popup = from_item.popup;

		if(popup.isShowingNow){ return; }
		popup.parentMenu = this;
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.arrowCell,
			orient: this.isLeftToRight() ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'},
			onCancel: function(){
				// called when the child menu is canceled
				dijit.popup.close(popup);
				from_item.focus();	// put focus back on my node
				self.currentPopup = null;
			}
		});

		this.currentPopup = popup;

		if(popup.focus){
			popup.focus();
		}
	}
}
);

dojo.declare(
	"dijit.MenuItem",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line item in a Menu2

	// Make 3 columns
	//   icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
	templateString:
		 '<tr class="dijitReset dijitMenuItem"'
		+'dojoAttachEvent="onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick">'
		+'<td class="dijitReset"><div class="dijitMenuItemIcon ${iconClass}" dojoAttachPoint="iconNode" ></div></td>'
		+'<td tabIndex="-1" class="dijitReset dijitMenuItemLabel" dojoAttachPoint="containerNode" waiRole="menuitem"></td>'
		+'<td class="dijitReset" dojoAttachPoint="arrowCell">'
			+'<div class="dijitMenuExpand" dojoAttachPoint="expand" style="display:none">'
			+'<span class="dijitInline dijitArrowNode dijitMenuExpandInner">+</span>'
			+'</div>'
		+'</td>'
		+'</tr>',

	// label: String
	//	menu text
	label: '',

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	// disabled: Boolean
	//  if true, the menu item is disabled
	//  if false, the menu item is enabled
	disabled: false,

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
		this.setDisabled(this.disabled);
		if(this.label){
			this.containerNode.innerHTML=this.label;
		}
	},

	_onHover: function(){
		// summary: callback when mouse is moved onto menu item
		this.getParent().onItemHover(this);
	},

	_onUnhover: function(){
		// summary: callback when mouse is moved off of menu item
		// if we are unhovering the currently selected item
		// then unselect it
		this.getParent().onItemUnhover(this);
	},

	_onClick: function(evt){
		this.getParent().onItemClick(this);
		dojo.stopEvent(evt);
	},

	onClick: function() {
		// summary
		//	User defined function to handle clicks
	},

	focus: function(){
		dojo.addClass(this.domNode, 'dijitMenuItemHover');
		try{
			dijit.focus(this.containerNode);
		}catch(e){
			// this throws on IE (at least) in some scenarios
		}
	},

	_blur: function(){
		dojo.removeClass(this.domNode, 'dijitMenuItemHover');
	},

	setDisabled: function(/*Boolean*/ value){
		// summary: enable or disable this menu item
		this.disabled = value;
		dojo[value ? "addClass" : "removeClass"](this.domNode, 'dijitMenuItemDisabled');
		dijit.setWaiState(this.containerNode, 'disabled', value ? 'true' : 'false');
	}
});

dojo.declare(
	"dijit.PopupMenuItem",
	dijit.MenuItem,
{
	_fillContent: function(){
		// my inner HTML contains both the menu item text and a popup widget, like
		// <div dojoType="dijit.PopupMenuItem">
		//		<span>pick me</span>
		//		<popup> ... </popup>
		// </div>
		// the first part holds the menu item text and the second part is the popup
		if(this.srcNodeRef){
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
		// land now.  move it to document.body.
		if(!this.popup){
			var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.popup = dijit.byNode(node);
		}
		dojo.body().appendChild(this.popup.domNode);

		this.popup.domNode.style.display="none";
		dojo.addClass(this.expand, "dijitMenuExpandEnabled");
		dojo.style(this.expand, "display", "");
		dijit.setWaiState(this.containerNode, "haspopup", "true");
	}
});

dojo.declare(
	"dijit.MenuSeparator",
	[dijit._Widget, dijit._Templated, dijit._Contained],
{
	// summary
	//	A line between two menu items

	templateString: '<tr class="dijitMenuSeparator"><td colspan=3>'
			+'<div class="dijitMenuSeparatorTop"></div>'
			+'<div class="dijitMenuSeparatorBottom"></div>'
			+'</td></tr>',

	postCreate: function(){
		dojo.setSelectable(this.domNode, false);
	},
	
	isFocusable: function(){
		// summary:
		//		over ride to always return false
		return false;
	}
});

}

/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.require( "dijit.Menu" );

dojo.provide( "ibm.tivoli.tip.dijit.ContextMenuAdapter" );
/*******************************************************************************
* ContextMenuAdapter is responsible for constructing and displaying dynamic
* context menus.  That is it will ask for the items that should be displayed
* each time it is told to present the context menu. 
*
* It can handle calling a JSON Url to retreive the items for you.
*
* It also has a method that can be overloaded to provide context to the code
* building the menu. (For instance -- the row of a table that is selected)
*
* The menu model is as follows:
* 
* [ { 
*     id: "uniqueID", 
*     label: "label", 
*     iconClass: "iconClass", 
*     submenu_items: [ {},{} ] 
*     actionID: "actionToFire" 
    } 
* ... ]
*
* id, iconClass, and submenu_items are all optional
*
* NOTE:
* ! IF "_separator_" is passed as actionID then a menu separator is created !
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dijit.ContextMenuAdapter", null,
{
   /* [Optional] The JSON Url to request context menu items from */
   url: null,
   
   /* [Optional] The unique name of the context menu -- for lookup of items */
   name: null,
   
   /****************************************************************************
   * The constructor of the adapter.  Mixes in parameters.
   ****************************************************************************/
   constructor: function( args )
   {
     dojo.mixin( this, args );
     
     this.menuItems = [];
     this.subMenus = [];
   },
      
   /****************************************************************************
   * Ensures that only one context menu is used during the life of this adapter.
   * It destroys any previously created menus and recreates a new menu.
   ****************************************************************************/
   createContextMenu: function()
   {
     this._menu = new dijit.Menu();
     
     this._onCloseListener = dojo.connect( this._menu, "onClose", this, "_destroyMenu" );
     
     //populate the menu
     this.populateMenu( this._menu ); 
     
     return this._menu;         
   },
   
   /****************************************************************************
   * When the menu closes -- cleanly destroy it.  There is a bug in Dojo that
   * requires us to find all orphaned pieces of the menu to cleanly destroy
   ****************************************************************************/   
   _destroyMenu: function()
   {
     dojo.disconnect( this._onCloseListener );
     
     delete this._onCloseListener;

	 for ( var i in this.menuItems )
	 {
	   if (i) {
	 	this.menuItems[ i ].destroy();
	   }
	 }
     
	 for ( i in this.subMenus )
	 {
	   if (i) {
	 	this.subMenus[ i ].destroy();
	   }
	 }

     this._menu.destroyRecursive();

     dojo.query( ".dijitPopup" ).forEach( function( domNode ) { dojo._destroyElement( domNode ); } );       
     
     delete this._menu;            
   },
   
   /****************************************************************************
   * This method must be called with a mouse event to show the Context menu.
   * You could pass in an object with x:xPos and y:yPos for event to trick the 
   * context menu to open at any given point on screen.
   *
   * The context object is any object that you want to send back to the server
   * or pass to the getMenuItems method.
   ****************************************************************************/
   contextMenuOpen: function ( event, context )
   {
      this._context = context;
      
      var menu = this.createContextMenu();

      menu._contextMenuWithMouse = true;
      menu._openMyself( event ); 
   },
   
   /****************************************************************************
   * Asks the JSON URL or callback method for the menu items then recursively
   * builds the menu.
   ****************************************************************************/   
   populateMenu: function( menu )
   {
     var model = null;
     
     if ( this.url !== null )
     { 
       model = this._loadMenuItems(); 
     }
     else
     {
       model = this.getMenuItems( this._context );
     }
     
     if ( model !== undefined && model !== null )
     {
       this._addItemsToMenu( menu, model );
       
       menu.startup();
     }               
   },
   
   /****************************************************************************
   * Recursive function to construct menu from array of items
   ****************************************************************************/
   _addItemsToMenu:function( parent, items )
   {
      var params;
      var subMenuIndex;
      var menuItemIndex;
      
      for ( var i in items )
      {
         if ( items[i].submenu_items )
         {
           subMenuIndex = this.subMenus.length;
           this.subMenus[ subMenuIndex ] = new dijit.Menu();
 
           params = 
           {
               popup: this.subMenus[ subMenuIndex ]
           };

           dojo.mixin( params, items[i] );
        
           parent.addChild( new dijit.PopupMenuItem( params ) );  
           
           this._addItemsToMenu( this.subMenus[ subMenuIndex ], items[i].submenu_items );           
         }
         else
         {
           // [2/15/08] The following if clause added for backward compatibility
           // with the previous version of TIPContextMenu
           if (items[i].id && items[i].id.toLowerCase() === "separator")
           {
              // Previous TIPContextMenu users indicated separators via the id
              items[i].actionID = "_separator_";
           } 
           else if (items[i].id && !items[i].actionID) 
           {
              // Previous TIPContextMenu users didn't use actionID
              items[i].actionID = items[i].id;
           }

           if ( items[i].actionID === "_separator_" )
           {
             menuItemIndex = this.menuItems.length;
             
             this.menuItems[ menuItemIndex ]  = new dijit.MenuSeparator();
             
             parent.addChild( this.menuItems[ menuItemIndex ] );
           }
           else
           {
             params = {
               adapter: this,
               onClick: function() { this.adapter.onMenuItemClick( this ); }
             };
             
             dojo.mixin( params, items[i] );

             menuItemIndex = this.menuItems.length;
             
             this.menuItems[ menuItemIndex ] = new dijit.MenuItem( params );
             
             parent.addChild( this.menuItems[ menuItemIndex ] );  
           }
         }         
      }
   },
      
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem ){ /*required overload*/ },
   
   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   * Calls the provided url with the following:
   *    name: this.name (unique name of context menu to retrieve)
   *    content: this._context (the context object passed to contextMenuOpen)
   *
   * Theoretically this method could be overloaded to do something other than
   * dojo.xhrPost()
   ****************************************************************************/
   _loadMenuItems: function()
   {
     var kw = {
            sync:         true,
            url:          this.url,
            name:         this.name,
            content:      this._context,
            handleAs:     "json",
            mimetype:     "application/json",            
            load:         dojo.hitch( this, function( response ) { this._menuitems = response; } )       
          };
     
     //call synchronously                                                                       
     dojo.xhrGet( kw );
     
     return this._menuitems;
   },  

   /****************************************************************************
   * This method will be called if 'url' parameter is not set.  Provides a means
   * for javascript to build the menu items.
   ****************************************************************************/
   getMenuItems: function( context ){ /* must overload to provide menu items if URL is not used for JSON */ }
   
});

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");
dojo.provide("dojo.fx.Toggler");

dojo.fx.chain = function(/*dojo._Animation[]*/ animations){
	// summary: Chain a list of dojo._Animation s to run in sequence
	// example:
	//	|	dojo.fx.chain([
	//	|		dojo.fadeIn({ node:node }),
	//	|		dojo.fadeOut({ node:otherNode })
	//	|	]).play();
	//
	var first = animations.shift();
	var previous = first;
	dojo.forEach(animations, function(current){
		dojo.connect(previous, "onEnd", current, "play");
		previous = current;
	});
	return first; // dojo._Animation
};

dojo.fx.combine = function(/*dojo._Animation[]*/ animations){
	// summary: Combine a list of dojo._Animation s to run in parallel
	// example:
	//	|	dojo.fx.combine([
	//	|		dojo.fadeIn({ node:node }),
	//	|		dojo.fadeOut({ node:otherNode })
	//	|	]).play();
	var ctr = new dojo._Animation({ curve: [0, 1] });
	if(!animations.length){ return ctr; }
	// animations.sort(function(a, b){ return a.duration-b.duration; });
	ctr.duration = animations[0].duration;
	dojo.forEach(animations, function(current){
		dojo.forEach([ "play", "pause", "stop" ],
			function(e){
				if(current[e]){
					dojo.connect(ctr, e, current, e);
				}
			}
		);
	});
	return ctr; // dojo._Animation
};

dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc.
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.wipeIn, 
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	// node: DomNode
	//	the node to toggle
	node: null,

	// showFunc: Function
	//	The function that returns the dojo._Animation to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function	
	//	The function that returns the dojo._Animation to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//	Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//	Time in milliseconds to run the hide Animation
	hideDuration: 200,

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	show: function(delay){
		// summary: Toggle the node to showing
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		return this.hideAnim.play(delay || 0);
	}
});

dojo.fx.wipeIn = function(/*Object*/ args){
	// summary
	//		Returns an animation that will expand the
	//		node defined in 'args' object from it's current height to
	//		it's natural height (with no scrollbar).
	//		Node must have no margin/border/padding.
	args.node = dojo.byId(args.node);
	var node = args.node, s = node.style, o;

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			height: {
				// wrapped in functions so we wait till the last second to query (in case value has changed)
				start: function(){
					// start at current [computed] height, but use 1px rather than 0
					// because 0 causes IE to display the whole panel
					o = s.overflow;
					s.overflow="hidden";
					if(s.visibility=="hidden"||s.display=="none"){
						s.height="1px";
						s.display="";
						s.visibility="";
						return 1;
					}else{
						var height = dojo.style(node, "height");
						return Math.max(height, 1);
					}
				},
				end: function(){
					return node.scrollHeight;
				}
			}
		}
	}, args));

	dojo.connect(anim, "onEnd", function(){ 
		s.height = "auto";
		s.overflow = o;
	});

	return anim; // dojo._Animation
}

dojo.fx.wipeOut = function(/*Object*/ args){
	// summary
	//		Returns an animation that will shrink node defined in "args"
	//		from it's current height to 1px, and then hide it.
	var node = args.node = dojo.byId(args.node);
	var s = node.style;
	var o;

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			height: {
				end: 1 // 0 causes IE to display the whole panel
			}
		}
	}, args));

	dojo.connect(anim, "beforeBegin", function(){
		o = s.overflow;
		s.overflow = "hidden";
		s.display = "";
	});
	dojo.connect(anim, "onEnd", function(){
		s.overflow = o;
		s.height = "auto";
		s.display = "none";
	});

	return anim; // dojo._Animation
}

dojo.fx.slideTo = function(/*Object?*/ args){
	// summary
	//		Returns an animation that will slide "node" 
	//		defined in args Object from its current position to
	//		the position defined by (args.left, args.top).
	// example:
	//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", unit:"px" }).play()

	var node = (args.node = dojo.byId(args.node));
	
	var top = null;
	var left = null;
	
	var init = (function(n){
		return function(){
			var cs = dojo.getComputedStyle(n);
			var pos = cs.position;
			top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
			left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
			if(pos != 'absolute' && pos != 'relative'){
				var ret = dojo.coords(n, true);
				top = ret.y;
				left = ret.x;
				n.style.position="absolute";
				n.style.top=top+"px";
				n.style.left=left+"px";
			}
		};
	})(node);
	init();

	var anim = dojo.animateProperty(dojo.mixin({
		properties: {
			top: { end: args.top||0 },
			left: { end: args.left||0 }
		}
	}, args));
	dojo.connect(anim, "beforeBegin", anim, init);

	return anim; // dojo._Animation
}

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");




dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary
		//		Mixin for widgets that contain a list of children like SplitContainer.
		//		Widgets which mixin this code must define layout() to lay out the children

		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under document.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }
			this._started=true;

			if(this.getChildren){
				dojo.forEach(this.getChildren(), function(child){ child.startup(); });
			}

			// If I am a top level widget
			if(!this.getParent || !this.getParent()){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// since my parent isn't a layout container, and my style is width=height=100% (or something similar),
				// then I need to watch when the window resizes, and size myself accordingly
				// (passing in no argument to resize means that it has to glean the size itself)
				this.connect(window, 'onresize', function(){this.resize();});
			}
		},

		resize: function(args){
			// summary:
			//		Explicitly set this widget's size (in pixels),
			//		and then call layout() to resize contents (and maybe adjust child widgets)
			//	
			// args: Object?
			//		{w: int, h: int, l: int, t: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(args){
				dojo.marginBox(node, args);

				// set offset of the node
				if(args.t){ node.style.top = args.t + "px"; }
				if(args.l){ node.style.left = args.l + "px"; }
			}
			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = dojo.mixin(dojo.marginBox(node), args||{});

			// Save the size of my content box.
			this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

			// Callback for widget to adjust size of it's children
			this.layout();
		},

		layout: function(){
			//	summary
			//		Widgets override this method to size & position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return it's content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me=dojo._getMarginExtents(node, cs);
	var pb=dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		/**
		 * summary
		 *		Layout a bunch of child dom nodes within a parent dom node
		 * container:
		 *		parent node
		 * dim:
		 *		{l, t, w, h} object specifying dimensions of container into which to place children
		 * children:
		 *		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]
		 */

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect it's height.
			if(pos=="top" || pos=="bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos=="top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos=="left" || pos=="right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos=="left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos=="client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		dojo.requireLocalization() or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (var foo = { bar: dojo.i18n.getLocalization() }).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in dojo.adOnLoad()), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in RFC 3066.
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		tests/hello/nls/en-us/salutations.js is loaded as the object
	//		tests.hello.nls.salutations.en_us={...} The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			if(targetLocale.indexOf(flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(djConfig.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = djConfig.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = djConfig.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");








dojo.declare(
	"dijit.layout.ContentPane",
	dijit._Widget,
{
	// summary:
	//		A widget that acts as a Container for other widgets, and includes a ajax interface
	// description:
	//		A widget that can be used as a standalone widget
	//		or as a baseclass for other widgets
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .setContent('<b>new content</b>')
	//
	//		Or you can send it a NodeList, .setContent(dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .setHref('url')
	//
	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; see setHref();
	href: "",

	// extractContent: Boolean
	//	Extract visible content from inside of <body> .... </body>
	extractContent: false,

	// parseOnLoad: Boolean
	//	parse content and create the widgets, if any
	parseOnLoad:	true,

	// preventCache: Boolean
	//		Cache content retreived externally
	preventCache:	false,

	// preload: Boolean
	//	Force load of data even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//	Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>", 

	// errorMessage: String
	//	Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>", 

	// isLoaded: Boolean
	//	Tells loading status see onLoad|onUnload for event hooks
	isLoaded: false,

	// class: String
	//	Class name to apply to ContentPane dom nodes
	"class": "dijitContentPane",

	postCreate: function(){
		// remove the title attribute so it doesn't show up when i hover
		// over a node
		this.domNode.title = "";

		if(this.preload){
			this._loadCheck();
		}

		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// for programatically created ContentPane (with <span> tag), need to muck w/CSS
		// or it's as though overflow:visible is set
		dojo.addClass(this.domNode, this["class"]);
	},

	startup: function(){
		if(this._started){ return; }
		this._checkIfSingleChild();
		if(this._singleChild){
			this._singleChild.startup();
		}
		this._loadCheck();
		this._started = true;
	},

	_checkIfSingleChild: function(){
		// summary:
		// 	Test if we have exactly one widget as a child, and if so assume that we are a container for that widget,
		//	and should propogate startup() and resize() calls to it.
		var childNodes = dojo.query(">", this.containerNode || this.domNode),
			childWidgets = childNodes.filter("[widgetId]");

		if(childNodes.length == 1 && childWidgets.length == 1){
			this.isContainer = true;
			this._singleChild = dijit.byNode(childWidgets[0]);
		}else{
			delete this.isContainer;
			delete this._singleChild;
		}
	},

	refresh: function(){
		// summary:
		//	Force a refresh (re-download) of content, be sure to turn off cache

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad(true);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage
		this.href = href;

		// we return result of _prepareLoad here to avoid code dup. in dojox.layout.ContentPane
		return this._prepareLoad();
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we cant run refresh and clear content
		// refresh should only work if we downloaded the content
		if(!this._isDownloaded){
			this.href = "";
			this._onUnloadHandler();
		}

		this._setContent(data || "");

		this._isDownloaded = false; // must be set after _setContent(..), pathadjust in dojox.layout.ContentPane

		if(this.parseOnLoad){
			this._createSubWidgets();
		}

		this._checkIfSingleChild();
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}

		this._onLoadHandler();
	},

	cancel: function(){
		// summary:
		//		Cancels a inflight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect
	},

	destroy: function(){
		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		// make sure we call onUnload
		this._onUnloadHandler();
		this._beingDestroyed = true;
		this.inherited("destroy",arguments);
	},

	resize: function(size){
		dojo.marginBox(this.domNode, size);

		// Compute content box size in case we [later] need to size child
		// If either height or width wasn't specified by the user, then query node for it.
		// But note that setting the margin box and then immediately querying dimensions may return
		// inaccurate results, so try not to depend on it.
		var node = this.containerNode || this.domNode,
			mb = dojo.mixin(dojo.marginBox(node), size||{});

		this._contentBox = dijit.layout.marginBox2contentBox(node, mb);

		// If we have a single widget child then size it to fit snugly within my borders
		if(this._singleChild && this._singleChild.resize){
			this._singleChild.resize(this._contentBox);
		}
	},

	_prepareLoad: function(forceLoad){
		// sets up for a xhrLoad, load is deferred until widget onShow
		// cancels a inflight download
		this.cancel();
		this.isLoaded = false;
		this._loadCheck(forceLoad);
	},

	_loadCheck: function(forceLoad){
		// call this when you change onShow (onSelected) status when selected in parent container
		// it's used as a trigger for href download when this.domNode.display != 'none'

		// sequence:
		// if no href -> bail
		// forceLoad -> always load
		// this.preload -> load when download not in progress, domNode display doesn't matter
		// this.refreshOnShow -> load when download in progress bails, domNode display !='none' AND
		//						this.open !== false (undefined is ok), isLoaded doesn't matter
		// else -> load when download not in progress, if this.open !== false (undefined is ok) AND
		//						domNode display != 'none', isLoaded must be false

		var displayState = ((this.open !== false) && (this.domNode.style.display != 'none'));

		if(this.href &&	
			(forceLoad ||
				(this.preload && !this._xhrDfd) ||
				(this.refreshOnShow && displayState && !this._xhrDfd) ||
				(!this.isLoaded && displayState && !this._xhrDfd)
			)
		){
			this._downloadExternalContent();
		}
	},

	_downloadExternalContent: function(){
		this._onUnloadHandler();

		// display loading message
		this._setContent(
			this.onDownloadStart.call(this)
		);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs);

		hand.addCallback(function(html){
			try{
				self.onDownloadEnd.call(self);
				self._isDownloaded = true;
				self.setContent.call(self, html); // onload event is called from here
			}catch(err){
				self._onError.call(self, 'Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.cancelled){
				// show error message in the pane
				self._onError.call(self, 'Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});
	},

	_onLoadHandler: function(){
		this.isLoaded = true;
		try{
			this.onLoad.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code');
		}
	},

	_onUnloadHandler: function(){
		this.isLoaded = false;
		this.cancel();
		try{
			this.onUnload.call(this);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code');
		}
	},

	_setContent: function(cont){
		this.destroyDescendants();

		try{
			var node = this.containerNode || this.domNode;
			while(node.firstChild){
				dojo._destroyElement(node.firstChild);
			}
			if(typeof cont == "string"){
				// dijit.ContentPane does only minimal fixes,
				// No pathAdjustments, script retrieval, style clean etc
				// some of these should be available in the dojox.layout.ContentPane
				if(this.extractContent){
					match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
					if(match){ cont = match[1]; }
				}
				node.innerHTML = cont;
			}else{
				// domNode or NodeList
				if(cont.nodeType){ // domNode (htmlNode 1 or textNode 3)
					node.appendChild(cont);
				}else{// nodelist or array such as dojo.Nodelist
					dojo.forEach(cont, function(n){
						node.appendChild(n.cloneNode(true));
					});
				}
			}
		}catch(e){
			// check if a domfault occurs when we are appending this.errorMessage
			// like for instance if domNode is a UL and we try append a DIV
			var errMess = this.onContentError(e);
			try{
				node.innerHTML = errMess;
			}catch(e){
				console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
			}
		}
	},

	_onError: function(type, err, consoleText){
		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent.call(this, errText);
		}
	},

	_createSubWidgets: function(){
		// summary: scan my contents and create subwidgets
		var rootNode = this.containerNode || this.domNode;
		try{
			dojo.parser.parse(rootNode, true);
		}catch(e){
			this._onError('Content', e, "Couldn't create widgets in "+this.id
				+(this.href ? " from "+this.href : ""));
		}
	},

	// EVENT's, should be overide-able
	onLoad: function(e){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
	},

	onUnload: function(e){
		// summary:
		//		Event hook, is called before old content is cleared
	},

	onDownloadStart: function(){
		// summary:
		//		called before download starts
		//		the string returned by this function will be the html
		//		that tells the user we are loading something
		//		override with your own function if you want to change text
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		called on DOM faults, require fault etc in content
		//		default is to display errormessage inside pane
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs, default is to display
		//		errormessage inside pane. Overide function to change that.
		//		The string returned by this function will be the html
		//		that tells the user a error happend
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		called when download is finished
	}
});

}

/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/
dojo.require( "dojo.fx" );
dojo.require( "dijit._Templated" );
dojo.require( "dijit.layout.ContentPane" );

dojo.provide( "ibm.tivoli.tip.dijit.DescriptionArea" );
/******************************************************************************
* Colorful description area for Portlet Descriptions per Tivoli Guidlines
*
* Use innerDIV attachpoint to add graphics to the the description area
* background.
*
* Can be opened and closed.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.DescriptionArea", [dijit.layout.ContentPane, dijit._Templated],
{
  portletTitle: "Portlet Title Here",
  
  description: "Portlet description here...",

  imageSrc: null,
  
  backgroundClass: null,

  templateString: "<div class='tipDescriptionArea open'><div dojoAttachPoint='innerDIV'><div dojoAttachPoint='hideNode'><div class='dijitReset' dojoAttachPoint='wipeNode'><div dojoAttachPoint='containerNode'><table border='0' cellpadding='0' cellspacing='0' width='100%'><tr><td rowspan='2' dojoAttachPoint='imageNode'></td><td width='100%' class='tipDescriptionAreaTitle'>${portletTitle}</td></tr><tr><td  width='100%' valign='top' class='tipDescriptionAreaContent'>${description}</td></tr></table></div></div></div></div><div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' dojoAttachEvent=\"onclick:_toggle\"></div></div>",

  // open: Boolean
  //    Whether description is opened or closed on first render.
  open: true,

  // duration: Integer
  //    Time in milliseconds to wipe in/wipe out
  duration: 500,
  
  /*****************************************************************************
  * Setup initial styles and state of the widget
  * Create and save all animations
  *****************************************************************************/
  postCreate: function()
  {
    if( !this.open )
    {
      this.hideNode.style.display = this.wipeNode.style.display = "none";
      this._updateClass();    
    }

    this.inherited("postCreate",arguments);
    
    if ( this.imageSrc !== null )
    {
      var image = dojo.doc.createElement( 'img' );
      image.src = this.imageSrc;
      image.border = "0";
      this.imageNode.appendChild( image );
    }

    if ( this.backgroundClass !== null )
    {
      this.innerDIV.className = this.backgroundClass;
    }
  
    // setup open/close animations
    var hideNode = this.hideNode, 
        wipeNode = this.wipeNode;
        
    this._wipeIn = dojo.fx.wipeIn({
      node: this.wipeNode,
      duration: this.duration,
      beforeBegin: function(){
        hideNode.style.display="";
      }
    });
    
    this._wipeOut = dojo.fx.wipeOut({
      node: this.wipeNode,
      duration: this.duration,
      onEnd: function(){
        hideNode.style.display="none";
      }
    });
  },
  
  /*****************************************************************************
  * The toggle open/close button div was clicked -- toggle open/close state
  *****************************************************************************/
  _toggle: function()
  {
    dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
      if(animation.status() == "playing"){
        animation.stop();
      }
    });

    this[this.open ? "_wipeOut" : "_wipeIn"].play();
    this.open =! this.open;    
    this._updateClass();    
  },
  
  /*****************************************************************************
  * based on the open/close state of widget change CSS to update button
  *****************************************************************************/
  _updateClass: function()
  {
    if ( this.open === true )
    {
      dojo.removeClass( this.domNode, "close"  );
         dojo.addClass( this.domNode, "open" );          
    }
    else
    {
      dojo.removeClass( this.domNode, "open" );
         dojo.addClass( this.domNode, "close"  );          
    }  
  }  
});

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPButton"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.TIPButton");




dojo.declare("ibm.tivoli.tip.dijit.TIPButton",
              dijit.form._FormWidget,
{

	// label: String
	//	text to display in button
	label: "",

	// showLabel: Boolean
	//	whether or not to display the text label in button
	showLabel: true,

	// iconClass: String
	//	class to apply to div in button to make it display an icon
	iconClass: "",

	type: "button",
	baseClass: "dijitButton",
	templateString:"<div class=\"dijit dijitLeft dijitInline dijitButton\"\r\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\r\n\t><div class='dijitRight'\r\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\r\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\r\n\t\t\t>\r\n\t\t\t<div dojoAttachPoint=\"buttonDiv\">\r\n              <table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"0\">\r\n               <tr>\r\n                <td style=\"white-space: nowrap;\">\r\n\t\t\t\t  <div align=\"center\">\r\n\t\t\t\t\t<span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\">\r\n\t \t\t\t\t\t<span class=\"dijitToggleButtonIconChar\">&#10003</span>\r\n \t\t\t\t\t</span>\r\n\t\t\t\t\t<span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span>\r\n\t\t\t\t  </div>\r\n\t\t\t\t</td>\r\n               </tr>\r\n              </table>\r\n             </div>\t\t\t\r\n\t\t</button>\r\n\t</div>\r\n</div>\r\n",	

	constructor: function () {
	},
	_onClick: function(/*Event*/ e){
		// summary: internal function to handle click actions
		if(this.disabled){ return false; }
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary: callback when the user mouse clicks the button portion
		dojo.stopEvent(e);
		var okToSubmit = this._onClick(e) !== false; // returning nothing is same as true

		// for some reason type=submit buttons don't automatically submit the form; do it manually
		if(this.type=="submit" && okToSubmit){
			for(var node=this.domNode; node; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && widget._onSubmit){
					widget._onSubmit(e);
					break;
				}
				if(node.tagName.toLowerCase() == "form"){
					if(!node.onsubmit || node.onsubmit()){ node.submit(); }
					break;
				}
			}
		}
	},

	postCreate: function(){
		// summary:
		//	get label and set as title on button icon if necessary
		if (this.showLabel === false){
			var labelText = "";
			this.label = this.containerNode.innerHTML;
			labelText = dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
			// set title attrib on iconNode
			this.titleNode.title=labelText;
			dojo.addClass(this.containerNode,"dijitDisplayNone");
    } else {
        // need to set button div width to 100% because
        // firefox has issues fitting the label into the button
        this.buttonDiv.style.width="100%";
    }
		this.inherited(arguments);
		//this.connectEvents();
	},

	onClick: function(/*Event*/ e){
		// summary: user callback for when button is clicked
		//      if type="submit", return value != false to perform submit
		return true;
	},

	_clicked: function(/*Event*/ e){
		// summary: internal replaceable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary: reset the label (text) of the button; takes an HTML string
		this.containerNode.innerHTML = this.label = content;
		if(dojo.isMozilla){ // Firefox has re-render issues with tables
			var oldDisplay = dojo.getComputedStyle(this.domNode).display;
			this.domNode.style.display="none";
			var _this = this;
			setTimeout(function(){_this.domNode.style.display=oldDisplay;},1);
		}
		if (this.showLabel === false){
				this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
		}
	},
	connectEvents: function () {
		dojo.connect (this, "onmousedown", this, this._onMouseDown);
		dojo.connect (this, "onmouseup", this, this._onMouseUp);
		dojo.connect (this, "onkeydown", this, this._onMouseDown);
		dojo.connect (this, "onkeydown", this, this._onMouseUp);
		
		dojo.connect (this, "onmouseover", this, this._onFocus);
		dojo.connect (this, "onmouseout", this, this._onBlur);
		dojo.connect (this, "onfocus", this, this._onFocus);
		dojo.connect (this, "onblur", this, this._onBlur);
		
	},
	_onMouseDown: function () {
		this.buttonDiv.className = "button-down";
	},
	_onMouseUp: function () {
		this.buttonDiv.className = "button-up";
	},
 	_onFocus: function () {
		this.buttonDiv.className = "button-over";
 	},
 	_onBlur: function () {
		this.buttonDiv.className = "button-out";
 	},
 	destroy: function () {
 		console.log ("enter TIPButton.destroy");
 		this.inherited (arguments);
 		console.log ("exit TIPButton.destroy");
 	}

});

}

if(!dojo._hasResource["dojox.gfx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.gfx"] = true;
dojo.provide("dojox.gfx");




(function(){
	var renderers = (typeof djConfig["gfxRenderer"] == "string" ?
		djConfig["gfxRenderer"] : "svg,vml,silverlight,canvas").split(",");
	for(var i = 0; i < renderers.length; ++i){
		switch(renderers[i]){
			case "svg":
				//TODO: need more comprehensive test for SVG
				if(!dojo.isIE && (navigator.userAgent.indexOf("iPhone") < 0) && (navigator.userAgent.indexOf("iPod") < 0)){ dojox.gfx.renderer = "svg"; }
				break;
			case "vml":
				if(dojo.isIE != 0){ dojox.gfx.renderer = "vml"; }
				break;
			case "silverlight":
				//TODO: need more comprehensive test for Silverlight
				if(window.Silverlight){ dojox.gfx.renderer = "silverlight"; }
				break;
			case "canvas":
				//TODO: need more comprehensive test for Canvas
				if(dojo.isIE == 0){ dojox.gfx.renderer = "canvas"; }
				break;
		}
		if(dojox.gfx.renderer){ break; }
	}
})();

// include a renderer conditionally


dojo.requireIf(dojox.gfx.renderer == "silverlight", "dojox.gfx.silverlight");
dojo.requireIf(dojox.gfx.renderer == "canvas", "dojox.gfx.canvas");

}

if(!dojo._hasResource["dojox.lang.functional"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.functional"] = true;
dojo.provide("dojox.lang.functional");

// This module adds high-level functions and related constructs:
//	- list comprehensions similar to JavaScript 1.7
//	- anonymous functions built from the string
//	- zip combiners
//	- "reduce" family of functions
//	- currying and partial functions
//	- argument pre-processing: mixer and flip
//	- miscellaneous useful functions

// Acknoledgements:
//	- parts of this module (most notably lambda, constFun, invoke, pluck, and partial) 
//		are based on work by Oliver Steele (http://osteele.com/sources/javascript/functional/functional.js)
//		which was published under MIT License
//	- Simple "maybe" monad was donated by Alex Russell.

// Notes:
//	- Dojo provides following high-level functions in dojo/_base/array.js: 
//		forEach, map, filter, every, some
//	- These functions implemented with optional lambda expression as a parameter.
//	- missing high-level functions are provided with the compatible API: 
//		foldl, foldl1, scanl, scanl1, foldr, foldr1, scanr, scanr1,
//		reduce, reduceRight
//	- lambda() and listcomp() produce functions, which after the compilation step are 
//		as fast as regular JS functions (at least theoretically).

(function(){
	var d = dojo, df = dojox.lang.functional, g_re = /\bfor\b|\bif\b/gm, empty = {};
	
	// split() is augmented on IE6 to ensure the uniform behavior
	var split = "ab".split(/a*/).length > 1 ? String.prototype.split :
			function(sep){
				 var r = this.split.call(this, sep),
					 m = sep.exec(this);
				 if(m && m.index == 0){ r.unshift(""); }
				 return r;
			};
	var lambda = function(/*String*/ s){
		var args = [], sects = split.call(s, /\s*->\s*/m);
		if(sects.length > 1){
			while(sects.length){
				s = sects.pop();
				args = sects.pop().split(/\s*,\s*|\s+/m);
				if(sects.length){ sects.push("(function(" + args + "){return (" + s + ")})"); }
			}
		} else if(s.match(/\b_\b/)) {
			args = ["_"];
		} else {
			var l = s.match(/^\s*(?:[+*\/%&|\^\.=<>]|!=)/m),
				r = s.match(/[+\-*\/%&|\^\.=<>!]\s*$/m);
			if(l || r){
				if(l){
					args.push("$1");
					s = "$1" + s;
				}
				if(r){
					args.push("$2");
					s = s + "$2";
				}
			} else {
				var vars = s.
					replace(/(?:\b[A-Z]|\.[a-zA-Z_$])[a-zA-Z_$\d]*|[a-zA-Z_$][a-zA-Z_$\d]*:|this|true|false|null|undefined|typeof|instanceof|in|delete|new|void|arguments|decodeURI|decodeURIComponent|encodeURI|encodeURIComponent|escape|eval|isFinite|isNaN|parseFloat|parseInt|unescape|dojo|dijit|dojox|'(?:[^'\\]|\\.)*'|"(?:[^"\\]|\\.)*"/g, "").
					match(/([a-z_$][a-z_$\d]*)/gi) || [];
				var t = {};
				d.forEach(vars, function(v){
					if(!(v in t)){
						args.push(v);
						t[v] = 1;
					}
				});
			}
		}
		return {args: args, body: "return (" + s + ");"};	// Object
	};
	
	var listcomp = function(/*String*/ s){
		var frag = s.split(g_re), act = s.match(g_re),
			head = ["var r = [];"], tail = [];
		for(var i = 0; i < act.length;){
			var a = act[i], f = frag[++i];
			if(a == "for" && !/^\s*\(\s*(;|var)/.test(f)){
				f = f.replace(/^\s*\(/, "(var ");
			}
			head.push(a, f, "{");
			tail.push("}");
		}
		return head.join("") + "r.push(" + frag[0] + ");" + tail.join("") + "return r;";	// String
	};
	
	var currying = function(/*Object*/ info){
		return function(){	// Function
			if(arguments.length + info.args.length < info.arity){
				return currying({func: info.func, arity: info.arity, 
					args: Array.prototype.concat.apply(info.args, arguments)});
			}
			return info.func.apply(this, Array.prototype.concat.apply(info.args, arguments));
		};
	};
	
	var identity = function(x){ return x; };
	var compose = function(/*Array*/ a){
		return a.length ? function(){
			var i = a.length - 1, x = df.lambda(a[i]).apply(this, arguments);
			for(--i; i >= 0; --i){ x = df.lambda(a[i]).call(this, x); }
			return x;
		} : identity;
	};
	
	d.mixin(df, {
		// lambda
		buildLambda: function(/*String*/ s){
			// summary: builds a function from a snippet, returns a string, 
			//	which represents the function.
			// description: This method returns a textual representation of a function 
			//	built from the snippet. It is meant to be evaled in the proper context, 
			//	so local variables can be pulled from the environment.
			s = lambda(s);
			return "function(" + s.args.join(",") + "){" + s.body + "}";	// String
		},
		lambda: function(/*Function|String|Array*/ s){
			// summary: builds a function from a snippet, or array (composing), returns 
			//	a function object; functions are passed through unmodified.
			// description: This method is used to normalize a functional representation
			//	(a text snippet, an array, or a function) to a function object.
			if(typeof s == "function"){ return s; }
			if(s instanceof Array){ return compose(s); }
			s = lambda(s);
			return new Function(s.args, s.body);	// Function
		},
		// sequence generators
		repeat: function(/*Number*/ n, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by repeatedly applying a unary function N times 
			//	with a seed value Z.
			o = o || d.global; f = df.lambda(f);
			var t = new Array(n);
			t[0] = z;
			for(var i = 1; i < n; t[i] = z = f.call(o, z), ++i);
			return t;	// Array
		},
		until: function(/*Function|String|Array*/ pr, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: builds an array by repeatedly applying a unary function with 
			//	a seed value Z until the predicate is satisfied.
			o = o || d.global; f = df.lambda(f); pr = df.lambda(pr);
			var t = [];
			for(; !pr.call(o, z); t.push(z), z = f.call(o, z));
			return t;	// Array
		},
		buildListcomp: function(/*String*/ s){
			// summary: builds a function from a text snippet, which represents a valid
			//	JS 1.7 list comprehension, returns a string, which represents the function.
			// description: This method returns a textual representation of a function 
			//	built from the list comprehension text snippet (conformant to JS 1.7). 
			//	It is meant to be evaled in the proper context, so local variable can be 
			//	pulled from the environment.
			return "function(){" + listcomp(s) + "}";	// String
		},
		compileListcomp: function(/*String*/ s){
			// summary: builds a function from a text snippet, which represents a valid
			//	JS 1.7 list comprehension, returns a function object.
			// description: This method returns a function built from the list 
			//	comprehension text snippet (conformant to JS 1.7). It is meant to be 
			//	reused several times.
			return new Function([], listcomp(s));	// Function
		},
		listcomp: function(/*String*/ s){
			// summary: executes the list comprehension building an array.
			return (new Function([], listcomp(s)))();	// Array
		},
		// classic reduce-class functions
		foldl: function(/*Array*/ a, /*Function*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right using a seed value as a starting point; returns the final 
			//	value.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			for(var i = 0; i < a.length; z = f.call(o, z, a[i], i, a), ++i);
			return z;	// Object
		},
		foldl1: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right; returns the final value.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var z = a[0];
			for(var i = 1; i < a.length; z = f.call(o, z, a[i], i, a), ++i);
			return z;	// Object
		},
		scanl: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right using a seed value as a starting point; returns an array
			//	of values produced by foldl() at that point.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n + 1);
			t[0] = z;
			for(var i = 0; i < n; z = f.call(o, z, a[i], i, a), t[++i] = z);
			return t;	// Array
		},
		scanl1: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from left 
			//	to right; returns an array of values produced by foldl1() at that 
			//	point.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), z = a[0];
			t[0] = z;
			for(var i = 1; i < n; z = f.call(o, z, a[i], i, a), t[i++] = z);
			return t;	// Array
		},
		foldr: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left using a seed value as a starting point; returns the final 
			//	value.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			for(var i = a.length; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		foldr1: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left; returns the final value.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, z = a[n - 1];
			for(var i = n - 1; i > 0; --i, z = f.call(o, z, a[i], i, a));
			return z;	// Object
		},
		scanr: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left using a seed value as a starting point; returns an array
			//	of values produced by foldr() at that point.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n + 1);
			t[n] = z;
			for(var i = n; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		},
		scanr1: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object*/ z, /*Object?*/ o){
			// summary: repeatedly applies a binary function to an array from right
			//	to left; returns an array of values produced by foldr1() at that 
			//	point.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n), z = a[n - 1];
			t[n - 1] = z;
			for(var i = n - 1; i > 0; --i, z = f.call(o, z, a[i], i, a), t[i] = z);
			return t;	// Array
		},
		// JS 1.6 standard array functions, which can take a lambda as a parameter.
		// Consider using dojo._base.array functions, if you don't need the lambda support.
		filter: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with all elements that pass the test 
			//	implemented by the provided function.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = [], v;
			for(var i = 0; i < n; ++i){
				v = a[i];
				if(f.call(o, v, i, a)){ t.push(v); }
			}
			return t;	// Array
		},
		forEach: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: executes a provided function once per array element.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length;
			for(var i = 0; i < n; f.call(o, a[i], i, a), ++i);
		},
		map: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: creates a new array with the results of calling 
			//	a provided function on every element in this array.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length, t = new Array(n);
			for(var i = 0; i < n; t[i] = f.call(o, a[i], i, a), ++i);
			return t;	// Array
		},
		every: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether all elements in the array pass the test 
			//	implemented by the provided function.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length;
			for(var i = 0; i < n; ++i){
				if(!f.call(o, a[i], i, a)){
					return false;	// Boolean
				}
			}
			return true;	// Boolean
		},
		some: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: tests whether some element in the array passes the test 
			//	implemented by the provided function.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			var n = a.length;
			for(var i = 0; i < n; ++i){
				if(f.call(o, a[i], i, a)){
					return true;	// Boolean
				}
			}
			return false;	// Boolean
		},
		// JS 1.8 standard array functions, which can take a lambda as a parameter.
		reduce: function(/*Array*/ a, /*Function*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array 
			//	(from left-to-right) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldl1(a, f) : df.foldl(a, f, z);	// Object
		},
		reduceRight: function(/*Array*/ a, /*Function*/ f, /*Object?*/ z){
			// summary: apply a function simultaneously against two values of the array 
			//	(from right-to-left) as to reduce it to a single value.
			return arguments.length < 3 ? df.foldr1(a, f) : df.foldr(a, f, z);	// Object
		},
		// currying and partial functions
		curry: function(/*Function|String|Array*/ f, /*Number?*/ arity){
			// summary: curries a function until the arity is satisfied, at 
			//	which point it returns the calculated value.
			f = df.lambda(f);
			arity = typeof arity == "number" ? arity : f.length;
			return currying({func: f, arity: arity, args: []});	// Function
		},
		arg: {},	// marker for missing arguments
		partial: function(/*Function|String|Array*/ f){
			// summary: creates a function where some arguments are bound, and
			//	some arguments (marked as dojox.lang.functional.arg) are will be 
			//	accepted by the final function in the order they are encountered.
			// description: This method is used to produce partially bound 
			//	functions. If you want to change the order of arguments, use
			//	dojox.lang.functional.mixer() or dojox.lang.functional.flip().
			var a = arguments, args = new Array(a.length - 1), p = [];
			f = df.lambda(f);
			for(var i = 1; i < a.length; ++i){
				var t = a[i];
				args[i - 1] = t;
				if(t == df.arg){
					p.push(i - 1);
				}
			}
			return function(){	// Function
				var t = Array.prototype.slice.call(args, 0); // clone the array
				for(var i = 0; i < p.length; ++i){
					t[p[i]] = arguments[i];
				}
				return f.apply(this, t);
			};
		},
		// argument pre-processing
		mixer: function(/*Function|String|Array*/ f, /*Array*/ mix){
			// summary: changes the order of arguments using an array of
			//	numbers mix --- i-th argument comes from mix[i]-th place
			//	of supplied arguments.
			f = df.lambda(f);
			return function(){	// Function
				var t = new Array(mix.length);
				for(var i = 0; i < mix.length; ++i){
					t[i] = arguments[mix[i]];
				}
				return f.apply(this, t);
			};
		},
		flip: function(/*Function|String|Array*/ f){
			// summary: changes the order of arguments by reversing their
			//	order.
			f = df.lambda(f);
			return function(){	// Function
				// reverse arguments
				var a = arguments, l = a.length - 1, t = new Array(l + 1), i;
				for(i = 0; i <= l; ++i){
					t[l - i] = a[i];
				}
				return f.apply(this, t);
			};
		},
		// combiners
		zip: function(){
			// summary: returns an array of arrays, where the i-th array 
			//	contains the i-th element from each of the argument arrays.
			// description: This is the venerable zip combiner (for example,
			//	see Python documentation for general details). The returned
			//	array is truncated to match the length of the shortest input
			//	array.
			var n = arguments[0].length, m = arguments.length, i;
			for(i = 1; i < m; n = Math.min(n, arguments[i++].length));
			var t = new Array(n), j;
			for(i = 0; i < n; ++i){
				var p = new Array(m);
				for(j = 0; j < m; p[j] = arguments[j][i], ++j);
				t[i] = p;
			}
			return t;	// Array
		},
		unzip: function(/*Array*/ a){
			// summary: similar to dojox.lang.functional.zip(), but takes 
			//	a single array of arrays as the input.
			// description: This function is similar to dojox.lang.functional.zip() 
			//	and can be used to unzip objects packed by 
			//	dojox.lang.functional.zip(). It is here mostly to provide 
			//	a short-cut for the different method signature.
			return df.zip.apply(null, a);	// Array
		},
		// miscelaneous functional adapters
		constFun: function(/*Object*/ x){
			// summary: returns a function, which produces a constant value 
			//	regardless of supplied parameters.
			return function(){ return x; };	// Function
		},
		invoke: function(/*String*/ m){
			// summary: returns a function, which invokes a method on supplied 
			//	object using optional parameters.
			return function(/*Object*/ o){	// Function
				return o[m].apply(o, Array.prototype.slice.call(arguments, 1));
			};
		},
		pluck: function(/*String*/ m){
			// summary: returns a function, which returns a named object member.
			return function(/*Object*/ o){	// Function
				return o[m];
			};
		},
		// object helpers
		forIn: function(/*Object*/ obj, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: iterates over all object members skipping members, which 
			//	are present in the empty object (IE and/or 3rd-party libraries).
			o = o || d.global; f = df.lambda(f);
			for(var i in obj){
				if(i in empty){ continue; }
				f.call(o, obj[i], i, obj);
			}
		},
		forEachReversed: function(/*Array*/ a, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: executes a provided function once per array element.
			a = typeof a == "string" ? a.split("") : a; o = o || d.global; f = df.lambda(f);
			for(var i = a.length - 1; i >= 0; f.call(o, a[i], i, a), --i);
		}
	});

	// monads
	dojo.declare("dojox.lang.functional.MaybeMonad", null, {
		constructor: function(/*Object*/ value){
			// summary: constructs a monad optionally initializing all additional members
			if(arguments.length){
				this.value = value;
			}
		},
		bind: function(/*dojox.lang.functional.Monad*/ monad, /*Function|String|Array*/ f, /*Object?*/ o){
			// summary: this is the classic bind method, which applies a function to a monad,
			//	and returns a result as a monad; it is meant to be overwritten to incorporate
			//	side effects
			if(!("value" in monad)){
				return new this.constructor();	// dojox.lang.functional.MaybeMonad
			}
			// => possible side-effects go here
			o = o || d.global; f = df.lambda(f);
			return f.call(o, monad.value);	// dojox.lang.functional.Monad
		},
		// class-specific methods
		isNothing: function(){
			// summary: check if there is no bound value.
			return !("value" in this);	// Boolean
		}
	});
	df.MaybeMonad.returnMonad = function(/*Object*/ value){
		// summary: puts a valye in the Maybe monad.
		return new df.MaybeMonad(value);	// dojox.lang.functional.MaybeMonad
	};
	df.MaybeMonad.zero = new df.MaybeMonad();
})();

}

if(!dojo._hasResource["dojox.charting._color"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting._color"] = true;
dojo.provide("dojox.charting._color");

dojox.charting._color={};
dojox.charting._color.fromHsb=function(/* int */hue, /* int */saturation, /* int */brightness){
	//	summary
	//	Creates an instance of dojo.Color based on HSB input (360, %, %)
	hue=Math.round(hue);
	saturation=Math.round((saturation/100)*255); 
	brightness=Math.round((brightness/100)*255);

	var r, g, b;
	if(saturation==0){
		r=g=b=brightness;
	} else {
		var tint1=brightness, 
			tint2=(255-saturation)*brightness/255, 
			tint3=(tint1-tint2)*(hue%60)/60;
		if(hue<60){ r=tint1, g=tint2+tint3, b=tint2; }
		else if(hue<120){ r=tint1-tint3, g=tint1, b=tint2; }
		else if(hue<180){ r=tint2, g=tint1, b=tint2+tint3; }
		else if(hue<240){ r=tint2, g=tint1-tint3, b=tint1; }
		else if(hue<300){ r=tint2+tint3, g=tint2, b=tint1; }
		else if(hue<360){ r=tint1, g=tint2, b=tint1-tint3; }
	}

	r=Math.round(r); g=Math.round(g); b=Math.round(b);
	return new dojo.Color({ r:r, g:g, b:b });
};

dojox.charting._color.toHsb=function(/* int|Object|dojo.Color */ red, /* int? */ green, /* int? */blue){
	//	summary
	//	Returns the color in HSB representation (360, %, %)
	var r=red,g=green,b=blue;
	if(dojo.isObject(red)){
		r=red.r,g=red.g,b=red.b;
	}
	var min=Math.min(r,g,b);
	var max=Math.max(r,g,b);
	var delta=max-min;

	var hue=0, saturation=(max!=0?delta/max:0), brightness=max/255;
	if(saturation==0){ hue=0; }
	else {
		if(r==max){ hue=((max-b)/delta)-((max-g)/delta); }
		else if(g==max){ hue=2+(((max-r)/delta)-((max-b)/delta)); }
		else { hue=4+(((max-g)/delta)-((max-r)/delta)); }
		hue/=6;
		if(hue<0) hue++;
	}
	
	hue=Math.round(hue*360);
	saturation=Math.round(saturation*100);
	brightness=Math.round(brightness*100);
	return { 
		h:hue, s:saturation, b:brightness,
		hue:hue, saturation:saturation, brightness:brightness
	};	//	Object
};

}

if(!dojo._hasResource["dojox.charting.Theme"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Theme"] = true;
dojo.provide("dojox.charting.Theme");


(function(){
	var dxc=dojox.charting;
	//	TODO: Legend information
	dxc.Theme=function(/*object?*/kwArgs){
		kwArgs=kwArgs||{};
		this.chart=dojo.mixin(dojo.clone(dxc.Theme._def.chart), kwArgs.chart||{});
		this.plotarea=dojo.mixin(dojo.clone(dxc.Theme._def.plotarea), kwArgs.plotarea||{});
		this.axis=dojo.mixin(dojo.clone(dxc.Theme._def.axis), kwArgs.axis||{});
		this.series=dojo.mixin(dojo.clone(dxc.Theme._def.series), kwArgs.series||{});
		this.marker=dojo.mixin(dojo.clone(dxc.Theme._def.marker), kwArgs.marker||{});
		this.markers=dojo.mixin(dojo.clone(dxc.Theme.Markers), kwArgs.markers||{});
		this.colors=[];
		this.antiAlias=("antiAlias" in kwArgs)?kwArgs.antiAlias:true;
		this.assignColors=("assignColors" in kwArgs)?kwArgs.assignColors:true;
		this.assignMarkers=("assignMarkers" in kwArgs)?kwArgs.assignMarkers:true;
		this._colorCache=null;

		//	push the colors, use _def colors if none passed.
		kwArgs.colors=kwArgs.colors||dxc.Theme._def.colors;
		dojo.forEach(kwArgs.colors, function(item){ 
			this.colors.push(item); 
		}, this);

		//	private variables for color and marker indexing
		this._current={ color:0, marker: 0 };
		this._markers=[];
		this._buildMarkerArray();
	};

	//	"static" fields
	//	default markers.
	//	A marker is defined by an SVG path segment; it should be defined as
	//		relative motion, and with the assumption that the path segment
	//		will be moved to the value point (i.e prepend Mx,y)
	dxc.Theme.Markers={
		CIRCLE:"m-3,0 c0,-4 6,-4 6,0 m-6,0 c0,4 6,4 6,0", 
		SQUARE:"m-3,-3 l0,6 6,0 0,-6 z", 
		DIAMOND:"m0,-3 l3,3 -3,3 -3,-3 z", 
		CROSS:"m0,-3 l0,6 m-3,-3 l6,0", 
		X:"m-3,-3 l6,6 m0,-6 l-6,6", 
		TRIANGLE:"m-3,3 l3,-6 3,6 z", 
		TRIANGLE_INVERTED:"m-3,-3 l3,6 3,-6 z"
	};
	dxc.Theme._def={
		//	all objects are structs used directly in dojox.gfx
		chart:{ 
			stroke:null,
			fill: "white"
		},
		plotarea:{ 
			stroke:null,
			fill: "white"
		},
		//	TODO: label rotation on axis
		axis:{
			stroke:{ color:"#333",width:1 },							//	the axis itself
			line:{ color:"#ccc",width:1,style:"Dot",cap:"round" },		//	gridlines
			majorTick:{ color:"#666", width:1, length:6, position:"center" },	//	major ticks on axis
			minorTick:{ color:"#666", width:0.8, length:3, position:"center" },	//	minor ticks on axis
			font:"normal normal normal 7pt Tahoma",						//	labels on axis
			fontColor:"#333"											//	color of labels
		},
		series:{
			outline: {width: 2, color: "#ccc"},							//	line or outline
			stroke: {width: 2, color: "#333"},							//	line or outline
			fill: "#ccc",												//	fill, if appropriate
			font: "normal normal normal 7pt Tahoma",					//	if there's a label
			fontColor: "#000"											// 	color of labels
		},
		marker:{	//	any markers on a series.
			stroke: {width:1},											//	stroke or outline
			fill: "#333",												//	fill if needed
			font: "normal normal normal 7pt Tahoma",					//	label
			fontColor: "#000"
		},
		colors:[
			"#000","#111","#222","#333",
			"#444","#555","#666","#777",
			"#888","#999","#aaa","#bbb",
			"#ccc"
		]
	};
	
	//	prototype methods
	dojo.extend(dxc.Theme, {
		defineColors: function(obj){
			//	we can generate a set of colors based on keyword arguments
			var kwArgs=obj||{};

			//	deal with caching
			var cache=false;
			if(kwArgs.cache===undefined){ cache=true; }
			if(kwArgs.cache==true){ cache=true; }
			
			if(cache){
				this._colorCache=kwArgs;
			} else {
				var mix=this._colorCache||{};
				kwArgs=dojo.mixin(dojo.clone(mix), kwArgs);
			}

			var c=[], n=kwArgs.num||32;	//	the number of colors to generate
			if(kwArgs.colors){
				//	we have an array of colors predefined, so fix for the number of series.
				var l=kwArgs.colors.length;
				for(var i=0; i<n; i++){
					c.push(kwArgs.colors[i%l]);
				}
				this.colors=c;
			}
			else if(kwArgs.hue){
				//	single hue, generate a set based on brightness
				var s=kwArgs.saturation||100;	//	saturation
				var st=kwArgs.low||30;
				var end=kwArgs.high||90;
				var step=(end-st)/n;			//	brightness steps
				for(var i=0; i<n; i++){
					c.push(dxc._color.fromHsb(kwArgs.hue, s, st+(step*i)).toHex());
				}
				this.colors=c;
			}
			else if(kwArgs.stops){
				//	create color ranges that are either equally distributed, or
				//	(optionally) based on a passed "offset" property.  If you
				//	pass an array of Colors, it will equally distribute, if
				//	you pass an array of structs { color, offset }, it will
				//	use the offset (0.0 - 1.0) to distribute.  Note that offset
				//	values should be plotted on a line from 0.0 to 1.0--i.e.
				//	they should be additive.  For example:
				//	[ {color, offset:0}, { color, offset:0.2 }, { color, offset:0.5 }, { color, offset:1.0 } ]
				//	
				//	If you use stops for colors, you MUST have a color at 0.0 and one
				//	at 1.0.
			
				//	figure out how many stops we have
				var l=kwArgs.stops.length;
				if(l<2){
					throw new Error(
						"dojox.charting.Theme::defineColors: when using stops to "
						+ "define a color range, you MUST specify at least 2 colors."
					);
				}

				//	figure out if the distribution is equal or not.  Note that
				//	colors may not exactly match the stops you define; because
				//	color generation is linear (i.e. evenly divided on a linear
				//	axis), it's very possible that a color will land in between
				//	two stops and not exactly *at* a stop.
				//
				//	The only two colors guaranteed will be the end stops (i.e.
				//	the first and last stop), which will *always* be set as
				//	the end stops.
				if(typeof(kwArgs.stops[0].offset)=="undefined"){ 
					//	set up equal offsets
					var off=1/(l-1);
					for(var i=0; i<l; i++){
						kwArgs.stops[i]={
							color:kwArgs.stops[i],
							offset:off*i
						};
					}
				}
				//	ensure the ends.
				kwArgs.stops[0].offset=0;
				kwArgs.stops[l-1].offset=1;
				kwArgs.stops.sort(function(a,b){ return a.offset-b.offset; });

				//	create the colors.
				//	first stop.
				c.push(kwArgs.stops[0].color.toHex());

				//	TODO: calculate the blend at n/steps and set the color

				//	last stop
				c.push(kwArgs.stops[l-1].color.toHex());
				this.colors=c;
			}
		},
	
		_buildMarkerArray: function(){
			this._markers=[];
			for(var p in this.markers){ this._markers.push(this.markers[p]); }
			//	reset the position
			this._current.marker=0;
		},

		addMarker:function(/* string */name, /*string*/segment){
			//	summary
			//	Add a custom marker to this theme.
			//
			//	example
			//	myTheme.addMarker("Ellipse", foo);
			this.markers[name]=segment;
			this._buildMarkerArray();
		},
		setMarkers:function(/* object */obj){
			//	summary
			//	Set all the markers of this theme at once.  obj should be
			//	a dictionary of keys and path segments.
			//
			//	example
			//	myTheme.setMarkers({ "CIRCLE": foo });
			this.markers=obj;
			this._buildMarkerArray();
		},

		next: function(/* string? */type){
			//	summary
			//	get either the next color or the next marker, depending on what was
			//	passed.  If type is not passed, it assumes color.
			//
			//	example
			//	var color=myTheme.next();
			//	var color=myTheme.next("color");
			//	var marker=myTheme.next("marker");
			if(!type) type="color";
			if(type=="color"){
				return this.colors[this._current.color++%this.colors.length];
			}
			else if(type=="marker"){
				return this._markers[this._current.marker++%this._markers.length];
			}
		},
		clear: function(){
			this._current = {color: 0, marker: 0};
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.Element"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Element"] = true;
dojo.provide("dojox.charting.Element");

dojo.declare("dojox.charting.Element", null, {
	constructor: function(chart){
		this.chart = chart;
		this.group = null;
		this.htmlElements = [];
		this.dirty = true;
	},
	createGroup: function(creator){
		if(!creator){ creator = this.chart.surface; }
		if(!this.group){
			this.group = creator.createGroup();
		}
		return this;
	},
	purgeGroup: function(){
		this.destroyHtmlElements();
		if(this.group){
			this.group.clear();
			this.group.removeShape();
			this.group = null;
		}
		this.dirty = true;
		return this;
	},
	cleanGroup: function(creator){
		this.destroyHtmlElements();
		if(!creator){ creator = this.chart.surface; }
		if(this.group){
			this.group.clear();
		}else{
			this.group = creator.createGroup();
		}
		this.dirty = true;
		return this;
	},
	destroyHtmlElements: function(){
		dojo.forEach(this.htmlElements, dojo._destroyElement);
		this.htmlElements = [];
	},
	destroy: function(){
		this.purgeGroup();
	}
});

}

if(!dojo._hasResource["dojox.charting.Series"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Series"] = true;
dojo.provide("dojox.charting.Series");



dojo.declare("dojox.charting.Series", dojox.charting.Element, {
	constructor: function(chart, data, kwArgs){
		dojo.mixin(this, kwArgs);
		if(typeof this.plot != "string"){ this.plot = "default"; }
		this.data = data;
		this.dirty = true;
		this.clear();
	},
	clear: function(){
		this.dyn = {};
	}
});

}

if(!dojo._hasResource["dojox.charting.scaler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.scaler"] = true;
dojo.provide("dojox.charting.scaler");

(function(){
	var deltaLimit = 3;	// pixels
	
	var isText = function(val, text){
		val = val.toLowerCase();
		for(var i = 0; i < text.length; ++i){
			if(val == text[i]){ return true; }
		}
		return false;
	};
	
	var calcTicks = function(min, max, kwArgs, majorTick, minorTick, microTick, span){
		kwArgs = dojo.clone(kwArgs);
		if(!majorTick){
			if(kwArgs.fixUpper == "major"){ kwArgs.fixUpper = "minor"; }
			if(kwArgs.fixLower == "major"){ kwArgs.fixLower = "minor"; }
		}
		if(!minorTick){
			if(kwArgs.fixUpper == "minor"){ kwArgs.fixUpper = "micro"; }
			if(kwArgs.fixLower == "minor"){ kwArgs.fixLower = "micro"; }
		}
		if(!microTick){
			if(kwArgs.fixUpper == "micro"){ kwArgs.fixUpper = "none"; }
			if(kwArgs.fixLower == "micro"){ kwArgs.fixLower = "none"; }
		}
		var lowerBound = isText(kwArgs.fixLower, ["major"]) ? 
				Math.floor(min / majorTick) * majorTick :
					isText(kwArgs.fixLower, ["minor"]) ? 
						Math.floor(min / minorTick) * minorTick :
							isText(kwArgs.fixLower, ["micro"]) ?
								Math.floor(min / microTick) * unit : min,
			upperBound = isText(kwArgs.fixUpper, ["major"]) ? 
				Math.ceil(max / majorTick) * majorTick :
					isText(kwArgs.fixUpper, ["minor"]) ? 
						Math.ceil(max / minorTick) * minorTick :
							isText(kwArgs.fixUpper, ["unit"]) ?
								Math.ceil(max / unit) * unit : max,
			majorStart = (isText(kwArgs.fixLower, ["major"]) || !majorTick) ?
				lowerBound : Math.ceil(lowerBound / majorTick) * majorTick,
			minorStart = (isText(kwArgs.fixLower, ["major", "minor"]) || !minorTick) ?
				lowerBound : Math.ceil(lowerBound / minorTick) * minorTick,
			microStart = (isText(kwArgs.fixLower, ["major", "minor", "micro"]) || ! microTick) ?
				lowerBound : Math.ceil(lowerBound / microTick) * microTick,
			majorCount = !majorTick ? 0 : (isText(kwArgs.fixUpper, ["major"]) ?
				Math.round((upperBound - majorStart) / majorTick) :
				Math.floor((upperBound - majorStart) / majorTick)) + 1,
			minorCount = !minorTick ? 0 : (isText(kwArgs.fixUpper, ["major", "minor"]) ?
				Math.round((upperBound - minorStart) / minorTick) :
				Math.floor((upperBound - minorStart) / minorTick)) + 1,
			microCount = !microTick ? 0 : (isText(kwArgs.fixUpper, ["major", "minor", "micro"]) ?
				Math.round((upperBound - microStart) / microTick) :
				Math.floor((upperBound - microStart) / microTick)) + 1,
			minorPerMajor  = minorTick ? Math.round(majorTick / minorTick) : 0,
			microPerMinor  = microTick ? Math.round(minorTick / microTick) : 0,
			majorPrecision = majorTick ? Math.floor(Math.log(majorTick) / Math.LN10) : 0,
			minorPrecision = minorTick ? Math.floor(Math.log(minorTick) / Math.LN10) : 0,
			scale = span / (upperBound - lowerBound);
		if(!isFinite(scale)){ scale = 1; }
		return {
			bounds: {
				lower:	lowerBound,
				upper:	upperBound
			},
			major: {
				tick:	majorTick,
				start:	majorStart,
				count:	majorCount,
				prec:	majorPrecision
			},
			minor: {
				tick:	minorTick,
				start:	minorStart,
				count:	minorCount,
				prec:	minorPrecision
			},
			micro: {
				tick:	microTick,
				start:	microStart,
				count:	microCount,
				prec:	0
			},
			minorPerMajor:	minorPerMajor,
			microPerMinor:	microPerMinor,
			scale:			scale
		};
	};

	dojox.charting.scaler = function(min, max, span, kwArgs){
		var h = {fixUpper: "none", fixLower: "none", natural: false};
		if(kwArgs){
			if("fixUpper" in kwArgs){ h.fixUpper = String(kwArgs.fixUpper); }
			if("fixLower" in kwArgs){ h.fixLower = String(kwArgs.fixLower); }
			if("natural"  in kwArgs){ h.natural  = Boolean(kwArgs.natural); }
		}
		
		if(max <= min){
			return calcTicks(min, max, h, 0, 0, 0, span);	// Object
		}
		
		var mag = Math.floor(Math.log(max - min) / Math.LN10),
			major = kwArgs && ("majorTick" in kwArgs) ? kwArgs.majorTick : Math.pow(10, mag), 
			minor = 0, micro = 0, ticks;
			
		// calculate minor ticks
		if(kwArgs && ("minorTick" in kwArgs)){
			minor = kwArgs.minorTick;
		}else{
			do{
				minor = major / 10;
				if(!h.natural || minor > 0.9){
					ticks = calcTicks(min, max, h, major, minor, 0, span);
					if(ticks.scale * ticks.minor.tick > deltaLimit){ break; }
				}
				minor = major / 5;
				if(!h.natural || minor > 0.9){
					ticks = calcTicks(min, max, h, major, minor, 0, span);
					if(ticks.scale * ticks.minor.tick > deltaLimit){ break; }
				}
				minor = major / 2;
				if(!h.natural || minor > 0.9){
					ticks = calcTicks(min, max, h, major, minor, 0, span);
					if(ticks.scale * ticks.minor.tick > deltaLimit){ break; }
				}
				return calcTicks(min, max, h, major, 0, 0, span);	// Object
			}while(false);
		}

		// calculate micro ticks
		if(kwArgs && ("microTick" in kwArgs)){
			micro = kwArgs.microTick;
			ticks = calcTicks(min, max, h, major, minor, micro, span);
		}else{
			do{
				micro = minor / 10;
				if(!h.natural || micro > 0.9){
					ticks = calcTicks(min, max, h, major, minor, micro, span);
					if(ticks.scale * ticks.micro.tick > deltaLimit){ break; }
				}
				micro = minor / 5;
				if(!h.natural || micro > 0.9){
					ticks = calcTicks(min, max, h, major, minor, micro, span);
					if(ticks.scale * ticks.micro.tick > deltaLimit){ break; }
				}
				micro = minor / 2;
				if(!h.natural || micro > 0.9){
					ticks = calcTicks(min, max, h, major, minor, micro, span);
					if(ticks.scale * ticks.micro.tick > deltaLimit){ break; }
				}
				micro = 0;
			}while(false);
		}

		return micro ? ticks : calcTicks(min, max, h, major, minor, 0, span);	// Object
	};
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.common"] = true;
dojo.provide("dojox.charting.axis2d.common");



(function(){
	var g = dojox.gfx;
	
	dojo.mixin(dojox.charting.axis2d.common, {
		createText: {
			gfx: function(chart, creator, x, y, align, text, font, fontColor){
				return creator.createText({
					x: x, y: y, text: text, align: align
				}).setFont(font).setFill(fontColor);
			},
			html: function(chart, creator, x, y, align, text, font, fontColor){
				var p = dojo.doc.createElement("div"), s = p.style;
				s.marginLeft   = "0px";
				s.marginTop    = "0px";
				s.marginRight  = "0px";
				s.marginBottom = "0px";
				s.paddingLeft   = "0px";
				s.paddingTop    = "0px";
				s.paddingRight  = "0px";
				s.paddingBottom = "0px";
				s.borderLeftWidth   = "0px";
				s.borderTopWidth    = "0px";
				s.borderRightWidth  = "0px";
				s.borderBottomWidth = "0px";
				s.position = "absolute";
				s.font = font;
				p.innerHTML = text;
				s.color = fontColor;
				chart.node.appendChild(p);
				var parent = chart.getCoords(), 
					box = dojo.marginBox(p),
					size = g.normalizedLength(g.splitFontString(font).size),
					top = parent.y + Math.floor(y - size);
				switch(align){
					case "middle":
						dojo.marginBox(p, {l: parent.x + Math.floor(x - box.w / 2), t: top});
						break;
					case "end":
						dojo.marginBox(p, {l: parent.x + Math.floor(x - box.w), t: top});
						break;
					//case "start":
					default:
						dojo.marginBox(p, {l: parent.x + Math.floor(x), t: top});
						break;
				}
				return p;
			}
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.Base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.Base"] = true;
dojo.provide("dojox.charting.axis2d.Base");



dojo.declare("dojox.charting.axis2d.Base", dojox.charting.Element, {
	constructor: function(chart, kwArgs){
		this.vertical = kwArgs && kwArgs.vertical;
	},
	clear: function(){
		return this;
	},
	initialized: function(){
		return false;
	},
	calculate: function(min, max, span){
		return this;
	},
	getScaler: function(){
		return null;
	},
	getOffsets: function(){
		return {l: 0, r: 0, t: 0, b: 0};
	},
	render: function(dim, offsets){
		return this;
	}
});

}

if(!dojo._hasResource["dojo.colors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.colors"] = true;
dojo.provide("dojo.colors");

(function(){
	// this is a standard convertion prescribed by the CSS3 Color Module
	var hue2rgb = function(m1, m2, h){
		if(h < 0){ ++h; }
		if(h > 1){ --h; }
		var h6 = 6 * h;
		if(h6 < 1){ return m1 + (m2 - m1) * h6; }
		if(2 * h < 1){ return m2; }
		if(3 * h < 2){ return m1 + (m2 - m1) * (2 / 3 - h) * 6; }
		return m1;
	};
	
	dojo.colorFromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		get rgb(a) array from css-style color declarations
		// description:
		//		this function can handle all 4 CSS3 Color Module formats: rgb,
		//		rgba, hsl, hsla, including rgb(a) with percentage values.
		var m = color.toLowerCase().match(/^(rgba?|hsla?)\(([\s\.\-,%0-9]+)\)/);
		if(m){
			var c = m[2].split(/\s*,\s*/), l = c.length, t = m[1];
			if((t == "rgb" && l == 3) || (t == "rgba" && l == 4)){
				var r = c[0];
				if(r.charAt(r.length - 1) == "%"){
					// 3 rgb percentage values
					var a = dojo.map(c, function(x){
						return parseFloat(x) * 2.56;
					});
					if(l == 4){ a[3] = c[3]; }
					return dojo.colorFromArray(a, obj);	// dojo.Color
				}
				return dojo.colorFromArray(c, obj);	// dojo.Color
			}
			if((t == "hsl" && l == 3) || (t == "hsla" && l == 4)){
				// normalize hsl values
				var H = ((parseFloat(c[0]) % 360) + 360) % 360 / 360,
					S = parseFloat(c[1]) / 100,
					L = parseFloat(c[2]) / 100,
					// calculate rgb according to the algorithm 
					// recommended by the CSS3 Color Module 
					m2 = L <= 0.5 ? L * (S + 1) : L + S - L * S, 
					m1 = 2 * L - m2,
					a = [hue2rgb(m1, m2, H + 1 / 3) * 256,
						hue2rgb(m1, m2, H) * 256, hue2rgb(m1, m2, H - 1 / 3) * 256, 1];
				if(l == 4){ a[3] = c[3]; }
				return dojo.colorFromArray(a, obj);	// dojo.Color
			}
		}
		return null;	// dojo.Color
	};
	
	var confine = function(c, low, high){
		// summary:
		//		sanitize a color component by making sure it is a number,
		//		and clamping it to valid values
		c = Number(c);
		return isNaN(c) ? high : c < low ? low : c > high ? high : c;	// Number
	};
	
	dojo.Color.prototype.sanitize = function(){
		// summary: makes sure that the object has correct attributes
		var t = this;
		t.r = Math.round(confine(t.r, 0, 255));
		t.g = Math.round(confine(t.g, 0, 255));
		t.b = Math.round(confine(t.b, 0, 255));
		t.a = confine(t.a, 0, 1);
		return this;	// dojo.Color
	};
})();


dojo.colors.makeGrey = function(/*Number*/ g, /*Number?*/ a){
	// summary: creates a greyscale color with an optional alpha
	return dojo.colorFromArray([g, g, g, a]);
};


// mixin all CSS3 named colors not already in _base, along with SVG 1.0 variant spellings
dojo.Color.named = dojo.mixin({
	aliceblue:	[240,248,255],
	antiquewhite:	[250,235,215],
	aquamarine:	[127,255,212],
	azure:	[240,255,255],
	beige:	[245,245,220],
	bisque:	[255,228,196],
	blanchedalmond:	[255,235,205],
	blueviolet:	[138,43,226],
	brown:	[165,42,42],
	burlywood:	[222,184,135],
	cadetblue:	[95,158,160],
	chartreuse:	[127,255,0],
	chocolate:	[210,105,30],
	coral:	[255,127,80],
	cornflowerblue:	[100,149,237],
	cornsilk:	[255,248,220],
	crimson:	[220,20,60],
	cyan:	[0,255,255],
	darkblue:	[0,0,139],
	darkcyan:	[0,139,139],
	darkgoldenrod:	[184,134,11],
	darkgray:	[169,169,169],
	darkgreen:	[0,100,0],
	darkgrey:	[169,169,169],
	darkkhaki:	[189,183,107],
	darkmagenta:	[139,0,139],
	darkolivegreen:	[85,107,47],
	darkorange:	[255,140,0],
	darkorchid:	[153,50,204],
	darkred:	[139,0,0],
	darksalmon:	[233,150,122],
	darkseagreen:	[143,188,143],
	darkslateblue:	[72,61,139],
	darkslategray:	[47,79,79],
	darkslategrey:	[47,79,79],
	darkturquoise:	[0,206,209],
	darkviolet:	[148,0,211],
	deeppink:	[255,20,147],
	deepskyblue:	[0,191,255],
	dimgray:	[105,105,105],
	dimgrey:	[105,105,105],
	dodgerblue:	[30,144,255],
	firebrick:	[178,34,34],
	floralwhite:	[255,250,240],
	forestgreen:	[34,139,34],
	gainsboro:	[220,220,220],
	ghostwhite:	[248,248,255],
	gold:	[255,215,0],
	goldenrod:	[218,165,32],
	greenyellow:	[173,255,47],
	grey:	[128,128,128],
	honeydew:	[240,255,240],
	hotpink:	[255,105,180],
	indianred:	[205,92,92],
	indigo:	[75,0,130],
	ivory:	[255,255,240],
	khaki:	[240,230,140],
	lavender:	[230,230,250],
	lavenderblush:	[255,240,245],
	lawngreen:	[124,252,0],
	lemonchiffon:	[255,250,205],
	lightblue:	[173,216,230],
	lightcoral:	[240,128,128],
	lightcyan:	[224,255,255],
	lightgoldenrodyellow:	[250,250,210],
	lightgray:	[211,211,211],
	lightgreen:	[144,238,144],
	lightgrey:	[211,211,211],
	lightpink:	[255,182,193],
	lightsalmon:	[255,160,122],
	lightseagreen:	[32,178,170],
	lightskyblue:	[135,206,250],
	lightslategray:	[119,136,153],
	lightslategrey:	[119,136,153],
	lightsteelblue:	[176,196,222],
	lightyellow:	[255,255,224],
	limegreen:	[50,205,50],
	linen:	[250,240,230],
	magenta:	[255,0,255],
	mediumaquamarine:	[102,205,170],
	mediumblue:	[0,0,205],
	mediumorchid:	[186,85,211],
	mediumpurple:	[147,112,219],
	mediumseagreen:	[60,179,113],
	mediumslateblue:	[123,104,238],
	mediumspringgreen:	[0,250,154],
	mediumturquoise:	[72,209,204],
	mediumvioletred:	[199,21,133],
	midnightblue:	[25,25,112],
	mintcream:	[245,255,250],
	mistyrose:	[255,228,225],
	moccasin:	[255,228,181],
	navajowhite:	[255,222,173],
	oldlace:	[253,245,230],
	olivedrab:	[107,142,35],
	orange:	[255,165,0],
	orangered:	[255,69,0],
	orchid:	[218,112,214],
	palegoldenrod:	[238,232,170],
	palegreen:	[152,251,152],
	paleturquoise:	[175,238,238],
	palevioletred:	[219,112,147],
	papayawhip:	[255,239,213],
	peachpuff:	[255,218,185],
	peru:	[205,133,63],
	pink:	[255,192,203],
	plum:	[221,160,221],
	powderblue:	[176,224,230],
	rosybrown:	[188,143,143],
	royalblue:	[65,105,225],
	saddlebrown:	[139,69,19],
	salmon:	[250,128,114],
	sandybrown:	[244,164,96],
	seagreen:	[46,139,87],
	seashell:	[255,245,238],
	sienna:	[160,82,45],
	skyblue:	[135,206,235],
	slateblue:	[106,90,205],
	slategray:	[112,128,144],
	slategrey:	[112,128,144],
	snow:	[255,250,250],
	springgreen:	[0,255,127],
	steelblue:	[70,130,180],
	tan:	[210,180,140],
	thistle:	[216,191,216],
	tomato:	[255,99,71],
	transparent: [0, 0, 0, 0],
	turquoise:	[64,224,208],
	violet:	[238,130,238],
	wheat:	[245,222,179],
	whitesmoke:	[245,245,245],
	yellowgreen:	[154,205,50]
}, dojo.Color.named);

}

if(!dojo._hasResource["dojox.lang.utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.lang.utils"] = true;
dojo.provide("dojox.lang.utils");

(function(){
	var empty = {}, du = dojox.lang.utils;
	
	dojo.mixin(dojox.lang.utils, {
		coerceType: function(target, source){
			switch(typeof target){
				case "number":	return Number(eval("(" + source + ")"));
				case "string":	return String(source);
				case "boolean":	return Boolean(eval("(" + source + ")"));
			}
			return eval("(" + source + ")");
		},
		
		updateWithObject: function(target, source, conv){
			// summary: updates an existing object in place with properties from an "source" object.
			// target: Object: the "target" object to be updated
			// source: Object: the "source" object, whose properties will be used to source the existed object.
			// conv: Boolean?: force conversion to the original type
			if(!source){ return target; }
			for(var x in target){
				if(x in source && !(x in empty)){
					var t = target[x];
					if(t && typeof t == "object"){
						du.updateObject(t, source[x]);
					}else{
						target[x] = conv ? du.coerceType(t, source[x]) : dojo.clone(source[x]);
					}
				}
			}
			return target;	// Object
		},
	
		updateWithPattern: function(target, source, pattern, conv){
			// summary: updates an existing object in place with properties from an "source" object.
			// target: Object: the "target" object to be updated
			// source: Object: the "source" object, whose properties will be used to source the existed object.
			// pattern: Array: an array of properties to be copied
			// conv: Boolean?: force conversion to the original type
			if(!source || !pattern){ return target; }
			for(var x in pattern){
				if(x in source && !(x in empty)){
					target[x] = conv ? du.coerceType(pattern[x], source[x]) : dojo.clone(source[x]);
				}
			}
			return target;	// Object
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.axis2d.Default"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.axis2d.Default"] = true;
dojo.provide("dojox.charting.axis2d.Default");










(function(){
	var dc = dojox.charting, 
		df = dojox.lang.functional, 
		du = dojox.lang.utils, 
		g = dojox.gfx,
		labelGap = 4,				// in pixels
		labelFudgeFactor = 0.8;		// in percents (to convert font's heigth to label width)
		
	var eq = function(/* Number */ a, /* Number */ b){
		// summary: compare two FP numbers for equality
		return Math.abs(a - b) <= 1e-6 * (Math.abs(a) + Math.abs(b));	// Boolean
	};

	dojo.declare("dojox.charting.axis2d.Default", dojox.charting.axis2d.Base, {
		 defaultParams: {
			vertical:    false,		// true for vertical axis
			fixUpper:    "none",	// align the upper on ticks: "major", "minor", "micro", "none"
			fixLower:    "none",	// align the lower on ticks: "major", "minor", "micro", "none"
			natural:     false,		// all tick marks should be made on natural numbers
			leftBottom:  true,		// position of the axis, used with "vertical"
			includeZero: false,		// 0 should be included
			fixed:       true,		// all labels are fixed numbers
			majorLabels: true,		// draw major labels
			minorTicks:  true,		// draw minor ticks
			minorLabels: true,		// draw minor labels
			microTicks:  false,		// draw micro ticks
			htmlLabels:  true		// use HTML to draw labels
		},
		optionalParams: {
			"min":           0,		// minimal value on this axis
			"max":           1,		// maximal value on this axis
			"majorTickStep": 4,		// major tick step
			"minorTickStep": 2,		// minor tick step
			"microTickStep": 1,		// micro tick step
			"labels":        [],	// array of labels for major ticks
									// with corresponding numeric values
									// ordered by values
			// theme components
			"stroke":        {},	// stroke for an axis
			"majorTick":     {},	// stroke + length for a tick
			"minorTick":     {},	// stroke + length for a tick
			"font":          "",	// font for labels
			"fontColor":     ""		// color for labels as a string
		},

		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
		},
		dependOnData: function(){
			return !("min" in this.opt) || !("max" in this.opt);
		},
		clear: function(){
			delete this.scaler;
			this.dirty = true;
			return this;
		},
		initialized: function(){
			return "scaler" in this;
		},
		calculate: function(min, max, span, labels){
			if(this.initialized()){ return this; }
			this.labels = "labels" in this.opt ? this.opt.labels : labels;
			if("min" in this.opt){ min = this.opt.min; }
			if("max" in this.opt){ max = this.opt.max; }
			if(this.opt.includeZero){
				if(min > 0){ min = 0; }
				if(max < 0){ max = 0; }
			}
			var minMinorStep = 0, ta = this.chart.theme.axis, 
				taFont = "font" in this.opt ? this.opt.font : ta.font,
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
			if(this.vertical){
				if(size){
					minMinorStep = size + labelGap;
				}
			}else{
				if(size){
					var labelLength = Math.ceil(Math.log(Math.max(Math.abs(min), Math.abs(max))) / Math.LN10);
					if(min < 0 || max < 0){ ++labelLength; }
					var precision = Math.floor(Math.log(max - min) / Math.LN10);
					if(precision > 0){ labelLength += precision; }
					if(this.labels){
						labelLength = df.foldl(df.map(this.labels, "x.text.length"), "Math.max(a, b)", labelLength);
					}
					minMinorStep = Math.floor(size * labelLength * labelFudgeFactor) + labelGap;
				}
			}
			var kwArgs = {
				fixUpper: this.opt.fixUpper, 
				fixLower: this.opt.fixLower, 
				natural:  this.opt.natural
			};
			if("majorTickStep" in this.opt){ kwArgs.majorTick = this.opt.majorTickStep; }
			if("minorTickStep" in this.opt){ kwArgs.minorTick = this.opt.minorTickStep; }
			if("microTickStep" in this.opt){ kwArgs.microTick = this.opt.microTickStep; }
			this.scaler = dojox.charting.scaler(min, max, span, kwArgs);
			this.scaler.minMinorStep = minMinorStep;
			return this;
		},
		getScaler: function(){
			return this.scaler;
		},
		getOffsets: function(){
			var offsets = {l: 0, r: 0, t: 0, b: 0};
			var offset = 0, ta = this.chart.theme.axis,
				taFont = "font" in this.opt ? this.opt.font : ta.font,
				taMajorTick = "majorTick" in this.opt ? this.opt.majorTick : ta.majorTick,
				taMinorTick = "minorTick" in this.opt ? this.opt.minorTick : ta.minorTick,
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
			if(this.vertical){
				if(size){
					var s = this.scaler,
						a = this._getLabel(s.major.start, s.major.prec).length,
						b = this._getLabel(s.major.start + s.major.count * s.major.tick, s.major.prec).length,
						c = this._getLabel(s.minor.start, s.minor.prec).length,
						d = this._getLabel(s.minor.start + s.minor.count * s.minor.tick, s.minor.prec).length,
						labelLength = Math.max(a, b, c, d);
					if(this.labels){
						labelLength = df.foldl(df.map(this.labels, "x.text.length"), "Math.max(a, b)", labelLength);
					}
					offset = Math.floor(size * labelLength * labelFudgeFactor) + labelGap;
				}
				offset += labelGap + Math.max(taMajorTick.length, taMinorTick.length);
				offsets[this.opt.leftBottom ? "l" : "r"] = offset;
				offsets.t = offsets.b = size / 2;
			}else{
				if(size){
					offset = size + labelGap;
				}
				offset += labelGap + Math.max(taMajorTick.length, taMinorTick.length);
				offsets[this.opt.leftBottom ? "b" : "t"] = offset;
				if(size){
					var s = this.scaler,
						a = this._getLabel(s.major.start, s.major.prec).length,
						b = this._getLabel(s.major.start + s.major.count * s.major.tick, s.major.prec).length,
						c = this._getLabel(s.minor.start, s.minor.prec).length,
						d = this._getLabel(s.minor.start + s.minor.count * s.minor.tick, s.minor.prec).length,
						labelLength = Math.max(a, b, c, d);
					if(this.labels){
						labelLength = df.foldl(df.map(this.labels, "x.text.length"), "Math.max(a, b)", labelLength);
					}
					offsets.l = offsets.r = Math.floor(size * labelLength * labelFudgeFactor) / 2;
				}
			}
			return offsets;
		},
		render: function(dim, offsets){
			if(!this.dirty){ return this; }
			// prepare variable
			var start, stop, axisVector, tickVector, labelOffset, labelAlign,
				ta = this.chart.theme.axis, 
				taStroke = "stroke" in this.opt ? this.opt.stroke : ta.stroke,
				taMajorTick = "majorTick" in this.opt ? this.opt.majorTick : ta.majorTick,
				taMinorTick = "minorTick" in this.opt ? this.opt.minorTick : ta.minorTick,
				taFont = "font" in this.opt ? this.opt.font : ta.font,
				taFontColor = "fontColor" in this.opt ? this.opt.fontColor : ta.fontColor,
				tickSize = Math.max(taMajorTick.length, taMinorTick.length),
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
			if(this.vertical){
				start = {y: dim.height - offsets.b};
				stop  = {y: offsets.t};
				axisVector = {x: 0, y: -1};
				if(this.opt.leftBottom){
					start.x = stop.x = offsets.l;
					tickVector = {x: -1, y: 0};
					labelAlign = "end";
				}else{
					start.x = stop.x = dim.width - offsets.r;
					tickVector = {x: 1, y: 0};
					labelAlign = "start";
				}
				labelOffset = {x: tickVector.x * (tickSize + labelGap), y: size * 0.4};
			}else{
				start = {x: offsets.l};
				stop  = {x: dim.width - offsets.r};
				axisVector = {x: 1, y: 0};
				labelAlign = "middle";
				if(this.opt.leftBottom){
					start.y = stop.y = dim.height - offsets.b;
					tickVector = {x: 0, y: 1};
					labelOffset = {y: tickSize + labelGap + size};
				}else{
					start.y = stop.y = offsets.t;
					tickVector = {x: 0, y: -1};
					labelOffset = {y: -tickSize - labelGap};
				}
				labelOffset.x = 0;
			}
			
			// render shapes
			this.cleanGroup();
			var s = this.group, c = this.scaler, step, next,
				nextMajor = c.major.start, nextMinor = c.minor.start, nextMicro = c.micro.start;
			s.createLine({x1: start.x, y1: start.y, x2: stop.x, y2: stop.y}).setStroke(taStroke);
			if(this.opt.microTicks && c.micro.tick){
				step = c.micro.tick, next = nextMicro;
			}else if(this.opt.minorTicks && c.minor.tick){
				step = c.minor.tick, next = nextMinor;
			}else if(c.major.tick){
				step = c.major.tick, next = nextMajor;
			}else{
				// don't draw anything
				return this;
			}
			while(next <= c.bounds.upper + 1/c.scale){
				var offset = (next - c.bounds.lower) * c.scale,
					x = start.x + axisVector.x * offset,
					y = start.y + axisVector.y * offset;
				if(Math.abs(nextMajor - next) < step / 2){
					// major tick
					s.createLine({
						x1: x, y1: y,
						x2: x + tickVector.x * taMajorTick.length,
						y2: y + tickVector.y * taMajorTick.length
					}).setStroke(taMajorTick);
					if(this.opt.majorLabels){
						var elem = dc.axis2d.common.createText[this.opt.htmlLabels ? "html" : "gfx"]
										(this.chart, s, x + labelOffset.x, y + labelOffset.y, labelAlign,
											this._getLabel(nextMajor, c.major.prec), taFont, taFontColor);
						if(this.opt.htmlLabels){ this.htmlElements.push(elem); }
					}
					nextMajor += c.major.tick;
					nextMinor += c.minor.tick;
					nextMicro += c.micro.tick;
				}else if(Math.abs(nextMinor - next) < step / 2){
					// minor tick
					if(this.opt.minorTicks){
						s.createLine({
							x1: x, y1: y,
							x2: x + tickVector.x * taMinorTick.length,
							y2: y + tickVector.y * taMinorTick.length
						}).setStroke(taMinorTick);
						if(this.opt.minorLabels && (c.minMinorStep <= c.minor.tick * c.scale)){
							var elem = dc.axis2d.common.createText[this.opt.htmlLabels ? "html" : "gfx"]
											(this.chart, s, x + labelOffset.x, y + labelOffset.y, labelAlign,
												this._getLabel(nextMinor, c.minor.prec), taFont, taFontColor);
							if(this.opt.htmlLabels){ this.htmlElements.push(elem); }
						}
					}
					nextMinor += c.minor.tick;
					nextMicro += c.micro.tick;
				}else{
					// micro tick
					if(this.opt.microTicks){
						s.createLine({
							x1: x, y1: y,
							// use minor ticks for now
							x2: x + tickVector.x * taMinorTick.length,
							y2: y + tickVector.y * taMinorTick.length
						}).setStroke(taMinorTick);
					}
					nextMicro += c.micro.tick;
				}
				next += step;
			}
			this.dirty = false;
			return this;
		},
		
		// utilities
		_getLabel: function(number, precision){
			if(this.opt.labels){
				// classic binary search
				var l = this.opt.labels, lo = 0, hi = l.length;
				while(lo < hi){
					var mid = Math.floor((lo + hi) / 2), val = l[mid].value;
					if(val < number){
						lo = mid + 1;
					}else{
						hi = mid;
					}
				}
				// lets take into account FP errors
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				--lo;
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				lo += 2;
				if(lo < l.length && eq(l[lo].value, number)){
					return l[lo].text;
				}
				// otherwise we will produce a number
			}
			return this.opt.fixed ? number.toFixed(precision < 0 ? -precision : 0) : number.toString();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.common"] = true;
dojo.provide("dojox.charting.plot2d.common");





(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common;
	
	dojo.mixin(dojox.charting.plot2d.common, {
		makeStroke: function(stroke){
			if(!stroke){ return stroke; }
			if(typeof stroke == "string" || stroke instanceof dojo.Color){
				stroke = {color: stroke};
			}
			return dojox.gfx.makeParameters(dojox.gfx.defaultStroke, stroke);
		},
		augmentColor: function(target, color){
			var t = new dojo.Color(target),
				c = new dojo.Color(color);
			c.a = t.a;
			return c;
		},
		augmentStroke: function(stroke, color){
			var s = dc.makeStroke(stroke);
			if(s){
				s.color = dc.augmentColor(s.color, color);
			}
			return s;
		},
		augmentFill: function(fill, color){
			var fc, c = new dojo.Color(color);
			if(typeof fill == "string" || fill instanceof dojo.Color){
				return dc.augmentColor(fill, color);
			}
			return fill;
		},
		
		defaultStats: {hmin: Number.POSITIVE_INFINITY, hmax: Number.NEGATIVE_INFINITY, 
			vmin: Number.POSITIVE_INFINITY, vmax: Number.NEGATIVE_INFINITY},
		
		collectSimpleStats: function(series){
			var stats = dojo.clone(dc.defaultStats);
			for(var i = 0; i < series.length; ++i){
				var run = series[i];
				if(!run.data.length){ continue; }
				if(typeof run.data[0] == "number"){
					// 1D case
					var old_vmin = stats.vmin, old_vmax = stats.vmax;
					if(!("ymin" in run) || !("ymax" in run)){
						dojo.forEach(run.data, function(val, i){
							var x = i + 1, y = val;
							if(isNaN(y)){ y = 0; }
							stats.hmin = Math.min(stats.hmin, x);
							stats.hmax = Math.max(stats.hmax, x);
							stats.vmin = Math.min(stats.vmin, y);
							stats.vmax = Math.max(stats.vmax, y);
						});
					}
					if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
					if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
				}else{
					// 2D case
					var old_hmin = stats.hmin, old_hmax = stats.hmax,
						old_vmin = stats.vmin, old_vmax = stats.vmax;
					if(!("xmin" in run) || !("xmax" in run) || !("ymin" in run) || !("ymax" in run)){
						dojo.forEach(run.data, function(val, i){
							var x = val.x, y = val.y;
							if(isNaN(x)){ x = 0; }
							if(isNaN(y)){ y = 0; }
							stats.hmin = Math.min(stats.hmin, x);
							stats.hmax = Math.max(stats.hmax, x);
							stats.vmin = Math.min(stats.vmin, y);
							stats.vmax = Math.max(stats.vmax, y);
						});
					}
					if("xmin" in run){ stats.hmin = Math.min(old_hmin, run.xmin); }
					if("xmax" in run){ stats.hmax = Math.max(old_hmax, run.xmax); }
					if("ymin" in run){ stats.vmin = Math.min(old_vmin, run.ymin); }
					if("ymax" in run){ stats.vmax = Math.max(old_vmax, run.ymax); }
				}
			}
			return stats;
		},
		
		collectStackedStats: function(series){
			// collect statistics
			var stats = dojo.clone(dc.defaultStats);
			if(series.length){
				// 1st pass: find the maximal length of runs
				stats.hmin = Math.min(stats.hmin, 1);
				stats.hmax = df.foldl(series, "seed, run -> Math.max(seed, run.data.length)", stats.hmax);
				// 2nd pass: stack values
				for(var i = 0; i < stats.hmax; ++i){
					var v = series[0].data[i];
					if(isNaN(v)){ v = 0; }
					stats.vmin = Math.min(stats.vmin, v);
					for(var j = 1; j < series.length; ++j){
						var t = series[j].data[i];
						if(isNaN(t)){ t = 0; }
						v += t;
					}
					stats.vmax = Math.max(stats.vmax, v);
				}
			}
			return stats;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Base"] = true;
dojo.provide("dojox.charting.plot2d.Base");




dojo.declare("dojox.charting.plot2d.Base", dojox.charting.Element, {
	clear: function(){
		this.series = [];
		this._hAxis = null;
		this._vAxis = null;
		this.dirty = true;
		return this;
	},
	setAxis: function(axis){
		if(axis){
			this[axis.vertical ? "_vAxis" : "_hAxis"] = axis;
		}
		return this;
	},
	addSeries: function(run){
		this.series.push(run);
		return this;
	},
	calculateAxes: function(dim){
		return this;
	},
	render: function(dim, offsets){
		return this;
	},
	getRequiredColors: function(){
		return this.series.length;
	},
	
	// utilities
	_calc: function(dim, stats){
		// calculate scaler
		if(this._hAxis){
			if(!this._hAxis.initialized()){
				this._hAxis.calculate(stats.hmin, stats.hmax, dim.width);
			}
			this._hScaler = this._hAxis.getScaler();
		}else{
			this._hScaler = {bounds: {lower: stats.hmin, upper: stats.hmax}, 
				scale: dim.width / (stats.hmax - stats.hmin)};
		}
		if(this._vAxis){
			if(!this._vAxis.initialized()){
				this._vAxis.calculate(stats.vmin, stats.vmax, dim.height);
			}
			this._vScaler = this._vAxis.getScaler();
		}else{
			this._vScaler = {bounds: {lower: stats.vmin, upper: stats.vmax}, 
				scale: dim.height / (stats.vmax - stats.vmin)};
		}
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Default"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Default"] = true;
dojo.provide("dojox.charting.plot2d.Default");







(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Default", dojox.charting.plot2d.Base, {
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			lines:   true,	// draw lines
			areas:   false,	// draw areas
			markers: false,	// draw markers
			shadows: 0		// draw shadows
		},
		optionalParams: {},	// no optional parameters
		
		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
		},
		
		calculateAxes: function(dim){
			this._calc(dim, dc.collectSimpleStats(this.series));
			return this;
		},
		render: function(dim, offsets){
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, stroke, outline, color, marker;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				if(!run.data.length){
					run.dirty = false;
					continue;
				}
				var s = run.group, lpoly;
				if(typeof run.data[0] == "number"){
					lpoly = dojo.map(run.data, function(v, i){
						return {
							x: this._hScaler.scale * (i + 1 - this._hScaler.bounds.lower) + offsets.l,
							y: dim.height - offsets.b - this._vScaler.scale * (v - this._vScaler.bounds.lower)
						};
					}, this);
				}else{
					lpoly = dojo.map(run.data, function(v, i){
						return {
							x: this._hScaler.scale * (v.x - this._hScaler.bounds.lower) + offsets.l,
							y: dim.height - offsets.b - this._vScaler.scale * (v.y - this._vScaler.bounds.lower)
						};
					}, this);
				}
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				if(this.opt.areas){
					var apoly = dojo.clone(lpoly);
					apoly.push({x: lpoly[lpoly.length - 1].x, y: dim.height - offsets.b});
					apoly.push({x: lpoly[0].x, y: dim.height - offsets.b});
					apoly.push(lpoly[0]);
					var fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
					run.dyn.fill = s.createPolyline(apoly).setFill(fill).getFill();
				}
				if(this.opt.lines || this.opt.markers){
					// need a stroke
					stroke = run.stroke ? dc.makeStroke(run.stroke) : dc.augmentStroke(t.series.stroke, color);
					if(run.outline || t.series.outline){
						outline = dc.makeStroke(run.outline ? run.outline : t.series.outline);
						outline.width = 2 * outline.width + stroke.width;
					}
				}
				if(this.opt.markers){
					// need a marker
					marker = run.dyn.marker = run.marker ? run.marker : t.next("marker");
				}
				if(this.opt.shadows && stroke){
					var sh = this.opt.shadows, shadowColor = new dojo.Color([0, 0, 0, 0.3]),
						spoly = dojo.map(lpoly, function(c){
							return {x: c.x + sh.dx, y: c.y + sh.dy};
						}),
						shadowStroke = dojo.clone(outline ? outline : stroke);
					shadowStroke.color = shadowColor;
					shadowStroke.width += sh.dw ? sh.dw : 0;
					if(this.opt.lines){
						s.createPolyline(spoly).setStroke(shadowStroke);
					}
					if(this.opt.markers){
						dojo.forEach(spoly, function(c){
							s.createPath("M" + c.x + " " + c.y + " " + marker).setStroke(shadowStroke).setFill(shadowColor);
						}, this);
					}
				}
				if(this.opt.lines){
					if(outline){
						run.dyn.outline = s.createPolyline(lpoly).setStroke(outline).getStroke();
					}
					run.dyn.stroke = s.createPolyline(lpoly).setStroke(stroke).getStroke();
				}
				if(this.opt.markers){
					dojo.forEach(lpoly, function(c){
						var path = "M" + c.x + " " + c.y + " " + marker;
						if(outline){
							s.createPath(path).setStroke(outline);
						}
						s.createPath(path).setStroke(stroke).setFill(stroke.color);
					}, this);
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Lines"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Lines"] = true;
dojo.provide("dojox.charting.plot2d.Lines");



dojo.declare("dojox.charting.plot2d.Lines", dojox.charting.plot2d.Default, {
	constructor: function(){
		this.opt.lines = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Areas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Areas"] = true;
dojo.provide("dojox.charting.plot2d.Areas");



dojo.declare("dojox.charting.plot2d.Areas", dojox.charting.plot2d.Default, {
	constructor: function(){
		this.opt.lines = true;
		this.opt.areas = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Markers"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Markers"] = true;
dojo.provide("dojox.charting.plot2d.Markers");



dojo.declare("dojox.charting.plot2d.Markers", dojox.charting.plot2d.Default, {
	constructor: function(){
		this.opt.markers = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.MarkersOnly"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.MarkersOnly"] = true;
dojo.provide("dojox.charting.plot2d.MarkersOnly");



dojo.declare("dojox.charting.plot2d.MarkersOnly", dojox.charting.plot2d.Default, {
	constructor: function(){
		this.opt.lines   = false;
		this.opt.markers = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Scatter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Scatter"] = true;
dojo.provide("dojox.charting.plot2d.Scatter");



dojo.declare("dojox.charting.plot2d.Scatter", dojox.charting.plot2d.Default, {
	constructor: function(){
		this.opt.lines   = false;
		this.opt.markers = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Stacked"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Stacked"] = true;
dojo.provide("dojox.charting.plot2d.Stacked");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Stacked", dojox.charting.plot2d.Default, {
		calculateAxes: function(dim){
			var stats = dc.collectStackedStats(this.series);
			this._maxRunLength = stats.hmax;
			this._calc(dim, stats);
			return this;
		},
		render: function(dim, offsets){
			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] += v;
				}
			}
			// draw runs in backwards
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, stroke, outline, color, marker;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group,
					lpoly = dojo.map(acc, function(v, i){
						return {
							x: this._hScaler.scale * (i + 1 - this._hScaler.bounds.lower) + offsets.l,
							y: dim.height - offsets.b - this._vScaler.scale * (v - this._vScaler.bounds.lower)
						};
					}, this);
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = new dojo.Color(t.next("color"));
				}
				if(this.opt.areas){
					var apoly = dojo.clone(lpoly);
					apoly.push({x: lpoly[lpoly.length - 1].x, y: dim.height - offsets.b});
					apoly.push({x: lpoly[0].x, y: dim.height - offsets.b});
					apoly.push(lpoly[0]);
					var fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
					s.createPolyline(apoly).setFill(fill);
				}
				if(this.opt.lines || this.opt.markers){
					// need a stroke
					stroke = run.stroke ? dc.makeStroke(run.stroke) : dc.augmentStroke(t.series.stroke, color);
					if(run.outline || t.series.outline){
						outline = dc.makeStroke(run.outline ? run.outline : t.series.outline);
						outline.width = 2 * outline.width + stroke.width;
					}
				}
				if(this.opt.markers){
					// need a marker
					marker = run.marker ? run.marker : t.next("marker");
				}
				if(this.opt.shadows && stroke){
					var sh = this.opt.shadows, shadowColor = new dojo.Color([0, 0, 0, 0.3]),
						spoly = dojo.map(lpoly, function(c){
							return {x: c.x + sh.dx, y: c.y + sh.dy};
						}),
						shadowStroke = dojo.clone(outline ? outline : stroke);
					shadowStroke.color = shadowColor;
					shadowStroke.width += sh.dw ? sh.dw : 0;
					if(this.opt.lines){
						s.createPolyline(spoly).setStroke(shadowStroke);
					}
					if(this.opt.markers){
						dojo.forEach(spoly, function(c){
							s.createPath("M" + c.x + " " + c.y + " " + marker).setStroke(shadowStroke).setFill(shadowColor);
						}, this);
					}
				}
				if(this.opt.lines){
					if(outline){
						s.createPolyline(lpoly).setStroke(outline);
					}
					s.createPolyline(lpoly).setStroke(stroke);
				}
				if(this.opt.markers){
					dojo.forEach(lpoly, function(c){
						var path = "M" + c.x + " " + c.y + " " + marker;
						if(outline){
							s.createPath(path).setStroke(outline);
						}
						s.createPath(path).setStroke(stroke).setFill(stroke.color);
					}, this);
				}
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] -= v;
				}
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedLines"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedLines"] = true;
dojo.provide("dojox.charting.plot2d.StackedLines");



dojo.declare("dojox.charting.plot2d.StackedLines", dojox.charting.plot2d.Stacked, {
	constructor: function(){
		this.opt.lines = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedAreas"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedAreas"] = true;
dojo.provide("dojox.charting.plot2d.StackedAreas");



dojo.declare("dojox.charting.plot2d.StackedAreas", dojox.charting.plot2d.Stacked, {
	constructor: function(){
		this.opt.lines = true;
		this.opt.areas = true;
	}
});

}

if(!dojo._hasResource["dojox.charting.plot2d.Columns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Columns"] = true;
dojo.provide("dojox.charting.plot2d.Columns");







(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Columns", dojox.charting.plot2d.Base, {
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			shadows: null	// draw shadows
		},
		optionalParams: {},	// no optional parameters
		
		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
		},
		
		calculateAxes: function(dim){
			var stats = dc.collectSimpleStats(this.series);
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			this._calc(dim, stats);
			return this;
		},
		render: function(dim, offsets){
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._hScaler.scale / 3 ? this.opt.gap : 0;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				var baseline = Math.max(0, this._vScaler.bounds.lower),
					xoff = offsets.l + this._hScaler.scale * (0.5 - this._hScaler.bounds.lower) + gap,
					yoff = dim.height - offsets.b - this._vScaler.scale * (baseline - this._vScaler.bounds.lower);
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j], 
						width  = this._hScaler.scale - 2 * gap,
						height = this._vScaler.scale * (v - baseline),
						h = Math.abs(height);
					if(width >= 1 && h >= 1){
						var rect = {
								x: xoff + this._hScaler.scale * j,
								y: yoff - (height < 0 ? 0 : height),
								width: width, height: h
							},
							shape = s.createRect(rect).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedColumns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedColumns"] = true;
dojo.provide("dojox.charting.plot2d.StackedColumns");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.StackedColumns", dojox.charting.plot2d.Columns, {
		calculateAxes: function(dim){
			var stats = dc.collectStackedStats(this.series);
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			this._calc(dim, stats);
			return this;
		},
		render: function(dim, offsets){
			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] += v;
				}
			}
			// draw runs in backwards
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._hScaler.scale / 3 ? this.opt.gap : 0;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				for(var j = 0; j < acc.length; ++j){
					var v = acc[j],
						width  = this._hScaler.scale - 2 * gap,
						height = this._vScaler.scale * (v - this._vScaler.bounds.lower);
					if(width >= 1 && height >= 1){
						var shape = s.createRect({
							x: offsets.l + this._hScaler.scale * (j + 0.5 - this._hScaler.bounds.lower) + gap,
							y: dim.height - offsets.b - this._vScaler.scale * (v - this._vScaler.bounds.lower),
							width: width, height: height
						}).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] -= v;
				}
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.ClusteredColumns"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.ClusteredColumns"] = true;
dojo.provide("dojox.charting.plot2d.ClusteredColumns");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.ClusteredColumns", dojox.charting.plot2d.Columns, {
		render: function(dim, offsets){
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._hScaler.scale / 3 ? this.opt.gap : 0,
				thickness = (this._hScaler.scale - 2 * gap) / this.series.length;
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				var baseline = Math.max(0, this._vScaler.bounds.lower),
					xoff = offsets.l + this._hScaler.scale * (0.5 - this._hScaler.bounds.lower) + gap + thickness * i,
					yoff = dim.height - offsets.b - this._vScaler.scale * (baseline - this._vScaler.bounds.lower);
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j],
						width  = thickness,
						height = this._vScaler.scale * (v - baseline),
						h = Math.abs(height);
					if(width >= 1 && h >= 1){
						var shape = s.createRect({
							x: xoff + this._hScaler.scale * j,
							y: yoff - (height < 0 ? 0 : height),
							width: width, height: h
						}).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Bars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Bars"] = true;
dojo.provide("dojox.charting.plot2d.Bars");







(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.Bars", dojox.charting.plot2d.Base, {
		defaultParams: {
			hAxis: "x",		// use a horizontal axis named "x"
			vAxis: "y",		// use a vertical axis named "y"
			gap:	0,		// gap between columns in pixels
			shadows: null	// draw shadows
		},
		optionalParams: {},	// no optional parameters
		
		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.series = [];
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
		},
		
		calculateAxes: function(dim){
			var stats = dc.collectSimpleStats(this.series), t;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;
			this._calc(dim, stats);
			return this;
		},
		render: function(dim, offsets){
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._vScaler.scale / 3 ? this.opt.gap : 0;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				var baseline = Math.max(0, this._hScaler.bounds.lower),
					xoff = offsets.l + this._hScaler.scale * (baseline - this._hScaler.bounds.lower),
					yoff = dim.height - offsets.b - this._vScaler.scale * (1.5 - this._vScaler.bounds.lower) + gap;
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j],
						width  = this._hScaler.scale * (v - baseline),
						height = this._vScaler.scale - 2 * gap,
						w = Math.abs(width);
					if(w >= 1 && height >= 1){
						var shape = s.createRect({
							x: xoff + (width < 0 ? width : 0),
							y: yoff - this._vScaler.scale * j,
							width: w, height: height
						}).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.StackedBars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.StackedBars"] = true;
dojo.provide("dojox.charting.plot2d.StackedBars");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.StackedBars", dojox.charting.plot2d.Bars, {
		calculateAxes: function(dim){
			var stats = dc.collectStackedStats(this.series), t;
			this._maxRunLength = stats.hmax;
			stats.hmin -= 0.5;
			stats.hmax += 0.5;
			t = stats.hmin, stats.hmin = stats.vmin, stats.vmin = t;
			t = stats.hmax, stats.hmax = stats.vmax, stats.vmax = t;
			this._calc(dim, stats);
			return this;
		},
		render: function(dim, offsets){
			// stack all values
			var acc = df.repeat(this._maxRunLength, "-> 0", 0);
			for(var i = 0; i < this.series.length; ++i){
				var run = this.series[i];
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] += v;
				}
			}
			// draw runs in backwards
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._vScaler.scale / 3 ? this.opt.gap : 0;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				for(var j = 0; j < acc.length; ++j){
					var v = acc[j],
						width  = this._hScaler.scale * (v - this._hScaler.bounds.lower),
						height = this._vScaler.scale - 2 * gap;
					if(width >= 1 && height >= 1){
						var shape = s.createRect({
							x: offsets.l,
							y: dim.height - offsets.b - this._vScaler.scale * (j + 1.5 - this._vScaler.bounds.lower) + gap,
							width: width, height: height
						}).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
				// update the accumulator
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j];
					if(isNaN(v)){ v = 0; }
					acc[j] -= v;
				}
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.ClusteredBars"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.ClusteredBars"] = true;
dojo.provide("dojox.charting.plot2d.ClusteredBars");






(function(){
	var df = dojox.lang.functional, dc = dojox.charting.plot2d.common,
		purgeGroup = df.lambda("item.purgeGroup()");

	dojo.declare("dojox.charting.plot2d.ClusteredBars", dojox.charting.plot2d.Bars, {
		render: function(dim, offsets){
			if(this.dirty){
				dojo.forEach(this.series, purgeGroup);
				this.cleanGroup();
				var s = this.group;
				df.forEachReversed(this.series, function(item){ item.cleanGroup(s); });
			}
			var t = this.chart.theme, color, stroke, fill, f,
				gap = this.opt.gap < this._vScaler.scale / 3 ? this.opt.gap : 0,
				thickness = (this._vScaler.scale - 2 * gap) / this.series.length;
			for(var i = this.series.length - 1; i >= 0; --i){
				var run = this.series[i];
				if(!this.dirty && !run.dirty){ continue; }
				run.cleanGroup();
				var s = run.group;
				if(!run.fill || !run.stroke){
					// need autogenerated color
					color = run.dyn.color = new dojo.Color(t.next("color"));
				}
				stroke = run.stroke ? run.stroke : dc.augmentStroke(t.series.stroke, color);
				fill = run.fill ? run.fill : dc.augmentFill(t.series.fill, color);
				var baseline = Math.max(0, this._hScaler.bounds.lower),
					xoff = offsets.l + this._hScaler.scale * (baseline - this._hScaler.bounds.lower),
					yoff = dim.height - offsets.b - this._vScaler.scale * (1.5 - this._vScaler.bounds.lower) + 
						gap + thickness * (this.series.length - i - 1);
				for(var j = 0; j < run.data.length; ++j){
					var v = run.data[j],
						width  = this._hScaler.scale * (v - baseline),
						height = thickness, w = Math.abs(width);
					if(w >= 1 && height >= 1){
						var shape = s.createRect({
							x: xoff + (width < 0 ? width : 0),
							y: yoff - this._vScaler.scale * j,
							width: w, height: height
						}).setFill(fill).setStroke(stroke);
						run.dyn.fill   = shape.getFill();
						run.dyn.stroke = shape.getStroke();
					}
				}
				run.dirty = false;
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Grid"] = true;
dojo.provide("dojox.charting.plot2d.Grid");





(function(){
	var du = dojox.lang.utils;

	dojo.declare("dojox.charting.plot2d.Grid", dojox.charting.Element, {
		defaultParams: {
			hAxis: "x",			// use a horizontal axis named "x"
			vAxis: "y",			// use a vertical axis named "y"
			hMajorLines: true,	// draw horizontal major lines
			hMinorLines: false,	// draw horizontal minor lines
			vMajorLines: true,	// draw vertical major lines
			vMinorLines: false,	// draw vertical minor lines
			hStripes: "none",	// TBD
			vStripes: "none"	// TBD
		},
		optionalParams: {},	// no optional parameters
		
		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			this.hAxis = this.opt.hAxis;
			this.vAxis = this.opt.vAxis;
		},
		clear: function(){
			this._hAxis = null;
			this._vAxis = null;
			this.dirty = true;
			return this;
		},
		setAxis: function(axis){
			if(axis){
				this[axis.vertical ? "_vAxis" : "_hAxis"] = axis;
			}
			return this;
		},
		addSeries: function(run){
			// nothing
			return this;
		},
		calculateAxes: function(dim){
			// nothing
			return this;
		},
		getRequiredColors: function(){
			return 0;
		},
		render: function(dim, offsets){
			// draw horizontal stripes and lines
			if(!this.dirty){ return this; }
			this.cleanGroup();
			var s = this.group, ta = this.chart.theme.axis,
				scaler = this._vAxis.getScaler();
			if(this.opt.hMinorLines && scaler.minor.tick){
				for(var i = 0; i < scaler.minor.count; ++i){
					var y = dim.height - offsets.b - scaler.scale * 
							(scaler.minor.start - scaler.bounds.lower + i * scaler.minor.tick);
					s.createLine({
						x1: offsets.l,
						y1: y,
						x2: dim.width - offsets.r,
						y2: y
					}).setStroke(ta.minorTick);
				}
			}
			if(this.opt.hMajorLines && scaler.major.tick){
				for(var i = 0; i < scaler.major.count; ++i){
					var y = dim.height - offsets.b - scaler.scale * 
							(scaler.major.start - scaler.bounds.lower + i * scaler.major.tick);
					s.createLine({
						x1: offsets.l,
						y1: y,
						x2: dim.width - offsets.r,
						y2: y
					}).setStroke(ta.majorTick);
				}
			}
			// draw vertical stripes and lines
			scaler = this._hAxis.getScaler();
			if(this.opt.vMinorLines && scaler.minor.tick){
				for(var i = 0; i < scaler.minor.count; ++i){
					var x = offsets.l + scaler.scale * 
							(scaler.minor.start - scaler.bounds.lower + i * scaler.minor.tick);
					s.createLine({
						x1: x,
						y1: offsets.t,
						x2: x,
						y2: dim.height - offsets.b
					}).setStroke(ta.minorTick);
				}
			}
			if(this.opt.vMajorLines && scaler.major.tick){
				for(var i = 0; i < scaler.major.count; ++i){
					var x = offsets.l + scaler.scale * 
							(scaler.major.start - scaler.bounds.lower + i * scaler.major.tick);
					s.createLine({
						x1: x,
						y1: offsets.t,
						x2: x,
						y2: dim.height - offsets.b
					}).setStroke(ta.majorTick);
				}
			}
			this.dirty = false;
			return this;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.plot2d.Pie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.plot2d.Pie"] = true;
dojo.provide("dojox.charting.plot2d.Pie");








(function(){
	var df = dojox.lang.functional, du = dojox.lang.utils,
		dc = dojox.charting.plot2d.common,
		da = dojox.charting.axis2d.common,
		g = dojox.gfx,
		labelFudgeFactor = 0.8;		// in percents (to convert font's heigth to label width)

	dojo.declare("dojox.charting.plot2d.Pie", dojox.charting.Element, {
		defaultParams: {
			labels:			true,
			ticks:			false,
			fixed:			true,
			precision:		1,
			labelOffset:	20,
			labelStyle:		"default",	// default/rows/auto
			htmlLabels:		true		// use HTML to draw labels
		},
		optionalParams: {
			font:		"",
			fontColor:	"",
			radius:		0
		},

		constructor: function(chart, kwArgs){
			this.opt = dojo.clone(this.defaultParams);
			du.updateWithObject(this.opt, kwArgs);
			du.updateWithPattern(this.opt, kwArgs, this.optionalParams);
			this.run = null;
			this.dyn = [];
		},
		clear: function(){
			this.dirty = true;
			this.dyn = [];
			return this;
		},
		setAxis: function(axis){
			// nothing
			return this;
		},
		addSeries: function(run){
			this.run = run;
			return this;
		},
		calculateAxes: function(dim){
			// nothing
			return this;
		},
		getRequiredColors: function(){
			return this.run ? this.run.data.length : 0;
		},
		render: function(dim, offsets){
			if(!this.dirty){ return this; }
			this.dirty = false;
			this.cleanGroup();
			var s = this.group, color, t = this.chart.theme;

			// calculate the geometry
			var rx = (dim.width  - offsets.l - offsets.r) / 2,
				ry = (dim.height - offsets.t - offsets.b) / 2,
				r  = Math.min(rx, ry),
				taFont = "font" in this.opt ? this.opt.font : t.axis.font,
				taFontColor = "fontColor" in this.opt ? this.opt.fontColor : t.axis.fontColor,
				sum = df.foldl1(this.run.data, "+"), start = 0, step,
				slices = dojo.map(this.run.data, function(x){ return x / sum; }),
				shift, size, labelR;
			if(this.opt.labels){
				var labels = dojo.map(slices, function(x){
					return this._getLabel(x * 100) + "%";
				}, this);
				shift = df.foldl1(dojo.map(labels, df.pluck("length")), "x, y -> Math.max(x, y)");
				size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
				shift = Math.max(shift * labelFudgeFactor, 1) / 2 * size;
				if(this.opt.labelOffset < 0){
					r = Math.min(rx - 2 * shift, ry - size) + this.opt.labelOffset;
				}
				labelR = r - this.opt.labelOffset;
			}
			if("radius" in this.opt){
				r = this.opt.radius;
				labelR = r - this.opt.labelOffset;
			}
			var	circle = {
					cx: offsets.l + rx,
					cy: offsets.t + ry,
					r:  r
				};

			this.dyn = [];			
			if(!this.run || !this.run.data.length){
				return this;
			}
			if(this.run.data.length == 1){
				// need autogenerated color
				color = new dojo.Color(t.next("color"));
				var shape = s.createCircle(circle).
						setFill(dc.augmentFill(t.run.fill, color)).
						setStroke(dc.augmentStroke(t.series.stroke, color));
				this.dyn.push({color: color, fill: shape.getFill(), stroke: shape.getStroke()});
				if(this.opt.labels){
					var shift = 4,
						taFont = "font" in this.opt ? this.opt.font : t.axis.font,
						taFontColor = "fontColor" in this.opt ? this.opt.fontColor : t.axis.fontColor,
						size = taFont ? g.normalizedLength(g.splitFontString(taFont).size) : 0;
					shift = Math.max(shift * labelFudgeFactor, 1) / 2 * size;
					// draw the label
					var elem = da.createText[this.opt.htmlLabels ? "html" : "gfx"]
									(this.chart, s, circle.cx, circle.cy + size / 2, "middle",
										"100%", taFont, taFontColor);
					if(this.opt.htmlLabels){ this.htmlElements.push(elem); }
				}
				return this;
			}
			// draw slices
			dojo.forEach(slices, function(x, i){
				// calculate the geometry of the slice
				var end = start + x * 2 * Math.PI;
				if(i + 1 == slices.length){
					end = 2 * Math.PI;
				}
				var	step = end - start,
					x1 = circle.cx + r * Math.cos(start),
					y1 = circle.cy + r * Math.sin(start),
					x2 = circle.cx + r * Math.cos(end),
					y2 = circle.cy + r * Math.sin(end);
				// draw the slice
				color = new dojo.Color(t.next("color"));
				var shape = s.createPath({}).
						moveTo(circle.cx, circle.cy).
						lineTo(x1, y1).
						arcTo(r, r, 0, step > Math.PI, true, x2, y2).
						lineTo(circle.cx, circle.cy).
						closePath().
						setFill(dc.augmentFill(t.series.fill, color)).
						setStroke(dc.augmentStroke(t.series.stroke, color));
				this.dyn.push({color: color, fill: shape.getFill(), stroke: shape.getStroke()});
				start = end;
			}, this);
			// draw labels
			if(this.opt.labels){
				start = 0;
				dojo.forEach(slices, function(x, i){
					// calculate the geometry of the slice
					var end = start + x * 2 * Math.PI;
					if(i + 1 == slices.length){
						end = 2 * Math.PI;
					}
					var	labelAngle = (start + end) / 2,
						x = circle.cx + labelR * Math.cos(labelAngle),
						y = circle.cy + labelR * Math.sin(labelAngle) + size / 2;
					// draw the label
					var elem = da.createText[this.opt.htmlLabels ? "html" : "gfx"]
									(this.chart, s, x, y, "middle",
										labels[i], taFont, taFontColor);
					if(this.opt.htmlLabels){ this.htmlElements.push(elem); }
					start = end;
				}, this);
			}
			return this;
		},
		
		// utilities
		_getLabel: function(number){
			return this.opt.fixed ? number.toFixed(this.opt.precision) : number.toString();
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.Chart2D"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.Chart2D"] = true;
dojo.provide("dojox.charting.Chart2D");



























(function(){
	var df = dojox.lang.functional, dc = dojox.charting, 
		clear = df.lambda("item.clear()"), 
		purge = df.lambda("item.purgeGroup()"),
		destroy = df.lambda("item.destroy()"),
		makeClean = df.lambda("item.dirty = false"),
		makeDirty = df.lambda("item.dirty = true");
		
	dojo.declare("dojox.charting.Chart2D", null, {
		constructor: function(node, kwArgs){
			// initialize parameters
			if(!kwArgs){ kwArgs = {}; }
			this.margins = kwArgs.margins ? kwArgs.margins : {l: 10, t: 10, r: 10, b: 10};
			this.stroke  = kwArgs.stroke;
			this.fill    = kwArgs.fill;
			
			// default initialization
			this.theme = null;
			this.axes = {};		// map of axes
			this.stack = [];	// stack of plotters
			this.plots = {};	// map of plotter indices
			this.series = [];	// stack of data runs
			this.runs = {};		// map of data run indices
			this.dirty = true;
			this.coords = null;
			
			// create a surface
			this.node = dojo.byId(node);
			var box = dojo.marginBox(node);
			this.surface = dojox.gfx.createSurface(this.node, box.w, box.h);
		},
		destroy: function(){
			dojo.forEach(this.series, destroy);
			dojo.forEach(this.stack,  destroy);
			df.forIn(this.axes, destroy);
		},
		getCoords: function(){
			if(!this.coords){
				this.coords = dojo.coords(this.node, true);
			}
			return this.coords;
		},
		setTheme: function(theme){
			this.theme = theme;
			this.dirty = true;
			return this;
		},
		addAxis: function(name, kwArgs){
			var axis;
			if(!kwArgs || !("type" in kwArgs)){
				axis = new dc.axis2d.Default(this, kwArgs);
			}else{
				axis = typeof kwArgs.type == "string" ?
					new dc.axis2d[kwArgs.type](this, kwArgs) :
					new kwArgs.type(this, kwArgs);
			}
			axis.name = name;
			axis.dirty = true;
			if(name in this.axes){
				this.axes[name].destroy();
			}
			this.axes[name] = axis;
			this.dirty = true;
			return this;
		},
		addPlot: function(name, kwArgs){
			var plot;
			if(!kwArgs || !("type" in kwArgs)){
				plot = new dc.plot2d.Default(this, kwArgs);
			}else{
				plot = typeof kwArgs.type == "string" ?
					new dc.plot2d[kwArgs.type](this, kwArgs) :
					new kwArgs.type(this, kwArgs);
			}
			plot.name = name;
			plot.dirty = true;
			if(name in this.plots){
				this.stack[this.plots[name]].destroy();
				this.stack[this.plots[name]] = plot;
			}else{
				this.plots[name] = this.stack.length;
				this.stack.push(plot);
			}
			this.dirty = true;
			return this;
		},
		addSeries: function(name, data, kwArgs){
			var run = new dc.Series(this, data, kwArgs);
			if(name in this.runs){
				this.series[this.runs[name]].destroy();
				this.series[this.runs[name]] = run;
			}else{
				this.runs[name] = this.series.length;
				this.series.push(run);
			}
			this.dirty = true;
			// fix min/max
			if(!("ymin" in run) && "min" in run){ run.ymin = run.min; }
			if(!("ymax" in run) && "max" in run){ run.ymax = run.max; }
			return this;
		},
		updateSeries: function(name, data){
			if(name in this.runs){
				var run = this.series[this.runs[name]],
					plot = this.stack[this.plots[run.plot]], axis;
				run.data = data;
				run.dirty = true;
				// check to see if axes and plot should be updated
				if(plot.hAxis){
					axis = this.axes[plot.hAxis];
					if(axis.dependOnData()){
						axis.dirty = true;
						// find all plots and mark them dirty
						dojo.forEach(this.stack, function(p){
							if(p.hAxis && p.hAxis == plot.hAxis){
								p.dirty = true;
							}
						});
					}
				}else{
					plot.dirty = true;
				}
				if(plot.vAxis){
					axis = this.axes[plot.vAxis];
					if(axis.dependOnData()){
						axis.dirty = true;
						// find all plots and mark them dirty
						dojo.forEach(this.stack, function(p){
							if(p.vAxis && p.vAxis == plot.vAxis){
								p.dirty = true;
							}
						});
					}
				}else{
					plot.dirty = true;
				}
			}
			return this;
		},
		resize: function(width, height){
			var box;
			switch(arguments.length){
				case 0:
					box = dojo.marginBox(this.node);
					break;
				case 1: 
					box = width;
					break;
				default:
					box = {w: width, h: height};
					break;
			}
			dojo.marginBox(this.node, box);
			this.surface.setDimensions(box.w, box.h);
			this.dirty = true;
			this.coords = null;
			return this.render();
		},
		render: function(){
			if(this.dirty){
				return this.fullRender();
			}
			
			// calculate geometry
			dojo.forEach(this.stack, function(plot){
				if(plot.dirty || (plot.hAxis && this.axes[plot.hAxis].dirty) ||
						(plot.vAxis && this.axes[plot.vAxis].dirty)){
					plot.calculateAxes(this.plotArea);
				}
			}, this);

			// go over the stack backwards
			df.forEachReversed(this.stack, function(plot){ plot.render(this.dim, this.offsets); }, this);
			
			// go over axes
			df.forIn(this.axes, function(axis){ axis.render(this.dim, this.offsets); }, this);

			this._makeClean();

			// BEGIN FOR HTML CANVAS 
			if(this.surface.render){ this.surface.render(); };	
			// END FOR HTML CANVAS
			
			return this;
		},
		fullRender: function(){
			this._makeDirty();
			
			// clear old values
			dojo.forEach(this.stack,  clear);
			dojo.forEach(this.series, purge);
			df.forIn(this.axes, purge);
			dojo.forEach(this.stack,  purge);
			this.surface.clear();
			
			// rebuild new connections, and add defaults
			
			// assign series
			dojo.forEach(this.series, function(run){
				if(!(run.plot in this.plots)){
					var plot = new dc.plot2d.Default(this, {});
					plot.name = run.plot;
					this.plots[run.plot] = this.stack.length;
					this.stack.push(plot);
				}
				this.stack[this.plots[run.plot]].addSeries(run);
			}, this);
			// assign axes
			dojo.forEach(this.stack, function(plot){
				if(plot.hAxis){
					plot.setAxis(this.axes[plot.hAxis]);
				}
				if(plot.vAxis){
					plot.setAxis(this.axes[plot.vAxis]);
				}
			}, this);
			// set up a theme
			if(!this.theme){
				this.theme = new dojox.charting.Theme(dojox.charting._def);
			}
			var requiredColors = df.foldl(this.stack, "z + plot.getRequiredColors()", 0);
			this.theme.defineColors({num: requiredColors, cache: false});
			
			// calculate geometry
			
			// 1st pass
			var dim = this.dim = this.surface.getDimensions();
			dim.width  = dojox.gfx.normalizedLength(dim.width);
			dim.height = dojox.gfx.normalizedLength(dim.height);
			df.forIn(this.axes, clear);
			dojo.forEach(this.stack, function(plot){ plot.calculateAxes(dim); });
			
			// assumption: we don't have stacked axes yet
			var offsets = this.offsets = {l: 0, r: 0, t: 0, b: 0};
			df.forIn(this.axes, function(axis){
				df.forIn(axis.getOffsets(), function(o, i){ offsets[i] += o; });
			});
			// add margins
			df.forIn(this.margins, function(o, i){ offsets[i] += o; });
			
			// 2nd pass with realistic dimensions
			this.plotArea = {width: dim.width - offsets.l - offsets.r, height: dim.height - offsets.t - offsets.b};
			df.forIn(this.axes, clear);
			dojo.forEach(this.stack, function(plot){ plot.calculateAxes(this.plotArea); }, this);
			
			// generate shapes
			
			// draw a chart background
			var t = this.theme,
				fill   = this.fill   ? this.fill   : (t.chart && t.chart.fill),
				stroke = this.stroke ? this.stroke : (t.chart && t.chart.stroke);
			if(fill){
				this.surface.createRect({
					width:  dim.width, 
					height: dim.height
				}).setFill(fill);
			}
			if(stroke){
				this.surface.createRect({
					width:  dim.width - 1, 
					height: dim.height - 1
				}).setStroke(stroke);
			}
			// draw a plot background
			fill   = t.plotarea && t.plotarea.fill;
			stroke = t.plotarea && t.plotarea.stroke;
			if(fill){
				this.surface.createRect({
					x: offsets.l, y: offsets.t,
					width:  dim.width  - offsets.l - offsets.r, 
					height: dim.height - offsets.t - offsets.b
				}).setFill(fill);
			}
			if(stroke){
				this.surface.createRect({
					x: offsets.l, y: offsets.t,
					width:  dim.width  - offsets.l - offsets.r - 1, 
					height: dim.height - offsets.t - offsets.b - 1
				}).setStroke(stroke);
			}
			
			// go over the stack backwards
			df.foldr(this.stack, function(z, plot){ return plot.render(dim, offsets), 0; }, 0);
			
			// go over axes
			df.forIn(this.axes, function(axis){ axis.render(dim, offsets); });
			
			this._makeClean();
			
			return this;
		},
		_makeClean: function(){
			// reset dirty flags
			dojo.forEach(this.axes,   makeClean);
			dojo.forEach(this.stack,  makeClean);
			dojo.forEach(this.series, makeClean);
			this.dirty = false;
		},
		_makeDirty: function(){
			// reset dirty flags
			dojo.forEach(this.axes,   makeDirty);
			dojo.forEach(this.stack,  makeDirty);
			dojo.forEach(this.series, makeDirty);
			this.dirty = true;
		}
	});
})();

}

if(!dojo._hasResource["dojox.charting.themes.PlotKit.orange"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.charting.themes.PlotKit.orange"] = true;
dojo.provide("dojox.charting.themes.PlotKit.orange");


(function(){
	var dxc=dojox.charting;
	dxc.themes.PlotKit.orange=new dxc.Theme({
		chart:{
			stroke:null,
			fill: "white"
		},
		plotarea:{
			stroke:null,
			fill: "#f5eee6"
		},
		axis:{
			stroke:{ color:"#fff",width:2 },
			line:{ color:"#fff",width:1 },
			majorTick:{ color:"#fff", width:2, length:12 },
			minorTick:{ color:"#fff", width:1, length:8 },
			font:"normal normal normal 8pt Tahoma",
			fontColor:"#999"
		},
		series:{
			outline:{ width:1, color:"#fff" },
			stroke:{ width:2, color:"#666" },
			fill:new dojo.Color([0x66, 0x66, 0x66, 0.8]),
			font:"normal normal normal 7pt Tahoma",	//	label
			fontColor:"#000"
		},
		marker:{	//	any markers on a series.
			stroke:{ width:2 },
			fill:"#333",
			font:"normal normal normal 7pt Tahoma",	//	label
			fontColor:"#000"
		},
		colors:[]
	});
	dxc.themes.PlotKit.orange.defineColors({ hue:31, saturation:60, low:40, high:88 });
})();

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPClientChart"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPClientChart"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dijit.TIPClientChart");






dojo.declare(
	"ibm.tivoli.tip.dijit.TIPClientChart",
	[dijit._Widget, dijit._Templated],
{
//	templateString: "<div>hello world</div>",
	
	constructor: function () {
		console.log ("enter TIPClientChart.constructor");
		this.chart = null;
		this.limit=10;
		this.magnitude = 30;
		console.log ("exit TIPClientChart.constructor");
	},
	postCreate: function () {
		console.log ("enter TIPClientChart.postCreate");
		this.seriesA = this.makeSeries(this.limit);
		this.seriesB = this.makeSeries(this.limit);
		this.seriesC = this.makeSeries(this.limit);
		this.makeObjects();
		console.log ("exit TIPClientChart.postCreate");
	},
    randomValue: function() {
		return Math.random() * this.magnitude;
	},
	makeSeries: function(len){
	    var s = [];
	    do{
    	    s.push(this.randomValue());
		}while(s.length < len);
	    return s;
	},
	makeObjects: function(){
		this.chart = new dojox.charting.Chart2D("test");
		this.chart.setTheme(dojox.charting.themes.PlotKit.orange);
	   	this.chart.addAxis("x", {fixLower: "minor", natural: true, min: 1, max: this.limit});
		this.chart.addAxis("y", {vertical: true, min: 0, max: 30, majorTickStep: 5, minorTickStep: 1});
		this.chart.addPlot("default", {type: "Lines"});
		this.chart.addSeries("Series A", this.seriesA);
		this.chart.addSeries("Series B", this.seriesB);
		this.chart.addSeries("Series C", this.seriesC);
		this.chart.addPlot("grid", {type: "Grid", hMinorLines: true});
		this.chart.render();
	    setInterval(dojo.hitch (this, this.updateTest), 3000);
	},
	updateTest: function() {
		this.seriesA.shift();
		this.seriesA.push(this.randomValue());
		this.chart.updateSeries("Series A", this.seriesA);

		this.seriesB.shift();
		this.seriesB.push(this.randomValue());
		this.chart.updateSeries("Series B", this.seriesB);

		this.seriesC.shift();
		this.seriesC.push(this.randomValue());
		this.chart.updateSeries("Series C", this.seriesC);

		this.chart.render();
	}
	
});

}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd._copyKey = navigator.appVersion.indexOf("Macintosh") < 0 ? "ctrlKey" : "metaKey";

dojo.dnd.getCopyKeyState = function(e) {
	// summary: abstracts away the difference between selection on Mac and PC,
	//	and returns the state of the "copy" key to be pressed.
	// e: Event: mouse event
	return e[dojo.dnd._copyKey];	// Boolean
};

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary: returns a unique string for use with any DOM element
	var id;
	do{
		id = "dojoUnique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary: returns true, if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");

dojo.dnd.getViewport = function(){
	// summary: returns a viewport size (visible part of the window)

	// FIXME: need more docs!!
	var d = dojo.doc, dd = d.documentElement, w = window, b = dojo.body();
	if(dojo.isMozilla){
		return {w: dd.clientWidth, h: w.innerHeight};	// Object
	}else if(!dojo.isOpera && w.innerWidth){
		return {w: w.innerWidth, h: w.innerHeight};		// Object
	}else if (!dojo.isOpera && dd && dd.clientWidth){
		return {w: dd.clientWidth, h: dd.clientHeight};	// Object
	}else if (b.clientWidth){
		return {w: b.clientWidth, h: b.clientHeight};	// Object
	}
	return null;	// Object
};

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event:
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.dnd.getViewport(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event:
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo._abs(n, true);
				// console.debug(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				b.l += t.x + n.scrollLeft;
				b.t += t.y + n.scrollTop;
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2), 
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - b.l, ry = e.pageY - b.t, dx = 0, dy = 0;
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
					}else if(rx > b.w - w){
						dx = dojo.dnd.H_AUTOSCROLL_VALUE;
					}
				}
				//console.debug("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
					}else if(ry > b.h - h){
						dy = dojo.dnd.V_AUTOSCROLL_VALUE;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				// if(dx || dy){ console.debug(oldLeft + ", " + oldTop + "\n" + dx + ", " + dy + "\n" + n.scrollLeft + ", " + n.scrollTop); }
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary: an object, which makes a node follow the mouse, 
		//	used as a default mover, and as a base class for custom movers
		// node: Node: a node (or node's id) to be moved
		// e: Event: a mouse event, which started the move;
		//	only pageX and pageY properties are used
		// host: Object?: object which implements the functionality of the move,
		//	 and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = this.host = host, d = node.ownerDocument, 
			firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo, "stopEvent"),
			dojo.connect(d, "onselectstart", dojo, "stopEvent"),
			firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary: event processor for onmousemove
		// e: Event: mouse event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY});
	},
	onMouseUp: function(e){
		if(this.mouseButton == e.button){
			this.destroy();
		}
	},
	// utilities
	onFirstMove: function(){
		// summary: makes the node absolute; it is meant to be called only once
		this.node.style.position = "absolute";	// enforcing the absolute mode
		var m = dojo.marginBox(this.node);
		m.l -= this.marginBox.l;
		m.t -= this.marginBox.t;
		this.marginBox = m;
		this.host.onFirstMove(this);
		dojo.disconnect(this.events.pop());
	},
	destroy: function(){
		// summary: stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		handle: Node: a node (or node's id), which is used as a mouse handle
		//			if omitted, the node itself is used as a handle
		//		delay: Number: delay move by this number of pixels
		//		skip: Boolean: skip move of form elements
		//		mover: Object: a constructor of custom Mover
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary: stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary: event processor for onmousedown, creates a Mover for the node
		// e: Event: mouse event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(dojo.connect(this.handle, "onmousemove", this, "onMouseMove"));
			this.events.push(dojo.connect(this.handle, "onmouseup", this, "onMouseUp"));
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			new this.mover(this.node, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary: event processor for onmousemove, used only for delayed drags
		// e: Event: mouse event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			new this.mover(this.node, e, this);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary: event processor for onmouseup, used only for delayed delayed drags
		// e: Event: mouse event
		dojo.disconnect(this.events.pop());
		dojo.disconnect(this.events.pop());
	},
	onSelectStart: function(e){
		// summary: event processor for onselectevent and ondragevent
		// e: Event: mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary: called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove"); 
		dojo.addClass(this.node, "dojoMoveItem"); 
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary: called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary: called during the very first move notification,
		//	can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called during every move notification,
		//	should actually move the node, can be overwritten.
		this.onMoving(mover, leftTop);
		dojo.marginBox(mover.node, leftTop);
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called before every incremental move,
		//	can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called after every incremental move,
		//	can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		constraints: Function: a function, which calculates a constraint box,
		//			it is called in a context of the moveable object.
		//		within: Boolean: restrict move within boundaries.
		//	the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary: called during the very first move notification,
		//	can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover), m = mover.marginBox;
		c.r = c.l + c.w - (this.within ? m.w : 0);
		c.b = c.t + c.h - (this.within ? m.h : 0);
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary: called during every move notification,
		//	should actually move the node, can be overwritten.
		var c = this.constraintBox;
		leftTop.l = leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l;
		leftTop.t = leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t;
		dojo.marginBox(mover.node, leftTop);
	}
});

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		box: Object: a constraint box
		//	the rest is passed to the base class
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary: an object, which makes a node moveable
		// node: Node: a node (or node's id) to be moved
		// params: Object: an optional object with additional parameters;
		//	following parameters are recognized:
		//		area: String: a parent's area to restrict the move,
		//			can be "margin", "border", "padding", or "content".
		//	the rest is passed to the base class
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode, 
				s = dojo.getComputedStyle(n), 
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// WARNING: below are obsolete objects, instead of custom movers use custom moveables (above)

dojo.dnd.move.constrainedMover = function(fun, within){
	// summary: returns a constrained version of dojo.dnd.Mover
	// description: this function produces n object, which will put a constraint on 
	//	the margin box of dragged object in absolute coordinates
	// fun: Function: called on drag, and returns a constraint box
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	var mover = function(node, e, notifier){
		dojo.dnd.Mover.call(this, node, e, notifier);
	};
	dojo.extend(mover, dojo.dnd.Mover.prototype);
	dojo.extend(mover, {
		onMouseMove: function(e){
			// summary: event processor for onmousemove
			// e: Event: mouse event
			dojo.dnd.autoScroll(e);
			var m = this.marginBox, c = this.constraintBox,
				l = m.l + e.pageX, t = m.t + e.pageY;
			l = l < c.l ? c.l : c.r < l ? c.r : l;
			t = t < c.t ? c.t : c.b < t ? c.b : t;
			this.host.onMove(this, {l: l, t: t});
		},
		onFirstMove: function(){
			// summary: called once to initialize things; it is meant to be called only once
			dojo.dnd.Mover.prototype.onFirstMove.call(this);
			var c = this.constraintBox = fun.call(this), m = this.marginBox;
			c.r = c.l + c.w - (within ? m.w : 0);
			c.b = c.t + c.h - (within ? m.h : 0);
		}
	});
	return mover;	// Object
};

dojo.dnd.move.boxConstrainedMover = function(box, within){
	// summary: a specialization of dojo.dnd.constrainedMover, which constrains to the specified box
	// box: Object: a constraint box (l, t, w, h)
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	return dojo.dnd.move.constrainedMover(function(){ return box; }, within);	// Object
};

dojo.dnd.move.parentConstrainedMover = function(area, within){
	// summary: a specialization of dojo.dnd.constrainedMover, which constrains to the parent node
	// area: String: "margin" to constrain within the parent's margin box, "border" for the border box,
	//	"padding" for the padding box, and "content" for the content box; "content" is the default value.
	// within: Boolean: if true, constraints the whole dragged object withtin the rectangle, 
	//	otherwise the constraint is applied to the left-top corner
	var fun = function(){
		var n = this.node.parentNode, 
			s = dojo.getComputedStyle(n), 
			mb = dojo._getMarginBox(n, s);
		if(area == "margin"){
			return mb;	// Object
		}
		var t = dojo._getMarginExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "border"){
			return mb;	// Object
		}
		t = dojo._getBorderExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "padding"){
			return mb;	// Object
		}
		t = dojo._getPadExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		return mb;	// Object
	};
	return dojo.dnd.move.constrainedMover(fun, within);	// Object
};

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dijit.form.Form"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Form"] = true;
dojo.provide("dijit.form.Form");




dojo.declare("dijit.form._FormMixin", null,
	{
		/*
		summary:
			Widget corresponding to <form> tag, for validation and serialization

		usage:
			<form dojoType="dijit.form.Form" id="myForm">
				Name: <input type="text" name="name" />
			</form>
			myObj={name: "John Doe"};
			dijit.byId('myForm').setValues(myObj);

			myObj=dijit.byId('myForm').getValues();
		TODO:
		* Repeater
		* better handling for arrays.  Often form elements have names with [] like
		* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])

		*/

		// HTML <FORM> attributes

		action: "",
		method: "",
		enctype: "",
		name: "",
		"accept-charset": "",
		accept: "",
		target: "",

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{action: "", method: "", enctype: "", "accept-charset": "", accept: "", target: ""}),

		// execute: Function
		//	User defined function to do stuff when the user hits the submit button
		execute: function(/*Object*/ formContents){},

		// onCancel: Function
		//	Callback when user has canceled dialog, to notify container
		//	(user shouldn't override)
		onCancel: function(){},

		// onExecute: Function
		//	Callback when user is about to execute dialog, to notify container
		//	(user shouldn't override)
		onExecute: function(){},

		templateString: "<form dojoAttachPoint='containerNode' dojoAttachEvent='onsubmit:_onSubmit' name='${name}' enctype='multipart/form-data'></form>",

		_onSubmit: function(/*event*/e) {
			// summary: callback when user hits submit button
			dojo.stopEvent(e);
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.getValues());
		},

		submit: function() {
			// summary: programatically submit form
			this.containerNode.submit();
		},

		setValues: function(/*object*/obj) {
			// summary: fill in form values from a JSON structure

			// generate map from name --> [list of widgets with that name]
			var map = {};
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			// call setValue() or setChecked() for each widget, according to obj
			for(var name in map){
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(widgets[0].setChecked){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.setChecked(dojo.indexOf(values, w.value) != -1);
					});
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.setValue(values[i]);
					});					
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets

			dojo.forEach(this.containerNode.elements, function(element){
				if (element.name == ''){return};	// like "continue"	
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined") {
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if (typeof(myObj) == "undefined") {
					return;		// like "continue"
				}
				if (typeof(myObj[name]) == "undefined" && this.ignoreNullValues) {
					return;		// like "continue"
				}

				// TODO: widget values (just call setValue() on the widget)

				switch(element.type) {
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val==element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name]==element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function() {
			// summary: generate JSON structure from form values

			// get widget values
			var obj = {};
			dojo.forEach(this.getDescendants(), function(widget){
				var value = widget.getValue ? widget.getValue() : widget.value;
				var name = widget.name;
				if(!name){ return; }

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(widget.setChecked){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(widget.checked){
							dojo.setObject(name, value, obj);
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(widget.checked){
							ary.push(value);
						}
					}
				}else{
					// plain input
					dojo.setObject(name, value, obj);
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if (!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j) {
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if (nameA.length > 1) {
						if(typeof(myObj[nameA[0]]) == "undefined") {
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined") {
							myObj[nameA[0]][nameIndex]={};
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined") {
						myObj[nameA[0]]={}
					} // if

					if (nameA.length == 1) {
						myObj=myObj[nameA[0]];
					} else {
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if ((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type=="radio" && elm.checked)) {
					if(name == name.split("[")[0]) {
						myObj[name]=elm.value;
					} else {
						// can not set value when there is no name
					}
				} else if (elm.type == "checkbox" && elm.checked) {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if (elm.type == "select-multiple") {
					if(typeof(myObj[name]) == 'undefined') {
						myObj[name]=[ ];
					}
					for (var jdx=0,len3=elm.options.length; jdx<len3; ++jdx) {
						if (elm.options[jdx].selected) {
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

	 	isValid: function() {
	 		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 		// make sure that every widget that has a validator function returns true
	 		return dojo.every(this.getDescendants(), function(widget){
	 			return !widget.isValid || widget.isValid();
	 		});
		}
	});

dojo.declare(
	"dijit.form.Form",
	[dijit._Widget, dijit._Templated, dijit.form._FormMixin],
	null
);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");









dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary: the thing that grays out the screen behind the dialog

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class=dijitDialogUnderlayWrapper id='${id}_underlay'><div class=dijitDialogUnderlay dojoAttachPoint='node'></div></div>",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		layout: function(){
			// summary
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.

			var viewport = dijit.getViewport();
			var is = this.node.style,
				os = this.domNode.style;

			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";

			// process twice since the scroll bar may have been removed
			// by the previous resizing
			var viewport2 = dijit.getViewport();
			if(viewport.w != viewport2.w){ is.width = viewport2.w + "px"; }
			if(viewport.h != viewport2.h){ is.height = viewport2.h + "px"; }
		},

		show: function(){
			this.domNode.style.display = "block";
			this.layout();
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "block";
			}
			this._resizeHandler = this.connect(window, "onresize", "layout");
		},

		hide: function(){
			this.domNode.style.display = "none";
			if(this.bgIframe.iframe){
				this.bgIframe.iframe.style.display = "none";
			}
			this.disconnect(this._resizeHandler);
		},

		uninitialize: function(){
			if(this.bgIframe){
				this.bgIframe.destroy();
			}
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)

		templateString: null,
		templateString:"<div class=\"dijitDialog\">\r\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\" tabindex=\"0\" waiRole=\"dialog\">\r\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\">${title}</span>\r\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: hide\">\r\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\">x</span>\r\n\t</span>\r\n\t</div>\r\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\r\n\t<span dojoAttachPoint=\"tabEnd\" dojoAttachEvent=\"onfocus:_cycleFocus\" tabindex=\"0\"></span>\r\n</div>\r\n",

		// open: Boolean
		//		is True or False depending on state of dialog
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: 400,

		_lastFocusItem:null,

		attributeMap: dojo.mixin(dojo.clone(dijit._Widget.prototype.attributeMap),
			{title: "titleBar"}),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);
			this.inherited("postCreate",arguments);
			this.domNode.style.display="none";
			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
		},

		onLoad: function(){
			// summary: 
			//		when href is specified we need to reposition the dialog after the data is loaded
			this._position();
			this.inherited("onLoad",arguments);
		},

		_setup: function(){
			// summary:
			//		stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons)

			this._modalconnects = [];

			if(this.titleBar){
				this._moveable = new dojo.dnd.Moveable(this.domNode, { handle: this.titleBar });
			}

			this._underlay = new dijit.DialogUnderlay();

			var node = this.domNode;
			this._fadeIn = dojo.fx.combine(
				[dojo.fadeIn({
					node: node,
					duration: this.duration
				 }),
				 dojo.fadeIn({
					node: this._underlay.domNode,
					duration: this.duration,
					onBegin: dojo.hitch(this._underlay, "show")
				 })
				]
			);

			this._fadeOut = dojo.fx.combine(
				[dojo.fadeOut({
					node: node,
					duration: this.duration,
					onEnd: function(){
						node.style.display="none";
					}
				 }),
				 dojo.fadeOut({
					node: this._underlay.domNode,
					duration: this.duration,
					onEnd: dojo.hitch(this._underlay, "hide")
				 })
				]
			);
		},

		uninitialize: function(){
			if(this._underlay){
				this._underlay.destroy();
			}
		},

		_position: function(){
			// summary: position modal dialog in center of screen
			
			if(dojo.hasClass(dojo.body(),"dojoMove")){ return; }
			var viewport = dijit.getViewport();
			var mb = dojo.marginBox(this.domNode);

			var style = this.domNode.style;
			style.left = Math.floor((viewport.l + (viewport.w - mb.w)/2)) + "px";
			style.top = Math.floor((viewport.t + (viewport.h - mb.h)/2)) + "px";
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary:  called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to titleBar

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.titleBar.focus();
		},

		_onKey: function(/*Event*/ evt){
			if(evt.keyCode){
				var node = evt.target;
				// see if we are shift-tabbing from titleBar
				if(node == this.titleBar && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
					if(this._lastFocusItem){
						this._lastFocusItem.focus(); // send focus to last item in dialog if known
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode){
							if(evt.keyCode == dojo.keys.ESCAPE){
								this.hide(); 
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.keyCode != dojo.keys.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if (!dojo.isOpera){
						try{
							this.titleBar.focus();
						}catch(e){/*squelch*/}
					}
				}
			}
		},

		show: function(){
			// summary: display the dialog

			// first time we show the dialog, there's some initialization stuff to do			
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(document.documentElement, "onkeypress", this, "_onKey"));

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this._modalconnects.push(dojo.connect(this.containerNode, ev, this, "_findLastFocus"));

			dojo.style(this.domNode, "opacity", 0);
			this.domNode.style.display="block";
			this.open = true;
			this._loadCheck(); // lazy load trigger

			this._position();

			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);

			// set timeout to allow the browser to render dialog
			setTimeout(dojo.hitch(this, function(){
				dijit.focus(this.titleBar);
			}), 50);
		},

		hide: function(){
			// summary
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return		
			if(!this._alreadyInitialized){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}
			this._fadeOut.play();

			if (this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			this.connect(this._fadeOut,"onEnd",dojo.hitch(this,function(){
				dijit.focus(this._savedFocus);
			}));
			this.open = false;
		},

		layout: function() {
			// summary: position the Dialog and the underlay
			if(this.domNode.style.display == "block"){
				this._underlay.layout();
				this._position();
			}
		}
	}
);

dojo.declare(
	"dijit.TooltipDialog",
	[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin],
	{
		// summary:
		//		Pops up a dialog that appears like a Tooltip
		// title: String
		// 		Description of tooltip dialog (required for a11Y)
		title: "",

		_lastFocusItem: null,

		templateString: null,
		templateString:"<div class=\"dijitTooltipDialog\" >\r\n\t<div class=\"dijitTooltipContainer\">\r\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"0\" waiRole=\"dialog\"></div>\r\n\t</div>\r\n\t<span dojoAttachPoint=\"tabEnd\" tabindex=\"0\" dojoAttachEvent=\"focus:_cycleFocus\"></span>\r\n\t<div class=\"dijitTooltipConnector\" ></div>\r\n</div>\r\n",

		postCreate: function(){
			this.inherited("postCreate",arguments);
			this.connect(this.containerNode, "onkeypress", "_onKey");

			// IE doesn't bubble onblur events - use ondeactivate instead
			var ev = typeof(document.ondeactivate) == "object" ? "ondeactivate" : "onblur";
			this.connect(this.containerNode, ev, "_findLastFocus");
			this.containerNode.title=this.title;
		},

		orient: function(/*Object*/ corner){
			// summary: configure widget to be displayed in given position relative to the button
			this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above");
		},

		onOpen: function(/*Object*/ pos){
			// summary: called when dialog is displayed
			this.orient(pos.corner);
			this._loadCheck(); // lazy load trigger
			this.containerNode.focus();
		},

		_onKey: function(/*Event*/ evt){
			// summary: keep keyboard focus in dialog; close dialog on escape key
			if(evt.keyCode == dojo.keys.ESCAPE){
				this.onCancel();
			}else if(evt.target == this.containerNode && evt.shiftKey && evt.keyCode == dojo.keys.TAB){
				if (this._lastFocusItem){
					this._lastFocusItem.focus();
				}
				dojo.stopEvent(evt);
			}else if(evt.keyCode == dojo.keys.TAB){
				// we want the browser's default tab handling to move focus
				// but we don't want the tab to propagate upwards
				evt.stopPropagation();
			}
		},

		_findLastFocus: function(/*Event*/ evt){
			// summary: called from onblur of dialog container to determine the last focusable item
			this._lastFocused = evt.target;
		},

		_cycleFocus: function(/*Event*/ evt){
			// summary: when tabEnd receives focus, advance focus around to containerNode

			// on first focus to tabEnd, store the last focused item in dialog
			if(!this._lastFocusItem){
				this._lastFocusItem = this._lastFocused;
			}
			this.containerNode.focus();
		}
	}	
);


}

if(!dojo._hasResource["dijit.form.CheckBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.CheckBox"] = true;
dojo.provide("dijit.form.CheckBox");



dojo.declare(
	"dijit.form.CheckBox",
	dijit.form.ToggleButton,
	{
		// summary:
		// 		Same as an HTML checkbox, but with fancy styling.
		//
		// description:
		// User interacts with real html inputs.
		// On onclick (which occurs by mouse click, space-bar, or
		// using the arrow keys to switch the selected radio button),
		// we update the state of the checkbox/radio.
		//
		// There are two modes:
		//   1. High contrast mode
		//   2. Normal mode
		// In case 1, the regular html inputs are shown and used by the user.
		// In case 2, the regular html inputs are invisible but still used by
		// the user. They are turned quasi-invisible and overlay the background-image.

		templateString:"<fieldset class=\"dijitReset dijitInline\" waiRole=\"presentation\"\r\n\t><input\r\n\t \ttype=\"${type}\" name=\"${name}\"\r\n\t\tclass=\"dijitReset dijitCheckBoxInput\"\r\n\t\tdojoAttachPoint=\"inputNode,focusNode\"\r\n\t \tdojoAttachEvent=\"onmouseover:_onMouse,onmouseout:_onMouse,onclick:_onClick\"\r\n/></fieldset>\r\n",

		baseClass: "dijitCheckBox",

		//	Value of "type" attribute for <input>
		type: "checkbox",

		// value: Value
		//	equivalent to value field on normal checkbox (if checked, the value is passed as
		//	the value when form is submitted)
		value: "on",

		postCreate: function(){
			dojo.setSelectable(this.inputNode, false);
			this.setChecked(this.checked);
			this.inherited(arguments);
		},

		setChecked: function(/*Boolean*/ checked){
			if(dojo.isIE){
				if(checked){ this.inputNode.setAttribute('checked', 'checked'); }
				else{ this.inputNode.removeAttribute('checked'); }
			}else{ this.inputNode.checked = checked; }
			this.inherited(arguments);
		},

		setValue: function(/*String*/ value){
			if(value == null){ value = ""; }
			this.inputNode.value = value;
			dijit.form.CheckBox.superclass.setValue.call(this,value);
		}
	}
);

dojo.declare(
	"dijit.form.RadioButton",
	dijit.form.CheckBox,
	{
		// summary:
		// 		Same as an HTML radio, but with fancy styling.
		//
		// description:
		// Implementation details
		//
		// Specialization:
		// We keep track of dijit radio groups so that we can update the state
		// of all the siblings (the "context") in a group based on input
		// events. We don't rely on browser radio grouping.

		type: "radio",
		baseClass: "dijitRadio",

		// This shared object keeps track of all widgets, grouped by name
		_groups: {},

		postCreate: function(){
			// add this widget to _groups
			(this._groups[this.name] = this._groups[this.name] || []).push(this);

			this.inherited(arguments);
		},

		uninitialize: function(){
			// remove this widget from _groups
			dojo.forEach(this._groups[this.name], function(widget, i, arr){
				if(widget === this){
					arr.splice(i, 1);
					return;
				}
			}, this);
		},

		setChecked: function(/*Boolean*/ checked){
			// If I am being checked then have to deselect currently checked radio button
			if(checked){
				dojo.forEach(this._groups[this.name], function(widget){
					if(widget != this && widget.checked){
						widget.setChecked(false);
					}
				}, this);
			}
			this.inherited(arguments);			
		},

		_clicked: function(/*Event*/ e){
			if(!this.checked){
				this.setChecked(true);
			}
		}
	}
);

}

if(!dojo._hasResource["dojox.layout.ResizeHandle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.layout.ResizeHandle"] = true;
dojo.provide("dojox.layout.ResizeHandle");
dojo.experimental("dojox.layout.ResizeHandle"); 


 
 

dojo.declare("dojox.layout.ResizeHandle", [dijit._Widget, dijit._Templated], {
	// summary
	//	The handle on the bottom-right corner of FloatingPane or other widgets that allows
	//	the widget to be resized.
	//	Typically not used directly.

	// targetId: String
	//	id of the Widget OR DomNode that I will size
	targetId: '',

	// targetContainer: DomNode
	//	over-ride targetId and attch this handle directly to a reference of a DomNode
	targetContainer: null, 

	// resizeAxis: String
	//	one of: x|y|xy limit resizing to a single axis, default to xy ... 
	resizeAxis: "xy",

	// activeResize: Boolean
	// 	if true, node will size realtime with mouse movement, 
	//	if false, node will create virtual node, and only resize target on mouseUp
	activeResize: false,
	
	// activeResizeClass: String
	//	css class applied to virtual resize node. 
	activeResizeClass: 'dojoxResizeHandleClone',

	// animateSizing: Boolean
	//	only applicable if activeResize = false. onMouseup, animate the node to the
	//	new size
	animateSizing: true,
	
	// animateMethod: String
	// 	one of "chain" or "combine" ... visual effect only. combine will "scale" 
	// 	node to size, "chain" will alter width, then height
	animateMethod: 'chain',

	// animateDuration: Integer
	//	time in MS to run sizing animation. if animateMethod="chain", total animation 
	//	playtime is 2*animateDuration
	animateDuration: 225,

	// minHeight: Integer
	//	smallest height in px resized node can be
	minHeight: 100,

	// minWidth: Integer
	//	smallest width in px resize node can be
	minWidth: 100,

	// resize handle template, fairly easy to override: 
	templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',

	// private propteries and holders
	_isSizing: false,
	_connects: [],
	_activeResizeNode: null,	
	_activeResizeLastEvent: null,
	// defaults to match default resizeAxis. set resizeAxis variable to modify. 
	_resizeX: true,
	_resizeY: true,


	postCreate: function(){
		// summary: setup our one major listener upon creation
		dojo.connect(this.resizeHandle, "onmousedown", this, "_beginSizing");
		if(!this.activeResize){ 
			this._activeResizeNode = document.createElement('div');
			dojo.addClass(this._activeResizeNode,this.activeResizeClass); 
		}else{ this.animateSizing = false; } 	

		if (!this.minSize) { 
			this.minSize = { w: this.minWidth, h: this.minHeight };
		}
		// should we modify the css for the cursor hover to n-resize nw-resize and w-resize?
		this._resizeX = this._resizeY = false; 
		switch (this.resizeAxis.toLowerCase()) {
		case "xy" : 
			this._resizeX = this._resizeY = true; 
			// FIXME: need logic to determine NW or NE class to see
			// based on which [todo] corner is clicked
			dojo.addClass(this.resizeHandle,"dojoxResizeNW"); 
			break; 
		case "x" : 
			this._resizeX = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeW");
			break;
		case "y" : 
			this._resizeY = true; 
			dojo.addClass(this.resizeHandle,"dojoxResizeN");
			break;
		}
	},

	_beginSizing: function(/*Event*/ e){
		// summary: setup movement listeners and calculate initial size
		
		if (this._isSizing){ return false; }

		this.targetWidget = dijit.byId(this.targetId);

		// FIXME: resizing widgets does weird things, disable virtual resizing for now:
		if (this.targetWidget) { this.activeResize = true; } 

		this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dojo.byId(this.targetId);
		if (this.targetContainer) { this.targetDomNode = this.targetContainer; } 
		if (!this.targetDomNode){ return; }

		if (!this.activeResize) {
			this.targetDomNode.appendChild(this._activeResizeNode); 
			dojo.fadeIn({ node: this._activeResizeNode, duration:120, 
				beforeBegin: dojo.hitch(this,function(){
					this._activeResizeNode.style.display=''; 
				})
			}).play(); 
		}

		this._isSizing = true;
		this.startPoint  = {'x':e.clientX, 'y':e.clientY};

		// FIXME: this is funky: marginBox adds height, contentBox ignores padding (expected, but foo!)
		var mb = (this.targetWidget) ? dojo.marginBox(this.targetDomNode) : dojo.contentBox(this.targetDomNode);  
		this.startSize  = { 'w':mb.w, 'h':mb.h };

		this._connects = []; 
		this._connects.push(dojo.connect(document,"onmousemove",this,"_updateSizing")); 
		this._connects.push(dojo.connect(document,"onmouseup", this, "_endSizing"));

		e.preventDefault();
	},

	_updateSizing: function(/*Event*/ e){
		// summary: called when moving the ResizeHandle ... determines 
		//	new size based on settings/position and sets styles.

		if(this.activeResize){
			this._changeSizing(e);
		}else{
			var tmp = this._getNewCoords(e);	
			if(tmp === false){ return; }
			dojo.style(this._activeResizeNode,"width",tmp.width+"px");
			dojo.style(this._activeResizeNode,"height",tmp.height+"px"); 
			this._activeResizeNode.style.display=''; 
		}
	},

	_getNewCoords: function(/* Event */ e){
		
		// On IE, if you move the mouse above/to the left of the object being resized,
		// sometimes clientX/Y aren't set, apparently.  Just ignore the event.
		try{
			if(!e.clientX  || !e.clientY){ return false; }
		}catch(e){
			// sometimes you get an exception accessing above fields...
			return false;
		}
		this._activeResizeLastEvent = e; 

		var dx = this.startPoint.x - e.clientX;
		var dy = this.startPoint.y - e.clientY;
		
		var newW = (this._resizeX) ? this.startSize.w - dx : this.startSize.w;
		var newH = (this._resizeY) ? this.startSize.h - dy : this.startSize.h;

		// minimum size check
		if(this.minSize){
			//var mb = dojo.marginBox(this.targetDomNode);
			if(newW < this.minSize.w){
				newW = this.minSize.w;
			}
			if(newH < this.minSize.h){
				newH = this.minSize.h;
			}
		}
		return {width:newW, height:newH};  // Object
	},
	
	_changeSizing: function(/*Event*/ e){
		// summary: apply sizing information based on information in (e) to attached node
		var tmp = this._getNewCoords(e);
		if(tmp===false){ return; }

		if(this.targetWidget && typeof this.targetWidget.resize == "function"){ 
			this.targetWidget.resize({ w: tmp.width, h: tmp.height });
		}else{
			if(this.animateSizing){
				var anim = dojo.fx[this.animateMethod]([
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							width: { start: this.startSize.w, end: tmp.width, unit:'px' } 
						},	
						duration: this.animateDuration
					}),
					dojo.animateProperty({
						node: this.targetDomNode,
						properties: { 
							height: { start: this.startSize.h, end: tmp.height, unit:'px' }
						},
						duration: this.animateDuration
					})
				]);
				anim.play();
			}else{
				dojo.style(this.targetDomNode,"width",tmp.width+"px"); 
				dojo.style(this.targetDomNode,"height",tmp.height+"px");
			}
		}	
		e.preventDefault();
	},

	_endSizing: function(/*Event*/ e){
		// summary: disconnect listenrs and cleanup sizing
		dojo.forEach(this._connects,function(c){
			dojo.disconnect(c); 
		});
		if(!this.activeResize){
			dojo.fadeOut({ node:this._activeResizeNode, duration:250,
				onEnd: dojo.hitch(this,function(){
					this._activeResizeNode.style.display="none";
				})
			}).play();
			this._changeSizing(e);
		}
		this._isSizing = false;
	}

});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageDialog"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007, 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/

dojo.provide("ibm.tivoli.tip.dijit.TIPMessageDialog");

 





// NLS


// define the widget class
dojo.declare(
	"ibm.tivoli.tip.dijit.TIPMessageDialog", 
    [ dijit._Widget ],
{
    // resizeAxis: String
    //  x | xy | y to limit pane's sizing direction
    resizeAxis: "xy",

	// type: String: defines the type of message dialog ("Error", "Warning", "Info", "Confirm")
	type: "Error",
	
	// portlet title associated w/ this message
	portletTitle: null,

	// message: String: the displayable translated message
	message: null,

	// messageId: String: the displayable translated message's unique identifier
	messageId: null,
	
	// width: String: gives the pixel width of the dialog (Default: 500px)
	width: "380px",
	
	// height: String: gives the pixel height of the dialog (Default: 200px)
	height: "225px",
	
	// buttons: String Array: a list of buttons to display (Default: OK)
	buttons: [],
	
	// callback: Function: a callback function when the dialog is closed
	callback: null,
	
	// parameters to pass on callback when dialog closes
	callbackParms: null,
	
	// url: String: a URL that links the message to an InfoCenter page
	url: null,
	
	// show "Never show again checkbox"
	showCheckbox: false,
	
	// Destroy the dialog on close
	destroyOnClose: true,

	constructor: function() {
		// summary: An extension to the Dojo Dialog widget that supports 
		//      advanced features.
		// resourceMessages_: JSON
		//		Resource bundle.
		// dialog_: dojo.widget.FloatingPane
		//		An instance of the Dialog widget.
		// title_: String
		//		Title bar string.
		// icon_: String
		//		The dialog icon's uri.
		// buttonPressedId_: String
		//		The ID of the button pressed; this value is passed to the callback function.

	    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
	    
	    this.dialog_ = null;

	    this.title_ = null;
	    this.icon_ = null;
	    this.buttonPressedId_ = -1;
	},


	postCreate: function() { 
	    if (this.buttons.length === 0) {
	        this.buttons.push(this.resources_.DIALOG_BUTTON_OK);
	    }

		var className = null;
	    if (this.type.toLowerCase() == "error") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_ERROR, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_ERROR;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_critical.png");
	        className = "critical";
	    }
	    else if (this.type.toLowerCase() == "warning") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_WARNING, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_WARNING;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_warning.png");
	        className = "warning";
	    }
	    else if (this.type.toLowerCase() == "confirm") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_CONFIRM, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_CONFIRM;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_confirm.png");
	        className = "confirm";
	    }
	    else if (this.type.toLowerCase() == "validation") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_VALIDATION, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_VALIDATION;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_validation.png");
	        className = "validation";
	    }
	    else { // if (this.type.toLowerCase() == "info") {
	    	if (this.portletTitle) {
		    	this.title_ = dojo.string.substitute(this.resources_.MESSAGE_TITLE, { 0: this.resources_.DIALOG_INFO, 1: this.portletTitle });
	    	} else {
		        this.title_ = this.resources_.DIALOG_INFO;
		    }
	        this.icon_ = dojo.moduleUrl("ibm.tivoli.tip.dijit", "themes/images/i_message_info.png");
	        className = "info";
	    }
	    
		var div = document.createElement('div'); 
//		div.style.width = this.width; 
//		div.style.height = this.height;
        // The following div settings are needed to ensure the dialog is seen when the bottom frames
        // are rendered due to refresh or re-submit during error scenarios (see main_ruleset.jsp).
        // Without the settings, the bottom frame was overlaying the error message. 
		//div.style.top = 0; 
		//div.style.left = 0;
		document.body.appendChild(div);

        this.dialog_ = new dijit.Dialog (
			                             { title: this.title_
		                                 },
		                                 div); 

		var dojoAttachpoint = this.dialog_.containerNode;
		dojoAttachpoint.innerHTML = this.getContent(className);

        //Build and place resize handle       
        var resizeHandleDOM = dojo.doc.createElement( 'span' );
        resizeHandleDOM.className = "dojoxFloatingResizeHandle";    
        dojo.place( resizeHandleDOM, this.dialog_.domNode, "last" );
    
        this.resizeHandle = new dojox.layout.ResizeHandle(
        { 
              targetId: this.dialog_.id, 
            resizeAxis: this.resizeAxis 
        }, resizeHandleDOM );   
	
        this.dialog_.domNode.style.width = this.width;	
        this.dialog_.domNode.style.height = this.height;	
        //this.dialog_.containerNode.style.overflow="auto";
        this.dialog_.domNode.style.overflow = "hidden";        
    
		for (var i = 0; i < this.buttons.length; i++) {
			var button = dojo.byId(this.id + "mmd_button_" + i);
			dojo.connect(button, 'onclick', this, 'onClick');    
		}
    
        this.onCloseListener  = dojo.connect( this.dialog_, "hide", this, "_onHide" );
	  this.onResizeListener = dojo.connect( this.dialog_, "resize", this, "customLayout" );    
	},
	
    /**************************************************************************
    * The dialog has been hidden.  Wait 500 miliseconds to destroy the dialog.
    *
    * HACK -- Can't figure out a better way to destroy the dialog on close.
    **************************************************************************/
    _onHide:function()
    {
      setTimeout( dojo.hitch( this, "destroy" ), 500 );
    },  
	
	onClick: function(/* HTMLEvent */e) {
		var index = this.id.length + "mmd_button_".length;
		this.buttonPressedId_ = parseInt(e.target.id.substring(index), 0);
	    console.log ("TIPMessageDialog.onClick: buttonPressedId_: " + this.buttonPressedId_);
	    var checkbox = dojo.byId (this.id + "neverShowAgain");
	    var checked = checkbox ? checkbox.checked : false;

	    this.dialog_.hide();
	    
	    if (this.callback) {
	    	this.callback(this.buttonPressedId_, this.messageId, checked, this.callbackParms);
	    }
    },

	show: function() {
		this.dialog_.show();
		this.customLayout();
		this.correctPNG();

	},

      customLayout: function() 
      {
		console.debug( dojo.byId( this.id + '_messagediv' ) );
		console.debug( dojo.contentBox( this.dialog_.domNode ) );
		
		var messageDiv = dojo.byId( this.id + '_messagediv' );
		var dialogSize = dojo.contentBox( this.dialog_.domNode );

		var titleBarSize = dojo.contentBox( this.dialog_.titleBar );

		var newSize = {w:0,h:0};

		newSize.w = dialogSize.w - 75;
		if (this.showCheckbox === true) {
			var neverShowCoords = dojo.coords (this.id + "neverShowAgain");
			newSize.h = dialogSize.h - titleBarSize.h - neverShowCoords.h - 90;
		} else {		
			newSize.h = dialogSize.h - titleBarSize.h - 90;
		}

	    dojo.contentBox( messageDiv, newSize );
	},
	
	getContent: function(className) {
		var innerHTML = "";

		innerHTML += "<table class=\"" + className + "\">";
		innerHTML += "<tr valign=\"top\">";
		innerHTML += "<td>";

		innerHTML += "<table>";
		innerHTML += "<tr>";
		innerHTML += "<td rowspan=\"2\">";
		innerHTML += "&nbsp;";
		innerHTML += "<img align=\"middle\" src=\"" + this.icon_ + "\"/>";
		innerHTML += "&nbsp;&nbsp;";
		innerHTML += "</td>";
		innerHTML += "<td class=\"message-id\" align=\"left\">";
		
		// add URL to infocenter
		if (this.url) {
			innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + '\')">';
			innerHTML += this.messageId;
			innerHTML += "</a>";
		}
		else {
			innerHTML += this.messageId;
		}
		
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "<tr>";
		innerHTML += "<td class=\"message-description\" align=\"left\">";
		innerHTML += "<div id='" + this.id + "_messagediv" + "' style='overflow:auto; width:100%;'>";
		innerHTML += "<p>";
		innerHTML += this.message;
		innerHTML += "</p>";
		innerHTML += "</div>";
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		innerHTML += "</td>";
		innerHTML += "</tr>";

		if (this.showCheckbox === true) {
			innerHTML += "<tr>";
			innerHTML += "<td class=\"checkbox\" valign=\"bottom\">";
			innerHTML += "&nbsp;&nbsp;&nbsp;<input type=\"checkbox\" dojoType=\"dijit.form.CheckBox\" id=\"" + this.id + "neverShowAgain\"/><label for=\"" + this.id + "neverShowAgain\">" + this.resources_.NEVER_SHOW + "</label>";
			innerHTML += "</td>";
			innerHTML += "</tr>";
		}

		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";

		innerHTML += "<div>";
		innerHTML += "<table width='100%'>";
		innerHTML += "<tr>";
		innerHTML += "<td valign=\"bottom\">";
		innerHTML += "<div align=\"right\">";
		for (var i = 0; i < this.buttons.length; i++) {
			innerHTML += "&nbsp;<button dojoType=\"ibm.tivoli.tip.dijit.TIPButton\" id=\"" + this.id + "mmd_button_" + i + "\" value=\"" + this.buttons[i] + "\">";
			innerHTML += this.buttons[i] + "</button>";
		}
		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";
		innerHTML += "</div>";

		innerHTML += "</td>";
		innerHTML += "</tr>";
		innerHTML += "</table>";

		return innerHTML;
	},
	
	// correctly handle PNG transparency in Win IE 5.5 & 6.
	correctPNG: function() {
		var arVersion = navigator.appVersion.split("MSIE");
		var version = parseFloat(arVersion[1]);
		if ((version >= 5.5) && document.body.filters) {
			for (var i = 0; i < document.images.length; i++) {
				var img = document.images[i];
				var imgName = img.src.toUpperCase();
				if (imgName.substring(imgName.length-3, imgName.length) == "PNG") {
					var imgID = img.id ? "id='" + img.id + "' " : "";
					var imgClass = img.className ? "class='" + img.className + "' " : "";
					var imgTitle = img.title ? "title='" + img.title + "' " : "title='" + img.alt + "' ";
					var imgStyle = "display:inline-block;" + img.style.cssText;
					if (img.align == "left") {
						imgStyle = "float:left;" + imgStyle;
					}
					if (img.align == "right") {
						imgStyle = "float:right;" + imgStyle;
					}
					if (img.parentElement.href) {
						imgStyle = "cursor:hand;" + imgStyle;
					}
					var strNewHTML = "<span " + imgID + imgClass + imgTitle +
									 " style=\"" + "width:" + img.width + "px; height:" + img.height + "px;" + imgStyle + ";" +
									 "filter:progid:DXImageTransform.Microsoft.AlphaImageLoader" +
									 "(src=\'" + img.src + "\', sizingMethod='scale');\"></span>";
					img.outerHTML = strNewHTML;
					i = i - 1;
				}
			}
		}    
	},
	
	destroy: function () {
        dojo.disconnect( this.onCloseListener );
        dojo.disconnect( this.onResizeListener );
        
        //TODO:  The fact this can NOT be called means we have not created the widget correctly
        //       This widget should directly extend dialog to be created correctly
        //this.inherited( "destroy", arguments );
        
        if ( this.dialog_ ) { this.dialog_.destroy(); dijit.registry.remove (this.dialog_.id); delete this.dialog_; }
        
		if (dijit.byId (this.id + "neverShowAgain")) {
			dijit.registry.remove (this.id + "neverShowAgain");
		}

		for (var i = 0; i < this.buttons.length; i++) {
			dijit.registry.remove (this.id + "mmd_button_" + i);
		}

		delete this.type;
		delete this.message;
		delete this.messageId;
		delete this.width;
		delete this.height;
		delete this.buttons;
		delete this.callbackParms;
		delete this.url;
		delete this.showCheckbox;
		delete this.destroyOnClose;
		delete this.title_;
		delete this.icon_;
		delete this.resources_;

		dijit.registry.remove (this.id);
	}

});



}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Global"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Global"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* xxxx-xxx 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/



dojo.provide("ibm.tivoli.tip.dojo.Global");
dojo.provide("ibm.tivoli.tip.dojo.Global.InvalidContextError");
dojo.provide("ibm.tivoli.tip.dojo._Global");

/****************************************************************************
* Global 
*
* (Q) What is this?
* (A) Since the TIP applications will be running inside a portal environment
*     it is possible that multiple instances of the application could be
*     running on one browser.  Therefore, it is necessary that our global
*     data and variables are referenced by Portlet Namespace to avoid
*     collisions between multipile instances.
*
*     This object is also used to provide common functions that are namespace
*     dependant.
****************************************************************************/

ibm.tivoli.tip.dojo.Global.IFrame        = "IframeTransport";
ibm.tivoli.tip.dojo.Global.instances     = [];
ibm.tivoli.tip.dojo.Global.anyException  = "*"; 

/****************************************************************************
* Global getSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (optional: used only if global object does NOT exist) an object of 
*			properties that will get mixed into the Global object for storage
*
*  When creating the context, you create it like this:
*  var context = {
*                  controllerURL: "/SimpleExample/AjaxController",
*                  namespace: myNamespace,
*                  nlsResources : dojo.i18n.getLocalization( "ibm.tivoli.tbsm", "resources" )
*                  contextPath: "<%=rootContextPath%>"
*                };
*   controllerURL (required) - specifies the URL to use to send requests to.
*                 this is the URL which handles the requests usually using the
*                 TIP CommandHandler.
*   namespace (optional) - Should always be specified.  If not specified, then
*             this routine will set it on the context.
*   nlsResources (optional) - Must be specified if you intend to use the nls 
*                facility provided in this object
*   contextPath (optional) - Should always be specified but not required. 
*               Must be specified if you intend to use the helper
*               function "contextualURL".
*   ===========
*   any other properties you add into the context will be avaible to your app.
****************************************************************************/
ibm.tivoli.tip.dojo.Global.getSingleton = function( namespace, context )
{
  var singleton = ibm.tivoli.tip.dojo.Global.instances[ namespace ];
  
  //if a singleton was not found then assume we need to create one now
  if ( singleton === undefined )
  {
  	//check to make sure a context object was passed before creating a Global object
  	//if the context object wasn't given then assume that an attempt was made to 
  	//access a Global object that hasn't been created -- throw an error
  	if ( context === undefined )
  	{
  		throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
  		"singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
  		"object was not provided on creation." );
  	}
  	
  	//make sure the namespace is saved on the context object
  	if ( context.namespace === undefined )
  	{
  		context.namespace = namespace;	
  	}  	
  	
    singleton = new ibm.tivoli.tip.dojo._Global( context );
   
   	//save the new singleton instance on the Global memory space
    ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton;
  }  
  
  return( singleton );	
};

/****************************************************************************
* Global setSingleton function
*
* namespace (required) -- the namespace of the global object space
* context   (required) an object of 
   properties that will get mixed into the Global object for storage
****************************************************************************/
ibm.tivoli.tip.dojo.Global.setSingleton = function (namespace, context) 
{
   //check to make sure a context object was passed before creating a Global object
   //if the context object wasn't given then assume that an attempt was made to 
   //access a Global object that hasn't been created -- throw an error
   if ( context === undefined )
   {
    throw new ibm.tivoli.tip.dojo.Global.InvalidContextError( "An attempt was made to access a " + 
    "singleton ibm.tivoli.tip.dojo.Global object that has not been created yet.  Or the context " + 
    "object was not provided on creation." );
   }
   
   //make sure the namespace is saved on the context object
   if ( context.namespace === undefined )
   {
    context.namespace = namespace; 
   }   
   
   var singleton = new ibm.tivoli.tip.dojo._Global( context );
   
   //save the new singleton instance on the Global memory space
   ibm.tivoli.tip.dojo.Global.instances[ namespace ] = singleton; 
   
   return (singleton);
};


/****************************************************************************
* _Global -- the actual Global object
*
* Notice that there is a '_' underscore prefixing this object.  This is
* important because removing it will delete the Global module functions
* such as getSingleton.  This is OK because we will never construct one
* of these things without calling getSingleton.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Global",null,
{	 
  _actionBroker:  null,
  _portletTitle: null,
	
	constructor: function( context )
	{
      //add all functions, fields, objects inside context to this object
      dojo.mixin( this, context );
      
      //create path to JSON controller servlet
      this.controller = context.controllerURL;
         
      // Create resource bundle for TIP based resource strings
      this.nlsTIPResources = dojo.i18n.getLocalization( "ibm.tivoli.tip.dojo", "resources" );
         
        
	},

  /************************************************************************
  * Get the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  getActionBroker : function() 
  {
    return( _actionBroker );
  },

  /************************************************************************
  * Set the action broker.
  ************************************************************************/
  /* ibm.tivoli.tip.dojo.ActionBroker */ 
  setActionBroker : function( /* ibm.tivoli.tip.dojo.ActionBroker */ broker ) 
  {
    return( _actionBroker = broker );
  },
  
  getPortletTitle : function () {
  	return this._portletTitle;
  },
  
  setPortletTitle: function (/*String*/ title) {
  	this._portletTitle = title;
  },
	
	/************************************************************************
	* Generate Conextual URL
	************************************************************************/
	contextualURL : function( baseURL ) 
	{
  	return( this.contextPath + baseURL );
	},

	/************************************************************************
	* Generate namespaced ID from a BASE ID.
	************************************************************************/
	namespaceID: function( baseID ) 
	{
  	return( this.namespace + baseID );
	},	

  /************************************************************************
  * show error message dialog
  ************************************************************************/
  showErrorMessage: function( msgMixin )
  {
    var msgKW = {
              type:         "Error",
              showCheckbox: false,
              portletTitle: this._portletTitle
            };

    dojo.mixin( msgKW, msgMixin );

    var x = new ibm.tivoli.tip.dijit.TIPMessageDialog( msgKW );
    x.show();
  },

  /************************************************************************
  * Retrieve a specific NLS string from the TIP resource bundle.  Deployers
  * could create a method like this for their own product that calls nls
  * with their resource bundle for convenience.  This is meant to only
  * be called by TIP code.
  ************************************************************************/
  tipnls: function( nlsKey, inserts )
  {
     return this.nls( nlsKey, inserts, this.nlsTIPResources );
  },

  /************************************************************************
  * Retrieve a specific NLS string optionally performing substitution(s).
  * This is a fairly generic routine meant to be used by exploiters of 
  * the Global object.  If an exploiter specifies a resource bundle in 
  * the Global context via the name 'nlsResources' when creating the Global object
  * then a bundle need not be specified on the invocation of this method
  * because it will use nlsResources instead.
  ************************************************************************/
  nls: function( nlsKey, inserts, myBundle )
  {
    // attempt to retrieve the specified string via its key
    var s = null;
    
    // If a bundle has been specified, then use it, else
    // use the bundle specified on nlsResources.
    if (myBundle !== null)
    {
       s = myBundle[ nlsKey ];
    }
    else
    {
       s = this.nlsResources[ nlsKey ];
    }

    // substitute "key" if not able to resolve
    if ( ! s )
    {
      s = nlsKey;
    }

    // string insert(s) specified?
    if ( inserts ) 
    {
      // if not already a KW object -or- array 
      if ( ! ( typeof inserts == 'object' ) )
      {
        // convert single number/string/... to array
        inserts = [ inserts ];
      }

      // perform the substitutions
      s = dojo.string.substitute( s, inserts );
    }

    return( s );
  }
    
} );

//Declare a new Error type to throw when a Global context error is detected
dojo.declare("ibm.tivoli.tip.dojo.Global.InvalidContextError", Error, {});

}

if(!dojo._hasResource["dojo.date"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date"] = true;
dojo.provide("dojo.date");

dojo.date.getDaysInMonth = function(/*Date*/dateObject){
	//	summary:
	//		Returns the number of days in the month used by dateObject
	var month = dateObject.getMonth();
	var days = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
	if(month == 1 && dojo.date.isLeapYear(dateObject)){ return 29; } // Number
	return days[month]; // Number
}

dojo.date.isLeapYear = function(/*Date*/dateObject){
	//	summary:
	//		Determines if the year of the dateObject is a leap year
	//	description:
	//		Leap years are years with an additional day YYYY-02-29, where the
	//		year number is a multiple of four with the following exception: If
	//		a year is a multiple of 100, then it is only a leap year if it is
	//		also a multiple of 400. For example, 1900 was not a leap year, but
	//		2000 is one.

	var year = dateObject.getFullYear();
	return !(year%400) || (!(year%4) && !!(year%100)); // Boolean
}

// FIXME: This is not localized
dojo.date.getTimezoneName = function(/*Date*/dateObject){
	//	summary:
	//		Get the user's time zone as provided by the browser
	// dateObject:
	//		Needed because the timezone may vary with time (daylight savings)
	//	description:
	//		Try to get time zone info from toString or toLocaleString method of
	//		the Date object -- UTC offset is not a time zone.  See
	//		http://www.twinsun.com/tz/tz-link.htm Note: results may be
	//		inconsistent across browsers.

	var str = dateObject.toString(); // Start looking in toString
	var tz = ''; // The result -- return empty string if nothing found
	var match;

	// First look for something in parentheses -- fast lookup, no regex
	var pos = str.indexOf('(');
	if(pos > -1){
		tz = str.substring(++pos, str.indexOf(')'));
	}else{
		// If at first you don't succeed ...
		// If IE knows about the TZ, it appears before the year
		// Capital letters or slash before a 4-digit year 
		// at the end of string
		var pat = /([A-Z\/]+) \d{4}$/;
		if((match = str.match(pat))){
			tz = match[1];
		}else{
		// Some browsers (e.g. Safari) glue the TZ on the end
		// of toLocaleString instead of putting it in toString
			str = dateObject.toLocaleString();
			// Capital letters or slash -- end of string, 
			// after space
			pat = / ([A-Z\/]+)$/;
			if((match = str.match(pat))){
				tz = match[1];
			}
		}
	}

	// Make sure it doesn't somehow end up return AM or PM
	return (tz == 'AM' || tz == 'PM') ? '' : tz; // String
}

// Utility methods to do arithmetic calculations with Dates

dojo.date.compare = function(/*Date*/date1, /*Date?*/date2, /*String?*/portion){
	//	summary:
	//		Compare two date objects by date, time, or both.
	//	description:
	//  	Returns 0 if equal, positive if a > b, else negative.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	portion:
	//		A string indicating the "date" or "time" portion of a Date object.
	//		Compares both "date" and "time" by default.  One of the following:
	//		"date", "time", "datetime"

	// Extra step required in copy for IE - see #3112
	date1 = new Date(Number(date1));
	date2 = new Date(Number(date2 || new Date()));

	if(typeof portion !== "undefined"){
		if(portion == "date"){
			// Ignore times and compare dates.
			date1.setHours(0, 0, 0, 0);
			date2.setHours(0, 0, 0, 0);
		}else if(portion == "time"){
			// Ignore dates and compare times.
			date1.setFullYear(0, 0, 0);
			date2.setFullYear(0, 0, 0);
		}
	}
	
	if(date1 > date2){ return 1; } // int
	if(date1 < date2){ return -1; } // int
	return 0; // int
};

dojo.date.add = function(/*Date*/date, /*String*/interval, /*int*/amount){
	//	summary:
	//		Add to a Date in intervals of different size, from milliseconds to years
	//	date: Date
	//		Date object to start with
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//	amount:
	//		How much to add to the date.

	var sum = new Date(Number(date)); // convert to Number before copying to accomodate IE (#3112)
	var fixOvershoot = false;
	var property = "Date";

	switch(interval){
		case "day":
			break;
		case "weekday":
			//i18n FIXME: assumes Saturday/Sunday weekend, but even this is not standard.  There are CLDR entries to localize this.
			var days, weeks;
			var adj = 0;
			// Divide the increment time span into weekspans plus leftover days
			// e.g., 8 days is one 5-day weekspan / and two leftover days
			// Can't have zero leftover days, so numbers divisible by 5 get
			// a days value of 5, and the remaining days make up the number of weeks
			var mod = amount % 5;
			if(!mod){
				days = (amount > 0) ? 5 : -5;
				weeks = (amount > 0) ? ((amount-5)/5) : ((amount+5)/5);
			}else{
				days = mod;
				weeks = parseInt(amount/5);
			}
			// Get weekday value for orig date param
			var strt = date.getDay();
			// Orig date is Sat / positive incrementer
			// Jump over Sun
			if(strt == 6 && amount > 0){
				adj = 1;
			}else if(strt == 0 && amount < 0){
			// Orig date is Sun / negative incrementer
			// Jump back over Sat
				adj = -1;
			}
			// Get weekday val for the new date
			var trgt = strt + days;
			// New date is on Sat or Sun
			if(trgt == 0 || trgt == 6){
				adj = (amount > 0) ? 2 : -2;
			}
			// Increment by number of weeks plus leftover days plus
			// weekend adjustments
			amount = 7 * weeks + days + adj;
			break;
		case "year":
			property = "FullYear";
			// Keep increment/decrement from 2/29 out of March
			fixOvershoot = true;
			break;
		case "week":
			amount *= 7;
			break;
		case "quarter":
			// Naive quarter is just three months
			amount *= 3;
			// fallthrough...
		case "month":
			// Reset to last day of month if you overshoot
			fixOvershoot = true;
			property = "Month";
			break;
		case "hour":
		case "minute":
		case "second":
		case "millisecond":
			property = "UTC" + interval.charAt(0).toUpperCase() + interval.substring(1) + "s";
	}

	if(property){
		sum["set"+property](sum["get"+property]()+amount);
	}

	if(fixOvershoot && (sum.getDate() < date.getDate())){
		sum.setDate(0);
	}

	return sum; // Date
};

dojo.date.difference = function(/*Date*/date1, /*Date?*/date2, /*String?*/interval){
	//	summary:
	//		Get the difference in a specific unit of time (e.g., number of
	//		months, weeks, days, etc.) between two dates, rounded to the
	//		nearest integer.
	//	date1:
	//		Date object
	//	date2:
	//		Date object.  If not specified, the current Date is used.
	//	interval:
	//		A string representing the interval.  One of the following:
	//			"year", "month", "day", "hour", "minute", "second",
	//			"millisecond", "quarter", "week", "weekday"
	//		Defaults to "day".

	date2 = date2 || new Date();
	interval = interval || "day";
	var yearDiff = date2.getFullYear() - date1.getFullYear();
	var delta = 1; // Integer return value

	switch(interval){
		case "quarter":
			var m1 = date1.getMonth();
			var m2 = date2.getMonth();
			// Figure out which quarter the months are in
			var q1 = Math.floor(m1/3) + 1;
			var q2 = Math.floor(m2/3) + 1;
			// Add quarters for any year difference between the dates
			q2 += (yearDiff * 4);
			delta = q2 - q1;
			break;
		case "weekday":
			var days = Math.round(dojo.date.difference(date1, date2, "day"));
			var weeks = parseInt(dojo.date.difference(date1, date2, "week"));
			var mod = days % 7;

			// Even number of weeks
			if(mod == 0){
				days = weeks*5;
			}else{
				// Weeks plus spare change (< 7 days)
				var adj = 0;
				var aDay = date1.getDay();
				var bDay = date2.getDay();

				weeks = parseInt(days/7);
				mod = days % 7;
				// Mark the date advanced by the number of
				// round weeks (may be zero)
				var dtMark = new Date(date1);
				dtMark.setDate(dtMark.getDate()+(weeks*7));
				var dayMark = dtMark.getDay();

				// Spare change days -- 6 or less
				if(days > 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = -1;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 0;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = -1;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = -2;
							break;
						// Range contains weekend
						case (dayMark + mod) > 5:
							adj = -2;
					}
				}else if(days < 0){
					switch(true){
						// Range starts on Sat
						case aDay == 6:
							adj = 0;
							break;
						// Range starts on Sun
						case aDay == 0:
							adj = 1;
							break;
						// Range ends on Sat
						case bDay == 6:
							adj = 2;
							break;
						// Range ends on Sun
						case bDay == 0:
							adj = 1;
							break;
						// Range contains weekend
						case (dayMark + mod) < 0:
							adj = 2;
					}
				}
				days += adj;
				days -= (weeks*2);
			}
			delta = days;
			break;
		case "year":
			delta = yearDiff;
			break;
		case "month":
			delta = (date2.getMonth() - date1.getMonth()) + (yearDiff * 12);
			break;
		case "week":
			// Truncate instead of rounding
			// Don't use Math.floor -- value may be negative
			delta = parseInt(dojo.date.difference(date1, date2, "day")/7);
			break;
		case "day":
			delta /= 24;
			// fallthrough
		case "hour":
			delta /= 60;
			// fallthrough
		case "minute":
			delta /= 60;
			// fallthrough
		case "second":
			delta /= 1000;
			// fallthrough
		case "millisecond":
			delta *= date2.getTime() - date1.getTime();
	}

	// Round for fractional values and DST leaps
	return Math.round(delta); // Number (integer)
};

}

if(!dojo._hasResource["dojo.cldr.supplemental"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cldr.supplemental"] = true;
dojo.provide("dojo.cldr.supplemental");



dojo.cldr.supplemental.getFirstDayOfWeek = function(/*String?*/locale){
// summary: Returns a zero-based index for first day of the week
// description:
//		Returns a zero-based index for first day of the week, as used by the local (Gregorian) calendar.
//		e.g. Sunday (returns 0), or Monday (returns 1)

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/firstDay
	var firstDay = {/*default is 1=Monday*/
		mv:5,
		ae:6,af:6,bh:6,dj:6,dz:6,eg:6,er:6,et:6,iq:6,ir:6,jo:6,ke:6,kw:6,lb:6,ly:6,ma:6,om:6,qa:6,sa:6,
		sd:6,so:6,tn:6,ye:6,
		as:0,au:0,az:0,bw:0,ca:0,cn:0,fo:0,ge:0,gl:0,gu:0,hk:0,ie:0,il:0,is:0,jm:0,jp:0,kg:0,kr:0,la:0,
		mh:0,mo:0,mp:0,mt:0,nz:0,ph:0,pk:0,sg:0,th:0,tt:0,tw:0,um:0,us:0,uz:0,vi:0,za:0,zw:0,
		et:0,mw:0,ng:0,tj:0,
		gb:0,
		sy:4
	};

	var country = dojo.cldr.supplemental._region(locale);
	var dow = firstDay[country];
	return (typeof dow == 'undefined') ? 1 : dow; /*Number*/
};

dojo.cldr.supplemental._region = function(/*String?*/locale){
	locale = dojo.i18n.normalizeLocale(locale);
	var tags = locale.split('-');
	var region = tags[1];
	if(!region){
		// IE often gives language only (#2269)
		// Arbitrary mappings of language-only locales to a country:
        region = {de:"de", en:"us", es:"es", fi:"fi", fr:"fr", hu:"hu", it:"it",
        ja:"jp", ko:"kr", nl:"nl", pt:"br", sv:"se", zh:"cn"}[tags[0]];
	}else if(region.length == 4){
		// The ISO 3166 country code is usually in the second position, unless a
		// 4-letter script is given. See http://www.ietf.org/rfc/rfc4646.txt
		region = tags[2];
	}
	return region;
}

dojo.cldr.supplemental.getWeekend = function(/*String?*/locale){
// summary: Returns a hash containing the start and end days of the weekend
// description:
//		Returns a hash containing the start and end days of the weekend according to local custom using locale,
//		or by default in the user's locale.
//		e.g. {start:6, end:0}

	// from http://www.unicode.org/cldr/data/common/supplemental/supplementalData.xml:supplementalData/weekData/weekend{Start,End}
	var weekendStart = {/*default is 6=Saturday*/
		eg:5,il:5,sy:5,
		'in':0,
		ae:4,bh:4,dz:4,iq:4,jo:4,kw:4,lb:4,ly:4,ma:4,om:4,qa:4,sa:4,sd:4,tn:4,ye:4		
	};

	var weekendEnd = {/*default is 0=Sunday*/
		ae:5,bh:5,dz:5,iq:5,jo:5,kw:5,lb:5,ly:5,ma:5,om:5,qa:5,sa:5,sd:5,tn:5,ye:5,af:5,ir:5,
		eg:6,il:6,sy:6
	};

	var country = dojo.cldr.supplemental._region(locale);
	var start = weekendStart[country];
	var end = weekendEnd[country];
	if(typeof start == 'undefined'){start=6;}
	if(typeof end == 'undefined'){end=0;}
	return {start:start, end:end}; /*Object {start,end}*/
};

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

//	return str.replace(/([\f\b\n\t\r[\^$|?*+(){}])/gm, "\\$1"); // string
	return str.replace(/([\.$?*!=:|{}\(\)\[\]\\\/^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
}

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression. 
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
}

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. 
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
}

}

if(!dojo._hasResource["dojo.date.locale"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.locale"] = true;
dojo.provide("dojo.date.locale");

// Localization methods for Date.   Honor local customs using locale-dependent dojo.cldr data.







// Load the bundles containing localization information for
// names and formats


//NOTE: Everything in this module assumes Gregorian calendars.
// Other calendars will be implemented in separate modules.

(function(){
	// Format a pattern without literals
	function formatPattern(dateObject, bundle, pattern){
		return pattern.replace(/([a-z])\1*/ig, function(match){
			var s;
			var c = match.charAt(0);
			var l = match.length;
			var pad;
			var widthList = ["abbr", "wide", "narrow"];
			switch(c){
				case 'G':
					s = bundle[(l < 4) ? "eraAbbr" : "eraNames"][dateObject.getFullYear() < 0 ? 0 : 1];
					break;
				case 'y':
					s = dateObject.getFullYear();
					switch(l){
						case 1:
							break;
						case 2:
							s = String(s); s = s.substr(s.length - 2);
							break;
						default:
							pad = true;
					}
					break;
				case 'Q':
				case 'q':
					s = Math.ceil((dateObject.getMonth()+1)/3);
//					switch(l){
//						case 1: case 2:
							pad = true;
//							break;
//						case 3: case 4: // unimplemented
//					}
					break;
				case 'M':
				case 'L':
					var m = dateObject.getMonth();
					var width;
					switch(l){
						case 1: case 2:
							s = m+1; pad = true;
							break;
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "L") ? "standalone" : "format";
						var prop = ["months",type,width].join("-");
						s = bundle[prop][m];
					}
					break;
				case 'w':
					var firstDay = 0;
					s = dojo.date.locale._getWeekOfYear(dateObject, firstDay); pad = true;
					break;
				case 'd':
					s = dateObject.getDate(); pad = true;
					break;
				case 'D':
					s = dojo.date.locale._getDayOfYear(dateObject); pad = true;
					break;
				case 'E':
				case 'e':
				case 'c': // REVIEW: don't see this in the spec?
					var d = dateObject.getDay();
					var width;
					switch(l){
						case 1: case 2:
							if(c == 'e'){
								var first = dojo.cldr.supplemental.getFirstDayOfWeek(options.locale);
								d = (d-first+7)%7;
							}
							if(c != 'c'){
								s = d+1; pad = true;
								break;
							}
							// else fallthrough...
						case 3: case 4: case 5:
							width = widthList[l-3];
							break;
					}
					if(width){
						var type = (c == "c") ? "standalone" : "format";
						var prop = ["days",type,width].join("-");
						s = bundle[prop][d];
					}
					break;
				case 'a':
					var timePeriod = (dateObject.getHours() < 12) ? 'am' : 'pm';
					s = bundle[timePeriod];
					break;
				case 'h':
				case 'H':
				case 'K':
				case 'k':
					var h = dateObject.getHours();
					// strange choices in the date format make it impossible to write this succinctly
					switch (c) {
						case 'h': // 1-12
							s = (h % 12) || 12;
							break;
						case 'H': // 0-23
							s = h;
							break;
						case 'K': // 0-11
							s = (h % 12);
							break;
						case 'k': // 1-24
							s = h || 24;
							break;
					}
					pad = true;
					break;
				case 'm':
					s = dateObject.getMinutes(); pad = true;
					break;
				case 's':
					s = dateObject.getSeconds(); pad = true;
					break;
				case 'S':
					s = Math.round(dateObject.getMilliseconds() * Math.pow(10, l-3)); pad = true;
					break;
				case 'v': // FIXME: don't know what this is. seems to be same as z?
				case 'z':
					// We only have one timezone to offer; the one from the browser
					s = dojo.date.getTimezoneName(dateObject);
					if(s){break;}
					l=4;
					// fallthrough... use GMT if tz not available
				case 'Z':
					var offset = dateObject.getTimezoneOffset();
					var tz = [
						(offset<=0 ? "+" : "-"),
						dojo.string.pad(Math.floor(Math.abs(offset)/60), 2),
						dojo.string.pad(Math.abs(offset)% 60, 2)
					];
					if(l==4){
						tz.splice(0, 0, "GMT");
						tz.splice(3, 0, ":");
					}
					s = tz.join("");
					break;
//				case 'Y': case 'u': case 'W': case 'F': case 'g': case 'A':
//					console.debug(match+" modifier unimplemented");
				default:
					throw new Error("dojo.date.locale.format: invalid pattern char: "+pattern);
			}
			if(pad){ s = dojo.string.pad(s, l); }
			return s;
		});
	}

dojo.date.locale.format = function(/*Date*/dateObject, /*Object?*/options){
	// summary:
	//		Format a Date object as a String, using locale-specific settings.
	//
	// description:
	//		Create a string from a Date object using a known localized pattern.
	//		By default, this method formats both date and time from dateObject.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/tr35-4.html#Date_Format_Patterns
	//
	// dateObject:
	//		the date and/or time to be formatted.  If a time only is formatted,
	//		the values in the year, month, and day fields are irrelevant.  The
	//		opposite is true when formatting only dates.
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string}
	//		selector- choice of 'time','date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules

	options = options || {};

	var locale = dojo.i18n.normalizeLocale(options.locale);
	var formatLength = options.formatLength || 'short';
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var str = [];
	var sauce = dojo.hitch(this, formatPattern, dateObject, bundle);
	if(options.selector == "year"){
		// Special case as this is not yet driven by CLDR data
		var year = dateObject.getFullYear();
		if(locale.match(/^zh|^ja/)){
			year += "\u5E74";
		}
		return year;
	}
	if(options.selector != "time"){
		var datePattern = options.datePattern || bundle["dateFormat-"+formatLength];
		if(datePattern){str.push(_processPattern(datePattern, sauce));}
	}
	if(options.selector != "date"){
		var timePattern = options.timePattern || bundle["timeFormat-"+formatLength];
		if(timePattern){str.push(_processPattern(timePattern, sauce));}
	}
	var result = str.join(" "); //TODO: use locale-specific pattern to assemble date + time
	return result; // String
};

dojo.date.locale.regexp = function(/*Object?*/options){
	// summary:
	//		Builds the regular needed to parse a localized date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		locale- override the locale used to determine formatting rules

	return dojo.date.locale._parseInfo(options).regexp; // String
};

dojo.date.locale._parseInfo = function(/*Object?*/options){
	options = options || {};
	var locale = dojo.i18n.normalizeLocale(options.locale);
	var bundle = dojo.date.locale._getGregorianBundle(locale);
	var formatLength = options.formatLength || 'short';
	var datePattern = options.datePattern || bundle["dateFormat-" + formatLength];
	var timePattern = options.timePattern || bundle["timeFormat-" + formatLength];
	var pattern;
	if(options.selector == 'date'){
		pattern = datePattern;
	}else if(options.selector == 'time'){
		pattern = timePattern;
	}else{
		pattern = datePattern + ' ' + timePattern; //TODO: use locale-specific pattern to assemble date + time
	}

	var tokens = [];
	var re = _processPattern(pattern, dojo.hitch(this, _buildDateTimeRE, tokens, bundle, options));
	return {regexp: re, tokens: tokens, bundle: bundle};
};

dojo.date.locale.parse = function(/*String*/value, /*Object?*/options){
	// summary:
	//		Convert a properly formatted string to a primitive Date object,
	//		using locale-specific settings.
	//
	// description:
	//		Create a Date object from a string using a known localized pattern.
	//		By default, this method parses looking for both date and time in the string.
	//		Formatting patterns are chosen appropriate to the locale.  Different
	//		formatting lengths may be chosen, with "full" used by default.
	//		Custom patterns may be used or registered with translations using
	//		the addCustomFormats method.
	//		Formatting patterns are implemented using the syntax described at
	//		http://www.unicode.org/reports/tr35/#Date_Format_Patterns
	//
	// value:
	//		A string representation of a date
	//
	// options: object {selector: string, formatLength: string, datePattern: string, timePattern: string, locale: string, strict: boolean}
	//		selector- choice of 'time', 'date' (default: date and time)
	//		formatLength- choice of long, short, medium or full (plus any custom additions).  Defaults to 'short'
	//		datePattern,timePattern- override pattern with this string
	//		am,pm- override strings for am/pm in times
	//		locale- override the locale used to determine formatting rules
	//		strict- strict parsing, off by default

	var info = dojo.date.locale._parseInfo(options);
	var tokens = info.tokens, bundle = info.bundle;
	var re = new RegExp("^" + info.regexp + "$");
	var match = re.exec(value);
	if(!match){ return null; } // null

	var widthList = ['abbr', 'wide', 'narrow'];
	//1972 is a leap year.  We want to avoid Feb 29 rolling over into Mar 1,
	//in the cases where the year is parsed after the month and day.
	var result = new Date(1972, 0);
	var expected = {};
	var amPm = "";
	dojo.forEach(match, function(v, i){
		if(!i){return;}
		var token=tokens[i-1];
		var l=token.length;
		switch(token.charAt(0)){
			case 'y':
				if(l != 2){
					//interpret year literally, so '5' would be 5 A.D.
					result.setFullYear(v);
					expected.year = v;
				}else{
					if(v<100){
						v = Number(v);
						//choose century to apply, according to a sliding window
						//of 80 years before and 20 years after present year
						var year = '' + new Date().getFullYear();
						var century = year.substring(0, 2) * 100;
						var yearPart = Number(year.substring(2, 4));
						var cutoff = Math.min(yearPart + 20, 99);
						var num = (v < cutoff) ? century + v : century - 100 + v;
						result.setFullYear(num);
						expected.year = num;
					}else{
						//we expected 2 digits and got more...
						if(options.strict){
							return null;
						}
						//interpret literally, so '150' would be 150 A.D.
						//also tolerate '1950', if 'yyyy' input passed to 'yy' format
						result.setFullYear(v);
						expected.year = v;
					}
				}
				break;
			case 'M':
				if(l>2){
					var months = bundle['months-format-' + widthList[l-3]].concat();
					if(!options.strict){
						//Tolerate abbreviating period in month part
						//Case-insensitive comparison
						v = v.replace(".","").toLowerCase();
						months = dojo.map(months, function(s){ return s.replace(".","").toLowerCase(); } );
					}
					v = dojo.indexOf(months, v);
					if(v == -1){
//						console.debug("dojo.date.locale.parse: Could not parse month name: '" + v + "'.");
						return null;
					}
				}else{
					v--;
				}
				result.setMonth(v);
				expected.month = v;
				break;
			case 'E':
			case 'e':
				var days = bundle['days-format-' + widthList[l-3]].concat();
				if(!options.strict){
					//Case-insensitive comparison
					v = v.toLowerCase();
					days = dojo.map(days, "".toLowerCase);
				}
				v = dojo.indexOf(days, v);
				if(v == -1){
//					console.debug("dojo.date.locale.parse: Could not parse weekday name: '" + v + "'.");
					return null;
				}

				//TODO: not sure what to actually do with this input,
				//in terms of setting something on the Date obj...?
				//without more context, can't affect the actual date
				//TODO: just validate?
				break;
			case 'd':
				result.setDate(v);
				expected.date = v;
				break;
			case 'D':
				//FIXME: need to defer this until after the year is set for leap-year?
				result.setMonth(0);
				result.setDate(v);
				break;
			case 'a': //am/pm
				var am = options.am || bundle.am;
				var pm = options.pm || bundle.pm;
				if(!options.strict){
					var period = /\./g;
					v = v.replace(period,'').toLowerCase();
					am = am.replace(period,'').toLowerCase();
					pm = pm.replace(period,'').toLowerCase();
				}
				if(options.strict && v != am && v != pm){
//					console.debug("dojo.date.locale.parse: Could not parse am/pm part.");
					return null;
				}

				// we might not have seen the hours field yet, so store the state and apply hour change later
				amPm = (v == pm) ? 'p' : (v == am) ? 'a' : '';
				break;
			case 'K': //hour (1-24)
				if(v==24){v=0;}
				// fallthrough...
			case 'h': //hour (1-12)
			case 'H': //hour (0-23)
			case 'k': //hour (0-11)
				//TODO: strict bounds checking, padding
				if(v > 23){
//					console.debug("dojo.date.locale.parse: Illegal hours value");
					return null;
				}

				//in the 12-hour case, adjusting for am/pm requires the 'a' part
				//which could come before or after the hour, so we will adjust later
				result.setHours(v);
				break;
			case 'm': //minutes
				result.setMinutes(v);
				break;
			case 's': //seconds
				result.setSeconds(v);
				break;
			case 'S': //milliseconds
				result.setMilliseconds(v);
//				break;
//			case 'w':
//TODO				var firstDay = 0;
//			default:
//TODO: throw?
//				console.debug("dojo.date.locale.parse: unsupported pattern char=" + token.charAt(0));
		}
	});

	var hours = result.getHours();
	if(amPm === 'p' && hours < 12){
		result.setHours(hours + 12); //e.g., 3pm -> 15
	}else if(amPm === 'a' && hours == 12){
		result.setHours(0); //12am -> 0
	}

	//validate parse date fields versus input date fields
	if(expected.year && result.getFullYear() != expected.year){
//		console.debug("dojo.date.locale.parse: Parsed year: '" + result.getFullYear() + "' did not match input year: '" + expected.year + "'.");
		return null;
	}
	if(expected.month && result.getMonth() != expected.month){
//		console.debug("dojo.date.locale.parse: Parsed month: '" + result.getMonth() + "' did not match input month: '" + expected.month + "'.");
		return null;
	}
	if(expected.date && result.getDate() != expected.date){
//		console.debug("dojo.date.locale.parse: Parsed day of month: '" + result.getDate() + "' did not match input day of month: '" + expected.date + "'.");
		return null;
	}

	//TODO: implement a getWeekday() method in order to test 
	//validity of input strings containing 'EEE' or 'EEEE'...
	return result; // Date
};

function _processPattern(pattern, applyPattern, applyLiteral, applyAll){
	//summary: Process a pattern with literals in it

	// Break up on single quotes, treat every other one as a literal, except '' which becomes '
	var identity = function(x){return x;};
	applyPattern = applyPattern || identity;
	applyLiteral = applyLiteral || identity;
	applyAll = applyAll || identity;

	//split on single quotes (which escape literals in date format strings) 
	//but preserve escaped single quotes (e.g., o''clock)
	var chunks = pattern.match(/(''|[^'])+/g); 
	var literal = false;

	dojo.forEach(chunks, function(chunk, i){
		if(!chunk){
			chunks[i]='';
		}else{
			chunks[i]=(literal ? applyLiteral : applyPattern)(chunk);
			literal = !literal;
		}
	});
	return applyAll(chunks.join(''));
}

function _buildDateTimeRE(tokens, bundle, options, pattern){
	pattern = dojo.regexp.escapeString(pattern);
	if(!options.strict){ pattern = pattern.replace(" a", " ?a"); } // kludge to tolerate no space before am/pm
	return pattern.replace(/([a-z])\1*/ig, function(match){
		// Build a simple regexp.  Avoid captures, which would ruin the tokens list
		var s;
		var c = match.charAt(0);
		var l = match.length;
		var p2 = '', p3 = '';
		if(options.strict){
			if(l > 1){ p2 = '0' + '{'+(l-1)+'}'; }
			if(l > 2){ p3 = '0' + '{'+(l-2)+'}'; }
		}else{
			p2 = '0?'; p3 = '0{0,2}';
		}
		switch(c){
			case 'y':
				s = '\\d{2,4}';
				break;
			case 'M':
				s = (l>2) ? '\\S+' : p2+'[1-9]|1[0-2]';
				break;
			case 'D':
				s = p2+'[1-9]|'+p3+'[1-9][0-9]|[12][0-9][0-9]|3[0-5][0-9]|36[0-6]';
				break;
			case 'd':
				s = p2+'[1-9]|[12]\\d|3[01]';
				break;
			case 'w':
				s = p2+'[1-9]|[1-4][0-9]|5[0-3]';
				break;
		    case 'E':
				s = '\\S+';
				break;
			case 'h': //hour (1-12)
				s = p2+'[1-9]|1[0-2]';
				break;
			case 'k': //hour (0-11)
				s = p2+'\\d|1[01]';
				break;
			case 'H': //hour (0-23)
				s = p2+'\\d|1\\d|2[0-3]';
				break;
			case 'K': //hour (1-24)
				s = p2+'[1-9]|1\\d|2[0-4]';
				break;
			case 'm':
			case 's':
				s = '[0-5]\\d';
				break;
			case 'S':
				s = '\\d{'+l+'}';
				break;
			case 'a':
				var am = options.am || bundle.am || 'AM';
				var pm = options.pm || bundle.pm || 'PM';
				if(options.strict){
					s = am + '|' + pm;
				}else{
					s = am + '|' + pm;
					if(am != am.toLowerCase()){ s += '|' + am.toLowerCase(); }
					if(pm != pm.toLowerCase()){ s += '|' + pm.toLowerCase(); }
				}
				break;
			default:
			// case 'v':
			// case 'z':
			// case 'Z':
				s = ".*";
//				console.debug("parse of date format, pattern=" + pattern);
		}

		if(tokens){ tokens.push(match); }

		return "(" + s + ")"; // add capture
	}).replace(/[\xa0 ]/g, "[\\s\\xa0]"); // normalize whitespace.  Need explicit handling of \xa0 for IE.
}
})();

(function(){
var _customFormats = [];
dojo.date.locale.addCustomFormats = function(/*String*/packageName, /*String*/bundleName){
	// summary:
	//		Add a reference to a bundle containing localized custom formats to be
	//		used by date/time formatting and parsing routines.
	//
	// description:
	//		The user may add custom localized formats where the bundle has properties following the
	//		same naming convention used by dojo for the CLDR data: dateFormat-xxxx / timeFormat-xxxx
	//		The pattern string should match the format used by the CLDR.
	//		See dojo.date.format for details.
	//		The resources must be loaded by dojo.requireLocalization() prior to use

	_customFormats.push({pkg:packageName,name:bundleName});
};

dojo.date.locale._getGregorianBundle = function(/*String*/locale){
	var gregorian = {};
	dojo.forEach(_customFormats, function(desc){
		var bundle = dojo.i18n.getLocalization(desc.pkg, desc.name, locale);
		gregorian = dojo.mixin(gregorian, bundle);
	}, this);
	return gregorian; /*Object*/
};
})();

dojo.date.locale.addCustomFormats("dojo.cldr","gregorian");

dojo.date.locale.getNames = function(/*String*/item, /*String*/type, /*String?*/use, /*String?*/locale){
	// summary:
	//		Used to get localized strings from dojo.cldr for day or month names.
	//
	// item: 'months' || 'days'
	// type: 'wide' || 'narrow' || 'abbr' (e.g. "Monday", "Mon", or "M" respectively, in English)
	// use: 'standAlone' || 'format' (default)
	// locale: override locale used to find the names

	var label;
	var lookup = dojo.date.locale._getGregorianBundle(locale);
	var props = [item, use, type];
	if(use == 'standAlone'){
		label = lookup[props.join('-')];
	}
	props[1] = 'format';

	// return by copy so changes won't be made accidentally to the in-memory model
	return (label || lookup[props.join('-')]).concat(); /*Array*/
};

dojo.date.locale.isWeekend = function(/*Date?*/dateObject, /*String?*/locale){
	// summary:
	//	Determines if the date falls on a weekend, according to local custom.

	var weekend = dojo.cldr.supplemental.getWeekend(locale);
	var day = (dateObject || new Date()).getDay();
	if(weekend.end < weekend.start){
		weekend.end += 7;
		if(day < weekend.start){ day += 7; }
	}
	return day >= weekend.start && day <= weekend.end; // Boolean
};

// These are used only by format and strftime.  Do they need to be public?  Which module should they go in?

dojo.date.locale._getDayOfYear = function(/*Date*/dateObject){
	// summary: gets the day of the year as represented by dateObject
	return dojo.date.difference(new Date(dateObject.getFullYear(), 0, 1), dateObject) + 1; // Number
};

dojo.date.locale._getWeekOfYear = function(/*Date*/dateObject, /*Number*/firstDayOfWeek){
	if(arguments.length == 1){ firstDayOfWeek = 0; } // Sunday

	var firstDayOfYear = new Date(dateObject.getFullYear(), 0, 1).getDay();
	var adj = (firstDayOfYear - firstDayOfWeek + 7) % 7;
	var week = Math.floor((dojo.date.locale._getDayOfYear(dateObject) + adj - 1) / 7);

	// if year starts on the specified day, start counting weeks at 1
	if(firstDayOfYear == firstDayOfWeek){ week++; }

	return week; // Number
};

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.util"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.util"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2007
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.util");

ibm.tivoli.tip.dojo.util._stdDateFormat = { // selector:     'dateOnly', 
                                      formatLength: 'short' 
                                    };

/************************************************************************
* "apply" any GLOBAL-based data of interest.
************************************************************************/	
ibm.tivoli.tip.dojo.util.applyGlobal = function( global ) 
{
  ibm.tivoli.tip.dojo.util._stdDateFormat = global.l10n.date.stdTS;

  console.debug( "** util.applyGlobal **", dojo.toJson(ibm.tivoli.tip.dojo.util._stdDateFormat ) ); 
};

/************************************************************************
* Format a DATE with NLS rules
************************************************************************/	
ibm.tivoli.tip.dojo.util.formatDate = function( date ) 
{
	fDate = ( date == null )
          ? ""
          : dojo.date.locale.format( date, ibm.tivoli.tip.dojo.util._stdDateFormat );
 
	return( fDate );
};

/************************************************************************
* Standard formatter for a table column containing Date objects.
************************************************************************/	
ibm.tivoli.tip.dojo.util.dateColumnFormater = function( /* Date */ cellValue )
 {
   // console.debug( "cell: ", cellValue );

   return( ibm.tivoli.tip.dojo.util.formatDate( cellValue ) );
 };

/************************************************************************
* Refresh the page
************************************************************************/	
ibm.tivoli.tip.dojo.util.refreshPage = function()
{
	window.location.reload( false );
};

/*****************************************************************************
* Return a string of all the fields contained in an object.
*****************************************************************************/
ibm.tivoli.tip.dojo.util.fieldsOf = function( object )
{ 
  s = "";

  if ( object && ( object !== null ) ) 
  {
    for ( var field in object )
    {
      s += field + "; ";
    }
  }

  return( s );
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
* 
* Returns dojo.Deferred() -- must add Callback
*
* example usage:
* ibm.tivoli.tip.dojo.util.clearDataStore( store ).addCallback( function(){} );
************************************************************************/ 
ibm.tivoli.tip.dojo.util.clearDataStore = function( store )
{
  var deferred = new dojo.Deferred();
    
  var callback = 
  {  
        deferred: deferred,
           store: store,        
      onComplete: function( items, request ) 
                  {                    
                    for ( var i in items )
                    {
                      this.store.deleteItem( items[i] );
                    }

                    this.deferred.callback();
                  }
  };
  
  store.fetch( { onComplete: dojo.hitch( callback, "onComplete" ) } );
  
  return deferred;
};

/************************************************************************
* Clear a datastore (must implement dojo.data.api.Write)
************************************************************************/ 
ibm.tivoli.tip.dojo.util.addItemsToDataStore = function( store, items )
{
  for ( var i in items )
  {
    store.newItem( items[i] );
  }

  store.save();
};

/************************************************************************
* Find the default name for a specific value using a name/value array.
*
* [
*  { value: "-1", name: this.global.nls( "CBOX_CONFIG_DEFAULT", { config : config.maxSnapshotsDefault } ) }, 
*  { value:  "0", name: bundle.CBOX_DISABLED }, 
*  { value:  "1", name:  "1" }
* ]
************************************************************************/ 
ibm.tivoli.tip.dojo.util.findDefaultName = function( value, nameValueArray )
{
  name = nameValueArray[ 0 ].name;     // default to first element

  for ( var i in nameValueArray )
  {
    nv = nameValueArray[ i ];

    if ( nv.value == value ) 
    {
      name = nv.name;
      break;
    }
  }

  // console.debug( "value: " + value + ", name: " + name );

  return( name );
};


}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.Server"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.Server"] = true;
/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/


dojo.provide("ibm.tivoli.tip.dojo.Server");
dojo.provide("ibm.tivoli.tip.dojo._Server");

ibm.tivoli.tip.dojo.Server.anyException  = "*"; 

ibm.tivoli.tip.dojo.Server._singleton    = null;

/****************************************************************************
* Get the "singleton" Server instance.
****************************************************************************/
/* "ibm.tivoli.tip.dojo._Server" */
ibm.tivoli.tip.dojo.Server.getSingleton = function( /* required: ibm.tivoli.tip.dojo._Global */ global )
{
  if ( global === undefined ) {
  	throw new Error( "ibm.tivoli.tip.dojo.Server requires a 'global' object reference" );
  }
  		
  //if the singleton doesn't exist yet --> create it now
  //there must be a unique singleton per global instance
  if ( global._serverSingleton === undefined || global._serverSingleton === null )
  {
    global._serverSingleton = new ibm.tivoli.tip.dojo._Server( { global : global } );
  }  
  
  return global._serverSingleton;	
};

/****************************************************************************
* Provides I/O access to the controller servlet.
* Handles TIP protocol of returning server-side exceptions in a valid
* data response besides Dojo's error and timeout conditions.
****************************************************************************/
dojo.declare("ibm.tivoli.tip.dojo._Server", 
             null,
{	 
	_totalRequests:   0,        // total server requests made
  _activeRequests:  0,        // # server request(s) currently active
	
  /************************************************************************
  * Object constructor
  ************************************************************************/
	constructor: function( mixinData )
	{
    dojo.mixin( this, mixinData );
	},
	
	/************************************************************************
	* Perform a HTTP GET request.
	************************************************************************/
	/* void */ get: function( /* Object */ requestKW )
	{
    this._preRequest( requestKW );
	
    //console.debug( "Ready to xhrGet", requestKW );
    //console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(requestKW) );
    //console.debug( "Timeout", requestKW.timeout );

	  dojo.xhrGet( requestKW ); 
	},
	
	/************************************************************************
	* Perform a HTTP POST request.  Preferred for data forms that do
  * NOT contain any file-input fields...
	************************************************************************/
	/* void */ post: function( /* Object */ requestKW )
	{
    this._preRequest( requestKW );
	
	  dojo.xhrPost( requestKW ); 
	},

  /************************************************************************
  * Common PRE request processing of the request KW object.
  ************************************************************************/
  /* Object */ _preRequest: function( /* Object */ requestKW )
  {
    this._requestStarting( ( requestKW.showBusyDialog !== undefined ) ? requestKW.showBusyDialog : true );                 

    requestKW._server      = this;                 // save self reference for use in load/error handlers below   
    requestKW._requestID   = this._totalRequests;  // unofficial request ID

    requestKW.url          = requestKW.appendURL ? 
      this.global.controller + requestKW.appendURL :
      this.global.controller;

    requestKW.handleAs	   = "json-comment-filtered";  
    requestKW.mimetype     = "application/json";
    
    requestKW.content.requestType  = "global";     // indicates that the request was sent here

    requestKW.preventCache = true;     // fix IE cache bug

    // provide default timeout if none specified

    if ( ! requestKW.timeout ) {
           requestKW.timeout = 0; 
    }

    // our code handles it all!  NOTE: trying to set callbacks on "deferred" object did NOT work

    requestKW.load  = this._loadHandler;
    requestKW.error = this._errorHandler;

    return( requestKW ); 
  },
	
	/************************************************************************
	* Standard JSON request "load" handler.
  *
  * Use the appropriate "how to handle" directions from the request:
  * - "onExc"      for server-side errors returned in the 'exc' field of response
  * - "onResponse" for response data
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used (aka requestKW)
	************************************************************************/
  /* response */ _loadHandler: function( response, ioArgs ) 
	{
    //console.debug( "************************ Server._loadHandler ************************" );

//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(this) );      proved its requestKW
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(response) );
//  console.debug( ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );    ioArgs.args == requestKW

    //console.debug( "  ** response", dojo.toJson(response) );

    var server = this._server;

    server._requestCompleted();

	  // was an exception detected/reported from the server-side?

    var exc = response.exc;

	  if ( exc )
	  { 
      //console.debug( "    !!!!! handling EXC !!!!!" );

      // ISC session timeout?
      if ( exc == "ISCSessionTimeoutException" ) 
      {
        ibm.tivoli.tip.dojo.util.refreshPage();
        return;
      }

      var howTo  = this.onExc;

      if ( ! howTo ) {
        howTo = { showMsg : true };    // use default
      }

      //console.debug( "  ?? howTo",  dojo.toJson(howTo) );

      // Crop the exc message to maximum size
      server.cropExcMsg( response );

      //TODO: handle new NLS enabled exceptions...
      var msgKW = null;
      
      if (response.excID) {
	      msgKW = { 
                   messageId : response.excID,
                   message   : response.excMsg
                  };
      } else {
    	  msgKW = { 
                   messageId : server.global.tipnls( "BIND_ERROR_ID" ),
                   message   : server.global.tipnls( "BIND_ERROR_MSG", response )
                  };
	  }

      //console.debug( "  ?? msgKW",  dojo.toJson(msgKW) );

      if ( howTo.showMsg ) {
        server.global.showErrorMessage( msgKW );
      }

      if ( howTo.callback ) {
        howTo.callback( msgKW, this /* requestKW */ );
      }
	  }
	
	  // No exception --> forward to callback
	  else
	  {
      howTo = this.onResponse;

      if ( howTo && howTo.callback ) {
	      howTo.callback( response, this /* requestKW */ );
      }
	  }

    return( response ); // Dojo best practice 
	},

	/************************************************************************
	* Standard JSON request error handler
  *
  * How to handle directions are contained in the request's "onExc" field.
  *
  *	NOTE "this" will be the object passed as argument to dojo.xhrXxxx() used
  *
  * error.message:    specific error message, "too much recursion"
  * error.fileName:   "https://localhost:16316/ibm/console/secure/isclite/scripts/dojo/dojo.js"
  * error.lineNumber: 20
  * error.stack:      callback stack (can be very deep )
  * error.name:       "InternalError"
  *
  * error.dojoType:   "timeout" or "cancel", but other error types are possible.
  * dojoType=timeout message=timeout exceeded
  *
  * Error: bad http response code:404 message=bad http response code:404
  * 
  * console.error("HTTP status code: ", ioArgs.xhr.status); 
	************************************************************************/
	/* data */ _errorHandler: function( error, ioArgs ) 
	{
    //console.debug( "************************ Server._errorHandler ************************" );

    //console.debug( "  >> this",   ibm.tivoli.tip.dojo.util.fieldsOf(this) );
    //console.debug( "  >> error",  ibm.tivoli.tip.dojo.util.fieldsOf(error) );
    //console.debug( "  >> ioArgs", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs) );
    //console.debug( "  >> ioArgs.args", ibm.tivoli.tip.dojo.util.fieldsOf(ioArgs.args) );

    //console.debug( "  -- error",  dojo.toJson(error) );

    var server = this._server;

    server._requestCompleted();

    var howTo = this.onError;
    var msgKW = null;

    if ( ! howTo ) {
      howTo = { showMsg : true };    // use default
    }

    if ( error.dojoType && ( error.dojoType === "timeout" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "BIND_TIMEOUT_ID" ),
                message   : server.global.tipnls( "BIND_TIMEOUT_MSG", error )
              }; 
    }
    else if ( error.name && ( error.name === "InternalError" ) ) 
    {
      msgKW = { 
                messageId : server.global.tipnls( "JS_ERROR_ID" ),
                message   : server.global.tipnls( "JS_ERROR_MSG", error )
              }; 
    }
    else
    {
      msgKW = { 
                messageId : server.global.tipnls( "XHR_ERROR_ID" ),
                message   : server.global.tipnls( "XHR_ERROR_MSG", error )
              }; 
    }

    if ( howTo.showMsg ) {
	    server.global.showErrorMessage( msgKW );
    }

    if ( howTo.callback ) {
      howTo.callback( msgKW, this /* requestKW */ );
    }

    return( error ); // Dojo best practice 
	},

  /************************************************************************
  * Starting a new server request.
  ************************************************************************/
  _requestStarting: function( showBusyDialog ) 
  {
    // increment total server requests 
    this._totalRequests++;

    // increment outstanding server requests 
    this._activeRequests++;

    // show wait dialog on first active request
    if ( showBusyDialog === true && this._activeRequests == 1 )
    {
      this.wait_timer = setInterval( dojo.hitch( this, "_showBusyPane" ), 100 );
    }
  },
  
  /************************************************************************
  * Show Busy Pane
  *    
  * Constructing a ibm.tivoli.tip.dijit.BusyPane and passing its ID on 
  * busyPaneID when constructing Server object will provide feedback
  * when requests are made
  ************************************************************************/
  _showBusyPane: function()
  {
    clearInterval( this.wait_timer );
    // in case of race condition, make sure that active requests > 0
    if ( this._activeRequests > 0 ) {
	    var busyPane = dijit.byId( this.busyPaneID );
	    if ( busyPane )
	    {
	      busyPane.show(true); // size to portlet
	    }
    }
  },

  /************************************************************************
  * Completing a server request.
  ************************************************************************/
  _requestCompleted: function() 
  {
	  //decrement the # outstanding requests 
	  this._activeRequests--;

	  // if no more active requests --> close wait dialog
	  if ( this._activeRequests === 0 )
	  {
		  clearInterval( this.wait_timer );
      
          var busyPane = dijit.byId( this.busyPaneID );
          if ( busyPane )
          {
            busyPane.hide();
          }
	  }  	
  },

  /***************************************************************************
  * Crop the message in the 'excMsg' field to the maximum allowed length.
  ***************************************************************************/
  cropExcMsg: function( response )
  {
    // provide a default exc message if field isn't already available
    if ( ! response.excMsg ) {
      response.excMsg = "-?-";
    }

    // access externalized MAX length
    var msgMax = this.global.nlsTIPResources.MAX_WIDTH_EXC_MSG;

    // exc message too long?
    if ( response.excMsg.length > msgMax ) 
    {
      response.excMsg = response.excMsg.substring( 0, msgMax ) + this.global.tipnls( "TEXT_CROPPED" );
    }
  }

} );

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenu"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/






dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenu");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProxy");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProvider");
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProvider");




dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenu",
              dijit.Menu,
{
	removeItemsOnClose: false,

	constructor: function () {
	},
	// override dijit..PopupMenu
	onOpen: function(/*Event*/ e){
		// summary: 
		//		Callback when menu is opened.
		this.inherited ("onOpen", arguments);
		if (this.removeItemsOnClose) {
			// hide temporarily while the menu is updated
			this.domNode.style.display = "none";
			this.removeItemsOnClose(e);
		}
	},

	// override parent
	closeAll: function(/*Boolean?*/force) {	
		this.inherited ("closeAll", arguments);
//		this._removeOnClose();
	},
	
/**
 * Display the context menu at an x,y position.
 *
 */
    displayContextMenu: function(x,y){
      //   dojo.stopEvent(e);

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			dijit.focus(savedFocus);
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: x,
			y: y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};

    },
/**
 * Destroy the menu and its submenus from the DOM. 
 * This is used to reuse IDs when dynamically creating
 * on demand context menus.
 * <pre>
 * Example below, checks to see if there is a menu already anmd removes
 * it before creating a new menu which is demand loaded.
 * 
 *          var cleanup=dijit.byId("progMenu");
 *          if (cleanup) {          //cleanup previous menu construction
 *             cleanup.destroyMenuDOM();
 *          }
 * </pre>
 **/
    destroyMenuDOM: function(){
      dojo.forEach(this.getDescendants(), function(child){ 
        child.destroy(true);
        if (child.popup) {
           child.popup.destroymenuDOM();
        }

      });
      this.destroy(true);
    }
});

dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProxy",
	[ dijit._Widget, dijit._Templated ],
{
	provider: null,  /* TIPContextMenuProvider */
	// Callback function that returns the selected data for the context menu
	getSelectedData: null,
	targetNodeIds: null,
	removeItemsOnClose: false,

	constructor: function() {
		// summary: A reusable widget for displaying a dynamic set of parameters to be 
		//  filled in at runtime.
	    this.menuItems_ = [];
	    this.menus_= [];
	    this.root_ =  null;
	    this.nodesToBind = [];
	    this.done_ = false; // this flag is needed to prevent redundant menus
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "messages");
	},
	postCreate: function() {
		this.inherited ("postCreate", arguments);
		if (this.provider !== null) {
			this.provider.initialize(this.id, dojo.hitch(this, this.addMenuItemsCallback), dojo.hitch (this, this.getSelectedData));
		} else {
			this.startup();
		}
	},
	startup: function () {
		this.inherited (arguments);
	},
	getTIPContextMenu: function () {
		return (this.root_);
	},
	bindDomNode: function (node) {
		if (this.root_) {
			this.root_.bindDomNode (node);
		} else {
			this.nodesToBind.push (node);
		}
	},
	bindPendingNodes: function () {
		for (var x in this.nodesToBind) {
			if (x) {
				this.root_.bindDomNode (this.nodesToBind[x]);
				this.nodesToBind.pop();
			}
		}
	},
	addMenuItemsCallback: function () {
		if (this.provider !== null && !this.done_) {
//			var items = this.provider.getItems();
			var menus = this.provider.getMenus();
			var aMenu = null;
			var items = null;
			var x = null;
			
			for (x in menus) {
				if (x) {
					aMenu = menus[x];
					items = aMenu.menuitems;
					if (items && items.length > 0) {
						// create a dijit:Menu
						var menu = null;
						if (aMenu.id !== this.id + "ROOT") {
							var pMenu = this.getById (this.menus_, aMenu.parentMenuId);
						 	menu = new dijit.Menu ({id: aMenu.id, parentMenu:pMenu });
						} else {
							this.id = this.id + "ROOT";
							menu = new dijit.Menu ({id: aMenu.id, targetNodeIds:this.targetNodeIds, removeItemsOnClose: this.removeItemsOnClose });
							this.root_ = menu;
							this.bindPendingNodes();
						}
						this.menus_.push(menu);
					}
				}
			}
			
			for (x in menus) {
				if (x) {
					aMenu = menus[x];
					items = aMenu.menuitems;
				
					if (items && items.length > 0) {
						var menu2 = this.getById (this.menus_, aMenu.id);
					
			            for (var i = 0; i < items.length; i++) {
			            	var item = null;
			            	if (items[i].id.toLowerCase() == "separator") {
			            		item = new dijit.MenuSeparator();
		    	        		menu2.addChild(item);
		        	    	} else if (items[i].submenuId) {
		            			var submenu = this.getById (this.menus_, items[i].submenuId);
		            			var submenuItem = new dijit.PopupMenuItem ({label:items[i].label, popup:submenu, id:items[i].id});
			            		menu2.addChild (submenuItem);
		    	        	} else { // dijit:MenuItem
		        	    		var fnc = new Function(
		            				"if (this.provider) { this.provider.getMenuCommand(\"" + items[i].id + "\", this.getSelectedData()); }"		);
//		            				+ "this._removeOnClose();"
			            		item = new dijit.MenuItem({ id: items[i].id,
    	       												label: items[i].label,
    	       												iconClass : (items[i].iconClass) ? items[i].iconClass : ""
           											});

								item.onClick = dojo.hitch (this, fnc);
           						if (items[i].enabled === false) {
            						item.setDisabled(true);
		            			} else {
	            					item.setDisabled(false);
    	        				}
  
    	        				menu2.addChild(item);
	    			       		this.menuItems_[item.id] = item;
	    			       		item.startup();
//	    			       		debugger;
		        	    	}
		            	}
					}
				}
			}

			// show the menu with the updates
			if (this.removeItemsOnClose) {
//				this.domNode.style.display = "";
			}
			
			// set flag to note that the menu has been built
			this.done_ = true;
		}


		for (var y in this.menus_) {
			if (y) {
				var myMenu = this.menus_[y];
				myMenu.startup();
			}
		}
	},
	
	getById: function (myArray, id) {
		for (var x = myArray.length - 1; x >= 0; x--) {
			var entry = myArray[x];
			if (entry.id == id) {
				return (entry);
			}
		}
		
		return (null);
	},
	
	onContextMenuOpen: function() {
		if (this.provider) {
			this.provider.onContextMenuOpen();
			var items = this.provider.getItems();
            for (var i = 0; i < items.length; i++) {
            	if (items[i].id.toLowerCase() != "separator") {
            		if (items[i].enabled === false) {
            			this.menuItems_[items[i].id].setDisabled(true);
            		}
            		else {
            			this.menuItems_[items[i].id].setDisabled(false);
            		}
            	}
            }
		}
	},
	
	updateProviderContext: function(/* JSON */ context) {
		// summary:
		// Updates the provider with the given context and then re-builds the 
		// menu items
		if (this.provider) {
			this._removeOnClose(); // in case closeAll was not called
			var callback = dojo.hitch(this, this.addMenuItemsCallback);
			this.provider.update(context, callback);
			this.provider.initialize(this.id, callback, dojo.hitch (this, this.getSelectedData));
		}
	},

	_removeOnClose: function() {
		if (this.root_.removeItemsOnClose) {
			this.removeAllItems();
		}
	},
	
	removeAllItems: function () {
		console.info ("enter removeAllItems");
		for (var x = this.menus_.length - 1; x >= 0; x--) {
			var menu = this.menus_[x];
			for (var y = menu.getChildren().length - 1; y >= 0; y--) {
				var menuItem = menu.getChildren()[y];
				menu.removeChild(menuItem);
//				console.log ("removing from dijit.registry: " + menuItem.id);
				dijit.registry.remove (menuItem.id);
			}
			
			// delete all menus but the root
			if (menu.id != this.id) {
//				console.log ("removing from dijit.registry: " + menu.id);
				dijit.registry.remove (menu.id);
				this.menus_.pop();
			}
		}
		
		// indicate that the menu is not built
		this.done_ = false;
		console.info ("exit removeAllItems");
	}
	
});


dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProvider", 
	         [ dijit._Widget, dijit._Templated ],
{
        iscPortletNamespace: null,
        getItemsCallback: null,

	    constructor: function() {
    	    // summary:
	        //      The base class for all providers that populate the TIPContextMenu
			this.id_ = "1_";
	        this.menus_ = [];
	    },

		initialize : function (id, callback, getSelectedData) {
			console.info ("enter initialize: " + this.id);
			this.id_ = id;
			if (this.getItemsCallback) {
				console.info ("getting items callback");
				var menuItems = this.getItemsCallback();
				console.info ("about to recur add menu items");
				this.recursivelyAddMenuItems (menuItems);
			}
			
			if (callback) {
				callback();
			}
		},

		addMenu: function(/*dijit:Menu*/ menu, /*String*/ parentMenuId) {
        	parentMenuId = (parentMenuId) ? parentMenuId : this.id_ + "ROOT";
        	var parentMenu = this.menus_[parentMenuId];
        	if (!parentMenu && parentMenuId == this.id_ + "ROOT") {
        		parentMenu = {  id: this.id_ + "ROOT", menuitems: [] };
        		this.menus_[parentMenu.id] = parentMenu;
        	}
        	else if (!parentMenu) {
        		console.debug("ibm.tivoli.tip.dijit.TIPContextMenuProvider.addMenu: Unknown parentMenuId: " + parentMenuId);
        		return;
        	}
        	
        	var menuitem = { id: "menu_" + menu.id, submenuId: menu.id, label: menu.label, iconClass: menu.iconClass, parentMenuId: parentMenuId };
        	parentMenu.menuitems.push(menuitem);
			this.menus_[menu.id] = menu;
		},
		
		getMenus: function() {
			return this.menus_;
		},
				
        addItem: function(/*dijit:MenuItem */ item, /*String*/parentMenuId) {
        	parentMenuId = (parentMenuId) ? parentMenuId : this.id_ + "ROOT";
        	var parentMenu = this.menus_[parentMenuId];
        	if (!parentMenu && parentMenuId == this.id_ + "ROOT") {
        		parentMenu = {  id: this.id_ + "ROOT", menuitems: [] };
        	}
        	else if (!parentMenu) {
        		console.debug("ibm.tivoli.tip.dijit.TIPContextMenuProvider.addItem: Unknown parentMenuId: " + parentMenuId);
        		return;
        	}
        	
        	parentMenu.menuitems.push(item);
        	this.menus_[parentMenuId] = parentMenu;
//        	this.items_.push (item);
        },

        getItems: function(/*String*/parentMenuId) {
        	var menuitems = [];
        	parentMenuId = (parentMenuId) ? parentMenuId : this.id_ + "ROOT";
        	var parentMenu = this.menus_[parentMenuId];
        	if (parentMenu) {
        		menuitems = parentMenu.menuitems;
        	}
        	return menuitems;
//        	return (this.items_);
        },
        getItem: function(/*String*/ id) {
        	for (x in this.menus_) {
        		if (x) {
	        		var menuitems = this.menus_[x].menuitems;
		        	for (var i=0; i<menuitems.length; i++) {
	    	    		if (menuitems[i].id == id) {
							return (menuitems[i]);
	        			}
	        		}
	        	}
	        }
        },
        getItemWithParent: function(/*String*/ id, /*String*/parentMenuId) {
        	var menuitems = this.getItems(parentMenuId);
        	for (var i=0; i<menuitems.length; i++) {
        		if (menuitems[i].id == id) {
					return (menuitems[i]);
        		}
        	}
        },
        removeItem: function (/*String*/ id) {
        	loop:
        	for (x in this.menus_) {
        		if (x) {
	        		var menuItems = this.menus_[x].menuitems;
		        	for (var i=0; i<menuitems.length; i++) {
	    	    		if (menuitems[i].id == id) {
		        			// remove item i
	    	    			menuitems = menuitems.slice(0, i-1).concat (menuitems.slice(i));
	    	    			break loop;
    	    			}
	        		}
        		}
        	}
        },
        removeItemWithParent: function (/*String*/ id, /*String*/parentMenuId) {
        	var menuitems = this.getItems(parentMenuId);
        	for (var i=0; i<menuitems.length; i++) {
        		if (menuitems[i].id == id) {
        			// remove item i
        			menuitems = menuitems.slice(0, i-1).concat (menuitems.slice(i));
        		}
        	}
        },
	    setEnabled: function (/*String*/ id, /*boolean*/ enabled) {
        	loop:
        	for (x in this.menus_) {
        		if (x) {
	        		var menuItems = this.menus_[x].menuitems;
			       	for (var i=0; i<menuitems.length; i++) {
	        			if (menuitems[i].id == id) {
	        				menuitems[i].enabled = enabled;
	    	    			break loop;
	        			}
	        		}
	        	}
        	}
    	},
	    setEnabled: function (/*String*/ id, /*boolean*/ enabled, /*String*/parentMenuId) {
        	var menuitems = this.getItems(parentMenuId);
        	for (var i=0; i<menuitems.length; i++) {
        		if (menuitems[i].id == id) {
        			menuitems[i].enabled = enabled;
        		}
        	}
    	},
	    isEnabled: function (/*String*/ id) {
        	for (x in this.menus_) {
        		if (x) {
	        		var menuItems = this.menus_[x].menuitems;
    	    		for (var i=0; i<menuitems.length; i++) {
	        			if (menuitems[i].id == id) {
    		    			return menuitems[i].enabled;
	    	    		}
	        		}
        		}
        	}
    	},
	    isEnabled: function (/*String*/ id, /*String*/parentMenuId) {
        	var menuitems = this.getItems(parentMenuId);
        	for (var i=0; i<menuitems.length; i++) {
        		if (menuitems[i].id == id) {
        			return menuitems[i].enabled;
        		}
        	}
    	},
        onContextMenuOpen: function () {
        },
        destroy: function() {
            // summary: Destructor
        },
		clear: function() {
			this.menus_ = [];
		},
		update: function(/* JSON */context, /* Function */callback) {
   			this.clear();
		},
   		recursivelyAddMenuItems: function( menuItems ) {
   			console.info ("enter recursivelyAddMenuItems");
   			console.info (menuItems);
		     var newMenu = null;
     
		     for ( var i = 0; i < menuItems.length; i++ )
		     {
		     	if ( menuItems[i].submenu ) {
		    	    newMenu = {    id: menuItems[i].id, 
	    	                    label: menuItems[i].label,
	    	                     iconClass: menuItems[i].iconClass, 
   	                     parentMenuId: menuItems[i].parentMenuId ? menuItems[i].parentMenuId : this.id_ + "ROOT",
    		                menuitems:[] };
     	     		    	
					this.addMenu( newMenu, (menuItems[i].parentMenuId) ? menuItems[i].parentMenuId : this.id_ + "ROOT" );
					
					this.recursivelyAddMenuItems( menuItems[i].submenu_items );
		     	} else {
		       		this.addItem ( menuItems[i], (menuItems[i].parentMenuId) ? menuItems[i].parentMenuId : this.id_ + "ROOT" );
		     	}
		     }
	   },
	   getMenuCommand: function (commandID, selectedData) {
	   		//alert ("running command: " + commandID + " with selectedData: " + selectedData);
	   		var item = this.getItem (commandID);
	   		this.handleMenuAction (item);
	   },
   handleMenuAction: function (data, ioArgs) {
   		console.info ("handleMenuAction: ");
   		console.info (data);
		if (data.objectType === "serverException") {
			this.handleServerException (data);
		}

      	if (data.launchType == "JavaScript") {
      		console.info ("calling handleJavascript");
			this.handleJavascript (data);
       	} else if (data.launchType == "WEB_URL") {
			this.launchURL (data);
      	} else if (data.launchType == "CLIENT_EVENT") {
			this.sendEvent (data);
      	} else if (data.launchType == "PORTLET") {
      		this.launchPortlet (data);
      	} else if (data.launchType == "PORTAL_PAGE") {
      		this.launchPage(data);
      	} else {
      		console.debug ("unsupported launchtype: " + data.launchType);
      	}
   },
   handleJavascript: function (data) {
   		console.info ("enter handleJavascript");
     	var fnc = new Function ("eval (\"" + data.uri + "\");");
     	fnc();
   },
   launchURL: function(data) {
       	window.open (data.uri);
   },
   launchPortlet: function (data) {
      		console.info ("enter launchPortlet");
      		var portletEvent = {
				name: "http://ibm.com/isclite#launchPortlet",
				portletName: data.portletName,
				portletApplicationID: data.portletAppUID
			};
			if (data.parameters) {
	      		for (var j=0; j<data.parameters.length; j++) {
					eval ("portletEvent." + data.parameters[j].name + "= '" + data.parameters[j].value + "'");
	      		}
      		}
//      		alert ("about to send portlet event: " + portletEvent.portletName);
      		try {
//      			console.log ("about to send event with iscnamespace: " + this.iscPortletNamespace);
				EventBroker.sendEvent (this.iscPortletNamespace, portletEvent);
			} catch (e) {
				console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
				alert (e);
			}
//     		alert ("sent event: " + portletEvent.name);
   },
   launchPage: function (data) {
      		var pageEvent = {
				name: "http://ibm.com/isclite#launchPage",
				NavigationNode: data.portletPageID,
				pageInstanceRef: null,
				switchPage: true
			};
			if (data.parameters) {
	      		for (var k=0; k<data.parameters.length; k++) {
					eval ("pageEvent." + data.parameters[k].name + "= '" + data.parameters[k].value + "'");
	      		}
      		}
//      		alert ("about to send page event: " + pageEvent.NavigationNode);
      		try {
				EventBroker.sendEvent (this.iscPortletNamespace, pageEvent);
			} catch (ex) {
				console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
				alert (ex);
			}
//      		alert ("sent event: " + pageEvent.name);
   
   },
   
   // Example
   /* {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441"

,"parameters":[{"type":2,"value":"managed_system_name=Primary:LOCO:NT","name":"srcToken"},{"type":2,"value"

:"ibm-cdm:\/\/\/CDMMSS\/Hostname=loco.raleigh.ibm.com,ManufacturerName=IBM,ProductName=IBM Tivoli Monitoring"

,"name":"mssName"}],"portletAppUID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","description"

:"Send Node Selected Event Description","event":"http:\/\/tivoli.ibm.com\/tip#nodeSelectedEvent","portletReuse"

:null,"browserWindowID":null,"launchType":"CLIENT_EVENT","id":"ibm_tivoli_tip_dijit_TIPContextMenuProxy_0com

.ibm.tivoli.itm.menu5","uri":null,"isSeparator":false,"label":"Send Node Selected Event","portletPageID"

:null,"ordinal":3,"portletName":"SendEventPortlet","enabled":"true","icon":""} */
   sendEvent: function (data) {
      		var parms = data.parameters;
	        var event = {
                name: data.event
    	    };
			if (data.parameters) {
	      		for (var i=0; i<parms.length; i++) {
					eval ("event." + parms[i].name + "= '" + parms[i].value + "'");
	      		}
	      	}
      		
//      		alert ("about to send event: " + event.name);
			EventBroker.sendEvent (this.iscPortletNamespace, event);
//      		alert ("sent event: " + event.name);
   },
   destroy: function () {
   	console.log ("enter TIPContextMenuProvider.destroy");
   	this.inherited (arguments);
   	console.log ("exit TIPContextMenuProvider.destroy");
   }
	   
});

dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProvider", ibm.tivoli.tip.dijit.TIPContextMenuProvider, 
{
	getMenuItemsCallback_ : null,
	global: null,
	rcfItems: null, 
	menuName: null, 
	parentid: null, 

	constructor: function () {
		this.id_ = "1_";
	    this.messages_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "messages");
	},
	
	setParentId: function (parentid) {
		this.parentid = parentid;
	},
	
	setResourceContextFilters: function (rcfItems) {
		this.rcfItems = rcfItems;
	},
	
	getRCFURL: function (kw) {
		if (this.rcfItems) {
			for (var i in this.rcfItems) {
				if (i) {
					for (var name in this.rcfItems[i]) {
						if (name) {
					        kw.content["rcf_" + name]=this.rcfItems[i][name];
						}
					}
				}
			}
		}	
	},

	initialize: function(id, getMenuItemsCallback, getSelectedData) {
	  	var kw = null;
		this.id_ = id;
		this.getMenuItemsCallback_ = getMenuItemsCallback;
		
  	    // We use this context if we do have a Global
        kw = {
                content:   {
                              command: "getMenuItems",
                              arg_idPrefix: this.id_,
                              arg_portletNamespace: this.global.namespace,
                              arg_parentId: this.parentid
                           },
                onResponse:{
                              callback: dojo.hitch(this, this.onGetMenuItemsCallback)
                           }
             };

   		this.dataToURL (getSelectedData(),kw);
		
		var url = null;
	    
        this.getRCFURL(kw);
			
		if (this.menuName) {
	        kw.content.arg_menuName = this.menuName;
		}
		
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
	},
	
	handleError: function (response, ioArgs) {
		console.error ("HTTP status code: ", ioArgs.xhr.status);
		return response;
	},
	
	handleTimeout: function (response, ioArgs) {
		console.error ("Timeout for request: " + ioArgs.xhr);
		return response;
	},
	handleServerException: function (exception) {
		console.error ("Received server exception: " + exception.messageID + ": " + exception.messageString);
		console.error ("with server stack trace: ");
		console.error (exception.stackTrace);
		this.showMessageDialog (exception.type, exception.messageID, exception.messageString);
	},
	showMessageDialog: function (type, msgID, msg) {
		if (msg === null) {
			msg = this.messages_.TIPUI1001E;
			msgID = "TIPUI1001E";
		}
	    var modalWindow = new ibm.tivoli.tip.dijit.TIPMessageDialog (
		                                            { 
		                                              type: type,
	    	                                          message: msg,
	        	                                      messageId: msgID,
	        	                                      showCheckbox: false
	                    	                        }); 
		modalWindow.show();
	},
    onGetMenuItemsCallback: function(data, ioArgs) {
		if (data.objectType === "serverException") {
			this.handleServerException(data);
		} else {
	  	    this.recursivelyAddMenuItems( data );		
			if ( this.getMenuItemsCallback_ !== null) {
				 this.getMenuItemsCallback_();
			}
		}
   },
   


//    onGetMenuItemsCallback: function(type,data,evt) {
//        var items = data;
//        for (var i=0; i<items.length; i++) {
//      		this.addItem (items[i]);
//        }
//
//		if (this.getMenuItemsCallback_ !== null) {
//			this.getMenuItemsCallback_();
//		}
//   },

   onContextMenuOpen: function() {
//    	onContextMenuOpenCallback();
   },   
   destroy: function() {
            // summary: Destructor
   },
   getMenuCommand: function (commandName, selectedData) {
        var kw = {
                    content:   {
                                  command: "getMenuCommand",
                                  arg_menuCommandName: commandName,
                                  arg_idPrefix: this.id_,
                                  arg_portletNamespace: this.global.namespace,
                                  arg_parentId: this.parentid
                               },
                    onResponse:{
                                  callback: dojo.hitch(this, this.handleMenuAction)
                               }
                 };
                 
   		this.dataToURL (selectedData,kw);
   		this.getRCFURL(kw);
   		
   		if (this.menuName) {
   		    kw.content.arg_menuName = this.menuName;
		}
		
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
   },
   
   getIdPrefix: function() {
       return (this.id_);
   },
   
   dataToURL: function (selectedData, kw) {
   		var columns = selectedData.columns;
   		var rows = selectedData.rows;
   		var row = null;
   		if (this.isArray (rows)) {
	   		row = rows[0];
	   	} else {
	   		row = rows;
	   	}
   		
   		for (var i=0; i<columns.length; i++) {
   			var rowField = columns[i].field;
//   			var value = eval ("row." + rowField);
			var value = row[columns[i].field];
			if (!value) {
				value = "";
			}
			if (columns[i].isNamingAttribute === "true") {
			    kw.content["rn_"+columns[i].field]=value;
			} else {
	            kw.content["sv_"+columns[i].field]=value;
	   		}
   		}
   		
   		if (row.id) {
	        kw.content.arg_rowId = row.id;
   		}
   },
   isArray: function (obj) {
	   if (obj.constructor.toString().indexOf("Array") == -1) {
	      return false;
	   } else {
	      return true;   	
	   }
   },
   getMenuItemsSample: function() {
  	// Example menu items JSON
	//   [{"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","submenu_items" :[{"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"WEB_URL","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu1","uri":null,"isSeparator":"false","label":"Launch TEP (sourceToken)","portletPageID":null,"ordinal":1,"parentMenuId":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.launchMenu","enabled":"true","iconClass":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"WEB_URL","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu2","uri":null,"isSeparator":"false","label":"Launch TEP (DotNotation)","portletPageID":null,"ordinal":2,"parentMenuId":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.launchMenu","enabled":"true","icon":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"WEB_URL","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu3","uri":null,"isSeparator":"false","label":"Launch TEP (Server_Name)","portletPageID":null,"ordinal":3,"parentMenuId":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.launchMenu","enabled":"true","icon":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"WEB_URL","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu4","uri":null,"isSeparator":"false","label":"Launch TEP (ManagedSystemName)","portletPageID":null,"ordinal":3,"parentMenuId":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.launchMenu","enabled":"true","icon":""}],
	//    "portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.launchMenu","uri":null,"submenu":"true","isSeparator":"false","label":"com.ibm.tivoli.itm.launchMenuDisplay","portletPageID":null,"ordinal":0,"enabled":"true","icon":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"CLIENT_EVENT","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu5","uri":null,"isSeparator":"false","label":"Send Node Selected Event","portletPageID":null,"ordinal":3,"enabled":"true","icon":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"PORTAL_PAGE","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu7","uri":null,"isSeparator":"false","label":"Launch Sample Page","portletPageID":null,"ordinal":3,"enabled":"true","icon":""},
	//    {"applicationURLAsRegistered":null,"appID":"com.ibm.sendeventportlet.SendEventPortlet.4318727441","portletAppUID":null,"description":"","event":null,"portletReuse":null,"browserWindowID":null,"launchType":"PORTLET","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0com.ibm.tivoli.itm.menu6","uri":null,"isSeparator":"false","label":"Launch Sample Portlet","portletPageID":null,"ordinal":6,"enabled":"true","icon":""},{"id":"Separator"},{"label":"Print Preview","icon":"/ibm/TIPChartPortlet/images/toggle_window_16.gif","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0PrintPreviewTable"},
	//    {"label":"Save to File","icon":"/ibm/TIPChartPortlet/images/toggle_window_16.gif","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0SaveTableAsFile"},{"label":"Show/Hide Toolbar","icon":"/ibm/TIPChartPortlet/images/toggle_window_16.gif","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0ToggleToolbar"},{"id":"Separator"},{"label":"Refresh","icon":"/ibm/TIPChartPortlet/images/refresh_enabled.gif","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0Refresh"},{"label":"Properties","icon":"/ibm/TIPChartPortlet/images/preferences.gif","id":"ibm.tivoli.tip.dojo_TIPContextMenu_0Properties"}]
   
		var menuItems = [
			{ id: "Properties", label: "Properties", iconClass: "", launchType: "External", uri: "http://www.google.com", enabled: true },
			{ id: "Refresh", label: "Refresh", iconClass: "", launchType: "JavaScript", uri: "alert ('clicked refresh');", enabled: false }
		];
		
		return (menuItems);
   }
});


dojo.declare("ibm.tivoli.tip.dijit.TIPPopupManager", dijit.PopupManager, 
{
	constructor: function () {
		this.registerWin = function(/*Window*/win) {
		// summary: 
		//		Override the parent to only register a window so that when clicks in it, the popup 
		//		can be closed automatically.  The parent closed when the user clicked on the scroll,
		//		which is problemmatic if the context menu is displayed outside of the scroll area.
			if(!win.__PopupManagerRegistered) {
				dojo.connect(win.document, 'onmousedown', this, 'onClick');
	//			dojo.connect(win, "onscroll", this, "onClick");
				dojo.connect(win.document, "onkey", this, 'onKey');
				win.__PopupManagerRegistered = true;
				this.registeredWindows.push(win);
			}
		};

		this.unRegisterWin = function(/*Window*/win){
			// summary: 
			//		remove listeners on the registered window
			if(win.__PopupManagerRegistered) {
				dojo.disconnect(win.document, 'onmousedown', this, 'onClick');
	//			dojo.disconnect(win, "onscroll", this, "onClick");
				dojo.disconnect(win.document, "onkey", this, 'onKey');
				win.__PopupManagerRegistered = false;
			}
		};
	}
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenuV2"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPContextMenuV2"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/







dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuV2");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProviderV2");
dojo.provide("ibm.tivoli.tip.dijit.CMSMenuProviderV2");
dojo.provide("ibm.tivoli.tip.dijit.TIPContextMenuProxyV2");


/**
 *  Provides dynamic context menu support.
 *  How do you use this?
 *  First, you create an instance of this context menu similar to:
 *     this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenuV2({cmProvider: this.cmProvider});
 *  Note: you pass in an instance of a context menu provider (see objects lower in this file)
 *
 *  Second, you latch whatever event you want to create the context menu.  For 
 *  Grids, you might do something like this:
 *     this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
 *  Don't forget to disconnect this listener when your object is destroyed.
 *
 *  Third, you create the method that gets called by the above connect, and call
 *  the contextMenuOpen on this context menu with whatever context you want 
 *  processed.  Again, in a Grid example:
 *   _onRowContextMenu: function( e )
 *   { 
 *     var row = e.grid.model.getRow( e.rowIndex );
 *     
 *     if ( row )
 *     {
 *       //select the row that the context menu was initiated on
 *       e.grid.selection.unselectAll( e.rowIndex );
 *       e.grid.selection.setSelected( e.rowIndex, true );
 *               
 *       //tell the context menu adapter to show the menu
 *       // no context is provided because we overload the _loadMenuItems method
 *       this._contextMenuAdapter.contextMenuOpen( e, {selectedData: this.getSelectionData()} );
 *     }
 *   },
 *
 *  
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuV2",
              ibm.tivoli.tip.dijit.ContextMenuAdapter,
{
   cmProvider: null,       // TIPContextMenuProvider
   url: "no_url_needed",   // Note we have to pass something to ContextMenuAdapter
                           // for the url so that we can override _loadMenuItems
   
   /****************************************************************************
   * Callback that should be overloaded to be told when a menu item is clicked
   * and an action should occur
   ****************************************************************************/
   onMenuItemClick: function( menuItem )
   { 
      // console.debug("-------------- menuItem :",menuItem);
      this.cmProvider.getMenuCommand(menuItem.actionID, this._context, menuItem);
   },

   /****************************************************************************
   * Synchronously Call a JSON Url to get the menu items
   *
   ****************************************************************************/
   _loadMenuItems: function()
   {
     return this.cmProvider.getMenuItems( this._context );
   }

});

/**
 *  What is this?
 *  This is a helper for the TIPContextMenu.  It allows you pass in DOM nodes 
 *  and it will set up the context menu action for those nodes.  You must call 
 *  destroy on this object when you are done with it.  You provide the following
 *  when you create it:
 *   - targetNodeIds (required): these are string IDs for the DOM nodes you want 
 *                               context menus on
 *   - getContextCallback (optional): this is a function that will return any 
 *                                    context that you want passed along on the 
 *                                    context menu open call.  It will get 
 *                                    passed to your provider.
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProxyV2",
              ibm.tivoli.tip.dijit.TIPContextMenuV2,
{

   targetNodeIds: null,

   getContextCallback: null,
   
   _bindings: null,
   
   constructor: function() 
   {
      this._bindings = [];
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.bindDomNode));
   },

   // Called when it is time to notify the user that something was clicked.
   _openContextMenu: function(event)
   {
      var result = null;
      if (this.getContextCallback !== null) 
      {
         result = this.getContextCallback(event);
      }
      this.contextMenuOpen( event, result );
   },
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	bindDomNode: function(/*String|DomNode*/ node){
		// summary: attach menu to given node
		node = dojo.byId(node);

		//TODO: this is to support context popups in Editor.  Maybe this shouldn't be in dijit.Menu
		var win = dijit.getDocumentWindow(node.ownerDocument);
		if(node.tagName.toLowerCase()=="iframe"){
			win = this._iframeContentWindow(node);
			node = dojo.withGlobal(win, dojo.body);
		}

		// to capture these events at the top level,
		// attach to document, not body
		var cn = (node == dojo.body() ? dojo.doc : node);

		node[this.id] = this._bindings.push([
         // <kjp> Changed this next line to point to my _openContextMenu.
         // The line previously pointed to _openMyself
			dojo.connect(cn, "oncontextmenu", this, "_openContextMenu"),
			dojo.connect(cn, "onkeydown", this, "_contextKey")
         // <kjp> I removed the next line (and removed the comma from the previous)
			//dojo.connect(cn, "onmousedown", this, "_contextMouse")
		]);
	},

   // This is a complete copy from dijit.Menu
	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary: detach menu from given node
		var node = dojo.byId(nodeName);
		var bid = node[this.id]-1, b = this._bindings[bid];
		dojo.forEach(b, dojo.disconnect);
		delete this._bindings[bid];
	},
   
   // This method mostly copied from dijit.Menu to provide the binding to the 
   // DOM node.  I needed to copy it so that I could properly inject the 
   // contextMenuAdapter.  I'll note changes with <kjp> tags
	_contextKey: function(e){
		this._contextMenuWithMouse = false;
		if (e.keyCode == dojo.keys.F10) {
			dojo.stopEvent(e);
			if (e.shiftKey && e.type=="keydown") {
				// FF: copying the wrong property from e will cause the system
				// context menu to appear in spite of stopEvent. Don't know
				// exactly which properties cause this effect.
				var _e = { target: e.target, pageX: e.pageX, pageY: e.pageY };
				_e.preventDefault = _e.stopPropagation = function(){};
				// IE: without the delay, focus work in "open" causes the system
				// context menu to appear in spite of stopEvent.
            //<kjp> I changed the following line from calling this_openMyself to this._openContextMenu
				window.setTimeout(dojo.hitch(this, function(){ this._openContextMenu(_e); }), 1);
			}
		}
	},

   destroy: function()
   {
   	console.log ("enter TIPContextMenuV2.destroy");
      dojo.forEach(this.targetNodeIds, dojo.hitch(this, this.unBindDomNode));
      if (this.cmProvider !== null)
      {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }
   }
});


/**
 *  You provide a context menu provider to do things like populate the context menu with
 *  items, and to handle the actions when one of those items are clicked.
 *  You provide:
 *  - getMenuItemsCallback (required)   (optional alternative: override getMenuItems)
 *      Parameters: (context)
 *                 context (that was passed on the contextMenuOpen call (in your code)
 *
 *      Summary: This callback provides the menu items to be inserted into the context menu.          
 *      The following defines the callback that is used to return menu items to appear
 *      in the list.  This method should return JSON data in the form:
 *       [ 
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon", 
 *            "actionID": "testActionID" 
 *       },
 *       { 
 *            "actionID": "_separator_" 
 *       },
 *       { 
 *            "label": "submenu", 
 *            "iconClass": "tipMenuIcon", 
 *            "submenu_items": 
 *            [ 
 *             { 
 *                  "label": "Delete", 
 *                 "iconClass": "tipMenuIcon tipDeleteIcon", 
 *                 "actionID": "testDeleteActionID" 
 *             }
 *            ] 
 *       } 
 *       ]
 *
 *  - getMenuCommandCallback (optional)  (optional alternative: override getMenuCommand)
 *      Parameters: (commandName, context, menuItem)
 *                 commandName - the command name of the context menu item.  Provided on the
 *                               'actionID' item above. 
 *                 context - (that was passed on the contextMenuOpen call (in your code)
 *                 menuItem - the actual menu item object.
 *      This callback provides the function needed to process a menu item click.
 *      The response depends largely on the action you wish to take.  A sample response:
 *
 *      {
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *      }
 *      Launch types include: JavaScript, WEB_URL, CLIENT_EVENT, PORTLET, PORTAL_PAGE, NOOP
 *
 *      The actions listed above may be included in the original information returned by 
 *      the getMenuItemsCallback.  If these actions are included, you need not specify
 *      a getMenuCommandCallback, and the provider will process these actions for you
 *      automatically.  For example:
 *       { 
 *            "label": "testAction",
 *            "iconClass": "tipMenuIcon tipPropertiesIcon",
 *            "actionID": "testActionID", 
 *            "launchType":"JavaScript",
 *            "uri":"alert('Hello World');"
 *       },
 */
dojo.declare("ibm.tivoli.tip.dijit.TIPContextMenuProviderV2", null,
{
   iscPortletNamespace: null,

   getMenuItemsCallback: null,
   
   getMenuCommandCallback: null,

   menus_: null, // List of menus saved from getMenuItems

   constructor: function( args ) 
   {
      // summary:
      //      The base class for all providers that populate the TIPContextMenu
      dojo.mixin( this, args );

      this.id_ = "1_";
   },

   getMenuItems: function(context)
   {
      this.menus_ = this.getMenuItemsCallback(context);
      return this.menus_;
   },

   getItem: function(/*String*/ id) 
   {
      var submenus = [];
      var currentMenu = null;
      var x = null;
      submenus.push(this.menus_);
      
      currentMenu = submenus.pop();
      while (currentMenu)
      {
         x = currentMenu.pop();
         while (x)
         {
            if (x) 
            {
               if (x.submenu_items)
               {
                  submenus.push(x.submenu_items);
               }
               else
               {
                  if (x.id == id || x.actionID == id)
                  {
                     return x;
                  }
               }
            }
            x = currentMenu.pop();
         }
         currentMenu = submenus.pop();
      }
   },
   
   isArray: function (obj) 
   {
      if (obj.constructor.toString().indexOf("Array") == -1) 
      {
         return false;
      } else {
         return true;      
      }
   },

   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
      }
      else
      {
         result = this.getItem(commandName);
      }

      // console.debug("------------ handleMenuAction result",result);
      this.handleMenuAction(result);
   },

   handleMenuAction: function (data, ioArgs) 
   {
      // console.info ("handleMenuAction: ");
      // console.info (data);

      if (data.launchType == "JavaScript") {
         this.handleJavascript (data);

      } else if (data.launchType == "WEB_URL") {
         this.launchURL (data);

      } else if (data.launchType == "CLIENT_EVENT") {
         this.sendEvent (data);

      } else if (data.launchType == "PORTLET") {
         this.launchPortlet (data);

      } else if (data.launchType == "PORTAL_PAGE") {
         this.launchPage(data);

      } else if (data.launchType == "NOOP") {
         // Do nothing in the case of a no operation type

      } else {
         console.error ("unsupported launchtype: " + data.launchType);
      }
   },

   handleJavascript: function (data) 
   {
      // console.log ("enter handleJavascript");
      var fnc = new Function ("eval (\"" + data.uri + "\");");
      fnc();
   },

   launchURL: function(data) 
   {
      // console.log ("enter launchURL");
      window.open (data.uri);
   },

   launchPortlet: function (data) 
   {
      // console.log ("enter launchPortlet");
      var portletEvent = {
                            name: "http://ibm.com/isclite#launchPortlet",
                            portletName: data.portletName,
                            portletApplicationID: data.portletAppUID
                         };

      if (data.parameters) 
      {
         for (var j=0; j<data.parameters.length; j++) 
         {
            eval ("portletEvent." + data.parameters[j].name + "= '" + data.parameters[j].value + "'");
         }
      }

      try 
      {
         EventBroker.sendEvent (this.iscPortletNamespace, portletEvent);
      } catch (e) {
         console.error ("Caught javascript exception in TIPContextMenuProvider.launchPortlet: " + e);
         alert (e);
      }
   },
   
   launchPage: function (data) 
   {
      var pageEvent = {
                         name: "http://ibm.com/isclite#launchPage",
                         NavigationNode: data.portletPageID,
                         pageInstanceRef: null,
                         switchPage: true
                      };

      if (data.parameters) 
      {
         for (var k=0; k<data.parameters.length; k++) 
         {
            eval ("pageEvent." + data.parameters[k].name + "= '" + data.parameters[k].value + "'");
         }
      }

      try 
      {
         EventBroker.sendEvent (this.iscPortletNamespace, pageEvent);
      } catch (ex) {
         console.error ("Caught javascript exception in TIPContextMenuProvider.launchPage: " + ex);
         alert (ex);
      }
   },
   
   sendEvent: function (data) 
   {
      var parms = data.parameters;
      var event = {
                     name: data.event
                  };

      if (data.parameters) 
      {
         for (var i=0; i<parms.length; i++) 
         {
            eval ("event." + parms[i].name + "= '" + parms[i].value + "'");
         }
      }

	  console.log ("TIPContextMenuV2: sending event: ");            
	  console.log (event);
      EventBroker.sendEvent (this.iscPortletNamespace, event);
   },

   destroy: function()
   {
      // Placeholder: for overrides.  Exploiters should call this as best practice. 
   }
      
});

/**
 *  The CMSMenuProviderV2 is a specialized menu provider for use with the 
 *  context menu service.  Certainly you may use this provider if not using CMS,
 *  but it returns extra data that is used for CMS.  The CMS provider will
 *  make requests to the server to obtain both the menu items as well as the
 *  menu commands.
 *
 *  The CMSMenuProviderV2 expects the following to be included in the context 
 *  passed to contextMenuOpen
 *  {
 *     selectedData: <the selection data>
 *  }
 */
dojo.declare("ibm.tivoli.tip.dijit.CMSMenuProviderV2", 
             ibm.tivoli.tip.dijit.TIPContextMenuProviderV2, 
{
   global: null,
   rcfItems: null, 
   menuName: null, 
   parentid: null,
   
   // When the following is true, then this provider will first look 
   // at the menu items for a launchType, and if one exists, it will
   // execute that action here instead of going back to the host.
   hasLocalActions: true, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,

   getIdPrefix: function() 
   {
       return this.id_;
   },
   
   setParentId: function (parentid) 
   {
      this.parentid = parentid;
   },
   
   setResourceContextFilters: function (rcfItems) 
   {
      this.rcfItems = rcfItems;
   },
   
   getRCFURL: function (kw) 
   {
      if (this.rcfItems) 
      {
         for (var i in this.rcfItems) 
         {
            if (i) 
            {
               for (var name in this.rcfItems[i]) 
               {
                  if (name) 
                  {
                       kw.content["rcf_" + name]=this.rcfItems[i][name];
                  }
               }
            }
         }
      }   
   },

   getMenuItems: function(context) 
   {
      var kw = null;

      kw = {
              sync:         true,
              noBusyDialog: true,
              content:      {
                               command:              "getMenuItems",
                               arg_idPrefix:         this.getIdPrefix(),
                               arg_portletNamespace: this.global.namespace,
                               arg_parentId:         this.parentid
                            },
              timeout:      this.requestTimeout,
              onResponse:   {
                               callback: dojo.hitch( this, function( response ) { this.menus_ = response; } )
                            }
            };

      this.dataToURL (context.selectedData,kw);
      
      this.getRCFURL(kw);
         
      if (this.menuName) 
      {
         kw.content.arg_menuName = this.menuName;
      }
      
      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );

      return this.menus_;
   },
   
   getMenuCommand: function (commandName, context, menuItem) 
   {
      var result = null;
      
      if (this.getMenuCommandCallback !== null) 
      {
         result = this.getMenuCommandCallback(commandName, context, menuItem);
         this.handleMenuAction(result);
      }
      else
      {
         this.getMenuCommandNoCallback(commandName, context, menuItem);
      }
   },

   getMenuCommandNoCallback: function (commandName, context, menuItem) 
   {
      var actionHandled = false;
      if (this.hasLocalActions === true)
      {
         // Check if the menu item contains a launchType.  If so,
         // handle it locally.
	 var result = this.getItem(commandName);
         if (result && result.launchType)
         {
             actionHandled = true;
             this.handleMenuAction(result);
         }
      }
      
      if (actionHandled === false)
      {
         this.getMenuCommandFromServer(commandName, context, menuItem);
      }
   },
   
   getMenuCommandFromServer: function (commandName, context, menuItem) 
   {
      var kw = {
                  content:   {
                                command: "getMenuCommand",
                                arg_menuCommandName: commandName,
                                arg_idPrefix: this.getIdPrefix(),
                                arg_portletNamespace: this.global.namespace,
                                arg_parentId: this.parentid
                             },
                  timeout:   this.requestTimeout,
                  onResponse:{
                                callback: dojo.hitch(this, this.handleMenuAction)
                             }
               };
               
       this.dataToURL (context.selectedData,kw);
       this.getRCFURL(kw);
       
       if (this.menuName) 
       {
          kw.content.arg_menuName = this.menuName;
       }
      
       ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
   },   
   
   dataToURL: function (selectedData, kw) 
   {
      // if you get an exception here, it means you did not provide
      // selection data in your context.  I'll put a catch of the 
      // exception here because I bet this happens a lot.
      var columns = null; 
      try 
      {
         columns = selectedData.columns;
      } catch (ex) {
         console.error ("Caught javascript exception in cmsContextMenuProvider.dataToURL: " + ex);
         console.error ("You must provide 'selectedData' on your context.");
         alert (ex);
      }
      var rows = selectedData.rows;
      var row = null;
      if (this.isArray (rows)) {
         row = rows[0];
      } else {
         row = rows;
      }
      
      for (var i=0; i<columns.length; i++) 
      {
         var rowField = columns[i].field;
         var value = row[columns[i].field];

         if (!value) 
         {
            value = "";
         }

         if (columns[i].isNamingAttribute === "true") 
         {
            kw.content["rn_"+columns[i].field]=value;
         } else {
            kw.content["sv_"+columns[i].field]=value;
         }
      }
         
      if (row.id) 
      {
        kw.content.arg_rowId = row.id;
      }
   }
   
});


}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageArea"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPMessageArea"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPMessageArea");
dojo.provide("ibm.tivoli.tip.dijit.TIPMessageAreaManager");
/******************************************************************************
* Collapsable Message Area (Info, Warning, Error)
*
* The purpose of this widget is to be placed into dialogs and forms to aid in
* providing feedback on form errors.
*
* Author: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPMessageArea", [dijit._Widget, dijit._Templated],
{
	//the message box type: critical, info, warning
	type: "critical",
	
	//the message in the are -- can be HTML string
	message: "",
	
	//if you provide a message id then it will use bigger message images
	messageId: null,

	//the duration of time to play the wipe fx on show and close	
	duration: 200,
	
	// url: String: a URL that links the message to an InfoCenter page	
	url: null,
	
	// boolean: create the message area already open?
	isShowing: false,
	
	//the HTML template for the label placement and null value checkbox
	templateString:"<div>\r\n\t<div dojoAttachPoint=\"wipeNode\">\r\n\t<table width=\"100%\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" dojoAttachPoint=\"styleArea\">\r\n\t<tr>\r\n\t\t<td width=\"17\" valign=\"top\">\r\n\t\t\t<img dojoAttachPoint=\"messageIcon\" src=\"${criticalIcon}\" alt=\"${resources.DIALOG_ERROR}\" border=\"0\" style=\"padding-right:7px\" />\r\n\t\t</td>\r\n\t\t<td>\r\n\t\t\t<div dojoAttachPoint=\"messageNode\" class=\"message-description\"></div>\r\n\t\t</td>\r\n\t\t<td width=\"18\">\r\n\t\t\t<img dojoAttachPoint=\"closeBtn\" class=\"messageAreaCloseIcon\" src=\"${closeIcon}\" alt=\"${resources.DIALOG_BUTTON_CLOSE}\" border=\"0\" dojoAttachEvent=\"onclick: _onCloseClick\" />\r\n\t\t</td>\r\n\t</tr>\r\n\t</table>\r\n\t</div>\r\n</div>\r\n",
	   closeIcon: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/message_close.gif"),
	criticalIcon: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_critical_10.png"),	  
	
	constructor: function () 
	{
		this.resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
	}, 			
	/**************************************************************************
	* Setup the widget
	**************************************************************************/	
	postCreate: function()
	{
		//call super
		this.inherited( "postCreate", arguments );

		//initialize message
		this.setMessageId( this.messageId );
		
		//initialize message
		this.setMessage( this.message );

		//initialize type
		this.setType( this.type );		
		
		//create a WipeIn Effect Object
		this._wipeIn = dojo.fx.wipeIn(
		{
			node: this.wipeNode,
			duration: this.duration
		});
		
		//create a WipeOut Effect Object
		this._wipeOut = dojo.fx.wipeOut(
		{
			node: this.wipeNode,
			duration: this.duration			
		});
		
		if ( !this.isShowing ) 
		{
			this.wipeNode.style.display="none";
		}
	},

	/**************************************************************************
	* Set the type of the message box (critical, warning, info)
	**************************************************************************/		
	setType: function( type )
	{
		this.type = type;
		
		this._updateClass();
	},
	
	/**************************************************************************
	* Set the message ( can be HTML string )
	**************************************************************************/		
	setMessage: function( message )
	{
		this.messageNode.innerHTML = message;
	},
	
	/**************************************************************************
	* Set the messageId ( can be HTML string )
	**************************************************************************/		
	setMessageId: function( messageId )
	{
	   this.messageId = messageId;
	   
	   var suffix = "_30.png";
	   
	   if ( messageId === undefined || messageId === null || messageId == "" )
	   {
	     suffix = "_10.png";
	     
	     if( this.messageIdNode )
	     {
	     	this.messageIdNode.parentNode.removeChild( this.messageIdNode );
	     	delete this.messageIdNode;
	     }
	   }
	   else
	   {
	     if( !this.messageIdNode )
	     {
		   	 this.messageIdNode = dojo.doc.createElement( 'DIV' );
		   	 this.messageIdNode.className = "message-id";
		     dojo.place( this.messageIdNode, this.messageNode, "before" );
		 }
		   	 
	   	 var innerHTML = "";
	   	 
	   	 //if infocenter URL set then create hyperlink
	   	 if ( this.url ) 
	   	 {
	   	   innerHTML += '<a href="javascript:void(0)" onclick="window.open(\'' + this.url + '\')">';
		   innerHTML += this.messageId;
		   innerHTML += "</a>";
		 }
		 else 
		 {
		   innerHTML += this.messageId;
	     }
	   	 
	     this.messageIdNode.innerHTML = innerHTML;		     
	   }
	   
	   this.criticalIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_critical" + suffix);
           this.infoIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_info"  + suffix);
	   this. warningIcon = dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/i_message_warning"  + suffix);	    
	   
	   this._updateClass();	    
	},
	
	/**************************************************************************
	* Show a info message
	* messageID is optional
	**************************************************************************/		
	showInfoMessage: function( message, messageID )
	{
		this.setMessage( message );
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		this.setType( "info" );
		
		this.show();
	},

	/**************************************************************************
	* Show a warning message
	* messageID is optional
	**************************************************************************/		
	showWarningMessage: function( message, messageID )
	{
		this.setMessage( message );
		
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		
		this.setType( "warning" );
		
		this.show();
	},

	/**************************************************************************
	* Show an error message
	* messageID is optional
	**************************************************************************/		
	showErrorMessage: function( message, messageID )
	{
		this.setMessage( message );
		
		if (messageID)
		{
			this.setMessageId( messageID );
		}
		else
		{
			this.setMessageId( null );
		}
		
		this.setType( "critical" );
		
		this.show();
	},
	
	/**************************************************************************
	* Show the message area -- ensures message area not already visible for you
	**************************************************************************/		
	show: function()
	{
		if ( !this.isShowing )
		{
			this.isShowing = true;
			this._wipeIn.play();
		}
	},
	
	/**************************************************************************
	* Hide the message area -- ensures message area not already hidden for you
	**************************************************************************/		
	hide: function()
	{
		if ( this.isShowing )
		{
			this.isShowing = false;
			this._wipeOut.play();
		}
	},
	
	/**************************************************************************
	* updates the CSS styling and icon based on current type setting
	**************************************************************************/		
	_updateClass: function()
	{
		var type = this.type.toLowerCase();
		
		dojo.removeClass( this.styleArea, "critical" );
		dojo.removeClass( this.styleArea, "info" );
		dojo.removeClass( this.styleArea, "warning" );
		
		if( type == "critical" )
		{
			this.messageIcon.src = this.criticalIcon;
			this.messageIcon.alt = this.resources.DIALOG_ERROR;
		}
		else if ( type == "info" )
		{
			this.messageIcon.src = this.infoIcon;
			this.messageIcon.alt = this.resources.DIALOG_INFO;
		}
		else
		{
			this.messageIcon.src = this.warningIcon;
			this.messageIcon.alt = this.resources.DIALOG_WARNING;
		}
		
		dojo.addClass( this.styleArea, type );
		
		//overload the stylesheet (because it was written for a dialog)
		this.styleArea.style.padding="2px";
	},
		
	/**************************************************************************
	* Called when the close button is clicked
	**************************************************************************/				
	_onCloseClick: function()
	{
		this.hide();
	}
});

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormWidget,
	{
		// summary:
		//		A generic textbox field.
		//		Serves as a base class to derive more specialized functionality in subclasses.

		//	trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		//	uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		//	lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		//	propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		// maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		templateString:"<input class=\"dojoTextBox\" dojoAttachPoint='textbox,focusNode' name=\"${name}\"\r\n\tdojoAttachEvent='onmouseenter:_onMouse,onmouseleave:_onMouse,onfocus:_onMouse,onblur:_onMouse,onkeyup,onkeypress:_onKeyPress'\r\n\tautocomplete=\"off\" type=\"${type}\"\r\n\t/>\r\n",
		baseClass: "dijitTextBox",

		attributeMap: dojo.mixin(dojo.clone(dijit.form._FormWidget.prototype.attributeMap),
			{maxLength:"focusNode"}),

		getDisplayedValue: function(){
			return this.filter(this.textbox.value);
		},

		getValue: function(){
			return this.parse(this.getDisplayedValue(), this.constraints);
		},

		setValue: function(value, /*Boolean, optional*/ priorityChange, /*String, optional*/ formattedValue){
			var filteredValue = this.filter(value);
			if((typeof filteredValue == typeof value) && (formattedValue == null || formattedValue == undefined)){
				formattedValue = this.format(filteredValue, this.constraints);
			}
			if(formattedValue != null && formattedValue != undefined){
				this.textbox.value = formattedValue;
			}
			dijit.form.TextBox.superclass.setValue.call(this, filteredValue, priorityChange);
		},

		setDisplayedValue: function(/*String*/value){
			this.textbox.value = value;
			this.setValue(this.getValue(), true);
		},

		forWaiValuenow: function(){
			return this.getDisplayedValue();
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a value to a properly formatted string
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary: Replacable function to convert a formatted string to a value
			return value;
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			this.textbox.setAttribute("value", this.getDisplayedValue());
			this.inherited('postCreate', arguments);

			if(this.srcNodeRef){
				dojo.style(this.textbox, "cssText", this.style);
				this.textbox.className += " " + this["class"];
			}
			this._layoutHack();
		},

		_layoutHack: function(){
			// summary: work around table sizing bugs on FF2 by forcing redraw
			if(dojo.isFF == 2 && this.domNode.tagName=="TABLE"){
				var node=this.domNode;
				var old = node.style.opacity;
				node.style.opacity = "0.999";
				setTimeout(function(){
					node.style.opacity = old;
				}, 0);
			}			
		},

		filter: function(val){
			// summary: Apply various filters to textbox value
			if(val == undefined || val == null){ return ""; }
			else if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		// event handlers, you can over-ride these in your own subclasses
		_onBlur: function(){
			this.setValue(this.getValue(), (this.isValid ? this.isValid() : true));
		},

		onkeyup: function(){
			// TODO: it would be nice to massage the value (ie: automatic uppercase, etc) as the user types
			// but this messes up the cursor position if you are typing into the middle of a word, and
			// also trimming doesn't work correctly (it prevents spaces between words too!)
			// this.setValue(this.getValue());
		}
	}
);

}

if(!dojo._hasResource["dojox.grid._grid.lib"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.lib"] = true;
dojo.provide("dojox.grid._grid.lib");

dojo.isNumber = function(v){
	return (typeof v == 'number') || (v instanceof Number);
}

// summary:
//	grid utility library

dojo.mixin(dojox.grid, {
	na: '...',
	nop: function() {
	},
	getTdIndex: function(td){
		return td.cellIndex >=0 ? td.cellIndex : dojo.indexOf(td.parentNode.cells, td);
	},
	getTrIndex: function(tr){
		return tr.rowIndex >=0 ? tr.rowIndex : dojo.indexOf(tr.parentNode.childNodes, tr);
	},
	getTr: function(rowOwner, index){
		return rowOwner && ((rowOwner.rows||0)[index] || rowOwner.childNodes[index]);
	},
	getTd: function(rowOwner, rowIndex, cellIndex){
		return (dojox.grid.getTr(inTable, rowIndex)||0)[cellIndex];
	},
	findTable: function(node){
		for (var n=node; n && n.tagName!='TABLE'; n=n.parentNode);
		return n;
	},
	ascendDom: function(inNode, inWhile){
		for (var n=inNode; n && inWhile(n); n=n.parentNode);
		return n;
	},
	makeNotTagName: function(inTagName){
		var name = inTagName.toUpperCase();
		return function(node){ return node.tagName != name; };
	},
	fire: function(ob, ev, args){
		var fn = ob && ev && ob[ev];
		return fn && (args ? fn.apply(ob, args) : ob[ev]());
	},
	// from lib.js
	setStyleText: function(inNode, inStyleText){
		if(inNode.style.cssText == undefined){
			inNode.setAttribute("style", inStyleText);
		}else{
			inNode.style.cssText = inStyleText;
		}
	},
	getStyleText: function(inNode, inStyleText){
		return (inNode.style.cssText == undefined ? inNode.getAttribute("style") : inNode.style.cssText);
	},
	setStyle: function(inElement, inStyle, inValue){
		if(inElement && inElement.style[inStyle] != inValue){
			inElement.style[inStyle] = inValue;
		}
	},
	setStyleHeightPx: function(inElement, inHeight){
		if(inHeight >= 0){
			dojox.grid.setStyle(inElement, 'height', inHeight + 'px');
		}
	},
	mouseEvents: [ 'mouseover', 'mouseout', /*'mousemove',*/ 'mousedown', 'mouseup', 'click', 'dblclick', 'contextmenu' ],
	keyEvents: [ 'keyup', 'keydown', 'keypress' ],
	funnelEvents: function(inNode, inObject, inMethod, inEvents){
		var evts = (inEvents ? inEvents : dojox.grid.mouseEvents.concat(dojox.grid.keyEvents));
		for (var i=0, l=evts.length; i<l; i++){
			dojo.connect(inNode, 'on' + evts[i], inObject, inMethod);
		}
	},
	removeNode: function(inNode){
		inNode = dojo.byId(inNode);
		inNode && inNode.parentNode && inNode.parentNode.removeChild(inNode);
		return inNode;
	},
	getScrollbarWidth: function(){
		if(this._scrollBarWidth){
			return this._scrollBarWidth;
		}
		this._scrollBarWidth = 18;
		try{
			var e = document.createElement("div");
			e.style.cssText = "top:0;left:0;width:100px;height:100px;overflow:scroll;position:absolute;visibility:hidden;";
			document.body.appendChild(e);
			this._scrollBarWidth = e.offsetWidth - e.clientWidth;
			document.body.removeChild(e);
			delete e;
		}catch (ex){}
		return this._scrollBarWidth;
	},
	// needed? dojo has _getProp
	getRef: function(name, create, context){
		var obj=context||dojo.global, parts=name.split("."), prop=parts.pop();
		for(var i=0, p; obj&&(p=parts[i]); i++){
			obj = (p in obj ? obj[p] : (create ? obj[p]={} : undefined));
		}
		return { obj: obj, prop: prop }; 
	},
	getProp: function(name, create, context){
		with(dojox.grid.getRef(name, create, context)){
			return (obj)&&(prop)&&(prop in obj ? obj[prop] : (create ? obj[prop]={} : undefined));
		}
	},
	indexInParent: function(inNode){
		var i=0, n, p=inNode.parentNode;
		while(n = p.childNodes[i++]){
			if(n == inNode){
				return i - 1;
			}
		}
		return -1;
	},
	cleanNode: function(inNode){
		if(!inNode){
			return;
		}
		var filter = function(inW){
			return inW.domNode && dojo.isDescendant(inW.domNode, inNode, true);
		}
		var ws = dijit.registry.filter(filter);
		for(var i=0, w; (w=ws[i]); i++){
			w.destroy();
		}
		delete ws;
	},
	getTagName: function(inNodeOrId){
		var node = dojo.byId(inNodeOrId);
		return (node && node.tagName ? node.tagName.toLowerCase() : '');
	},
	nodeKids: function(inNode, inTag){
		var result = [];
		var i=0, n;
		while(n = inNode.childNodes[i++]){
			if(dojox.grid.getTagName(n) == inTag){
				result.push(n);
			}
		}
		return result;
	},
	divkids: function(inNode){
		return dojox.grid.nodeKids(inNode, 'div');
	},
	focusSelectNode: function(inNode){
		try{
			dojox.grid.fire(inNode, "focus");
			dojox.grid.fire(inNode, "select");
		}catch(e){// IE sux bad
		}
	},
	whenIdle: function(/*inContext, inMethod, args ...*/){
		setTimeout(dojo.hitch.apply(dojo, arguments), 0);
	},
	arrayCompare: function(inA, inB){
		for(var i=0,l=inA.length; i<l; i++){
			if(inA[i] != inB[i]){return false;}
		}
		return (inA.length == inB.length);
	},
	arrayInsert: function(inArray, inIndex, inValue){
		if(inArray.length <= inIndex){
			inArray[inIndex] = inValue;
		}else{
			inArray.splice(inIndex, 0, inValue);
		}
	},
	arrayRemove: function(inArray, inIndex){
		inArray.splice(inIndex, 1);
	},
	arraySwap: function(inArray, inI, inJ){
		var cache = inArray[inI];
		inArray[inI] = inArray[inJ];
		inArray[inJ] = cache;
	},
	initTextSizePoll: function(inInterval) {
		var f = document.createElement("div");
		with (f.style) {
			top = "0px";
			left = "0px";
			position = "absolute";
			visibility = "hidden";
		}
		f.innerHTML = "TheQuickBrownFoxJumpedOverTheLazyDog";
		document.body.appendChild(f);
		var fw = f.offsetWidth;
		var job = function() {
			if (f.offsetWidth != fw) {
				fw = f.offsetWidth;
				dojox.grid.textSizeChanged();
			}
		}
		window.setInterval(job, inInterval||200);
		dojox.grid.initTextSizePoll = dojox.grid.nop;
	},
	textSizeChanged: function() {
	}
});

dojox.grid.jobs = {
	cancel: function(inHandle){
		if(inHandle){
			window.clearTimeout(inHandle);
		}
	},
	jobs: [],
	job: function(inName, inDelay, inJob){
		dojox.grid.jobs.cancelJob(inName);
		var job = function(){
			delete dojox.grid.jobs.jobs[inName];
			inJob();
		}
		dojox.grid.jobs.jobs[inName] = setTimeout(job, inDelay);
	},
	cancelJob: function(inName){
		dojox.grid.jobs.cancel(dojox.grid.jobs.jobs[inName]);
	}
}

}

if(!dojo._hasResource['dojox.grid._grid.scroller']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._grid.scroller'] = true;
dojo.provide('dojox.grid._grid.scroller');

dojo.declare('dojox.grid.scroller.base', null, {
	// summary:
	//	virtual scrollbox, abstract class
	//	Content must in /rows/
	//	Rows are managed in contiguous sets called /pages/
	//	There are a fixed # of rows per page
	//	The minimum rendered unit is a page
	constructor: function(){
		this.pageHeights = [];
		this.stack = [];
	},
	// specified
	rowCount: 0, // total number of rows to manage
	defaultRowHeight: 10, // default height of a row
	keepRows: 100, // maximum number of rows that should exist at one time
	contentNode: null, // node to contain pages
	scrollboxNode: null, // node that controls scrolling
	// calculated
	defaultPageHeight: 0, // default height of a page
	keepPages: 10, // maximum number of pages that should exists at one time
	pageCount: 0,
	windowHeight: 0,
	firstVisibleRow: 0,
	lastVisibleRow: 0,
	// private
	page: 0,
	pageTop: 0,
	// init
	init: function(inRowCount, inKeepRows, inRowsPerPage){
		switch(arguments.length){
			case 3: this.rowsPerPage = inRowsPerPage;
			case 2: this.keepRows = inKeepRows;
			case 1: this.rowCount = inRowCount;
		}
		this.defaultPageHeight = this.defaultRowHeight * this.rowsPerPage;
		//this.defaultPageHeight = this.defaultRowHeight * Math.min(this.rowsPerPage, this.rowCount);
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		this.keepPages = Math.max(Math.ceil(this.keepRows / this.rowsPerPage), 2);
		this.invalidate();
		if(this.scrollboxNode){
			this.scrollboxNode.scrollTop = 0;
			this.scroll(0);
			this.scrollboxNode.onscroll = dojo.hitch(this, 'onscroll');
		}
	},
	// updating
	invalidate: function(){
		this.invalidateNodes();
		this.pageHeights = [];
		this.height = (this.pageCount ? (this.pageCount - 1)* this.defaultPageHeight + this.calcLastPageHeight() : 0);
		this.resize();
	},
	updateRowCount: function(inRowCount){
		this.invalidateNodes();
		this.rowCount = inRowCount;
		// update page count, adjust document height
		oldPageCount = this.pageCount;
		this.pageCount = Math.ceil(this.rowCount / this.rowsPerPage);
		if(this.pageCount < oldPageCount){
			for(var i=oldPageCount-1; i>=this.pageCount; i--){
				this.height -= this.getPageHeight(i);
				delete this.pageHeights[i]
			}
		}else if(this.pageCount > oldPageCount){
			this.height += this.defaultPageHeight * (this.pageCount - oldPageCount - 1) + this.calcLastPageHeight();
		}
		this.resize();
	},
	// abstract interface
	pageExists: function(inPageIndex){
	},
	measurePage: function(inPageIndex){
	},
	positionPage: function(inPageIndex, inPos){
	},
	repositionPages: function(inPageIndex){
	},
	installPage: function(inPageIndex){
	},
	preparePage: function(inPageIndex, inPos, inReuseNode){
	},
	renderPage: function(inPageIndex){
	},
	removePage: function(inPageIndex){
	},
	pacify: function(inShouldPacify){
	},
	// pacification
	pacifying: false,
	pacifyTicks: 200,
	setPacifying: function(inPacifying){
		if(this.pacifying != inPacifying){
			this.pacifying = inPacifying;
			this.pacify(this.pacifying);
		}
	},
	startPacify: function(){
		this.startPacifyTicks = new Date().getTime();
	},
	doPacify: function(){
		var result = (new Date().getTime() - this.startPacifyTicks) > this.pacifyTicks;
		this.setPacifying(true);
		this.startPacify();
		return result;
	},
	endPacify: function(){
		this.setPacifying(false);
	},
	// default sizing implementation
	resize: function(){
		if(this.scrollboxNode){
			this.windowHeight = this.scrollboxNode.clientHeight;
		}
		dojox.grid.setStyleHeightPx(this.contentNode, this.height);
	},
	calcLastPageHeight: function(){
		if(!this.pageCount){
			return 0;
		}
		var lastPage = this.pageCount - 1;
		var lastPageHeight = ((this.rowCount % this.rowsPerPage)||(this.rowsPerPage)) * this.defaultRowHeight;
		this.pageHeights[lastPage] = lastPageHeight;
		return lastPageHeight;
	},
	updateContentHeight: function(inDh){
		this.height += inDh;
		this.resize();
	},
	updatePageHeight: function(inPageIndex){
		if(this.pageExists(inPageIndex)){
			var oh = this.getPageHeight(inPageIndex);
			var h = (this.measurePage(inPageIndex))||(oh);
			this.pageHeights[inPageIndex] = h;
			if((h)&&(oh != h)){
				this.updateContentHeight(h - oh)
				this.repositionPages(inPageIndex);
			}
		}
	},
	rowHeightChanged: function(inRowIndex){
		this.updatePageHeight(Math.floor(inRowIndex / this.rowsPerPage));
	},
	// scroller core
	invalidateNodes: function(){
		while(this.stack.length){
			this.destroyPage(this.popPage());
		}
	},
	createPageNode: function(){
		var p = document.createElement('div');
		p.style.position = 'absolute';
		//p.style.width = '100%';
		p.style.left = '0';
		return p;
	},
	getPageHeight: function(inPageIndex){
		var ph = this.pageHeights[inPageIndex];
		return (ph !== undefined ? ph : this.defaultPageHeight);
	},
	// FIXME: this is not a stack, it's a FIFO list
	pushPage: function(inPageIndex){
		return this.stack.push(inPageIndex);
	},
	popPage: function(){
		return this.stack.shift();
	},
	findPage: function(inTop){
		var i = 0, h = 0;
		for(var ph = 0; i<this.pageCount; i++, h += ph){
			ph = this.getPageHeight(i);
			if(h + ph >= inTop){
				break;
			}
		}
		this.page = i;
		this.pageTop = h;
	},
	buildPage: function(inPageIndex, inReuseNode, inPos){
		this.preparePage(inPageIndex, inReuseNode);
		this.positionPage(inPageIndex, inPos);
		// order of operations is key below
		this.installPage(inPageIndex);
		this.renderPage(inPageIndex);
		// order of operations is key above
		this.pushPage(inPageIndex);
	},
	needPage: function(inPageIndex, inPos){
		var h = this.getPageHeight(inPageIndex), oh = h;
		if(!this.pageExists(inPageIndex)){
			this.buildPage(inPageIndex, (this.keepPages)&&(this.stack.length >= this.keepPages), inPos);
			h = this.measurePage(inPageIndex) || h;
			this.pageHeights[inPageIndex] = h;
			if(h && (oh != h)){
				this.updateContentHeight(h - oh)
			}
		}else{
			this.positionPage(inPageIndex, inPos);
		}
		return h;
	},
	onscroll: function(){
		this.scroll(this.scrollboxNode.scrollTop);
	},
	scroll: function(inTop){
		this.startPacify();
		this.findPage(inTop);
		var h = this.height;
		var b = this.getScrollBottom(inTop);
		for(var p=this.page, y=this.pageTop; (p<this.pageCount)&&((b<0)||(y<b)); p++){
			y += this.needPage(p, y);
		}
		this.firstVisibleRow = this.getFirstVisibleRow(this.page, this.pageTop, inTop);
		this.lastVisibleRow = this.getLastVisibleRow(p - 1, y, b);
		// indicates some page size has been updated
		if(h != this.height){
			this.repositionPages(p-1);
		}
		this.endPacify();
	},
	getScrollBottom: function(inTop){
		return (this.windowHeight >= 0 ? inTop + this.windowHeight : -1);
	},
	// events
	processNodeEvent: function(e, inNode){
		var t = e.target;
		while(t && (t != inNode) && t.parentNode && (t.parentNode.parentNode != inNode)){
			t = t.parentNode;
		}
		if(!t || !t.parentNode || (t.parentNode.parentNode != inNode)){
			return false;
		}
		var page = t.parentNode;
		e.topRowIndex = page.pageIndex * this.rowsPerPage;
		e.rowIndex = e.topRowIndex + dojox.grid.indexInParent(t);
		e.rowTarget = t;
		return true;
	},
	processEvent: function(e){
		return this.processNodeEvent(e, this.contentNode);
	},
	dummy: 0
});

dojo.declare('dojox.grid.scroller', dojox.grid.scroller.base, {
	// summary:
	//	virtual scroller class, makes no assumption about shape of items being scrolled
	constructor: function(){
		this.pageNodes = [];
	},
	// virtual rendering interface
	renderRow: function(inRowIndex, inPageNode){
	},
	removeRow: function(inRowIndex){
	},
	// page node operations
	getDefaultNodes: function(){
		return this.pageNodes;
	},
	getDefaultPageNode: function(inPageIndex){
		return this.getDefaultNodes()[inPageIndex];
	},
	positionPageNode: function(inNode, inPos){
		inNode.style.top = inPos + 'px';
	},
	getPageNodePosition: function(inNode){
		return inNode.offsetTop;
	},
	repositionPageNodes: function(inPageIndex, inNodes){
		var last = 0;
		for(var i=0; i<this.stack.length; i++){
			last = Math.max(this.stack[i], last);
		}
		//
		var n = inNodes[inPageIndex];
		var y = (n ? this.getPageNodePosition(n) + this.getPageHeight(inPageIndex) : 0);
		//console.log('detected height change, repositioning from #%d (%d) @ %d ', inPageIndex + 1, last, y, this.pageHeights[0]);
		//
		for(var p=inPageIndex+1; p<=last; p++){
			n = inNodes[p];
			if(n){
				//console.log('#%d @ %d', inPageIndex, y, this.getPageNodePosition(n));
				if(this.getPageNodePosition(n) == y){
					return;
				}
				//console.log('placing page %d at %d', p, y);
				this.positionPage(p, y);
			}
			y += this.getPageHeight(p);
		}
	},
	invalidatePageNode: function(inPageIndex, inNodes){
		var p = inNodes[inPageIndex];
		if(p){
			delete inNodes[inPageIndex];
			this.removePage(inPageIndex, p);
			dojox.grid.cleanNode(p);
			p.innerHTML = '';
		}
		return p;
	},
	preparePageNode: function(inPageIndex, inReusePageIndex, inNodes){
		var p = (inReusePageIndex === null ? this.createPageNode() : this.invalidatePageNode(inReusePageIndex, inNodes));
		p.pageIndex = inPageIndex;
		p.id = 'page-' + inPageIndex;
		inNodes[inPageIndex] = p;
	},
	// implementation for page manager
	pageExists: function(inPageIndex){
		return Boolean(this.getDefaultPageNode(inPageIndex));
	},
	measurePage: function(inPageIndex){
		return this.getDefaultPageNode(inPageIndex).offsetHeight;
	},
	positionPage: function(inPageIndex, inPos){
		this.positionPageNode(this.getDefaultPageNode(inPageIndex), inPos);
	},
	repositionPages: function(inPageIndex){
		this.repositionPageNodes(inPageIndex, this.getDefaultNodes());
	},
	preparePage: function(inPageIndex, inReuseNode){
		this.preparePageNode(inPageIndex, (inReuseNode ? this.popPage() : null), this.getDefaultNodes());
	},
	installPage: function(inPageIndex){
		this.contentNode.appendChild(this.getDefaultPageNode(inPageIndex));
	},
	destroyPage: function(inPageIndex){
		var p = this.invalidatePageNode(inPageIndex, this.getDefaultNodes());
		dojox.grid.removeNode(p);
	},
	// rendering implementation
	renderPage: function(inPageIndex){
		var node = this.pageNodes[inPageIndex];
		for(var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
			this.renderRow(j, node);
		}
	},
	removePage: function(inPageIndex){
		for(var i=0, j=inPageIndex*this.rowsPerPage; i<this.rowsPerPage; i++, j++){
			this.removeRow(j);
		}
	},
	// scroll control
	getPageRow: function(inPage){
		return inPage * this.rowsPerPage;
	},
	getLastPageRow: function(inPage){
		return Math.min(this.rowCount, this.getPageRow(inPage + 1)) - 1;
	},
	getFirstVisibleRowNodes: function(inPage, inPageTop, inScrollTop, inNodes){
		var row = this.getPageRow(inPage);
		var rows = dojox.grid.divkids(inNodes[inPage]);
		for(var i=0,l=rows.length; i<l && inPageTop<inScrollTop; i++, row++){
			inPageTop += rows[i].offsetHeight;
		}
		return (row ? row - 1 : row);
	},
	getFirstVisibleRow: function(inPage, inPageTop, inScrollTop){
		if(!this.pageExists(inPage)){
			return 0;
		}
		return this.getFirstVisibleRowNodes(inPage, inPageTop, inScrollTop, this.getDefaultNodes());
	},
	getLastVisibleRowNodes: function(inPage, inBottom, inScrollBottom, inNodes){
		var row = this.getLastPageRow(inPage);
		var rows = dojox.grid.divkids(inNodes[inPage]);
		for(var i=rows.length-1; i>=0 && inBottom>inScrollBottom; i--, row--){
			inBottom -= rows[i].offsetHeight;
		}
		return row + 1;
	},
	getLastVisibleRow: function(inPage, inBottom, inScrollBottom){
		if(!this.pageExists(inPage)){
			return 0;
		}
		return this.getLastVisibleRowNodes(inPage, inBottom, inScrollBottom, this.getDefaultNodes());
	},
	findTopRowForNodes: function(inScrollTop, inNodes){
		var rows = dojox.grid.divkids(inNodes[this.page]);
		for(var i=0,l=rows.length,t=this.pageTop,h; i<l; i++){
			h = rows[i].offsetHeight;
			t += h;
			if(t >= inScrollTop){
				this.offset = h - (t - inScrollTop);
				return i + this.page * this.rowsPerPage;
			}
		}
		return -1;
	},
	findScrollTopForNodes: function(inRow, inNodes){
		var rowPage = Math.floor(inRow / this.rowsPerPage);
		var t = 0;
		for(var i=0; i<rowPage; i++){
			t += this.getPageHeight(i);
		}
		this.pageTop = t;
		this.needPage(rowPage, this.pageTop);
		var rows = dojox.grid.divkids(inNodes[rowPage]);
		var r = inRow - this.rowsPerPage * rowPage;
		for(var i=0,l=rows.length; i<l && i<r; i++){
			t += rows[i].offsetHeight;
		}
		return t;
	},
	findTopRow: function(inScrollTop){
		return this.findTopRowForNodes(inScrollTop, this.getDefaultNodes());
	},
	findScrollTop: function(inRow){
		return this.findScrollTopForNodes(inRow, this.getDefaultNodes());
	},
	dummy: 0
});

dojo.declare('dojox.grid.scroller.columns', dojox.grid.scroller, {
	// summary:
	//	Virtual scroller class that scrolls list of columns. Owned by grid and used internally
	//	for virtual scrolling.
	constructor: function(inContentNodes){
		this.setContentNodes(inContentNodes);
	},
	// nodes
	setContentNodes: function(inNodes){
		this.contentNodes = inNodes;
		this.colCount = (this.contentNodes ? this.contentNodes.length : 0);
		this.pageNodes = [];
		for(var i=0; i<this.colCount; i++){
			this.pageNodes[i] = [];
		}
	},
	getDefaultNodes: function(){
		return this.pageNodes[0] || [];
	},
	scroll: function(inTop) {
		if(this.colCount){
			dojox.grid.scroller.prototype.scroll.call(this, inTop);
		}
	},
	// resize
	resize: function(){
		if(this.scrollboxNode){
			this.windowHeight = this.scrollboxNode.clientHeight;
		}
		for(var i=0; i<this.colCount; i++){
			dojox.grid.setStyleHeightPx(this.contentNodes[i], this.height);
		}
	},
	// implementation for page manager
	positionPage: function(inPageIndex, inPos){
		for(var i=0; i<this.colCount; i++){
			this.positionPageNode(this.pageNodes[i][inPageIndex], inPos);
		}
	},
	preparePage: function(inPageIndex, inReuseNode){
		var p = (inReuseNode ? this.popPage() : null);
		for(var i=0; i<this.colCount; i++){
			this.preparePageNode(inPageIndex, p, this.pageNodes[i]);
		}
	},
	installPage: function(inPageIndex){
		for(var i=0; i<this.colCount; i++){
			this.contentNodes[i].appendChild(this.pageNodes[i][inPageIndex]);
		}
	},
	destroyPage: function(inPageIndex){
		for(var i=0; i<this.colCount; i++){
			dojox.grid.removeNode(this.invalidatePageNode(inPageIndex, this.pageNodes[i]));
		}
	},
	// rendering implementation
	renderPage: function(inPageIndex){
		var nodes = [];
		for(var i=0; i<this.colCount; i++){
			nodes[i] = this.pageNodes[i][inPageIndex];
		}
		//this.renderRows(inPageIndex*this.rowsPerPage, this.rowsPerPage, nodes);
		for(var i=0, j=inPageIndex*this.rowsPerPage; (i<this.rowsPerPage)&&(j<this.rowCount); i++, j++){
			this.renderRow(j, nodes);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.drag"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.drag"] = true;
dojo.provide("dojox.grid._grid.drag");

// summary:
//	utility functions for dragging as used in grid.
// begin closure
(function(){

var dgdrag = dojox.grid.drag = {};

dgdrag.dragging = false;
dgdrag.hysteresis = 2;

dgdrag.capture = function(inElement) {
	//console.debug('dojox.grid.drag.capture');
	if (inElement.setCapture)
		inElement.setCapture();
	else {
		document.addEventListener("mousemove", inElement.onmousemove, true);
		document.addEventListener("mouseup", inElement.onmouseup, true);
		document.addEventListener("click", inElement.onclick, true);
	}
}

dgdrag.release = function(inElement) {
	//console.debug('dojox.grid.drag.release');
	if(inElement.releaseCapture){
		inElement.releaseCapture();
	}else{
		document.removeEventListener("click", inElement.onclick, true);
		document.removeEventListener("mouseup", inElement.onmouseup, true);
		document.removeEventListener("mousemove", inElement.onmousemove, true);
	}
}

dgdrag.start = function(inElement, inOnDrag, inOnEnd, inEvent, inOnStart){
	if(/*dgdrag.elt ||*/ !inElement || dgdrag.dragging){
		console.debug('failed to start drag: bad input node or already dragging');
		return;
	}
	dgdrag.dragging = true;
	dgdrag.elt = inElement;
	dgdrag.events = {
		drag: inOnDrag || dojox.grid.nop, 
		end: inOnEnd || dojox.grid.nop, 
		start: inOnStart || dojox.grid.nop, 
		oldmove: inElement.onmousemove, 
		oldup: inElement.onmouseup, 
		oldclick: inElement.onclick 
	};
	dgdrag.positionX = (inEvent && ('screenX' in inEvent) ? inEvent.screenX : false);
	dgdrag.positionY = (inEvent && ('screenY' in inEvent) ? inEvent.screenY : false);
	dgdrag.started = (dgdrag.position === false);
	inElement.onmousemove = dgdrag.mousemove;
	inElement.onmouseup = dgdrag.mouseup;
	inElement.onclick = dgdrag.click;
	dgdrag.capture(dgdrag.elt);
}

dgdrag.end = function(){
	//console.debug("dojox.grid.drag.end");
	dgdrag.release(dgdrag.elt);
	dgdrag.elt.onmousemove = dgdrag.events.oldmove;
	dgdrag.elt.onmouseup = dgdrag.events.oldup;
	dgdrag.elt.onclick = dgdrag.events.oldclick;
	dgdrag.elt = null;
	try{
		if(dgdrag.started){
			dgdrag.events.end();
		}
	}finally{
		dgdrag.dragging = false;
	}
}

dgdrag.calcDelta = function(inEvent){
	inEvent.deltaX = inEvent.screenX - dgdrag.positionX;
	inEvent.deltaY = inEvent.screenY - dgdrag.positionY;
}

dgdrag.hasMoved = function(inEvent){
	return Math.abs(inEvent.deltaX) + Math.abs(inEvent.deltaY) > dgdrag.hysteresis;
}

dgdrag.mousemove = function(inEvent){
	inEvent = dojo.fixEvent(inEvent);
	dojo.stopEvent(inEvent);
	dgdrag.calcDelta(inEvent);
	if((!dgdrag.started)&&(dgdrag.hasMoved(inEvent))){
		dgdrag.events.start(inEvent);
		dgdrag.started = true;
	}
	if(dgdrag.started){
		dgdrag.events.drag(inEvent);
	}
}

dgdrag.mouseup = function(inEvent){
	//console.debug("dojox.grid.drag.mouseup");
	dojo.stopEvent(dojo.fixEvent(inEvent));
	dgdrag.end();
}

dgdrag.click = function(inEvent){
	dojo.stopEvent(dojo.fixEvent(inEvent));
	//dgdrag.end();
}

})();
// end closure

}

if(!dojo._hasResource["dojox.grid._grid.builder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.builder"] = true;
dojo.provide("dojox.grid._grid.builder");


dojo.declare("dojox.grid.Builder", null, {
	// summary:
	//	Base class to produce html for grid content.
	//	Also provide event decoration, providing grid related information inside the event object
	// passed to grid events.
	constructor: function(inView){
		this.view = inView;
		this.grid = inView.grid;
	},
	view: null,
	// boilerplate HTML
	_table: '<table class="dojoxGrid-row-table" border="0" cellspacing="0" cellpadding="0" role="wairole:presentation">',
	// generate starting tags for a cell
	generateCellMarkup: function(inCell, inMoreStyles, inMoreClasses, isHeader){
		var result = [], html;
		if (isHeader){
			html = [ '<th tabIndex="-1" role="wairole:columnheader"' ];
		}else{
			html = [ '<td tabIndex="-1" role="wairole:gridcell"' ];
		}
		inCell.colSpan && html.push(' colspan="', inCell.colSpan, '"');
		inCell.rowSpan && html.push(' rowspan="', inCell.rowSpan, '"');
		html.push(' class="dojoxGrid-cell ');
		inCell.classes && html.push(inCell.classes, ' ');
		inMoreClasses && html.push(inMoreClasses, ' ');
		// result[0] => td opener, style
		result.push(html.join(''));
		// SLOT: result[1] => td classes 
		result.push('');
		html = ['" idx="', inCell.index, '" style="'];
		html.push(inCell.styles, inMoreStyles||'');
		inCell.unitWidth && html.push('width:', inCell.unitWidth, ';');
		// result[2] => markup
		result.push(html.join(''));
		// SLOT: result[3] => td style 
		result.push('');
		html = [ '"' ];
		inCell.attrs && html.push(" ", inCell.attrs);
		html.push('>');
		// result[4] => td postfix
		result.push(html.join(''));
		// SLOT: result[5] => content
		result.push('');
		// result[6] => td closes
		result.push('</td>');
		return result;
	},
	// cell finding
	isCellNode: function(inNode){
		return Boolean(inNode && inNode.getAttribute && inNode.getAttribute("idx"));
	},
	getCellNodeIndex: function(inCellNode){
		return inCellNode ? Number(inCellNode.getAttribute("idx")) : -1;
	},
	getCellNode: function(inRowNode, inCellIndex){
		for(var i=0, row; row=dojox.grid.getTr(inRowNode.firstChild, i); i++){
			for(var j=0, cell; cell=row.cells[j]; j++){
				if(this.getCellNodeIndex(cell) == inCellIndex){
					return cell;
				}
			}
		}
	},
	findCellTarget: function(inSourceNode, inTopNode){
		var n = inSourceNode;
		while(n && (!this.isCellNode(n) || (dojox.grid.gridViewTag in n.offsetParent.parentNode && n.offsetParent.parentNode[dojox.grid.gridViewTag] != this.view)) && (n!=inTopNode)){
			n = n.parentNode;
		}
		return n!=inTopNode ? n : null 
	},
	// event decoration
	baseDecorateEvent: function(e){
		e.dispatch = 'do' + e.type;
		e.grid = this.grid;
		e.sourceView = this.view;
		e.cellNode = this.findCellTarget(e.target, e.rowNode);
		e.cellIndex = this.getCellNodeIndex(e.cellNode);
		e.cell = (e.cellIndex >= 0 ? this.grid.getCell(e.cellIndex) : null);
	},
	// event dispatch
	findTarget: function(inSource, inTag){
		var n = inSource;
		while(n && (n!=this.domNode) && (!(inTag in n) || (dojox.grid.gridViewTag in n && n[dojox.grid.gridViewTag] != this.view))){
			n = n.parentNode;
		}
		return (n != this.domNode) ? n : null; 
	},
	findRowTarget: function(inSource){
		return this.findTarget(inSource, dojox.grid.rowIndexTag);
	},
	isIntraNodeEvent: function(e){
		try{
			return (e.cellNode && e.relatedTarget && dojo.isDescendant(e.relatedTarget, e.cellNode));
		}catch(x){
			// e.relatedTarget has permission problem in FF if it's an input: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;
		}
	},
	isIntraRowEvent: function(e){
		try{
			var row = e.relatedTarget && this.findRowTarget(e.relatedTarget);
			return !row && (e.rowIndex==-1) || row && (e.rowIndex==row.gridRowIndex);			
		}catch(x){
			// e.relatedTarget on INPUT has permission problem in FF: https://bugzilla.mozilla.org/show_bug.cgi?id=208427
			return false;
		}
	},
	dispatchEvent: function(e){
		if(e.dispatch in this){
			return this[e.dispatch](e);
		}
	},
	// dispatched event handlers
	domouseover: function(e){
		if(e.cellNode && (e.cellNode!=this.lastOverCellNode)){
			this.lastOverCellNode = e.cellNode;
			this.grid.onMouseOver(e);
		}
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(e.cellNode && (e.cellNode==this.lastOverCellNode) && !this.isIntraNodeEvent(e, this.lastOverCellNode)){
			this.lastOverCellNode = null;
			this.grid.onMouseOut(e);
			if(!this.isIntraRowEvent(e)){
				this.grid.onMouseOutRow(e);
			}
		}
	}
});

dojo.declare("dojox.grid.contentBuilder", dojox.grid.Builder, {
	// summary:
	//	Produces html for grid data content. Owned by grid and used internally 
	//	for rendering data. Override to implement custom rendering.
	update: function(){
		this.prepareHtml();
	},
	// cache html for rendering data rows
	prepareHtml: function(){
		var defaultGet=this.grid.get, rows=this.view.structure.rows;
		for(var j=0, row; (row=rows[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				cell.get = cell.get || (cell.value == undefined) && defaultGet;
				cell.markup = this.generateCellMarkup(cell, cell.cellStyles, cell.cellClasses, false);
			}
		}
	},
	// time critical: generate html using cache and data source
	generateHtml: function(inDataIndex, inRowIndex){
		var
			html = [ this._table ],
			v = this.view,
			obr = v.onBeforeRow,
			rows = v.structure.rows;
		obr && obr(inRowIndex, rows);
		for(var j=0, row; (row=rows[j]); j++){
			if(row.hidden || row.header){
				continue;
			}
			html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGrid-invisible">');
			for(var i=0, cell, m, cc, cs; (cell=row[i]); i++){
				m = cell.markup, cc = cell.customClasses = [], cs = cell.customStyles = [];
				// content (format can fill in cc and cs as side-effects)
				m[5] = cell.format(inDataIndex);
				// classes
				m[1] = cc.join(' ');
				// styles
				m[3] = cs.join(';');
				// in-place concat
				html.push.apply(html, m);
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');
	},
	decorateEvent: function(e){
		e.rowNode = this.findRowTarget(e.target);
		if(!e.rowNode){return false};
		e.rowIndex = e.rowNode[dojox.grid.rowIndexTag];
		this.baseDecorateEvent(e);
		e.cell = this.grid.getCell(e.cellIndex);
		return true;
	}
});

dojo.declare("dojox.grid.headerBuilder", dojox.grid.Builder, {
	// summary:
	//	Produces html for grid header content. Owned by grid and used internally 
	//	for rendering data. Override to implement custom rendering.
	bogusClickTime: 0,
	overResizeWidth: 4,
	minColWidth: 1,
	_table: '<table class="dojoxGrid-row-table" border="0" cellspacing="0" cellpadding="0" role="wairole:presentation"',
	update: function(){
		this.tableMap = new dojox.grid.tableMap(this.view.structure.rows);
	},
	generateHtml: function(inGetValue, inValue){
		var html = [this._table], rows = this.view.structure.rows;
		
		// render header with appropriate width, if possible so that views with flex columns are correct height
		if(this.view.viewWidth){
			html.push([' style="width:', this.view.viewWidth, ';"'].join(''));
		}
		html.push('>');
		dojox.grid.fire(this.view, "onBeforeRow", [-1, rows]);
		for(var j=0, row; (row=rows[j]); j++){
			if(row.hidden){
				continue;
			}
			html.push(!row.invisible ? '<tr>' : '<tr class="dojoxGrid-invisible">');
			for(var i=0, cell, markup; (cell=row[i]); i++){
				cell.customClasses = [];
				cell.customStyles = [];
				markup = this.generateCellMarkup(cell, cell.headerStyles, cell.headerClasses, true);
				// content
				markup[5] = (inValue != undefined ? inValue : inGetValue(cell));
				// styles
				markup[3] = cell.customStyles.join(';');
				// classes
				markup[1] = cell.customClasses.join(' '); //(cell.customClasses ? ' ' + cell.customClasses : '');
				html.push(markup.join(''));
			}
			html.push('</tr>');
		}
		html.push('</table>');
		return html.join('');
	},
	// event helpers
	getCellX: function(e){
		var x = e.layerX;
		if(dojo.isMoz){
			var n = dojox.grid.ascendDom(e.target, dojox.grid.makeNotTagName("th"));
			x -= (n && n.offsetLeft) || 0;
			//x -= getProp(ascendDom(e.target, mkNotTagName("td")), "offsetLeft") || 0;
		}
		var n = dojox.grid.ascendDom(e.target, function(){
			if(!n || n == e.cellNode){
				return false;
			}
			// Mozilla 1.8 (FF 1.5) has a bug that makes offsetLeft = -parent border width
			// when parent has border, overflow: hidden, and is positioned
			// handle this problem here ... not a general solution!
			x += (n.offsetLeft < 0 ? 0 : n.offsetLeft);
			return true;
		});
		return x;
	},
	// event decoration
	decorateEvent: function(e){
		this.baseDecorateEvent(e);
		e.rowIndex = -1;
		e.cellX = this.getCellX(e);
		return true;
	},
	// event handlers
	// resizing
	prepareLeftResize: function(e){
		var i = dojox.grid.getTdIndex(e.cellNode);
		e.cellNode = (i ? e.cellNode.parentNode.cells[i-1] : null);
		e.cellIndex = (e.cellNode ? this.getCellNodeIndex(e.cellNode) : -1);
		return Boolean(e.cellNode);
	},
	canResize: function(e){
		if(!e.cellNode || e.cellNode.colSpan > 1){
			return false;
		}
		var cell = this.grid.getCell(e.cellIndex); 
		return !cell.noresize && !cell.isFlex();
	},
	overLeftResizeArea: function(e){
		return (e.cellIndex>0) && (e.cellX < this.overResizeWidth) && this.prepareLeftResize(e);
	},
	overRightResizeArea: function(e){
		return e.cellNode && (e.cellX >= e.cellNode.offsetWidth - this.overResizeWidth);
	},
	domousemove: function(e){
		//console.log(e.cellIndex, e.cellX, e.cellNode.offsetWidth);
		var c = (this.overRightResizeArea(e) ? 'e-resize' : (this.overLeftResizeArea(e) ? 'w-resize' : ''));
		if(c && !this.canResize(e)){
			c = 'not-allowed';
		}
		e.sourceView.headerNode.style.cursor = c || ''; //'default';
	},
	domousedown: function(e){
		if(!dojox.grid.drag.dragging){
			if((this.overRightResizeArea(e) || this.overLeftResizeArea(e)) && this.canResize(e)){
				this.beginColumnResize(e);
			}
			//else{
			//	this.beginMoveColumn(e);
			//}
		}
	},
	doclick: function(e) {
		if (new Date().getTime() < this.bogusClickTime) {
			dojo.stopEvent(e);
			return true;
		}
	},
	// column resizing
	beginColumnResize: function(e){
		dojo.stopEvent(e);
		var spanners = [], nodes = this.tableMap.findOverlappingNodes(e.cellNode);
		for(var i=0, cell; (cell=nodes[i]); i++){
			spanners.push({ node: cell, index: this.getCellNodeIndex(cell), width: cell.offsetWidth });
			//console.log("spanner: " + this.getCellNodeIndex(cell));
		}
		var drag = {
			view: e.sourceView,
			node: e.cellNode,
			index: e.cellIndex,
			w: e.cellNode.clientWidth,
			spanners: spanners
		};
		//console.log(drag.index, drag.w);
		dojox.grid.drag.start(e.cellNode, dojo.hitch(this, 'doResizeColumn', drag), dojo.hitch(this, 'endResizeColumn', drag), e);
	},
	doResizeColumn: function(inDrag, inEvent){
		var w = inDrag.w + inEvent.deltaX;
		if(w >= this.minColWidth){
			for(var i=0, s, sw; (s=inDrag.spanners[i]); i++){
				sw = s.width + inEvent.deltaX;
				s.node.style.width = sw + 'px';
				inDrag.view.setColWidth(s.index, sw);
				//console.log('setColWidth', '#' + s.index, sw + 'px');
			}
			inDrag.node.style.width = w + 'px';
			inDrag.view.setColWidth(inDrag.index, w);
		}
		if(inDrag.view.flexCells && !inDrag.view.testFlexCells()){
			var t = dojox.grid.findTable(inDrag.node);
			t && (t.style.width = '');
		}
	},
	endResizeColumn: function(inDrag){
		this.bogusClickTime = new Date().getTime() + 30;
		setTimeout(dojo.hitch(inDrag.view, "update"), 50);
	}
});

dojo.declare("dojox.grid.tableMap", null, {
	// summary:
	//	Maps an html table into a structure parsable for information about cell row and col spanning.
	//	Used by headerBuilder
	constructor: function(inRows){
		this.mapRows(inRows);
	},
	map: null,
	// map table topography
	mapRows: function(inRows){
		//console.log('mapRows');
		// # of rows
		var rowCount = inRows.length;
		if(!rowCount){
			return;
		}
		// map which columns and rows fill which cells
		this.map = [ ];
		for(var j=0, row; (row=inRows[j]); j++){
			this.map[j] = [];
		}
		for(var j=0, row; (row=inRows[j]); j++){
			for(var i=0, x=0, cell, colSpan, rowSpan; (cell=row[i]); i++){
				while (this.map[j][x]){x++};
				this.map[j][x] = { c: i, r: j };
				rowSpan = cell.rowSpan || 1;
				colSpan = cell.colSpan || 1;
				for(var y=0; y<rowSpan; y++){
					for(var s=0; s<colSpan; s++){
						this.map[j+y][x+s] = this.map[j][x];
					}
				}
				x += colSpan;
			}
		}
		//this.dumMap();
	},
	dumpMap: function(){
		for(var j=0, row, h=''; (row=this.map[j]); j++,h=''){
			for(var i=0, cell; (cell=row[i]); i++){
				h += cell.r + ',' + cell.c + '   ';
			}
			console.log(h);
		}
	},
	// find node's map coords by it's structure coords
	getMapCoords: function(inRow, inCol){
		for(var j=0, row; (row=this.map[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				if(cell.c==inCol && cell.r == inRow){
					return { j: j, i: i };
				}
				//else{console.log(inRow, inCol, ' : ', i, j, " : ", cell.r, cell.c); };
			}
		}
		return { j: -1, i: -1 };
	},
	// find a node in inNode's table with the given structure coords
	getNode: function(inTable, inRow, inCol){
		var row = inTable && inTable.rows[inRow];
		return row && row.cells[inCol];
	},
	_findOverlappingNodes: function(inTable, inRow, inCol){
		var nodes = [];
		var m = this.getMapCoords(inRow, inCol);
		//console.log("node j: %d, i: %d", m.j, m.i);
		var row = this.map[m.j];
		for(var j=0, row; (row=this.map[j]); j++){
			if(j == m.j){ continue; }
			with(row[m.i]){
				//console.log("overlaps: r: %d, c: %d", r, c);
				var n = this.getNode(inTable, r, c);
				if(n){ nodes.push(n); }
			}
		}
		//console.log(nodes);
		return nodes;
	},
	findOverlappingNodes: function(inNode){
		return this._findOverlappingNodes(dojox.grid.findTable(inNode), dojox.grid.getTrIndex(inNode.parentNode), dojox.grid.getTdIndex(inNode));
	}
});

dojox.grid.rowIndexTag = "gridRowIndex";
dojox.grid.gridViewTag = "gridView";

}

if(!dojo._hasResource["dojox.grid._grid.view"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.view"] = true;
dojo.provide("dojox.grid._grid.view");




dojo.declare('dojox.GridView', [dijit._Widget, dijit._Templated], {
	// summary:
	//	A collection of grid columns. A grid is comprised of a set of views that stack horizontally.
	//	Grid creates views automatically based on grid's layout structure.
	//	Users should typically not need to access individual views directly.
	defaultWidth: "18em",
	// viewWidth: string
	// width for the view, in valid css unit
	viewWidth: "",
	templateString: '<div class="dojoxGrid-view"><div class="dojoxGrid-header" dojoAttachPoint="headerNode"><div style="width: 9000em"><div dojoAttachPoint="headerContentNode"></div></div></div><input type="checkbox" class="dojoxGrid-hidden-focus" dojoAttachPoint="hiddenFocusNode" /><input type="checkbox" class="dojoxGrid-hidden-focus" /><div class="dojoxGrid-scrollbox" dojoAttachPoint="scrollboxNode"><div class="dojoxGrid-content" dojoAttachPoint="contentNode" hidefocus="hidefocus"></div></div></div>',
	themeable: false,
	classTag: 'dojoxGrid',
	marginBottom: 0,
	rowPad: 2,
	postMixInProperties: function(){
		this.rowNodes = [];
	},
	postCreate: function(){
		dojo.connect(this.scrollboxNode, "onscroll", dojo.hitch(this, "doscroll"));
		dojox.grid.funnelEvents(this.contentNode, this, "doContentEvent", [ 'mouseover', 'mouseout', 'click', 'dblclick', 'contextmenu' ]);
		dojox.grid.funnelEvents(this.headerNode, this, "doHeaderEvent", [ 'dblclick', 'mouseover', 'mouseout', 'mousemove', 'mousedown', 'click', 'contextmenu' ]);
		this.content = new dojox.grid.contentBuilder(this);
		this.header = new dojox.grid.headerBuilder(this);
	},
	destroy: function(){
		dojox.grid.removeNode(this.headerNode);
		this.inherited("destroy", arguments);
	},
	// focus 
	focus: function(){
		if(dojo.isSafari || dojo.isOpera){
			this.hiddenFocusNode.focus();
		}else{
			this.scrollboxNode.focus();
		}
	},
	setStructure: function(inStructure){
		var vs = this.structure = inStructure;
		// FIXME: similar logic is duplicated in layout
		if(vs.width && dojo.isNumber(vs.width)){
			this.viewWidth = vs.width + 'em';
		}else{
			this.viewWidth = vs.width || this.viewWidth; //|| this.defaultWidth;
		}
		this.onBeforeRow = vs.onBeforeRow;
		this.noscroll = vs.noscroll;
		if(this.noscroll){
			this.scrollboxNode.style.overflow = "hidden";
		}
		// bookkeeping
		this.testFlexCells();
		// accomodate new structure
		this.updateStructure();
	},
	testFlexCells: function(){
		// FIXME: cheater, this function does double duty as initializer and tester
		this.flexCells = false;
		for(var j=0, row; (row=this.structure.rows[j]); j++){
			for(var i=0, cell; (cell=row[i]); i++){
				cell.view = this;
				this.flexCells = this.flexCells || cell.isFlex();
			}
		}
		return this.flexCells;
	},
	updateStructure: function(){
		// header builder needs to update table map
		this.header.update();
		// content builder needs to update markup cache
		this.content.update();
	},
	getScrollbarWidth: function(){
		return (this.noscroll ? 0 : dojox.grid.getScrollbarWidth());
	},
	getColumnsWidth: function(){
		return this.headerContentNode.firstChild.offsetWidth;
	},
	getWidth: function(){
		return this.viewWidth || (this.getColumnsWidth()+this.getScrollbarWidth()) +'px';
	},
	getContentWidth: function(){
		return Math.max(0, dojo._getContentBox(this.domNode).w - this.getScrollbarWidth()) + 'px';
	},
	render: function(){
		this.scrollboxNode.style.height = '';
		this.renderHeader();
	},
	renderHeader: function(){
		this.headerContentNode.innerHTML = this.header.generateHtml(this._getHeaderContent);
	},
	// note: not called in 'view' context
	_getHeaderContent: function(inCell){
		var n = inCell.name || inCell.grid.getCellName(inCell);
		if(inCell.index != inCell.grid.getSortIndex()){
			return n;
		}
		return [ '<div class="', inCell.grid.sortInfo > 0 ? 'dojoxGrid-sort-down' : 'dojoxGrid-sort-up', '">', n, '</div>' ].join(''); 
	},
	resize: function(){
		this.resizeHeight();
		this.resizeWidth();
	},
	hasScrollbar: function(){
		return (this.scrollboxNode.clientHeight != this.scrollboxNode.offsetHeight);
	},
	resizeHeight: function(){
		if(!this.grid.autoHeight){
			var h = this.domNode.clientHeight;
			if(!this.hasScrollbar()){ // no scrollbar is rendered
				h -= dojox.grid.getScrollbarWidth();
			}
			dojox.grid.setStyleHeightPx(this.scrollboxNode, h);
		}
	},
	resizeWidth: function(){
		if(this.flexCells){
			// the view content width
			this.contentWidth = this.getContentWidth();
			this.headerContentNode.firstChild.style.width = this.contentWidth;
		}
		// FIXME: it should be easier to get w from this.scrollboxNode.clientWidth, 
		// but clientWidth seemingly does not include scrollbar width in some cases
		var w = this.scrollboxNode.offsetWidth - this.getScrollbarWidth();
		w = Math.max(w, this.getColumnsWidth()) + 'px';
		with(this.contentNode){
			style.width = '';
			offsetWidth;
			style.width = w;
		}
	},
	setSize: function(w, h){
		with(this.domNode.style){
			if(w){
				width = w;
			}
			height = (h >= 0 ? h + 'px' : '');
		}
		with(this.headerNode.style){
			if(w){
				width = w;
			}
		}
	},
	renderRow: function(inRowIndex, inHeightPx){
		var rowNode = this.createRowNode(inRowIndex);
		this.buildRow(inRowIndex, rowNode, inHeightPx);
		this.grid.edit.restore(this, inRowIndex);
		return rowNode;
	},
	createRowNode: function(inRowIndex){
		var node = document.createElement("div");
		node.className = this.classTag + '-row';
		node[dojox.grid.gridViewTag] = this;
		node[dojox.grid.rowIndexTag] = inRowIndex;
		this.rowNodes[inRowIndex] = node;
		return node;
	},
	buildRow: function(inRowIndex, inRowNode){
		this.buildRowContent(inRowIndex, inRowNode);
		this.styleRow(inRowIndex, inRowNode);
	},
	buildRowContent: function(inRowIndex, inRowNode){
		inRowNode.innerHTML = this.content.generateHtml(inRowIndex, inRowIndex); 
		if(this.flexCells){
			// FIXME: accessing firstChild here breaks encapsulation
			inRowNode.firstChild.style.width = this.contentWidth;
		}
	},
	rowRemoved:function(inRowIndex){
		this.grid.edit.save(this, inRowIndex);
		delete this.rowNodes[inRowIndex];
	},
	getRowNode: function(inRowIndex){
		return this.rowNodes[inRowIndex];
	},
	getCellNode: function(inRowIndex, inCellIndex){
		var row = this.getRowNode(inRowIndex);
		if(row){
			return this.content.getCellNode(row, inCellIndex);
		}
	},
	// styling
	styleRow: function(inRowIndex, inRowNode){
		inRowNode._style = dojox.grid.getStyleText(inRowNode);
		this.styleRowNode(inRowIndex, inRowNode);
	},
	styleRowNode: function(inRowIndex, inRowNode){
		if(inRowNode){
			this.doStyleRowNode(inRowIndex, inRowNode);
		}
	},
	doStyleRowNode: function(inRowIndex, inRowNode){
		this.grid.styleRowNode(inRowIndex, inRowNode);
	},
	// updating
	updateRow: function(inRowIndex, inHeightPx, inPageNode){
		var rowNode = this.getRowNode(inRowIndex);
		if(rowNode){
			rowNode.style.height = '';
			this.buildRow(inRowIndex, rowNode);
		}
		return rowNode;
	},
	updateRowStyles: function(inRowIndex){
		this.styleRowNode(inRowIndex, this.getRowNode(inRowIndex));
	},
	// scrolling
	lastTop: 0,
	doscroll: function(inEvent){
		this.headerNode.scrollLeft = this.scrollboxNode.scrollLeft;
		// 'lastTop' is a semaphore to prevent feedback-loop with setScrollTop below
		var top = this.scrollboxNode.scrollTop;
		if(top != this.lastTop){
			this.grid.scrollTo(top);
		}
	},
	setScrollTop: function(inTop){
		// 'lastTop' is a semaphore to prevent feedback-loop with doScroll above
		this.lastTop = inTop;
		this.scrollboxNode.scrollTop = inTop;
		return this.scrollboxNode.scrollTop;
	},
	// event handlers (direct from DOM)
	doContentEvent: function(e){
		if(this.content.decorateEvent(e)){
			this.grid.onContentEvent(e);
		}
	},
	doHeaderEvent: function(e){
		if(this.header.decorateEvent(e)){
			this.grid.onHeaderEvent(e);
		}
	},
	// event dispatch(from Grid)
	dispatchContentEvent: function(e){
		return this.content.dispatchEvent(e);
	},
	dispatchHeaderEvent: function(e){
		return this.header.dispatchEvent(e);
	},
	// column resizing
	setColWidth: function(inIndex, inWidth){
		this.grid.setCellWidth(inIndex, inWidth + 'px');
	},
	update: function(){
		var left = this.scrollboxNode.scrollLeft;
		this.content.update();
		this.grid.update();
		this.scrollboxNode.scrollLeft = left;
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.views"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.views"] = true;
dojo.provide("dojox.grid._grid.views");

dojo.declare('dojox.grid.views', null, {
	// summary:
	//	A collection of grid views. Owned by grid and used internally for managing grid views.
	//	Grid creates views automatically based on grid's layout structure.
	//	Users should typically not need to access individual views or the views collection directly.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	defaultWidth: 200,
	views: [],
	// operations
	resize: function(){
		this.onEach("resize");
	},
	render: function(){
		this.onEach("render");
		this.normalizeHeaderNodeHeight();
	},
	// views
	addView: function(inView){
		inView.idx = this.views.length;
		this.views.push(inView);
	},
	destroyViews: function(){
		for (var i=0, v; v=this.views[i]; i++)
			v.destroy();
		this.views = [];
	},
	getContentNodes: function(){
		var nodes = [];
		for(var i=0, v; v=this.views[i]; i++){
			nodes.push(v.contentNode);
		}
		return nodes;
	},
	forEach: function(inCallback){
		for(var i=0, v; v=this.views[i]; i++){
			inCallback(v, i);
		}
	},
	onEach: function(inMethod, inArgs){
		inArgs = inArgs || [];
		for(var i=0, v; v=this.views[i]; i++){
			if(inMethod in v){
				v[inMethod].apply(v, inArgs);
			}
		}
	},
	// layout
	normalizeHeaderNodeHeight: function(){
		var rowNodes = [];
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.headerContentNode.firstChild){
				rowNodes.push(v.headerContentNode)
			};
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	normalizeRowNodeHeights: function(inRowNodes){
		var h = 0; 
		for(var i=0, n, o; (n=inRowNodes[i]); i++){
			h = Math.max(h, (n.firstChild.clientHeight)||(n.firstChild.offsetHeight));
		}
		h = (h >= 0 ? h : 0);
		//
		var hpx = h + 'px';
		for(var i=0, n; (n=inRowNodes[i]); i++){
			if(n.firstChild.clientHeight!=h){
				n.firstChild.style.height = hpx;
			}
		}
		//
		//console.log('normalizeRowNodeHeights ', h);
		//
		// querying the height here seems to help scroller measure the page on IE
		if(inRowNodes&&inRowNodes[0]){
			inRowNodes[0].parentNode.offsetHeight;
		}
	},
	renormalizeRow: function(inRowIndex){
		var rowNodes = [];
		for(var i=0, v, n; (v=this.views[i])&&(n=v.getRowNode(inRowIndex)); i++){
			n.firstChild.style.height = '';
			rowNodes.push(n);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	getViewWidth: function(inIndex){
		return this.views[inIndex].getWidth() || this.defaultWidth;
	},
	measureHeader: function(){
		this.forEach(function(inView){
			inView.headerContentNode.style.height = '';
		});
		var h = 0;
		this.forEach(function(inView){
			//console.log('headerContentNode', inView.headerContentNode.offsetHeight, inView.headerContentNode.offsetWidth);
			h = Math.max(inView.headerNode.offsetHeight, h);
		});
		return h;
	},
	measureContent: function(){
		var h = 0;
		this.forEach(function(inView) {
			h = Math.max(inView.domNode.offsetHeight, h);
		});
		return h;
	},
	findClient: function(inAutoWidth){
		// try to use user defined client
		var c = this.grid.elasticView || -1;
		// attempt to find implicit client
		if(c < 0){
			for(var i=1, v; (v=this.views[i]); i++){
				if(v.viewWidth){
					for(i=1; (v=this.views[i]); i++){
						if(!v.viewWidth){
							c = i;
							break;
						}
					}
					break;
				}
			}
		}
		// client is in the middle by default
		if(c < 0){
			c = Math.floor(this.views.length / 2);
		}
		return c;
	},
	_arrange: function(l, t, w, h){
		var i, v, vw, len = this.views.length;
		// find the client
		var c = (w <= 0 ? len : this.findClient());
		// layout views
		var setPosition = function(v, l, t){
			with(v.domNode.style){
				left = l + 'px';
				top = t + 'px';
			}
			with(v.headerNode.style){
				left = l + 'px';
				top = 0;
			}
		}
		// for views left of the client
		for(i=0; (v=this.views[i])&&(i<c); i++){
			// get width
			vw = this.getViewWidth(i);
			// process boxes
			v.setSize(vw, h);
			setPosition(v, l, t);
			vw = v.domNode.offsetWidth;
			// update position
			l += vw;
		}
		// next view (is the client, i++ == c) 
		i++;
		// start from the right edge
		var r = w;
		// for views right of the client (iterated from the right)
		for(var j=len-1; (v=this.views[j])&&(i<=j); j--){
			// get width
			vw = this.getViewWidth(j);
			// set size
			v.setSize(vw, h);
			// measure in pixels
			vw = v.domNode.offsetWidth;
			// update position
			r -= vw;
			// set position
			setPosition(v, r, t);
		}
		if(c<len){
			v = this.views[c];
			// position the client box between left and right boxes	
			vw = Math.max(1, r-l);
			// set size
			v.setSize(vw + 'px', h);
			setPosition(v, l, t);
		}
		return l;
	},
	arrange: function(l, t, w, h){
		var w = this._arrange(l, t, w, h);
		this.resize();
		return w;
	},
	// rendering
	renderRow: function(inRowIndex, inNodes){
		var rowNodes = [];
		for(var i=0, v, n, rowNode; (v=this.views[i])&&(n=inNodes[i]); i++){
			rowNode = v.renderRow(inRowIndex);
			n.appendChild(rowNode);
			rowNodes.push(rowNode);
		}
		this.normalizeRowNodeHeights(rowNodes);
	},
	rowRemoved: function(inRowIndex){
		this.onEach("rowRemoved", [ inRowIndex ]);
	},
	// updating
	updateRow: function(inRowIndex, inHeight){
		for(var i=0, v; v=this.views[i]; i++){
			v.updateRow(inRowIndex, inHeight);
		}
		this.renormalizeRow(inRowIndex);
	},
	updateRowStyles: function(inRowIndex){
		this.onEach("updateRowStyles", [ inRowIndex ]);
	},
	// scrolling
	setScrollTop: function(inTop){
		var top = inTop;
		for(var i=0, v; v=this.views[i]; i++){
			top = v.setScrollTop(inTop);
		}
		return top;
		//this.onEach("setScrollTop", [ inTop ]);
	},
	getFirstScrollingView: function(){
		for(var i=0, v; (v=this.views[i]); i++){
			if(v.hasScrollbar()){
				return v;
			}
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.cell"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.cell"] = true;
dojo.provide("dojox.grid._grid.cell");

dojo.declare("dojox.grid.cell", null, {
	// summary:
	//	Respresents a grid cell and contains information about column options and methods
	//	for retrieving cell related information.
	//	Each column in a grid layout has a cell object and most events and many methods
	//	provide access to these objects.
	styles: '',
	constructor: function(inProps){
		dojo.mixin(this, inProps);
		if(this.editor){this.editor = new this.editor(this);}
	},
	// data source
	format: function(inRowIndex){
		// summary:
		//	provides the html for a given grid cell.
		// inRowIndex: int
		// grid row index
		// returns: html for a given grid cell
		var f, i=this.grid.edit.info, d=this.get ? this.get(inRowIndex) : this.value;
		if(this.editor && (this.editor.alwaysOn || (i.rowIndex==inRowIndex && i.cell==this))){
			return this.editor.format(d, inRowIndex);
		}else{
			return (f = this.formatter) ? f.call(this, d, inRowIndex) : d;
		}
	},
	// utility
	getNode: function(inRowIndex){
		// summary:
		//	gets the dom node for a given grid cell.
		// inRowIndex: int
		// grid row index
		// returns: dom node for a given grid cell
		return this.view.getCellNode(inRowIndex, this.index);
	},
	isFlex: function(){
		var uw = this.unitWidth;
		return uw && (uw=='auto' || uw.slice(-1)=='%');
	},
	// edit support
	applyEdit: function(inValue, inRowIndex){
		this.grid.edit.applyCellEdit(inValue, this, inRowIndex);
	},
	cancelEdit: function(inRowIndex){
		this.grid.doCancelEdit(inRowIndex);
	},
	_onEditBlur: function(inRowIndex){
		if(this.grid.edit.isEditCell(inRowIndex, this.index)){
			//console.log('editor onblur', e);
			this.grid.edit.apply();
		}
	},
	registerOnBlur: function(inNode, inRowIndex){
		if(this.commitOnBlur){
			dojo.connect(inNode, "onblur", function(e){
				// hack: if editor still thinks this editor is current some ms after it blurs, assume we've focused away from grid
				setTimeout(dojo.hitch(this, "_onEditBlur", inRowIndex), 250);
			});
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.layout"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.layout"] = true;
dojo.provide("dojox.grid._grid.layout");


dojo.declare("dojox.grid.layout", null, {
	// summary:
	//	Controls grid cell layout. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	// flat array of grid cells
	cells: null,
	// structured array of grid cells
	structure: null,
	// default cell width
	defaultWidth: '6em',
	// methods
	setStructure: function(inStructure){
		this.fieldIndex = 0;
		this.cells = [];
		var s = this.structure = [];
		for(var i=0, viewDef, rows; (viewDef=inStructure[i]); i++){
			s.push(this.addViewDef(viewDef));
		}
		this.cellCount = this.cells.length;
	},
	addViewDef: function(inDef){
		this._defaultCellProps = inDef.defaultCell || {};
		return dojo.mixin({}, inDef, {rows: this.addRowsDef(inDef.rows || inDef.cells)});
	},
	addRowsDef: function(inDef){
		var result = [];
		for(var i=0, row; inDef && (row=inDef[i]); i++){
			result.push(this.addRowDef(i, row));
		}
		return result;
	},
	addRowDef: function(inRowIndex, inDef){
		var result = [];
		for(var i=0, def, cell; (def=inDef[i]); i++){
			cell = this.addCellDef(inRowIndex, i, def);
			result.push(cell);
			this.cells.push(cell);
		}
		return result;
	},
	addCellDef: function(inRowIndex, inCellIndex, inDef){
		var w = 0;
		if(inDef.colSpan > 1){
			w = 0;
		}else if(!isNaN(inDef.width)){
			w = inDef.width + "em";
		}else{
			w = inDef.width || this.defaultWidth;
		}
		// fieldIndex progresses linearly from the last indexed field
		// FIXME: support generating fieldIndex based a text field name (probably in Grid)
		var fieldIndex = inDef.field != undefined ? inDef.field : (inDef.get ? -1 : this.fieldIndex);
		if((inDef.field != undefined) || !inDef.get){
			this.fieldIndex = (inDef.field > -1 ? inDef.field : this.fieldIndex) + 1; 
		}
		return new dojox.grid.cell(
			dojo.mixin({}, this._defaultCellProps, inDef, {
				grid: this.grid,
				subrow: inRowIndex,
				layoutIndex: inCellIndex,
				index: this.cells.length,
				fieldIndex: fieldIndex,
				unitWidth: w
			}));
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.rows"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.rows"] = true;
dojo.provide("dojox.grid._grid.rows");

dojo.declare("dojox.grid.rows", null, {
	//	Stores information about grid rows. Owned by grid and used internally.
	constructor: function(inGrid){
		this.grid = inGrid;
	},
	linesToEms: 2,
	defaultRowHeight: 1, // lines
	overRow: -2,
	// metrics
	getHeight: function(inRowIndex){
		return '';
	},
	getDefaultHeightPx: function(){
		// summmary:
		// retrieves the default row height
		// returns: int, default row height
		return 32;
		//return Math.round(this.defaultRowHeight * this.linesToEms * this.grid.contentPixelToEmRatio);
	},
	// styles
	prepareStylingRow: function(inRowIndex, inRowNode){
		return {
			index: inRowIndex, 
			node: inRowNode,
			odd: Boolean(inRowIndex&1),
			selected: this.grid.selection.isSelected(inRowIndex),
			over: this.isOver(inRowIndex),
			customStyles: "",
			customClasses: "dojoxGrid-row"
		}
	},
	styleRowNode: function(inRowIndex, inRowNode){
		var row = this.prepareStylingRow(inRowIndex, inRowNode);
		this.grid.onStyleRow(row);
		this.applyStyles(row);
	},
	applyStyles: function(inRow){
		with(inRow){
			node.className = customClasses;
			var h = node.style.height;
			dojox.grid.setStyleText(node, customStyles + ';' + (node._style||''));
			node.style.height = h;
		}
	},
	updateStyles: function(inRowIndex){
		this.grid.updateRowStyles(inRowIndex);
	},
	// states and events
	setOverRow: function(inRowIndex){
		var last = this.overRow;
		this.overRow = inRowIndex;
		if((last!=this.overRow)&&(last >=0)){
			this.updateStyles(last);
		}
		this.updateStyles(this.overRow);
	},
	isOver: function(inRowIndex){
		return (this.overRow == inRowIndex);
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.focus"] = true;
dojo.provide("dojox.grid._grid.focus");

// focus management
dojo.declare("dojox.grid.focus", null, {
	// summary:
	//	Controls grid cell focus. Owned by grid and used internally for focusing.
	//	Note: grid cell actually receives keyboard input only when cell is being edited.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.cell = null;
		this.rowIndex = -1;
		dojo.connect(this.grid.domNode, "onfocus", this, "doFocus");
	},
	tabbingOut: false,
	focusClass: "dojoxGrid-cell-focus",
	focusView: null,
	initFocusView: function(){
		this.focusView = this.grid.views.getFirstScrollingView();
	},
	isFocusCell: function(inCell, inRowIndex){
		// summary:
		//	states if the given cell is focused
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		// returns:
		//	true of the given grid cell is focused
		return (this.cell == inCell) && (this.rowIndex == inRowIndex);
	},
	isLastFocusCell: function(){
		return (this.rowIndex == this.grid.rowCount-1) && (this.cell.index == this.grid.layout.cellCount-1);
	},
	isFirstFocusCell: function(){
		return (this.rowIndex == 0) && (this.cell.index == 0);
	},
	isNoFocusCell: function(){
		return (this.rowIndex < 0) || !this.cell;
	},
	_focusifyCellNode: function(inBork){
		var n = this.cell && this.cell.getNode(this.rowIndex);
		if(n){
			dojo.toggleClass(n, this.focusClass, inBork);
			this.scrollIntoView();
			try{
				if(!this.grid.edit.isEditing())
					dojox.grid.fire(n, "focus");
			}catch(e){}
		}
	},
	scrollIntoView: function() {
		if(!this.cell){
			return;
		}
		var 
			c = this.cell,
			s = c.view.scrollboxNode,
			sr = {
				w: s.clientWidth,
				l: s.scrollLeft,
				t: s.scrollTop,
				h: s.clientHeight
			},
			n = c.getNode(this.rowIndex),
			r = c.view.getRowNode(this.rowIndex),
			rt = this.grid.scroller.findScrollTop(this.rowIndex);
		// place cell within horizontal view
		if(n.offsetLeft + n.offsetWidth > sr.l + sr.w){
			s.scrollLeft = n.offsetLeft + n.offsetWidth - sr.w;
		}else if(n.offsetLeft < sr.l){
			s.scrollLeft = n.offsetLeft;
		}
		// place cell within vertical view
		if(rt + r.offsetHeight > sr.t + sr.h){
			this.grid.setScrollTop(rt + r.offsetHeight - sr.h);
		}else if(rt < sr.t){
			this.grid.setScrollTop(rt);
		}
	},
	styleRow: function(inRow){
		return;
	},
	setFocusIndex: function(inRowIndex, inCellIndex){
		// summary:
		//	focuses the given grid cell
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		this.setFocusCell(this.grid.getCell(inCellIndex), inRowIndex);
	},
	setFocusCell: function(inCell, inRowIndex){
		// summary:
		//	focuses the given grid cell
		// inCell: object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(inCell && !this.isFocusCell(inCell, inRowIndex)){
			this.tabbingOut = false;
			this.focusGrid();
			this._focusifyCellNode(false);
			this.cell = inCell;
			this.rowIndex = inRowIndex;
			this._focusifyCellNode(true);
		}
		// even if this cell isFocusCell, the document focus may need to be rejiggered
		// call opera on delay to prevent keypress from altering focus
		if(dojo.isOpera){
			setTimeout(dojo.hitch(this.grid, 'onCellFocus', this.cell, this.rowIndex), 1);
		}else{
			this.grid.onCellFocus(this.cell, this.rowIndex);
		}
	},
	next: function(){
		// summary:
		//	focus next grid cell
		var row=this.rowIndex, col=this.cell.index+1, cc=this.grid.layout.cellCount-1, rc=this.grid.rowCount-1;
		if(col > cc){
			col = 0;
			row++;
		}
		if(row > rc){
			col = cc;
			row = rc;
		}
		this.setFocusIndex(row, col);
	},
	previous: function(){
		// summary:
		//	focus previous grid cell
		var row=(this.rowIndex || 0), col=(this.cell.index || 0) - 1;
		if(col < 0){
			col = this.grid.layout.cellCount-1;
			row--;
		}
		if(row < 0){
			row = 0;
			col = 0;
		}
		this.setFocusIndex(row, col);
	},
	move: function(inRowDelta, inColDelta) {
		// summary:
		//	focus grid cell based on position relative to current focus
		// inRowDelta: int
		// vertical distance from current focus
		// inColDelta: int
		// horizontal distance from current focus
		var
			rc = this.grid.rowCount-1,
			cc = this.grid.layout.cellCount-1,
			r = this.rowIndex,
			i = this.cell.index,
			row = Math.min(rc, Math.max(0, r+inRowDelta)),
			col = Math.min(cc, Math.max(0, i+inColDelta));
		this.setFocusIndex(row, col);
		if(inRowDelta){
			this.grid.updateRow(r);
		}
	},
	previousKey: function(e){
		if(this.isFirstFocusCell()){
			this.tabOut(this.grid.domNode);
		}else{
			dojo.stopEvent(e);
			this.previous();
		}
	},
	nextKey: function(e) {
		if(this.isLastFocusCell()){
			this.tabOut(this.grid.lastFocusNode);
		}else{
			dojo.stopEvent(e);
			this.next();
		}
	},
	tabOut: function(inFocusNode){
		this.tabbingOut = true;
		inFocusNode.focus();
	},
	focusGrid: function(){
		dojox.grid.fire(this.focusView, "focus");
		this._focusifyCellNode(true);
	},
	doFocus: function(e){
		// trap focus only for grid dom node
		if(e && e.target != e.currentTarget){
			return;
		}
		// do not focus for scrolling if grid is about to blur
		if(!this.tabbingOut && this.isNoFocusCell()){
			// establish our virtual-focus, if necessary
			this.setFocusIndex(0, 0);
		}
		this.tabbingOut = false;
	}
});

}

if(!dojo._hasResource['dojox.grid._grid.selection']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._grid.selection'] = true;
dojo.provide('dojox.grid._grid.selection');

dojo.declare("dojox.grid.selection", null, {
	// summary:
	//	Manages row selection for grid. Owned by grid and used internally 
	//	for selection. Override to implement custom selection.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.selected = [];
	},
	multiSelect: true,
	selected: null,
	updating: 0,
	selectedIndex: -1,
	onCanSelect: function(inIndex){
		return this.grid.onCanSelect(inIndex);
	},
	onCanDeselect: function(inIndex){
		return this.grid.onCanDeselect(inIndex);
	},
	onSelected: function(inIndex){
		return this.grid.onSelected(inIndex);
	},
	onDeselected: function(inIndex){
		return this.grid.onDeselected(inIndex);
	},
	//onSetSelected: function(inIndex, inSelect) { };
	onChanging: function(){
	},
	onChanged: function(){
		return this.grid.onSelectionChanged();
	},
	isSelected: function(inIndex){
		return this.selected[inIndex];
	},
	getFirstSelected: function(){
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},
	getNextSelected: function(inPrev){
		for(var i=inPrev+1, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				return i;
			}
		}
		return -1;
	},
	getSelected: function(){
		var result = [];
		for(var i=0, l=this.selected.length; i<l; i++){
			if(this.selected[i]){
				result.push(i);
			}
		}
		return result;
	},
	getSelectedCount: function(){
		var c = 0;
		for(var i=0; i<this.selected.length; i++){
			if(this.selected[i]){
				c++;
			}
		}
		return c;
	},
	beginUpdate: function(){
		if(this.updating == 0){
			this.onChanging();
		}
		this.updating++;
	},
	endUpdate: function(){
		this.updating--;
		if(this.updating == 0){
			this.onChanged();
		}
	},
	select: function(inIndex){
		this.unselectAll(inIndex);
		this.addToSelection(inIndex);
	},
	addToSelection: function(inIndex){
		inIndex = Number(inIndex);
		if(this.selected[inIndex]){
			this.selectedIndex = inIndex;
		}else{
			if(this.onCanSelect(inIndex) !== false){
				this.selectedIndex = inIndex;
				this.beginUpdate();
				this.selected[inIndex] = true;
				this.grid.onSelected(inIndex);
				//this.onSelected(inIndex);
				//this.onSetSelected(inIndex, true);
				this.endUpdate();
			}
		}
	},
	deselect: function(inIndex){
		inIndex = Number(inIndex);
		if(this.selectedIndex == inIndex){
			this.selectedIndex = -1;
		}
		if(this.selected[inIndex]){
			if(this.onCanDeselect(inIndex) === false){
				return;
			}
			this.beginUpdate();
			delete this.selected[inIndex];
			this.grid.onDeselected(inIndex);
			//this.onDeselected(inIndex);
			//this.onSetSelected(inIndex, false);
			this.endUpdate();
		}
	},
	setSelected: function(inIndex, inSelect){
		this[(inSelect ? 'addToSelection' : 'deselect')](inIndex);
	},
	toggleSelect: function(inIndex){
		this.setSelected(inIndex, !this.selected[inIndex])
	},
	insert: function(inIndex){
		this.selected.splice(inIndex, 0, false);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex++;
		}
	},
	remove: function(inIndex){
		this.selected.splice(inIndex, 1);
		if(this.selectedIndex >= inIndex){
			this.selectedIndex--;
		}
	},
	unselectAll: function(inExcept){
		for(var i in this.selected){
			if((i!=inExcept)&&(this.selected[i]===true)){
				this.deselect(i);
			}
		}
	},
	shiftSelect: function(inFrom, inTo){
		var s = (inFrom >= 0 ? inFrom : inTo), e = inTo;
		if(s > e){
			e = s;
			s = inTo;
		}
		for(var i=s; i<=e; i++){
			this.addToSelection(i);
		}
	},
	clickSelect: function(inIndex, inCtrlKey, inShiftKey){
		this.beginUpdate();
		if(!this.multiSelect){
			this.select(inIndex);
		}else{
			var lastSelected = this.selectedIndex;
			if(!inCtrlKey){
				this.unselectAll(inIndex);
			}
			if(inShiftKey){
				this.shiftSelect(lastSelected, inIndex);
			}else if(inCtrlKey){
				this.toggleSelect(inIndex);
			}else{
				this.addToSelection(inIndex)
			}
		}
		this.endUpdate();
	},
	clickSelectEvent: function(e){
		this.clickSelect(e.rowIndex, e.ctrlKey, e.shiftKey);
	},
	clear: function(){
		this.beginUpdate();
		this.unselectAll();
		this.endUpdate();
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.edit"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.edit"] = true;
dojo.provide("dojox.grid._grid.edit");

dojo.declare("dojox.grid.edit", null, {
	// summary:
	//	Controls grid cell editing process. Owned by grid and used internally for editing.
	constructor: function(inGrid){
		this.grid = inGrid;
		this.connections = [];
		if(dojo.isIE){
			this.connections.push(dojo.connect(document.body, "onfocus", dojo.hitch(this, "_boomerangFocus")));
		}
	},
	info: {},
	destroy: function(){
		dojo.forEach(this.connections, function(c){
			dojo.disconnect(c);
		});
	},
	cellFocus: function(inCell, inRowIndex){
		// summary:
		//	invoke editing when cell is focused
		// inCell: cell object
		//	grid cell object
		// inRowIndex: int
		//	grid row index
		if(this.grid.singleClickEdit || this.isEditRow(inRowIndex)){
			// if same row or quick editing, edit
			this.setEditCell(inCell, inRowIndex);
		}else{
			// otherwise, apply any pending row edits
			this.apply();
		}
		// if dynamic or static editing...
		if(this.isEditing() || (inCell && (inCell.editor||0).alwaysOn)){
			// let the editor focus itself as needed
			this._focusEditor(inCell, inRowIndex);
		}
	},
	rowClick: function(e){
		if(this.isEditing() && !this.isEditRow(e.rowIndex)){
			this.apply();
		}
	},
	styleRow: function(inRow){
		if(inRow.index == this.info.rowIndex){
			inRow.customClasses += ' dojoxGrid-row-editing';
		}
	},
	dispatchEvent: function(e){
		var c = e.cell, ed = c && c.editor;
		return ed && ed.dispatchEvent(e.dispatch, e);
	},
	// Editing
	isEditing: function(){
		// summary:
		//	indicates editing state of the grid.
		// returns:
		//	 true if grid is actively editing
		return this.info.rowIndex !== undefined;
	},
	isEditCell: function(inRowIndex, inCellIndex){
		// summary:
		//	indicates if the given cell is being edited.
		// inRowIndex: int
		//	grid row index
		// inCellIndex: int
		//	grid cell index
		// returns:
		//	 true if given cell is being edited
		return (this.info.rowIndex === inRowIndex) && (this.info.cell.index == inCellIndex);
	},
	isEditRow: function(inRowIndex){
		// summary:
		//	indicates if the given row is being edited.
		// inRowIndex: int
		//	grid row index
		// returns:
		//	 true if given row is being edited
		return this.info.rowIndex === inRowIndex;
	},
	setEditCell: function(inCell, inRowIndex){
		// summary:
		//	set the given cell to be edited
		// inRowIndex: int
		//	grid row index
		// inCell: object
		//	grid cell object
		if(!this.isEditCell(inRowIndex, inCell.index)){
			this.start(inCell, inRowIndex, this.isEditRow(inRowIndex) || inCell.editor);
		}
	},
	_focusEditor: function(inCell, inRowIndex){
		dojox.grid.fire(inCell.editor, "focus", [inRowIndex]);
	},
	focusEditor: function(){
		if(this.isEditing()){
			this._focusEditor(this.info.cell, this.info.rowIndex);
		}
	},
	// implement fix for focus boomerang effect on IE
	_boomerangWindow: 500,
	_shouldCatchBoomerang: function(){
		return this._catchBoomerang > new Date().getTime();
	},
	_boomerangFocus: function(){
		//console.log("_boomerangFocus");
		if(this._shouldCatchBoomerang()){
			// make sure we don't utterly lose focus
			this.grid.focus.focusGrid();
			// let the editor focus itself as needed
			this.focusEditor();
			// only catch once
			this._catchBoomerang = 0;
		}
	},
	_doCatchBoomerang: function(){
		// give ourselves a few ms to boomerang IE focus effects
		if(dojo.isIE){this._catchBoomerang = new Date().getTime() + this._boomerangWindow;}
	},
	// end boomerang fix API
	start: function(inCell, inRowIndex, inEditing){
		this.grid.beginUpdate();
		this.editorApply();
		if(this.isEditing() && !this.isEditRow(inRowIndex)){
			this.applyRowEdit();
			this.grid.updateRow(inRowIndex);
		}
		if(inEditing){
			this.info = { cell: inCell, rowIndex: inRowIndex };
			this.grid.doStartEdit(inCell, inRowIndex); 
			this.grid.updateRow(inRowIndex);
		}else{
			this.info = {};
		}
		this.grid.endUpdate();
		// make sure we don't utterly lose focus
		this.grid.focus.focusGrid();
		// let the editor focus itself as needed
		this._focusEditor(inCell, inRowIndex);
		// give ourselves a few ms to boomerang IE focus effects
		this._doCatchBoomerang();
	},
	_editorDo: function(inMethod){
		var c = this.info.cell
		//c && c.editor && c.editor[inMethod](c, this.info.rowIndex);
		c && c.editor && c.editor[inMethod](this.info.rowIndex);
	},
	editorApply: function(){
		this._editorDo("apply");
	},
	editorCancel: function(){
		this._editorDo("cancel");
	},
	applyCellEdit: function(inValue, inCell, inRowIndex){
		this.grid.doApplyCellEdit(inValue, inRowIndex, inCell.fieldIndex);
	},
	applyRowEdit: function(){
		this.grid.doApplyEdit(this.info.rowIndex);
	},
	apply: function(){
		// summary:
		//	apply a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorApply();
			this.applyRowEdit();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},
	cancel: function(){
		// summary:
		//	cancel a grid edit
		if(this.isEditing()){
			this.grid.beginUpdate();
			this.editorCancel();
			this.info = {};
			this.grid.endUpdate();
			this.grid.focus.focusGrid();
			this._doCatchBoomerang();
		}
	},
	save: function(inRowIndex, inView){
		// summary:
		//	save the grid editing state
		// inRowIndex: int
		//	grid row index
		// inView: object
		//	grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && (!inView || c.view==inView) && c.editor){
			c.editor.save(c, this.info.rowIndex);
		}
	},
	restore: function(inView, inRowIndex){
		// summary:
		//	restores the grid editing state
		// inRowIndex: int
		//	grid row index
		// inView: object
		//	grid view
		var c = this.info.cell;
		if(this.isEditRow(inRowIndex) && c.view == inView && c.editor){
			c.editor.restore(c, this.info.rowIndex);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.rowbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.rowbar"] = true;
dojo.provide("dojox.grid._grid.rowbar");


dojo.declare('dojox.GridRowView', dojox.GridView, {
	// summary:
	//	Custom grid view. If used in a grid structure, provides a small selectable region for grid rows.
	defaultWidth: "3em",
	noscroll: true,
	padBorderWidth: 2,
	buildRendering: function(){
		this.inherited('buildRendering', arguments);
		this.scrollboxNode.style.overflow = "hidden";
		this.headerNode.style.visibility = "hidden";
	},	
	getWidth: function(){
		return this.viewWidth || this.defaultWidth;
	},
	buildRowContent: function(inRowIndex, inRowNode){
		var w = this.contentNode.offsetWidth - this.padBorderWidth 
		inRowNode.innerHTML = '<table style="width:' + w + 'px;" role="wairole:presentation"><tr><td class="dojoxGrid-rowbar-inner"></td></tr></table>';
	},
	renderHeader: function(){
	},
	resize: function(){
		this.resizeHeight();
	},
	// styling
	doStyleRowNode: function(inRowIndex, inRowNode){
		var n = [ "dojoxGrid-rowbar" ];
		if(this.grid.rows.isOver(inRowIndex)){
			n.push("dojoxGrid-rowbar-over");
		}
		if(this.grid.selection.isSelected(inRowIndex)){
			n.push("dojoxGrid-rowbar-selected");
		}
		inRowNode.className = n.join(" ");
	},
	// event handlers
	domouseover: function(e){
		this.grid.onMouseOverRow(e);
	},
	domouseout: function(e){
		if(!this.isIntraRowEvent(e)){
			this.grid.onMouseOutRow(e);
		}
	}
});

}

if(!dojo._hasResource["dojox.grid._grid.publicEvents"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._grid.publicEvents"] = true;
dojo.provide("dojox.grid._grid.publicEvents");

dojox.grid.publicEvents = {
	// summary:
	//	VirtualGrid mixin that provides default implementations for grid events.
	//	dojo.connect to events to retain default implementation or override them for custom handling.
	
	//cellOverClass: string
	// css class to apply to grid cells over which the cursor is placed.
	cellOverClass: "dojoxGrid-cell-over",
	// top level handlers (more specified handlers below)
	onKeyEvent: function(e){
		this.dispatchKeyEvent(e);
	},
	onContentEvent: function(e){
		this.dispatchContentEvent(e);
	},
	onHeaderEvent: function(e){
		this.dispatchHeaderEvent(e);
	},
	onStyleRow: function(inRow){
		// summary:
		//	Perform row styling on a given row. Called whenever row styling is updated.
		// inRow: object
		// Object containing row state information: selected, true if the row is selcted; over:
		// true of the mouse is over the row; odd: true if the row is odd. Use customClasses and
		// customStyles to control row css classes and styles; both properties are strings.
		with(inRow){
			customClasses += (odd?" dojoxGrid-row-odd":"") + (selected?" dojoxGrid-row-selected":"") + (over?" dojoxGrid-row-over":"");
		}
		this.focus.styleRow(inRow);
		this.edit.styleRow(inRow);
	},
	onKeyDown: function(e){
		// summary:
		// grid key event handler. By default enter begins editing and applies edits, escape cancels and edit,
		// tab, shift-tab, and arrow keys move grid cell focus.
		if(e.altKey || e.ctrlKey || e.metaKey ){
			return;
		}
		switch(e.keyCode){
			case dojo.keys.ESCAPE:
				this.edit.cancel();
				break;
			case dojo.keys.ENTER:
				if (!e.shiftKey) {
					var isEditing = this.edit.isEditing();
					this.edit.apply();
					if(!isEditing){
						this.edit.setEditCell(this.focus.cell, this.focus.rowIndex);
					}
				}
				break;
			case dojo.keys.TAB:
				this.focus[e.shiftKey ? 'previousKey' : 'nextKey'](e);
				break;
			case dojo.keys.LEFT_ARROW:
				if(!this.edit.isEditing()){
				this.focus.move(0, -1);
				}
				break;
			case dojo.keys.RIGHT_ARROW:
				if(!this.edit.isEditing()){
					this.focus.move(0, 1);
				}
				break;
			case dojo.keys.UP_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex != 0){
					dojo.stopEvent(e); 
					this.focus.move(-1, 0);
				}
				break;
			case dojo.keys.DOWN_ARROW:
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.model.count){
					dojo.stopEvent(e);
					this.focus.move(1, 0);
				}
				break;
			case dojo.keys.PAGE_UP: 
				if(!this.edit.isEditing() && this.focus.rowIndex != 0){ 
					dojo.stopEvent(e); 
					if(this.focus.rowIndex != this.scroller.firstVisibleRow+1){ 
						this.focus.move(this.scroller.firstVisibleRow-this.focus.rowIndex, 0); 
					}else{ 
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex-1)); 
						this.focus.move(this.scroller.firstVisibleRow-this.scroller.lastVisibleRow+1, 0); 
					} 
				}
				break; 
			case dojo.keys.PAGE_DOWN: 
				if(!this.edit.isEditing() && this.focus.rowIndex+1 != this.model.count){  
					dojo.stopEvent(e); 
					if(this.focus.rowIndex != this.scroller.lastVisibleRow-1){ 
						this.focus.move(this.scroller.lastVisibleRow-this.focus.rowIndex-1, 0); 
					}else{ 
						this.setScrollTop(this.scroller.findScrollTop(this.focus.rowIndex+1)); 
						this.focus.move(this.scroller.lastVisibleRow-this.scroller.firstVisibleRow-1, 0); 
					} 
				} 
				break; 
		}
	},
	onMouseOver: function(e){
		// summary:
		//	event fired when mouse is over the grid.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOver(e) : this.onCellMouseOver(e);
	},
	onMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of the grid.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		e.rowIndex == -1 ? this.onHeaderCellMouseOut(e) : this.onCellMouseOut(e);
	},
	onMouseOverRow: function(e){
		// summary:
		//	event fired when mouse is over any row (data or header).
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(!this.rows.isOver(e.rowIndex)){
			this.rows.setOverRow(e.rowIndex);
			e.rowIndex == -1 ? this.onHeaderMouseOver(e) : this.onRowMouseOver(e);
		}
	},
	onMouseOutRow: function(e){
		// summary:
		//	event fired when mouse moves out of any row (data or header).
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(this.rows.isOver(-1)){
			this.onHeaderMouseOut(e);
		}else if(!this.rows.isOver(-2)){
			this.rows.setOverRow(-2);
			this.onRowMouseOut(e);
		}
	},
	// cell events
	onCellMouseOver: function(e){
		// summary:
		//	event fired when mouse is over a cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.addClass(e.cellNode, this.cellOverClass);
	},
	onCellMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.removeClass(e.cellNode, this.cellOverClass);
	},
	onCellClick: function(e){
		// summary:
		//	event fired when a cell is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this._click[0] = this._click[1];
		this._click[1] = e;
		if(!this.edit.isEditCell(e.rowIndex, e.cellIndex)){
			this.focus.setFocusCell(e.cell, e.rowIndex);
		}
		this.onRowClick(e);
	},
	onCellDblClick: function(e){
		// summary:
		//	event fired when a cell is double-clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		if(dojo.isIE){
			this.edit.setEditCell(this._click[1].cell, this._click[1].rowIndex);
		}else if(this._click[0].rowIndex != this._click[1].rowIndex){
			this.edit.setEditCell(this._click[0].cell, this._click[0].rowIndex);
		}else{
			this.edit.setEditCell(e.cell, e.rowIndex);
		}
		this.onRowDblClick(e);
	},
	onCellContextMenu: function(e){
		// summary:
		//	event fired when a cell context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onRowContextMenu(e);
	},
	onCellFocus: function(inCell, inRowIndex){
		// summary:
		//	event fired when a cell receives focus.
		// inCell: object
		//	cell object containing properties of the grid column.
		// inRowIndex: int
		//	index of the grid row
		this.edit.cellFocus(inCell, inRowIndex);
	},
	// row events
	onRowClick: function(e){
		// summary:
		//	event fired when a row is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.edit.rowClick(e);
		this.selection.clickSelectEvent(e);
	},
	onRowDblClick: function(e){
		// summary:
		//	event fired when a row is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over a data row.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a data row.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onRowContextMenu: function(e){
		// summary:
		//	event fired when a row context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.stopEvent(e);
	},
	// header events
	onHeaderMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over the grid header.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of the grid header.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellMouseOver: function(e){
		// summary:
		//	event fired when mouse moves over a header cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.addClass(e.cellNode, this.cellOverClass);
	},
	onHeaderCellMouseOut: function(e){
		// summary:
		//	event fired when mouse moves out of a header cell.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.removeClass(e.cellNode, this.cellOverClass);
	},
	onHeaderClick: function(e){
		// summary:
		//	event fired when the grid header is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellClick: function(e){
		// summary:
		//	event fired when a header cell is clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.setSortIndex(e.cell.index);
		this.onHeaderClick(e);
	},
	onHeaderDblClick: function(e){
		// summary:
		//	event fired when the grid header is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
	},
	onHeaderCellDblClick: function(e){
		// summary:
		//	event fired when a header cell is double clicked.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onHeaderDblClick(e);
	},
	onHeaderCellContextMenu: function(e){
		// summary:
		//	event fired when a header cell context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		this.onHeaderContextMenu(e);
	},
	onHeaderContextMenu: function(e){
		// summary:
		//	event fired when the grid header context menu is accessed via mouse right click.
		// e: decorated event object 
		//	contains reference to grid, cell, and rowIndex
		dojo.stopEvent(e);
	},
	// editing
	onStartEdit: function(inCell, inRowIndex){
		// summary:
		//	event fired when editing is started for a given grid cell
		// inCell: object
		//	cell object containing properties of the grid column.
		// inRowIndex: int
		//	index of the grid row
	},
	onApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		// summary:
		//	event fired when editing is applied for a given grid cell
		// inValue: string
		//	value from cell editor
		// inRowIndex: int
		//	index of the grid row
		// inFieldIndex: int
		//	index in the grid's data model
	},
	onCancelEdit: function(inRowIndex){
		// summary:
		//	event fired when editing is cancelled for a given grid cell
		// inRowIndex: int
		//	index of the grid row
	},
	onApplyEdit: function(inRowIndex){
		// summary:
		//	event fired when editing is applied for a given grid row
		// inRowIndex: int
		//	index of the grid row
	},
	onCanSelect: function(inRowIndex){
		// summary:
		//	event to determine if a grid row may be selected
		// inRowIndex: int
		//	index of the grid row
		// returns:
		//	true if the row can be selected
		return true // boolean;
	},
	onCanDeselect: function(inRowIndex){
		// summary:
		//	event to determine if a grid row may be deselected
		// inRowIndex: int
		//	index of the grid row
		// returns:
		//	true if the row can be deselected
		return true // boolean;
	},
	onSelected: function(inRowIndex){
		// summary:
		//	event fired when a grid row is selected
		// inRowIndex: int
		//	index of the grid row
		this.updateRowStyles(inRowIndex);
	},
	onDeselected: function(inRowIndex){
		// summary:
		//	event fired when a grid row is deselected
		// inRowIndex: int
		//	index of the grid row
		this.updateRowStyles(inRowIndex);
	},
	onSelectionChanged: function(){
	}
}

}

if(!dojo._hasResource["dojox.grid.VirtualGrid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.VirtualGrid"] = true;
dojo.provide("dojox.grid.VirtualGrid");












dojo.declare('dojox.VirtualGrid', 
	[ dijit._Widget, dijit._Templated ], 
{
	// summary:
	// 		A grid widget with virtual scrolling, cell editing, complex rows,
	// 		sorting, fixed columns, sizeable columns, etc.
	//
	//	description:
	//		VirtualGrid provides the full set of grid features without any
	//		direct connection to a data store.
	//
	//		The grid exposes a get function for the grid, or optionally
	//		individual columns, to populate cell contents.
	//
	//		The grid is rendered based on its structure, an object describing
	//		column and cell layout.
	//
	//	example:
	//		A quick sample:
	//		
	//		define a get function
	//	|	function get(inRowIndex){ // called in cell context
	//	|		return [this.index, inRowIndex].join(', ');
	//	|	}
	//		
	//		define the grid structure:
	//	|	var structure = [ // array of view objects
	//	|		{ cells: [// array of rows, a row is an array of cells
	//	|			[
	//	|				{ name: "Alpha", width: 6 }, 
	//	|				{ name: "Beta" }, 
	//	|				{ name: "Gamma", get: get }]
	//	|		]}
	//	|	];
	//		
	//	|	<div id="grid" 
	//	|		rowCount="100" get="get" 
	//	|		structure="structure" 
	//	|		dojoType="dojox.VirtualGrid"></div>

	templateString: '<div class="dojoxGrid" hidefocus="hidefocus" role="wairole:grid"><div class="dojoxGrid-master-header" dojoAttachPoint="headerNode"></div><div class="dojoxGrid-master-view" dojoAttachPoint="viewsNode"></div><span dojoAttachPoint="lastFocusNode" tabindex="0"></span></div>',
	// classTag: string
	// css class applied to the grid's domNode
	classTag: 'dojoxGrid',
	get: function(inRowIndex){
		/* summary: Default data getter. 
			description:
				Provides data to display in a grid cell. Called in grid cell context.
				So this.cell.index is the column index.
			inRowIndex: integer
				row for which to provide data
			returns:
				data to display for a given grid cell.
		*/
	},
	// settings
	// rowCount: int
	//	Number of rows to display. 
	rowCount: 5,
	// keepRows: int
	//	Number of rows to keep in the rendering cache.
	keepRows: 75, 
	// rowsPerPage: int
	//	Number of rows to render at a time.
	rowsPerPage: 25,
	// autoWidth: boolean
	//	If autoWidth is true, grid width is automatically set to fit the data.
	autoWidth: false,
	// autoHeight: boolean
	//	If autoHeight is true, grid height is automatically set to fit the data.
	autoHeight: false,
	// autoRender: boolean
	//	If autoRender is true, grid will render itself after initialization.
	autoRender: true,
	// defaultHeight: string
	//	default height of the grid, measured in any valid css unit.
	defaultHeight: '15em',
	// structure: object or string
	//	View layout defintion. Can be set to a layout object, or to the (string) name of a layout object.
	structure: '',
	// elasticView: int
	//	Override defaults and make the indexed grid view elastic, thus filling available horizontal space.
	elasticView: -1,
	// singleClickEdit: boolean
	//	Single-click starts editing. Default is double-click
	singleClickEdit: false,
	// Used to store the last two clicks, to ensure double-clicking occurs based on the intended row
	_click: null,
	// private
	sortInfo: 0,
	themeable: true,
	// initialization
	buildRendering: function(){
		this.inherited(arguments);
		// reset get from blank function (needed for markup parsing) to null, if not changed
		if(this.get == dojox.VirtualGrid.prototype.get){
			this.get = null;
		}
		if(!this.domNode.getAttribute('tabIndex')){
			this.domNode.tabIndex = "0";
		}
		this.createScroller();
		this.createLayout();
		this.createViews();
		this.createManagers();
		dojox.grid.initTextSizePoll();
		this.connect(dojox.grid, "textSizeChanged", "textSizeChanged");
		dojox.grid.funnelEvents(this.domNode, this, 'doKeyEvent', dojox.grid.keyEvents);
		this.connect(this, "onShow", "renderOnIdle");
	},
	postCreate: function(){
		// replace stock styleChanged with one that triggers an update
		this.styleChanged = this._styleChanged;
		this.setStructure(this.structure);
		this._click = [];
	},
	destroy: function(){
		this.domNode.onReveal = null;
		this.domNode.onSizeChange = null;
		this.edit.destroy();
		this.views.destroyViews();
		this.inherited(arguments);
	},
	styleChanged: function(){
		this.setStyledClass(this.domNode, '');
	},
	_styleChanged: function(){
		this.styleChanged();
		this.update();
	},
	textSizeChanged: function(){
		setTimeout(dojo.hitch(this, "_textSizeChanged"), 1);
	},
	_textSizeChanged: function(){
		if(this.domNode){
			this.views.forEach(function(v){
				v.content.update();
			});
			this.render();
		}
	},
	sizeChange: function(){
		dojox.grid.jobs.job(this.id + 'SizeChange', 50, dojo.hitch(this, "update"));
	},
	renderOnIdle: function() {
		setTimeout(dojo.hitch(this, "render"), 1);
	},
	// managers
	createManagers: function(){
		// summary:
		//	create grid managers for various tasks including rows, focus, selection, editing
		
		// row manager
		this.rows = new dojox.grid.rows(this);
		// focus manager
		this.focus = new dojox.grid.focus(this);
		// selection manager
		this.selection = new dojox.grid.selection(this);
		// edit manager
		this.edit = new dojox.grid.edit(this);
	},
	// virtual scroller
	createScroller: function(){
		this.scroller = new dojox.grid.scroller.columns();
		this.scroller.renderRow = dojo.hitch(this, "renderRow");
		this.scroller.removeRow = dojo.hitch(this, "rowRemoved");
	},
	// layout
	createLayout: function(){
		this.layout = new dojox.grid.layout(this);
	},
	// views
	createViews: function(){
		this.views = new dojox.grid.views(this);
		this.views.createView = dojo.hitch(this, "createView");
	},
	createView: function(inClass){
		var c = eval(inClass);
		var view = new c({ grid: this });
		this.viewsNode.appendChild(view.domNode);
		this.headerNode.appendChild(view.headerNode);
		this.views.addView(view);
		return view;
	},
	buildViews: function(){
		for(var i=0, vs; (vs=this.layout.structure[i]); i++){
			this.createView(vs.type || "dojox.GridView").setStructure(vs);
		}
		this.scroller.setContentNodes(this.views.getContentNodes());
	},
	setStructure: function(inStructure){
		// summary:
		//		Install a new structure and rebuild the grid.
		// inStructure: Object
		//		Structure object defines the grid layout and provides various
		//		options for grid views and columns
		//	description:
		//		A grid structure is an array of view objects. A view object can
		//		specify a view type (view class), width, noscroll (boolean flag
		//		for view scrolling), and cells. Cells is an array of objects
		//		corresponding to each grid column. The view cells object is an
		//		array of subrows comprising a single row. Each subrow is an
		//		array of column objects. A column object can have a name,
		//		width, value (default), get function to provide data, styles,
		//		and span attributes (rowSpan, colSpan).

		this.views.destroyViews();
		this.structure = inStructure;
		if((this.structure)&&(dojo.isString(this.structure))){
			this.structure=dojox.grid.getProp(this.structure);
		}
		if(!this.structure){
			this.structure=window["layout"];
		}
		if(!this.structure){
			return;
		}
		this.layout.setStructure(this.structure);
		this._structureChanged();
	},
	_structureChanged: function() {
		this.buildViews();
		if(this.autoRender){
			this.render();
		}
	},
	// sizing
	resize: function(){
		// summary:
		//		Update the grid's rendering dimensions and resize it
		
		// FIXME: If grid is not sized explicitly, sometimes bogus scrollbars 
		// can appear in our container, which may require an extra call to 'resize'
		// to sort out.
		
		// if we have set up everything except the DOM, we cannot resize
		if(!this.domNode.parentNode){
			return;
		}
		// useful measurement
		var padBorder = dojo._getPadBorderExtents(this.domNode);
		// grid height
		if(this.autoHeight){
			this.domNode.style.height = 'auto';
			this.viewsNode.style.height = '';
		}else if(this.flex > 0){
		}else if(this.domNode.clientHeight <= padBorder.h){
			if(this.domNode.parentNode == document.body){
				this.domNode.style.height = this.defaultHeight;
			}else{
				this.fitTo = "parent";
			}
		}
		if(this.fitTo == "parent"){
			var h = dojo._getContentBox(this.domNode.parentNode).h;
			dojo.marginBox(this.domNode, { h: Math.max(0, h) });
		}
		var h = dojo._getContentBox(this.domNode).h;
		if(h == 0){
			// We need to hide the header, since the Grid is essentially hidden.
			this.headerNode.style.display = "none";
		}else{
			// Otherwise, show the header and give it an appropriate height.
			this.headerNode.style.display = "block";
			// header height
			var t = this.views.measureHeader();
			this.headerNode.style.height = t + 'px';
		}
		// content extent
		var l = 1, h = (this.autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
		if(this.autoWidth){
			// grid width set to total width
			this.domNode.style.width = this.views.arrange(l, 0, 0, h) + 'px';
		}else{
			// views fit to our clientWidth
			var w = this.domNode.clientWidth || (this.domNode.offsetWidth - padBorder.w);
			this.views.arrange(l, 0, w, h);
		}
		// virtual scroller height
		this.scroller.windowHeight = h; 
		// default row height (FIXME: use running average(?), remove magic #)
		this.scroller.defaultRowHeight = this.rows.getDefaultHeightPx() + 1;
		this.postresize();
	},
	resizeHeight: function(){
		var t = this.views.measureHeader();
		this.headerNode.style.height = t + 'px';
		// content extent
		var h = (this.autoHeight ? -1 : Math.max(this.domNode.clientHeight - t, 0) || 0);
		//this.views.arrange(0, 0, 0, h);
		this.views.onEach('setSize', [0, h]);
		this.views.onEach('resizeHeight');
		this.scroller.windowHeight = h; 
	},
	// render 
	render: function(){
		// summary:
		//	Render the grid, headers, and views. Edit and scrolling states are reset. To retain edit and 
		// scrolling states, see Update.

		if(!this.domNode){return;}
		//
		this.update = this.defaultUpdate;
		this.scroller.init(this.rowCount, this.keepRows, this.rowsPerPage);
		this.prerender();
		this.setScrollTop(0);
		this.postrender();
	},
	prerender: function(){
		this.views.render();
		this.resize();
	},
	postrender: function(){
		this.postresize();
		this.focus.initFocusView();
		// make rows unselectable
		dojo.setSelectable(this.domNode, false);
	},
	postresize: function(){
		// views are position absolute, so they do not inflate the parent
		if(this.autoHeight){
			this.viewsNode.style.height = this.views.measureContent() + 'px';
		}
	},
	// private, used internally to render rows
	renderRow: function(inRowIndex, inNodes){
		this.views.renderRow(inRowIndex, inNodes);
	},
	// private, used internally to remove rows
	rowRemoved: function(inRowIndex){
		this.views.rowRemoved(inRowIndex);
	},
	invalidated: null,
	updating: false,
	beginUpdate: function(){
		// summary:
		//	Use to make multiple changes to rows while queueing row updating.
		// NOTE: not currently supporting nested begin/endUpdate calls
		this.invalidated = [];
		this.updating = true;
	},
	endUpdate: function(){
		// summary:
		//	Use after calling beginUpdate to render any changes made to rows.
		this.updating = false;
		var i = this.invalidated;
		if(i.all){
			this.update();
		}else if(i.rowCount != undefined){
			this.updateRowCount(i.rowCount);
		}else{
			for(r in i){
				this.updateRow(Number(r));
			}
		}
		this.invalidated = null;
	},
	// update
	defaultUpdate: function(){
		// note: initial update calls render and subsequently this function.
		if(this.updating){
			this.invalidated.all = true;
			return;
		}
		//this.edit.saveState(inRowIndex);
		this.prerender();
		this.scroller.invalidateNodes();
		this.setScrollTop(this.scrollTop);
		this.postrender();
		//this.edit.restoreState(inRowIndex);
	},
	update: function(){
		// summary:
		//	Update the grid, retaining edit and scrolling states.
		this.render();
	},
	updateRow: function(inRowIndex){
		// summary:
		//	Render a single row.
		// inRowIndex: int
		//	index of the row to render
		inRowIndex = Number(inRowIndex);
		if(this.updating){
			this.invalidated[inRowIndex]=true;
			return;
		}
		this.views.updateRow(inRowIndex, this.rows.getHeight(inRowIndex));
		this.scroller.rowHeightChanged(inRowIndex);
	},
	updateRowCount: function(inRowCount){
		//summary: 
		//	Change the number of rows.
		// inRowCount: int
		//	Number of rows in the grid.
		if(this.updating){
			this.invalidated.rowCount = inRowCount;
			return;
		}
		this.rowCount = inRowCount;
		this.scroller.updateRowCount(inRowCount);
		this.setScrollTop(this.scrollTop);
		this.resize();
	},
	updateRowStyles: function(inRowIndex){
		// summary:
		//	Update the styles for a row after it's state has changed.
		
		this.views.updateRowStyles(inRowIndex);
	},
	rowHeightChanged: function(inRowIndex){
		// summary: 
		//	Update grid when the height of a row has changed. Row height is handled automatically as rows
		//	are rendered. Use this function only to update a row's height outside the normal rendering process.
		// inRowIndex: int
		// index of the row that has changed height
		
		this.views.renormalizeRow(inRowIndex);
		this.scroller.rowHeightChanged(inRowIndex);
	},
	// fastScroll: boolean
	//	flag modifies vertical scrolling behavior. Defaults to true but set to false for slower 
	//	scroll performance but more immediate scrolling feedback
	fastScroll: true,
	delayScroll: false,
	// scrollRedrawThreshold: int
	//	pixel distance a user must scroll vertically to trigger grid scrolling.
	scrollRedrawThreshold: (dojo.isIE ? 100 : 50),
	// scroll methods
	scrollTo: function(inTop){
		// summary:
		//	Vertically scroll the grid to a given pixel position
		// inTop: int
		//	vertical position of the grid in pixels
		if(!this.fastScroll){
			this.setScrollTop(inTop);
			return;
		}
		var delta = Math.abs(this.lastScrollTop - inTop);
		this.lastScrollTop = inTop;
		if(delta > this.scrollRedrawThreshold || this.delayScroll){
			this.delayScroll = true;
			this.scrollTop = inTop;
			this.views.setScrollTop(inTop);
			dojox.grid.jobs.job('dojoxGrid-scroll', 200, dojo.hitch(this, "finishScrollJob"));
		}else{
			this.setScrollTop(inTop);
		}
	},
	finishScrollJob: function(){
		this.delayScroll = false;
		this.setScrollTop(this.scrollTop);
	},
	setScrollTop: function(inTop){
		this.scrollTop = this.views.setScrollTop(inTop);
		this.scroller.scroll(this.scrollTop);
	},
	scrollToRow: function(inRowIndex){
		// summary:
		//	Scroll the grid to a specific row.
		// inRowIndex: int
		// grid row index
		this.setScrollTop(this.scroller.findScrollTop(inRowIndex) + 1);
	},
	// styling (private, used internally to style individual parts of a row)
	styleRowNode: function(inRowIndex, inRowNode){
		if(inRowNode){
			this.rows.styleRowNode(inRowIndex, inRowNode);
		}
	},
	// cells
	getCell: function(inIndex){
		// summary:
		//	retrieves the cell object for a given grid column.
		// inIndex: int
		// grid column index of cell to retrieve
		// returns:
		//	a grid cell
		return this.layout.cells[inIndex];
	},
	setCellWidth: function(inIndex, inUnitWidth) {
		this.getCell(inIndex).unitWidth = inUnitWidth;
	},
	getCellName: function(inCell){
		return "Cell " + inCell.index;
	},
	// sorting
	canSort: function(inSortInfo){
		// summary:
		//	determines if the grid can be sorted
		// inSortInfo: int
		//	Sort information, 1-based index of column on which to sort, positive for an ascending sort
		// and negative for a descending sort
		// returns:
		//	true if grid can be sorted on the given column in the given direction
	},
	sort: function(){
	},
	getSortAsc: function(inSortInfo){
		// summary:
		//	returns true if grid is sorted in an ascending direction.
		inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
		return Boolean(inSortInfo > 0);
	},
	getSortIndex: function(inSortInfo){
		// summary:
		//	returns the index of the column on which the grid is sorted
		inSortInfo = inSortInfo == undefined ? this.sortInfo : inSortInfo;
		return Math.abs(inSortInfo) - 1;
	},
	setSortIndex: function(inIndex, inAsc){
		// summary:
		// Sort the grid on a column in a specified direction
		// inIndex: int
		// Column index on which to sort.
		// inAsc: boolean
		// If true, sort the grid in ascending order, otherwise in descending order
		var si = inIndex +1;
		if(inAsc != undefined){
			si *= (inAsc ? 1 : -1);
		} else if(this.getSortIndex() == inIndex){
			si = -this.sortInfo;
		}
		this.setSortInfo(si);
	},
	setSortInfo: function(inSortInfo){
		if(this.canSort(inSortInfo)){
			this.sortInfo = inSortInfo;
			this.sort();
			this.update();
		}
	},
	// DOM event handler
	doKeyEvent: function(e){
		e.dispatch = 'do' + e.type;
		this.onKeyEvent(e);
	},
	// event dispatch
	//: protected
	_dispatch: function(m, e){
		if(m in this){
			return this[m](e);
		}
	},
	dispatchKeyEvent: function(e){
		this._dispatch(e.dispatch, e);
	},
	dispatchContentEvent: function(e){
		this.edit.dispatchEvent(e) || e.sourceView.dispatchContentEvent(e) || this._dispatch(e.dispatch, e);
	},
	dispatchHeaderEvent: function(e){
		e.sourceView.dispatchHeaderEvent(e) || this._dispatch('doheader' + e.type, e);
	},
	dokeydown: function(e){
		this.onKeyDown(e);
	},
	doclick: function(e){
		if(e.cellNode){
			this.onCellClick(e);
		}else{
			this.onRowClick(e);
		}
	},
	dodblclick: function(e){
		if(e.cellNode){
			this.onCellDblClick(e);
		}else{
			this.onRowDblClick(e);
		}
	},
	docontextmenu: function(e){
		if(e.cellNode){
			this.onCellContextMenu(e);
		}else{
			this.onRowContextMenu(e);
		}
	},
	doheaderclick: function(e){
		if(e.cellNode){
			this.onHeaderCellClick(e);
		}else{
			this.onHeaderClick(e);
		}
	},
	doheaderdblclick: function(e){
		if(e.cellNode){
			this.onHeaderCellDblClick(e);
		}else{
			this.onHeaderDblClick(e);
		}
	},
	doheadercontextmenu: function(e){
		if(e.cellNode){
			this.onHeaderCellContextMenu(e);
		}else{
			this.onHeaderContextMenu(e);
		}
	},
	// override to modify editing process
	doStartEdit: function(inCell, inRowIndex){
		this.onStartEdit(inCell, inRowIndex);
	},
	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
	},
	doCancelEdit: function(inRowIndex){
		this.onCancelEdit(inRowIndex);
	},
	doApplyEdit: function(inRowIndex){
		this.onApplyEdit(inRowIndex);
	},
	// row editing
	addRow: function(){
		// summary:
		//	add a row to the grid.
		this.updateRowCount(this.rowCount+1);
	},
	removeSelectedRows: function(){
		// summary:
		//	remove the selected rows from the grid.
		this.updateRowCount(Math.max(0, this.rowCount - this.selection.getSelected().length));
		this.selection.clear();
	}
});

dojo.mixin(dojox.VirtualGrid.prototype, dojox.grid.publicEvents);

}

if(!dojo._hasResource["dojox.grid._data.fields"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._data.fields"] = true;
dojo.provide("dojox.grid._data.fields");

dojo.declare("dojox.grid.data.Mixer", null, {
	// summary:
	//	basic collection class that provides a default value for items
	
	constructor: function(){
		this.defaultValue = {};
		this.values = [];
	},
	count: function(){
		return this.values.length;
	},
	clear: function(){
		this.values = [];
	},
	build: function(inIndex){
		var result = dojo.mixin({owner: this}, this.defaultValue);
		result.key = inIndex;
		this.values[inIndex] = result;
		return result;
	},
	getDefault: function(){
		return this.defaultValue;
	},
	setDefault: function(inField /*[, inField2, ... inFieldN] */){
		for(var i=0, a; (a = arguments[i]); i++){
			dojo.mixin(this.defaultValue, a);
		}
	},
	get: function(inIndex){
		return this.values[inIndex] || this.build(inIndex);
	},
	_set: function(inIndex, inField /*[, inField2, ... inFieldN] */){
		// each field argument can be a single field object of an array of field objects
		var v = this.get(inIndex);
		for(var i=1; i<arguments.length; i++){
			dojo.mixin(v, arguments[i]);
		}
		this.values[inIndex] = v;
	},
	set: function(/* inIndex, inField [, inField2, ... inFieldN] | inArray */){
		if(arguments.length < 1){
			return;
		}
		var a = arguments[0];
		if(!dojo.isArray(a)){
			this._set.apply(this, arguments);
		}else{
			if(a.length && a[0]["default"]){
				this.setDefault(a.shift());
			}
			for(var i=0, l=a.length; i<l; i++){
				this._set(i, a[i]);
			}
		}
	},
	insert: function(inIndex, inProps){
		if (inIndex >= this.values.length){
			this.values[inIndex] = inProps;
		}else{
			this.values.splice(inIndex, 0, inProps);
		}
	},
	remove: function(inIndex){
		this.values.splice(inIndex, 1);
	},
	swap: function(inIndexA, inIndexB){
		dojox.grid.arraySwap(this.values, inIndexA, inIndexB);
	},
	move: function(inFromIndex, inToIndex){
		dojox.grid.arrayMove(this.values, inFromIndex, inToIndex);
	}
});

dojox.grid.data.compare = function(a, b){
	return (a > b ? 1 : (a == b ? 0 : -1));
}

dojo.declare('dojox.grid.data.Field', null, {
	constructor: function(inName){
		this.name = inName;
		this.compare = dojox.grid.data.compare;
	},
	na: dojox.grid.na
});

dojo.declare('dojox.grid.data.Fields', dojox.grid.data.Mixer, {
	constructor: function(inFieldClass){
		var fieldClass = inFieldClass ? inFieldClass : dojox.grid.data.Field;
		this.defaultValue = new fieldClass();
	},
	indexOf: function(inKey){
		for(var i=0; i<this.values.length; i++){
			var v = this.values[i];
			if(v && v.key == inKey){return i;}
		}
		return -1;
	}
});

}

if(!dojo._hasResource['dojox.grid._data.model']){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource['dojox.grid._data.model'] = true;
dojo.provide('dojox.grid._data.model');


dojo.declare("dojox.grid.data.Model", null, {
	// summary:
	//	Base abstract grid data model.
	//	Makes no assumptions about the structure of grid data.
	constructor: function(inFields, inData){
		this.observers = [];
		this.fields = new dojox.grid.data.Fields();
		if(inFields){
			this.fields.set(inFields);
		}
		this.setData(inData);
	},
	count: 0,
	updating: 0,
	// observers 
	observer: function(inObserver, inPrefix){
		this.observers.push({o: inObserver, p: inPrefix||'model' });
	},
	notObserver: function(inObserver){
		for(var i=0, m, o; (o=this.observers[i]); i++){
			if(o.o==inObserver){
				this.observers.splice(i, 1);
				return;
			}
		}
	},
	notify: function(inMsg, inArgs){
		if(!this.isUpdating()){
			var a = inArgs || [];
			for(var i=0, m, o; (o=this.observers[i]); i++){
				m = o.p + inMsg, o = o.o;
				(m in o)&&(o[m].apply(o, a));
			}
		}
	},
	// updates
	clear: function(){
		this.fields.clear();
		this.clearData();
	},
	beginUpdate: function(){
		this.updating++;
	},
	endUpdate: function(){
		if(this.updating){
			this.updating--;
		}
		/*if(this.updating){
			if(!(--this.updating)){
				this.change();
			}
		}
		}*/
	},
	isUpdating: function(){
		return Boolean(this.updating);
	},
	// data
	clearData: function(){
		this.setData(null);
	},
	// observer events
	change: function(){
		this.notify("Change", arguments);
	},
	insertion: function(/* index */){
		this.notify("Insertion", arguments);
		this.notify("Change", arguments);
	},
	removal: function(/* keys */){
		this.notify("Removal", arguments);
		this.notify("Change", arguments);
	},
	// insert
	insert: function(inData /*, index */){
		if(!this._insert.apply(this, arguments)){
			return false;
		}
		this.insertion.apply(this, dojo._toArray(arguments, 1));
		return true;
	},
	// remove
	remove: function(inData /*, index */){
		if(!this._remove.apply(this, arguments)){
			return false;
		}
		this.removal.apply(this, arguments);
		return true;
	},
	// sort
	canSort: function(/* (+|-)column_index+1, ... */){
		return this.sort != null;
	},
	makeComparator: function(inIndices){
		var idx, col, field, result = null;
		for(var i=inIndices.length-1; i>=0; i--){
			idx = inIndices[i];
			col = Math.abs(idx) - 1;
			if(col >= 0){
				field = this.fields.get(col);
				result = this.generateComparator(field.compare, field.key, idx > 0, result);
			}
		}
		return result;
	},
	sort: null,
	dummy: 0
});

dojo.declare("dojox.grid.data.Rows", dojox.grid.data.Model, {
	// observer events
	allChange: function(){
		this.notify("AllChange", arguments);
		this.notify("Change", arguments);
	},
	rowChange: function(){
		this.notify("RowChange", arguments);
	},
	datumChange: function(){
		this.notify("DatumChange", arguments);
	},
	// copyRow: function(inRowIndex); // abstract
	// update
	beginModifyRow: function(inRowIndex){
		if(!this.cache[inRowIndex]){
			this.cache[inRowIndex] = this.copyRow(inRowIndex);
		}
	},
	endModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			var data = this.getRow(inRowIndex);
			if(!dojox.grid.arrayCompare(cache, data)){
				this.update(cache, data, inRowIndex);
			}
			delete this.cache[inRowIndex];
		}
	},
	cancelModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			this.setRow(cache, inRowIndex);
			delete this.cache[inRowIndex];
		}
	},
	generateComparator: function(inCompare, inField, inTrueForAscend, inSubCompare){
		return function(a, b){
			var ineq = inCompare(a[inField], b[inField]);
			return ineq ? (inTrueForAscend ? ineq : -ineq) : inSubCompare && inSubCompare(a, b);
		}
	}
});

dojo.declare("dojox.grid.data.Table", dojox.grid.data.Rows, {
	// summary:
	//	Basic grid data model for static data in the form of an array of rows
	//	that are arrays of cell data
	constructor: function(){
		this.cache = [];
	},
	colCount: 0, // tables introduce cols
	data: null,
	cache: null,
	// morphology
	measure: function(){
		this.count = this.getRowCount();
		this.colCount = this.getColCount();
		this.allChange();
		//this.notify("Measure");
	},
	getRowCount: function(){
		return (this.data ? this.data.length : 0);
	},
	getColCount: function(){
		return (this.data && this.data.length ? this.data[0].length : this.fields.count());
	},
	badIndex: function(inCaller, inDescriptor){
		console.debug('dojox.grid.data.Table: badIndex');
	},
	isGoodIndex: function(inRowIndex, inColIndex){
		return (inRowIndex >= 0 && inRowIndex < this.count && (arguments.length < 2 || (inColIndex >= 0 && inColIndex < this.colCount)));
	},
	// access
	getRow: function(inRowIndex){
		return this.data[inRowIndex];
	},
	copyRow: function(inRowIndex){
		return this.getRow(inRowIndex).slice(0);
	},
	getDatum: function(inRowIndex, inColIndex){
		return this.data[inRowIndex][inColIndex];
	},
	get: function(){
		throw('Plain "get" no longer supported. Use "getRow" or "getDatum".');
	},
	setData: function(inData){
		this.data = (inData || []);
		this.allChange();
	},
	setRow: function(inData, inRowIndex){
		this.data[inRowIndex] = inData;
		this.rowChange(inData, inRowIndex);
		this.change();
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		this.data[inRowIndex][inColIndex] = inDatum;
		this.datumChange(inDatum, inRowIndex, inColIndex);
	},
	set: function(){
		throw('Plain "set" no longer supported. Use "setData", "setRow", or "setDatum".');
	},
	setRows: function(inData, inRowIndex){
		for(var i=0, l=inData.length, r=inRowIndex; i<l; i++, r++){
			this.setRow(inData[i], r);
		}
	},
	// update
	update: function(inOldData, inNewData, inRowIndex){
		//delete this.cache[inRowIndex];	
		//this.setRow(inNewData, inRowIndex);
		return true;
	},
	// insert
	_insert: function(inData, inRowIndex){
		dojox.grid.arrayInsert(this.data, inRowIndex, inData);
		this.count++;
		return true;
	},
	// remove
	_remove: function(inKeys){
		for(var i=inKeys.length-1; i>=0; i--){
			dojox.grid.arrayRemove(this.data, inKeys[i]);
		}
		this.count -= inKeys.length;
		return true;
	},
	// sort
	sort: function(/* (+|-)column_index+1, ... */){
		this.data.sort(this.makeComparator(arguments));
	},
	swap: function(inIndexA, inIndexB){
		dojox.grid.arraySwap(this.data, inIndexA, inIndexB);
		this.rowChange(this.getRow(inIndexA), inIndexA);
		this.rowChange(this.getRow(inIndexB), inIndexB);
		this.change();
	},
	dummy: 0
});

dojo.declare("dojox.grid.data.Objects", dojox.grid.data.Table, {
	constructor: function(inFields, inData, inKey){
		if(!inFields){
			this.autoAssignFields();
		}
	},
	autoAssignFields: function(){
		var d = this.data[0], i = 0;
		for(var f in d){
			this.fields.get(i++).key = f;
		}
	},
	getDatum: function(inRowIndex, inColIndex){
		return this.data[inRowIndex][this.fields.get(inColIndex).key];
	}
});

dojo.declare("dojox.grid.data.Dynamic", dojox.grid.data.Table, {
	// summary:
	//	Grid data model for dynamic data such as data retrieved from a server.
	//	Retrieves data automatically when requested and provides notification when data is received
	constructor: function(){
		this.page = [];
		this.pages = [];
	},
	page: null,
	pages: null,
	rowsPerPage: 100,
	requests: 0,
	bop: -1,
	eop: -1,
	// data
	clearData: function(){
		this.pages = [];
		this.bop = this.eop = -1;
		this.setData([]);
	},
	getRowCount: function(){
		return this.count;
	},
	getColCount: function(){
		return this.fields.count();
	},
	setRowCount: function(inCount){
		this.count = inCount;
		this.change();
	},
	// paging
	requestsPending: function(inBoolean){
	},
	rowToPage: function(inRowIndex){
		return (this.rowsPerPage ? Math.floor(inRowIndex / this.rowsPerPage) : inRowIndex);
	},
	pageToRow: function(inPageIndex){
		return (this.rowsPerPage ? this.rowsPerPage * inPageIndex : inPageIndex);
	},
	requestRows: function(inRowIndex, inCount){
		// summary:
		//		stub. Fill in to perform actual data row fetching logic. The
		//		returning logic must provide the data back to the system via
		//		setRow
	},
	rowsProvided: function(inRowIndex, inCount){
		this.requests--;
		if(this.requests == 0){
			this.requestsPending(false);
		}
	},
	requestPage: function(inPageIndex){
		var row = this.pageToRow(inPageIndex);
		var count = Math.min(this.rowsPerPage, this.count - row);
		if(count > 0){
			this.requests++;
			this.requestsPending(true);
			setTimeout(dojo.hitch(this, "requestRows", row, count), 1);
			//this.requestRows(row, count);
		}
	},
	needPage: function(inPageIndex){
		if(!this.pages[inPageIndex]){
			this.pages[inPageIndex] = true;
			this.requestPage(inPageIndex);
		}
	},
	preparePage: function(inRowIndex, inColIndex){
		if(inRowIndex < this.bop || inRowIndex >= this.eop){
			var pageIndex = this.rowToPage(inRowIndex);
			this.needPage(pageIndex);
			this.bop = pageIndex * this.rowsPerPage;
			this.eop = this.bop + (this.rowsPerPage || this.count);
		}
	},
	isRowLoaded: function(inRowIndex){
		return Boolean(this.data[inRowIndex]);
	},
	// removal
	removePages: function(inRowIndexes){
		for(var i=0, r; ((r=inRowIndexes[i]) != undefined); i++){
			this.pages[this.rowToPage(r)] = false;
		}
		this.bop = this.eop =-1;
	},
	remove: function(inRowIndexes){
		this.removePages(inRowIndexes);
		dojox.grid.data.Table.prototype.remove.apply(this, arguments);
	},
	// access
	getRow: function(inRowIndex){
		var row = this.data[inRowIndex];
		if(!row){
			this.preparePage(inRowIndex);
		}
		return row;
	},
	getDatum: function(inRowIndex, inColIndex){
		var row = this.getRow(inRowIndex);
		return (row ? row[inColIndex] : this.fields.get(inColIndex).na);
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		var row = this.getRow(inRowIndex);
		if(row){
			row[inColIndex] = inDatum;
			this.datumChange(inDatum, inRowIndex, inColIndex);
		}else{
			console.debug('[' + this.declaredClass + '] dojox.grid.data.dynamic.set: cannot set data on an non-loaded row');
		}
	},
	// sort
	canSort: function(){
		return false;
	}
});

// FIXME: deprecated: (included for backward compatibility only)
dojox.grid.data.table = dojox.grid.data.Table;
dojox.grid.data.dynamic = dojox.grid.data.Dyanamic;

// we treat dojo.data stores as dynamic stores because no matter how they got
// here, they should always fill that contract
dojo.declare("dojox.grid.data.DojoData", dojox.grid.data.Dynamic, {
	//	summary:
	//		A grid data model for dynamic data retreived from a store which
	//		implements the dojo.data API set. Retrieves data automatically when
	//		requested and provides notification when data is received
	//	description:
	//		This store subclasses the Dynamic grid data object in order to
	//		provide paginated data access support, notification and view
	//		updates for stores which support those features, and simple
	//		field/column mapping for all dojo.data stores.
	constructor: function(inFields, inData, args){
		this.count = 1;
		this._rowIdentities = {};
		this._currentlyProcessing = [];
		if(args){
			dojo.mixin(this, args);
		}
		if(this.store){
			var f = this.store.getFeatures();
			this._canNotify = f['dojo.data.api.Notification'];
			this._canWrite = f['dojo.data.api.Write'];
			this._canIdentify = f['dojo.data.api.Identity'];
			if(this._canNotify){
				dojo.connect(this.store, "onSet", this, "_storeDatumChange");
				dojo.connect(this.store, "onDelete", this, "_storeDatumDelete");
				dojo.connect(this.store, "onNew", this, "_storeDatumNew");
			}
			if(this._canWrite) {
				dojo.connect(this.store, "revert", this, "refresh");
			}
		}
	},
	markupFactory: function(args, node){
		return new dojox.grid.data.DojoData(null, null, args);
	},
	query: { name: "*" }, // default, stupid query
	store: null,
	_currentlyProcessing: null,
	_canNotify: false,
	_canWrite: false,
	_canIdentify: false,
	_rowIdentities: {},
	clientSort: false,
	sortFields: null,
	queryOptions: null,

	// data
	setData: function(inData){
		this.store = inData;
		this.data = [];
		this.allChange();
	},
	setRowCount: function(inCount){
		//console.debug("inCount:", inCount);
		this.count = inCount;
		this.allChange();
	},
	beginReturn: function(inCount){
		if(this.count != inCount){
			// this.setRowCount(0);
			// this.clear();
			// console.debug(this.count, inCount);
			this.setRowCount(inCount);
		}
	},
	_setupFields: function(dataItem){
		// abort if we already have setup fields
		if(this.fields._nameMaps){
			return;
		}
		// set up field/index mappings
		var m = {};
		//console.debug("setting up fields", m);
		var fields = dojo.map(this.store.getAttributes(dataItem),
			function(item, idx){ 
				m[item] = idx;
				m[idx+".idx"] = item;
				// name == display name, key = property name
				return { name: item, key: item };
			},
			this
		);
		this.fields._nameMaps = m;
		// console.debug("new fields:", fields);
		this.fields.set(fields);
		this.notify("FieldsChange");
	},
	_getRowFromItem: function(item){
		// gets us the row object (and row index) of an item
	},
	_createRow: function(item){
		var row = {}; 
		row.__dojo_data_item = item;
		dojo.forEach(this.fields.values, function(a){
			value = this.store.getValue(item, a.name);
			row[a.name] = (value === undefined || value === null)?"":value;
		}, this);
		return row;
	},
	processRows: function(items, request){
		// console.debug(arguments);
		if(!items || items.length == 0){ return; }
		this._setupFields(items[0]);
		dojo.forEach(items, function(item, idx){
			var row = this._createRow(item);
			this._setRowId(item, request.start, idx);
			this.setRow(row, request.start+idx);
		}, this);
		// FIXME: 
		//	Q: scott, steve, how the hell do we actually get this to update
		//		the visible UI for these rows?
		//	A: the goal is that Grid automatically updates to reflect changes
		//		in model. In this case, setRow -> rowChanged -> (observed by) Grid -> modelRowChange -> updateRow
	},
	// request data 
	requestRows: function(inRowIndex, inCount){
		var row  = inRowIndex || 0;
		var params = { 
			start: row,
			count: this.rowsPerPage,
			query: this.query,
			sort: this.sortFields,
			queryOptions: this.queryOptions,
			onBegin: dojo.hitch(this, "beginReturn"),
			onComplete: dojo.hitch(this, "processRows"), // add to deferred?
			onError: dojo.hitch(this, "processError")
		};
		this.store.fetch(params);
	},
	getDatum: function(inRowIndex, inColIndex){
		//console.debug("getDatum", inRowIndex, inColIndex);
		var row = this.getRow(inRowIndex);
		var field = this.fields.values[inColIndex];
		return row && field ? row[field.name] : field ? field.na : '?';
		//var idx = row && this.fields._nameMaps[inColIndex+".idx"];
		//return (row ? row[idx] : this.fields.get(inColIndex).na);
	},
	setDatum: function(inDatum, inRowIndex, inColIndex){
		var n = this.fields._nameMaps[inColIndex+".idx"];
		// console.debug("setDatum:", "n:"+n, inDatum, inRowIndex, inColIndex);
		if(n){
			this.data[inRowIndex][n] = inDatum;
			this.datumChange(inDatum, inRowIndex, inColIndex);
		}
	},
	// modification, update and store eventing
	copyRow: function(inRowIndex){
		var row = {};
		var backstop = {};
		var src = this.getRow(inRowIndex);
		for(var x in src){
			if(src[x] != backstop[x]){
				row[x] = src[x];
			}
		}
		return row;
	},
	_attrCompare: function(cache, data){
		dojo.forEach(this.fields.values, function(a){
			if(cache[a.name] != data[a.name]){ return false; }
		}, this);
		return true;
	},
	endModifyRow: function(inRowIndex){
		var cache = this.cache[inRowIndex];
		if(cache){
			var data = this.getRow(inRowIndex);
			if(!this._attrCompare(cache, data)){
				this.update(cache, data, inRowIndex);
			}
			delete this.cache[inRowIndex];
		}
	},
	cancelModifyRow: function(inRowIndex){
		// console.debug("cancelModifyRow", arguments);
		var cache = this.cache[inRowIndex];
		if(cache){
			this.setRow(cache, inRowIndex);
			delete this.cache[inRowIndex];
		}
	},
	_setRowId: function(item, offset, idx){
		// FIXME: where else do we need to keep this in sync?
		//Handle stores that implement identity and try to handle those that do not.
		if (this._canIdentify) {
			this._rowIdentities[this.store.getIdentity(item)] = {rowId: offset+idx, item: item};
		}else{
			var identity = dojo.toJson(this.query) + ":start:" + offset + ":idx:" + idx + ":sort:" + dojo.toJson(this.sortFields);
			this._rowIdentities[identity] = {rowId: offset+idx, item: item};
		}
	},
	_getRowId: function(item, isNotItem){
		//	summary:
		//		Function determine the row index for a particular item
		//	item:
		//		The store item to examine to determine row index.
		//	isNotItem:
		//		Boolean flag to indicate if the item passed is a store item or not.
		var rowId = null;
		//Handle identity and nonidentity capable stores.
		if(this._canIdentify && !isNotItem){
			rowId = this._rowIdentities[this.store.getIdentity(item)].rowId;
		}else{
			//Not efficient, but without identity support, 
			//not a better way to do it.  Basically, do our best to locate it
			//This may or may not work, but best we can do here.
			var id;
			for(id in this._rowIdentities){
				if(this._rowIdentities[id].item === item){
					rowId = this._rowIdentities[id].rowId;
					break;
				}
			}
		}
		return rowId;
	},
	_storeDatumChange: function(item, attr, oldVal, newVal){
		// the store has changed some data under us, need to update the display
		var rowId = this._getRowId(item);
		var row = this.getRow(rowId);
		if(row){
			row[attr] = newVal;
			var colId = this.fields._nameMaps[attr];
			this.notify("DatumChange", [ newVal, rowId, colId ]);
		}
	},
	_storeDatumDelete: function(item){
		if(dojo.indexOf(this._currentlyProcessing, item) != -1)
			return;
		// the store has deleted some item under us, need to remove that item from
		// the view if possible.  It may be the deleted item isn't even in the grid.
		var rowId = this._getRowId(item, true);
		if(rowId != null){
			this._removeItems([rowId]);
		}
	},
	_storeDatumNew: function(item){
		if(this._disableNew)
			return;
		// the store has added some item under us, need to add it to the view.
		this._insertItem(item, this.data.length);
	},
	insert: function(item, index){
		// Push the given item back to the store
		this._disableNew = true;
		var i = this.store.newItem(item);
		this._disableNew = false;
		this._insertItem(i, index);
	},
	_insertItem: function(storeItem, index){
		var row = this._createRow(storeItem);
		this._setRowId(storeItem, 0, index);
		dojox.grid.data.Dynamic.prototype.insert.apply(this, [row, index]);
	},
	datumChange: function(value, rowIdx, colIdx){
		if(this._canWrite){
			// we're chaning some data, which means we need to write back
			var row = this.getRow(rowIdx);
			var field = this.fields._nameMaps[colIdx+".idx"];
			this.store.setValue(row.__dojo_data_item, field, value);
			// we don't need to call DatumChange, an eventing store will tell
			// us about the row change events
		}else{
			// we can't write back, so just go ahead and change our local copy
			// of the data
			this.notify("DatumChange", arguments);
		}
	},
	insertion: function(/* index */){
		console.debug("Insertion", arguments);
		this.notify("Insertion", arguments);
		this.notify("Change", arguments);
	},
	removal: function(/* keys */){
		console.debug("Removal", arguments);
		this.notify("Removal", arguments);
		this.notify("Change", arguments);
	},
	remove: function(inRowIndexes){
		//	summary:
		//		Function to remove a set of items from the store based on the row index.
		//	inRowIndexes:
		//		An array of row indexes from the grid to remove from the store.
		/* Call delete on the store */ 
		for(var i=inRowIndexes.length-1; i>=0; i--){
			// Need to find the item, then remove each from the data store
			var item = this.data[inRowIndexes[i]].__dojo_data_item;
			this._currentlyProcessing.push(item);
			this.store.deleteItem(item);
		}
		/* Remove from internal data structure and the view */
		this._removeItems(inRowIndexes);
		this._currentlyProcessing = [];
	},
	_removeItems: function(inRowIndexes /*array*/){
		//	summary:
		//		Function to remove a set of items from the store based on the row index.
		//	inRowIndexes:
		//		An array of row indexes from the grid to remove from the store.
		dojox.grid.data.Dynamic.prototype.remove.apply(this, arguments);
		// Rebuild _rowIdentities
		this._rowIdentities = {};
		for (var i = 0; i < this.data.length; i++){
			this._setRowId(this.data[i].__dojo_data_item, 0, i);
		}
	},
	canSort: function(){
		// Q: Return true and re-issue the queries?
		// A: Return true only. Re-issue the query in 'sort'.
		// Note, above are original comments :)
		return true;
	},
	sort: function(colIndex){
		var col = Math.abs(colIndex) - 1;
		this.sortFields = [{'attribute': this.fields.values[col].name, 'descending': (colIndex>0)}];
		
		// Since we're relying on the data store to sort, we have to refresh our data.
		this.refresh();
	},
	refresh: function(){
		//	summary:
		//		Function to cause the model to re-query the store and rebuild the current viewport.
		this.clearData(true);
		this.requestRows();
	},
	clearData: function(/* boolean */ keepStore){
		this._rowIdentities = {};
		this.pages = [];
		this.bop = this.eop = -1;
		this.count = 0;
		this.setData((keepStore?this.store:[]));
	},
	processError: function(error, request){
		//	summary:
		//		Hook function to trap error messages from the store and emit them.  
		//		Intended for connecting to and handling the error object or at least reporting it.
		//
		//	error:
		//		The error object returned by the store when a problem occurred.
		//	request:
		//		The request object that caused the error.
		console.log(error);
	}
});

}

if(!dojo._hasResource["dojox.grid._data.editors"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid._data.editors"] = true;
dojo.provide("dojox.grid._data.editors");
dojo.provide("dojox.grid.editors");

dojo.declare("dojox.grid.editors.Base", null, {
	// summary:
	//	base grid editor class. Other grid editors should inherited from this class.
	constructor: function(inCell){
		this.cell = inCell;
	},
	//private
	_valueProp: "value",
	_formatPending: false,
	format: function(inDatum, inRowIndex){
		// summary:
		//	formats the cell for editing
		// inDatum: anything
		//	cell data to edit
		// inRowIndex: int
		//	grid row index
		// returns: string of html to place in grid cell
	},
	//protected
	needFormatNode: function(inDatum, inRowIndex){
		this._formatPending = true;
		dojox.grid.whenIdle(this, "_formatNode", inDatum, inRowIndex);
	},
	cancelFormatNode: function(){
		this._formatPending = false;
	},
	//private
	_formatNode: function(inDatum, inRowIndex){
		if(this._formatPending){
			this._formatPending = false;
			// make cell selectable
			dojo.setSelectable(this.cell.grid.domNode, true);
			this.formatNode(this.getNode(inRowIndex), inDatum, inRowIndex);
		}
	},
	//protected
	getNode: function(inRowIndex){
		return (this.cell.getNode(inRowIndex) || 0).firstChild || 0;
	},
	formatNode: function(inNode, inDatum, inRowIndex){
		// summary:
		//	format the editing dom node. Use when editor is a widget.
		// inNode: dom node
		// dom node for the editor
		// inDatum: anything
		//	cell data to edit
		// inRowIndex: int
		//	grid row index
		if(dojo.isIE){
			// IE sux bad
			dojox.grid.whenIdle(this, "focus", inRowIndex, inNode);
		}else{
			this.focus(inRowIndex, inNode);
		}
	},
	dispatchEvent: function(m, e){
		if(m in this){
			return this[m](e);
		}
	},
	//public
	getValue: function(inRowIndex){
		// summary:
		//	returns value entered into editor
		// inRowIndex: int
		// grid row index
		// returns:
		//	value of editor
		return this.getNode(inRowIndex)[this._valueProp];
	},
	setValue: function(inRowIndex, inValue){
		// summary:
		//	set the value of the grid editor
		// inRowIndex: int
		// grid row index
		// inValue: anything
		//	value of editor
		var n = this.getNode(inRowIndex);
		if(n){
			n[this._valueProp] = inValue
		};
	},
	focus: function(inRowIndex, inNode){
		// summary:
		//	focus the grid editor
		// inRowIndex: int
		// grid row index
		// inNode: dom node
		//	editor node
		dojox.grid.focusSelectNode(inNode || this.getNode(inRowIndex));
	},
	save: function(inRowIndex){
		// summary:
		//	save editor state
		// inRowIndex: int
		// grid row index
		this.value = this.value || this.getValue(inRowIndex);
		//console.log("save", this.value, inCell.index, inRowIndex);
	},
	restore: function(inRowIndex){
		// summary:
		//	restore editor state
		// inRowIndex: int
		// grid row index
		this.setValue(inRowIndex, this.value);
		//console.log("restore", this.value, inCell.index, inRowIndex);
	},
	//protected
	_finish: function(inRowIndex){
		// summary:
		//	called when editing is completed to clean up editor
		// inRowIndex: int
		// grid row index
		dojo.setSelectable(this.cell.grid.domNode, false);
		this.cancelFormatNode(this.cell);
	},
	//public
	apply: function(inRowIndex){
		// summary:
		//	apply edit from cell editor
		// inRowIndex: int
		// grid row index
		this.cell.applyEdit(this.getValue(inRowIndex), inRowIndex);
		this._finish(inRowIndex);
	},
	cancel: function(inRowIndex){
		// summary:
		//	cancel cell edit
		// inRowIndex: int
		// grid row index
		this.cell.cancelEdit(inRowIndex);
		this._finish(inRowIndex);
	}
});
dojox.grid.editors.base = dojox.grid.editors.Base; // back-compat

dojo.declare("dojox.grid.editors.Input", dojox.grid.editors.Base, {
	// summary
	// grid cell editor that provides a standard text input box
	constructor: function(inCell){
		this.keyFilter = this.keyFilter || this.cell.keyFilter;
	},
	// keyFilter: object
	// optional regex for disallowing keypresses
	keyFilter: null,
	format: function(inDatum, inRowIndex){
		this.needFormatNode(inDatum, inRowIndex);
		return '<input class="dojoxGrid-input" type="text" value="' + inDatum + '">';
	},
	formatNode: function(inNode, inDatum, inRowIndex){
		this.inherited(arguments);
		// FIXME: feels too specific for this interface
		this.cell.registerOnBlur(inNode, inRowIndex);
	},
	doKey: function(e){
		if(this.keyFilter){
			var key = String.fromCharCode(e.charCode);
			if(key.search(this.keyFilter) == -1){
				dojo.stopEvent(e);
			}
		}
	},
	_finish: function(inRowIndex){
		this.inherited(arguments);
		var n = this.getNode(inRowIndex);
		try{
			dojox.grid.fire(n, "blur");
		}catch(e){}
	}
});
dojox.grid.editors.input = dojox.grid.editors.Input; // back compat

dojo.declare("dojox.grid.editors.Select", dojox.grid.editors.Input, {
	// summary:
	// grid cell editor that provides a standard select
	// options: text of each item
	// values: value for each item
	// returnIndex: editor returns only the index of the selected option and not the value
	constructor: function(inCell){
		this.options = this.options || this.cell.options;
		this.values = this.values || this.cell.values || this.options;
	},
	format: function(inDatum, inRowIndex){
		this.needFormatNode(inDatum, inRowIndex);
		var h = [ '<select class="dojoxGrid-select">' ];
		for (var i=0, o, v; (o=this.options[i])&&(v=this.values[i]); i++){
			h.push("<option", (inDatum==o ? ' selected' : ''), /*' value="' + v + '"',*/ ">", o, "</option>");
		}
		h.push('</select>');
		return h.join('');
	},
	getValue: function(inRowIndex){
		var n = this.getNode(inRowIndex);
		if(n){
			var i = n.selectedIndex, o = n.options[i];
			return this.cell.returnIndex ? i : o.value || o.innerHTML;
		}
	}
});
dojox.grid.editors.select = dojox.grid.editors.Select; // back compat

dojo.declare("dojox.grid.editors.AlwaysOn", dojox.grid.editors.Input, {
	// summary:
	// grid cell editor that is always on, regardless of grid editing state
	// alwaysOn: boolean
	// flag to use editor to format grid cell regardless of editing state.
	alwaysOn: true,
	_formatNode: function(inDatum, inRowIndex){
		this.formatNode(this.getNode(inRowIndex), inDatum, inRowIndex);
	},
	applyStaticValue: function(inRowIndex){
		var e = this.cell.grid.edit;
		e.applyCellEdit(this.getValue(inRowIndex), this.cell, inRowIndex);
		e.start(this.cell, inRowIndex, true);
	}
});
dojox.grid.editors.alwaysOn = dojox.grid.editors.AlwaysOn; // back-compat

dojo.declare("dojox.grid.editors.Bool", dojox.grid.editors.AlwaysOn, {
	// summary:
	// grid cell editor that provides a standard checkbox that is always on
	_valueProp: "checked",
	format: function(inDatum, inRowIndex){
		return '<input class="dojoxGrid-input" type="checkbox"' + (inDatum ? ' checked="checked"' : '') + ' style="width: auto" />';
	},
	doclick: function(e){
		if(e.target.tagName == 'INPUT'){
			this.applyStaticValue(e.rowIndex);
		}
	}
});
dojox.grid.editors.bool = dojox.grid.editors.Bool; // back-compat

}

if(!dojo._hasResource["dojox.grid.Grid"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.grid.Grid"] = true;
dojo.provide("dojox.grid.Grid");




dojo.declare('dojox.Grid', dojox.VirtualGrid, {
	//	summary:
	//		A grid widget with virtual scrolling, cell editing, complex rows,
	//		sorting, fixed columns, sizeable columns, etc.
	//	description:
	//		Grid is a subclass of VirtualGrid, providing binding to a data
	//		store.
	//	example:
	//		define the grid structure:
	//	|	var structure = [ // array of view objects
	//	|		{ cells: [// array of rows, a row is an array of cells
	//	|			[	{ name: "Alpha", width: 6 }, 
	//	|				{ name: "Beta" }, 
	//	|				{ name: "Gamma", get: formatFunction }
	//	|			]
	//	|		]}
	//	|	];
	//	  	
	//		define a grid data model
	//	|	var model = new dojox.grid.data.table(null, data);
	//	|
	//	|	<div id="grid" model="model" structure="structure" 
	//	|		dojoType="dojox.VirtualGrid"></div>
	//	

	//	model:
	//		string or object grid data model
	model: 'dojox.grid.data.Table',
	// life cycle
	postCreate: function(){
		if(this.model){
			var m = this.model;
			if(dojo.isString(m)){
				m = dojo.getObject(m);
			}
			this.model = (dojo.isFunction(m)) ? new m() : m;
			this._setModel(this.model);
		}
		this.inherited(arguments);
	},
	destroy: function(){
		this.setModel(null);
		this.inherited(arguments);
	},
	// structure
	_structureChanged: function() {
		this.indexCellFields();
		this.inherited(arguments);
	},
	// model
	_setModel: function(inModel){
		// if(!inModel){ return; }
		this.model = inModel;
		if(this.model){
			this.model.observer(this);
			this.model.measure();
			this.indexCellFields();
		}
	},
	setModel: function(inModel){
		// summary:
		//		set the grid's data model
		// inModel:
		//		model object, usually an instance of a dojox.grid.data.Model
		//		subclass
		if(this.model){
			this.model.notObserver(this);
		}
		this._setModel(inModel);
	},
	// data socket (called in cell's context)
	get: function(inRowIndex){
		return this.grid.model.getDatum(inRowIndex, this.fieldIndex);
	},
	// model modifications
	modelAllChange: function(){
		this.rowCount = (this.model ? this.model.getRowCount() : 0);
		this.updateRowCount(this.rowCount);
	},
	modelRowChange: function(inData, inRowIndex){
		this.updateRow(inRowIndex);
	},
	modelDatumChange: function(inDatum, inRowIndex, inFieldIndex){
		this.updateRow(inRowIndex);
	},
	modelFieldsChange: function() {
		this.indexCellFields();
		this.render();
	},
	// model insertion
	modelInsertion: function(inRowIndex){
		this.updateRowCount(this.model.getRowCount());
	},
	// model removal
	modelRemoval: function(inKeys){
		this.updateRowCount(this.model.getRowCount());
	},
	// cells
	getCellName: function(inCell){
		var v = this.model.fields.values, i = inCell.fieldIndex;
		return i>=0 && i<v.length && v[i].name || this.inherited(arguments);
	},
	indexCellFields: function(){
		var cells = this.layout.cells;
		for(var i=0, c; cells && (c=cells[i]); i++){
			if(dojo.isString(c.field)){
				c.fieldIndex = this.model.fields.indexOf(c.field);
			}
		}
	},
	// utility
	refresh: function(){
		// summary:
		//	re-render the grid, getting new data from the model
		this.edit.cancel();
		this.model.measure();
	},
	// sorting
	canSort: function(inSortInfo){
		var f = this.getSortField(inSortInfo);
		// 0 is not a valid sort field
		return f && this.model.canSort(f);
	},
	getSortField: function(inSortInfo){
		// summary:
		//	retrieves the model field on which to sort data.
		// inSortInfo: int
		//	1-based grid column index; positive if sort is ascending, otherwise negative
		var c = this.getCell(this.getSortIndex(inSortInfo));
		// we expect c.fieldIndex == -1 for non model fields
		// that yields a getSortField value of 0, which can be detected as invalid
		return (c.fieldIndex+1) * (this.sortInfo > 0 ? 1 : -1);
	},
	sort: function(){
		this.edit.apply();
		this.model.sort(this.getSortField());
	},
	// row editing
	addRow: function(inRowData, inIndex){
		this.edit.apply();
		var i = inIndex || -1;
		if(i<0){
			i = this.selection.getFirstSelected() || 0;
		}
		if(i<0){
			i = 0;
		}
		this.model.insert(inRowData, i);
		this.model.beginModifyRow(i);
		// begin editing row
		// FIXME: add to edit
		for(var j=0, c; ((c=this.getCell(j)) && !c.editor); j++){}
		if(c&&c.editor){
			this.edit.setEditCell(c, i);
			this.focus.setFocusCell(c, i);
		}else{
			this.focus.setFocusCell(this.getCell(0), i);
		}
	},
	removeSelectedRows: function(){
		this.edit.apply();
		var s = this.selection.getSelected();
		if(s.length){
			this.model.remove(s);
			this.selection.clear();
		}
	},
	//: protected
	// editing
	canEdit: function(inCell, inRowIndex){
		// summary: 
		//	determines if a given cell may be edited
		//	inCell: grid cell
		// inRowIndex: grid row index
		// returns: true if given cell may be edited
		return (this.model.canModify ? this.model.canModify(inRowIndex) : true);
	},
	doStartEdit: function(inCell, inRowIndex){
		var edit = this.canEdit(inCell, inRowIndex);
		if(edit){
			this.model.beginModifyRow(inRowIndex);
			this.onStartEdit(inCell, inRowIndex);
		}
		return edit;
	},
	doApplyCellEdit: function(inValue, inRowIndex, inFieldIndex){
		this.model.setDatum(inValue, inRowIndex, inFieldIndex);
		this.onApplyCellEdit(inValue, inRowIndex, inFieldIndex);
	},
	doCancelEdit: function(inRowIndex){
		this.model.cancelModifyRow(inRowIndex);
		this.onCancelEdit.apply(this, arguments);
	},
	doApplyEdit: function(inRowIndex){
		this.model.endModifyRow(inRowIndex);
		this.onApplyEdit(inRowIndex);
	},
	// Perform row styling 
	styleRowState: function(inRow){
		if(this.model.getState){
			var states=this.model.getState(inRow.index), c='';
			for(var i=0, ss=["inflight", "error", "inserting"], s; s=ss[i]; i++){
				if(states[s]){
					c = ' dojoxGrid-row-' + s;
					break;
				}
			}
			inRow.customClasses += c;
		}
	},
	onStyleRow: function(inRow){
		this.styleRowState(inRow);
		this.inherited(arguments);
	},
	junk: 0
});

}

if(!dojo._hasResource["dijit.Toolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Toolbar"] = true;
dojo.provide("dijit.Toolbar");





dojo.declare(
	"dijit.Toolbar",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	templateString:
		'<div class="dijit dijitToolbar" waiRole="toolbar" tabIndex="${tabIndex}" dojoAttachPoint="containerNode">' +
//			'<table style="table-layout: fixed" class="dijitReset dijitToolbarTable">' + // factor out style
//				'<tr class="dijitReset" dojoAttachPoint="containerNode"></tr>'+
//			'</table>' +
		'</div>',

	tabIndex: "0",

	postCreate: function(){
		this.connectKeyNavHandlers(
			this.isLeftToRight() ? [dojo.keys.LEFT_ARROW] : [dojo.keys.RIGHT_ARROW],
			this.isLeftToRight() ? [dojo.keys.RIGHT_ARROW] : [dojo.keys.LEFT_ARROW]
		);
	},

	startup: function(){
		this.startupKeyNavChildren();
	}
}
);

// Combine with dijit.MenuSeparator??
dojo.declare(
	"dijit.ToolbarSeparator",
	[ dijit._Widget, dijit._Templated ],
{
	// summary
	//	A line between two menu items
	templateString: '<div class="dijitToolbarSeparator dijitInline"></div>',
	postCreate: function(){ dojo.setSelectable(this.domNode, false); },
	isFocusable: function(){ return false; }
});

}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: 200,

		templateString:"<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\r\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\r\n\t<div class=\"dijitTooltipConnector\"></div>\r\n</div>\r\n",

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
			// summary:
			//	Display tooltip w/specified contents to right specified node
			//	(To left if there's no space on the right, or if LTR==right)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			// Firefox bug. when innerHTML changes to be shorter than previous
			// one, the node size will not be updated until it moves.
			this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";

			// position the element and change CSS according to position	
			var align = this.isLeftToRight() ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
			this.domNode.className="dijitTooltip dijitTooltip" + (pos.corner=='BL' ? "Right" : "Left");//FIXME: might overwrite class

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		_onShow: function(){
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary: hide the tooltip
			if(!this.aroundNode || this.aroundNode !== aroundNode){
				return;
			}
			if(this._onDeck){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
				return;
			}
			this.fadeIn.stop();
			this.isShowingNow = false;
			this.aroundNode = null;
			this.fadeOut.play();
		},

		_onHide: function(){
			this.domNode.style.cssText="";	// to position offscreen again
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
	// summary:
	//	Display tooltip w/specified contents to right specified node
	//	(To left if there's no space on the right, or if LTR==right)
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode);
};

dijit.hideTooltip = function(aroundNode){
	// summary: hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: String[]
		//		Id(s) of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		connectId: [],

		postCreate: function(){
			if(this.srcNodeRef){
				this.srcNodeRef.style.display = "none";
			}

			this._connectNodes = [];
			
			dojo.forEach(this.connectId, function(id) {
				var node = dojo.byId(id);
				if (node) {
					this._connectNodes.push(node);
					dojo.forEach(["onMouseOver", "onMouseOut", "onFocus", "onBlur", "onHover", "onUnHover"], function(event){
						this.connect(node, event.toLowerCase(), "_"+event);
					}, this);
					if(dojo.isIE){
						// BiDi workaround
						node.style.zoom = 1;
					}
				}
			}, this);
		},

		_onMouseOver: function(/*Event*/ e){
			this._onHover(e);
		},

		_onMouseOut: function(/*Event*/ e){
			if(dojo.isDescendant(e.relatedTarget, e.target)){
				// false event; just moved from target to target child; ignore.
				return;
			}
			this._onUnHover(e);
		},

		_onFocus: function(/*Event*/ e){
			this._focus = true;
			this._onHover(e);
		},
		
		_onBlur: function(/*Event*/ e){
			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// keep a tooltip open if the associated element has focus
			if(this._focus){ return; }
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary: display the tooltip; usually not called directly.
			target = target || this._connectNodes[0];
			if(!target){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target);
			
			this._connectNode = target;
		},

		close: function(){
			// summary: hide the tooltip; usually not called directly.
			dijit.hideTooltip(this._connectNode);
			delete this._connectNode;
			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		uninitialize: function(){
			this.close();
		}
	}
);

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableSearchField"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/








// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableSearchField");

dojo.declare("ibm.tivoli.tip.dijit.TIPTableSearchField",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,
  
  /* The TIPTable that this search bar is attached to */
  table: null,
  
  _ALL_COLS_NAME: "QF_ALL_COLS",
  
  /** An array of column names that should participate in filtering **/
  columnsToFilterOn: { "QF_ALL_COLS": true },
  
  /** Keeps track of all topic subscriptions to cleanly destroy later **/
  topics: [],
  
  _qf_arrow_src: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/qf_arrow.gif"),
  
  /** Keeps track of all event connections **/
  objectConnections: [],
  
  /** Keeps track of all children widgets in dropdown **/
  childrenWidgets: [],
  
  templateString:"<div class=\"quick-filter\" align=\"right\">\r\n<table cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n<tbody>\r\n<tr>\r\n<td dojoAttachPoint=\"colFilter\" class=\"qf-lcap\" width=\"20px\" valign=\"top\" dojoAttachEvent='onmouseover:_onHover,onmouseout:_onUnHover'>\r\n</td>\r\n<td class=\"qf-mid\" width=\"170\" valign=\"middle\">\r\n<label title=\"${resources_.SEARCH_TEXT}\">\r\n<input dojoAttachPoint=\"searchField\" class=\"qf-textfield-label\" type=\"text\" valign=\"middle\" dojoAttachEvent='onfocus:_onFocus,onblur:_onBlur,onkeypress:_onKeyPress' />\r\n</label>\r\n</td>\r\n<td class=\"qf-rcap\" width=\"15\"/>\r\n</tr>\r\n</tbody>\r\n</table>\r\n</div>\r\n",

  constructor: function( parms ) 
  {
  	this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
  	this.lastColumns = null;
  },
  
  postCreate: function()
  {
    if ( this.table !== null )
    {
      this.setTable( this.table );
    }        
  },  
   
  /********************************************************************
  * This method must be called to set the underlying table that the 
  * search bar will filter on.  As soon as this method is called
  * the table is queried for its "structure" (columns) and the dropdown
  * column filter dialog is populated
  *********************************************************************/
  setTable: function( table )
  {
  	// Save a reference to the table
  	this.table = table;
  	
  	this.clear(); //ensure that we start with a fresh slate
  	  	
  	var div  = document.createElement( 'DIV' );
  	var div2 = document.createElement( 'DIV' );
  	div2.className="qf-col-filter-title";	  
  
  	div2.appendChild( document.createTextNode( this.resources_.FILTER_ON ) );
  	div.appendChild( div2 );

  	div2 = document.createElement( 'DIV' );
	div2.className="qf-col-chkbox-label-row";
  	
  	var checkbox = new dijit.form.CheckBox( { id     : table.id + this._ALL_COLS_NAME,
  											  pId    : table.id,
  	                                          name   : this._ALL_COLS_NAME,
  	                                          checked: true } );
//  	                                          checked: this.columnsToFilterOn[ this._ALL_COLS_NAME ] } );

	this.childrenWidgets[ checkbox.id ] = checkbox;
	
	checkbox.onColsFilter = function()
	{
		this.setChecked( false );
	};
	
	checkbox.onCheckedEvent = function ()
	{
  		if ( this.checked )
  		{
	  		dojo.publish( this.pId + "_filterOnAll", ["Filter on all rows"] );
  		}
	};  	           
	
  	this.topics[ this._ALL_COLS_NAME ] = dojo.subscribe( table.id + "_filterByCol", checkbox, "onColsFilter" );                                                                         
	this.objectConnections[ this._ALL_COLS_NAME ] = dojo.connect( checkbox, "setChecked", checkbox, "onCheckedEvent" );  	                                          

	var label = document.createElement( 'LABEL' );
	label.className = "qf-col-chkbox-label";
	label["for"] = checkbox.id;
	label.title = this.resources_.ALL_COLUMNS;
	
	label.appendChild( document.createTextNode( this.resources_.ALL_COLUMNS ) );
	
	div2.appendChild( checkbox.domNode );  	                                         
	div2.appendChild( label );  	                                         

	div.appendChild( div2 );

	var columns = [];
	var i = 0;
	var j = 0;
	
	table.views.forEach( function( view, i )
	{
		if ( view.structure.cells )
		{
			for ( i in view.structure.cells )
			{
				for ( j in view.structure.cells[i] )
				{
					 var v = view.structure.cells[i][j];				
		             if (v.field !== undefined) {
                         columns.push( v );
	                 }
				}			
			}
		}
	});  	
  	  	
  	for ( i in columns )
  	{
	  if ( columns[i].isSearchable === false ) {
	    continue;
	  }
  		div2 = document.createElement( 'DIV' );
  		div2.className="qf-col-chkbox-label-row";
  		
  		checkbox = new dijit.form.CheckBox( { id     : table.id + columns[i].name,
  											  pId    : table.id,
  											  columnID: columns[i].field,
  	                                          name   : columns[i].name,
  	                                          checked: this.columnsToFilterOn[ columns[i].name ] } );

		this.childrenWidgets[ checkbox.id ] = checkbox;
		
		checkbox.onAllColsFilter = function()
		{
			this.setChecked( false );
		};
		
		checkbox.onCheckedEvent = function ()
		{
	  		if ( this.checked )
	  		{
		  		dojo.publish( this.pId + "_filterByCol", ["Filter on individual columns"] );		
	  		}
		};  	           
		
		this.topics[ columns[i].name ] = dojo.subscribe( table.id + "_filterOnAll", checkbox, "onAllColsFilter" );                               
  	    
		this.objectConnections[ columns[i].name ] = dojo.connect( checkbox, "setChecked", checkbox, "onCheckedEvent" );  	                                          
  	                                          
		div2.appendChild( checkbox.domNode );  	                            

		label = document.createElement( 'LABEL' );
		label["for"] = checkbox.id;
		label.title = columns[i].name;
		label.className = "qf-col-chkbox-label";
		
		label.appendChild( document.createTextNode( columns[i].name ) );
		
		div2.appendChild( label );  	                                         		             
  		
  		div.appendChild( div2 );  		
  	}
  	  	
  	var button = new dijit.form.Button( { id: table.id + "close_btn",
                                        type: "submit", 
                                        onClick: dojo.hitch (this, "onFilterOK"),
  								       label: this.resources_.OK } );

	this.childrenWidgets[ button.id ] = button;	  								       
  								     
	div.appendChild( button.domNode );  								     
  	
  	var tooltipDialog = new dijit.TooltipDialog( { id: table.id + "gf_tooltipdialog" } );

	this.childrenWidgets[ tooltipDialog.id ] = tooltipDialog;	  								       
  	  	
  	tooltipDialog.containerNode.appendChild( div );
  	
  	/** OVERLOAD to allow our own styling to be added **/
  	tooltipDialog.orient = function(/*Object*/ corner)
  	{
		// summary: configure widget to be displayed in given position relative to the button
		this.domNode.className="dijitTooltipDialog " +" dijitTooltipAB"+(corner.charAt(1)=='L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0)=='T' ? "Below" : "Above" ) + " qf-TooltipDialog";
	};  	
  	
  	tooltipDialog.execute = function( args )
  	{  		
  	    this.columnsToFilterOn = [];
  	    
  		for ( var i in args )
  		{
			this.columnsToFilterOn[i] = args[i][0] ? true : false ;
  		}  	
  	};
  	
	var btn = new dijit.form.DropDownButton( { 	id: table.id + "_coltrigger", 
												dropDown: tooltipDialog,
												label: "",
												baseClass: "qf-col-trigger" } );
												
	this.tooltip_dropdownhelp = new dijit.Tooltip({label:this.resources_.SEARCH_DROP_DOWN_HELP, connectId:[btn.id]}, document.createElement("div"));
												
	
	this.childrenWidgets[ btn.id ] = btn;	  								       

	this.colFilter.appendChild( btn.domNode );
		
	btn.startup();
	
	var downArrowSpan =  dojo.query("span.dijitA11yDownArrow", btn.domNode );	
	
	dojo.query("span.dijitButtonText", btn.domNode ).orphan();
	
	downArrowSpan[0].innerHTML = "";
	dojo.removeClass( downArrowSpan[0], "dijitA11yDownArrow" );
	downArrowSpan[0].style.padding="0px";
	
	var image = dojo.doc.createElement( 'IMG' );	
	image.src = this._qf_arrow_src;
	image.border = 0;
	image.style.padding="0px";
	image.style.margin="0px";
	image.alt = this.resources_.SEARCH_DROP_DOWN_HELP;
	
	downArrowSpan[0].appendChild( image );	
  },
  
  onFilterOK: function () {
  	console.log ("enter onFilterOK");
	var columns = this.getSelectedColumns();
	if (this.columnsEqual (columns, this.lastColumns)) {
		console.log ("columns match.  not calling search");
	} else {
		console.log ("calling search");
  		dojo.publish (this.id + "searchInvoked", [this, null]);
  		this.lastColumns = columns;
	}
  },
  
  columnsEqual: function (columns1, columns2) {
  	if ((columns1 === null) && (columns2 === null)) {
  		return (true);
  	} else if ((columns1 === null) || (columns2 === null)) {
  		return (false);
  	} else if (columns1.length !== columns2.length) {
  		return (false);
  	} else {
		for (var x in columns1) {
			if (columns1[x] !== columns2[x]) {
				return (false);
			}
  		}
  		
  		return (true);
  	}
  },
  
  _onFocus: function()
  {
	this.domNode.className="quick-filter-focus";  
  },
  
  _onBlur: function()
  {
	this.domNode.className="quick-filter";
  },
  
  _onKeyPress: function(event)
  {
  	if (event.keyCode == 13) {
	  	event.cancelBubble = true;
  		event.returnValue = false;
  		dojo.publish ("onEnter", [this, event]);
  		dojo.publish (this.id + "searchInvoked", [this, event]);
	}
  },
  
  _onHover: function()
  {
	this.colFilter.className="qf-lcap-over";
  },
  
  _onUnHover: function()
  {
	this.colFilter.className="qf-lcap";
  },
  
  _setFilterCol: function ()
  {  	
  },
  

  getSelectedColumns: function ()
  {
  	var result = [];
  	
  	if (this.childrenWidgets[this.table.id + this._ALL_COLS_NAME].checked) {
	  	for (var x in this.childrenWidgets) {
	  		if (x) {
	  			if (this.childrenWidgets[x].columnID !== undefined ) {
		  			result.push (this.childrenWidgets[x].columnID);
		  		}
  			}
  		}
  	} else {
	  	for (var y in this.childrenWidgets) {
	  		if (y) {
	  			if (this.childrenWidgets[y].checked) {
		  			if (this.childrenWidgets[y].columnID !== undefined ) {
		  				result.push (this.childrenWidgets[y].columnID);
		  			}
	  			}
	  		}
	  	}
  	}
  	
	return (result);
  },
  
  /*************************************************************
  * Clears the contents of the search drop down to be recreated
  *************************************************************/
  clear: function()
  {
  	/** Destroy all subscriptions **/
  	for ( var i in this.topics )
  	{
  		dojo.unsubscribe( this.topics[i] );
  	}
  	
  	delete this.topics;
  	this.topics = [];

  	/** Destroy all event connections **/
  	for ( i in this.objectConnections )
  	{
  		dojo.disconnect( this.objectConnections[i] );
  	}  	

  	delete this.objectConnections;
  	this.objectConnections = [];
  	
  	/** Destroy all children widgets **/
  	for ( i in this.childrenWidgets )
  	{
  		this.childrenWidgets[i].destroy();
  	}  
  	
  	if ( this.tooltip_dropdownhelp )
  	{
  		this.tooltip_dropdownhelp.destroy();
  	}

  	delete this.childrenWidgets;
  	this.childrenWidgets = [];
  	
  	delete this.columnsToFilterOn;
  	this.columnsToFilterOn = [];
  },
  
  /*************************************************************
  * Destroy this widget -- remove all subscriptions and event
  * connections
  *************************************************************/
  destroy: function()
  {  	
  	this.clear();
  	
  	/** call super **/
  	this.inherited( "destroy", arguments );
  }
  
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPDojoData"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPDojoData"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPDojoData");

// dojox.grid._data.model is the module with dojox.grid.data.DojoData


dojo.declare("ibm.tivoli.tip.dojo.TIPDojoData",
	          dojox.grid.data.DojoData,
{
//	returnTimes: 0,
	realCount: 0,
	
    // custom override of requestRows()
    requestRows: function (inRowIndex, inCount) {
         // creates serverQuery-parameter
         var row  = inRowIndex || 0;
         var params = {
	                start: row,
                    count: inCount || this.rowsPerPage,
                    serverQuery: dojo.mixin(
                      { start: row,
                        count: inCount || this.rowsPerPage,
                        sort:(this.sortColumn || '')
                      },
                      this.query
                    ),
                    query: this.query,
                    onBegin: dojo.hitch(this, "beginReturn"),
                    onComplete: dojo.hitch(this, "processRows")
                }
          this.store.fetch(params);
    },

	beginReturn: function (inCount) {
		console.log ("in beginReturn with inCount: " + inCount);
		this.setRowCount (inCount);
		this.realCount = inCount;
		dojo.publish ("/ibm.tip/TIPDojoData#rowCountUpdated", [this.id, inCount]);
	},

	processRows: function (items, store) {
		console.log ("in processRows with items: ");
		console.log (items);
		console.log ("and store: ");
		console.log (store);
		if (!items || items.length===0){
		    console.log("not processing any rows");
		    return;
		}
		this.inherited (arguments);
	},

	getRealRowCount: function () {
		return (this.realCount);
	},
       
    getRowCount: function () {
    	return (this.rowsPerPage > this.count ? this.rowsPerPage : this.count);
    },
    
    // src: String
    setData: function(inData) {
//    	this.returnTimes = 0;
//    	this.setRowCount(this.rowsPerPage + 1);
          console.log("in setData() with data: ");
          console.log(inData);
          // edited not to reset the store
          this.data = [];
          this.allChange();
    },
    sort: function(colIndex) {
    	console.log ("enter sort for col: " + colIndex);
//    	this.returnTimes = 0;
        this.clearData();
        this.sortColumn = colIndex;
        this.requestRows(); 
    },
    
    getSortColumn: function () {
    	return (this.sortColumn);
    },
    
    // Overriding this method only to prevent the '?' from showing up
	getDatum: function(inRowIndex, inColIndex){
		//console.debug("getDatum", inRowIndex, inColIndex);
		var row = this.getRow(inRowIndex);
		var field = this.fields.values[inColIndex];
		return row && field ? row[field.name] : field ? field.na : '...';
		//var idx = row && this.fields._nameMaps[inColIndex+".idx"];
		//return (row ? row[idx] : this.fields.get(inColIndex).na);
	},
    canSort: function () {
          return true;
    }
});

}

if(!dojo._hasResource["dojox.data.QueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.QueryReadStore"] = true;
dojo.provide("dojox.data.QueryReadStore");



dojo.declare("dojox.data.QueryReadStore", null, {
	/*
	//	summary:
	//		This class provides a store that is mainly intended to be used
	//		for loading data dynamically from the server, used i.e. for
	//		retreiving chunks of data from huge data stores on the server (by server-side filtering!).
	//		Upon calling the fetch() method of this store the data are requested from
	//		the server if they are not yet loaded for paging (or cached).
	//
	//		For example used for a combobox which works on lots of data. It
	//		can be used to retreive the data partially upon entering the
	//		letters "ac" it returns only items like "action", "acting", etc.
	//
	//  note:
	//      The field name "id" in a query is reserved for looking up data
	//      by id. This is necessary as before the first fetch, the store
	//      has no way of knowing which field the server will declare as
	//      identifier.
	//
	//	examples:
	//		// The parameter "query" contains the data that are sent to the server.
	//		var store = new dojox.data.QueryReadStore({url:'/search.php'});
	//		store.fetch({query:{name:'a'}, queryOptions:{ignoreCase:false}});
	//
	//		// Since "serverQuery" is given, it overrules and those data are
	//		// sent to the server.
	//		var store = new dojox.data.QueryReadStore({url:'/search.php'});
	//		store.fetch({serverQuery:{name:'a'}, queryOptions:{ignoreCase:false}});
	//
	//	todo:
	//		- there is a bug in the paging, when i set start:2, count:5 after an initial fetch() and doClientPaging:true
	//		  it returns 6 elemetns, though count=5, try it in QueryReadStore.html
	//		- allow configuring if the paging shall takes place on the client or the server
	//		- add optional caching
	//		- when the first query searched for "a" and the next for a subset of
	//		  the first, i.e. "ab" then we actually dont need a server request, if
	//		  we have client paging, we just need to filter the items we already have
	//		  that might also be tooo much logic
	*/
	
	url:"",
	requestMethod:"get",
	//useCache:false,
	
	// We use the name in the errors, once the name is fixed hardcode it, may be.
	_className:"dojox.data.QueryReadStore",
	
	// This will contain the items we have loaded from the server.
	// The contents of this array is optimized to satisfy all read-api requirements
	// and for using lesser storage, so the keys and their content need some explaination:
	// 		this._items[0].i - the item itself 
	//		this._items[0].r - a reference to the store, so we can identify the item
	//			securly. We set this reference right after receiving the item from the
	//			server.
	_items:[],
	
	// Store the last query that triggered xhr request to the server.
	// So we can compare if the request changed and if we shall reload 
	// (this also depends on other factors, such as is caching used, etc).
	_lastServerQuery:null,
	
	
	// Store a hash of the last server request. Actually I introduced this
	// for testing, so I can check if no unnecessary requests were issued for
	// client-side-paging.
	lastRequestHash:null,
	
	// If this is false, every request is sent to the server.
	// If it's true a second request with the same query will not issue another
	// request, but use the already returned data. This assumes that the server
	// does not do the paging.
	doClientPaging:true,

	// If this is false, all the sorting is done serverside before the data is returned
	// which is the proper place to be doing it for really large datasets.
	doClientSorting:true,

	// Items by identify for Identify API
	_itemsByIdentity:null,
	
	// Identifier used
	_identifier:null,

	_features: {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true},

	_labelAttr: "label",
	
	constructor: function(/* Object */ params){
		dojo.mixin(this,params);
	},
	
	getValue: function(/* item */ item, /* attribute-name-string */ attribute, /* value? */ defaultValue){
		//	According to the Read API comments in getValue() and exception is
		//	thrown when an item is not an item or the attribute not a string!
		this._assertIsItem(item);
		if (!dojo.isString(attribute)) {
			throw new Error(this._className+".getValue(): Invalid attribute, string expected!");
		}
		if(!this.hasAttribute(item, attribute)){
			// read api says: return defaultValue "only if *item* does not have a value for *attribute*." 
			// Is this the case here? The attribute doesn't exist, but a defaultValue, sounds reasonable.
			if(defaultValue){
				return defaultValue;
			}
			console.log(this._className+".getValue(): Item does not have the attribute '"+attribute+"'.");
		}
		return item.i[attribute];
	},
	
	getValues: function(/* item */ item, /* attribute-name-string */ attribute){
		this._assertIsItem(item);
		var ret = [];
		if(this.hasAttribute(item, attribute)){
			ret.push(item.i[attribute]);
		}
		return ret;
	},
	
	getAttributes: function(/* item */ item){
		this._assertIsItem(item);
		var ret = [];
		for(var i in item.i){
			ret.push(i);
		}
		return ret;
	},

	hasAttribute: function(/* item */ item,	/* attribute-name-string */ attribute) {
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		return this.isItem(item) && typeof item.i[attribute]!="undefined";
	},
	
	containsValue: function(/* item */ item, /* attribute-name-string */ attribute, /* anything */ value){
		var values = this.getValues(item, attribute);
		var len = values.length;
		for(var i=0; i<len; i++){
			if(values[i]==value){
				return true;
			}
		}
		return false;
	},
	
	isItem: function(/* anything */ something){
		// Some basic tests, that are quick and easy to do here.
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem("");
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem({});
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem(0);
		// false
		//
		// >>> var store = new dojox.data.QueryReadStore({});
		// >>> store.isItem({name:"me", label:"me too"});
		// false
		//
		if(something){
			return typeof something.r!="undefined" && something.r==this;
		}
		return false;
	},
	
	isItemLoaded: function(/* anything */ something) {
		// Currently we dont have any state that tells if an item is loaded or not
		// if the item exists its also loaded.
		// This might change when we start working with refs inside items ...
		return this.isItem(something);
	},

	loadItem: function(/* object */ args){
		if(this.isItemLoaded(args.item)){
			return;
		}
		// Actually we have nothing to do here, or at least I dont know what to do here ...
	},

	fetch:function(/* Object? */ request){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
		//		only the paging, since it happens on the server if doClientPaging is
		//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
		//		Would be nice to be able to use simpleFetch() to reduce copied code,
		//		but i dont know how yet. Ideas please!
		request = request || {};
		if(!request.store){
			request.store = this;
		}
		var self = this;
	
		var _errorHandler = function(errorData, requestObject){
			if(requestObject.onError){
				var scope = requestObject.scope || dojo.global;
				requestObject.onError.call(scope, errorData, requestObject);
			}
		};
	
		var _fetchHandler = function(items, requestObject, numRows){
			var oldAbortFunction = requestObject.abort || null;
			var aborted = false;
			
			var startIndex = requestObject.start?requestObject.start:0;
			if (self.doClientPaging==false) {
				// For client paging we dont need no slicing of the result.
				startIndex = 0;
			}
			var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;
	
			requestObject.abort = function(){
				aborted = true;
				if(oldAbortFunction){
					oldAbortFunction.call(requestObject);
				}
			};
	
			var scope = requestObject.scope || dojo.global;
			if(!requestObject.store){
				requestObject.store = self;
			}
			if(requestObject.onBegin){
				requestObject.onBegin.call(scope, numRows, requestObject);
			}
			if(requestObject.sort && this.doClientSorting){
				items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
			}
			if(requestObject.onItem){
				for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
					var item = items[i];
					if(!aborted){
						requestObject.onItem.call(scope, item, requestObject);
					}
				}
			}
			if(requestObject.onComplete && !aborted){
				var subset = null;
				if (!requestObject.onItem) {
					subset = items.slice(startIndex, endIndex);
				}
				requestObject.onComplete.call(scope, subset, requestObject);   
			}
		};
		this._fetchItems(request, _fetchHandler, _errorHandler);
		return request;	// Object
	},

	getFeatures: function(){
		return this._features;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		// I have no idea if this is really needed ... 
	},

	getLabel: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item, this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
        },
	
	_fetchItems: function(request, fetchHandler, errorHandler){
		//	summary:
		// 		The request contains the data as defined in the Read-API.
		// 		Additionally there is following keyword "serverQuery".
		//
		//	The *serverQuery* parameter, optional.
		//		This parameter contains the data that will be sent to the server.
		//		If this parameter is not given the parameter "query"'s
		//		data are sent to the server. This is done for some reasons:
		//		- to specify explicitly which data are sent to the server, they
		//		  might also be a mix of what is contained in "query", "queryOptions"
		//		  and the paging parameters "start" and "count" or may be even
		//		  completely different things.
		//		- don't modify the request.query data, so the interface using this
		//		  store can rely on unmodified data, as the combobox dijit currently
		//		  does it, it compares if the query has changed
		//		- request.query is required by the Read-API
		//
		// 		I.e. the following examples might be sent via GET:
		//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?name=abc
		//
		//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?q=abc&c=true
		//		  // The serverQuery-parameter has overruled the query-parameter
		//		  // but the query parameter stays untouched, but is not sent to the server!
		//		  // The serverQuery contains more data than the query, so they might differ!
		//

		var serverQuery = request.serverQuery || request.query || {};
		//Need to add start and count
		if(!this.doClientPaging){
			serverQuery.start = request.start || 0;
			// Count might not be sent if not given.
			if (request.count) {
				serverQuery.count = request.count;
			}
		}
		if(!this.doClientSorting){
			if(request.sort){
				var sort = request.sort[0];
				if(sort && sort.attribute){
					var sortStr = sort.attribute;
					if(sort.descending){
						sortStr = "-" + sortStr;
					}
					serverQuery.sort = sortStr;
				}
			}
		}
		// Compare the last query and the current query by simply json-encoding them,
		// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
		if(this.doClientPaging && this._lastServerQuery!==null &&
			dojo.toJson(serverQuery)==dojo.toJson(this._lastServerQuery)
			){
			fetchHandler(this._items, request);
		}else{
			var xhrFunc = this.requestMethod.toLowerCase()=="post" ? dojo.xhrPost : dojo.xhrGet;
			var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
			xhrHandler.addCallback(dojo.hitch(this, function(data){
				data = this._filterResponse(data);
				if (data.label){
					this._labelAttr = data.label;
				}
				var numRows = data.numRows || -1;

				this._items = [];
				// Store a ref to "this" in each item, so we can simply check if an item
				// really origins form here (idea is from ItemFileReadStore, I just don't know
				// how efficient the real storage use, garbage collection effort, etc. is).
				dojo.forEach(data.items,function(e){ 
					this._items.push({i:e, r:this}); 
				},this); 
				
				var identifier = data.identifier;
				this._itemsByIdentity = {};
				if(identifier){
					this._identifier = identifier;
					for(i = 0; i < this._items.length; ++i){
						var item = this._items[i].i;
						var identity = item[identifier];
						if(!this._itemsByIdentity[identity]){
							this._itemsByIdentity[identity] = item;
						}else{
							throw new Error(this._className+":  The json data as specified by: [" + this.url + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
						}
					}
				}else{
					this._identifier = Number;
					for(i = 0; i < this._items.length; ++i){
						this._items[i].n = i;
					}
				}
				
				// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
				// (does it really sanititze them) and store the data optimal. should we? for security reasons???
				numRows = (numRows === -1) ? this._items.length : numRows;
				fetchHandler(this._items, request, numRows);
			}));
			xhrHandler.addErrback(function(error){
				errorHandler(error, request);
			});
			// Generate the hash using the time in milliseconds and a randon number.
			// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
			// probably just for esthetic reasons :-).
			this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
			this._lastServerQuery = dojo.mixin({}, serverQuery);
		}
	},
	
	_filterResponse: function(data){
		//	summary:
		//		If the data from servers needs to be processed before it can be processed by this
		//		store, then this function should be re-implemented in subclass. This default 
		//		implementation just return the data unchanged.
		//	data:
		//		The data received from server
		return data;
	},

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		It throws an error if item is not valid, so you can call it in every method that needs to
		//		throw an error when item is invalid.
		//	item: 
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){
			throw new Error(this._className+": Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute: 
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){ 
			throw new Error(this._className+": Invalid attribute argument ('"+attribute+"').");
		}
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// See if we have already loaded the item with that id
		// In case there hasn't been a fetch yet, _itemsByIdentity is null
		// and thus a fetch will be triggered below.
		if(this._itemsByIdentity){
			var item = this._itemsByIdentity[keywordArgs.identity];
			if(!(item === undefined)){
				if(keywordArgs.onItem){
					var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
					keywordArgs.onItem.call(scope, {i:item, r:this});
				}
				return;
			}
		}

		// Otherwise we need to go remote
		// Set up error handler
		var _errorHandler = function(errorData, requestObject){
			var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
			if(keywordArgs.onError){
				keywordArgs.onError.call(scope, error);
			}
		};
		
		// Set up fetch handler
		var _fetchHandler = function(items, requestObject){
			var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
			try{
				// There is supposed to be only one result
				var item = null;
				if(items && items.length == 1){
					item = items[0];
				}
				
				// If no item was found, item is still null and we'll
				// fire the onItem event with the null here
				if(keywordArgs.onItem){
					keywordArgs.onItem.call(scope, item);
				}
			}catch(error){
				if(keywordArgs.onError){
					keywordArgs.onError.call(scope, error);
				}
			}
		};
		
		// Construct query
		var request = {serverQuery:{id:keywordArgs.identity}};
		
		// Dispatch query
		this._fetchItems(request, _fetchHandler, _errorHandler);
	},
	
	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = null;
		if(this._identifier === Number){
			identifier = item.n; // Number
		}else{
			identifier = item.i[this._identifier];
		}
		return identifier;
	},
	
	getIdentityAttributes: function(/* item */ item){
		//	summary:
		//		See dojo.data.api.Identity.getIdentityAttributes()
		return [this._identifier];
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPQueryReadStore"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPQueryReadStore");




// Override QueryReadStore to make it work with paging
// See changes between: Begin Change - JSA
//                      End Change - JSA

dojo.declare("ibm.tivoli.tip.dojo.TIPQueryReadStore",
	          dojox.data.QueryReadStore,
{
    global: null,
    tableID: null,
    loadCompleteCallback: null, 

	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	
    
    setLoadCompleteCallback: function(aFunction){
        this.loadCompleteCallback = aFunction;
    },
    
	fetch:function(/* Object? */ request){
		//	summary:
		//		See dojo.data.util.simpleFetch.fetch() this is just a copy and I adjusted
		//		only the paging, since it happens on the server if doClientPaging is
		//		false, thx to http://trac.dojotoolkit.org/ticket/4761 reporting this.
		//		Would be nice to be able to use simpleFetch() to reduce copied code,
		//		but i dont know how yet. Ideas please!
		console.log ("enter fetch w/ request:");
		console.log (request);
		request = request || {};
		if(!request.store){
			request.store = this;
		}
		var self = this;
	
		var _errorHandler = function(errorData, requestObject){
			if(requestObject.onError){
				var scope = requestObject.scope || dojo.global;
				requestObject.onError.call(scope, errorData, requestObject);
			}
		};
	
	    
		// Begin Change - JSA
		var _fetchHandler = function(items, requestObject, data){
		// End Change - JSA
			var oldAbortFunction = requestObject.abort || null;
			var aborted = false;
			
			var startIndex = requestObject.start?requestObject.start:0;
			if (self.doClientPaging==false) {
				// For client paging we dont need no slicing of the result.
				startIndex = 0;
			}
			var endIndex   = requestObject.count?(startIndex + requestObject.count):items.length;
	
			requestObject.abort = function(){
				aborted = true;
				if(oldAbortFunction){
					oldAbortFunction.call(requestObject);
				}
			};
	
			var scope = requestObject.scope || dojo.global;
			if(!requestObject.store){
				requestObject.store = self;
			}
			if(requestObject.onBegin){
				// Begin Change - JSA 
				requestObject.onBegin.call(scope, data.rowCount, requestObject);
				// End Change - JSA
			}
			if(requestObject.sort){
				items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
			}
			if(requestObject.onItem){
				for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
					var item = items[i];
					if(!aborted){
						requestObject.onItem.call(scope, item, requestObject);
					}
				}
			}
			if(requestObject.onComplete && !aborted){
				var subset = null;
				if (!requestObject.onItem) {
					subset = items.slice(startIndex, endIndex);
				}
				requestObject.onComplete.call(scope, subset, requestObject);   
			}
		};
		
		
		this._fetchItems(request, _fetchHandler, _errorHandler);
		return request;	// Object
	},
	
	_fetchItems: function(request, fetchHandler, errorHandler){
		//	summary:
		// 		The request contains the data as defined in the Read-API.
		// 		Additionally there is following keyword "serverQuery".
		//
		//	The *serverQuery* parameter, optional.
		//		This parameter contains the data that will be sent to the server.
		//		If this parameter is not given the parameter "query"'s
		//		data are sent to the server. This is done for some reasons:
		//		- to specify explicitly which data are sent to the server, they
		//		  might also be a mix of what is contained in "query", "queryOptions"
		//		  and the paging parameters "start" and "count" or may be even
		//		  completely different things.
		//		- don't modify the request.query data, so the interface using this
		//		  store can rely on unmodified data, as the combobox dijit currently
		//		  does it, it compares if the query has changed
		//		- request.query is required by the Read-API
		//
		// 		I.e. the following examples might be sent via GET:
		//		  fetch({query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?name=abc
		//
		//		  fetch({serverQuery:{q:"abc", c:true}, query:{name:"abc"}, queryOptions:{ignoreCase:true}})
		//		  the URL will become:   /url.php?q=abc&c=true
		//		  // The serverQuery-parameter has overruled the query-parameter
		//		  // but the query parameter stays untouched, but is not sent to the server!
		//		  // The serverQuery contains more data than the query, so they might differ!
		//

		var serverQuery = request.serverQuery || request.query || {};
		//Need to add start and count
		if(!this.doClientPaging){
			serverQuery.start = request.start || 0;
			// Count might not be sent if not given.
			if (request.count) {
				serverQuery.count = request.count;
			}
		}
		// Compare the last query and the current query by simply json-encoding them,
		// so we dont have to do any deep object compare ... is there some dojo.areObjectsEqual()???
		if(this.doClientPaging && this._lastServerQuery!==null &&
			dojo.toJson(serverQuery)==dojo.toJson(this._lastServerQuery)
			){
			fetchHandler(this._items, request);
		}else{
			// var xhrFunc = this.requestMethod.toLowerCase()=="post" ? dojo.xhrPost : dojo.xhrGet;
			// var xhrHandler = xhrFunc({url:this.url, handleAs:"json-comment-optional", content:serverQuery});
			var kResponseHandler = dojo.hitch(this, function(data){
				data=this._filterResponse(data);
				this._items = [];
				// Store a ref to "this" in each item, so we can simply check if an item
				// really origins form here (idea is from ItemFileReadStore, I just don't know
				// how efficient the real storage use, garbage collection effort, etc. is).
				dojo.forEach(data.items,function(e){ 
					this._items.push({i:e, r:this}); 
				},this); 
				
				var identifier = data.identifier;
				this._itemsByIdentity = {};
				if(identifier){
					this._identifier = identifier;
					for(i = 0; i < this._items.length; ++i){
						var item = this._items[i].i;
						var identity = item[identifier];
						if(!this._itemsByIdentity[identity]){
							this._itemsByIdentity[identity] = item;
						}else{
							throw new Error("dojo.data.QueryReadStore:  The json data as specified is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
						}
					}
				}else{
					this._identifier = Number;
					for(i = 0; i < this._items.length; ++i){
						this._items[i].n = i;
					}
				}
				
				// TODO actually we should do the same as dojo.data.ItemFileReadStore._getItemsFromLoadedData() to sanitize
				// (does it really sanititze them) and store the data optimal. should we? for security reasons???
				// Begin Change - JSA
				fetchHandler(this._items, request, data);
				
				// If a callback is registered, then call it now.  Recall this is used to help
				// smoothly replace models on a refresh to avoid flashing.
				if (this.loadCompleteCallback) {
				    this.loadCompleteCallback();
				}
				// End Change - JSA
			});
			
			var kErrorHandler = function(error){
				errorHandler(error, request);
			};
			
			// Generate the hash using the time in milliseconds and a randon number.
			// Since Math.randon() returns something like: 0.23453463, we just remove the "0."
			// probably just for esthetic reasons :-).
			this.lastRequestHash = new Date().getTime()+"-"+String(Math.random()).substring(2);
			this._lastServerQuery = serverQuery;

            var kw = {
                    content:   {
                                  command: "getTableRows",
                                  arg_fqTableId: this.tableID,
                                  arg_portletNamespace: this.global.namespace
                               },
                    timeout:   this.requestTimeout,
                    onResponse:{
                                  callback: kResponseHandler
                               },
                    onError:{
                                  callback: kErrorHandler
                               }
            };
            
            // Add server query stuff
            kw.content = dojo.mixin(kw.content,serverQuery);
            
            if (this.requestMethod.toLowerCase()=="post") {
                ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).post( kw );
            } else {
                ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
            }
			
		}
	},
	
	_filterResponse: function(data){
		//	summary:
		//		If the data from servers needs to be processed before it can be processed by this
		//		store, then this function should be re-implemented in subclass. This default 
		//		implementation just return the data unchanged.
		//	data:
		//		The data received from server
		return data;
	}
	
	
});
	

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTable"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/





//









// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTable");
dojo.provide("ibm.tivoli.tip.dijit.TIPTableContainer");
/******************************************************************************
* This container places the search bar and toolbar around a given 
* dojox.grid.Grid 
*
* Authors: Joseph Firebaugh, Joshua Allen, Ken Parzygnat
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableContainer",
            [ dijit._Widget, dijit._Templated ],
{
	templateString:"<div>\r\n<table dojoAttachPoint=\"tableContainer\" class=\"tableToolbar\" id=\"${id}tableContainer\" border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\r\n\t<tr dojoAttachPoint=\"wipeNode\">\r\n\t\t<td class=\"TIPTableToolbarArea\">\r\n\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t</td>\r\n\t    <td class=\"TIPTableSearchFieldArea\">\r\n\t\t    <div dojoType=\"ibm.tivoli.tip.dijit.TIPTableSearchField\" dojoAttachPoint=\"searchField\"></div>\r\n\t    </td>\r\n\t</tr>\r\n\t<tr class=\"TIPTableToggle\">\r\n\t\t<td colspan=\"2\">\r\n\t\t\t<div class='tipDescriptionAreaClose' dojoAttachPoint='closeNode' tabIndex=\"0\" dojoAttachEvent=\"onclick:toggleToolbar,onkeypress:onToggleKeyPress\"></div>\r\n\t\t</td>\r\n\t</tr>\r\n<tr>\r\n  <td id=\"${id}gridContainerCell\" dojoAttachPoint=\"gridContainerCell\" colspan=\"2\" height=\"200px\" valign=\"top\">\r\n\t<div id=\"${id}gridContainer\" dojoAttachPoint=\"gridContainer\" ></div>\r\n  </td>\r\n</tr>\r\n<tr>\r\n  <td colspan=\"2\" valign=\"top\">\r\n\t<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t\t<tbody>\r\n\t\t\t<tr class=\"isc-table-footer\" dir=\"ltr\">\r\n\t\t\t\t<td class=\"isc-table-footer-cap-l\" valign=\"bottom\" nowrap=\"nowrap\" align=\"left\">\r\n\t\t\t\t</td>\r\n\t\t\t\t<td>\r\n\t\t\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"footerToolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t\t\t</td>\r\n\t\t\t\t\r\n\t\t\t\t<td class=\"isc-table-footer-data\" nowrap=\"nowrap\" align=\"right\"> \r\n\t\t\t\t\t<div id=\"${id}footer\" dojoAttachPoint=\"footer\" ></div>\r\n\t\t\t\t</td>\r\n\t\t\t\t<td class=\"isc-table-footer-cap-r\" valign=\"bottom\" nowrap=\"nowrap\" align=\"right\">\r\n\t\t\t\t</td>\r\n\t\t\t</tr>  \r\n\t\t</tbody>\r\n\t</table>\r\n  </td>\r\n</tr>\r\n</table>\r\n</div>\r\n",	
	widgetsInTemplate: true,
	
	/* Do you want the default buttons added to the toolbar? */
	makeDefaultButtons: true,	

	/* Reference to the dojox.grid.Grid */
	grid: null,
	
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	
	multiSelect: true,
	rowDblClick: null,
	
	// open: Boolean
	//    Whether toolbar is opened or closed on first render.
	open: false,

	// duration: Integer
	//    Time in milliseconds to wipe in/wipe out toolbar
	duration: 300,
	
	// null means auto width
	// 100% takes all available width
	width: "100%",
	
	height: "200px",

	constructor: function () 
	{
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.tooltips = [];
		this.connections = [];
		this.topics = [];
	},
	
	/**************************************************************************
	* Called after template has been processed.
	*
	* Lets check to see if a dojox.grid.Grid was passed in on params.  If so,
	* add it to the DOM tree and setup.
	**************************************************************************/
	postCreate: function ()
	{
	    if( !this.open ) {
			this.wipeNode.style.display = "none";
	    }
		this._updateToggleClass();    
	
		//call super
		this.inherited( "postCreate", arguments );
		
		//if grid passed on params then set it
		if ( this.grid !== null )
		{
			this.setGrid( this.grid );
		}
		
		//initialize the toolbar
		if ( this.makeDefaultButtons === true ) 
		{ 
			this.initializeToolbar(); 
		}
		
		// set width
		if (this.width !== null) {
			this.setWidth (this.width);
		}
		
		// set height
		if (this.height !== null) {
			this.setHeight (this.height);
		}
		
		this.topics.push (dojo.subscribe (this.searchField.id + "searchInvoked", this, "onSearch"));
		this.setupAnimation();
		this.dojoNLSFix();
	},
	
	setupAnimation: function () {
	    // setup open/close animations
		var wipeNode = this.wipeNode;
        
	    this._wipeIn = dojo.fx.wipeIn({
	      node: this.wipeNode,
	      duration: this.duration,
	      beforeBegin: function(){
	      }
	    });
    
	    this._wipeOut = dojo.fx.wipeOut({
	      node: this.wipeNode,
	      duration: this.duration,
	      onEnd: function(){
	      }
	    });
	},
	
	/**************************************************************************
	* Can be called at any time to place a grid within this "container"
	**************************************************************************/
	setGrid: function( grid, append )
	{
		this.grid = grid;
		
		this.searchField.setTable( grid );

		if (append === false) 
		{
		} 
		else 
		{
		  this.gridContainer.appendChild( grid.domNode );
		}
		
		this.grid.selection.multiSelect = this.multiSelect;
	},
	
	setWidth: function (width) 
	{
		this.width = width;
		//dojo.byId (this.id + "tableContainer").width = this.width;
		if (this.tableContainer && this.tableContainer.width) {
			this.tableContainer.width = width;
		}
	},
	
	setHeight: function (height)
	{
		this.height = height;
//		this.gridContainerCell.height = this.height;
//		dojo.byId (this.id + "gridContainerCell").height = this.height;
	},
	
	/**************************************************************************
	* Get the dojox.grid.Grid object
	**************************************************************************/
	getGrid: function()
	{
		return this.grid;
	},
	
	/**************************************************************************
	* Get the dijit.Toolbar attached to the table
	**************************************************************************/
	getToolbar: function() 
	{
		return this.toolbar;
	},
	
	getFooterToolbar: function () 
	{
		return this.footerToolbar;
	},
	
	/**************************************************************************
	* Initialize the toolbar with default buttons (like refresh)
	**************************************************************************/
	initializeToolbar: function() 
	{
		var button = new ibm.tivoli.tip.dijit.TIPButton(
			{	label: this.resources_.REFRESH, 
				id: this.id + "refreshButton",
				showLabel: false, 
				iconClass: "refreshIcon",
			 	onClick: dojo.hitch (this, this.onRefresh) 
			});
		this.toolbar.addChild(button);
		
		if (this.multiSelect) {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.SELECT_ALL, 
					showLabel: false, 
					iconClass: "selectAllIcon",
				 	onClick: dojo.hitch (this, this.selectAll) 
				});
			this.toolbar.addChild(button);
			
			button = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.DESELECT_ALL, 
					showLabel: false, 
					iconClass: "deselectAllIcon",
				 	onClick: dojo.hitch (this, this.deselectAll) 
				});
			this.toolbar.addChild(button);
		}
	},
	
	/**************************************************************************
	* This method fires a widgetID#Refresh event when the refresh buttons is
	* pressed.  OR you can overload this method instead. If you overload this
	* method you should call inherited so the table is cleared of contents.
	**************************************************************************/
	onRefresh: function() 
	{
//		this.setLoading();
		
		//create empty model
		this._model = new dojox.grid.data.DojoData(null, null);
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
	
	setLoading: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "refreshIcon");
			dojo.addClass (btn, "loadingIcon");
		}
	},
	
	setSteady: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "loadingIcon");
			dojo.addClass (btn, "refreshIcon");
		}
	},
	
	selectAll: function () 
	{
		var count = 0;
		if (this._model.getRealRowCount) {
			count = this._model.getRealRowCount();
		} else {
			count = this._model.getRowCount();
		}
		
		for (var i=0; i<count; i++) {
			this.grid.selection.addToSelection(i);
		}
		
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	deselectAll: function () 
	{
		this.grid.selection.unselectAll();
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	onSearch: function (tipSearchField, event) 
	{
		console.log ("enter onSearch with search value: " + tipSearchField.searchField.value);
		
		if (this.search) 
		{
			var columns = this.searchField.getSelectedColumns();
			this.search (columns, tipSearchField.searchField.value);
		}
	},
	onToggleKeyPress: function (event) {
	  	if (event.keyCode == 13) {
		  	event.cancelBubble = true;
	  		event.returnValue = false;
	  		console.log ("in onToggleKeyPress: pressed enter");
	  		this.toggleToolbar();
	  	}
	},
	
  /*****************************************************************************
  * The toggle open/close button div was clicked -- toggle open/close state
  *****************************************************************************/
  toggleToolbar: function()
  {
    dojo.forEach([this._wipeIn, this._wipeOut], function(animation){
      if(animation.status() == "playing"){
        animation.stop();
      }
    });

    this[this.open ? "_wipeOut" : "_wipeIn"].play();
    this.open =! this.open;    
    this._updateToggleClass();    
	dojo.publish (this.id + "#ToggleToolbar", [ "ToggleToolbar" ]);
  },
  openToolbar: function () {
  	this.open = false;
  	this.toggleToolbar();
  },
  closeToolbar: function () {
  	this.open = true;
  	this.toggleToolbar();
  },
  
  /*****************************************************************************
  * based on the open/close state of widget change CSS to update button
  *****************************************************************************/
  _updateToggleClass: function()
  {
    if ( this.open === true )
    {
      dojo.removeClass( this.domNode, "close"  );
         dojo.addClass( this.domNode, "open" );          
    }
    else
    {
      dojo.removeClass( this.domNode, "open" );
         dojo.addClass( this.domNode, "close"  );          
    }  
  },
  
  resize: function () {
  	// placeholder... do nothing for now
  },
  
 	/************************************************************************
	* A nasty hack to fix TIPTable when using i.e. Arabic language . It's an issue from
	* Dojo code so as soon as they'll fix it this method ought to be deleted.
	* It isn't a good solution, I know that.
	  This solutions is borrowed from TCR. Thanks!
	************************************************************************/
	dojoNLSFix: function(context)
	{
		console.log("ibm.tivoli.tip.dijit.TIPTable.dojoNLSFix ENTRY");

		if(dojo.isFF) {
			dojo.query("#page-0", this.domNode).style("position","");
		} else if(dojo.isIE) {
			dojo.query("#dojox_GridView_0", this.domNode).style("position","relative");
		}

		console.log("ibm.tivoli.tip.dijit.TIPTable.dojoNLSFix EXIT");
	}
  
	
});

/******************************************************************************
* TIP Based Table that creates the underlying dojox.grid.Grid for you based on
* a given data store or Global that uses defined JSON returns for model data
* and column structure.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTable",
              ibm.tivoli.tip.dijit.TIPTableContainer,
{
	global: null,
	tableID: null,
	cmProvider: null,
	store: null,
	tableArgs: null,
	gridParms: {autoWidth: false, autoHeight:false, rowsPerPage: "10", rowCount: "5" },
	query: null,
	_contextMenu: null,
	_model: null,
   _onRowContextMenuListener: null,
   _contextMenuAdapter : null,

   clientViewStructure : null, // Provide if you want to provide view structure from client
   clientRfcItems : null,      // Provide if you are providing above view structure.
   autoSavePreferences : false,

  
	constructor: function() {
		this.lastPreferences = null;
	},
	
	postCreate: function () {
		this.inherited (arguments);
		this.topics.push (dojo.subscribe ("/ibm.tip/TIPDojoData#rowCountUpdated", this, "updateFooter"));
	},

	startup: function() {
		console.info ("enter TIPTable.startup");
		try {
			console.info (this.grid);
			if (!this.grid) {
				this.makeGrid();
			}

			if (this.store) {
				this.configureTableModel();
				this.afterViewLoad();
			} else if (this.global) {
				this.getView();
				this.getStore();
//  		dojo.connect (this.grid, 'onmouseover', this, 'getSelectedData');
			}
			
			this.grid.startup();
			if (this.autoSavePreferences) {
				this.connections.push (dojo.connect (this.grid, "resize", this, "onResize"));
			}
		} catch (ex) {
			console.error ("Caught exception in TIPTable.startup(): ");
			console.error (ex);
		}
		
		console.info ("exit TIPTable.startup");
	},
	
	afterViewLoad: function (rcfItems) {
		console.log ("Enter TIPTable.afterViewLoad");
		if (this.cmProvider) {
			this.configureContextMenu(rcfItems);  			
	  	}	  	
	},

	makeGrid: function () {
		console.info ("enter makeGrid");
		this.setGrid( new dojox.Grid(this.gridParms, dojo.byId(this.id + "gridContainer")), false);
		this.connections.push (dojo.connect (this.grid, "onRowClick", this, "onRowClick"));
		this.connections.push (dojo.connect (this.grid, "onRowDblClick", this, "onRowDblClick"));
	},
  
   getContextMenuAdapter: function()
   {
      if (this._contextMenuAdapter === null) {
         // Create the contextMenuAdapter.
         this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenuV2({cmProvider: this.cmProvider});
      }
      return this._contextMenuAdapter;
   },
  
    /************************************************************************
    * This method is called each time a context menu is requred on a row
    ************************************************************************/
    _onRowContextMenu: function( e )
    { 
      var row = e.grid.model.getRow( e.rowIndex );
      
      if ( row )
      {
        //select the row that the context menu was initiated on
        e.grid.selection.unselectAll( e.rowIndex );
        e.grid.selection.setSelected( e.rowIndex, true );
                
        //tell the context menu adapter to show the menu
        // no context is provided because we overload the _loadMenuItems method
        this.getContextMenuAdapter().contextMenuOpen( e, {selectedData: this.getSelectionData()} );
      }
    },

	configureContextMenu: function (rcfItems) {
		if ((this.cmProvider.parentid === null) || (this.cmProvider.parentid === "")) {
			this.cmProvider.setParentId (this.id);
		}
		if ((this.cmProvider.rcfItems === null) && (rcfItems)) {
			this.cmProvider.rcfItems = rcfItems;
		}

      this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
      	 //FIXES CONTEXT MENU BUG with selected table rows IN DOJO 1.0.2
		 this.grid.focus.styleRow = function(inRow){};
      
	},
	
	getView: function () {
      if (this.clientViewStructure === null) {
          var kw = {
                      content:   {
                                    command: "getTableColumns",
                                    arg_fqTableId: this.id,
                                    arg_portletNamespace: this.global.namespace
                                 },
                      timeout:   this.requestTimeout,
                      onResponse:{
                                    callback: dojo.hitch(this, this.getViewCallback)
                                 }
                   };

          if (this.tableArgs) {
             for (var key in this.tableArgs) {
                if (key) {
                      kw.content[key] = this.tableArgs[key];
                }	
             }
          }

          ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
      }
      else
      {
         this.setupView( this.clientViewStructure, this.clientRfcItems );
      }
	},
	
	handleError: function (response, ioArgs) {
		console.error ("HTTP status code: ", ioArgs.xhr.status);
		return response;
	},
	
	handleTimeout: function (response, ioArgs) {
		console.error ("Timeout for request: " + ioArgs.xhr);
		return response;
	},
	
	setupView: function (structure, rcfItems) {
		this.grid.setStructure( structure );	
		this.searchField.setTable( this.grid );
 	    this.afterViewLoad(rcfItems);
	},
  
	getViewCallback: function (data, ioArgs) {
      var view1 = { cells: [ data.cells ] };
      // a grid layout is an array of views.
      var layout = [ view1 ];
      this.setupView( layout, data.rcfItems );
	},
  
	getStore: function () {
	  	console.info ("enter getStore");
		// TODO: Change doClientPaging:false when paging is fixed
	  	this.store = new ibm.tivoli.tip.dojo.TIPQueryReadStore ({ global: this.global, tableID: this.id, jsId: "jsonStore", doClientPaging:false, requestTimeout: this.requestTimeout });
  		console.log ("got the store: ");
	  	console.log (this.store);
//	  	this.store.fetch();
	  	this.configureTableModel();
  	},
  	
	onRefresh: function() {
//		this.setLoading();
	
		//create a new model to force a refresh
		//this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: true });

		//clear table
		//this.grid.setModel( this._model );
		
		//post publish event
		//dojo.publish (this.id + "#Refresh", [ "Refresh" ]);

        // What is going on here?
        // Previously, the preceding three lines were used in this function to refresh
        // the table.  Create a new model, set the model, and issue the event.  However,
        // the problem is that TIPTable gets its data from the server, so the new model
        // was set prior to it having any data.  This resulted in the table 
        // "flashing".  That is the table would go all blank, then turn around and fill
        // with the data from the server.  This didn't look good, so the below code fixes
        // this.  A callback was added to the store so that when it finishes handling
        // data from the server, it will call the callback.  This enables us to first 
        // create the model, and load its first page, then swap it in to the table.
        
        // Create the callback function that will swap in the new model when it has some data
        var cb = dojo.hitch(this,function(){  console.debug("----- Inside the refresh callback ----");
                                              this.store.setLoadCompleteCallback(null);
                                              this._model=this.my_new_model; 
                                              this.grid.setModel(this.my_new_model);
                                              // I found that the following update is needed when only
                                              // one row is in the model.
                                              this.grid.update();
                                              this.my_new_model = null; 
                                              this.updateFooter(this._model.id);
                                              dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
                                            });
                                            
        this.store.setLoadCompleteCallback(cb);  // set the callback

		this.my_new_model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: false });
		// preserve sort on a refresh
		this.my_new_model.sortColumn = this._model.sortColumn;
		
		// Now need to set the row count equal to the rowsPerPage and make a request for the first page
		// in order to pre-load the model.  Notice that when this request completes, our callback above
		// will be invoked.
		this.my_new_model.setRowCount(20);
		this.my_new_model.getRow(0);
		
	},
	
	refreshWithArgs: function (args) {
		this.tableArgs = args;
		var dataQuery = {query:this.query};
	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
	  				dataQuery[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}


		//create a new model to force a refresh
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:dataQuery, clientSort: false });
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
  	
  	configureTableModel: function () {
  		console.info ("enter TIPTable.configureTableModel");
  		this.query = '';
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: false });
		this.grid.setModel( this._model );
		console.info ("exit TIPTable.configureTableModel");
	},
	
	search: function (columns, value) {
		console.log ("enter search with columns: ");
		console.log (columns);
		console.log ("and value: ");
		console.log (value);
		this.query = '';
		for (var x in columns) {
			if (this.query === '') {
				this.query += columns[x];
			} else {
				this.query += "," + columns[x];
			}
		}
		this.query += ":" + value;
		console.log ("in search with query: " + this.query);
		// preserve sort
		var sort = this._model.sortColumn;
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: false });
		this._model.sortColumn = sort;
		
		//clear table
		this.grid.setModel( this._model );
	},

	getSelectedRowIndexes: function () {
		return this.grid.selection.getSelected();
	},
	
	getValueAt: function (row, column) {
		return this.grid.model.getDatum (row, column);
	},
	
	getViewColumnsIDs: function () {
	  	var columns = this.grid.structure[0].cells[0];
	  	var columnIDs = [];
	  	for ( var i in columns ) {
	  		if (i) {
				columnIDs.push (columns[i].field);
			}
		}
		
		return (columnIDs);
	},
	
	getAllColumnIndexes: function () {
		var count = this.grid.model.getColCount();
		var columnIDs = [];
		for (var i=0; i<count; i++) {
			columnIDs.push (i);
		}
		
		return (columnIDs);
	},
	
	isNamingAttribute: function (field) {
	  	var columns = this.grid.structure[0].cells[0];
	  	if (columns[field]) {
		  	return columns[field].isNamingAttribute;
		} else {
			return (false);
		}
	},

	getModelData: function () {
		var selectedRows = this.getSelectedRowIndexes ();
		var row = selectedRows[0];
		var columnIDs = this.getAllColumnIndexes();

		for (var column in columnIDs) {
			if (column) {
				var data = this.grid.model.getDatum (row, column);
				console.log ("cell value for row: " + row + ", and column: " + column + " : " + data);
			}
		}
	},
	
	getIdentifier: function () {
//	  	var columns = this.grid.structure[0].cells[0];
	  	return (this.grid.model.getColCount() - 1);
	},

	getSelectionData: function () {
		this.getModelData();
		var selectedRows = this.getSelectedRowIndexes ();
		console.log ("selectedRows: " + selectedRows);

		var rowData = null;
		var columnIDs = this.getAllColumnIndexes();
		var identifier = this.getIdentifier();
		var rowJSON = "{ ";
		
		dojo.forEach (selectedRows,
			function (row) {
				var firstTime = true;
				rowJSON += '"id": "' + this.getValueAt (row, identifier) + '", ';
			
				for (var column in columnIDs) {
					if (column) {
						console.log ("getting cell value for row: " + row + " and column: " + column);
						var cellValue = this.getValueAt (row, column);
						if (!firstTime) {
							rowJSON += ', ';
						} else {
							firstTime = false;
						}
						rowJSON += '"' + column + '": "' + cellValue + '"';
					}
				}
			}, this);

		rowJSON += "}";
		
		var columnsJSON = "";
		var firstLoop = true;
		for (var c in columnIDs) {
			if (c) {
				if (!firstLoop) {
					columnsJSON += ', ';
				} else {
					firstLoop = false;
				}
				columnsJSON += '{ "field": "' + c + '", "isNamingAttribute": "' + this.isNamingAttribute(c) + '" }';
			}
		}
		
		var resultStr = "result = {";
		resultStr += "columns: [ ";
		resultStr += columnsJSON;
		resultStr += " ], rows: ";
		resultStr += rowJSON;
		resultStr += "};";
		
		console.log ("resultStr: " + resultStr);

		var result = eval (resultStr);
		
//		var sample = { 
//		columns: [{ field: "srcToken", hidden: true },
//				  { field: "ManagedSystemName", label: "Managed System Name" },
//				  { field: "column_2", label: "baz" }],
//		
//		 rows: { srcToken: "1234", ManagedSystemName: "bar", column_2: "bang" }
//		 		};
		
		return (result);
	},

	onRowClick: function (evt) {
		console.log ("enter TIPTable.onRowClick");
		this.updateFooter (this._model.id);
	},

    // The following processes double clicks on a row.  The user will add the double
    // click context on construction such as:
    // var dblClickContext = { content: { command: "myDblClickCommand" },
    //                         callback: dojo.hitch(this, this.myDblClickAction)  
    //                       };
    // and include it on the declaration of TIPTable with "rowDblClick: dlbClickContext".
    // Acceptable attributes:
    //     content: (optional) content to be sent to the callback
    //     callback: (required) a javascript function to be called.  It can have the form:
    //             myfunction: function(command, table)
    //                   where command is the command passed in the context,
    //                         table is a reference to this table
    
	onRowDblClick: function (evt) { 
		console.log ("enter TIPTable.onRowDblClick");
		if (this.rowDblClick) {
		    if (this.rowDblClick.callback) {
		        this.rowDblClick.callback(this, this.rowDblClick.content);
		    }
		}
	},

	updateFooter: function (modelID) {
		if (modelID == this._model.id) {
			var numSelected = 0;
			var total = this._model.getRealRowCount();
			var selected = this.getSelectedRowIndexes();
			if (selected) {
				numSelected = selected.length; 
			}
			var footerText = dojo.string.substitute(this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: total });
			this.footer.innerHTML = '<p class="footerLabel">' + footerText +  '</p>';
		}
	},
	
	getPreferences: function () {
		console.log ("enter TIPTable.getPreferences");
		var preferences = null;
		
		if (this.grid.structure) {
			var ids = this.getViewColumnsIDs();
			preferences = '{ "columns" : [ ';
		
			for (var x in ids) {
				if (x) {
					preferences += ' {"id": "' + x + '", "width":"' + this.grid.getCell(x).unitWidth + '"} ';
					if (x < ids.length - 1) {
						preferences += ", ";
					}
				}
			}
			preferences += " ] }";
			console.log ("cell widths: ");
			console.log (preferences);
		} else {
			console.log ("grid has no structure yet");
		}
				
		return (preferences);
	},
	
	onResize: function () {
		console.log ("enter TIPTable.onResize");
		// make sure that the grid has structure already.
		var preferences = this.getPreferences();
		
		if (this.autoSavePreferences && 
			(preferences !== this.lastPreferences)) {
		         var kw = {
                      content:   {
                                    command: "saveTablePreferences",
                                    arg_fqTableId: this.id,
                                    arg_portletNamespace: this.global.namespace,
                                    arg_preferences: preferences
                                 },
                      timeout:   this.requestTimeout
                   };

   	      ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
          this.lastPreferences = preferences;
        }
	},
	
	
	
	destroy: function () {
		console.info ("enter TIPTable.destroy");
		dojo.forEach (this.tooltips, function (tt) {
			tt.destroy();
		});
		dojo.forEach (this.topics, function (topic) {
			dojo.unsubscribe (topic);
		});
		dojo.forEach (this.connections, function (conn) {
			dojo.disconnect (conn);
		});
		
		// remove the context menu because it lives outside of this domNode		
		if (this._contextMenu) {
			this._contextMenu.removeAllItems();
			this._contextMenu.destroy();
			dijit.registry.remove (this._contextMenu.id);
		} else {
			console.info ("no context menu");
		}

      if (this._onRowContextMenuListener) {
         dojo.disconnect(this._onRowContextMenuListener);
      }
      this._contextMenuAdapter = null;
      if (this.cmProvider) {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }

		this.inherited (arguments);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableFooter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableFooter"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableFooter");
/******************************************************************************
* The footer for the TIP Table
*
* Authors: Joseph Firebaugh, Joshua Allen
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableFooter",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,

  templateString:"<table class=\"tableFooter\" cellspacing=\"0\" cellpadding=\"0\" border=\"0\">\r\n\t<tbody>\r\n\t\t<tr class=\"isc-table-footer\" dir=\"ltr\">\r\n\t\t\t<td class=\"isc-table-footer-cap-l\" valign=\"bottom\" nowrap=\"nowrap\" align=\"left\">\r\n\t\t\t</td>\r\n\t\t\t<td class=\"isc-table-footer-data\" width=\"100%\" nowrap=\"nowrap\" align=\"left\"> \r\n\t\t\t\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t\t\t</td>\t\t\t\r\n\t\t\t<td class=\"isc-table-footer-data\" nowrap=\"nowrap\" valign=\"middle\" align=\"right\"> \r\n\t\t\t\t<div dojoAttachPoint=\"infoArea\" class=\"footerLabel\"></div>\r\n\t\t\t</td>\r\n\t\t\t<td class=\"isc-table-footer-cap-r\" valign=\"bottom\" nowrap=\"nowrap\" align=\"right\">\r\n\t\t\t</td>\r\n\t\t</tr>  \r\n\t</tbody>\r\n</table>\r\n",
  
  //the dojox.Grid that this footer belongs to
  table: null,
  
  constructor: function () 
  {
    this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");    
  },
  
  postCreate: function()
  {
    if ( this.table !== null )
    {
      this.setTable( this.table );
    }        
  },
  
  setTable: function( table )
  {
    this.table = table;
    
    this._rowClickListener = dojo.connect( this.table, "onRowClick", this, "_onRowClick");            
    this._updateListener   = dojo.connect( this.table, "setModel",   this, "_onRowClick" );            
    
  },
  
  _onRowClick: function()
  {      
    var numSelected = 0;
    var totalRows   = 0;
    
    if ( this.table.model )
    {
		if (this.table.model.getRealRowCount) {
			totalRows = this.table.model.getRealRowCount();
		} else {
			totalRows = this.table.model.getRowCount();
		}
    }

    numSelected = this.table.selection.getSelectedCount();
    
    this.updateFooter( numSelected, totalRows );
  },
    
  updateFooter: function( numSelected, totalRows )
  {
    this.setInfoAreaHTML( dojo.string.substitute( this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: totalRows } ) );  
  },
  
  setInfoAreaHTML: function ( html )
  {
    this.infoArea.innerHTML = html;
  },
  
  destroy: function()
  {
    if ( this._rowClickListener ) { dojo.disconnect( this._rowClickListener ); }
    if ( this._updateListener )   { dojo.disconnect( this._updateListener ); }
    
    this.toolbar.destroy();
    
    this.inherited( "destroy", arguments );
  }  
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableRefreshButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableRefreshButton"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/



// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableRefreshButton");
/******************************************************************************
* Provides Table Refresh button that allows for "busy" animation. 
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableRefreshButton", dijit.form.Button,
{
    baseClass: "tipTableRefreshButton",

	templateString:"<div class=\"dijit dijitLeft dijitInline ${baseClass}\"\n\tdojoAttachEvent=\"onclick:_onButtonClick,onmouseenter:_onMouse,onmouseleave:_onMouse,onmousedown:_onMouse\"\n\t><div class='dijitRight'\n\t\t><button class=\"dijitStretch dijitButtonNode dijitButtonContents\" dojoAttachPoint=\"focusNode,titleNode\"\n\t\t\ttype=\"${type}\" waiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitInline ${iconClass}\" dojoAttachPoint=\"iconNode\" \n \t\t\t\t><span class=\"dijitToggleButtonIconChar\">&#10003</span \n\t\t\t></span\n\t\t\t><span class=\"dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\">${label}</span\n\t\t></button\n\t></div\n></div>\n",
    
    postCreate: function()
    {
		var resources = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");
		this.titleNode.title = resources.REFRESH;
    }
    
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableTitleBar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableTitleBar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/


dojo.provide("ibm.tivoli.tip.dijit.TIPTableTitleBar");
/******************************************************************************
* Provides title bar for table that looks like TIP portlet title
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableTitleBar", [dijit._Widget, dijit._Templated],
{
  label: "",
  
  templateString: "<div class='tipTableTitleBar'><div class='tipTableTitleBarLabel' dojoAttachPoint='titleNode'>${label}</div></div>",

  /*****************************************************************************
  * Programmatically allow changing of the table title. String argument can
  * containe any HTML.
  *****************************************************************************/
  setTitle: function( label /*string*/ )
  {
    this.titleNode.innerHTML = label;
  }  

});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableToolbar"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableToolbar"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/






dojo.provide("ibm.tivoli.tip.dijit.TIPTableToolbar");

/******************************************************************************
* A dijit.Toolbar and a TIPTableSearchField combined into one.
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableToolbar",
            [ dijit._Widget, dijit._Templated ],
{
  widgetsInTemplate: true,
  
  templateString:"<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" class=\"tableToolbar\">\r\n<tr>\r\n\t<td class=\"TIPTableToolbarArea\">\r\n\t<div dojoType=\"dijit.Toolbar\" dojoAttachPoint=\"toolbar\" class=\"tableToolbarArea\" ></div>\r\n\t</td>\r\n    <td class=\"TIPTableSearchFieldArea\">\r\n    <div dojoType=\"ibm.tivoli.tip.dijit.TIPTableSearchField\" dojoAttachPoint=\"searchField\"></div>\r\n    </td>\r\n</tr>\r\n</table>\r\n",
  
  getToolbar: function()
  {
    return this.toolbar;
  },
  
  destroy: function()
  {
  	this.searchField.destroy();
  	this.toolbar.destroy();
  	
  	this.inherited( arguments );
  }    
});

}

if(!dojo._hasResource["dijit.layout.LayoutContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.LayoutContainer"] = true;
dojo.provide("dijit.layout.LayoutContainer");



dojo.declare(
	"dijit.layout.LayoutContainer",
	dijit.layout._LayoutWidget,
{
	// summary
	//	Provides Delphi-style panel layout semantics.
	//
	// details
	//	A LayoutContainer is a box with a specified size (like style="width: 500px; height: 500px;"),
	//	that contains children widgets marked with "layoutAlign" of "left", "right", "bottom", "top", and "client".
	//	It takes it's children marked as left/top/bottom/right, and lays them out along the edges of the box,
	//	and then it takes the child marked "client" and puts it into the remaining space in the middle.
	//
	//  Left/right positioning is similar to CSS's "float: left" and "float: right",
	//	and top/bottom positioning would be similar to "float: top" and "float: bottom", if there were such
	//	CSS.
	//
	//	Note that there can only be one client element, but there can be multiple left, right, top,
	//	or bottom elements.
	//
	// usage
	//	<style>
	//		html, body{ height: 100%; width: 100%; }
	//	</style>
	//	<div dojoType="dijit.layout.LayoutContainer" style="width: 100%; height: 100%">
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="top">header text</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="left" style="width: 200px;">table of contents</div>
	//		<div dojoType="dijit.layout.ContentPane" layoutAlign="client">client area</div>
	//	</div>
	//
	//	Lays out each child in the natural order the children occur in.
	//	Basically each child is laid out into the "remaining space", where "remaining space" is initially
	//	the content area of this widget, but is reduced to a smaller rectangle each time a child is added.
	//	

	layout: function(){
		dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
	},

	addChild: function(/*Widget*/ child, /*Integer?*/ insertIndex){
		dijit._Container.prototype.addChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	},

	removeChild: function(/*Widget*/ widget){
		dijit._Container.prototype.removeChild.apply(this, arguments);
		if(this._started){
			dijit.layout.layoutChildren(this.domNode, this._contentBox, this.getChildren());
		}
	}
});

// This argument can be specified for the children of a LayoutContainer.
// Since any widget can be specified as a LayoutContainer child, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// layoutAlign: String
	//		"none", "left", "right", "bottom", "top", and "client".
	//		See the LayoutContainer description for details on this parameter.
	layoutAlign: 'none'
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV2"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV2"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/










// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableV2");
dojo.provide("ibm.tivoli.tip.dijit.TableFilterAdapter");
/******************************************************************************
* Provides dojox.grid within a layout that contains Title Bar (optional),
* toolbar, filter, and footer.  You must pass all Grid parameters on
* construction.
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableV2", dijit.layout.LayoutContainer,
{
  /*** PRIVATE VARIABLES ***/
  _backupModel: null,     /*temporary storage of original model when filtering is on*/
  
  /*** INSTANCE VARIABLES ***/
  label:            null,  /* set label to have table title bar */
  filterAdapter:    null,  /* provide a custom filter adapter if warrented */
  tableParameters:  null,  /* required dojox.Grid params */
  addRefreshButton: true,  /* have refresh button? */
  
  multiselect:      false, /* table has multi-select? */
  
  /*****************************************************************************
  * After Grid Construction -- setup toolbar and footer
  *****************************************************************************/
  postCreate: function()
  {
    //ensure "super()" is called
    this.inherited( "postCreate", arguments );
    
    var cPane = new dijit.layout.ContentPane( { layoutAlign: "top" } );
    this.addChild( cPane );
            
    var div = dojo.doc.createElement( 'div' );
    cPane.setContent( div );

    //-----------------------------
    // Conditionally add Title bar
    //-----------------------------
    if ( this.tableTitle !== null )
    {
      this.titleBar = new ibm.tivoli.tip.dijit.TIPTableTitleBar( { label: this.label }, div );
    
      cPane = new dijit.layout.ContentPane( { layoutAlign: "top" } );
      this.addChild( cPane );

      div   = dojo.doc.createElement( 'div' );
      cPane.setContent( div );
    }

    //-----------------------------
    // Add Toolbar and filter area
    //-----------------------------
    this.filterBar = new ibm.tivoli.tip.dijit.TIPTableToolbar( {}, div );
    
    //-----------------------------
    // Create and add dojox.Grid            
    //-----------------------------    
    cPane = new dijit.layout.ContentPane( { layoutAlign: "client" } );
    cPane.style.padding="0px";
    cPane.style.margin="0px";    
    this.addChild( cPane );

    div   = dojo.doc.createElement( 'div' );
    cPane.setContent( div );

    dojo.connect( cPane, "resize", this, function() 
    {
     this.grid.resize();
    });    

    this.grid = new dojox.Grid( this.tableParameters, div );

    this.grid.selection.multiSelect = this.multiselect;    
    
    this._modelListener = dojo.connect( this.grid, "setModel", this, "clearFilter" );    
    //-----------------------------
    // Create and add footer            
    //-----------------------------    
    cPane = new dijit.layout.ContentPane( { layoutAlign: "bottom" } );
    this.addChild( cPane );

    div   = dojo.doc.createElement( 'div' );
    cPane.setContent( div );

    this.footer = new ibm.tivoli.tip.dijit.TIPTableFooter( {}, div );        
    this.footer.setTable( this.grid );
    
    /** Add refresh button **/
    if ( this.addRefreshButton )
    {
      this.filterBar.toolbar.addChild( new ibm.tivoli.tip.dijit.TIPTableRefreshButton( { onClick: dojo.hitch( this, "onRefresh" ) } ) );
    }
    
    this.filterBar.searchField.setTable( this.grid );    
    dojo.subscribe ( this.filterBar.searchField.id + "searchInvoked", this, "onFilter" );        
    
    this.resize();
  },
  
  /*****************************************************************************
  * Refresh button was pressed
  *****************************************************************************/
  onRefresh: function()
  {
    //post publish event
    dojo.publish (this.id + "#Refresh", [ "Refresh" ]);  
  },
  
  /*****************************************************************************
  * Enter was pressed on the Filter Fields
  *****************************************************************************/
  onFilter: function()
  {
     var fields       = this.filterBar.searchField.getSelectedColumns();
     var filterString = this.filterBar.searchField.searchField.value;
     
     if ( filterString.length < 1 )
     {
       this.clearFilter();
     }
     else
     {
       this.filter( fields, filterString );
     }
  },
    
  /****************************************************************************
  * Client side filtering of data model currently set on the table
  ****************************************************************************/
  filter: function( fields,        /* array of field numbers to filter on */
                    filterString ) /* simple string that will perform AND search in all fields */
  {
    if ( this._backupModel === null )
    {
      this._backupModel = this.grid.model;
    }
    
    if ( this.filterAdapter === null )
    {
      this.filterAdapter = new ibm.tivoli.tip.dijit.TableFilterAdapter();
    }
            
    var newModelData = [];
        
    var rowCount = this._backupModel.getRowCount();
    var rowData  = null;
    
    for ( var i = 0; i < rowCount; i++ )
    {
      var keep    = false;
      rowData = this._backupModel.getRow( i );
      
      for ( var j in fields )
      {
        if ( this.filterAdapter.rowIsVisible( rowData[ fields[j] ], filterString ) )
        {
          newModelData.push( rowData );
          break;
        }
      }
    }
    
    var newModel = new this._backupModel.constructor(null, newModelData);
    
    //borrowed this code from Grid.js -- can't call this.setModel() because clearFilter will occur
    if( this.grid.model )
    {
	  this.grid.model.notObserver( this.grid );
    }
    
    this.grid._setModel( newModel );
    
    this.grid.update();    
  },
  
  /****************************************************************************
  * Clear client side filter
  ****************************************************************************/
  clearFilter: function()
  {
    if ( this._backupModel !== null )
    {
      //borrowed this code from Grid.js -- can't call this.setModel() because clearFilter will occur
      if( this.grid.model )
      {
        this.grid.model.notObserver( this.grid );
      }

      this.filterBar.searchField.searchField.value = "";

      this.grid._setModel( this._backupModel );
            
      this._backupModel = null;
      
      this.grid.update();    
    }
  },
  
  destroy: function()
  {
    dojo.disconnect( this._modelListener );
    
    this.clearFilter();
    this.filterBar.searchField.destroy();
    this.filterBar.destroy();
    this.footer.destroy();
    
    this.grid.destroy();    
    
    this.inherited( "destroy", arguments );
  }
      
});

/******************************************************************************
* Filter Adapter (Simple Implementation -- extend and create new ones if 
* needed)
*
* This simple implementation looks for any instance of a string with the
* string form of the object -- case insensitive.
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TableFilterAdapter", null,
{
  /*****************************************************************************
  * This row visible or filtered out?
  *
  * This simple implementation looks for any instance of a string with the
  * string form of the object -- case insensitive.
  *
  * cellData = raw javascript object in table model for a cell
  *****************************************************************************/
  rowIsVisible: function( cellData, filterString )
  { 
    var cellDataStr = cellData.toString().toLowerCase();
     
    return ( cellDataStr.indexOf( filterString.toLowerCase() ) != -1 );
  }
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV3"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV3"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * 5724-C51
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/









// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableV3");
dojo.provide("ibm.tivoli.tip.dijit.TableFilterAdapter");
/******************************************************************************
* Extends dojox.data.Grid to add client side filtering and a toolbar.<b> 
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableV3", dojox.Grid,
{
  _backupModel: null, /*temporary storage of original model when filtering is on*/
  
  filterAdapter: null, /*provide a custom filter adapter if warrented*/
  
  /*****************************************************************************
  * Constructor
  *****************************************************************************/
  constructor: function()
  {
    //ensure that any model setting clears table filter
    this._modelListener = dojo.connect( this, "setModel", this, "clearFilter" );
  },
  
  /*****************************************************************************
  * After Grid Construction -- setup toolbar and footer
  *****************************************************************************/
  postCreate: function()
  {
    //ensure "super()" is called
    this.inherited( "postCreate", arguments );
        
    var div = dojo.doc.createElement( 'div' );
    dojo.place(  div, this.domNode, "before"  );
        
    this.filterBar = new ibm.tivoli.tip.dijit.TIPTableToolbar( {}, div );
    this.filterBar.searchField.setTable( this );    
        
    dojo.subscribe ( this.filterBar.searchField.id + "searchInvoked", this, "onFilter" );    
    
    div = dojo.doc.createElement( 'div' );
    dojo.place(  div, this.domNode, "after"  );
        
    this.footer = new ibm.tivoli.tip.dijit.TIPTableFooter( {}, div );        
    this.footer.setTable( this );
    
    var button = new ibm.tivoli.tip.dijit.TIPTableRefreshButton( { onClick: dojo.hitch( this, "onRefresh" ) } );    
    this.filterBar.toolbar.addChild( button );
  },
  
  onRefresh: function()
  {
    //post publish event
    dojo.publish (this.id + "#Refresh", [ "Refresh" ]);  
  },
  
  /*****************************************************************************
  * Enter was pressed on the Filter Fields
  *****************************************************************************/
  onFilter: function()
  {
     var fields       = this.filterBar.searchField.getSelectedColumns();
     var filterString = this.filterBar.searchField.searchField.value;
     
     if ( filterString.length < 1 )
     {
       this.clearFilter();
     }
     else
     {
       this.filter( fields, filterString );
     }
  },
    
  /****************************************************************************
  * Client side filtering of data model currently set on the table
  ****************************************************************************/
  filter: function( fields,        /* array of field numbers to filter on */
                    filterString ) /* simple string that will perform AND search in all fields */
  {
    if ( this._backupModel === null )
    {
      this._backupModel = this.model;
    }
    
    if ( this.filterAdapter === null )
    {
      this.filterAdapter = new ibm.tivoli.tip.dijit.TableFilterAdapter();
    }
            
    var newModelData = [];
        
    var rowCount = this._backupModel.getRowCount();
    var rowData  = null;
    
    for ( var i = 0; i < rowCount; i++ )
    {
      var keep     = false;
      rowData = this._backupModel.getRow( i );
      
      for ( var j in fields )
      {
        if ( this.filterAdapter.rowIsVisible( rowData[ fields[j] ], filterString ) )
        {
          newModelData.push( rowData );
          break;
        }
      }
    }
    
    var newModel = new this._backupModel.constructor(null, newModelData);
    
    //borrowed this code from Grid.js -- can't call this.setModel() because clearFilter will occur
    if( this.model )
    {
	  this.model.notObserver(this);
    }
    this._setModel( newModel );
    
    this.update();    
  },
  
  /****************************************************************************
  * Clear client side filter
  ****************************************************************************/
  clearFilter: function()
  {
    if ( this._backupModel !== null )
    {
      //borrowed this code from Grid.js -- can't call this.setModel() because clearFilter will occur
      if( this.model )
      {
        this.model.notObserver(this);
      }

      this.filterBar.searchField.searchField.value = "";

      this._setModel( this._backupModel );
            
      this._backupModel = null;
      
      this.update();    
    }
  },
  
  destroy: function()
  {
    dojo.disconnect( this._modelListener );
    
    this.clearFilter();
    this.filterBar.searchField.destroy();
    this.filterBar.destroy();
    this.footer.destroy();    
    
    this.inherited( "destroy", arguments );
  }
      
});

/******************************************************************************
* Filter Adapter (Simple Implementation -- extend and create new ones if 
* needed)
*
* This simple implementation looks for any instance of a string with the
* string form of the object -- case insensitive.
*
* Authors: Joseph Firebaugh
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TableFilterAdapter", null,
{
  /*****************************************************************************
  * This row visible or filtered out?
  *
  * This simple implementation looks for any instance of a string with the
  * string form of the object -- case insensitive.
  *
  * cellData = raw javascript object in table model for a cell
  *****************************************************************************/
  rowIsVisible: function( cellData, filterString )
  { 
    var cellDataStr = cellData.toString().toLowerCase();
     
    return ( cellDataStr.indexOf( filterString.toLowerCase() ) != -1 );
  }
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV4"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dijit.TIPTableV4"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2007 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/






dojo.require("dijit.layout.ContentPane" );













// NLS


dojo.provide("ibm.tivoli.tip.dijit.TIPTableV4");
dojo.provide("ibm.tivoli.tip.dijit.TIPTableContainerV4");
/******************************************************************************
* This container places the search bar and toolbar around a given 
* dojox.grid.Grid 
*
* Authors: Joseph Firebaugh, Joshua Allen, Ken Parzygnat
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableContainerV4",
             dijit.layout.LayoutContainer,
{
	/* Do you want the default buttons added to the toolbar? */
	makeDefaultButtons: true,	

    label: null,  /* set label to have table title bar */

	/* Reference to the dojox.grid.Grid */
	grid: null,
	
	/* Default timeout for requests made to the server */
	requestTimeout: 30000,
	
	multiSelect: true,
	rowDblClick: null,
	
	// null means auto width
	// 100% takes all available width
	width: "100%",
	
	height: "200px",

	constructor: function () 
	{
		this.resources_ = dojo.i18n.getLocalization("ibm.tivoli.tip.dijit", "resources");	
		this.tooltips = [];
		this.connections = [];
		this.topics = [];
	},
	
	resizeGrid: function ()
	{
        if ( this.grid ) 
        { 
          setTimeout( dojo.hitch( this.grid, "resize" ), 100 ); 
          setTimeout( dojo.hitch( this.grid, "update" ), 101 ); 
        }
	},
	
	/**************************************************************************
	* Called after template has been processed.
	*
	* Lets check to see if a dojox.grid.Grid was passed in on params.  If so,
	* add it to the DOM tree and setup.
	**************************************************************************/
	postCreate: function ()
	{
      //ensure "super()" is called
      this.inherited( "postCreate", arguments );
      
      this.cPane1 = new dijit.layout.ContentPane( { layoutAlign: "top" } );
      this.addChild( this.cPane1 );
              
      var div = dojo.doc.createElement( 'div' );
      this.cPane1.setContent( div );
  
      //-----------------------------
      // Conditionally add Title bar
      //-----------------------------
      if ( this.label !== null )
      {
        this.titleBar = new ibm.tivoli.tip.dijit.TIPTableTitleBar( { label: this.label }, div );
      
        this.cPane3 = new dijit.layout.ContentPane( { layoutAlign: "top" } );
        this.addChild( this.cPane3 );
  
        div   = dojo.doc.createElement( 'div' );
        this.cPane3.setContent( div );
      }
  
      //-----------------------------
      // Add Toolbar and filter area
      //-----------------------------
      this.filterBar = new ibm.tivoli.tip.dijit.TIPTableToolbar( {}, div );
      
      //-----------------------------
      // Create Grid Pane            
      //-----------------------------    
      this.gridPane = new dijit.layout.ContentPane( { layoutAlign: "client" } );
      //this.gridPane.style.paddingLeft="0px";
      //this.gridPane.style.paddingTop="0px";
      //this.gridPane.style.paddingRight="0px";
      //this.gridPane.style.paddingBottom="0px";
      this.gridPane.style.margin="0px";
          
      this.addChild( this.gridPane );
      
      this.gridDiv = dojo.doc.createElement( 'div' );
      this.gridPane.setContent( this.gridDiv );      
      
      dojo.connect( this.gridPane, "resize", this, "resizeGrid"  );
      
      //-----------------------------
      // Create and add footer            
      //-----------------------------    
      this.cPane2 = new dijit.layout.ContentPane( { layoutAlign: "bottom" } );
      this.addChild( this.cPane2 );
  
      div   = dojo.doc.createElement( 'div' );
      this.cPane2.setContent( div );
  
      this.footerToolbar = new ibm.tivoli.tip.dijit.TIPTableFooter( {}, div );        
      
      this.toolbar = this.filterBar.toolbar;      
      this.searchField = this.filterBar.searchField;
            		
      //if grid passed on params then set it
      if ( this.grid !== null )
      {
      	this.setGrid( this.grid );
      }
      
      //initialize the toolbar
      if ( this.makeDefaultButtons === true ) 
      { 
      	this.initializeToolbar(); 
      }
            
      this.topics.push (dojo.subscribe (this.searchField.id + "searchInvoked", this, "onSearch"));
	},
	
	/**************************************************************************
	* Can be called at any time to place a grid within this "container"
	**************************************************************************/
	setGrid: function( grid, append )
	{
		this.grid = grid;

		if ( append !== false ) 
		{
 	      //this.gridPane.setContent( grid );
		}
		
		this.searchField.setTable( grid );

		this.grid.selection.multiSelect = this.multiSelect;
	},
		
	/**************************************************************************
	* Get the dojox.grid.Grid object
	**************************************************************************/
	getGrid: function()
	{
		return this.grid;
	},
	
	/**************************************************************************
	* Get the dijit.Toolbar attached to the table
	**************************************************************************/
	getToolbar: function() 
	{
		return ( this.toolbar );
	},
	
	getFooterToolbar: function () 
	{
		return ( this.footerToolbar );
	},
	
	/**************************************************************************
	* Initialize the toolbar with default buttons (like refresh)
	**************************************************************************/
	initializeToolbar: function() 
	{
		this.refreshBtn = new ibm.tivoli.tip.dijit.TIPTableRefreshButton(
			{	id: this.id + "refreshButton",
			 	onClick: dojo.hitch (this, this.onRefresh) 
			});
		this.toolbar.addChild( this.refreshBtn );
		
		if (this.multiSelect) {
			this.toolbar.addChild (new dijit.ToolbarSeparator());
		
			this.selectAllBtn = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.SELECT_ALL, 
					showLabel: false, 
					iconClass: "selectAllIcon",
				 	onClick: dojo.hitch (this, this.selectAll) 
				});
			this.toolbar.addChild( this.selectAllBtn );
			
			this.deselectAllBtn = new ibm.tivoli.tip.dijit.TIPButton(
				{	label: this.resources_.DESELECT_ALL, 
					showLabel: false, 
					iconClass: "deselectAllIcon",
				 	onClick: dojo.hitch (this, this.deselectAll) 
				});
			this.toolbar.addChild( this.deselectAllBtn );
		}
	},
	
	/**************************************************************************
	* This method fires a widgetID#Refresh event when the refresh buttons is
	* pressed.  OR you can overload this method instead. If you overload this
	* method you should call inherited so the table is cleared of contents.
	**************************************************************************/
	onRefresh: function() 
	{
//		this.setLoading();
		
		//create empty model
		this._model = new dojox.grid.data.DojoData(null, null);
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
	
	setLoading: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "refreshIcon");
			dojo.addClass (btn, "loadingIcon");
		}
	},
	
	setSteady: function () 
	{
		var btn = dojo.byId (this.id + "refreshButton");
		if (btn) {
			dojo.removeClass (btn, "loadingIcon");
			dojo.addClass (btn, "refreshIcon");
		}
	},
	
	selectAll: function () 
	{
		var count = 0;
		if (this._model.getRealRowCount) {
			count = this._model.getRealRowCount();
		} else {
			count = this._model.getRowCount();
		}
		
		for (var i=0; i<count; i++) {
			this.grid.selection.addToSelection(i);
		}
		
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	deselectAll: function () 
	{
		this.grid.selection.unselectAll();
		if (this.updateFooter) {
			this.updateFooter (this._model.id);
		}
	},
	
	onSearch: function (tipSearchField, event) 
	{
		console.log ("enter onSearch with search value: " + tipSearchField.searchField.value);
		
		if (this.search) 
		{
			var columns = this.searchField.getSelectedColumns();
			this.search (columns, tipSearchField.searchField.value);
		}
	}
});

/******************************************************************************
* TIP Based Table that creates the underlying dojox.grid.Grid for you based on
* a given data store or Global that uses defined JSON returns for model data
* and column structure.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.TIPTableV4",
              ibm.tivoli.tip.dijit.TIPTableContainerV4,
{
	global: null,
	tableID: null,
	cmProvider: null,
	store: null,
	tableArgs: null,
	gridParms: { rowsPerPage: "10", rowCount: "5" },
	query: null,
	_contextMenu: null,
	_model: null,
   _onRowContextMenuListener: null,
   _contextMenuAdapter : null,
  
	constructor: function() {
	},
	
	postCreate: function () {
		this.inherited (arguments);
		this.topics.push (dojo.subscribe ("/ibm.tip/TIPDojoData#rowCountUpdated", this, "updateFooter"));
	},

	startup: function() {
		console.info ("enter TIPTable.startup");
		//try {
			console.info (this.grid);
			if (!this.grid) {
				this.makeGrid();
			}

			if (this.store) {
				this.configureTableModel();
				this.afterViewLoad();
			} else if (this.global) {
				this.getView();
				this.getStore();
//  		dojo.connect (this.grid, 'onmouseover', this, 'getSelectedData');
			}
			
			this.grid.startup();
		//} catch (ex) {
		//	console.error ("Caught exception in TIPTable.startup(): ");
		//	console.error (ex);
			
		//	alert( ex.message );
		//}
		
		console.info ("exit TIPTable.startup");
	},
	
	afterViewLoad: function (rcfItems) {
		if (this.cmProvider) {
			this.configureContextMenu(rcfItems);  			
	  	}	  	
	},

	makeGrid: function () 
	{

	try
	{
		console.info ("enter makeGrid");
		
		var grid = new dojox.Grid( this.gridParms, this.gridDiv );

		this.setGrid( grid , true );
		
		this.connections.push (dojo.connect (this.grid, "onRowClick", this, "onRowClick"));

		this.connections.push (dojo.connect (this.grid, "onRowDblClick", this, "onRowDblClick"));
	}catch (e){ alert( e.message ); }
		
	},
  
   getContextMenuAdapter: function()
   {
      if (this._contextMenuAdapter === null) {
         // Create the contextMenuAdapter.
         this._contextMenuAdapter = new ibm.tivoli.tip.dijit.TIPContextMenuV2({cmProvider: this.cmProvider});
      }
      return (this._contextMenuAdapter);
   },
  
    /************************************************************************
    * This method is called each time a context menu is requred on a row
    ************************************************************************/
    _onRowContextMenu: function( e )
    { 
      var row = e.grid.model.getRow( e.rowIndex );
      
      if ( row )
      {
        //select the row that the context menu was initiated on
        e.grid.selection.unselectAll( e.rowIndex );
        e.grid.selection.setSelected( e.rowIndex, true );
                
        //tell the context menu adapter to show the menu
        // no context is provided because we overload the _loadMenuItems method
        this.getContextMenuAdapter().contextMenuOpen( e, {selectedData: this.getSelectionData()} );
      }
    },

	configureContextMenu: function (rcfItems) {
		if ((this.cmProvider.parentid === null) || (this.cmProvider.parentid === "")) {
			this.cmProvider.setParentId (this.id);
		}
		if ((this.cmProvider.rcfItems === null) && (rcfItems)) {
			this.cmProvider.rcfItems = rcfItems;
		}

      this._onRowContextMenuListener = dojo.connect( this.grid, "onRowContextMenu", this, "_onRowContextMenu" );
      	 //FIXES CONTEXT MENU BUG with selected table rows IN DOJO 1.0.2
		 this.grid.focus.styleRow = function(inRow){};
      
	},
	
	getView: function () {
        var kw = {
                    content:   {
                                  command: "getTableColumns",
                                  arg_fqTableId: this.id,
                                  arg_portletNamespace: this.global.namespace
                               },
                    timeout:   this.requestTimeout,
                    onResponse:{
                                  callback: dojo.hitch(this, this.getViewCallback)
                               }
                 };
	  	
	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
		            kw.content[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}
	  	
        ibm.tivoli.tip.dojo.Server.getSingleton( this.global ).get( kw );
	},
	
	handleError: function (response, ioArgs) {
		console.error ("HTTP status code: ", ioArgs.xhr.status);
		return response;
	},
	
	handleTimeout: function (response, ioArgs) {
		console.error ("Timeout for request: " + ioArgs.xhr);
		return response;
	},
	
	getViewCallback: function (data, ioArgs) {
  		var view1 = { cells: [ data.cells ] };
		// a grid layout is an array of views.
		var layout = [ view1 ];
		this.grid.setStructure( layout );	
		this.searchField.setTable( this.grid );
		this.afterViewLoad(data.rcfItems);
	},
  
	getStore: function () {
	  	console.info ("enter getStore");
		// TODO: Change doClientPaging:false when paging is fixed
	  	this.store = new ibm.tivoli.tip.dojo.TIPQueryReadStore ({ global: this.global, tableID: this.id, jsId: "jsonStore", doClientPaging:false, requestTimeout: this.requestTimeout });
  		console.log ("got the store: ");
	  	console.log (this.store);
//	  	this.store.fetch();
	  	this.configureTableModel();
  	},
  	
	onRefresh: function() {
//		this.setLoading();
	
		//create a new model to force a refresh
		//this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: true });

		//clear table
		//this.grid.setModel( this._model );
		
		//post publish event
		//dojo.publish (this.id + "#Refresh", [ "Refresh" ]);

        // What is going on here?
        // Previously, the preceding three lines were used in this function to refresh
        // the table.  Create a new model, set the model, and issue the event.  However,
        // the problem is that TIPTable gets its data from the server, so the new model
        // was set prior to it having any data.  This resulted in the table 
        // "flashing".  That is the table would go all blank, then turn around and fill
        // with the data from the server.  This didn't look good, so the below code fixes
        // this.  A callback was added to the store so that when it finishes handling
        // data from the server, it will call the callback.  This enables us to first 
        // create the model, and load its first page, then swap it in to the table.
        
        // Create the callback function that will swap in the new model when it has some data
        var cb = dojo.hitch(this,function(){  console.debug("----- Inside the refresh callback ----");
                                              this.store.setLoadCompleteCallback(null);
                                              this._model=this.my_new_model; 
                                              this.grid.setModel(this.my_new_model);
                                              // I found that the following update is needed when only
                                              // one row is in the model.
                                              this.grid.update();
                                              this.my_new_model = null; 
                                              this.updateFooter(this._model.id);
                                              dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
                                            });
                                            
        this.store.setLoadCompleteCallback(cb);  // set the callback

		this.my_new_model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: true });
		// preserve sort on a refresh
		this.my_new_model.sortColumn = this._model.sortColumn;
		
		// Now need to set the row count equal to the rowsPerPage and make a request for the first page
		// in order to pre-load the model.  Notice that when this request completes, our callback above
		// will be invoked.
		this.my_new_model.setRowCount(20);
		this.my_new_model.getRow(0);
		
	},
	
	refreshWithArgs: function (args) {
		this.tableArgs = args;
		var dataQuery = {query:this.query};
	  	if (this.tableArgs) {
	  		for (var key in this.tableArgs) {
	  			if (key) {
	  				dataQuery[key] = this.tableArgs[key];
	  			}	
	  		}
	  	}


		//create a new model to force a refresh
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:dataQuery, clientSort: true });
		
		//clear table
		this.grid.setModel( this._model );
		
		//post publish event
		dojo.publish (this.id + "#Refresh", [ "Refresh" ]);
	},
  	
    configureTableModel: function () 
    {
  		console.info ("enter TIPTable.configureTableModel");
  		this.query = '';
        var cb = dojo.hitch(this,function(){  console.debug("----- Inside the load complete callback ----");
                                              dojo.publish (this.id + "#LoadComplete", [ "LoadComplete" ]);
                                            });

        this.store.setLoadCompleteCallback(cb);  // set the callback

		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query}, clientSort: true });
		this.grid.setModel( this._model );
		console.info ("exit TIPTable.configureTableModel");
	},
	
	search: function (columns, value) {
		console.log ("enter search with columns: ");
		console.log (columns);
		console.log ("and value: ");
		console.log (value);
		this.query = '';
		for (var x in columns) {
			if (this.query === '') {
				this.query += columns[x];
			} else {
				this.query += "," + columns[x];
			}
		}
		this.query += ":" + value;
		console.log ("in search with query: " + this.query);
		// preserve sort
		var sort = this._model.sortColumn;
		this._model = new ibm.tivoli.tip.dojo.TIPDojoData (null, null, { store: this.store, rowsPerPage: 20, query:{query:this.query, refresh:"true"}, clientSort: false });
		this._model.sortColumn = sort;
		
		//clear table
		this.grid.setModel( this._model );
	},

	getSelectedRowIndexes: function () {
		return (this.grid.selection.getSelected());
	},
	
	getValueAt: function (row, column) {
//		var cell = this.grid.views.views[0].getCellNode(viewRow, viewColumn);
//		return (cell.innerHTML);
		return (this.grid.model.getDatum (row, column));
	},
	
	getViewColumnsIDs: function () {
	  	var columns = this.grid.structure[0].cells[0];
	  	var columnIDs = [];
	  	for ( var i in columns ) {
	  		if (i) {
				columnIDs.push (columns[i].field);
			}
		}
		
		return (columnIDs);
	},
	
	getAllColumnIndexes: function () {
		var count = this.grid.model.getColCount();
		var columnIDs = [];
		for (var i=0; i<count; i++) {
			columnIDs.push (i);
		}
		
		return (columnIDs);
	},
	
	isNamingAttribute: function (field) {
	  	var columns = this.grid.structure[0].cells[0];
	  	if (columns[field]) {
		  	return (columns[field].isNamingAttribute);
		} else {
			return (false);
		}
	},

	getModelData: function () {
		var selectedRows = this.getSelectedRowIndexes ();
		var row = selectedRows[0];
		var columnIDs = this.getAllColumnIndexes();

		for (var column in columnIDs) {
			if (column) {
				var data = this.grid.model.getDatum (row, column);
				console.log ("cell value for row: " + row + ", and column: " + column + " : " + data);
			}
		}
	},
	
	getIdentifier: function () {
//	  	var columns = this.grid.structure[0].cells[0];
	  	return (this.grid.model.getColCount() - 1);
	},

	getSelectionData: function () {
		this.getModelData();
		var selectedRows = this.getSelectedRowIndexes ();
		console.log ("selectedRows: " + selectedRows);

		var rowData = null;
		var columnIDs = this.getAllColumnIndexes();
		var identifier = this.getIdentifier();
		var rowJSON = "{ ";
		
		dojo.forEach (selectedRows,
			function (row) {
				var firstTime = true;
				rowJSON += '"id": "' + this.getValueAt (row, identifier) + '", ';
			
				for (var column in columnIDs) {
					if (column) {
						console.log ("getting cell value for row: " + row + " and column: " + column);
						var cellValue = this.getValueAt (row, column);
						if (!firstTime) {
							rowJSON += ', ';
						} else {
							firstTime = false;
						}
						rowJSON += '"' + column + '": "' + cellValue + '"';
					}
				}
			}, this);

		rowJSON += "}";
		
		var columnsJSON = "";
		var firstLoop = true;
		for (var c in columnIDs) {
			if (c) {
				if (!firstLoop) {
					columnsJSON += ', ';
				} else {
					firstLoop = false;
				}
				columnsJSON += '{ "field": "' + c + '", "isNamingAttribute": "' + this.isNamingAttribute(c) + '" }';
			}
		}
		
		var resultStr = "result = {";
		resultStr += "columns: [ ";
		resultStr += columnsJSON;
		resultStr += " ], rows: ";
		resultStr += rowJSON;
		resultStr += "};";
		
		console.log ("resultStr: " + resultStr);

		var result = eval (resultStr);
		
//		var sample = { 
//		columns: [{ field: "srcToken", hidden: true },
//				  { field: "ManagedSystemName", label: "Managed System Name" },
//				  { field: "column_2", label: "baz" }],
//		
//		 rows: { srcToken: "1234", ManagedSystemName: "bar", column_2: "bang" }
//		 		};
		
		return (result);
	},

	onRowClick: function (evt) {
		console.log ("enter TIPTable.onRowClick");
		this.updateFooter (this._model.id);
	},

    // The following processes double clicks on a row.  The user will add the double
    // click context on construction such as:
    // var dblClickContext = { content: { command: "myDblClickCommand" },
    //                         callback: dojo.hitch(this, this.myDblClickAction)  
    //                       };
    // and include it on the declaration of TIPTable with "rowDblClick: dlbClickContext".
    // Acceptable attributes:
    //     content: (optional) content to be sent to the callback
    //     callback: (required) a javascript function to be called.  It can have the form:
    //             myfunction: function(command, table)
    //                   where command is the command passed in the context,
    //                         table is a reference to this table
    
	onRowDblClick: function (evt) { 
		console.log ("enter TIPTable.onRowDblClick");
		if (this.rowDblClick) {
		    if (this.rowDblClick.callback) {
		        this.rowDblClick.callback(this, this.rowDblClick.content);
		    }
		}
	},

	updateFooter: function ( modelID ) 
    {
		if ( modelID == this._model.id ) 
        {
			var numSelected = 0;
			var total = this._model.getRealRowCount();
			var selected = this.getSelectedRowIndexes();
			if (selected) {
				numSelected = selected.length; 
			}
			var footerText = dojo.string.substitute(this.resources_.TABLE_FOOTER_TEXT, { 0: numSelected, 1: total });

            
            this.getFooterToolbar().setInfoAreaHTML( footerText );
		}
	},
	
	
	destroy: function () {
		console.info ("enter TIPTable.destroy");
		dojo.forEach (this.tooltips, function (tt) {
			tt.destroy();
		});
		dojo.forEach (this.topics, function (topic) {
			dojo.unsubscribe (topic);
		});
		dojo.forEach (this.connections, function (conn) {
			dojo.disconnect (conn);
		});
		
		// remove the context menu because it lives outside of this domNode		
		if (this._contextMenu) {
			this._contextMenu.removeAllItems();
			this._contextMenu.destroy();
			dijit.registry.remove (this._contextMenu.id);
		} else {
			console.info ("no context menu");
		}

      if (this._onRowContextMenuListener) {
         dojo.disconnect(this._onRowContextMenuListener);
      }
      this._contextMenuAdapter = null;
      if (this.cmProvider) {
         this.cmProvider.destroy();
         this.cmProvider = null;
      }
      
      if( this.titleBar )
      {
      	this.titleBar.destroy();
      }
      
      this.refreshBtn.destroy();
      
      if ( this.selectAllBtn )
      {
      	this.selectAllBtn.destroy();
      }

      if ( this.deselectAllBtn )
      {
      	this.deselectAllBtn.destroy();
      }
      
      this.filterBar.destroy();
      this.footerToolbar.destroy();
      this.grid.destroy();

	  this.gridPane.destroy();

	  this.cPane1.destroy();
	  this.cPane2.destroy();
	  
	  if (  this.cPane3 )
	  {
	  	this.cPane3.destroy();
	  }
	  
	  this.inherited (arguments);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.tip.dojo.TIPHelper"] = true;
/******************************************************* {COPYRIGHT-TOP-OCO} ***
 * Licensed Materials - Property of IBM
 *
 * (C) Copyright IBM Corp. 2008 All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication, or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************* {COPYRIGHT-END-OCO} ***/
dojo.provide("ibm.tivoli.tip.dojo.TIPHelper");

dojo.declare("ibm.tivoli.tip.dojo.TIPHelper",
	          null,
{
	constructor: function () {
	},
	// childNode must be a node within your portlet
	// padding is the amount of padding you want around your portlet
	//   e.g., "0px"
	setPortletPadding: function (childNode, padding) {
		var node = this.getAncestorByClass (childNode, "isc-portlet-bkg");
		dojo.style (node, "padding", padding);
	    var nextnode = dojo.query( 'table[id$="_portlet_content"]', node )[0];
        if (nextnode) {
        	dojo.style( nextnode, "padding", padding );
        }
	},
	getAncestorByClass: function (element, className) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				if(dojo.hasClass (element, className)) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByName: function (element, name) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementname = element.getAttribute ("name");
				if (elementname == name) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
	},
	getAncestorByID: function (element, id) {
		var result=null;
		try {
			while(result===null && element!==null) {
				element=element.parentNode;
				var elementid = element.getAttribute ("id");
				if (elementid == id) {
					result=element;
				}
			}
		} catch(e) {
			//no node
		}
		//console.log ("Nodename = " + result);
		return(result);
		
	},
	getPortletOID: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
			console.log ("get portlet data: ");
			console.log (node.modelObject);
			var result = node.modelObject.portletData.oid;
			return (result);
		}
	},
	getPortletWindowID: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		if (node) {
			var result = node.modelObject.portletData.portletName;
			return (result);
		}
	},
	getPortletPageID: function (element) {
		var node = this.getAncestorByName (element, "IscPage");
		if (node) {
			var result = node.modelObject.pageData.uniquePageName;
			return (result);
		}
	},
	getPortletPageName: function (element) {
		var parent = this.getAncestorByName (element, "IscPage");
		if (parent) {
		var node = dojo.query ("td.isc-layout-titletext", parent);
//		var node = dojo.query ("td.isc-tab-sel-l", frames["taskbar"].document);
		if (node) {
			if (node.length == 1) {
//				var result = node[0].innerHTML.alt;
				var result = node[0].childNodes[0].nodeValue;
				if (result) {
					// strip leading white space
					result = result.replace(/^\s+/,"");
					// strip trailing white space
					result = result.replace(/\s+$/,"");
				}
				console.log ("portlet page name: ");
				console.log (result);
				return (result);
			} else if (node.length === 0) {
				console.log ("could not find node for portlet page name");
			} else {
				console.log ("found multiple nodes for portlet page names");
				console.log (node);
			}
		} else {
			console.log ("could not find node for portlet page name");
		}
		} else {
			console.log ("no parent node to get portlet page name");
		}
	},
	getTopPortletNode: function (element) {
		var node = this.getAncestorByName (element, "IscPortlet");
		return (node);		
	},
	getPortletTitle: function (element) {
		var oid = this.getPortletOID (element);
		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				return node.innerHTML;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("cannot find oid for element: ");
			console.log (element);
		}
	},
	setPortletTitle: function (element, title) {
		console.log ("enter setPortletTitle: " + title);
		var oid = this.getPortletOID (element);

		if (oid) {
			var node = dojo.byId ("titleText." + oid);
			if (node) {
				node.innerHTML = title;
			} else {
				console.log ("cannot find node for title text: titleText." + oid);
			}
		} else {
			console.log ("cannot find oid for element: ");
			console.log (element);
		}
	}
});

}

/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/






dojo.provide( "ibm.tivoli.tip.dijit.BusyPane" );
/******************************************************************************
* Busy Pane that blocks all user input and shows progress message.
******************************************************************************/
dojo.declare("ibm.tivoli.tip.dijit.BusyPane", [dijit._Widget, dijit._Templated],
{
  processingImageSrc: dojo.moduleUrl("ibm.tivoli.tip.dijit","themes/images/processing-toolbar.gif"),
    
  //processing label var
  processingLabel : "",  
  
  templateString: "<div class='tipBusyPane'><table width='100%' height='100%' cellspacing='0' cellpadding='0' border='0'><tr><td valign='middle' align='middle'><div class='tipBusyPaneProcessing'><table width='100%' cellspacing='0' cellpadding='0' border='0'><tbody><tr><td class='process-anim' width='1%'><img width='16' height='16' src='${processingImageSrc}'/></td><td class='process-message' dojoAttachPoint='messageArea'>${processingLabel}</td></tr></tbody></table></div></td></tr></table></div>",
  
  constructor: function () {
	  this.processingLabel = dojo.i18n.getLocalization( "ibm.tivoli.tip.dijit", "resources" ).PROCESSING;
  },
  
  postCreate: function()
  {
    this.inherited( arguments );
    
    this.hide();
  },
  
  show: function( sizeToPortlet )
  {
  	var tipHelper = null;
  	var div       = null;
  	var coords    = null;
  	
	if ( sizeToPortlet ) 
	{
		
		tipHelper = new ibm.tivoli.tip.dojo.TIPHelper();
		
		div = tipHelper.getTopPortletNode (this.domNode);
		
		if ( div ) 
		{
		    coords = dojo.coords( div );
		    //console.log ("in BusyPane.show.  got coords: ");
		    //console.log (coords);
		    dojo.style (this.domNode, "top",  ( coords.y < 0 ) ? 0 : coords.y );
		    dojo.style (this.domNode, "left", ( coords.x < 0 ) ? 0 : coords.x );
		    dojo.style (this.domNode, "width", coords.w);
			dojo.style (this.domNode, "height", coords.h);
		}
	} 
	else 
	{
	     //console.log ("in BusyPane.show.  covering whole page/frame.");
	    dojo.style (this.domNode, "top", "0");
	    dojo.style (this.domNode, "left", "0");
	    dojo.style (this.domNode, "width", "100%");
		dojo.style (this.domNode, "height", "100%");
	}
    
    this.domNode.style.display="";
  },
  
  hide: function()
  {
    this.domNode.style.display="none";
  }
  
});

/****************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* 5724-Q87 
*
* (C) Copyright IBM Corp. 2008
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
********************************************************** {COPYRIGHT-END} **/

dojo.provide( "ibm.tivoli.tip.dojo.ActionBroker" );
dojo.provide( "ibm.tivoli.tip.dojo.Action" );

// Action Broker CONSTANTS
ibm.tivoli.tip.dojo.ActionBroker.TOPIC = "com.ibm.tivoli.tip.dojo.ActionBroker.perform";

/*******************************************************************************
* ActionBroker
*
* This manager is responsible for registering and running actions.
*
* The manager will subscribe to ibm.tivoli.tcr.ActionBroker.TOPIC.

* Use the runAction() method to initiate an action
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dojo.ActionBroker", //className
              null,                               //extends
{
   _actionMap: {},
   
  /*****************************************************************************
  * Construct Object and mixin all params
  *****************************************************************************/
  constructor: function( params )
  {
    dojo.mixin( this, params );
    
    this._performActionTopic = dojo.subscribe( ibm.tivoli.tip.dojo.ActionBroker.TOPIC, this, "performAction" );
  },

  /*****************************************************************************
  * Return a string of all registered action IDs.
  *****************************************************************************/
  registeredActions: function()
  { 
    return( this._fieldsOf( this._actionMap ) );
  },
  
  /*****************************************************************************
  * Register a given action with the broker.  Action must extend 
  * ibm.tivoli.tcr.Action
  *****************************************************************************/
  registerAction: function( actionObj )
  { 
     if ( ! actionObj.id || ! actionObj.run )
     {
       console.log( "ERROR: Trying to register an action that does not extend ibm.tivoli.tip.dojo.Action" );
       return;
     }
      
    this._actionMap[ actionObj.id ] = actionObj;
  },

  /*****************************************************************************
  * Unregister an action
  *****************************************************************************/
  unregisterAction: function( actionObj )
  { 
    if( actionObj.id )
    {
      this.unregisterActionByID( actionObj.id );
    }
  },

  /*****************************************************************************
  * Unregister an action by id
  *****************************************************************************/
  unregisterActionByID: function( actionID )
  { 
    if( this._actionMap[ actionID ] )
    {
      delete this._actionMap[ actionID ];
    }
  },
  
  /*****************************************************************************
  * The topic com.ibm.tivoli.tcr.performAction has been trapped or this method 
  * has been invoked.  Run the action.
  *
  * If more than one contextObj is provided it is assumed that the action
  * should be fired for each contextObj.  (for multiselect scenarios)
  *
  * actionDef = [ { actionID: id, context: [ contextObj, contextObj, ... ] } ]
  *****************************************************************************/
  performAction: function( actionDef )
  {
//  console.debug( "ActionBroker.performAction", actionDef );
//  console.debug( "  >> defined actions: ", dojo.toJson(  this._actionMap ) );

    var action = this._actionMap[ actionDef.actionID ];

//  console.debug( "ActionBroker.performAction: action", dojo.toJson( action ) );

    if ( action )
    {
      action.run( actionDef.context );
    }
  },
  
  /*****************************************************************************
  * Destroy the action broker.
  *****************************************************************************/
  destroy: function()
  {
    dojo.unsubscribe( this._performActionTopic );
    
    for ( var i in this._actionMap )
    {
      this._actionMap[i].destroy();
    }
    
    delete this._actionMap;  
  },

  /*****************************************************************************
  * (Indirectly) Run an action. Includes support for "hierarchical" IDs
  *****************************************************************************/
  /* void */ 
  runAction: function( /* string */ actionID, /* Object */ context )
  {
//  console.debug( ">>> runAction", actionID, dojo.toJson( context ) );

    // create default publish data object

    publishData = { 
                    actionID: actionID,  // used in locating       actual action impl
                    context:  context    // ONLY this is passed to actual action impl     
                  };      

    // is this a hierarchical action ID?  format: actionID/subactionID

    index = actionID.indexOf( "/" );

    if ( index != -1) 
    {
      subActionID = actionID.substring( index + 1 );
         actionID = actionID.substring( 0, index );   

      publishData.           actionID =    actionID;
      publishData.context.   actionID =    actionID;
      publishData.context.subActionID = subActionID;  // used to locate subaction action impl
    }

    // finally, "publish" the action request

//  console.debug( ">>> runAction: topic data", dojo.toJson( publishData ) );

    dojo.publish( ibm.tivoli.tcr.ActionBroker.TOPIC, [ publishData ] );      
  },
  
  /*****************************************************************************
  * Return a string of all the fields contained in an object.
  *****************************************************************************/
  _fieldsOf: function( object )
  { 
    s = "";
  
    if ( object && ( object !== null ) ) 
    {
      for ( var field in object )
      {
        s += field + "; ";
      }
    }
  
    return( s );
  } 
});

/*******************************************************************************
* Action Interface
*
* Extend this to make new actions that can be registered with the broker
*******************************************************************************/
dojo.declare( "ibm.tivoli.tip.dojo.Action", //className
              null,                        //extends
{
  id:  null,                        /*required*/
  
  run: function( context ){ /*requried implement*/ }  
} );


dojo.i18n._preloadLocalizations("dojo.nls.tipdojo", ["es-es", "es", "hu", "it-it", "de", "pt-br", "pl", "fr-fr", "zh-cn", "pt", "en-us", "zh", "ru", "xx", "fr", "zh-tw", "it", "cs", "en-gb", "de-de", "ja-jp", "ko-kr", "ko", "en", "ROOT", "ja"]);
